(function() {
  var svg;

  //save off default references
  var d3 = window.d3, topojson = window.topojson;

  var defaultOptions = {
    scope: 'world',
    responsive: false,
    aspectRatio: 0.5625,
    setProjection: setProjection,
    projection: 'equirectangular',
    dataType: 'json',
    data: {},
    done: function() {},
    fills: {
      defaultFill: '#ABDDA4'
    },
    filters: {},
    geographyConfig: {
        dataUrl: null,
        hideAntarctica: true,
        hideHawaiiAndAlaska : false,
        borderWidth: 1,
        borderColor: '#FDFDFD',
        popupTemplate: function(geography, data) {
          return '<div class="hoverinfo"><strong>' + geography.properties.name + '</strong></div>';
        },
        popupOnHover: true,
        highlightOnHover: true,
        highlightFillColor: '#FC8D59',
        highlightBorderColor: 'rgba(250, 15, 160, 0.2)',
        highlightBorderWidth: 2
    },
    projectionConfig: {
      rotation: [97, 0]
    },
    bubblesConfig: {
        borderWidth: 2,
        borderColor: '#FFFFFF',
        popupOnHover: true,
        radius: null,
        popupTemplate: function(geography, data) {
          return '<div class="hoverinfo"><strong>' + data.name + '</strong></div>';
        },
        fillOpacity: 0.75,
        animate: true,
        highlightOnHover: true,
        highlightFillColor: '#FC8D59',
        highlightBorderColor: 'rgba(250, 15, 160, 0.2)',
        highlightBorderWidth: 2,
        highlightFillOpacity: 0.85,
        exitDelay: 100,
        key: JSON.stringify
    },
    arcConfig: {
      strokeColor: '#DD1C77',
      strokeWidth: 1,
      arcSharpness: 1,
      animationSpeed: 600
    }
  };

  /*
    Getter for value. If not declared on datumValue, look up the chain into optionsValue
  */
  function val( datumValue, optionsValue, context ) {
    if ( typeof context === 'undefined' ) {
      context = optionsValue;
      optionsValues = undefined;
    }
    var value = typeof datumValue !== 'undefined' ? datumValue : optionsValue;

    if (typeof value === 'undefined') {
      return  null;
    }

    if ( typeof value === 'function' ) {
      var fnContext = [context];
      if ( context.geography ) {
        fnContext = [context.geography, context.data];
      }
      return value.apply(null, fnContext);
    }
    else {
      return value;
    }
  }

  function addContainer( element, height, width ) {
    this.svg = d3.select( element ).append('svg')
      .attr('width', width || element.offsetWidth)
      .attr('data-width', width || element.offsetWidth)
      .attr('class', 'datamap')
      .attr('height', height || element.offsetHeight)
      .style('overflow', 'hidden'); // IE10+ doesn't respect height/width when map is zoomed in

    if (this.options.responsive) {
      d3.select(this.options.element).style({'position': 'relative', 'padding-bottom': (this.options.aspectRatio*100) + '%'});
      d3.select(this.options.element).select('svg').style({'position': 'absolute', 'width': '100%', 'height': '100%'});
      d3.select(this.options.element).select('svg').select('g').selectAll('path').style('vector-effect', 'non-scaling-stroke');

    }

    return this.svg;
  }

  // setProjection takes the svg element and options
  function setProjection( element, options ) {
    var width = options.width || element.offsetWidth;
    var height = options.height || element.offsetHeight;
    var projection, path;
    var svg = this.svg;

    if ( options && typeof options.scope === 'undefined') {
      options.scope = 'world';
    }

    if ( options.scope === 'usa' ) {
      projection = d3.geo.albersUsa()
        .scale(width)
        .translate([width / 2, height / 2]);
    }
    else if ( options.scope === 'world' ) {
      projection = d3.geo[options.projection]()
        .scale((width + 1) / 2 / Math.PI)
        .translate([width / 2, height / (options.projection === "mercator" ? 1.45 : 1.8)]);
    }

    if ( options.projection === 'orthographic' ) {

      svg.append("defs").append("path")
        .datum({type: "Sphere"})
        .attr("id", "sphere")
        .attr("d", path);

      svg.append("use")
          .attr("class", "stroke")
          .attr("xlink:href", "#sphere");

      svg.append("use")
          .attr("class", "fill")
          .attr("xlink:href", "#sphere");
      projection.scale(250).clipAngle(90).rotate(options.projectionConfig.rotation)
    }

    path = d3.geo.path()
      .projection( projection );

    return {path: path, projection: projection};
  }

  function addStyleBlock() {
    if ( d3.select('.datamaps-style-block').empty() ) {
      d3.select('head').append('style').attr('class', 'datamaps-style-block')
      .html('.datamap path.datamaps-graticule { fill: none; stroke: #777; stroke-width: 0.5px; stroke-opacity: .5; pointer-events: none; } .datamap .labels {pointer-events: none;} .datamap path {stroke: #FFFFFF; stroke-width: 1px;} .datamaps-legend dt, .datamaps-legend dd { float: left; margin: 0 3px 0 0;} .datamaps-legend dd {width: 20px; margin-right: 6px; border-radius: 3px;} .datamaps-legend {padding-bottom: 20px; z-index: 1001; position: absolute; left: 4px; font-size: 12px; font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;} .datamaps-hoverover {display: none; font-family: "Helvetica Neue", Helvetica, Arial, sans-serif; } .hoverinfo {padding: 4px; border-radius: 1px; background-color: #FFF; box-shadow: 1px 1px 5px #CCC; font-size: 12px; border: 1px solid #CCC; } .hoverinfo hr {border:1px dotted #CCC; }');
    }
  }

  function drawSubunits( data ) {
    var fillData = this.options.fills,
        colorCodeData = this.options.data || {},
        geoConfig = this.options.geographyConfig;


    var subunits = this.svg.select('g.datamaps-subunits');
    if ( subunits.empty() ) {
      subunits = this.addLayer('datamaps-subunits', null, true);
    }

    var geoData = topojson.feature( data, data.objects[ this.options.scope ] ).features;
    if ( geoConfig.hideAntarctica ) {
      geoData = geoData.filter(function(feature) {
        return feature.id !== "ATA";
      });
    }

    if ( geoConfig.hideHawaiiAndAlaska ) {
      geoData = geoData.filter(function(feature) {
        return feature.id !== "HI" && feature.id !== 'AK';
      });
    }

    var geo = subunits.selectAll('path.datamaps-subunit').data( geoData );

    geo.enter()
      .append('path')
      .attr('d', this.path)
      .attr('class', function(d) {
        return 'datamaps-subunit ' + d.id;
      })
      .attr('data-info', function(d) {
        return JSON.stringify( colorCodeData[d.id]);
      })
      .style('fill', function(d) {
        //if fillKey - use that
        //otherwise check 'fill'
        //otherwise check 'defaultFill'
        var fillColor;

        var datum = colorCodeData[d.id];
        if ( datum && datum.fillKey ) {
          fillColor = fillData[ val(datum.fillKey, {data: colorCodeData[d.id], geography: d}) ];
        }

        if ( typeof fillColor === 'undefined' ) {
          fillColor = val(datum && datum.fillColor, fillData.defaultFill, {data: colorCodeData[d.id], geography: d});
        }

        return fillColor;
      })
      .style('stroke-width', geoConfig.borderWidth)
      .style('stroke', geoConfig.borderColor);
  }

  function handleGeographyConfig () {
    var hoverover;
    var svg = this.svg;
    var self = this;
    var options = this.options.geographyConfig;

    if ( options.highlightOnHover || options.popupOnHover ) {
      svg.selectAll('.datamaps-subunit')
        .on('mouseover', function(d) {
          var $this = d3.select(this);
          var datum = self.options.data[d.id] || {};
          if ( options.highlightOnHover ) {
            var previousAttributes = {
              'fill':  $this.style('fill'),
              'stroke': $this.style('stroke'),
              'stroke-width': $this.style('stroke-width'),
              'fill-opacity': $this.style('fill-opacity')
            };

            $this
              .style('fill', val(datum.highlightFillColor, options.highlightFillColor, datum))
              .style('stroke', val(datum.highlightBorderColor, options.highlightBorderColor, datum))
              .style('stroke-width', val(datum.highlightBorderWidth, options.highlightBorderWidth, datum))
              .style('fill-opacity', val(datum.highlightFillOpacity, options.highlightFillOpacity, datum))
              .attr('data-previousAttributes', JSON.stringify(previousAttributes));

            //as per discussion on https://github.com/markmarkoh/datamaps/issues/19
            if ( ! /((MSIE)|(Trident))/.test(navigator.userAgent) ) {
             moveToFront.call(this);
            }
          }

          if ( options.popupOnHover ) {
            self.updatePopup($this, d, options, svg);
          }
        })
        .on('mouseout', function() {
          var $this = d3.select(this);

          if (options.highlightOnHover) {
            //reapply previous attributes
            var previousAttributes = JSON.parse( $this.attr('data-previousAttributes') );
            for ( var attr in previousAttributes ) {
              $this.style(attr, previousAttributes[attr]);
            }
          }
          $this.on('mousemove', null);
          d3.selectAll('.datamaps-hoverover').style('display', 'none');
        });
    }

    function moveToFront() {
      this.parentNode.appendChild(this);
    }
  }

  //plugin to add a simple map legend
  function addLegend(layer, data, options) {
    data = data || {};
    if ( !this.options.fills ) {
      return;
    }

    var html = '<dl>';
    var label = '';
    if ( data.legendTitle ) {
      html = '<h2>' + data.legendTitle + '</h2>' + html;
    }
    for ( var fillKey in this.options.fills ) {

      if ( fillKey === 'defaultFill') {
        if (! data.defaultFillName ) {
          continue;
        }
        label = data.defaultFillName;
      } else {
        if (data.labels && data.labels[fillKey]) {
          label = data.labels[fillKey];
        } else {
          label= fillKey + ': ';
        }
      }
      html += '<dt>' + label + '</dt>';
      html += '<dd style="background-color:' +  this.options.fills[fillKey] + '">&nbsp;</dd>';
    }
    html += '</dl>';

    var hoverover = d3.select( this.options.element ).append('div')
      .attr('class', 'datamaps-legend')
      .html(html);
  }

    function addGraticule ( layer, options ) {
      var graticule = d3.geo.graticule();
      this.svg.insert("path", '.datamaps-subunits')
        .datum(graticule)
        .attr("class", "datamaps-graticule")
        .attr("d", this.path);
  }

  function handleArcs (layer, data, options) {
    var self = this,
        svg = this.svg;

    if ( !data || (data && !data.slice) ) {
      throw "Datamaps Error - arcs must be an array";
    }

    // For some reason arc options were put in an `options` object instead of the parent arc
    // I don't like this, so to match bubbles and other plugins I'm moving it
    // This is to keep backwards compatability
    for ( var i = 0; i < data.length; i++ ) {
      data[i] = defaults(data[i], data[i].options);
      delete data[i].options;
    }

    if ( typeof options === "undefined" ) {
      options = defaultOptions.arcConfig;
    }

    var arcs = layer.selectAll('path.datamaps-arc').data( data, JSON.stringify );

    var path = d3.geo.path()
        .projection(self.projection);

    arcs
      .enter()
        .append('svg:path')
        .attr('class', 'datamaps-arc')
        .style('stroke-linecap', 'round')
        .style('stroke', function(datum) {
          return val(datum.strokeColor, options.strokeColor, datum);
        })
        .style('fill', 'none')
        .style('stroke-width', function(datum) {
            return val(datum.strokeWidth, options.strokeWidth, datum);
        })
        .attr('d', function(datum) {
            var originXY = self.latLngToXY(val(datum.origin.latitude, datum), val(datum.origin.longitude, datum))
            var destXY = self.latLngToXY(val(datum.destination.latitude, datum), val(datum.destination.longitude, datum));
            var midXY = [ (originXY[0] + destXY[0]) / 2, (originXY[1] + destXY[1]) / 2];
            if (options.greatArc) {
                  // TODO: Move this to inside `if` clause when setting attr `d`
              var greatArc = d3.geo.greatArc()
                  .source(function(d) { return [val(d.origin.longitude, d), val(d.origin.latitude, d)]; })
                  .target(function(d) { return [val(d.destination.longitude, d), val(d.destination.latitude, d)]; });

              return path(greatArc(datum))
            }
            var sharpness = val(datum.arcSharpness, options.arcSharpness, datum);
            return "M" + originXY[0] + ',' + originXY[1] + "S" + (midXY[0] + (50 * sharpness)) + "," + (midXY[1] - (75 * sharpness)) + "," + destXY[0] + "," + destXY[1];
        })
        .transition()
          .delay(100)
          .style('fill', function(datum) {
            /*
              Thank you Jake Archibald, this is awesome.
              Source: http://jakearchibald.com/2013/animated-line-drawing-svg/
            */
            var length = this.getTotalLength();
            this.style.transition = this.style.WebkitTransition = 'none';
            this.style.strokeDasharray = length + ' ' + length;
            this.style.strokeDashoffset = length;
            this.getBoundingClientRect();
            this.style.transition = this.style.WebkitTransition = 'stroke-dashoffset ' + val(datum.animationSpeed, options.animationSpeed, datum) + 'ms ease-out';
            this.style.strokeDashoffset = '0';
            return 'none';
          })

    arcs.exit()
      .transition()
      .style('opacity', 0)
      .remove();
  }

  function handleLabels ( layer, options ) {
    var self = this;
    options = options || {};
    var labelStartCoodinates = this.projection([-67.707617, 42.722131]);
    this.svg.selectAll(".datamaps-subunit")
      .attr("data-foo", function(d) {
        var center = self.path.centroid(d);
        var xOffset = 7.5, yOffset = 5;

        if ( ["FL", "KY", "MI"].indexOf(d.id) > -1 ) xOffset = -2.5;
        if ( d.id === "NY" ) xOffset = -1;
        if ( d.id === "MI" ) yOffset = 18;
        if ( d.id === "LA" ) xOffset = 13;

        var x,y;

        x = center[0] - xOffset;
        y = center[1] + yOffset;

        var smallStateIndex = ["VT", "NH", "MA", "RI", "CT", "NJ", "DE", "MD", "DC"].indexOf(d.id);
        if ( smallStateIndex > -1) {
          var yStart = labelStartCoodinates[1];
          x = labelStartCoodinates[0];
          y = yStart + (smallStateIndex * (2+ (options.fontSize || 12)));
          layer.append("line")
            .attr("x1", x - 3)
            .attr("y1", y - 5)
            .attr("x2", center[0])
            .attr("y2", center[1])
            .style("stroke", options.labelColor || "#000")
            .style("stroke-width", options.lineWidth || 1)
        }

        layer.append("text")
          .attr("x", x)
          .attr("y", y)
          .style("font-size", (options.fontSize || 10) + 'px')
          .style("font-family", options.fontFamily || "Verdana")
          .style("fill", options.labelColor || "#000")
          .text( d.id );
        return "bar";
      });
  }


  function handleBubbles (layer, data, options ) {
    var self = this,
        fillData = this.options.fills,
        filterData = this.options.filters,
        svg = this.svg;

    if ( !data || (data && !data.slice) ) {
      throw "Datamaps Error - bubbles must be an array";
    }

    var bubbles = layer.selectAll('circle.datamaps-bubble').data( data, options.key );

    bubbles
      .enter()
        .append('svg:circle')
        .attr('class', 'datamaps-bubble')
        .attr('cx', function ( datum ) {
          var latLng;
          if ( datumHasCoords(datum) ) {
            latLng = self.latLngToXY(datum.latitude, datum.longitude);
          }
          else if ( datum.centered ) {
            latLng = self.path.centroid(svg.select('path.' + datum.centered).data()[0]);
          }
          if ( latLng ) return latLng[0];
        })
        .attr('cy', function ( datum ) {
          var latLng;
          if ( datumHasCoords(datum) ) {
            latLng = self.latLngToXY(datum.latitude, datum.longitude);
          }
          else if ( datum.centered ) {
            latLng = self.path.centroid(svg.select('path.' + datum.centered).data()[0]);
          }
          if ( latLng ) return latLng[1];
        })
        .attr('r', function(datum) {
          // if animation enabled start with radius 0, otherwise use full size.
          return options.animate ? 0 : val(datum.radius, options.radius, datum);
        })
        .attr('data-info', function(d) {
          return JSON.stringify(d);
        })
        .attr('filter', function (datum) {
          var filterKey = filterData[ val(datum.filterKey, options.filterKey, datum) ];

          if (filterKey) {
            return filterKey;
          }
        })
        .style('stroke', function ( datum ) {
          return val(datum.borderColor, options.borderColor, datum);
        })
        .style('stroke-width', function ( datum ) {
          return val(datum.borderWidth, options.borderWidth, datum);
        })
        .style('fill-opacity', function ( datum ) {
          return val(datum.fillOpacity, options.fillOpacity, datum);
        })
        .style('fill', function ( datum ) {
          var fillColor = fillData[ val(datum.fillKey, options.fillKey, datum) ];
          return fillColor || fillData.defaultFill;
        })
        .on('mouseover', function ( datum ) {
          var $this = d3.select(this);

          if (options.highlightOnHover) {
            //save all previous attributes for mouseout
            var previousAttributes = {
              'fill':  $this.style('fill'),
              'stroke': $this.style('stroke'),
              'stroke-width': $this.style('stroke-width'),
              'fill-opacity': $this.style('fill-opacity')
            };

            $this
              .style('fill', val(datum.highlightFillColor, options.highlightFillColor, datum))
              .style('stroke', val(datum.highlightBorderColor, options.highlightBorderColor, datum))
              .style('stroke-width', val(datum.highlightBorderWidth, options.highlightBorderWidth, datum))
              .style('fill-opacity', val(datum.highlightFillOpacity, options.highlightFillOpacity, datum))
              .attr('data-previousAttributes', JSON.stringify(previousAttributes));
          }

          if (options.popupOnHover) {
            self.updatePopup($this, datum, options, svg);
          }
        })
        .on('mouseout', function ( datum ) {
          var $this = d3.select(this);

          if (options.highlightOnHover) {
            //reapply previous attributes
            var previousAttributes = JSON.parse( $this.attr('data-previousAttributes') );
            for ( var attr in previousAttributes ) {
              $this.style(attr, previousAttributes[attr]);
            }
          }

          d3.selectAll('.datamaps-hoverover').style('display', 'none');
        })

    bubbles.transition()
      .duration(400)
      .attr('r', function ( datum ) {
        return val(datum.radius, options.radius, datum);
      });

    bubbles.exit()
      .transition()
        .delay(options.exitDelay)
        .attr("r", 0)
        .remove();

    function datumHasCoords (datum) {
      return typeof datum !== 'undefined' && typeof datum.latitude !== 'undefined' && typeof datum.longitude !== 'undefined';
    }
  }

  //stolen from underscore.js
  function defaults(obj) {
    Array.prototype.slice.call(arguments, 1).forEach(function(source) {
      if (source) {
        for (var prop in source) {
          if (obj[prop] == null) obj[prop] = source[prop];
        }
      }
    });
    return obj;
  }
  /**************************************
             Public Functions
  ***************************************/

  function Datamap( options ) {

    if ( typeof d3 === 'undefined' || typeof topojson === 'undefined' ) {
      throw new Error('Include d3.js (v3.0.3 or greater) and topojson on this page before creating a new map');
   }
    //set options for global use
    this.options = defaults(options, defaultOptions);
    this.options.geographyConfig = defaults(options.geographyConfig, defaultOptions.geographyConfig);
    this.options.projectionConfig = defaults(options.projectionConfig, defaultOptions.projectionConfig);
    this.options.bubblesConfig = defaults(options.bubblesConfig, defaultOptions.bubblesConfig);
    this.options.arcConfig = defaults(options.arcConfig, defaultOptions.arcConfig);

    //add the SVG container
    if ( d3.select( this.options.element ).select('svg').length > 0 ) {
      addContainer.call(this, this.options.element, this.options.height, this.options.width );
    }

    /* Add core plugins to this instance */
    this.addPlugin('bubbles', handleBubbles);
    this.addPlugin('legend', addLegend);
    this.addPlugin('arc', handleArcs);
    this.addPlugin('labels', handleLabels);
    this.addPlugin('graticule', addGraticule);

    //append style block with basic hoverover styles
    if ( ! this.options.disableDefaultStyles ) {
      addStyleBlock();
    }

    return this.draw();
  }

  // resize map
  Datamap.prototype.resize = function () {

    var self = this;
    var options = self.options;

    if (options.responsive) {
      var newsize = options.element.clientWidth,
          oldsize = d3.select( options.element).select('svg').attr('data-width');

      d3.select(options.element).select('svg').selectAll('g').attr('transform', 'scale(' + (newsize / oldsize) + ')');
    }
  }

  // actually draw the features(states & countries)
  Datamap.prototype.draw = function() {
    //save off in a closure
    var self = this;
    var options = self.options;

    //set projections and paths based on scope
    var pathAndProjection = options.setProjection.apply(self, [options.element, options] );

    this.path = pathAndProjection.path;
    this.projection = pathAndProjection.projection;

    //if custom URL for topojson data, retrieve it and render
    if ( options.geographyConfig.dataUrl ) {
      d3.json( options.geographyConfig.dataUrl, function(error, results) {
        if ( error ) throw new Error(error);
        self.customTopo = results;
        draw( results );
      });
    }
    else {
      draw( this[options.scope + 'Topo'] || options.geographyConfig.dataJson);
    }

    return this;

      function draw (data) {
        // if fetching remote data, draw the map first then call `updateChoropleth`
        if ( self.options.dataUrl ) {
          //allow for csv or json data types
          d3[self.options.dataType](self.options.dataUrl, function(data) {
            //in the case of csv, transform data to object
            if ( self.options.dataType === 'csv' && (data && data.slice) ) {
              var tmpData = {};
              for(var i = 0; i < data.length; i++) {
                tmpData[data[i].id] = data[i];
              }
              data = tmpData;
            }
            Datamaps.prototype.updateChoropleth.call(self, data);
          });
        }
        drawSubunits.call(self, data);
        handleGeographyConfig.call(self);

        if ( self.options.geographyConfig.popupOnHover || self.options.bubblesConfig.popupOnHover) {
          hoverover = d3.select( self.options.element ).append('div')
            .attr('class', 'datamaps-hoverover')
            .style('z-index', 10001)
            .style('position', 'absolute');
        }

        //fire off finished callback
        self.options.done(self);
      }
  };
  /**************************************
                TopoJSON
  ***************************************/
  Datamap.prototype.worldTopo = '__WORLD__';
  Datamap.prototype.abwTopo = '__ABW__';
  Datamap.prototype.afgTopo = '__AFG__';
  Datamap.prototype.agoTopo = '__AGO__';
  Datamap.prototype.aiaTopo = '__AIA__';
  Datamap.prototype.albTopo = '__ALB__';
  Datamap.prototype.aldTopo = '__ALD__';
  Datamap.prototype.andTopo = '__AND__';
  Datamap.prototype.areTopo = '__ARE__';
  Datamap.prototype.argTopo = '__ARG__';
  Datamap.prototype.armTopo = '__ARM__';
  Datamap.prototype.asmTopo = '__ASM__';
  Datamap.prototype.ataTopo = '__ATA__';
  Datamap.prototype.atcTopo = '__ATC__';
  Datamap.prototype.atfTopo = '__ATF__';
  Datamap.prototype.atgTopo = '__ATG__';
  Datamap.prototype.ausTopo = '__AUS__';
  Datamap.prototype.autTopo = '__AUT__';
  Datamap.prototype.azeTopo = '__AZE__';
  Datamap.prototype.bdiTopo = '__BDI__';
  Datamap.prototype.belTopo = '__BEL__';
  Datamap.prototype.benTopo = '__BEN__';
  Datamap.prototype.bfaTopo = '__BFA__';
  Datamap.prototype.bgdTopo = '__BGD__';
  Datamap.prototype.bgrTopo = '__BGR__';
  Datamap.prototype.bhrTopo = '__BHR__';
  Datamap.prototype.bhsTopo = '__BHS__';
  Datamap.prototype.bihTopo = '__BIH__';
  Datamap.prototype.bjnTopo = '__BJN__';
  Datamap.prototype.blmTopo = '__BLM__';
  Datamap.prototype.blrTopo = '__BLR__';
  Datamap.prototype.blzTopo = '__BLZ__';
  Datamap.prototype.bmuTopo = '__BMU__';
  Datamap.prototype.bolTopo = '__BOL__';
  Datamap.prototype.braTopo = '__BRA__';
  Datamap.prototype.brbTopo = '__BRB__';
  Datamap.prototype.brnTopo = '__BRN__';
  Datamap.prototype.btnTopo = '__BTN__';
  Datamap.prototype.norTopo = '__NOR__';
  Datamap.prototype.bwaTopo = '__BWA__';
  Datamap.prototype.cafTopo = '__CAF__';
  Datamap.prototype.canTopo = {"type":"Topology","objects":{"can":{"type":"GeometryCollection","geometries":[{"type":"MultiPolygon","properties":{"name":null},"id":"-99","arcs":[[[0]],[[1]]]},{"type":"Polygon","properties":{"name":"Manitoba"},"id":"CA.MB","arcs":[[2,3,4,5,6]]},{"type":"Polygon","properties":{"name":"Saskatchewan"},"id":"CA.SK","arcs":[[7,8,9,-5]]},{"type":"Polygon","properties":{"name":"Alberta"},"id":"CA.AB","arcs":[[10,11,12,-9]]},{"type":"MultiPolygon","properties":{"name":"British Columbia"},"id":"CA.BC","arcs":[[[13]],[[14]],[[15]],[[16]],[[17]],[[18]],[[19]],[[20]],[[21]],[[22]],[[23]],[[24]],[[25]],[[26]],[[27]],[[28]],[[29]],[[30]],[[31]],[[32]],[[33]],[[34]],[[35]],[[36]],[[37]],[[38]],[[39]],[[40]],[[41]],[[42]],[[43]],[[44]],[[45]],[[46]],[[47]],[[48]],[[49]],[[50]],[[51]],[[52]],[[53]],[[54]],[[55]],[[56]],[[57]],[[58]],[[59]],[[60]],[[61]],[[62]],[[63]],[[64]],[[65]],[[66]],[[67]],[[68]],[[69]],[[70]],[[71]],[[72]],[[73]],[[74]],[[75]],[[76]],[[77]],[[78]],[[79]],[[80]],[[81]],[[82]],[[83]],[[84]],[[85]],[[86]],[[87]],[[88]],[[89]],[[90]],[[91]],[[92]],[[93]],[[94]],[[95]],[[96]],[[97]],[[98]],[[99]],[[100]],[[101]],[[102]],[[103]],[[104]],[[105]],[[106]],[[107]],[[108]],[[109]],[[110]],[[111]],[[112]],[[113]],[[114]],[[115]],[[116]],[[117]],[[118]],[[119]],[[120]],[[121]],[[122]],[[123]],[[124]],[[125]],[[126]],[[127]],[[128]],[[129]],[[130]],[[131]],[[132,133,134,-12]]]},{"type":"MultiPolygon","properties":{"name":"Nunavut"},"id":"CA.NU","arcs":[[[135]],[[136]],[[137]],[[138]],[[139]],[[140]],[[141]],[[142]],[[143]],[[144]],[[145]],[[146]],[[147]],[[148]],[[149]],[[150]],[[151]],[[152]],[[153]],[[154]],[[155]],[[156]],[[157]],[[158]],[[159]],[[160]],[[161]],[[162]],[[163]],[[164]],[[165]],[[166]],[[167]],[[168]],[[169]],[[170]],[[171]],[[172]],[[173]],[[174]],[[175]],[[176]],[[177]],[[178]],[[179]],[[180]],[[181]],[[182]],[[183]],[[184]],[[185]],[[186]],[[187]],[[188]],[[189]],[[190]],[[191]],[[192]],[[193]],[[194]],[[195]],[[196]],[[197]],[[198]],[[199]],[[200]],[[201]],[[202]],[[203]],[[204]],[[205]],[[206]],[[207]],[[208]],[[209]],[[210]],[[211]],[[212]],[[213]],[[214]],[[215]],[[216]],[[217]],[[218]],[[219]],[[220]],[[221]],[[222]],[[223]],[[224]],[[225]],[[226]],[[227]],[[228]],[[229]],[[230]],[[231]],[[232]],[[233]],[[234]],[[235]],[[236,237]],[[238]],[[239]],[[240]],[[241]],[[242]],[[243]],[[244]],[[245]],[[246]],[[247]],[[248]],[[249]],[[250]],[[251]],[[252]],[[253]],[[254]],[[255]],[[256]],[[257]],[[258]],[[259]],[[260]],[[261]],[[262]],[[263]],[[264]],[[265]],[[266]],[[267]],[[268]],[[269]],[[270]],[[271]],[[272]],[[273]],[[274]],[[275]],[[276]],[[277]],[[278]],[[279]],[[280]],[[281]],[[282]],[[283]],[[284]],[[285]],[[286]],[[287]],[[288]],[[289]],[[290]],[[291]],[[292]],[[293]],[[294]],[[295]],[[296]],[[297]],[[298]],[[299]],[[300]],[[301]],[[302]],[[303]],[[304]],[[305]],[[306]],[[307]],[[308]],[[309]],[[310]],[[311]],[[312]],[[313]],[[314]],[[315]],[[316]],[[317]],[[318]],[[319]],[[320]],[[321]],[[322]],[[323]],[[324]],[[325]],[[326]],[[327]],[[328]],[[329]],[[330]],[[331]],[[332]],[[333]],[[334]],[[335]],[[336]],[[337]],[[338]],[[339]],[[340]],[[341]],[[342]],[[343]],[[344]],[[345]],[[346]],[[347]],[[348]],[[349]],[[350]],[[351]],[[352]],[[353]],[[354]],[[355]],[[356]],[[357]],[[358]],[[359]],[[360]],[[361]],[[362]],[[363]],[[364]],[[365]],[[366]],[[367]],[[368]],[[369]],[[370]],[[371]],[[372]],[[373]],[[374]],[[375]],[[-6,376,377]],[[378]],[[379]],[[380]],[[381,382,383,384,385,386,387,388]],[[389]],[[390]],[[391]],[[392]],[[393]],[[394]],[[395]],[[396]],[[397]],[[398]],[[399]],[[400]],[[401]],[[402]],[[403]],[[404]],[[405]],[[406]],[[407]],[[408]],[[409]],[[410]],[[411]],[[412]],[[413,414,415,416,417,418]],[[419]],[[420]],[[421]],[[422]],[[423]],[[424]],[[425]],[[426,427]],[[428]],[[429]],[[430,431]],[[432]],[[433]],[[434]],[[435]],[[436]],[[437]],[[438]]]},{"type":"MultiPolygon","properties":{"name":"Northwest Territories"},"id":"CA.NT","arcs":[[[439]],[[440]],[[441]],[[442]],[[443]],[[444]],[[445]],[[446]],[[447]],[[448]],[[449]],[[450]],[[451]],[[-377,-10,-13,-135,452,453]],[[454]],[[455]],[[456,-384]],[[457]],[[458]],[[459]],[[-386,460]],[[461,-388]],[[-382,462]],[[463]],[[-416,464]],[[465]],[[466]],[[467,-418]],[[-414,468]],[[469]],[[470]],[[471]],[[-427,472]],[[-431,473]]]},{"type":"MultiPolygon","properties":{"name":"Yukon"},"id":"CA.YT","arcs":[[[-453,-134,474]],[[475]]]},{"type":"MultiPolygon","properties":{"name":"Ontario"},"id":"CA.ON","arcs":[[[476,477]],[[478,479,480,-3,481]]]},{"type":"MultiPolygon","properties":{"name":"Québec"},"id":"CA.QC","arcs":[[[482]],[[483]],[[484]],[[485]],[[486]],[[487]],[[488]],[[489]],[[490]],[[491,492,493]],[[494]],[[495]],[[496]],[[497]],[[498]],[[499]],[[500]],[[501]],[[502]],[[503]],[[504]],[[505]],[[506]],[[507]],[[508,509,-479,510,-478,511]]]},{"type":"MultiPolygon","properties":{"name":"New Brunswick"},"id":"CA.NB","arcs":[[[512]],[[513]],[[514]],[[515,516,-492,517]]]},{"type":"MultiPolygon","properties":{"name":"Nova Scotia"},"id":"CA.NS","arcs":[[[518]],[[519]],[[520]],[[521]],[[522]],[[-516,523]],[[524]],[[525]]]},{"type":"MultiPolygon","properties":{"name":"Newfoundland and Labrador"},"id":"CA.NF","arcs":[[[526]],[[527]],[[528]],[[529]],[[530]],[[531]],[[532]],[[533]],[[534]],[[535]],[[536]],[[537]],[[538]],[[539]],[[540]],[[541]],[[542]],[[543]],[[544]],[[545]],[[546]],[[547]],[[548]],[[549]],[[550]],[[551]],[[552]],[[553]],[[554]],[[555]],[[556]],[[557]],[[558]],[[559]],[[560]],[[561]],[[562]],[[563]],[[564]],[[565]],[[566]],[[567]],[[568,-509]],[[-237,569]]]},{"type":"Polygon","properties":{"name":"Prince Edward Island"},"id":"CA.PE","arcs":[[570]]}]}},"arcs":[[[9146,1331],[-1,0],[0,1],[0,1],[0,1],[0,1],[0,1],[1,1],[1,3],[1,0],[0,-2],[0,-2],[-1,-3],[0,-1],[-1,-1]],[[9003,1478],[-1,0],[-1,0],[-1,-1],[-2,0],[-2,-3],[-3,0],[0,1],[0,1],[6,4],[3,-1],[1,-1]],[[5889,3663],[-32,-41],[-28,-38],[-29,-37],[-28,-38],[-39,-54],[-40,-54],[-40,-54],[-39,-55],[-35,-49],[-34,-51],[-35,-50],[-34,-52],[-19,-28],[-19,-29],[-19,-29],[-19,-29],[-13,-21],[-13,-20],[-14,-20],[-13,-19],[-27,-34],[-26,-33],[-27,-34],[-27,-34],[-5,-7],[-6,-8],[-6,-7],[-6,-8],[-7,-8],[-6,-9],[-7,-8],[-6,-9],[-4,-6],[0,-13],[0,-95],[0,-95],[0,-95],[0,-94],[0,-73],[0,-73],[0,-72],[0,-73],[0,-26],[0,-27],[0,-18],[0,-17],[-1,-22],[0,-28],[0,-10],[0,-1]],[[5186,1858],[0,-14],[0,-22],[0,-27],[0,-24],[0,-2],[-1,-2],[-1,0],[-25,0],[-24,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-24,0],[-8,0],[-17,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-24,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-24,0],[-25,0],[-25,0],[-25,0],[-25,0],[-4,0]],[[4484,1767],[-2,44],[-2,51],[-2,52],[-2,52],[-3,51],[-2,52],[-2,52],[-2,51],[-3,52],[-2,52],[-2,51],[-2,52],[-3,52],[-2,52],[-2,51],[-2,52],[-3,52],[-2,51],[-2,52],[-2,52],[-3,51],[-2,52],[-2,52],[-3,51],[-2,52],[-2,52],[-2,52],[-3,51],[-2,52],[-2,52],[-2,51],[-3,52],[0,88],[0,88],[0,88],[0,88],[0,137],[0,137],[0,137],[0,137],[0,27],[0,27],[0,28],[0,27]],[[4412,4422],[51,0],[51,0],[51,0],[51,0],[51,0],[51,0],[51,0],[51,0],[51,0],[13,0],[37,0],[51,0],[51,0],[51,0],[51,0],[51,0],[51,0]],[[5227,4422],[1,-2],[-1,-3],[-2,-4],[0,-3],[2,-1],[-1,-4],[-1,-1],[1,-1],[0,-2],[1,-1],[-1,-6],[3,-11],[-1,-4],[2,-5],[0,-3],[0,-2],[-1,-5],[-1,-2],[0,-2],[-1,-4],[-1,-2],[1,-12],[0,-2],[-1,-1],[-1,-1],[0,-3],[1,-5],[0,-2],[0,-1],[1,-2],[1,-2],[1,-1],[1,-1],[1,-1],[1,-2],[-3,-1],[-1,-3],[0,-4],[0,-2],[1,-3],[1,0],[1,-1],[1,-2],[0,-1],[0,-1],[0,-2],[0,-2],[0,-2],[1,-1],[1,-1],[0,-1],[1,-1],[1,-2],[-1,-2],[-1,-1],[1,-7],[0,-1],[1,0],[0,-2],[0,-3],[0,-3],[-1,-1],[-1,-3],[2,-1],[2,1],[2,2],[2,1],[1,0],[-1,-2],[-2,-1],[-1,0],[-2,-4],[-1,-1],[-1,0],[0,-3],[1,0],[-5,-2],[-1,-2],[1,-3],[-1,-3],[-1,-1],[-1,-1],[2,-1],[0,-2],[0,-2],[-2,-2],[1,-1],[1,-2],[-1,-5],[0,-1],[-1,-1],[0,-1],[0,-1],[2,-3],[0,-1],[0,-2],[-1,-4],[-1,-2],[0,-9],[0,-1],[-6,-3],[-7,1],[-7,0],[-4,-8],[4,2],[17,1],[2,-1],[0,-1],[-1,-3],[0,-2],[0,-1],[1,-1],[0,-1],[-1,-2],[5,0],[5,-3],[3,-5],[1,-8],[0,-5],[-1,-3],[-3,-7],[-1,-2],[-1,0],[0,-2],[1,-1],[0,-2],[-1,-3],[-1,-3],[7,9],[4,3],[5,-1],[7,-6],[4,-4],[3,-4],[1,-6],[2,-13],[2,-5],[4,-2],[5,1],[7,6],[0,2],[0,1],[1,2],[1,5],[1,2],[2,1],[4,2],[1,0],[0,-2],[-1,-1],[-1,-1],[-1,-1],[1,-2],[0,-3],[-1,-3],[0,-4],[0,-4],[1,-2],[2,-1],[0,-2],[0,-3],[-1,-4],[-2,-2],[-3,-3],[-2,-3],[-2,-8],[-1,-6],[2,-14],[-2,-9],[2,-8],[0,-3],[-1,-4],[-3,-5],[-1,-4],[0,-2],[0,-4],[0,-2],[0,-3],[-1,-5],[0,-7],[-2,-11],[-1,-3],[1,-1],[2,10],[2,5],[2,3],[0,1],[2,9],[1,1],[2,1],[0,2],[-1,2],[4,6],[0,3],[0,2],[0,6],[0,2],[-1,2],[1,15],[0,5],[0,2],[-1,3],[0,2],[0,3],[0,3],[0,3],[2,5],[5,9],[2,6],[-1,2],[0,4],[1,3],[1,2],[1,2],[-1,3],[-2,4],[-1,3],[4,-3],[3,0],[7,1],[20,-4],[2,1],[1,1],[2,3],[2,1],[2,-1],[1,-1],[2,0],[2,1],[3,3],[1,1],[4,-1],[13,-6],[2,0],[2,2],[5,0],[1,0],[1,-4],[0,-5],[1,-4],[3,-2],[2,0],[1,-1],[2,0],[1,2],[3,7],[1,2],[3,2],[10,2],[3,0],[2,-2],[2,-4],[2,0],[0,-4],[2,-10],[1,-3],[-1,-3],[0,-5],[0,-9],[1,-3],[1,-4],[0,-3],[-1,-5],[0,-3],[1,-4],[3,-8],[5,-13],[1,-9],[1,-5],[1,-4],[4,-9],[2,-4],[0,-5],[1,-4],[3,-6],[0,-2],[2,-5],[1,-2],[1,-2],[3,-2],[1,-2],[1,-2],[-1,-2],[0,-2],[1,-2],[1,-3],[1,-6],[0,-2],[2,-1],[1,-3],[1,-6],[0,-13],[4,-21],[1,-11],[-3,-6],[1,-1],[4,-1],[1,-1],[1,-3],[1,-5],[2,-7],[3,-12],[3,-5],[3,-6],[1,-3],[1,-4],[4,-13],[1,-2],[-1,-6],[0,-2],[2,-2],[3,-2],[1,-2],[1,-1],[1,-3],[0,-2],[1,-5],[1,-2],[1,-2],[1,-2],[1,-4],[1,-11],[2,-8],[0,-5],[-1,-10],[-1,-9],[-2,-8],[-10,-28],[-6,-14],[-8,-11],[-2,-8],[-2,-3],[-2,-2],[-4,-1],[-10,-6],[-4,-1],[2,-2],[16,5],[5,3],[16,19],[5,5],[24,6],[4,3],[2,-1],[1,-2],[-1,-2],[-1,-2],[-2,-1],[-2,-2],[-8,-13],[-1,-1],[-2,-1],[-4,-1],[-2,-1],[-4,-4],[-1,-2],[0,-1],[5,3],[10,3],[2,2],[10,13],[13,5],[17,4],[22,13],[32,13],[31,15],[20,8],[20,0],[19,-8],[9,1],[3,-1],[8,-7],[3,-1],[6,-1],[3,-1],[2,-3],[1,-5],[7,-3],[6,-7],[11,-5],[1,-3],[1,0],[9,-5],[1,-1],[8,-10],[3,-3],[10,-1],[6,-5],[28,-8],[7,-4],[4,-1],[10,1],[3,-1],[5,-5],[6,-1],[9,-6],[9,-1],[2,-1],[2,-3],[2,-2],[2,0],[2,1],[2,1],[6,0]],[[4484,1767],[-21,0],[-25,0],[-24,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-24,0],[-25,0],[-9,0],[-16,0],[-25,0],[-25,0],[-25,0],[-25,0],[-24,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-24,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-24,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-10,0]],[[3508,1767],[0,2],[0,82],[0,83],[0,83],[0,83],[0,83],[0,83],[0,83],[0,83],[0,83],[0,83],[0,83],[0,83],[0,83],[0,83],[0,82],[0,83],[0,83],[0,83],[0,83],[0,83],[0,83],[0,83],[0,83],[0,83],[0,83],[0,83],[0,83],[0,83],[0,82],[0,83],[0,83],[0,83]],[[3508,4422],[56,0],[57,0],[56,0],[57,0],[56,0],[57,0],[56,0],[57,0],[56,0],[57,0],[57,0],[56,0],[57,0],[56,0],[57,0],[56,0]],[[3508,1767],[-15,0],[-24,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-24,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-24,0],[-25,0],[-23,0]],[[3048,1767],[0,2],[1,2],[1,1],[1,2],[0,3],[-1,3],[-2,2],[-3,0],[-2,1],[-1,2],[-2,3],[-2,3],[-1,3],[0,3],[0,2],[1,3],[0,2],[-1,1],[-2,2],[-2,3],[-2,1],[-1,2],[-2,1],[-2,0],[-2,0],[-2,1],[-1,1],[-1,0],[-2,0],[-2,0],[-2,0],[-2,2],[-1,3],[0,3],[1,2],[1,1],[0,2],[-1,2],[-2,0],[-2,1],[-1,0],[-1,2],[0,1],[0,2],[-1,2],[0,1],[-1,1],[-1,1],[0,1],[0,1],[-1,2],[0,2],[0,1],[-2,2],[-1,1],[-1,0],[-2,2],[0,1],[-1,0],[0,1],[0,1],[-1,1],[-1,1],[-1,0],[-2,5],[-2,5],[-1,7],[1,6],[1,4],[2,7],[0,6],[-1,4],[-1,0],[-2,-1],[-1,-2],[-1,0],[-1,0],[-2,1],[-1,1],[-1,0],[-1,-1],[-1,0],[-1,0],[-1,0],[0,2],[0,1],[0,1],[-1,0],[-1,1],[-1,0],[0,1],[-2,2],[0,1],[0,2],[0,3],[1,3],[1,1],[2,1],[1,1],[2,0],[0,1],[0,2],[0,2],[1,2],[1,2],[0,2],[0,2],[0,1],[0,1],[0,1],[0,1],[1,1],[0,1],[1,1],[1,1],[0,3],[0,2],[-1,1],[0,3],[-1,1],[1,1],[1,1],[1,1],[0,2],[0,2],[0,3],[-1,6],[-2,5],[-2,5],[-1,6],[-1,3],[0,5],[0,5],[1,3],[1,1],[1,0],[1,0],[1,2],[0,3],[0,2],[-1,3],[-1,2],[0,3],[0,2],[0,2],[0,2],[0,1],[0,1],[0,1],[0,2],[-1,0],[-1,0],[0,1],[-1,2],[-1,2],[0,2],[-1,2],[-1,1],[0,1],[-1,2],[0,3],[-1,3],[0,3],[0,4],[0,3],[0,3],[-1,4],[-1,3],[0,2],[0,3],[-1,3],[0,4],[0,1],[0,2],[-1,2],[-1,3],[-1,3],[0,2],[1,2],[0,2],[-2,4],[-2,3],[-2,3],[-1,1],[-1,1],[-1,1],[-1,2],[0,2],[-1,2],[0,2],[-1,2],[-3,5],[-2,4],[-2,5],[-2,5],[-2,3],[-1,3],[-1,3],[-2,2],[-1,1],[-2,1],[-2,0],[-2,1],[-2,1],[-4,-1],[-3,-2],[-2,-3],[0,-3],[-1,-1],[-2,-1],[-1,0],[-1,2],[-1,2],[0,2],[-1,3],[-2,3],[-4,5],[-2,5],[-1,3],[2,1],[1,0],[0,1],[0,3],[0,2],[-1,2],[-1,3],[0,2],[-1,2],[-1,1],[-1,2],[-1,1],[-2,1],[-1,-2],[-1,-2],[-1,0],[-2,1],[0,3],[-1,2],[-1,3],[-1,1],[-2,0],[-2,0],[-1,3],[-1,2],[-2,1],[-1,1],[-2,1],[-1,2],[-2,2],[-1,3],[-1,3],[-2,2],[-4,1],[-2,1],[0,6],[1,2],[2,1],[2,1],[2,0],[1,2],[0,2],[-2,1],[0,2],[-1,2],[-1,2],[0,2],[-1,3],[0,3],[-2,1],[-1,0],[-2,1],[0,2],[-1,1],[0,2],[-1,2],[-2,2],[-2,1],[-3,1],[-2,2],[-1,2],[0,2],[0,2],[0,2],[-1,2],[-2,1],[-1,1],[-1,0],[-3,1],[-4,0],[-3,0],[-3,2],[-3,2],[-3,3],[-3,3],[-1,3],[-1,5],[0,4],[0,5],[0,5],[-3,5],[-4,2],[-4,1],[-3,3],[0,2],[-1,2],[0,2],[-1,2],[-2,1],[-2,0],[-2,0],[-2,0],[-1,1],[-1,2],[-1,2],[-1,3],[0,1],[0,2],[0,1],[-1,2],[0,1],[-1,1],[-1,1],[0,2],[0,5],[0,6],[0,6],[-2,4],[-3,1],[-3,2],[-2,3],[-1,6],[0,3],[-1,2],[0,2],[-1,2],[-2,1],[-2,1],[-1,1],[-2,3],[0,3],[-1,2],[0,2],[-1,2],[-2,2],[-3,1],[-2,1],[-2,2],[-2,5],[-1,6],[0,5],[-1,4],[-1,1],[-1,2],[0,2],[-1,1],[-1,2],[-1,2],[0,3],[0,2],[-1,3],[-1,1],[-2,1],[-1,0],[-2,-1],[-1,-1],[-2,0],[-1,0],[-2,-2],[-1,-2],[-1,-3],[-1,-2],[-1,-1],[-1,-1],[-1,-2],[0,-1],[1,-1],[0,-1],[1,-1],[-2,-4],[-4,-2],[-4,0],[-3,1],[-2,3],[-2,5],[-1,6],[-2,5],[0,2],[-1,2],[0,3],[0,2],[-1,3],[-1,1],[-1,2],[-1,1],[-1,2],[0,2],[0,3],[0,2],[-1,3],[-1,3],[-1,2],[-2,2],[-1,2],[-1,2],[-1,2],[-1,2],[-2,2],[-2,1],[-2,1],[-2,1],[-2,2],[-1,3],[-2,3],[-1,2],[-1,1],[0,1],[-1,0],[0,1],[-1,1],[-1,0],[0,1],[0,1],[0,2],[-1,3],[-1,2],[-1,2],[0,4],[0,2],[0,3],[0,2],[0,3],[-1,2],[0,3],[-1,2],[0,1],[-2,0],[-1,-2],[0,-3],[0,-3],[-4,-3],[-5,0],[-4,2],[-4,1],[-4,0],[-4,-3],[-3,-1],[-3,2],[-3,6],[-3,4],[-2,2],[-4,1],[-4,2],[-2,4],[-3,5],[-2,6],[0,4],[2,4],[4,4],[3,4],[1,5],[0,5],[-1,5],[-2,3],[-5,3],[-5,1],[-5,1],[-5,4],[-2,3],[-2,4],[-3,3],[-2,0],[-1,-1],[0,-2],[-1,-2],[0,-2],[0,-1],[-1,-1],[-1,0],[0,-2],[1,-2],[0,-1],[0,-2],[0,-2],[-3,-2],[-3,-1],[-4,0],[-3,0],[-2,-2],[-2,-3],[-2,-2],[-2,1],[0,1],[0,2],[-1,2],[0,2],[-1,2],[-1,3],[-1,2],[0,4],[1,2],[2,1],[2,1],[1,2],[0,3],[-1,1],[-2,1],[-1,0],[-1,1],[-1,2],[-2,3],[0,2],[-1,2],[1,2],[0,2],[0,2],[-1,3],[-2,1],[-1,2],[-2,1],[0,2],[-1,2],[0,1],[-1,2],[0,3],[1,2],[1,3],[2,1],[1,3],[0,3],[-1,2],[-1,2],[-2,5],[-1,4],[-1,4],[-2,3],[-1,1],[-2,1],[-2,2],[-1,3],[0,2],[0,2],[1,2],[0,3],[0,3],[-1,2],[-2,1],[-2,1],[-1,1],[0,3],[0,2],[-1,3],[-2,1],[-2,0],[-2,0],[-2,-1],[-1,-1],[-2,-1],[-2,-2],[-1,0],[-2,2],[-1,5],[-1,6],[-1,4],[-3,4],[0,4],[1,3],[0,5],[-1,5],[-1,1],[-1,-1],[-2,1],[-1,2],[0,2],[-1,1],[-1,0],[-2,-1],[-1,-1],[-1,0],[-1,4],[1,2],[1,2],[1,2],[1,3],[0,2],[-1,2],[-1,1],[-1,1],[-3,5],[-3,4],[-3,4],[-3,3],[-1,1],[-2,1],[-2,1],[-1,2],[-2,2],[-1,2],[-2,1],[-2,0],[-1,-2],[-1,-2],[0,-2],[0,-2],[0,-3],[0,-1],[-1,-1],[-1,-2],[1,-2],[0,-2],[1,-2],[0,-2],[-2,-2],[-2,1],[-2,2],[-1,2],[-2,0],[-1,1],[-1,0],[-2,0],[-1,2],[-1,2],[-1,2],[-2,0],[-2,0],[-1,1],[-1,1],[-2,0],[-2,1],[-1,0],[-1,0],[-2,3],[-2,4],[-2,5],[-3,4],[-2,5],[0,2],[-1,2],[-1,3],[0,3],[0,3],[0,2],[-1,2],[-1,2],[-2,1],[-2,0],[-2,-1],[-1,-1],[-3,0],[-2,0],[-2,1],[-2,1],[-2,1],[-2,1],[-2,1],[-2,0],[-1,-2],[-1,-2],[-1,-2],[-1,0],[-3,2],[-3,3],[-3,3],[-3,3],[-1,4],[-2,4],[-1,5],[-1,4],[-2,6],[-2,3],[-3,1],[-2,-1],[-1,0],[-2,1],[-1,2],[0,3],[0,2],[1,2],[1,2],[0,2],[-1,3],[-2,2],[-1,2],[-1,3],[1,2],[2,0],[3,0],[1,0],[4,-2],[5,-1],[4,0],[3,5],[-1,4],[-2,4],[-2,3],[-2,3],[-1,2],[0,2],[-1,2],[-1,1],[-2,-1],[-1,-1],[-1,-1],[-1,1],[-2,2],[-1,0],[-2,0],[-1,1],[-1,2],[0,2],[0,1],[1,2],[0,2],[1,2],[0,2],[0,2],[-2,2],[-2,0],[-2,0],[-3,1],[-1,4],[-1,4],[0,5],[0,5],[0,13],[0,12],[0,13],[0,12],[0,46],[0,47],[0,46],[0,46],[0,29],[0,56],[0,56],[0,70],[0,70],[0,70],[0,70],[0,70],[0,70],[0,66],[0,65],[0,66],[0,65],[0,66],[0,66],[0,65],[0,66],[0,20],[0,19],[0,20],[0,20],[0,20],[0,20],[0,20],[0,20]],[[2376,4422],[71,0],[71,0],[70,0],[71,0],[71,0],[70,0],[71,0],[71,0],[71,0],[70,0],[71,0],[71,0],[70,0],[71,0],[71,0],[71,0]],[[2010,1629],[-1,0],[-1,1],[1,1],[1,0],[1,-1],[0,-1],[-1,0]],[[1998,1673],[1,-2],[-2,-2],[0,1],[-1,2],[1,-1],[0,1],[0,1],[-1,1],[-1,1],[-1,1],[1,0],[2,-1],[1,-1],[0,-1]],[[2000,1682],[1,-3],[1,-2],[2,-1],[0,-1],[1,0],[0,-3],[-1,-2],[-1,1],[-1,3],[-2,3],[-1,3],[0,-1],[0,1],[0,3],[0,1],[1,-2]],[[2003,1696],[-1,0],[-1,2],[-1,2],[-1,1],[1,2],[0,1],[2,0],[1,-2],[1,-2],[-1,-3],[0,-1]],[[1994,1705],[1,0],[1,-1],[0,-1],[-1,-2],[0,-1],[-1,0],[0,1],[-1,1],[0,1],[1,1],[0,1]],[[2009,1712],[1,-1],[2,0],[2,-1],[2,0],[0,-3],[0,-2],[-2,-1],[-3,1],[-1,0],[-2,1],[-3,3],[-1,1],[-2,2],[0,1],[0,5],[0,2],[-2,2],[0,1],[2,1],[1,0],[1,-1],[3,-3],[1,-2],[1,-3],[0,-3]],[[2027,1712],[-2,-1],[-2,1],[-2,0],[-2,-1],[-1,2],[-2,1],[-2,3],[0,2],[0,2],[0,1],[-1,2],[-1,1],[0,1],[0,2],[2,-1],[3,-3],[4,-3],[6,-1],[4,-1],[1,-1],[0,-1],[-1,-1],[-2,-1],[-1,0],[-1,-3]],[[1993,1731],[1,-1],[2,-2],[1,-2],[-1,0],[0,-1],[0,-1],[-1,1],[0,-1],[0,-1],[1,-1],[-1,1],[-2,1],[0,1],[-1,1],[-1,2],[1,1],[-1,0],[0,2],[1,-1],[1,1]],[[1789,1728],[-1,1],[0,1],[-1,2],[1,0],[2,-1],[-1,-2],[0,-1]],[[2004,1738],[5,-4],[-2,1],[0,-1],[2,-2],[0,-1],[0,-2],[0,-1],[0,-1],[1,-1],[1,-1],[0,-1],[-1,0],[-10,4],[0,1],[0,1],[0,1],[-1,1],[0,1],[1,1],[1,-1],[1,0],[1,1],[-1,2],[1,1],[0,1],[1,0]],[[1778,1736],[-2,0],[-1,0],[0,2],[1,1],[-1,1],[1,0],[1,-3],[1,-1]],[[1794,1739],[-1,-1],[0,1],[-1,-1],[1,2],[1,1],[1,2],[1,1],[1,-1],[0,-2],[-1,-1],[-2,-1]],[[1773,1742],[-1,0],[0,1],[0,1],[-1,1],[0,1],[1,0],[1,-2],[0,-2]],[[1775,1748],[-1,-2],[-1,0],[0,1],[0,1],[1,0],[1,0]],[[1781,1747],[0,2],[2,-1],[0,-1],[-2,0]],[[1780,1750],[0,-1],[0,-1],[-1,0],[-1,0],[0,1],[1,1],[-1,0],[-1,0],[0,1],[1,0],[2,-1]],[[1783,1752],[1,-1],[0,-1],[-1,1],[0,-1],[0,-1],[-1,1],[-1,1],[0,1],[1,-1],[0,1],[-1,0],[-1,0],[0,1],[-1,1],[1,0],[1,-1],[1,0],[1,0],[0,-1]],[[1980,1742],[8,-12],[-1,0],[-2,1],[-1,2],[-2,2],[-1,0],[-1,-1],[0,-2],[1,-1],[2,-4],[12,-13],[-1,-4],[-3,-1],[-6,3],[1,-1],[2,-6],[1,-2],[-2,-1],[-2,-2],[-3,-1],[-1,1],[-2,2],[-1,1],[-1,1],[-1,3],[-1,3],[0,3],[0,3],[2,3],[0,2],[-3,3],[-2,4],[0,4],[3,3],[-3,5],[-1,4],[-1,5],[0,8],[0,-1],[1,-1],[0,-3],[9,-10]],[[1801,1744],[-2,-1],[-1,1],[-1,1],[0,1],[1,0],[0,1],[0,1],[0,1],[0,1],[1,1],[0,1],[1,1],[1,0],[0,2],[1,1],[0,1],[1,0],[0,1],[1,-1],[1,0],[0,-2],[-1,-3],[0,-2],[0,-1],[-1,-1],[-1,-2],[-1,-1],[0,-1]],[[1963,1765],[0,-1],[1,0],[1,0],[0,-4],[1,-5],[-1,-2],[-1,1],[-1,1],[0,1],[-1,2],[0,2],[0,1],[0,1],[-1,1],[1,1],[0,1],[1,0]],[[1960,1773],[3,-5],[0,-2],[0,-1],[-2,0],[0,-1],[0,-1],[0,-1],[0,-1],[-1,1],[0,1],[-1,1],[-1,1],[0,3],[-1,3],[0,2],[1,-1],[1,1],[1,0]],[[2001,1737],[-1,-1],[-2,1],[-3,-2],[-3,3],[-5,7],[-10,10],[-12,19],[-2,3],[-1,1],[-4,10],[-1,5],[1,2],[1,0],[3,-9],[10,-19],[3,-4],[8,-9],[2,-2],[3,0],[5,-3],[2,-3],[4,-1],[1,-1],[1,-4],[0,-3]],[[1958,1802],[-1,-1],[-1,0],[-2,-1],[-2,0],[-3,1],[-7,4],[-3,3],[0,3],[0,1],[0,1],[-1,0],[0,2],[1,1],[4,1],[1,-1],[0,-1],[1,-2],[5,-4],[5,-2],[2,-1],[1,-2],[0,-2]],[[1702,1805],[-1,-1],[-1,1],[-4,2],[0,2],[-1,3],[-1,2],[0,3],[1,3],[2,0],[2,-1],[3,-1],[1,-1],[1,-1],[1,-1],[0,-2],[-1,0],[-1,-1],[-1,-2],[0,-5]],[[1720,1803],[-5,-2],[-4,3],[0,3],[-1,3],[-3,1],[-1,5],[1,4],[2,3],[4,3],[4,0],[3,-3],[2,-4],[-1,-7],[-1,-9]],[[1690,1836],[2,-3],[-2,-3],[-3,0],[-5,2],[-2,3],[-4,-1],[-4,1],[-2,6],[2,2],[0,3],[-1,3],[-1,2],[1,4],[0,2],[0,3],[1,1],[2,0],[7,3],[2,0],[2,-3],[3,-5],[1,-6],[-1,-5],[1,-3],[0,-3],[1,-3]],[[1987,1863],[-1,0],[-1,0],[-2,0],[-2,-2],[-1,0],[1,2],[1,2],[2,2],[0,1],[2,0],[1,-2],[1,-1],[-1,-2]],[[1994,1850],[-1,-1],[-3,1],[-1,1],[-1,1],[1,2],[0,3],[1,3],[3,6],[2,2],[2,1],[3,1],[1,-2],[0,-3],[-1,-2],[0,-2],[-1,-2],[0,-2],[-1,-3],[-2,-3],[-2,-1]],[[1928,1880],[-1,2],[-1,1],[0,2],[-1,1],[0,1],[1,0],[0,1],[0,1],[0,1],[-1,0],[1,0],[0,2],[0,-1],[1,0],[1,-2],[1,-2],[0,-1],[0,-1],[0,-1],[0,-1],[0,-1],[0,-1],[-1,-1]],[[1887,1892],[3,0],[2,0],[1,-1],[2,-1],[2,-1],[7,-8],[0,-3],[-1,-2],[-6,3],[-1,0],[-1,0],[-2,-1],[-3,1],[-2,1],[-4,3],[-1,1],[0,1],[1,2],[-2,2],[-1,0],[0,2],[0,2],[1,1],[2,1],[3,-3]],[[2000,1886],[0,-6],[-2,-5],[-4,-1],[1,4],[0,2],[-2,1],[-2,0],[-1,-2],[-1,-2],[-1,-1],[-2,0],[0,3],[-1,3],[0,2],[2,2],[0,2],[-1,3],[1,3],[3,3],[3,1],[1,1],[1,0],[1,0],[1,-1],[1,-7],[2,-5]],[[1856,1896],[1,-2],[-2,0],[-2,1],[-1,-2],[-1,-3],[-1,-2],[-1,0],[-1,1],[-2,2],[-2,3],[0,4],[2,3],[4,0],[5,-4],[1,-1]],[[1846,1885],[-2,-1],[-1,0],[-1,1],[-3,1],[-9,13],[-1,4],[0,5],[0,5],[1,1],[3,-4],[1,-2],[9,-19],[1,-1],[1,-1],[1,-2]],[[1643,1926],[-2,-2],[0,-1],[-1,0],[0,-1],[-1,-5],[-2,-1],[-1,0],[2,6],[1,2],[-1,0],[0,-2],[-1,-2],[-1,0],[-3,1],[0,1],[1,2],[0,1],[1,1],[1,1],[1,2],[0,1],[0,2],[1,1],[1,0],[1,-1],[3,-3],[0,-1],[0,-1],[0,-1]],[[1922,1956],[-1,0],[0,-1],[-6,-8],[0,-2],[-1,-2],[-3,-11],[-3,-5],[-3,1],[-2,4],[-2,7],[6,10],[2,1],[1,1],[2,-2],[0,3],[1,2],[1,0],[4,3],[3,0],[1,-1]],[[1887,1933],[0,-3],[3,-5],[1,-2],[1,-3],[1,-3],[11,-15],[3,-6],[3,-7],[-2,-2],[-3,1],[-4,6],[-20,20],[-7,2],[0,1],[0,2],[0,2],[-3,5],[-1,4],[0,1],[-1,0],[-1,0],[-3,2],[0,1],[-2,1],[-4,4],[-5,2],[-2,3],[-3,5],[1,2],[0,1],[0,2],[-1,1],[0,2],[1,2],[0,1],[-1,2],[2,0],[1,0],[2,-1],[1,-1],[1,-2],[3,-6],[1,-2],[5,-1],[4,-3],[10,-4],[5,-4],[3,-5]],[[1849,1972],[-1,-1],[0,3],[0,4],[2,3],[0,1],[1,-2],[1,-3],[-1,-3],[-1,-1],[-1,-1]],[[1614,1975],[2,-1],[2,2],[1,-1],[2,-5],[2,-11],[1,-4],[1,-3],[0,-5],[1,-11],[1,-10],[1,-5],[-3,-9],[-4,-1],[-15,9],[-1,2],[-3,4],[-1,3],[-7,6],[-7,11],[1,5],[1,1],[1,-2],[2,-3],[1,2],[1,3],[1,1],[3,1],[2,1],[1,2],[-1,2],[-1,0],[-1,-1],[-1,-1],[-1,0],[-1,1],[-1,2],[-1,0],[-1,1],[-1,2],[-1,1],[-1,0],[-3,1],[1,3],[1,2],[2,0],[10,5],[7,6],[4,1],[1,-3],[2,-4]],[[1818,2010],[1,0],[1,-1],[1,0],[1,0],[2,-2],[-1,-2],[-1,-3],[-2,0],[-1,1],[-1,2],[-1,0],[0,2],[0,3],[0,1],[1,-1]],[[1556,2010],[-2,-6],[-2,1],[-1,2],[-1,1],[-2,2],[0,2],[1,1],[0,2],[0,1],[1,2],[2,1],[3,-1],[2,-3],[1,-2],[0,-1],[0,-1],[-1,0],[-1,-1]],[[1813,2015],[-1,-2],[-2,1],[-1,3],[1,3],[0,2],[0,3],[0,2],[-1,2],[-4,6],[-1,3],[1,2],[3,5],[4,8],[1,2],[0,3],[-2,5],[-1,3],[1,0],[1,-2],[7,-13],[1,-5],[0,-1],[0,-2],[0,-1],[-1,-1],[1,-2],[0,-1],[1,-1],[0,-3],[0,-3],[1,-2],[1,-3],[-3,-2],[-4,-1],[-1,-1],[-2,-7]],[[1792,2011],[0,-3],[-2,2],[-1,2],[0,5],[-1,3],[-2,4],[-1,2],[0,1],[2,1],[-2,3],[-6,7],[-2,2],[0,1],[-2,2],[-1,1],[-1,3],[0,3],[0,7],[-2,6],[0,5],[1,5],[1,3],[2,2],[3,2],[3,1],[2,-2],[1,-5],[1,-1],[1,-1],[2,-5],[2,-2],[1,-2],[1,-4],[0,-3],[-1,-3],[-1,-3],[-1,-2],[1,-3],[-3,-2],[-2,-1],[0,-3],[2,-4],[1,-2],[1,0],[1,-1],[0,-2],[1,-2],[0,-8],[1,-9]],[[1831,2061],[3,-3],[3,-3],[1,-5],[-1,-3],[-2,-4],[-3,-3],[-2,-2],[-2,1],[-1,3],[-1,5],[-1,3],[2,2],[2,1],[1,1],[-1,4],[-2,0],[-4,-2],[-2,1],[-2,1],[-4,8],[0,4],[0,3],[1,1],[2,2],[1,4],[1,3],[4,-3],[5,-2],[2,-2],[1,-1],[-1,-2],[0,-4],[0,-8]],[[1842,2083],[4,-3],[3,-7],[1,-9],[-1,-8],[-4,-5],[-2,-2],[-1,0],[-1,1],[0,2],[-1,3],[0,4],[0,4],[2,5],[2,5],[0,3],[-1,3],[-1,1],[-1,0],[1,-2],[0,-2],[0,-2],[-1,-3],[-3,-8],[-1,-2],[-2,2],[-2,2],[-1,1],[0,2],[2,5],[1,4],[1,3],[2,1],[1,2],[3,0]],[[1788,2104],[1,0],[2,0],[1,-1],[1,-6],[0,-2],[0,-2],[3,-6],[2,-6],[4,-8],[1,-3],[0,-2],[1,-2],[1,-3],[0,-4],[-1,-2],[-1,0],[-1,1],[-1,1],[-1,-10],[-1,-4],[-3,-3],[-1,-1],[0,-1],[-2,2],[0,2],[2,7],[1,5],[0,5],[-1,5],[-3,3],[-2,4],[-3,14],[0,1],[-2,0],[-1,-2],[-1,0],[-1,1],[-2,2],[-1,0],[-1,-1],[-3,-1],[-1,-1],[0,-2],[-1,-1],[-3,-2],[-3,0],[-1,3],[-1,6],[1,4],[3,0],[4,-2],[2,2],[0,3],[0,5],[0,4],[1,3],[1,4],[1,2],[6,-5],[1,-1],[1,-1],[1,-3],[1,-1]],[[1754,2110],[0,-3],[-10,-4],[-2,1],[-2,0],[-1,-2],[-12,-1],[-3,0],[-1,1],[-1,1],[0,1],[0,1],[0,1],[0,1],[1,0],[4,2],[6,0],[3,2],[3,1],[2,2],[1,1],[2,0],[5,2],[2,-1],[2,-2],[1,-4]],[[1764,2097],[-4,-2],[-4,1],[-4,3],[-2,1],[-1,1],[1,2],[2,1],[2,3],[1,2],[0,3],[2,1],[7,5],[2,1],[1,2],[2,0],[3,-2],[2,-5],[-2,-5],[-1,-2],[-1,-1],[-3,-1],[1,-1],[0,-2],[0,-2],[-2,-2],[-2,-1]],[[1722,2109],[-2,-1],[-3,0],[-5,2],[-4,0],[-5,2],[-2,4],[2,1],[1,0],[6,4],[4,2],[5,1],[2,0],[3,-3],[0,-2],[0,-1],[1,-1],[2,-2],[0,-2],[-1,-1],[-1,-1],[-3,-2]],[[1613,2152],[0,-1],[1,0],[4,-2],[0,-1],[1,-1],[0,-2],[0,-1],[-1,0],[-6,1],[-2,1],[-1,2],[0,1],[1,2],[1,0],[0,-1],[1,0],[0,2],[1,0]],[[1632,2152],[5,-1],[1,-1],[1,-1],[0,-2],[-1,-1],[-2,0],[-4,-2],[-4,-1],[-4,2],[-1,1],[1,1],[0,2],[-2,2],[0,1],[1,0],[4,0],[1,0],[2,1],[1,0],[1,-1]],[[1671,2144],[-1,-1],[-1,1],[-2,0],[-1,-1],[-2,-4],[-1,0],[-12,-2],[-1,-1],[-3,-1],[-20,4],[4,3],[25,8],[1,1],[1,3],[1,1],[2,1],[4,-1],[3,-5],[2,-2],[2,-1],[-1,-3]],[[1655,2156],[-6,-4],[-1,0],[-2,0],[-3,-3],[-2,0],[-1,1],[-1,1],[1,0],[0,1],[-2,1],[-1,1],[-1,0],[-2,1],[2,3],[8,3],[8,1],[2,-1],[2,-2],[-1,-3]],[[1580,2170],[12,-1],[4,1],[2,-1],[2,-3],[3,-3],[0,-2],[0,-1],[-1,1],[-1,1],[-1,-1],[-1,0],[-3,-1],[-12,1],[-4,3],[-3,-1],[-6,-1],[-4,2],[1,4],[2,3],[2,1],[4,-1],[4,-1]],[[1624,2194],[0,-2],[1,1],[1,0],[-1,-2],[-2,-3],[-1,1],[1,1],[-1,0],[0,-1],[-3,-1],[-2,1],[-1,1],[0,1],[1,1],[2,3],[1,0],[2,0],[2,-1]],[[1403,2199],[-1,0],[-3,1],[0,3],[0,1],[1,2],[3,0],[0,2],[1,-2],[2,-1],[1,-2],[-1,-1],[-1,-1],[-2,0],[0,-2]],[[1676,2189],[-5,-4],[-1,-2],[-3,-8],[1,-5],[-3,-2],[-7,-1],[-10,4],[-3,-1],[2,-2],[0,-1],[-1,-1],[-2,-1],[-1,1],[-3,2],[-1,1],[-7,-2],[-3,1],[-1,4],[1,1],[2,1],[0,2],[0,3],[4,8],[3,2],[2,-1],[3,4],[6,9],[3,4],[3,1],[4,0],[2,1],[4,3],[2,0],[3,-3],[1,-3],[1,-2],[1,-3],[0,-3],[4,-3],[0,-2],[-1,-2]],[[1394,2204],[-1,-1],[-1,2],[-2,1],[0,3],[1,1],[1,2],[0,-2],[2,0],[1,0],[1,-1],[-1,-2],[1,-1],[1,-1],[-1,-1],[-2,0]],[[1628,2212],[3,-4],[1,-2],[2,-3],[-1,0],[-3,2],[-1,0],[-1,-1],[-3,-2],[-4,-1],[-7,-4],[-1,0],[-2,2],[0,1],[0,-1],[-1,-1],[-1,-1],[-2,0],[-6,2],[-4,7],[-2,2],[0,2],[2,0],[1,1],[2,1],[4,0],[1,-2],[1,-4],[2,-1],[1,3],[0,3],[1,1],[10,-1],[2,1],[1,1],[1,1],[1,1],[1,-1],[0,-1],[1,-1],[1,0]],[[1512,2219],[2,0],[0,-1],[1,-1],[0,-1],[0,-1],[0,-1],[0,-1],[0,-2],[-1,-1],[-1,1],[0,1],[-1,3],[-1,2],[0,1],[1,1]],[[1485,2219],[35,-17],[4,-4],[2,0],[1,-2],[1,-2],[1,-4],[0,-2],[-1,-2],[1,-2],[1,-2],[0,-2],[1,-2],[2,5],[1,5],[2,3],[3,-1],[-2,-4],[2,-5],[3,-5],[3,-3],[10,-5],[3,-3],[2,-2],[1,-2],[1,-1],[2,-1],[6,0],[2,0],[0,-1],[1,-2],[0,-2],[1,-1],[1,0],[1,1],[1,0],[6,-2],[2,0],[-1,0],[1,0],[1,0],[1,-1],[0,-1],[-1,-2],[-1,-1],[2,-1],[12,-1],[2,0],[0,-2],[0,-2],[-1,-2],[0,-1],[1,-1],[1,1],[1,3],[1,1],[4,0],[3,-1],[10,-6],[2,-1],[3,-2],[4,-6],[3,-1],[1,2],[0,3],[1,1],[0,-1],[2,-1],[2,0],[3,-2],[1,-1],[2,-2],[11,3],[3,-1],[5,-3],[3,-1],[9,1],[8,-3],[11,-8],[7,-7],[3,-1],[11,-1],[5,-2],[4,-5],[2,0],[17,3],[3,-1],[4,-4],[6,-2],[2,-3],[2,-4],[2,-4],[-1,-2],[1,-3],[1,-4],[3,-14],[4,-13],[-2,-2],[2,-3],[4,-3],[1,-1],[2,-4],[3,-7],[0,-3],[4,-8],[1,-2],[0,-4],[0,-2],[1,-4],[2,-7],[1,-3],[34,-49],[1,-3],[0,-4],[0,-4],[-2,-1],[-9,3],[-1,-1],[0,-3],[1,-2],[1,0],[1,-2],[1,-1],[3,-4],[1,-3],[8,-25],[2,-1],[1,-1],[1,-2],[0,-4],[1,-2],[2,-3],[6,1],[2,-3],[1,-1],[3,-2],[1,-1],[2,-5],[1,-1],[4,-5],[6,-3],[11,-4],[7,0],[3,-1],[2,-3],[2,-2],[1,-1],[12,-6],[1,0],[2,1],[1,1],[2,-2],[5,-7],[-2,-1],[-4,-1],[-1,-3],[1,-1],[2,0],[3,1],[1,0],[3,-2],[2,-1],[7,0],[3,-1],[3,-3],[0,-2],[1,-2],[0,-1],[-2,-1],[0,-2],[1,-3],[3,-10],[1,-2],[2,0],[2,1],[5,-3],[1,1],[2,-3],[1,-3],[1,-4],[1,-5],[3,-6],[1,-4],[-3,-1],[1,-4],[1,-2],[-1,-2],[-1,-1],[-2,2],[-1,3],[-1,2],[-3,-2],[0,-1],[6,-9],[5,-5],[1,-2],[2,-3],[14,-21],[1,-3],[-1,-2],[-1,-1],[0,-2],[0,-1],[0,-1],[1,0],[1,1],[1,0],[1,-1],[0,-3],[0,-5],[-1,-4],[-1,1],[-2,2],[-2,1],[-2,1],[-2,-1],[0,-2],[2,-3],[8,-7],[1,-2],[3,-4],[0,-2],[0,-3],[-1,-2],[0,-2],[0,-3],[0,-2],[1,-2],[1,-2],[1,-4],[-1,-4],[-1,-2],[-1,-1],[0,-3],[0,-3],[3,1],[4,7],[0,4],[0,7],[0,4],[4,3],[-2,3],[-1,3],[-1,3],[2,1],[1,0],[2,0],[2,0],[1,-1],[1,-1],[0,-2],[0,-3],[0,-1],[1,-3],[0,-3],[-1,-1],[1,-2],[4,-14],[1,-9],[1,-4],[2,-3],[2,-3],[1,-4],[1,-5],[-1,-5],[-1,-3],[-3,-2],[-4,-1],[-9,11],[-1,-3],[-3,-10],[-2,-2],[-2,-1],[0,-2],[0,-2],[-1,-2],[-1,-2],[-2,-2],[-1,-3],[1,-3],[-3,-2],[-2,1],[0,4],[-2,0],[-4,-2],[-1,-1],[-2,1],[-2,3],[-2,2],[-3,3],[-5,3],[-7,0],[-6,1],[-27,16],[-29,20],[-3,1],[-3,1],[1,3],[3,4],[2,2],[-2,1],[-3,1],[-2,-1],[-1,-1],[-1,-2],[-2,-3],[-1,0],[-2,1],[-18,7],[-7,5],[-4,5],[-4,6],[-2,1],[-31,20],[-2,3],[-4,8],[-1,3],[-3,4],[1,3],[7,6],[1,2],[1,4],[1,1],[1,2],[1,1],[1,0],[3,1],[1,0],[1,1],[1,2],[0,4],[-1,2],[0,1],[1,3],[2,4],[3,2],[5,2],[4,4],[4,7],[3,10],[1,9],[1,5],[2,4],[1,4],[-3,9],[-1,14],[0,-1],[-1,-1],[-1,-1],[0,-2],[0,-2],[1,-9],[0,-10],[-1,-5],[-1,-3],[-2,-4],[-2,-3],[-1,-11],[-1,-4],[-2,-2],[-2,-1],[-5,-3],[-3,-1],[-5,2],[-2,-1],[-2,-2],[-1,0],[-3,1],[-2,2],[-2,1],[-3,-1],[-3,-6],[-2,-3],[-3,0],[-2,2],[-6,0],[-2,2],[3,7],[3,3],[2,2],[0,1],[-4,0],[-1,0],[-1,1],[-1,2],[-1,1],[-1,-1],[-1,-3],[-1,-5],[-1,-4],[-4,-4],[-3,-7],[-2,-2],[-1,0],[0,-2],[0,-2],[-1,0],[-3,0],[-1,0],[0,1],[-2,1],[-1,1],[-2,2],[-2,2],[-1,2],[-4,1],[-2,2],[0,2],[-1,3],[0,1],[-4,6],[-3,3],[-2,2],[-3,3],[-11,4],[-1,1],[-2,2],[-1,2],[0,3],[1,2],[4,1],[1,2],[2,-2],[2,-2],[2,0],[2,1],[1,1],[4,5],[1,0],[1,3],[5,4],[1,3],[0,3],[1,2],[1,2],[1,5],[-10,-11],[-3,-5],[-3,-3],[-1,5],[0,13],[2,7],[0,4],[-1,-1],[-2,-2],[-1,-2],[-2,0],[-1,2],[0,2],[0,9],[0,3],[1,2],[1,4],[1,4],[0,4],[-1,0],[-1,-3],[-2,-4],[-1,-4],[-1,-7],[0,-3],[-1,-3],[-1,-1],[-1,0],[-1,2],[-2,1],[-2,-1],[-5,-8],[-2,-2],[-2,0],[-2,2],[-2,3],[-2,5],[3,4],[3,3],[3,4],[3,13],[2,6],[1,5],[-2,4],[-1,-4],[-4,-7],[-1,-3],[0,-5],[-1,-4],[-2,-3],[-2,-2],[-1,2],[-3,2],[-1,2],[-2,9],[0,3],[2,4],[0,2],[0,4],[0,2],[-1,0],[-6,-5],[-2,0],[-2,-2],[-2,0],[-1,-1],[0,-1],[0,-1],[-2,0],[-2,1],[-1,1],[-1,2],[0,4],[-1,2],[-1,1],[-2,-1],[1,-3],[0,-4],[0,-4],[-2,-4],[-1,-4],[0,-1],[-1,0],[-1,1],[0,1],[0,1],[-4,3],[-1,1],[-1,2],[-2,2],[-2,1],[-1,2],[1,3],[-1,2],[-2,6],[-1,2],[-2,-1],[-2,-2],[-2,-3],[-1,-3],[2,-5],[1,-3],[0,-2],[-3,-3],[-2,-1],[-2,0],[-2,2],[-1,2],[-2,4],[0,3],[0,3],[1,5],[0,1],[0,6],[0,2],[0,2],[0,2],[0,2],[1,0],[0,2],[-1,1],[-1,1],[-1,3],[1,1],[0,2],[0,1],[0,2],[1,2],[0,1],[1,2],[2,2],[4,2],[5,6],[4,1],[9,-2],[14,3],[2,2],[2,1],[1,0],[3,-1],[1,-1],[5,3],[0,2],[-1,2],[-1,1],[-1,1],[-9,-1],[-11,-5],[-3,-3],[-2,0],[-13,6],[-2,3],[-2,2],[1,4],[8,15],[0,1],[-2,1],[-3,2],[-1,1],[-1,-2],[0,-9],[-1,-3],[-1,-4],[-3,-3],[-3,-1],[-3,1],[-3,2],[-1,4],[-4,21],[0,4],[0,9],[-1,6],[-2,0],[-2,-3],[-2,-2],[-3,1],[-2,4],[-2,4],[-2,7],[-2,3],[-5,4],[0,-3],[4,-4],[1,-4],[-1,-3],[-2,-3],[-1,-3],[-2,-1],[-2,-1],[-1,1],[-1,2],[-2,16],[-1,3],[-1,-1],[0,-2],[0,-2],[-1,-4],[0,-12],[0,-4],[-1,-1],[-15,0],[-3,-1],[-2,-1],[-3,3],[-10,19],[0,4],[0,3],[4,8],[1,1],[1,0],[0,2],[0,1],[-1,1],[0,1],[1,5],[2,4],[2,3],[4,3],[2,4],[1,4],[-2,5],[-1,-2],[0,-2],[0,-2],[-1,-2],[-1,0],[-2,-2],[-2,-3],[-3,0],[-3,1],[-1,2],[-1,2],[-2,4],[-2,3],[-2,1],[-1,-1],[1,-4],[1,-5],[0,-4],[0,-4],[-1,-3],[-1,-3],[-2,-2],[-3,-2],[-3,0],[-2,0],[-2,2],[-4,4],[-2,4],[-1,3],[-1,5],[1,2],[5,3],[-8,0],[1,3],[2,3],[4,6],[-4,2],[-4,-5],[-3,-6],[-3,-3],[-5,-2],[-2,-1],[-2,3],[-1,4],[1,3],[1,4],[1,2],[-4,-1],[-5,-5],[-4,-5],[-5,-11],[-1,-2],[-2,0],[-1,1],[-2,3],[0,1],[-6,0],[-2,2],[-1,4],[1,3],[6,8],[8,14],[2,1],[1,1],[0,3],[-1,1],[-1,0],[-3,-1],[-2,0],[-2,4],[-1,1],[-2,2],[1,2],[3,1],[2,2],[0,3],[-1,1],[-2,1],[-1,-1],[-1,0],[-3,4],[-1,1],[-1,0],[-2,-4],[-2,0],[-2,1],[-3,3],[-1,4],[1,2],[1,7],[2,5],[2,2],[0,2],[-1,2],[0,3],[1,2],[1,1],[4,-2],[2,1],[0,3],[4,-2],[3,2],[3,2],[3,1],[18,0],[2,-2],[2,-3],[6,-16],[3,-4],[3,-1],[-4,14],[-3,7],[-6,7],[0,7],[1,5],[3,0],[3,6],[8,3],[1,5],[-11,-3],[-2,-2],[-1,-3],[-2,1],[-1,2],[-2,3],[-2,2],[-1,0],[-23,2],[-7,5],[1,-3],[3,-4],[3,-2],[2,-1],[13,0],[6,-2],[2,-3],[2,-2],[0,-4],[-2,-2],[-6,0],[-19,-11],[-1,4],[-3,-1],[-4,-4],[-5,-2],[-5,-1],[-2,2],[-2,3],[-2,0],[-2,-4],[1,0],[1,-3],[1,-1],[-1,-1],[-2,-1],[-2,1],[-1,2],[-2,3],[-3,1],[-1,1],[-1,2],[-1,3],[-1,2],[-2,2],[-1,1],[-2,0],[0,1],[-1,1],[0,2],[-2,5],[0,5],[-1,3],[-4,3],[-3,-1],[-1,1],[0,2],[-2,4],[0,3],[1,1],[3,1],[2,2],[1,1],[-2,1],[-2,0],[-2,-2],[-1,0],[-1,1],[-2,1],[-1,2],[-1,2],[0,5],[-3,8],[0,4],[-1,3],[1,2],[3,1],[2,1],[0,1],[2,3],[1,0],[4,-1],[4,3],[5,4],[4,4],[4,-2],[6,7],[3,1],[2,-1],[4,-1],[11,2],[5,-1]],[[1612,2216],[-5,-2],[-2,-2],[-5,1],[0,-1],[-5,0],[-2,2],[2,1],[5,0],[-1,1],[-1,1],[0,1],[-1,0],[0,1],[1,1],[3,3],[3,0],[0,1],[2,1],[5,-2],[2,-3],[1,-2],[0,-2],[-2,0]],[[1510,2212],[-2,0],[-6,2],[-3,3],[-3,1],[-4,1],[-2,3],[0,2],[6,3],[4,3],[4,0],[3,-3],[2,-5],[0,-5],[1,-5]],[[1489,2230],[0,-2],[-1,-2],[-1,-1],[-2,0],[-1,0],[-1,1],[-3,0],[-2,2],[-2,-1],[-3,2],[0,2],[5,2],[1,1],[1,2],[1,0],[6,0],[1,-2],[1,-2],[0,-2]],[[1523,2264],[-2,-1],[-2,0],[-1,1],[-1,1],[-2,3],[0,1],[2,1],[4,3],[2,0],[2,-2],[0,-4],[-2,-3]],[[1485,2365],[1,-4],[-1,-3],[-2,-3],[-2,-2],[-2,-1],[-1,1],[-1,1],[-1,2],[0,1],[-3,3],[-3,4],[-1,-1],[-1,-1],[-2,0],[-3,2],[-1,5],[-2,5],[-1,5],[0,4],[-4,10],[-1,10],[2,7],[3,6],[9,9],[2,1],[2,-6],[2,-4],[1,-5],[-2,-3],[4,-5],[3,-8],[2,-9],[-2,-9],[2,-2],[2,-4],[1,-6]],[[1423,2474],[-1,-5],[-1,-1],[-1,4],[-3,-2],[-1,4],[0,7],[6,7],[0,3],[2,2],[0,-5],[0,-7],[-1,-7]],[[1475,2508],[2,-4],[-1,-8],[0,-3],[1,-4],[-2,-4],[-1,-10],[-1,-4],[-6,-16],[1,-2],[-1,-2],[1,-3],[1,-2],[1,-3],[0,-3],[-1,-3],[-1,-2],[-4,-2],[-1,3],[-2,-1],[-3,-2],[-2,-2],[-1,2],[1,2],[0,3],[0,1],[1,2],[0,1],[1,2],[0,4],[-1,2],[-1,5],[0,1],[0,2],[0,1],[-1,1],[-1,-2],[-1,0],[-5,-1],[-3,1],[-2,1],[0,3],[1,4],[1,4],[4,2],[0,2],[-2,5],[-1,2],[0,2],[2,3],[3,10],[2,5],[2,2],[1,0],[3,2],[2,1],[3,0],[7,2],[4,0]],[[1130,2518],[1,0],[1,1],[4,-2],[1,-2],[0,-4],[-1,-2],[-2,-1],[-4,1],[-3,-2],[-1,1],[-1,4],[-1,-5],[2,-3],[5,-5],[2,-3],[-1,-1],[-3,2],[-2,2],[0,-3],[3,-5],[1,-4],[-4,-1],[1,-1],[0,-2],[1,-4],[-3,2],[-3,4],[-4,8],[2,2],[1,2],[0,3],[-1,1],[-2,1],[0,4],[-1,4],[-1,6],[1,3],[1,8],[1,2],[1,1],[2,-1],[1,0],[1,-1],[0,-2],[0,-2],[2,-3],[2,-1],[1,-2]],[[1485,2534],[-1,-6],[-1,-3],[-3,-4],[-1,-2],[0,-3],[-1,-2],[-2,-2],[-6,2],[-3,-2],[-2,-1],[-2,1],[-2,2],[-1,2],[-1,3],[0,4],[-2,3],[0,3],[2,0],[4,-2],[1,1],[2,3],[1,1],[10,0],[1,0],[2,-1],[2,0],[1,1],[1,2],[1,0]],[[1447,2497],[-2,-1],[-1,1],[-1,1],[0,1],[0,1],[0,2],[-1,2],[-2,10],[-2,4],[0,4],[2,5],[1,2],[0,1],[2,4],[0,2],[1,1],[2,2],[2,1],[3,1],[2,-1],[2,-1],[-1,-2],[0,-22],[-1,-5],[-1,-1],[-2,-2],[-1,-1],[0,-2],[-1,-5],[0,-1],[-1,-1]],[[1440,2550],[3,-2],[2,-4],[-2,-2],[-2,-2],[-1,-3],[-1,-4],[0,-1],[-2,-8],[-1,0],[-2,4],[-1,-8],[2,-6],[-2,-3],[-3,2],[-2,3],[2,6],[1,5],[-2,1],[-2,2],[-2,-3],[-2,-4],[-2,-1],[-1,2],[2,3],[1,2],[-6,0],[-3,0],[-2,1],[1,4],[4,3],[1,4],[2,2],[0,4],[1,3],[4,-1],[5,0],[2,1],[8,0]],[[1476,2564],[2,-11],[1,-3],[1,-3],[1,-3],[1,-3],[-1,-5],[-2,-1],[-3,0],[-4,2],[-6,-1],[-3,0],[-2,3],[0,3],[0,2],[2,2],[4,2],[2,2],[0,3],[-2,1],[-2,-1],[-3,-2],[-2,-1],[-2,0],[-2,1],[-2,1],[-2,2],[8,5],[2,4],[3,2],[4,1],[4,-1],[3,-1]],[[1543,2555],[-1,-2],[0,-1],[-1,0],[-2,2],[-1,0],[-5,-4],[-1,-4],[-2,-3],[-5,-4],[-2,-1],[-3,0],[-1,-1],[-1,-2],[-1,-5],[-1,-1],[-2,-1],[-2,-2],[-6,-17],[-1,-6],[0,-5],[0,-5],[-2,-4],[-3,-2],[-3,-1],[-5,-1],[-2,-1],[-4,-3],[-2,0],[-2,4],[1,8],[2,19],[1,3],[1,2],[0,1],[1,1],[1,2],[-1,7],[1,5],[3,7],[2,5],[3,0],[2,8],[5,1],[5,0],[5,6],[5,2],[3,4],[1,1],[2,0],[2,1],[1,3],[3,2],[1,4],[1,4],[1,2],[2,3],[1,1],[12,5],[2,2],[3,3],[1,0],[2,-2],[1,-5],[-1,-5],[1,-4],[3,-4],[-4,-9],[-4,-6],[-10,-7]],[[1395,2608],[-1,-3],[1,1],[1,0],[1,1],[0,1],[3,-2],[1,-2],[0,-2],[0,-5],[0,-2],[-1,-1],[2,-11],[-1,-11],[-3,-10],[-4,-5],[0,4],[-4,6],[-1,3],[0,3],[0,5],[0,2],[-2,2],[0,1],[0,3],[0,1],[-1,4],[-1,5],[0,5],[1,4],[1,2],[3,1],[5,2],[0,-2]],[[1313,2626],[-1,-1],[-1,1],[1,3],[1,-3]],[[1091,2631],[1,-3],[0,-1],[0,-1],[0,-2],[-4,-1],[-1,1],[-1,0],[-2,1],[-1,0],[-1,2],[0,1],[1,0],[1,1],[1,0],[1,0],[0,2],[1,-1],[0,1],[1,0],[1,2],[1,-1],[1,-1]],[[1412,2637],[6,-27],[-2,-11],[-1,-1],[-2,-1],[-2,0],[-2,2],[-1,3],[-4,2],[-3,4],[-3,3],[-2,1],[-2,1],[-1,3],[-1,1],[-1,-2],[1,-2],[-1,-2],[-1,-1],[-2,2],[-2,-1],[-3,2],[-2,3],[-2,2],[0,5],[2,1],[1,0],[2,-2],[1,-2],[1,-1],[1,-2],[1,0],[1,0],[1,2],[1,1],[1,-2],[1,2],[-1,1],[-2,2],[0,1],[1,1],[-1,0],[-2,2],[-1,2],[1,4],[-1,1],[1,1],[0,1],[1,0],[16,1],[3,3],[-1,4],[1,6],[1,3],[2,-3],[1,-13]],[[1311,2648],[-2,-2],[-2,2],[2,3],[1,4],[-1,2],[2,1],[2,-2],[0,-4],[0,-4],[-2,0]],[[1058,2666],[-2,-3],[1,1],[2,1],[1,-2],[1,-3],[2,1],[6,0],[1,1],[1,4],[3,2],[3,2],[2,-1],[2,-4],[1,-3],[-1,-4],[-1,-7],[1,-3],[-2,-2],[-3,-1],[-2,0],[-2,2],[-5,5],[-2,1],[0,-1],[1,-1],[-1,-5],[4,-5],[-3,-5],[-2,0],[-2,2],[-4,10],[1,5],[-6,8],[-1,5],[3,4],[5,0],[8,-2],[-3,-2],[-7,0]],[[1063,2675],[-1,-2],[-12,0],[-3,3],[0,1],[0,1],[13,1],[2,-2],[1,-2]],[[1428,2583],[-2,0],[-4,0],[-4,2],[1,4],[2,3],[2,3],[2,3],[0,5],[1,2],[-3,5],[-1,9],[-1,21],[0,1],[-1,2],[0,1],[1,3],[1,3],[-1,5],[1,2],[1,4],[0,4],[-1,3],[-1,3],[1,5],[2,5],[3,3],[2,0],[2,-9],[0,-10],[3,-5],[1,-2],[0,-3],[0,-3],[0,-1],[1,-17],[1,-2],[0,-1],[0,-2],[1,-3],[0,-5],[1,-3],[0,-3],[0,-3],[0,-2],[0,-2],[0,-3],[-3,-7],[0,-2],[-8,-13]],[[1352,2663],[12,-19],[2,-3],[1,-4],[0,-3],[-1,-4],[0,-10],[0,-7],[-2,-7],[-4,-4],[0,3],[0,3],[-1,2],[-2,0],[1,4],[0,4],[-1,2],[-2,-1],[-2,-2],[-2,2],[-1,3],[-1,5],[-2,-1],[-2,3],[-1,5],[1,6],[-4,-2],[-4,5],[-5,13],[1,1],[1,0],[1,0],[1,-1],[-1,4],[-6,5],[-2,3],[1,1],[-2,4],[-1,3],[1,3],[2,4],[-2,5],[1,3],[2,1],[9,-9],[3,-6],[2,-3],[4,-2],[2,-3],[3,-6]],[[1442,2641],[-2,-3],[-1,1],[-1,4],[1,4],[0,1],[0,1],[-1,4],[1,4],[-1,4],[-2,7],[-1,4],[0,1],[0,6],[0,1],[5,2],[5,0],[2,2],[1,3],[1,3],[1,3],[2,-2],[0,-2],[-1,-5],[0,-2],[0,-3],[-2,-4],[0,-1],[-1,-6],[0,-3],[-1,-2],[-1,-4],[-1,-2],[0,-1],[-1,-4],[-1,-9],[-1,-2]],[[1048,2695],[2,0],[5,0],[2,0],[1,-1],[0,-1],[-2,-2],[-1,0],[-2,-1],[-5,-2],[-4,0],[-4,1],[-2,3],[-1,3],[2,2],[7,2],[3,-1],[0,-2],[-1,-1]],[[1321,2690],[0,-5],[-2,0],[-2,4],[0,7],[1,4],[3,-3],[2,-2],[-2,-5]],[[1420,2683],[-1,-4],[-2,2],[0,-18],[-1,-4],[-1,-5],[-1,-4],[-1,0],[-2,6],[1,2],[-1,2],[-1,3],[0,3],[1,2],[0,1],[0,8],[0,3],[1,6],[2,14],[0,3],[2,-2],[2,-3],[1,-3],[0,-7],[1,-5]],[[1076,2704],[-1,-1],[-1,-1],[0,-1],[0,-1],[-1,0],[-2,1],[-1,1],[1,0],[1,1],[1,1],[1,0],[1,0],[1,0]],[[994,2728],[-10,-2],[-2,0],[-7,9],[1,0],[1,0],[16,-3],[1,0],[2,-2],[-2,-2]],[[1296,2731],[1,-3],[1,-3],[-1,-1],[-2,1],[-2,4],[-2,0],[-2,-5],[-1,-1],[-1,9],[-2,4],[-1,3],[6,1],[3,-2],[2,-4],[1,-3]],[[1282,2749],[6,-2],[5,1],[1,-1],[3,-4],[1,-1],[2,0],[-1,-2],[0,-2],[-1,-1],[-1,-1],[-1,2],[-1,2],[0,1],[-1,1],[-2,1],[0,-1],[-1,-1],[-1,-1],[-1,1],[-2,3],[-1,1],[-1,1],[-3,1],[-2,2],[2,0]],[[1292,2751],[-4,-2],[-5,1],[-3,4],[-3,5],[-3,6],[2,2],[4,0],[3,0],[1,-2],[3,-3],[3,-6],[2,-5]],[[1311,2767],[6,-14],[2,-3],[2,-2],[1,-2],[2,-4],[1,-8],[0,-5],[-1,-2],[-3,0],[-1,0],[-4,5],[-3,2],[-1,3],[-2,7],[-2,4],[-2,2],[-1,3],[-1,5],[0,3],[-1,-1],[-2,-3],[-1,1],[0,1],[0,2],[1,1],[-1,3],[-2,1],[-2,2],[0,6],[2,4],[2,2],[3,0],[3,-1],[0,-5],[2,-3],[3,-4]],[[1010,2780],[-5,-1],[0,1],[0,1],[0,2],[7,4],[2,0],[1,-1],[0,-2],[-1,-1],[-4,-3]],[[1042,2794],[3,-13],[0,-4],[-2,-2],[0,-1],[7,-4],[2,-1],[1,-1],[1,-4],[1,-2],[1,-1],[2,1],[1,-2],[1,-2],[3,-11],[0,-2],[1,0],[1,-1],[-2,-2],[-1,0],[-1,0],[-2,1],[-1,2],[-2,2],[-6,0],[-2,1],[-2,2],[-3,1],[-5,1],[-2,0],[-4,-3],[-2,0],[-12,0],[2,-2],[4,-2],[1,-2],[1,-4],[0,-3],[1,-3],[1,-2],[1,6],[1,2],[1,2],[3,2],[1,2],[2,1],[10,-7],[8,-2],[3,-3],[2,0],[1,-1],[2,-3],[1,-1],[1,-1],[-1,-3],[-2,-2],[0,-1],[-1,-3],[1,0],[0,-2],[1,-1],[-1,-2],[-3,-3],[-3,-5],[-3,-2],[-12,-1],[-2,1],[-2,4],[-2,5],[-2,4],[-3,2],[-2,0],[-2,0],[-1,-1],[0,-1],[1,-1],[1,-1],[2,-1],[2,0],[0,-1],[-10,-7],[0,-2],[3,1],[3,2],[3,1],[3,-2],[1,-2],[1,-1],[0,-2],[0,-1],[-1,-1],[0,-1],[-1,-1],[-1,-1],[0,-1],[1,-2],[1,1],[1,-1],[2,-6],[1,-1],[14,0],[4,1],[1,0],[1,-3],[-4,-1],[-5,0],[-4,-2],[-3,-5],[-2,2],[-1,1],[-2,-1],[-2,0],[2,0],[1,-1],[3,-3],[-3,-7],[-1,-3],[6,1],[3,0],[3,-4],[2,-4],[3,-7],[0,-5],[-2,-2],[2,-3],[3,-7],[2,-1],[3,-1],[-1,-3],[-2,-2],[-2,-1],[0,-2],[1,-1],[1,0],[2,0],[1,-1],[1,-2],[0,-2],[2,0],[1,1],[2,2],[1,2],[2,-2],[0,-4],[2,-1],[6,0],[-2,-6],[2,-3],[4,-1],[2,-3],[-3,-12],[-1,-3],[3,0],[3,-1],[1,-2],[-2,-5],[2,0],[1,2],[1,1],[2,-2],[0,5],[0,4],[0,3],[1,3],[2,2],[4,1],[5,2],[1,-3],[-1,-3],[-4,-4],[-2,-3],[-1,-4],[0,-4],[2,2],[1,1],[4,0],[-1,-4],[-1,-1],[-1,0],[-1,0],[-3,-2],[-1,-1],[-1,-2],[-1,3],[-1,3],[-1,0],[-1,-3],[2,-2],[1,-9],[2,-4],[2,3],[6,-7],[3,-1],[-3,4],[-2,3],[1,3],[1,1],[1,-1],[1,-1],[3,4],[2,1],[2,0],[3,0],[1,-3],[0,-2],[4,-2],[1,-3],[-1,-5],[-3,-1],[-4,-1],[-3,-2],[0,-1],[7,0],[3,-1],[2,-3],[4,3],[2,1],[2,-2],[0,-4],[-1,-4],[-1,-3],[-1,-2],[-2,-1],[-5,0],[-2,1],[-1,3],[0,3],[-1,3],[-2,1],[1,-9],[-1,-8],[-3,-5],[-3,4],[-1,3],[0,8],[-1,3],[-1,3],[-2,3],[-3,4],[3,-17],[1,-8],[-3,-3],[-2,2],[-1,4],[-1,4],[-1,3],[0,3],[-3,6],[-1,5],[-1,0],[0,-8],[-3,0],[-3,5],[-3,5],[-1,4],[-1,3],[-1,3],[-3,2],[-2,3],[0,3],[-2,3],[-1,-2],[-1,0],[-1,1],[-1,1],[1,3],[1,4],[-2,-2],[-1,0],[-1,1],[0,1],[-1,6],[0,1],[1,4],[3,0],[3,-2],[2,-1],[-1,4],[-2,3],[-2,2],[-2,1],[-2,-1],[-1,-1],[-1,-1],[-2,2],[-3,6],[0,1],[-2,5],[-2,2],[-6,3],[-3,3],[-14,15],[-2,3],[0,2],[0,3],[0,3],[-1,1],[-2,0],[0,1],[0,5],[-1,0],[-8,7],[-9,13],[3,6],[2,2],[2,-1],[1,-1],[5,-7],[1,1],[-4,8],[-2,2],[1,2],[5,-7],[2,-3],[1,-2],[2,-1],[-1,2],[0,1],[-1,1],[-1,1],[0,1],[1,5],[-3,3],[-6,4],[-2,3],[-4,8],[-2,2],[2,-8],[1,-5],[-1,-2],[-2,-2],[-3,-7],[-2,-1],[-1,1],[-4,6],[0,1],[-1,2],[-1,1],[-2,1],[0,1],[-2,3],[-1,4],[-1,4],[3,3],[-8,4],[-2,3],[-2,5],[-1,2],[-1,5],[2,2],[1,1],[3,-2],[1,1],[1,1],[1,0],[2,0],[1,-1],[4,-5],[2,-1],[2,3],[-2,1],[-1,1],[-1,2],[0,3],[7,-9],[0,3],[-1,2],[0,1],[-1,1],[0,2],[1,1],[2,2],[2,2],[0,3],[-2,1],[-6,-2],[-2,2],[-3,5],[-5,1],[-8,-1],[0,3],[0,1],[-2,-1],[-4,-1],[-1,-2],[-2,4],[-2,-1],[-2,-1],[-1,-2],[-3,1],[-1,2],[-1,3],[1,4],[-2,2],[-2,2],[-2,3],[1,2],[2,1],[-1,3],[-3,6],[2,1],[4,-1],[3,-2],[3,-2],[10,1],[3,1],[0,1],[2,3],[1,1],[2,0],[6,-2],[20,2],[2,3],[1,3],[7,7],[1,0],[1,0],[1,0],[0,1],[1,3],[5,3],[7,0],[1,1],[1,1],[1,2],[2,-1],[0,-1]],[[1401,2771],[4,-10],[0,-2],[0,-1],[5,-20],[2,-9],[0,-12],[0,-10],[-1,-10],[-4,-25],[0,-2],[-2,1],[1,-10],[-1,-3],[-2,-5],[1,0],[1,0],[0,-1],[1,-1],[-1,-2],[0,-6],[-1,-2],[-1,-1],[-11,0],[-3,-1],[-2,-3],[-1,2],[1,4],[1,4],[0,5],[0,4],[0,4],[-1,9],[0,4],[0,5],[3,7],[1,4],[0,6],[1,3],[1,1],[2,3],[3,11],[0,1],[0,5],[0,4],[0,2],[-1,1],[-1,2],[-1,0],[-1,-2],[2,-9],[0,-5],[-2,-2],[-2,1],[-2,2],[-2,2],[-1,-1],[1,-4],[1,-5],[0,-6],[-2,-2],[-1,-2],[-1,-6],[-1,-10],[-1,-10],[0,-3],[2,-7],[1,-2],[-1,-2],[-1,-1],[-1,1],[-1,2],[-1,3],[-1,2],[-2,-7],[-4,-3],[-4,1],[-4,3],[-10,10],[-3,4],[-1,2],[0,1],[1,1],[0,1],[1,0],[0,2],[-3,2],[-2,2],[-2,7],[-1,3],[-2,6],[0,6],[1,2],[3,0],[7,-5],[1,1],[-1,2],[-5,6],[4,3],[4,5],[1,2],[3,7],[2,2],[0,6],[4,5],[8,4],[-1,4],[-3,5],[-3,4],[-3,3],[-3,4],[-7,7],[1,-3],[0,-2],[0,-2],[1,-2],[0,-1],[1,-1],[0,-1],[2,2],[1,0],[1,-1],[1,-2],[3,-3],[1,-2],[2,-5],[2,-1],[1,-1],[0,-2],[-4,0],[-2,-1],[-2,-2],[-1,-1],[-2,-4],[-1,-2],[0,-1],[-1,-4],[-1,-1],[-2,-5],[-5,-7],[-5,-4],[-2,5],[-1,2],[-4,0],[-2,1],[-1,3],[0,5],[-1,9],[1,4],[2,6],[0,3],[1,3],[2,2],[2,0],[1,-3],[3,5],[1,5],[0,6],[0,7],[0,6],[0,3],[2,3],[0,4],[0,5],[-1,3],[-1,2],[-1,3],[0,2],[1,3],[2,1],[2,0],[3,0],[5,-3],[8,-1],[4,-5],[8,-13],[11,-10],[2,-2],[5,-1],[2,-2]],[[1342,2759],[-2,-1],[-9,0],[-3,1],[0,-3],[-1,0],[-1,1],[-1,3],[-1,1],[-2,3],[-1,1],[-1,6],[0,6],[0,5],[1,5],[7,11],[2,4],[0,3],[-1,2],[-2,4],[0,2],[2,1],[2,0],[3,-2],[2,-1],[1,-2],[1,-2],[1,-3],[-1,-4],[0,-3],[1,-3],[1,-3],[1,-2],[0,-3],[-1,-4],[0,-3],[1,-6],[2,-10],[-1,-4]],[[1179,2849],[0,-3],[-2,1],[-3,3],[-1,4],[2,2],[3,-3],[1,-4]],[[1367,2818],[0,-3],[-1,-2],[-1,-1],[-2,0],[-4,3],[-2,1],[-6,-1],[-1,-1],[-4,-4],[-1,1],[-4,10],[4,11],[6,11],[0,8],[2,5],[3,6],[2,3],[1,-4],[1,-7],[1,-5],[7,-17],[1,-4],[0,-3],[-1,-2],[-2,-3],[1,-2]],[[904,2867],[1,-1],[1,-1],[3,-1],[2,-4],[-3,-1],[-1,1],[0,2],[0,1],[-1,0],[0,1],[-1,0],[-1,0],[0,1],[0,1],[0,1]],[[1218,2871],[6,-6],[8,-6],[5,-7],[11,-8],[5,-6],[3,-6],[1,-1],[1,-1],[1,-5],[1,-3],[11,-25],[3,-7],[1,-4],[0,-6],[-2,-5],[-2,-3],[-2,1],[-1,2],[-1,2],[-1,2],[-2,-1],[-1,-2],[-1,-3],[-1,-1],[-3,0],[-3,0],[-2,1],[-1,1],[-6,11],[-2,4],[0,4],[-5,3],[-3,2],[-1,3],[0,4],[0,4],[-1,2],[-2,2],[-3,1],[-2,1],[0,4],[-1,2],[-1,1],[-2,0],[-1,0],[0,2],[-1,1],[-1,2],[-11,1],[-1,2],[-4,11],[-3,6],[-2,4],[-1,3],[1,3],[3,1],[5,0],[-3,3],[-3,0],[-2,-2],[-3,-1],[-2,0],[-1,1],[-1,1],[-4,6],[-1,1],[-1,-1],[-1,1],[0,2],[0,5],[1,6],[1,6],[2,2],[6,-1],[3,-2],[3,-2],[3,-2],[5,-5],[6,-2],[3,-3]],[[1372,2876],[-1,-3],[-8,3],[-3,-1],[-10,-16],[-1,-1],[-1,-2],[0,-4],[1,-6],[-1,-4],[-1,-2],[-2,-1],[-1,-2],[-3,-5],[-2,-5],[0,5],[-1,10],[1,11],[1,9],[-1,14],[0,6],[1,3],[3,5],[1,2],[5,3],[1,2],[1,2],[3,1],[5,3],[10,0],[2,1],[3,1],[2,1],[2,-2],[-4,-6],[-1,-3],[0,-4],[-1,-8],[0,-7]],[[1233,2879],[2,-8],[-3,0],[-7,5],[-8,2],[-5,5],[-5,1],[-1,3],[-2,5],[-1,2],[-1,2],[-2,2],[-1,1],[4,7],[8,7],[1,3],[-1,4],[0,4],[0,3],[3,0],[1,-1],[4,-3],[1,-2],[-1,-3],[-1,-3],[0,-3],[1,-3],[1,-2],[2,-1],[2,0],[5,-3],[2,-2],[1,-3],[-1,-2],[0,-2],[0,-3],[1,-3],[-1,-2],[1,-3],[1,-4]],[[1194,2926],[3,-4],[-2,-6],[-6,2],[-2,7],[-5,-2],[-6,3],[-1,6],[6,3],[6,-2],[7,-7]],[[1272,2895],[10,-18],[16,-21],[1,-2],[1,-3],[-2,-1],[-2,0],[-2,0],[-6,4],[-1,-1],[-1,-3],[2,0],[1,1],[3,-3],[3,-1],[7,1],[2,-1],[3,-4],[3,-5],[1,-3],[2,-5],[3,-4],[4,-5],[-1,-8],[2,-8],[0,-8],[-4,-4],[-2,8],[-2,3],[1,4],[-2,6],[-2,1],[-2,-5],[-2,-5],[-2,0],[-3,5],[-1,-2],[1,-3],[0,-4],[0,-3],[3,-5],[-1,-5],[-1,-2],[-3,3],[-3,-3],[-2,-1],[-2,3],[-1,5],[-1,2],[-5,10],[-2,3],[1,0],[1,2],[1,0],[-4,3],[0,2],[1,2],[-1,1],[-1,-1],[-2,-2],[-2,0],[-1,1],[-4,6],[0,1],[0,1],[0,2],[-1,1],[-3,-2],[-1,1],[-2,5],[-1,2],[1,7],[1,4],[-1,3],[-1,1],[-2,-2],[-3,0],[-2,3],[0,2],[2,8],[0,4],[0,4],[-1,2],[-2,0],[-3,0],[1,2],[1,2],[3,2],[-3,2],[-2,0],[-1,-1],[0,2],[1,0],[2,3],[-3,2],[-2,-2],[-1,-4],[-2,-2],[-5,-1],[-3,1],[0,5],[2,6],[6,3],[7,0],[5,-3],[0,2],[-3,4],[-5,2],[-5,-1],[-4,-2],[-4,-3],[-2,-1],[-1,2],[-1,4],[1,8],[-1,3],[-8,5],[-4,1],[-1,1],[0,5],[0,4],[1,3],[0,4],[-1,4],[-2,2],[-7,3],[2,8],[1,3],[1,2],[3,1],[4,5],[2,-1],[2,-4],[6,-4],[3,-5],[1,-1],[4,-1],[1,-1],[0,-2],[1,-2],[2,-2],[8,-10],[3,-2],[1,-2],[2,-3],[1,-1],[3,-2],[1,-1],[6,-11],[1,-3],[1,-2]],[[886,2961],[1,-1],[0,-1],[0,-1],[0,-1],[0,-1],[-1,0],[-1,-2],[0,1],[-2,1],[0,1],[0,1],[0,1],[0,1],[1,0],[1,0],[0,1],[1,0]],[[1215,2978],[2,-4],[2,-6],[0,-4],[-1,-4],[-5,-6],[-2,-3],[-3,-6],[-1,-3],[0,-2],[-1,-2],[-1,-2],[-1,-1],[-4,0],[-1,1],[-1,1],[-1,6],[0,1],[-2,-2],[-2,-4],[-2,-1],[-1,5],[-1,3],[1,1],[2,1],[1,2],[1,1],[5,13],[1,3],[4,1],[4,0],[0,2],[-2,0],[-1,2],[1,1],[1,2],[-2,1],[-1,-1],[-3,-2],[-5,0],[-2,-2],[-1,-4],[0,-7],[-1,-6],[-2,-3],[-4,1],[-3,2],[-4,8],[-2,4],[-2,2],[-1,-1],[-1,-2],[-1,-1],[-1,0],[-1,4],[-1,-2],[-1,-3],[-1,-3],[-1,-3],[1,-4],[2,-2],[2,-1],[2,-2],[1,-3],[1,-4],[-1,-3],[-3,-1],[-3,3],[-2,0],[-1,0],[-1,1],[0,4],[1,4],[0,4],[-1,2],[-1,2],[1,4],[1,4],[2,1],[0,1],[1,5],[1,1],[1,1],[1,0],[1,0],[0,-1],[1,3],[2,3],[2,3],[2,1],[2,1],[6,6],[-1,2],[8,2],[2,1],[1,5],[4,-1],[6,-6],[5,-10],[1,-2],[2,-2]],[[906,3014],[2,0],[1,1],[2,0],[2,-1],[1,-2],[1,-1],[1,-1],[8,-4],[7,-1],[3,1],[5,3],[3,1],[3,-1],[6,-5],[3,-1],[1,-2],[0,-6],[0,-6],[0,-4],[-2,-5],[-2,-2],[-3,-1],[-2,-2],[-4,-8],[0,-4],[2,-2],[1,0],[2,5],[2,2],[1,0],[2,1],[2,1],[2,4],[1,6],[0,4],[2,2],[8,4],[2,3],[2,3],[2,3],[12,3],[2,-1],[1,-3],[4,-6],[1,-4],[0,-3],[-1,-2],[1,-1],[1,-2],[2,-1],[3,-1],[2,-1],[1,-4],[0,-4],[0,-5],[-1,-5],[1,-3],[2,-5],[2,-5],[0,-2],[-4,-8],[-1,-1],[-3,0],[-1,-1],[-2,-4],[-2,-5],[-1,-4],[-4,-2],[-12,-2],[-2,-2],[-4,-6],[-2,-1],[-3,-1],[-6,-4],[-3,0],[-6,0],[-3,0],[-3,-3],[-1,-3],[1,-1],[3,2],[2,-1],[1,0],[1,-2],[1,-2],[0,-2],[-2,-5],[2,1],[3,6],[1,1],[2,0],[3,-4],[1,-1],[5,2],[8,4],[4,1],[-3,-6],[-5,-3],[-4,-5],[-3,-9],[2,0],[1,2],[1,2],[2,2],[3,2],[3,2],[7,0],[3,1],[2,4],[-2,0],[-2,0],[-2,2],[-1,3],[2,3],[2,0],[5,-2],[4,-2],[2,0],[1,3],[0,3],[4,4],[0,4],[-1,3],[-3,3],[0,4],[0,4],[1,4],[1,2],[1,2],[1,2],[5,6],[3,9],[1,4],[-1,2],[-3,5],[0,2],[0,3],[-2,8],[0,6],[0,2],[-1,2],[-1,0],[-1,2],[-3,4],[2,1],[4,-3],[9,-3],[6,1],[9,3],[6,7],[8,3],[5,6],[3,2],[2,1],[1,3],[2,3],[2,6],[0,-2],[0,-1],[-1,0],[1,-2],[0,-2],[0,-2],[-1,-1],[0,-11],[-1,-11],[-2,-10],[-2,-9],[-4,-9],[-8,-15],[-4,-9],[-1,-4],[-1,-5],[0,-4],[0,-6],[0,-5],[-1,-5],[-1,-5],[-3,-9],[-2,-5],[0,-6],[1,-5],[1,-2],[-1,-3],[-1,-3],[0,-2],[-1,-3],[1,-2],[1,-6],[1,-8],[1,-4],[-1,-12],[0,-3],[1,-4],[-2,-4],[-4,-14],[-2,-6],[-3,-1],[-10,0],[-8,-2],[-3,-3],[-2,0],[-2,-1],[0,-3],[-1,-2],[-1,0],[-4,1],[2,-3],[2,-1],[4,1],[9,0],[-1,-3],[-2,-3],[-2,-3],[-2,-1],[-4,-1],[-3,1],[-3,3],[-2,4],[-2,2],[0,-2],[0,-3],[0,-1],[-1,-2],[-1,0],[-5,-2],[-3,-2],[-1,0],[-3,0],[-2,0],[-1,1],[1,3],[0,2],[0,1],[0,2],[-1,1],[-1,0],[0,-1],[-1,-4],[-2,0],[-4,3],[0,2],[2,1],[-1,1],[-4,1],[-2,2],[2,3],[1,1],[0,2],[-2,0],[-6,5],[-5,1],[-5,1],[2,4],[3,2],[6,4],[0,1],[0,1],[0,1],[4,2],[2,-1],[1,-2],[1,0],[1,3],[-4,3],[-21,-5],[1,3],[4,10],[1,2],[2,0],[3,-3],[1,0],[14,0],[1,-1],[2,-4],[2,-1],[2,-4],[3,-1],[1,-1],[1,-2],[1,-3],[1,8],[-2,5],[-9,7],[0,2],[-1,5],[-1,1],[-1,1],[-3,0],[-10,4],[-2,2],[1,5],[-1,5],[-1,2],[-2,-8],[-2,-2],[-3,1],[-3,2],[0,1],[2,2],[1,4],[1,3],[-2,3],[-1,0],[-1,-2],[0,-2],[0,-1],[-1,0],[-9,-6],[-2,-1],[-1,2],[0,3],[4,6],[1,3],[-11,0],[1,2],[1,4],[2,2],[-2,2],[-2,0],[-3,-2],[-3,0],[1,5],[-2,3],[2,2],[2,0],[6,-2],[2,1],[-1,6],[-11,11],[-3,5],[2,-1],[1,0],[1,1],[1,2],[2,-2],[1,0],[1,1],[0,3],[-2,1],[-2,1],[-2,2],[1,4],[0,2],[0,5],[-1,1],[-1,0],[-1,-1],[-1,-1],[0,-1],[-2,0],[-1,1],[-2,1],[-5,4],[-1,1],[0,3],[2,2],[3,3],[-1,4],[-5,6],[-2,4],[-1,5],[1,4],[2,4],[0,3],[2,3],[0,4],[0,4],[0,3],[1,1],[2,-1],[1,1],[0,1],[0,1],[0,1],[2,2],[1,2],[0,2],[-1,2],[0,2],[0,2],[-2,7],[-1,1],[-1,1],[5,10],[0,4],[-4,1],[3,4],[3,-2],[3,-3]],[[1221,3004],[-2,-1],[-2,1],[-1,3],[-1,3],[-1,2],[2,7],[0,2],[2,1],[1,0],[2,0],[0,-2],[3,-2],[3,-4],[0,-5],[-6,-5]],[[1171,3003],[-2,-1],[-3,1],[-2,2],[-1,3],[-2,0],[-5,6],[-2,3],[1,3],[1,4],[1,2],[2,1],[9,-11],[4,-6],[-1,-7]],[[907,3021],[-1,0],[-2,1],[0,2],[-1,3],[-1,0],[-1,1],[-1,0],[-3,1],[-1,2],[1,3],[2,3],[2,0],[4,-1],[1,0],[2,-2],[1,-1],[0,-2],[-1,-4],[0,-3],[0,-2],[-1,-1]],[[1203,3035],[1,-2],[-4,3],[-5,9],[-3,3],[1,3],[1,3],[2,1],[1,0],[4,-5],[1,-2],[2,-6],[1,-2],[0,-1],[-1,-2],[0,-1],[-1,0],[0,-1]],[[1215,3042],[-4,-6],[-4,0],[-1,2],[-1,1],[0,1],[0,6],[2,6],[3,3],[4,0],[2,-5],[-1,-8]],[[1166,3073],[0,-1],[0,-1],[1,0],[0,-2],[-1,-1],[1,-5],[-3,-2],[-3,0],[-2,1],[-1,5],[1,3],[3,2],[2,3],[1,-2],[1,2],[0,-2]],[[1158,3088],[5,-8],[-1,-4],[-6,-2],[-4,2],[-1,5],[-4,-1],[1,5],[5,7],[3,2],[2,-6]],[[1124,3117],[-2,-2],[-2,3],[1,8],[4,0],[1,-5],[-2,-4]],[[1160,3124],[0,-2],[1,-1],[-1,-7],[-1,-3],[-2,-3],[-6,-7],[-1,-2],[-14,-15],[-1,2],[0,2],[1,2],[0,2],[0,13],[1,2],[1,3],[1,3],[1,3],[0,1],[0,2],[-1,2],[0,2],[1,2],[1,2],[2,1],[2,-1],[1,-3],[2,3],[2,-1],[2,-2],[2,-2],[1,2],[3,3],[1,1],[1,-4]],[[1194,3150],[-1,-3],[-2,0],[-4,2],[0,-1],[-1,-2],[-1,-1],[-2,1],[-1,2],[-3,4],[-1,2],[-1,1],[-1,2],[-1,2],[2,1],[1,1],[2,0],[7,8],[3,1],[3,-1],[1,-3],[2,-6],[-1,-2],[-1,-6],[0,-2]],[[1220,3166],[0,-1],[2,0],[0,-1],[0,-4],[-1,-1],[-1,-5],[-2,-3],[-5,-4],[-2,0],[-3,-1],[-1,0],[-3,-2],[-2,-1],[0,2],[4,8],[3,3],[5,3],[5,10],[2,1],[0,-1],[0,-2],[-1,-1]],[[1200,3160],[-1,0],[-1,1],[-2,4],[-1,2],[-1,2],[-1,4],[0,2],[4,5],[19,29],[6,12],[2,3],[2,-4],[2,-6],[0,-5],[-1,-6],[-2,-5],[-1,-3],[-3,-3],[-1,-3],[-4,-11],[-2,-3],[-1,-1],[-5,-4],[-5,-7],[-3,-3]],[[3048,1767],[-2,0],[-25,0],[-25,0],[-25,0],[-25,0],[-24,0],[-25,0],[-25,0],[-25,0],[-24,0],[-1,0],[-25,0],[-25,0],[-25,0],[-24,0],[-12,0],[-13,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-24,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-24,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-25,0],[-24,0],[-25,0],[-25,0],[-25,0],[-25,0],[-11,0],[-1,2],[-2,4],[-3,-2],[-2,-1],[-3,3],[-3,4],[0,5],[1,3],[0,3],[-2,1],[-3,1],[-2,0],[-3,-1],[-9,-8],[-2,-4],[1,-4],[1,-6],[-6,0],[-1,3],[0,4],[-1,3],[-2,3],[-2,2],[0,1],[1,4],[6,7],[3,4],[1,7],[-4,-5],[-5,-3],[-5,-1],[-5,4],[0,2],[1,2],[0,10],[0,4],[3,1],[2,-1],[2,-2],[0,3],[-4,4],[-4,3],[-3,3],[-1,6],[3,1],[5,-1],[4,1],[-2,5],[0,2],[2,0],[1,-1],[1,-2],[2,-2],[2,0],[1,2],[1,-2],[1,2],[2,0],[3,-1],[1,-1],[5,-1],[5,4],[3,8],[3,18],[1,4],[0,4],[0,2],[-1,2],[-1,0],[-1,-2],[0,-8],[0,-3],[-6,-19],[-9,-4],[-22,8],[-2,0],[-5,-2],[0,4],[-1,3],[-1,2],[1,3],[3,3],[1,4],[1,3],[0,4],[-2,17],[0,3],[2,7],[1,8],[1,2],[0,1],[-1,0],[0,1],[2,2],[0,3],[1,3],[-1,3],[5,6],[1,5],[-1,3],[-2,-4],[-2,-2],[-1,-1],[-2,-1],[-1,-1],[-1,-3],[-1,-2],[1,-6],[0,-3],[0,-3],[-1,-1],[-1,-1],[-3,-2],[-3,0],[-1,0],[-1,-1],[-2,-4],[0,-2],[-3,1],[-2,0],[-1,-1],[0,-1],[-4,-3],[-1,-1],[-4,-4],[-2,-3],[2,-2],[-1,-5],[2,-10],[-1,-5],[-2,-3],[-2,-2],[-1,-1],[-3,1],[-25,18],[-3,1],[-7,1],[-2,1],[-2,2],[-1,2],[-1,1],[0,2],[-1,1],[-2,1],[-1,1],[-1,0],[-1,3],[-2,4],[-4,13],[-1,2],[-2,3],[-1,2],[-1,3],[0,4],[1,4],[1,2],[1,1],[1,7],[1,3],[2,2],[2,1],[4,1],[2,-1],[0,-2],[1,-2],[1,-3],[6,-10],[3,-9],[0,-1],[1,-9],[0,-3],[1,-4],[2,-9],[1,-4],[2,-2],[1,2],[1,4],[1,3],[-1,4],[-1,4],[0,5],[1,4],[1,0],[4,1],[1,2],[0,1],[1,2],[7,1],[5,3],[5,5],[2,8],[-15,-12],[-15,-3],[-2,1],[0,4],[-1,4],[-7,14],[-4,4],[-1,2],[-1,5],[2,3],[3,7],[1,5],[0,4],[0,9],[-1,5],[-2,2],[-2,1],[-2,3],[-1,5],[0,5],[2,5],[5,2],[2,3],[1,1],[5,1],[1,1],[5,7],[2,5],[-1,3],[-2,2],[-4,9],[-2,2],[-7,6],[-4,2],[-5,6],[-3,1],[2,-4],[1,-3],[1,-3],[2,-2],[2,-2],[5,-2],[2,-2],[1,-2],[1,-3],[0,-1],[1,-1],[3,-2],[0,-1],[0,-3],[-2,-4],[-4,-5],[-13,-8],[-1,-3],[-1,-4],[0,-5],[0,-5],[1,-4],[1,-4],[2,-3],[4,-5],[2,-4],[-1,-2],[-1,-5],[-1,-3],[-3,-2],[0,-1],[-1,0],[0,-1],[-1,0],[-1,0],[-1,0],[-1,3],[-1,5],[0,1],[0,1],[1,1],[-1,2],[0,1],[-2,2],[0,2],[1,2],[0,4],[0,2],[-1,2],[-1,-2],[-4,-12],[2,-1],[1,-3],[0,-3],[-1,-1],[-1,0],[-2,-1],[0,-2],[1,-3],[-3,-3],[-5,-3],[-3,-1],[-8,0],[-3,-2],[0,-5],[-1,-1],[-3,1],[-2,5],[-3,0],[-6,-1],[-3,1],[-5,4],[-4,2],[-1,2],[-1,2],[0,6],[0,1],[-6,9],[-3,3],[-2,1],[-1,1],[-2,5],[-1,1],[-2,0],[-6,7],[-4,5],[-6,15],[-1,4],[1,4],[2,-2],[3,-5],[1,-2],[3,-1],[4,-8],[2,-2],[-2,8],[0,5],[2,2],[1,3],[1,2],[0,3],[-1,1],[-3,4],[1,2],[2,2],[1,1],[0,5],[2,2],[3,4],[1,4],[-1,4],[2,7],[1,2],[-1,2],[-2,4],[-1,3],[-1,4],[-2,2],[-3,2],[-2,3],[-1,2],[1,1],[3,4],[1,2],[0,3],[1,4],[3,3],[4,1],[3,-1],[3,-1],[1,0],[2,1],[1,3],[2,1],[2,1],[5,3],[2,2],[6,10],[1,3],[-1,3],[-3,-3],[-4,-8],[-9,-6],[-10,-4],[-9,2],[-2,-2],[-1,-5],[-4,-8],[-1,-2],[-1,-2],[-7,-7],[-2,-2],[-3,1],[-2,1],[-3,1],[-5,-1],[-1,1],[-1,3],[0,2],[1,2],[1,3],[-1,5],[-2,5],[-2,4],[-2,1],[1,-5],[1,-7],[-1,-5],[-5,-5],[-2,-3],[-3,-1],[0,6],[0,4],[1,0],[1,1],[1,2],[-1,8],[0,1],[0,2],[0,3],[0,2],[1,6],[0,2],[1,1],[1,1],[8,4],[1,2],[1,6],[2,3],[5,2],[1,1],[1,4],[0,1],[0,2],[0,4],[0,2],[1,4],[0,10],[1,5],[3,6],[1,4],[-2,5],[-7,4],[-1,2],[0,1],[0,2],[0,1],[3,5],[2,3],[1,2],[3,8],[2,2],[3,1],[-2,8],[-1,2],[-2,2],[-2,-1],[-2,0],[0,-3],[-1,-5],[0,-3],[-5,-10],[-2,-2],[-4,-3],[-2,-3],[2,-1],[2,-2],[0,-1],[0,-7],[0,-1],[3,0],[2,-2],[1,-4],[-2,-7],[-1,-9],[0,-2],[1,-4],[1,-2],[0,-2],[-3,-6],[-1,-1],[-1,0],[0,1],[0,1],[-1,-1],[0,-1],[-1,-2],[-6,-7],[-2,-7],[-7,-8],[-2,-6],[1,-5],[-2,-4],[-3,-3],[-3,-1],[-2,0],[-1,2],[-2,6],[-2,3],[0,2],[0,2],[0,2],[1,1],[-1,2],[-1,1],[-2,-2],[-2,-5],[-1,-3],[-1,0],[-1,1],[-1,1],[-2,4],[-1,7],[-1,6],[-3,-2],[0,-2],[0,-4],[-1,-3],[0,-3],[-2,-3],[-2,-2],[-10,-2],[-3,0],[-2,1],[-2,2],[1,2],[1,2],[2,2],[0,3],[2,9],[0,3],[0,2],[-1,4],[-1,2],[1,7],[1,4],[1,4],[2,3],[1,2],[3,2],[1,1],[1,2],[2,3],[1,2],[-4,2],[-3,-4],[-2,-4],[-3,-3],[-3,-2],[-2,-7],[0,-8],[1,-4],[-1,-4],[-1,-5],[-1,-4],[-4,-3],[0,-3],[1,-3],[1,-1],[-1,-4],[0,-2],[-2,-3],[-1,-2],[-2,-1],[-1,-1],[-4,-1],[-1,1],[-1,1],[-6,11],[-2,1],[1,3],[4,1],[1,2],[-5,3],[-3,0],[-3,-1],[-1,-1],[-1,-4],[-1,-1],[-10,-6],[-6,-1],[-6,1],[-4,6],[-2,-3],[-2,0],[-13,3],[-2,1],[-2,3],[0,1],[2,2],[2,2],[2,1],[2,-1],[3,-5],[0,7],[0,4],[0,3],[1,0],[4,1],[3,3],[1,1],[1,-1],[0,-1],[1,0],[2,3],[10,3],[2,1],[2,3],[-2,4],[-3,-1],[-5,-3],[-4,-1],[-7,-6],[-4,-1],[-9,0],[-1,1],[-1,4],[-1,1],[-1,1],[1,1],[3,2],[2,1],[12,1],[8,4],[3,0],[9,-3],[3,1],[10,5],[9,2],[1,0],[1,-1],[2,-1],[1,1],[1,2],[1,9],[1,2],[1,1],[1,2],[3,1],[1,2],[2,5],[-2,5],[-2,4],[-1,5],[1,4],[2,2],[7,1],[1,1],[1,4],[1,6],[1,1],[1,1],[1,1],[0,3],[0,3],[0,3],[-1,2],[-1,1],[-1,5],[-3,14],[-2,4],[-6,8],[0,-6],[0,-4],[3,-11],[3,-12],[1,-2],[2,-3],[1,-2],[-1,-4],[-2,-3],[-3,-2],[-3,0],[-2,-1],[-1,-2],[-2,-2],[-1,-2],[-1,-3],[0,-4],[0,-3],[0,-2],[4,-4],[1,-2],[-1,-3],[-1,-2],[-1,0],[-2,0],[-3,-4],[-3,-10],[-3,-3],[0,2],[-13,-1],[-2,-3],[-1,-2],[-2,-1],[-2,0],[-1,0],[-2,3],[-1,3],[-1,1],[-2,0],[-2,-2],[-3,-4],[-1,-1],[-15,-3],[-3,1],[-1,4],[2,4],[5,2],[2,3],[2,4],[8,7],[3,3],[0,2],[-6,0],[-2,-2],[-3,-2],[-3,-1],[-3,1],[-2,4],[1,4],[1,5],[0,5],[0,3],[-2,-1],[-2,-4],[-2,-1],[0,1],[-1,2],[0,1],[0,1],[-1,0],[-2,0],[-5,-2],[-6,0],[-2,-1],[-9,-7],[-3,-1],[-2,2],[-2,1],[-2,2],[-1,1],[-1,3],[0,3],[0,2],[1,3],[6,8],[1,0],[1,-1],[2,-1],[12,-2],[4,2],[11,-2],[2,1],[3,3],[1,4],[-1,4],[-1,-1],[-4,-4],[-2,-1],[-18,2],[-6,3],[-3,4],[-3,11],[-1,12],[0,3],[-3,-10],[-1,-7],[1,-5],[2,-7],[-1,-3],[-2,-3],[-1,-3],[-2,-3],[-11,-6],[-2,-1],[-3,2],[-3,2],[0,1],[-1,2],[-2,1],[-3,1],[-2,1],[-2,4],[-1,2],[-2,0],[-1,-1],[-1,-1],[-1,-3],[0,-2],[-1,-1],[-15,-1],[-3,3],[-1,1],[-2,1],[-3,2],[-3,-1],[-2,1],[-1,1],[-1,-3],[2,-2],[3,-2],[5,-2],[6,-3],[3,-1],[6,0],[1,-1],[2,-1],[0,-1],[-1,0],[-3,-2],[-5,-8],[-2,-2],[-2,0],[-3,2],[-7,9],[-10,2],[-1,1],[-3,5],[0,4],[-1,0],[-2,-1],[-1,0],[-3,0],[-2,1],[-11,6],[-2,3],[2,2],[1,2],[0,1],[-1,2],[-1,1],[-1,0],[-1,0],[1,0],[-3,-1],[-4,2],[-3,3],[0,3],[2,6],[1,7],[1,6],[2,2],[0,2],[-2,2],[-3,-1],[-3,-2],[-9,-3],[-3,1],[-3,3],[-1,4],[-10,9],[0,5],[-1,3],[0,2],[0,3],[0,3],[2,2],[2,2],[2,2],[13,4],[2,3],[4,4],[17,-3],[1,-1],[3,-2],[1,-1],[17,4],[5,5],[4,2],[1,1],[1,4],[1,5],[0,5],[0,4],[-2,-3],[-3,-10],[-2,-2],[-2,-1],[-7,-5],[-3,-2],[-26,3],[0,2],[3,2],[2,3],[1,2],[-2,1],[-4,-1],[-5,-4],[-3,-1],[-7,1],[-4,0],[-2,-2],[-2,-1],[-4,0],[-3,2],[-1,5],[1,3],[3,3],[3,3],[2,0],[3,1],[2,1],[5,11],[2,3],[2,2],[3,1],[1,1],[0,2],[0,2],[0,2],[1,2],[1,1],[1,2],[1,2],[0,6],[0,4],[-1,5],[1,5],[3,3],[2,2],[18,6],[30,3],[38,-5],[4,0],[2,2],[1,1],[0,1],[0,3],[0,1],[2,-1],[1,1],[0,2],[1,2],[0,3],[1,1],[-4,12],[0,3],[-1,2],[-1,2],[-1,3],[0,-2],[1,-1],[0,-1],[-1,-5],[1,-4],[1,-4],[1,-3],[-1,-3],[-1,-2],[-1,0],[-1,-2],[-3,-6],[-44,9],[-19,-4],[-3,1],[-5,5],[-2,0],[-1,-6],[-3,-3],[-4,-1],[-3,4],[1,0],[0,1],[0,1],[1,1],[0,1],[-1,0],[0,10],[0,2],[2,1],[4,2],[1,0],[1,3],[3,19],[-1,3],[-2,-3],[-1,-5],[-1,-5],[-1,-4],[0,-4],[-1,-1],[-5,-1],[-1,0],[-1,-3],[-2,-12],[-1,-3],[-4,1],[-3,2],[-4,0],[-3,-1],[-4,-2],[0,-2],[3,-1],[5,-3],[3,-1],[6,1],[2,0],[2,-3],[-2,-3],[-7,-4],[-2,-4],[-2,-7],[-1,-3],[0,-2],[2,-2],[0,-4],[0,-4],[-6,-12],[-2,-3],[-3,-1],[-5,-1],[-2,1],[-3,7],[-1,1],[2,3],[7,0],[3,1],[0,1],[-1,1],[-2,3],[-2,1],[-2,0],[-8,-2],[-1,0],[0,3],[0,1],[1,1],[0,2],[0,3],[-1,2],[0,3],[0,4],[-3,-2],[-1,3],[0,4],[0,5],[-2,3],[-1,1],[-1,3],[-1,5],[0,4],[0,2],[1,2],[1,2],[-1,2],[-1,1],[0,2],[0,6],[0,2],[-1,3],[-1,3],[1,1],[1,6],[-1,3],[0,1],[1,1],[1,1],[0,4],[1,5],[1,3],[4,2],[4,5],[2,1],[6,-1],[3,0],[2,3],[-1,1],[2,6],[1,16],[1,6],[3,6],[3,5],[4,4],[4,2],[3,0],[1,0],[1,-3],[1,-4],[-1,-4],[-2,-2],[-2,-1],[-7,1],[-1,-1],[-5,-16],[4,5],[3,7],[1,1],[10,0],[4,2],[3,3],[-4,12],[-1,2],[0,1],[-1,2],[0,2],[2,1],[3,1],[1,2],[1,1],[1,4],[3,3],[8,2],[3,2],[2,3],[6,12],[2,2],[3,0],[6,-2],[6,0],[2,-1],[3,-2],[4,-7],[2,-4],[0,-4],[-1,-5],[5,-9],[14,-19],[3,-5],[2,-2],[3,-5],[1,-3],[2,-9],[1,-3],[0,6],[1,5],[-1,4],[-1,3],[-1,2],[-2,1],[-1,1],[0,4],[-2,6],[-1,2],[-5,6],[-11,10],[-3,8],[1,4],[0,4],[0,2],[-3,6],[-1,3],[0,4],[2,4],[1,0],[3,1],[1,1],[0,2],[1,1],[2,-1],[4,3],[5,1],[5,0],[-3,5],[-3,1],[-15,-6],[-3,-2],[-3,-3],[-4,-3],[-4,0],[-9,3],[-1,0],[-2,0],[-1,0],[-1,1],[0,1],[0,2],[0,1],[-3,4],[0,2],[1,4],[-2,6],[-1,2],[-1,1],[-1,1],[-1,13],[1,3],[4,8],[2,3],[2,3],[5,2],[4,7],[2,2],[6,2],[2,3],[1,6],[1,4],[4,6],[1,5],[-1,5],[-3,8],[-1,10],[-3,4],[-5,5],[-1,2],[-1,1],[-1,1],[-1,1],[1,-6],[5,-11],[2,-8],[0,-2],[0,-5],[1,-1],[1,-4],[1,-2],[-1,-3],[-8,-14],[-11,-10],[-7,-9],[-1,-2],[-2,-1],[-2,-2],[1,-4],[-2,-2],[-1,-2],[-1,-1],[1,-3],[1,-2],[0,-2],[1,-1],[-1,-4],[-2,-2],[-19,-11],[-2,-6],[-3,-7],[-5,-4],[-5,-3],[-3,-3],[0,-5],[-2,0],[-2,1],[-3,0],[-2,-1],[-2,-3],[-2,-2],[-3,1],[0,3],[1,5],[0,1],[1,3],[3,7],[-3,-1],[-2,-4],[-3,-9],[-1,-3],[-1,-3],[0,-3],[0,-2],[-1,-1],[-2,-1],[-1,-2],[-1,-3],[-2,-3],[-1,1],[-2,5],[0,3],[0,3],[0,3],[-3,2],[-1,2],[-1,3],[0,3],[1,1],[0,2],[1,2],[1,2],[1,2],[3,0],[1,1],[2,3],[-2,3],[-4,7],[-2,7],[-1,2],[1,3],[1,2],[1,2],[1,2],[1,2],[1,2],[0,2],[-2,1],[-1,-1],[-1,-4],[-2,-2],[1,-2],[0,-2],[-1,-3],[0,-3],[0,-3],[0,-2],[2,-1],[1,-4],[-1,-5],[-2,-5],[-5,-10],[-4,4],[-2,2],[-1,7],[-1,2],[-2,2],[-1,3],[1,2],[-1,9],[1,8],[4,7],[4,2],[1,-1],[2,-1],[1,-2],[0,-1],[2,1],[0,2],[-1,3],[0,1],[1,1],[3,1],[1,1],[1,2],[0,3],[1,5],[-2,-1],[-4,-4],[-11,-4],[-2,-2],[-2,-3],[-1,-2],[-1,0],[-1,-1],[-1,-1],[0,-2],[-1,-1],[-5,-15],[0,-2],[-1,-3],[0,-2],[-1,-1],[-2,-2],[-1,-1],[-4,-15],[-6,-10],[-1,-2],[-2,0],[-1,1],[-1,1],[-1,1],[-3,0],[-2,1],[-1,1],[1,3],[2,2],[4,3],[-1,4],[0,5],[1,5],[2,3],[1,0],[1,0],[1,0],[1,0],[2,0],[0,2],[0,3],[1,2],[1,3],[2,4],[0,5],[-1,6],[0,5],[0,1],[-1,7],[0,3],[1,4],[4,13],[1,1],[0,3],[0,2],[1,2],[3,5],[0,2],[0,3],[0,1],[1,1],[1,5],[1,1],[0,1],[0,3],[0,5],[-1,9],[0,5],[0,7],[3,4],[3,3],[5,1],[1,1],[0,1],[-1,2],[-8,-3],[-1,1],[-2,3],[-1,-5],[-1,-5],[-2,-5],[-1,-3],[-4,-5],[-1,-2],[-1,-2],[-1,-2],[-1,-1],[-3,-1],[-3,1],[-8,4],[-7,1],[-1,1],[-1,6],[-1,1],[-1,1],[-1,2],[-2,6],[0,1],[1,2],[-1,6],[-1,8],[0,5],[-2,10],[0,5],[0,1],[-1,4],[0,3],[0,2],[1,1],[2,0],[1,1],[-2,2],[-2,0],[-2,1],[-1,5],[1,1],[6,2],[2,1],[-2,1],[-7,-1],[-2,1],[-1,2],[-7,15],[-1,2],[0,2],[-1,1],[-1,-2],[0,-2],[1,-2],[1,-4],[-3,-2],[-2,1],[-11,14],[-2,2],[-2,1],[-2,3],[-3,6],[-1,5],[1,5],[2,4],[2,2],[-4,0],[-1,2],[0,2],[1,3],[0,2],[0,1],[-1,4],[1,5],[4,5],[1,4],[-3,1],[-2,-1],[-1,-2],[-2,-3],[0,-2],[-2,0],[-1,2],[-2,5],[-2,11],[-1,6],[3,-1],[6,-4],[3,0],[2,2],[2,4],[2,3],[3,-1],[2,-3],[1,-11],[2,-4],[1,-1],[3,-2],[1,-2],[3,-5],[1,2],[-2,3],[-1,2],[0,1],[1,1],[2,0],[3,-4],[9,-13],[4,-5],[2,-2],[1,-1],[1,-1],[-1,-1],[0,-1],[-1,-1],[0,-2],[0,-1],[1,-3],[1,0],[1,3],[0,7],[0,3],[-1,1],[-2,1],[0,1],[1,1],[2,3],[1,1],[10,3],[2,3],[1,6],[5,-5],[7,-2],[11,0],[3,-1],[2,-2],[2,-2],[1,-5],[1,-2],[0,-1],[0,-2],[-1,-2],[0,-1],[-1,-2],[-2,-8],[4,6],[2,3],[3,-2],[1,-2],[5,-4],[2,-3],[0,-3],[1,-12],[1,-3],[8,-6],[-1,5],[-2,3],[-2,2],[-2,4],[0,4],[0,5],[0,3],[-1,2],[-4,2],[-2,1],[-1,1],[-3,5],[-1,1],[0,1],[-3,7],[0,1],[-2,6],[-1,1],[-1,1],[-1,2],[-1,2],[1,3],[-3,-1],[-7,1],[-12,-1],[-5,4],[-3,1],[-1,-3],[-1,-4],[-1,-3],[-2,-2],[-3,-2],[-1,0],[-4,0],[-2,0],[0,-2],[-1,-1],[-2,0],[-1,2],[-1,4],[-1,1],[-1,0],[-2,1],[-1,2],[0,3],[-2,4],[-3,2],[-4,1],[-1,1],[-1,3],[-1,4],[-3,5],[-2,-2],[-2,3],[-2,5],[-1,2],[-1,1],[-1,2],[-1,2],[0,3],[0,3],[0,2],[2,4],[3,8],[0,4],[-2,9],[0,5],[2,4],[2,3],[2,1],[3,0],[2,2],[1,4],[1,5],[2,1],[11,-5],[9,-2],[1,2],[-2,3],[-1,2],[-2,0],[-4,-2],[-3,1],[-5,4],[-3,1],[-1,2],[-1,0],[-1,0],[-1,0],[-1,1],[0,1],[0,2],[0,1],[1,0],[0,1],[1,4],[4,10],[1,2],[0,2],[0,4],[0,2],[1,2],[3,2],[-3,1],[-4,-3],[-3,-5],[-2,-5],[-2,-6],[-2,-5],[-1,-2],[0,-1],[-2,-2],[-1,0],[-1,2],[-1,-7],[-3,-4],[-3,-3],[-2,-8],[-8,-13],[0,5],[3,7],[0,4],[-1,3],[-1,-2],[-3,-9],[0,7],[0,3],[1,3],[-3,6],[-2,2],[-3,0],[0,-1],[4,-6],[1,-12],[-1,-10],[-3,-4],[-2,3],[-3,10],[-2,-1],[0,-5],[3,-10],[-1,-2],[-2,0],[-3,-2],[-3,-2],[-1,-3],[-1,-2],[-1,-3],[0,-3],[-2,-1],[-1,-1],[-1,0],[-1,-2],[-5,-7],[-1,-3],[0,-6],[0,-10],[-1,-6],[-2,-1],[0,-2],[3,-1],[0,-5],[0,-6],[0,-5],[-1,-3],[-2,-5],[-1,-5],[-2,-1],[-5,2],[-29,43],[-2,1],[-7,10],[-1,2],[-1,3],[-4,8],[-2,2],[0,1],[1,0],[2,-1],[2,-2],[-2,5],[-2,3],[-2,1],[-3,3],[-1,1],[-2,7],[-1,0],[-2,-1],[-1,1],[-1,1],[-3,5],[-4,4],[-1,1],[-1,2],[-1,5],[-1,2],[-1,1],[-5,2],[3,3],[1,3],[0,3],[0,1],[-5,-4],[-1,0],[-1,2],[-3,3],[-1,2],[-4,8],[-1,2],[2,6],[1,8],[-1,18],[0,5],[4,9],[0,4],[2,0],[6,2],[2,1],[2,-2],[0,-2],[0,-2],[1,-2],[1,-2],[2,-2],[1,-1],[3,-7],[3,-6],[0,-2],[0,-2],[1,-2],[0,-1],[1,-1],[1,0],[1,0],[1,-1],[1,-1],[1,-3],[0,-1],[1,0],[2,1],[1,-1],[0,-1],[0,-3],[0,-1],[2,-4],[1,-1],[1,-1],[-1,4],[-1,5],[-1,4],[-4,3],[-5,7],[-2,3],[-1,3],[-1,5],[-1,2],[-3,5],[-1,1],[-1,4],[-1,2],[-1,3],[0,5],[2,3],[3,3],[3,2],[2,0],[4,0],[2,-1],[3,-3],[1,0],[4,1],[9,-7],[3,1],[3,3],[2,4],[2,1],[2,1],[3,3],[2,1],[1,-1],[-2,4],[-2,-1],[-6,-4],[-1,-1],[-2,-5],[0,-1],[-4,0],[-16,9],[-7,1],[-6,-2],[-8,-5],[-3,-1],[-1,-1],[-2,-5],[-1,-1],[-5,1],[-2,-1],[-1,-1],[-1,-3],[-2,-1],[-1,1],[-2,1],[-1,2],[-1,2],[-2,3],[-11,3],[-1,2],[0,4],[-1,1],[1,1],[6,5],[4,0],[-1,2],[-3,2],[-1,2],[0,2],[0,4],[0,3],[-2,2],[0,2],[1,11],[0,5],[-1,-4],[-1,-4],[-1,-4],[-1,-1],[-3,-1],[-4,-4],[-3,0],[-6,3],[-2,0],[-2,1],[-2,2],[-1,4],[1,4],[-1,3],[1,4],[0,3],[1,3],[3,2],[5,0],[2,2],[-5,6],[-1,4],[-1,6],[0,5],[1,3],[0,2],[2,0],[2,-1],[1,0],[1,3],[-1,2],[-4,5],[-2,4],[1,2],[3,0],[3,-2],[2,-3],[2,-3],[3,-8],[3,-6],[1,-2],[1,-3],[25,-44],[3,-2],[2,-2],[3,-1],[2,1],[1,0],[-1,2],[-1,1],[-2,0],[-1,0],[-2,0],[-1,2],[-5,9],[-2,3],[0,5],[3,9],[-1,3],[1,4],[1,2],[4,1],[2,1],[2,3],[0,2],[-2,2],[-1,-1],[-6,-4],[-2,-3],[0,-3],[-1,-3],[0,-5],[-1,-4],[-4,3],[-7,10],[-2,3],[-9,18],[0,1],[1,3],[-4,4],[-5,7],[-1,8],[4,7],[5,4],[6,0],[6,-1],[7,-5],[3,-2],[1,-1],[1,-1],[2,-6],[3,-4],[3,-2],[10,-2],[2,0],[1,0],[2,2],[-1,1],[-1,1],[-8,0],[-3,1],[-3,2],[-2,4],[-5,9],[-2,2],[-1,0],[-2,1],[-2,3],[-2,1],[-3,1],[-2,1],[1,6],[1,2],[1,2],[0,1],[2,5],[0,2],[0,3],[1,2],[0,1],[2,0],[1,0],[3,4],[1,2],[2,3],[1,3],[1,3],[-2,1],[-1,-1],[-3,-6],[-2,-2],[-4,-2],[-2,-1],[1,3],[2,4],[2,3],[3,3],[5,13],[2,3],[3,2],[2,0],[3,-1],[2,-3],[-1,-3],[-1,-2],[-2,-4],[2,0],[1,1],[1,1],[1,1],[1,3],[0,1],[1,-1],[3,1],[3,1],[4,8],[2,2],[1,-1],[3,-3],[2,-1],[2,1],[3,2],[1,0],[6,-1],[3,2],[1,4],[-5,-3],[-32,3],[-2,-1],[-2,0],[-1,2],[0,2],[1,3],[2,6],[2,10],[2,5],[2,1],[3,8],[2,2],[1,1],[1,2],[1,2],[0,2],[0,2],[0,1],[1,2],[0,2],[2,5],[0,2],[1,2],[1,1],[3,2],[1,2],[1,1],[1,0],[0,2],[0,1],[0,2],[0,1],[4,9],[2,4],[2,2],[1,1],[0,3],[0,3],[0,1],[1,1],[2,6],[1,1],[3,2],[8,-2],[2,1],[4,4],[1,1],[0,1],[0,3],[0,1],[-1,-1],[-1,-1],[-1,-1],[-3,-3],[-1,-1],[-2,0],[-4,1],[-5,0],[-1,0],[-1,-2],[-4,-8],[0,-1],[-3,2],[0,10],[-3,5],[-1,4],[-4,23],[-2,7],[-2,-2],[1,-4],[0,-4],[0,-11],[3,-8],[1,-6],[0,-3],[-1,-1],[0,-3],[0,-2],[0,-2],[-1,-2],[-2,-2],[-1,-1],[-1,-2],[-1,-3],[0,-2],[0,-3],[1,-2],[1,1],[0,1],[0,3],[1,2],[1,1],[1,0],[1,1],[-1,-3],[-1,-1],[-1,-1],[1,-2],[0,-1],[1,-1],[1,0],[0,-2],[0,-2],[-1,-2],[-3,-1],[0,-2],[0,-6],[0,-2],[0,-1],[-1,-2],[-3,-6],[-2,-2],[-1,-5],[-2,-10],[-1,-4],[-10,-17],[-2,-8],[-2,-7],[-2,-7],[-1,-1],[-2,-2],[-3,0],[-2,2],[-1,4],[0,4],[-4,12],[3,6],[2,9],[1,1],[1,1],[2,5],[0,2],[1,1],[7,9],[7,11],[1,5],[-2,4],[-4,7],[0,4],[0,10],[-1,3],[0,2],[-1,6],[-1,5],[-1,2],[-1,2],[-1,2],[-3,8],[-2,7],[0,9],[1,11],[1,6],[0,3],[0,3],[-1,2],[-1,1],[-2,1],[-1,4],[-1,5],[0,5],[2,5],[9,17],[6,13],[2,4],[-1,3],[-1,1],[-2,-1],[-1,-1],[-1,-5],[0,3],[0,10],[0,10],[-1,5],[-1,5],[-4,10],[-3,4],[-2,1],[-11,-4],[-3,1],[-6,2],[-13,5],[-1,2],[-1,3],[-4,16],[-1,4],[-3,1],[-12,4],[-5,3],[-13,21],[-9,6],[-20,4],[-9,5],[-14,10],[-14,13],[-17,13],[-5,6],[-3,3],[-2,0],[-12,-3],[-13,1],[-3,1],[0,3],[0,10],[0,5],[-1,4],[-3,7],[-2,4],[0,2],[-1,3],[1,2],[1,4],[1,3],[-1,2],[-26,15],[8,41],[-15,6],[-20,7],[7,16],[6,12],[-9,20],[-6,16],[-11,18],[-11,19],[-8,19],[-9,21],[-14,30],[-10,22],[-9,15],[-9,16],[-2,6],[-5,21],[-3,6],[-10,15],[-14,20],[-4,10],[-2,10],[4,7],[-8,14],[-9,11],[-9,10],[-9,12],[-10,23],[-4,6],[-5,4],[-13,8],[-14,10],[-13,8],[-10,15],[-2,3],[0,4],[0,3],[1,1],[0,1],[-1,2],[-1,1],[-7,2],[2,11],[0,4],[-2,4],[-6,11],[-3,4],[-3,2],[-5,0],[-3,1],[-4,4],[-5,9],[-1,3],[-2,7],[-1,3],[-1,1],[-5,1],[-10,3],[-11,3],[-3,3],[-2,4],[-4,9],[0,4],[1,4],[1,3],[-11,9],[1,5],[5,5],[2,4],[0,4],[1,12],[-1,4],[-1,3],[-7,8],[-2,2],[-5,2],[-4,6],[-4,7],[-4,5],[-10,8],[-6,6],[-7,9],[-2,0],[-18,-11],[-22,-13],[-10,-6],[-12,-3],[-13,-4],[-13,-4],[-10,-7],[5,-4],[5,-5],[2,-7],[-4,-10],[-5,-6],[-5,-3],[-6,1],[-6,2],[0,-18],[0,-24],[-1,-7],[-1,-1],[-9,-19],[-5,-5],[-7,-1],[-13,2],[-6,-2],[-11,-9],[-12,-9],[-2,-2],[-12,-9],[-13,-10],[-6,-8],[-10,-11],[-3,-2],[-4,0],[-2,3],[0,7],[2,12],[-4,19],[-3,15],[-4,18],[-1,4],[-2,4],[-17,18],[-17,19],[-18,20],[-17,19],[-16,18],[-14,16],[-13,14],[-4,7],[-2,5],[-4,20],[-1,2],[-1,1],[-4,3],[-11,6],[-19,10],[-5,4],[-1,1]],[[220,4422],[2,0],[53,0],[54,0],[54,0],[53,0],[54,0],[54,0],[54,0],[54,0],[54,0],[54,0],[53,0],[54,0],[54,0],[54,0],[54,0],[54,0],[53,0],[54,0],[54,0],[54,0],[54,0],[54,0],[54,0],[53,0],[54,0],[54,0],[54,0],[54,0],[54,0],[54,0],[53,0],[54,0]],[[1944,4422],[54,0],[54,0],[54,0],[54,0],[54,0],[54,0],[54,0],[54,0]],[[7006,2406],[-1,0],[0,1],[0,1],[0,1],[1,1],[1,0],[1,0],[0,-2],[-2,-2]],[[6991,2437],[1,-1],[-1,0],[-1,-1],[-2,-2],[-5,-3],[-1,1],[0,2],[6,4],[3,0]],[[6983,2483],[3,-1],[1,1],[0,-2],[-1,-3],[-1,0],[-6,-3],[-1,1],[0,3],[1,2],[3,3],[1,-1]],[[6991,2491],[-2,-2],[-1,1],[0,2],[3,3],[1,0],[0,-2],[-1,-2]],[[6986,2511],[-5,-5],[-2,-3],[-1,-3],[0,-3],[-1,-2],[0,-7],[0,-3],[-1,-2],[-2,-2],[-2,-2],[-2,-1],[-2,-1],[-12,2],[-8,-3],[-2,-3],[-1,0],[-1,0],[0,1],[0,2],[0,2],[-4,11],[2,7],[1,2],[1,1],[1,-1],[2,-1],[1,-1],[3,1],[3,2],[2,4],[2,4],[2,1],[3,0],[1,0],[6,4],[3,4],[2,2],[2,0],[4,-2],[1,0],[2,-2],[2,-3]],[[7020,2510],[-5,4],[1,3],[1,2],[2,0],[1,-3],[0,-3],[0,-1],[0,-1],[0,-1]],[[7014,2516],[-3,-1],[-6,1],[0,2],[4,1],[2,1],[1,-1],[2,-1],[0,-2]],[[6961,2550],[-3,-1],[0,3],[2,1],[4,3],[4,-1],[3,-1],[0,-1],[0,-2],[-2,-1],[-8,0]],[[7042,2600],[2,0],[2,0],[1,0],[0,-1],[-1,-2],[-2,-2],[-2,-1],[1,-1],[2,0],[0,-1],[-2,-2],[-2,0],[0,2],[-1,0],[-1,0],[-1,0],[-1,0],[0,1],[0,3],[0,1],[-1,-1],[-1,-1],[-1,0],[1,2],[2,1],[4,2],[1,0]],[[6997,2624],[-1,-5],[-1,0],[0,1],[-1,1],[0,1],[0,1],[1,1],[0,1],[-1,1],[0,1],[1,0],[2,-1],[0,-2]],[[6945,2621],[-3,0],[0,1],[2,4],[1,4],[0,2],[1,0],[1,-1],[1,-1],[0,-2],[0,-2],[2,-2],[0,-1],[-1,0],[-4,-2]],[[7029,2727],[-1,-1],[-1,1],[-1,-1],[-1,0],[-2,-1],[4,4],[2,1],[1,0],[0,-3],[-1,0]],[[7014,2742],[0,-2],[-2,1],[0,1],[-1,1],[1,1],[2,-1],[0,-1]],[[6923,2757],[-1,-1],[-2,-1],[-2,-1],[-4,2],[-3,-1],[-2,1],[-1,8],[1,6],[2,5],[2,2],[5,-3],[0,-1],[1,-2],[0,-2],[4,-5],[1,-1],[0,-3],[1,0],[-1,-1],[-1,-2]],[[6740,2789],[28,-4],[6,-3],[13,-18],[3,-3],[1,-1],[1,-2],[2,-8],[1,-2],[7,-7],[11,-20],[1,-3],[1,-4],[4,-14],[1,-3],[5,-16],[1,-10],[-4,-5],[0,-5],[-2,-1],[-3,1],[-4,6],[-28,7],[-14,12],[-15,10],[-21,4],[-20,10],[-17,10],[-18,7],[-7,9],[-5,3],[1,4],[5,7],[0,2],[3,7],[4,9],[7,9],[3,1],[9,-1],[2,1],[11,5],[6,2],[13,0],[8,4]],[[7017,2802],[-3,-2],[-1,2],[1,1],[1,1],[1,0],[1,-1],[0,-1]],[[7014,2806],[0,-1],[-1,-1],[-4,-1],[0,1],[0,1],[1,1],[3,-1],[1,1]],[[6940,2808],[0,-1],[-3,0],[1,1],[2,0]],[[6907,2822],[0,-1],[0,-1],[0,-2],[-1,-7],[5,-6],[-1,-5],[-3,-2],[-4,-1],[-4,3],[-3,8],[-3,2],[-1,3],[1,6],[3,3],[8,0],[4,1],[-1,-1]],[[7015,2834],[0,-2],[0,-1],[-1,0],[-1,1],[-1,0],[-1,0],[0,1],[4,1]],[[7001,2843],[1,0],[1,0],[-1,-1],[-1,-1],[1,-2],[-1,-1],[0,1],[0,1],[-2,0],[0,-1],[0,-1],[-1,0],[-1,0],[0,1],[1,1],[2,3],[1,1],[0,-1]],[[6990,2858],[0,-1],[-1,0],[-1,0],[-1,0],[0,-1],[-1,1],[-1,0],[1,0],[0,1],[1,0],[2,0],[1,0]],[[6934,2856],[-2,0],[-1,1],[-2,2],[0,1],[-1,2],[-1,1],[1,1],[0,1],[7,-7],[-1,-1],[0,-1]],[[6998,2910],[-1,0],[-1,1],[0,1],[0,1],[1,0],[1,0],[1,-1],[-1,-2]],[[6916,2953],[-1,-1],[-3,3],[-2,2],[-1,4],[2,-1],[6,-4],[0,-2],[-1,0],[0,-1]],[[6966,3019],[0,-1],[-1,0],[-2,0],[-1,0],[0,-1],[-1,-2],[-1,0],[-1,0],[-1,0],[0,-1],[-1,1],[0,1],[1,2],[2,1],[1,-1],[2,2],[3,-1]],[[6967,3033],[-5,-1],[0,1],[1,0],[1,0],[0,1],[1,0],[2,0],[1,0],[-1,-1]],[[6971,3037],[-1,0],[-3,-1],[1,1],[0,1],[-1,1],[1,0],[1,0],[2,-1],[0,-1]],[[6777,3055],[0,-1],[-1,1],[0,3],[0,2],[0,3],[1,2],[1,-1],[0,-3],[0,-2],[0,-3],[-1,-1]],[[6955,3073],[-1,0],[0,1],[-1,0],[-1,1],[0,1],[1,-1],[1,0],[1,0],[0,-2]],[[6984,3190],[-3,-3],[-2,1],[-10,-8],[-23,-5],[-17,-12],[-3,1],[2,3],[3,2],[4,1],[2,0],[0,2],[-5,0],[4,6],[4,3],[21,8],[12,1],[2,1],[5,3],[2,1],[30,7],[-1,-2],[-1,-2],[-13,-6],[-10,-1],[-3,-1]],[[7174,3321],[-7,-2],[4,3],[19,23],[4,7],[4,2],[1,2],[3,5],[1,3],[2,1],[8,7],[4,2],[0,-4],[-2,0],[-1,-2],[-9,-12],[-4,-3],[-3,-2],[-17,-23],[-7,-7]],[[7017,3465],[-3,-4],[4,3],[2,1],[2,-2],[-4,-5],[-3,-5],[-3,-7],[-1,-9],[-2,-9],[-3,-11],[-4,-8],[-4,-1],[2,3],[2,6],[2,6],[4,17],[3,13],[0,2],[1,8],[0,6],[1,3],[1,1],[2,0],[2,4],[2,-1],[0,-2],[-1,-3],[0,-2],[1,-1],[0,-2],[-2,0],[-1,-1]],[[7271,3486],[-5,-4],[-1,1],[1,4],[1,4],[-1,3],[0,3],[1,3],[1,0],[1,-3],[2,-1],[1,-2],[1,-3],[-1,-3],[-1,-2]],[[7271,3507],[-4,-3],[1,1],[0,1],[1,1],[0,2],[-2,3],[-1,2],[1,1],[1,1],[1,0],[1,-2],[1,-2],[1,-3],[-1,-2]],[[7127,3530],[0,-1],[0,-2],[0,2],[-1,0],[-2,0],[-1,-2],[0,-1],[-1,-1],[-2,1],[-1,0],[-1,1],[1,2],[3,2],[4,-1],[1,0]],[[7144,3533],[-3,-3],[-4,-1],[-1,0],[-5,-3],[-1,0],[1,3],[1,2],[0,1],[1,1],[1,-1],[11,3],[-1,-2]],[[7277,3528],[-8,-4],[0,2],[6,8],[2,4],[0,1],[-1,1],[0,2],[2,-1],[0,-2],[-1,-1],[1,-1],[1,-1],[0,-2],[0,-2],[-2,-4]],[[7153,3536],[-1,0],[0,1],[1,2],[0,1],[-1,0],[1,2],[1,1],[0,-1],[0,-3],[-1,-1],[0,-2]],[[7158,3543],[0,-1],[-1,0],[-1,2],[-1,2],[1,1],[1,-2],[1,-2]],[[7154,3547],[0,-2],[0,-1],[-1,0],[0,1],[-1,1],[-1,-1],[-1,0],[1,3],[1,1],[1,0],[1,-2]],[[7184,3551],[-2,-3],[-2,1],[0,2],[0,2],[1,2],[1,1],[1,2],[1,1],[0,2],[0,2],[1,1],[1,0],[1,-1],[0,-2],[-1,-1],[-1,-2],[-1,-7]],[[7052,3542],[1,-8],[1,-7],[-1,-2],[0,-3],[0,-4],[0,-4],[0,-3],[-2,-3],[-1,-3],[1,-4],[-1,-1],[-2,-1],[-1,1],[-1,1],[-2,-4],[-2,-1],[-1,0],[1,2],[-2,1],[-3,-3],[-2,-4],[-2,-3],[-6,-2],[-5,-1],[0,3],[1,7],[1,3],[8,18],[2,5],[-2,2],[1,3],[2,3],[0,2],[-1,2],[1,2],[1,5],[-3,0],[1,3],[3,5],[1,5],[2,3],[2,5],[3,3],[3,1],[4,-2],[1,-5],[-1,-17]],[[7282,3562],[-2,-2],[-3,2],[0,2],[0,2],[2,5],[1,1],[2,-3],[1,-2],[0,-2],[-1,-3]],[[6942,3546],[-4,-9],[-5,-5],[-29,-26],[-6,-3],[-4,-1],[-3,1],[-1,2],[-1,1],[1,1],[0,7],[0,1],[4,8],[1,3],[-1,3],[0,2],[4,3],[3,2],[13,1],[2,1],[1,3],[0,2],[-2,3],[0,2],[0,1],[1,1],[1,0],[3,-1],[7,-3],[3,0],[2,0],[3,3],[2,0],[1,1],[2,5],[3,3],[2,4],[1,5],[1,4],[1,5],[1,4],[2,3],[2,2],[0,-2],[-1,-1],[0,-2],[-1,-1],[-1,-8],[-3,-10],[-5,-15]],[[7122,3589],[0,-1],[-1,0],[-1,1],[2,0]],[[7117,3594],[-2,-1],[-1,0],[0,2],[2,1],[1,-2]],[[7042,3616],[-3,-1],[-3,1],[-1,1],[-2,2],[-2,1],[1,0],[7,0],[2,1],[2,-1],[1,-2],[-2,-2]],[[6996,3616],[3,-6],[0,-4],[-2,-3],[-3,4],[-1,5],[-4,2],[-3,-4],[0,-9],[0,-8],[2,-3],[5,0],[3,-1],[3,0],[1,0],[1,-13],[3,-8],[4,-5],[5,-2],[6,1],[3,-1],[1,-5],[0,-3],[-1,-5],[0,-2],[1,-2],[1,-2],[1,-1],[-3,-1],[0,1],[-2,-4],[0,-2],[0,-1],[8,2],[-1,-4],[-2,-2],[-2,-1],[-2,-2],[-2,-3],[-1,-7],[-1,-4],[-10,-22],[-4,-21],[-11,-36],[-1,-11],[-1,1],[0,2],[0,3],[1,2],[1,4],[2,10],[3,11],[1,7],[3,8],[1,7],[14,43],[1,10],[1,9],[-3,2],[1,5],[1,3],[0,2],[-3,-1],[0,-1],[0,-1],[-1,-1],[0,-1],[-2,-1],[-1,0],[-1,-2],[-1,-3],[-5,-28],[0,-2],[0,-2],[2,1],[1,0],[1,1],[1,2],[-1,-3],[0,-2],[-1,0],[1,-2],[1,0],[-2,-5],[-4,-10],[-1,-6],[-1,-1],[-2,-6],[0,-3],[-8,-29],[-8,-21],[1,5],[9,30],[8,36],[0,5],[0,6],[-2,5],[-1,2],[-2,-1],[-2,-4],[-7,-17],[-12,-40],[-1,-2],[-2,-2],[-10,-20],[-4,-3],[-5,1],[-6,4],[-2,3],[-1,1],[-2,-2],[-10,-20],[-2,-2],[0,-1],[-1,-1],[-1,-1],[-2,0],[3,6],[15,30],[10,25],[2,4],[6,10],[0,4],[-2,3],[-4,1],[-2,-1],[-5,-5],[-15,-32],[-13,-19],[-2,-7],[-1,-2],[-1,-4],[-1,-2],[-2,-1],[-1,-2],[-1,4],[-1,4],[-1,2],[-1,2],[-1,4],[-2,1],[-7,1],[3,10],[2,3],[4,4],[24,46],[21,26],[3,6],[2,9],[4,24],[1,12],[-2,11],[1,0],[0,1],[1,0],[0,1],[-1,3],[0,1],[2,2],[1,-1],[2,-11],[0,-1],[1,-2],[-1,-4],[0,-4],[0,-8],[-1,-4],[-3,-8],[0,-3],[0,-1],[-1,-2],[-1,-2],[0,-3],[0,-6],[0,-2],[0,-4],[-3,-9],[-1,-4],[-7,-12],[-2,-3],[-1,-2],[0,-3],[0,-3],[0,-2],[1,0],[1,1],[1,2],[1,5],[1,2],[2,2],[1,2],[4,2],[6,2],[2,2],[2,3],[0,2],[6,28],[4,12],[1,9],[2,11],[1,26],[0,4],[0,14],[3,5],[6,1],[5,-7]],[[7079,3627],[4,-1],[4,0],[2,0],[0,-1],[0,-1],[-1,0],[-10,2],[0,1],[1,0]],[[7281,3590],[-1,0],[0,3],[0,3],[0,3],[0,4],[0,5],[2,8],[0,14],[1,-1],[1,-4],[0,-3],[-1,0],[0,-1],[0,-1],[0,-1],[1,-4],[0,-1],[-1,-1],[0,-3],[0,-4],[1,-7],[-1,-5],[-2,-4]],[[7281,3639],[1,0],[1,-2],[0,-1],[0,-1],[1,-2],[0,-3],[-1,0],[-3,2],[-1,2],[0,2],[0,1],[0,3],[1,1],[1,-1],[0,-1]],[[6952,3647],[1,0],[2,0],[1,0],[1,-1],[0,-3],[0,1],[0,1],[1,-1],[0,-1],[2,-7],[0,-4],[-1,-2],[1,-3],[0,-2],[-1,-8],[-1,0],[-2,1],[-2,-2],[-2,-6],[-2,-3],[-1,3],[-1,4],[1,2],[1,1],[1,3],[0,7],[1,6],[-1,5],[-4,5],[2,2],[0,2],[-1,1],[0,1],[0,2],[1,2],[1,2],[0,-1],[0,-1],[1,-1],[0,-1],[1,-4]],[[6915,3639],[-1,-1],[-3,0],[-3,3],[-1,6],[0,7],[1,6],[0,1],[3,4],[1,1],[4,1],[1,-1],[4,-4],[1,-4],[-1,-5],[-1,-4],[-1,-1],[-1,0],[-1,-1],[0,-1],[-1,-3],[0,-1],[-1,-3]],[[7281,3664],[-1,-2],[-1,0],[0,1],[-1,-1],[-1,1],[1,1],[0,2],[1,1],[0,2],[0,1],[1,-2],[1,-1],[0,-1],[0,-2]],[[6925,3684],[3,-3],[1,-3],[0,-3],[1,-3],[2,-9],[1,-8],[-1,-4],[-3,-2],[-3,-1],[-1,1],[0,3],[0,4],[0,3],[-1,7],[0,3],[-2,5],[-7,-3],[-3,2],[1,2],[2,5],[2,3],[1,1],[1,-1],[2,-2],[1,3],[1,3],[1,0],[1,-3]],[[7277,3689],[0,-2],[-1,0],[-1,1],[0,3],[0,1],[0,2],[1,2],[2,1],[1,-3],[0,-2],[-1,-2],[-1,-1]],[[7276,3697],[-1,-1],[-1,1],[0,2],[0,1],[-1,1],[0,2],[0,6],[0,1],[0,2],[1,0],[4,-2],[0,-1],[2,-4],[0,-3],[-2,-2],[-1,0],[-1,-1],[0,-1],[0,-1]],[[7279,3714],[-2,0],[-2,2],[-1,2],[0,5],[0,2],[-2,6],[0,2],[0,1],[-1,3],[0,4],[0,3],[1,1],[1,1],[2,-1],[2,-5],[0,-5],[1,-1],[2,-2],[0,-2],[-1,-2],[-1,-1],[-1,-2],[1,-3],[2,-2],[0,-2],[-1,-1],[1,0],[0,-1],[0,-1],[-1,-1]],[[7098,3745],[-1,0],[-1,0],[0,-1],[-1,1],[-1,0],[0,-2],[-1,-2],[-2,-1],[-1,0],[-1,0],[-2,0],[-3,1],[1,2],[1,1],[1,0],[0,1],[0,1],[1,0],[3,3],[1,0],[2,-1],[3,-2],[1,0],[0,-1]],[[7272,3752],[0,-2],[0,-1],[0,-1],[-1,1],[0,1],[0,1],[-1,1],[-1,-1],[-1,1],[-1,4],[-2,6],[1,1],[1,1],[2,-1],[1,-3],[1,-2],[0,-2],[0,-1],[1,-1],[0,-2]],[[7085,3768],[-4,-2],[-2,0],[-4,2],[-2,0],[0,1],[1,1],[2,2],[8,6],[4,0],[3,-3],[-2,-3],[-4,-4]],[[7075,3784],[-4,-1],[-2,1],[1,3],[3,2],[4,0],[3,-1],[0,-1],[-1,-2],[-4,-1]],[[7270,3767],[-2,-1],[-1,1],[0,5],[-1,2],[-1,2],[-1,4],[0,2],[-2,5],[0,3],[-1,3],[0,2],[-1,2],[-1,3],[0,1],[1,-1],[1,-1],[1,-1],[1,-2],[4,-3],[1,-3],[0,-1],[1,-1],[0,-3],[1,-2],[-1,-2],[0,-2],[0,-2],[0,-2],[1,-3],[1,-3],[-1,-2]],[[7069,3825],[-1,0],[-1,1],[-1,2],[0,2],[4,0],[0,-1],[0,-2],[-1,-2]],[[7255,3821],[-1,-1],[-2,11],[-1,6],[0,2],[1,0],[4,-8],[0,-1],[0,-2],[0,-2],[0,-2],[-1,-3]],[[6936,3832],[0,-6],[-1,-4],[-2,-2],[-5,4],[-2,-2],[0,-4],[0,-12],[-1,-4],[-2,1],[-2,5],[-2,5],[-1,3],[2,1],[1,-1],[2,0],[1,6],[1,1],[1,1],[0,2],[0,2],[-2,0],[-2,-3],[-1,1],[0,3],[1,4],[1,3],[2,4],[2,4],[2,1],[1,1],[0,2],[1,1],[1,-2],[1,-2],[1,-4],[1,-4],[1,-5]],[[6941,3916],[0,-2],[-1,0],[0,1],[0,1],[1,1],[-1,2],[1,1],[0,-2],[0,-2]],[[7176,3986],[2,-3],[2,-1],[-5,1],[-1,2],[-1,0],[-1,-1],[-1,1],[0,1],[0,2],[0,1],[-7,4],[-1,2],[2,0],[1,0],[2,-2],[5,-4],[1,-2],[1,-1],[1,0]],[[7165,3999],[-3,-1],[-5,2],[-24,19],[0,1],[3,0],[1,0],[-1,0],[12,-6],[14,-13],[3,-1],[0,-1]],[[7119,4031],[-1,-2],[-1,0],[-1,1],[-1,0],[-4,2],[-1,1],[-2,2],[-1,0],[-1,0],[0,1],[-2,1],[0,1],[10,-1],[3,-2],[2,-4]],[[6815,4041],[0,-2],[-1,1],[-1,-2],[-2,1],[-1,0],[0,1],[2,1],[0,-1],[1,-1],[1,1],[1,1]],[[7108,4043],[-1,-1],[-1,2],[-5,3],[-2,2],[0,1],[2,0],[4,-3],[0,-1],[2,-1],[0,-1],[1,-1]],[[7059,4093],[-1,-1],[-1,-1],[-1,1],[-1,1],[0,-2],[2,-3],[16,-15],[3,-1],[0,-2],[-20,14],[-6,6],[1,1],[0,1],[1,1],[1,0],[0,1],[-2,2],[-1,3],[-1,4],[0,5],[1,-2],[1,-1],[0,-2],[2,0],[1,-3],[3,-5],[2,-2]],[[7056,4131],[0,-2],[0,-1],[1,-1],[0,-1],[-1,0],[-1,0],[-1,0],[0,1],[-1,1],[-3,0],[-1,0],[-1,0],[-1,1],[1,2],[1,0],[1,-1],[1,0],[0,1],[-2,1],[1,1],[4,0],[1,-1],[0,-1],[1,0]],[[7065,4137],[0,-2],[-1,0],[0,-1],[-1,0],[0,-1],[1,-1],[-1,0],[-1,-1],[-1,2],[-1,0],[0,-1],[-2,0],[-1,1],[0,1],[-1,2],[1,0],[0,1],[1,-1],[2,0],[1,0],[0,2],[2,-2],[1,1],[0,1],[1,0],[0,-1]],[[7027,4135],[-1,1],[-1,1],[0,1],[-1,1],[-2,1],[-1,1],[0,1],[0,1],[1,1],[-1,4],[2,1],[1,-3],[2,-5],[0,-3],[1,-3]],[[7063,4167],[-1,-1],[-2,0],[0,1],[0,1],[0,1],[0,1],[1,0],[1,0],[1,0],[0,-1],[0,-2]],[[6835,4194],[-3,-1],[0,1],[0,1],[0,1],[0,1],[1,0],[1,0],[2,-2],[-1,-1]],[[6840,4204],[-1,-1],[-3,0],[-1,2],[2,1],[2,2],[1,-1],[1,-2],[-1,-1]],[[6832,4239],[-1,-3],[0,1],[-2,1],[0,1],[1,4],[0,1],[1,1],[0,-1],[1,-2],[0,-3]],[[6855,4247],[0,-1],[1,1],[0,-2],[-1,-1],[-1,-2],[-1,0],[0,1],[1,1],[0,1],[0,1],[1,1]],[[6836,4253],[0,-2],[-1,-2],[-2,0],[-1,3],[-2,0],[0,1],[1,0],[2,-1],[1,0],[1,1],[0,1],[1,-1]],[[6866,4273],[-1,-1],[0,1],[1,2],[1,1],[0,1],[0,2],[1,0],[0,-4],[-2,-2]],[[6872,4272],[-1,-1],[0,1],[1,2],[-1,1],[-1,2],[0,1],[1,1],[0,1],[1,1],[0,-2],[0,-6],[0,-1]],[[6847,4291],[-1,-3],[-3,-8],[0,-4],[-1,-4],[-2,-1],[-1,2],[1,7],[-1,0],[3,6],[1,2],[-2,0],[-1,0],[-1,2],[7,8],[2,1],[-1,-2],[0,-1],[1,0],[1,-1],[-1,-2],[0,-1],[-1,-1]],[[6825,4292],[2,3],[0,1],[0,2],[3,3],[1,0],[-1,-3],[-5,-6]],[[6867,4317],[-4,-4],[2,3],[2,3],[1,1],[0,-1],[-1,-2]],[[6839,4338],[-2,-1],[-1,1],[0,2],[-1,1],[1,1],[4,4],[0,1],[3,2],[1,-1],[-1,-2],[-1,-2],[-3,-6]],[[6857,4346],[-1,0],[-1,0],[0,-1],[0,-1],[-1,-2],[0,-1],[-1,-3],[-2,-1],[0,3],[1,4],[2,5],[2,3],[1,1],[1,0],[1,-1],[0,-1],[-1,-1],[-1,-2],[0,-2]],[[7171,4354],[1,-2],[0,-1],[-1,-1],[-4,-1],[-1,1],[-1,-1],[-1,2],[-1,1],[-1,-1],[0,1],[0,1],[2,0],[2,0],[1,0],[0,1],[2,1],[1,-1],[1,0]],[[6876,4357],[1,-2],[5,-1],[2,-2],[0,-1],[-1,-2],[0,-1],[1,-2],[-1,0],[0,-1],[-1,-1],[-4,-2],[-2,-1],[-1,-3],[-1,-1],[-1,-4],[-1,-1],[-1,0],[-2,-1],[-2,-2],[-2,0],[-1,0],[-1,1],[-1,1],[1,1],[0,3],[1,2],[2,2],[2,2],[-1,3],[2,1],[1,0],[2,-1],[1,0],[0,2],[-1,2],[0,1],[1,1],[1,2],[1,3],[0,2],[1,-1],[0,1]],[[7197,4384],[0,-1],[-1,1],[-1,1],[0,2],[1,0],[1,-1],[0,-2]],[[7193,4388],[0,-1],[0,1],[-1,1],[0,1],[1,1],[0,1],[1,0],[0,-1],[0,-2],[-1,-1]],[[6902,4391],[1,0],[1,0],[1,2],[2,1],[2,-2],[1,-2],[1,-1],[2,-1],[2,1],[-2,-4],[-1,-3],[-4,-5],[-2,-2],[-1,-1],[-2,0],[-1,-1],[-3,-6],[-5,-3],[-3,0],[-2,2],[1,2],[2,3],[1,4],[-2,2],[-1,-3],[-1,-1],[-1,-1],[-1,-2],[-1,-1],[-1,0],[0,-1],[-1,-2],[0,-2],[0,-2],[0,-1],[-1,0],[-2,1],[0,2],[6,17],[2,4],[1,1],[4,2],[2,5],[2,1],[1,0],[1,1],[1,0],[1,-4]],[[6916,4395],[-1,-2],[-1,1],[1,2],[0,1],[2,0],[-1,-2]],[[6909,4400],[-1,0],[-1,0],[0,2],[0,2],[1,1],[2,0],[1,-1],[-1,-2],[-1,-2]],[[6917,4406],[-4,-7],[0,1],[1,5],[0,1],[0,2],[4,4],[0,1],[1,0],[0,-1],[0,-1],[-2,-5]],[[7169,4470],[0,-1],[-1,-1],[-2,2],[-5,-1],[-2,2],[8,1],[1,-1],[1,0],[0,-1]],[[7164,4473],[-2,2],[0,1],[-2,1],[-2,1],[-1,1],[1,0],[1,0],[-1,-1],[1,0],[3,0],[0,-1],[2,0],[1,-1],[-1,-2],[0,-1]],[[7169,4487],[1,0],[1,-2],[0,-1],[-1,0],[0,-1],[-1,0],[-1,0],[0,1],[0,1],[1,1],[0,1]],[[8662,4512],[-3,0],[-3,-3],[-3,-4],[0,-6]],[[8653,4499],[-9,7],[-3,2],[-3,1],[-6,0],[-3,1],[-3,2],[-8,10],[-3,7],[-1,6],[4,4],[2,1],[9,-2],[5,0],[3,1],[2,2],[1,-2],[2,-2],[1,-2],[0,-2],[2,0],[2,0],[1,-2],[-1,-5],[3,-1],[1,-3],[0,-4],[0,-5],[1,0],[3,5],[1,1],[6,0],[1,-1],[0,-2],[0,-3],[-1,-1]],[[7143,4561],[-1,0],[-1,0],[-1,0],[0,1],[0,2],[1,-1],[1,0],[1,-1],[0,-1]],[[8250,4565],[6,-2],[3,1],[2,-6],[4,-8],[4,-5],[3,-3],[5,-2],[1,-5],[-2,-12],[-5,-9],[-2,-1],[-3,-7],[-3,-4],[1,-7],[-2,-3],[-2,0],[-2,2],[-6,-1],[-2,-1],[-1,-3],[-3,-2],[-3,0],[-4,-3],[-1,-4],[-2,-3],[-5,-2],[-4,0],[-5,-5],[-6,0],[-5,3],[-2,12],[3,11],[4,17],[5,13],[3,10],[2,8],[5,20],[13,2],[3,-1],[3,0]],[[8637,4568],[-4,-4],[-1,1],[-1,1],[-1,2],[3,3],[6,11],[2,6],[1,1],[1,-1],[1,-1],[0,-1],[0,-2],[0,-5],[-2,-5],[-2,-3],[-3,-3]],[[7139,4603],[-3,-1],[1,2],[1,2],[1,-2],[0,-1]],[[7115,4611],[1,-1],[2,0],[-1,-2],[-8,-5],[-1,1],[-1,1],[0,2],[0,1],[7,4],[1,-1]],[[7102,4624],[0,-2],[0,-4],[-1,0],[-12,-9],[5,1],[2,0],[0,-1],[-7,-4],[-19,-5],[-9,-5],[-10,-1],[-2,1],[0,1],[6,6],[1,2],[1,2],[0,2],[1,2],[1,0],[40,12],[1,2],[1,-1],[1,1]],[[8622,4814],[1,-1],[2,1],[2,2],[2,1],[0,-2],[-1,-1],[0,-2],[6,1],[2,-1],[1,-1],[1,-2],[0,-1],[-2,-1],[-1,-2],[-3,-5],[-1,-1],[-1,-1],[-1,-2],[0,-1],[1,-1],[2,0],[1,-1],[1,-1],[-4,-5],[4,0],[1,-1],[-1,-3],[0,-2],[-1,-4],[-1,-3],[-2,-5],[-7,-3],[-2,-4],[1,-3],[-2,-5],[-4,-6],[-1,-4],[-3,-1],[-3,3],[-1,5],[-2,-2],[-3,-2],[-2,1],[-1,4],[1,3],[2,2],[2,1],[2,3],[-6,3],[-11,0],[-6,5],[0,2],[2,-1],[1,1],[1,1],[0,2],[-5,0],[-3,0],[-2,3],[-1,2],[0,4],[-1,2],[-1,1],[-2,0],[-3,0],[-2,0],[-1,2],[-1,3],[-1,1],[-2,1],[-5,1],[-2,2],[-5,5],[-1,1],[-3,3],[-2,1],[-2,2],[0,6],[2,5],[2,3],[3,1],[43,9],[3,-1],[5,-3],[29,-4],[-2,-4],[-4,-3],[-2,-3]],[[8509,4864],[1,-1],[1,1],[2,3],[2,1],[1,-1],[1,-4],[1,-2],[1,-6],[2,-6],[1,-2],[-2,-2],[-3,1],[-4,4],[-2,3],[-2,-5],[-3,-1],[-7,1],[-3,2],[-3,3],[-2,4],[6,5],[6,7],[6,5],[4,-4],[-1,-1],[-4,-2],[1,-1],[0,-2]],[[5425,4887],[3,-2],[-1,-1],[0,-3],[6,-3],[2,0],[-3,-5],[-4,-6],[-4,-3],[-3,1],[-2,3],[-3,7],[-2,2],[-5,-2],[-3,1],[-1,6],[1,3],[3,2],[10,-1],[2,1],[2,3],[2,-3]],[[5453,4892],[3,-3],[-1,-1],[-4,1],[-2,-2],[-3,1],[-1,2],[3,2],[5,0]],[[8591,4888],[4,-10],[1,-8],[2,3],[1,4],[2,3],[2,-3],[0,-6],[0,-7],[1,-2],[1,0],[2,2],[2,1],[1,-1],[1,-1],[1,-5],[4,-6],[1,-2],[1,-3],[-1,-1],[-1,0],[-4,-5],[-3,-2],[-3,0],[-3,1],[-9,9],[-13,4],[0,2],[-1,3],[-1,2],[-1,1],[-3,1],[-1,1],[-1,2],[-3,10],[-1,3],[0,4],[2,4],[1,0],[1,0],[2,1],[0,2],[1,3],[1,1],[3,0],[5,-2],[6,-3]],[[5472,4914],[-2,-3],[-4,4],[-1,3],[-1,5],[7,-4],[1,-5]],[[5513,5003],[3,-2],[12,1],[0,-2],[-3,0],[-7,-8],[-2,-1],[-2,2],[-7,-2],[-3,1],[4,5],[2,2],[2,0],[-2,3],[-3,0],[-3,-2],[-3,-1],[0,1],[2,1],[1,2],[0,1],[1,1],[2,1],[2,0],[2,-2],[2,-1]],[[6965,4996],[0,-2],[1,-6],[1,-1],[1,0],[5,-5],[0,-1],[2,-5],[1,-2],[1,-1],[4,-1],[1,-2],[2,-5],[1,-3],[-1,-4],[0,-4],[0,-10],[0,-4],[-2,-5],[-3,-8],[-2,-5],[0,-6],[0,-3],[0,-4],[1,-7],[-1,0],[-1,1],[-1,0],[-2,3],[-1,0],[-1,-1],[-1,-2],[0,-5],[-1,-3],[-1,-2],[-2,-3],[-1,-3],[-1,-2],[-1,-5],[-2,-4],[-5,-8],[0,-3],[-1,-3],[0,-2],[-1,-2],[0,-5],[-1,4],[0,-6],[-1,-4],[-1,-4],[-3,-5],[-3,-5],[-3,-10],[-1,-2],[-5,-7],[-4,-3],[-1,-2],[-1,-4],[-1,-1],[-6,-2],[-3,0],[-2,2],[0,1],[-1,2],[0,2],[-1,1],[-1,1],[-1,1],[-4,8],[-3,8],[-1,2],[-1,1],[-4,4],[-4,2],[-2,4],[-1,2],[-6,4],[-2,2],[-5,1],[-3,4],[-1,3],[-1,2],[0,3],[-2,2],[-2,6],[0,8],[-2,4],[-1,3],[0,3],[3,12],[0,8],[1,9],[4,10],[0,11],[6,15],[4,5],[6,6],[4,8],[5,7],[2,8],[3,6],[2,3],[2,2],[5,3],[4,3],[2,-1],[1,-2],[2,-1],[4,2],[2,1],[1,-1],[2,-2],[-1,-1],[0,-1],[0,-2],[2,0],[1,0],[3,4],[2,1],[3,4],[1,0],[6,1],[2,-1],[1,0],[2,-3],[1,-2],[2,0],[2,-1],[2,-1],[2,-2]],[[5496,5000],[-1,0],[-1,0],[-11,-3],[-3,0],[-3,2],[-1,2],[3,1],[2,2],[1,3],[2,2],[4,-4],[7,0],[3,-1],[-1,-2],[-1,-2]],[[8633,5034],[1,-1],[1,1],[2,2],[1,1],[11,-2],[0,5],[3,0],[7,-3],[-1,-5],[8,2],[2,-2],[0,-2],[0,-1],[1,-2],[2,0],[0,-2],[-15,-3],[0,-2],[3,-1],[5,-1],[3,-1],[-4,-5],[-2,-2],[-2,1],[-1,1],[-1,1],[-2,0],[-1,-1],[1,-1],[3,-6],[-5,-5],[-3,-2],[-3,-1],[-1,0],[-1,-2],[-1,-1],[-1,0],[-2,1],[-4,1],[-4,2],[-10,2],[-16,6],[-3,4],[1,6],[-2,1],[-1,0],[4,6],[5,2],[11,-1],[-2,2],[-5,3],[-2,3],[3,7],[5,3],[5,0],[4,-4],[3,-4]],[[7143,5038],[-8,-1],[-17,3],[-2,1],[-1,2],[0,3],[2,1],[23,2],[4,-2],[2,-7],[-3,-2]],[[7155,5049],[3,-2],[1,0],[9,1],[2,-1],[-3,-5],[-2,-2],[-3,2],[-2,-2],[-2,-3],[-2,-2],[-2,0],[-2,1],[-2,1],[-1,2],[-1,3],[0,3],[-1,2],[-1,3],[1,0],[1,0],[1,-1],[0,-1],[2,1],[2,1],[1,0],[1,-1]],[[8611,5039],[-2,-1],[-2,1],[-1,0],[-1,-1],[-2,-3],[0,-1],[-1,-1],[-11,0],[-6,2],[-3,3],[7,2],[1,1],[1,2],[2,2],[1,2],[1,1],[4,2],[4,2],[5,-1],[3,-3],[5,-3],[-3,-4],[-2,-2]],[[5592,5057],[0,-2],[-1,1],[-10,7],[-2,4],[1,0],[1,0],[1,-1],[1,-1],[5,-2],[2,-1],[1,-2],[1,-3]],[[5652,5069],[7,-4],[-1,-1],[-2,1],[-16,0],[-6,3],[-2,0],[-4,0],[-2,1],[0,3],[3,2],[13,-4],[10,-1]],[[7529,5080],[1,0],[2,0],[1,0],[2,-3],[1,-1],[-1,-2],[0,-2],[2,-1],[1,-1],[25,2],[2,-1],[3,-2],[8,-1],[2,-1],[2,-3],[4,-10],[0,-2],[-17,-2],[-29,8],[-4,2],[-3,3],[-1,1],[-10,2],[-2,1],[-2,3],[-10,8],[2,3],[8,2],[3,3],[1,0],[1,0],[3,-1],[2,0],[0,-2],[1,-1],[1,-1],[1,-1]],[[7943,5108],[20,-11],[7,-5],[7,-2],[2,-4],[4,-2],[3,-3],[2,-3],[-3,0],[-1,0],[0,-1],[5,-4],[7,-3],[-3,-2],[-3,-2],[3,-1],[3,-1],[3,1],[2,-2],[3,-5],[1,-5],[5,-1],[2,-5],[-1,-5],[-5,-4],[-5,-3],[-9,-3],[-9,1],[-14,2],[-3,1],[-4,5],[-2,1],[-2,-1],[-1,-3],[-2,-1],[-5,1],[-1,1],[1,3],[0,3],[0,3],[-7,2],[-3,6],[0,11],[-3,6],[-3,5],[-6,7],[2,1],[2,1],[2,-1],[2,-1],[0,1],[2,-1],[2,-1],[3,0],[1,2],[1,2],[0,3],[-1,1],[-2,1],[-8,-2],[-4,1],[-6,7],[-4,2],[-5,0],[-4,-3],[-2,0],[-7,1],[-2,0],[-1,2],[4,3],[9,-1],[3,2],[-12,2],[-6,3],[-4,5],[0,4],[3,0],[47,-11]],[[8446,5110],[-7,-1],[-10,7],[-8,9],[0,4],[14,-8],[9,-6],[2,-5]],[[6663,5138],[19,-3],[5,-2],[4,-5],[-2,-3],[-2,-4],[-2,-5],[1,-5],[-3,-1],[-1,-1],[-2,-3],[2,-2],[0,-2],[-3,-4],[3,-6],[1,-3],[1,-4],[-1,-4],[-1,-5],[-2,-3],[-5,-3],[-4,-5],[-5,-3],[-6,-9],[-3,-2],[-15,-5],[-3,-3],[-3,-2],[-2,-2],[-3,-6],[-2,-3],[-2,-11],[-22,-19],[-14,-20],[-18,-16],[-19,-14],[-7,-3],[-2,1],[-1,3],[-1,4],[-1,2],[-3,3],[-6,5],[-3,1],[-15,-3],[-29,-23],[-4,-1],[-1,4],[0,20],[-1,11],[-3,6],[-18,20],[-3,7],[0,9],[3,7],[22,19],[5,9],[1,6],[1,1],[1,1],[7,7],[2,3],[1,5],[1,5],[0,3],[-1,2],[0,2],[1,5],[1,9],[2,4],[4,5],[5,8],[3,3],[13,9],[1,0],[1,-1],[1,-1],[1,-1],[4,0],[2,-2],[2,-3],[2,-5],[6,-7],[7,-1],[7,2],[29,21],[5,2],[10,0],[21,-4],[5,2],[0,2],[0,2],[0,2],[1,1],[-1,0],[3,0],[3,1],[3,2],[2,3],[3,1],[5,-2],[3,0],[-1,0],[10,-5]],[[8398,5133],[-3,-5],[-4,7],[-7,6],[-9,13],[-7,9],[-5,7],[9,-2],[8,-6],[14,-19],[4,-10]],[[8252,5200],[1,-8],[-9,7],[-6,7],[-4,7],[8,-1],[10,-12]],[[8266,5202],[5,-10],[-8,7],[-8,13],[-7,11],[8,-4],[10,-17]],[[8276,5225],[3,-9],[-5,0],[-6,11],[-4,11],[-1,7],[6,-4],[6,-12],[1,-4]],[[5700,5254],[-3,-1],[-6,0],[-2,2],[-2,3],[-3,9],[4,-1],[8,-2],[4,-2],[0,-1],[1,-1],[0,-1],[1,0],[-2,-5]],[[7081,5265],[4,-9],[11,6],[10,3],[3,-1],[6,-3],[3,-1],[3,3],[1,0],[7,-1],[10,0],[6,-2],[3,-5],[-1,0],[-1,-1],[-1,-2],[2,-2],[3,-1],[5,-1],[2,1],[4,3],[2,0],[0,-1],[0,-3],[0,-1],[1,0],[2,-1],[2,-1],[1,-2],[-1,0],[-1,-1],[2,-2],[1,0],[-1,-1],[0,-1],[0,-1],[-1,0],[2,-5],[2,-3],[3,-3],[3,-3],[0,-2],[1,-4],[1,-2],[1,0],[2,-1],[2,-1],[-1,-4],[-6,-8],[-2,-5],[1,-4],[0,-1],[-9,-10],[-7,-4],[-6,-4],[-8,-2],[-7,-4],[-6,0],[-2,3],[-10,7],[0,4],[-2,1],[-3,1],[-2,2],[-2,3],[-2,4],[-3,2],[-4,3],[-5,5],[-7,12],[-3,3],[-13,15],[-5,3],[-1,1],[-3,4],[3,5],[-3,2],[1,3],[-2,4],[3,9],[4,6],[5,2],[3,-6]],[[5660,5280],[24,-7],[-2,2],[-1,1],[-1,2],[2,2],[2,0],[4,-2],[4,-4],[2,-2],[-2,-3],[-2,0],[-4,1],[-2,-1],[-4,-2],[-2,0],[-4,1],[-9,8],[-5,2],[0,2]],[[8704,5275],[-3,-12],[-7,13],[-2,16],[4,3],[10,-13],[-2,-7]],[[5612,5280],[-2,0],[-2,0],[-2,1],[-2,1],[-1,3],[0,2],[-1,2],[-1,1],[-2,0],[-2,1],[-1,1],[0,3],[1,0],[7,0],[2,-2],[6,-3],[4,-1],[7,-6],[-13,0],[2,-3]],[[8687,5240],[1,0],[1,2],[1,0],[1,-1],[1,-4],[0,-1],[0,-2],[1,-1],[1,0],[4,-5],[2,-1],[1,0],[-1,-3],[1,-4],[2,-3],[2,-5],[-4,2],[-8,9],[-2,1],[1,-2],[0,-2],[0,-1],[-2,0],[-1,0],[-3,4],[-4,2],[-2,3],[-4,7],[-1,1],[0,2],[-1,2],[-1,0],[-2,1],[0,3],[0,3],[-1,2],[0,1],[-1,0],[0,1],[-1,1],[-1,3],[-1,3],[-1,1],[-2,1],[2,2],[2,1],[0,1],[-1,1],[-1,0],[0,4],[-1,5],[-2,4],[-2,2],[-1,2],[-3,9],[-1,4],[1,2],[1,3],[2,2],[2,1],[2,0],[2,1],[2,2],[0,3],[2,0],[2,-2],[2,-3],[1,-3],[1,-12],[1,-5],[1,-4],[1,-5],[1,-4],[0,-5],[-2,-3],[2,-1],[1,-2],[3,-7],[-3,0],[-1,0],[-1,-1],[1,-2],[2,-1],[1,0],[1,-1],[1,-1],[1,-4],[0,-1],[1,-1],[1,-1]],[[7224,5307],[1,-1],[3,1],[1,1],[-2,1],[0,2],[47,-28],[-2,-1],[2,-4],[3,-3],[3,-1],[3,-1],[1,-1],[-8,-9],[3,-2],[9,0],[4,-2],[0,-2],[-1,0],[-2,0],[-4,-6],[-2,-1],[-3,2],[0,-2],[1,-3],[-1,-5],[-3,-4],[-3,-3],[-2,0],[-5,3],[-7,1],[-2,1],[-2,2],[0,2],[-3,1],[-6,0],[-3,-1],[-1,0],[-3,3],[-4,3],[-3,5],[-3,3],[-7,9],[-1,1],[-2,0],[-1,1],[-2,1],[-2,2],[-1,1],[-1,0],[0,1],[0,2],[0,1],[-1,1],[-1,1],[0,-1],[-3,3],[-10,5],[1,1],[0,1],[1,0],[-7,0],[-2,2],[1,1],[1,0],[1,0],[1,0],[-2,1],[-2,2],[0,1],[2,1],[10,0],[0,2],[-9,1],[-4,2],[0,4],[3,5],[3,3],[3,2],[4,0],[5,-4],[10,0],[2,-1],[4,-3]],[[7753,5316],[1,-4],[-1,-3],[-2,-1],[-1,2],[-2,4],[-3,0],[-3,-2],[-3,-3],[2,-1],[3,-1],[1,-1],[-7,-4],[-3,0],[-6,6],[-3,0],[-8,-2],[0,2],[32,10],[3,-2]],[[8682,5330],[0,-9],[-6,3],[-4,8],[1,12],[0,6],[14,-1],[5,-9],[-4,-3],[-6,-7]],[[5435,5358],[-1,-1],[-9,3],[-3,2],[-2,4],[11,0],[2,-2],[2,-3],[0,-3]],[[8639,5362],[3,-2],[6,5],[1,-3],[-2,-3],[-3,-6],[-1,-4],[5,2],[4,0],[2,-2],[-7,-9],[-3,-2],[-2,-2],[2,0],[4,-2],[2,-1],[3,-4],[2,-2],[1,1],[2,2],[1,3],[2,2],[2,-1],[1,-1],[3,-10],[-2,0],[1,-4],[1,-4],[-1,-3],[-3,-3],[-2,0],[-2,0],[-1,1],[-2,1],[-1,2],[-2,5],[-1,2],[-6,1],[-1,1],[0,2],[-1,1],[-5,3],[-5,6],[-2,2],[-3,-1],[-5,-3],[-2,-1],[-6,3],[-5,4],[-3,7],[2,5],[9,9],[3,-7],[2,-3],[2,-3],[0,2],[0,3],[-1,2],[-2,1],[1,3],[2,1],[1,-1],[2,-1],[0,8],[4,3],[5,0],[1,-5]],[[7179,5392],[-5,0],[-2,-1],[-2,-4],[-2,-4],[-8,-10],[-3,-2],[-2,0],[-4,2],[-15,3],[-4,3],[-1,0],[-2,2],[1,3],[2,4],[1,3],[3,1],[2,1],[5,1],[8,-2],[2,2],[0,2],[20,1],[6,-3],[0,-2]],[[6740,5435],[-5,-2],[-5,2],[-3,2],[0,4],[5,3],[6,1],[6,-2],[1,-4],[-2,-3],[-3,-1]],[[7663,5445],[0,-1],[0,-1],[0,-1],[-1,-2],[3,-2],[6,1],[3,-2],[-3,-2],[-3,-2],[-5,-1],[0,-1],[0,-1],[1,-1],[-2,-5],[-1,-1],[-2,2],[-1,1],[-2,-1],[-2,-1],[-2,0],[0,1],[1,0],[1,1],[0,1],[1,2],[-2,0],[-5,-2],[1,4],[0,1],[0,3],[2,0],[5,-2],[2,3],[0,3],[-1,3],[-2,2],[0,1],[8,0]],[[7300,5437],[-1,-6],[-5,4],[-5,2],[0,7],[5,3],[3,-1],[1,-5],[2,-4]],[[7284,5453],[6,-4],[-4,-4],[-1,-4],[-2,0],[-4,5],[-3,2],[0,3],[4,0],[4,2]],[[8605,5437],[-6,-3],[-6,12],[-2,9],[-1,6],[3,5],[9,-4],[-2,-5],[4,-3],[7,-3],[-6,-5],[7,-3],[-7,-6]],[[7635,5505],[2,-1],[0,-2],[0,-7],[-1,-3],[-2,-3],[-3,-3],[0,-4],[0,-3],[1,-2],[3,0],[-2,-3],[-3,-1],[-2,-2],[-1,-4],[1,0],[1,-1],[1,-1],[0,-2],[-6,0],[-4,1],[-1,1],[0,17],[-1,5],[-1,4],[2,3],[2,2],[1,0],[2,0],[2,-1],[2,0],[0,1],[-2,3],[-1,4],[0,2],[3,0],[0,1],[-9,3],[-2,1],[-1,4],[1,2],[4,3],[3,3],[5,2],[4,0],[3,-3],[0,-2],[0,-2],[0,-1],[0,-1],[-1,-3],[-1,-1],[-1,-2],[1,-3],[1,-1]],[[8565,5560],[3,-1],[2,0],[-2,-3],[-5,-8],[4,0],[1,0],[2,-2],[-1,-1],[0,-2],[1,-2],[2,2],[1,0],[1,0],[0,-4],[-2,-1],[-5,0],[-1,-1],[-2,-2],[-4,-2],[-8,-1],[-3,-2],[2,-2],[2,-1],[4,1],[-3,-5],[-6,-3],[-12,0],[3,-3],[2,-1],[6,0],[-2,-3],[-4,-1],[-6,1],[-11,-3],[-3,1],[-1,8],[4,8],[6,7],[6,3],[6,11],[4,3],[2,3],[2,9],[2,3],[2,0],[1,-4],[0,-3],[-3,-13],[2,-1],[1,3],[2,8],[2,3],[4,5],[2,2],[-1,-4],[1,-2]],[[8379,5705],[-3,-8],[-4,3],[-2,6],[5,6],[5,5],[3,-5],[-4,-7]],[[5873,5728],[1,-1],[2,0],[2,0],[3,2],[1,-2],[0,-2],[2,-2],[3,-1],[-2,-1],[-2,0],[-3,2],[-3,2],[-2,-1],[-2,-1],[-1,1],[-1,3],[1,2],[1,-1]],[[5949,5739],[-5,-1],[-3,3],[0,2],[0,2],[4,0],[3,1],[5,-4],[1,-2],[-2,-1],[-3,0]],[[8324,5804],[-6,-5],[-6,2],[-10,-3],[-7,-3],[-2,3],[-1,3],[3,3],[6,1],[8,2],[14,1],[5,-2],[-4,-2]],[[8887,5809],[4,-2],[1,1],[1,1],[0,-1],[0,-1],[1,-2],[0,-1],[2,0],[1,1],[2,2],[2,2],[2,0],[5,0],[1,0],[0,-2],[-1,0],[0,-1],[0,-1],[-4,-4],[4,-2],[7,2],[4,-2],[-2,-2],[-2,-1],[0,-2],[2,2],[2,0],[2,-2],[2,-2],[-1,-1],[0,-5],[-2,-1],[-3,0],[-2,-1],[1,-2],[0,-1],[0,-2],[-1,-2],[-1,-1],[-1,1],[-1,1],[-2,1],[-1,2],[-1,5],[-1,3],[-1,-2],[-3,-8],[-1,-2],[-3,1],[-1,2],[1,4],[2,3],[1,2],[-1,-1],[-2,-2],[-1,0],[-2,-1],[-2,1],[-1,0],[0,1],[1,3],[0,1],[-2,-1],[-8,-1],[1,2],[1,1],[0,1],[2,1],[0,2],[-2,1],[-2,3],[-1,3],[0,4],[1,2],[3,0]],[[8508,5793],[-5,-1],[-2,7],[0,10],[-1,8],[7,-1],[4,-3],[-2,-15],[-1,-5]],[[8331,5824],[11,0],[4,0],[-1,-4],[-9,-2],[-7,1],[-4,2],[6,3]],[[6529,5833],[0,-3],[-2,-1],[-5,1],[-7,0],[-2,-1],[-1,-1],[-2,-4],[-1,0],[-4,0],[-4,2],[-4,5],[-1,5],[4,3],[20,0],[5,-2],[2,-2],[2,-2]],[[6280,5821],[4,-1],[3,2],[2,2],[2,5],[1,1],[2,1],[5,0],[14,-7],[3,-5],[1,-3],[0,-1],[-1,-1],[0,-2],[1,-2],[1,-2],[0,-2],[-1,0],[-2,-2],[2,-4],[10,-15],[3,-5],[0,-3],[-3,-5],[-4,-3],[-16,-7],[-4,2],[-1,-1],[-2,-4],[6,-8],[4,-3],[7,-3],[10,-6],[6,-8],[2,-2],[1,-3],[-1,-2],[-1,-8],[-1,-3],[3,-6],[7,-15],[1,-2],[0,-3],[-1,-3],[0,-3],[2,-1],[8,0],[4,2],[3,5],[0,2],[2,8],[0,1],[2,1],[0,1],[0,2],[0,1],[-1,1],[1,5],[0,4],[2,4],[3,4],[6,13],[3,3],[1,3],[3,5],[1,2],[3,-1],[5,-3],[7,-3],[2,-3],[-1,-5],[0,-2],[0,-2],[1,0],[2,0],[1,-1],[3,-4],[2,-2],[7,-1],[9,-6],[7,-2],[1,-1],[0,-2],[-1,-1],[-1,0],[-1,-1],[-1,-1],[0,-1],[0,-1],[0,-1],[-5,-5],[-1,-1],[1,-4],[1,-2],[5,-2],[9,-10],[3,-2],[3,-1],[8,1],[11,-2],[-5,-3],[2,-3],[2,-1],[5,0],[7,4],[6,-4],[20,0],[9,-4],[4,-1],[4,-2],[2,-4],[2,-6],[3,-4],[-1,-10],[4,-6],[10,-10],[1,-2],[0,-2],[0,-1],[12,-3],[6,-4],[4,0],[2,-1],[1,-1],[2,-3],[2,-2],[1,-1],[7,-2],[2,-1],[2,-3],[6,-12],[3,-3],[3,-3],[3,-1],[10,0],[3,-2],[6,-6],[1,-1],[1,1],[7,2],[5,2],[3,1],[4,0],[3,-2],[3,-3],[4,-5],[1,-1],[5,0],[2,-1],[-2,-5],[0,-2],[11,0],[3,-1],[2,-3],[1,-1],[1,-3],[0,-2],[24,-28],[3,0],[3,-2],[3,-3],[1,-4],[0,-1],[1,-1],[0,-2],[0,-2],[0,-2],[-2,-2],[0,-2],[0,-3],[0,-4],[2,-8],[-1,-2],[1,-4],[-1,-2],[-1,-3],[-1,-3],[1,-4],[1,-4],[5,-9],[3,-4],[3,-2],[5,-2],[1,-3],[3,-6],[-2,-7],[-1,-1],[1,-4],[-3,-3],[-9,-3],[-7,-4],[-10,-2],[-3,-2],[-3,-2],[-2,-3],[-1,-1],[-1,1],[0,3],[-2,-1],[-1,-3],[-1,-1],[1,-3],[-1,-2],[-1,-2],[-1,-2],[1,-3],[2,-1],[11,2],[11,0],[15,6],[7,2],[9,-1],[5,1],[3,5],[0,1],[-2,5],[2,2],[2,2],[2,1],[2,-1],[-2,-3],[-1,-1],[4,-2],[8,2],[6,-6],[22,-7],[9,-8],[5,0],[3,4],[-2,5],[-4,4],[-3,5],[4,4],[1,2],[1,2],[-2,5],[0,2],[7,-2],[7,-5],[7,-7],[5,-8],[1,-2],[1,-2],[0,-2],[10,-2],[8,-5],[2,-4],[-2,-3],[-1,-6],[6,-4],[-4,-3],[-5,-1],[-5,0],[-9,5],[3,-6],[3,-3],[30,-8],[12,-10],[2,0],[4,2],[2,0],[1,-2],[1,-3],[2,-5],[-1,-2],[0,-3],[-2,0],[-4,0],[-6,-3],[-21,-4],[-1,-1],[-1,-2],[-2,-5],[-1,-2],[-1,-2],[-1,0],[-3,-4],[-3,-4],[-1,-1],[-4,-2],[-11,-10],[-5,-3],[-3,-4],[-2,-1],[-1,-1],[0,-1],[-2,-2],[0,-1],[-1,0],[-1,0],[-5,0],[-2,-1],[-1,-2],[0,-3],[-1,-3],[0,-3],[1,-1],[-6,-6],[-2,-1],[-2,0],[-12,3],[-11,9],[-26,12],[-5,6],[-4,1],[-12,-1],[-3,1],[-2,4],[-2,4],[-3,4],[-4,3],[-4,1],[-4,-1],[-6,-2],[-1,1],[-2,1],[-3,3],[-6,2],[-6,5],[-3,2],[-19,-1],[-3,-2],[1,-4],[-4,5],[-3,-6],[-4,0],[-14,6],[-1,1],[-1,2],[0,4],[0,2],[-4,3],[-2,2],[0,4],[2,6],[1,3],[2,1],[3,1],[5,3],[3,1],[2,0],[0,1],[1,1],[1,3],[1,3],[1,0],[0,3],[-1,3],[0,2],[1,2],[-2,4],[-2,3],[-10,6],[-2,1],[-1,1],[-3,3],[0,1],[-24,0],[-11,3],[-3,0],[-10,-2],[-4,-3],[-4,0],[-4,-3],[-3,1],[-5,5],[0,1],[1,2],[0,1],[-2,1],[0,2],[3,3],[4,7],[4,3],[7,13],[1,3],[1,2],[0,3],[-1,2],[-4,7],[-1,1],[-8,0],[-4,-1],[-3,-3],[-2,-5],[-2,-4],[-2,-3],[-3,1],[-5,3],[-3,0],[-4,-1],[-4,-3],[-2,-2],[-2,-1],[-1,0],[-1,0],[-2,2],[-1,1],[-1,0],[-6,-1],[-3,-2],[-3,-4],[-2,-4],[-1,-6],[-1,-2],[-2,-1],[-2,0],[-2,-2],[-4,-4],[1,-4],[1,-3],[6,-8],[0,-1],[0,-1],[1,-1],[0,-1],[0,-2],[0,-1],[-1,-2],[-1,-5],[-2,-3],[0,-2],[-1,-1],[1,-3],[4,-6],[1,-3],[-1,0],[-1,0],[0,-1],[0,-2],[-1,-7],[-1,-2],[-2,-1],[-2,0],[-3,3],[-3,1],[-4,-1],[-3,-3],[-13,-12],[-1,-2],[-2,-4],[-1,-1],[-1,-1],[-5,-2],[-2,-1],[-6,-12],[-3,-3],[-4,0],[-7,5],[-5,-2],[-4,2],[-1,0],[-1,-1],[-1,-1],[0,-1],[-1,-3],[-1,-2],[-5,-4],[-4,-5],[-2,-1],[0,-7],[-5,-11],[0,-5],[-4,-16],[-1,-3],[-3,-2],[-8,-14],[-3,-2],[-7,-2],[-5,-5],[-4,-2],[-3,-3],[-19,-15],[-16,-9],[-20,-8],[-21,-1],[-6,2],[-9,11],[0,3],[-2,6],[-3,8],[-1,2],[0,4],[0,1],[0,5],[-1,12],[0,3],[0,4],[0,11],[0,5],[1,4],[2,6],[2,5],[0,5],[-1,3],[-1,5],[-1,4],[1,2],[1,1],[2,5],[1,2],[0,3],[-1,4],[-1,3],[-1,5],[-4,2],[-3,1],[-3,3],[-1,4],[0,4],[0,5],[0,3],[0,2],[-6,-10],[-3,-5],[-3,-2],[-15,-1],[-30,-12],[-8,-1],[-23,6],[-16,-2],[-3,-1],[-3,-4],[-6,-10],[-7,-6],[-8,-3],[-22,-2],[-7,2],[-5,5],[-5,9],[-1,9],[2,9],[3,8],[26,40],[6,8],[34,16],[4,5],[8,2],[8,7],[8,5],[11,1],[4,2],[3,4],[1,5],[0,5],[-2,5],[-1,4],[-3,3],[-7,6],[-2,4],[0,2],[0,2],[1,2],[1,2],[-3,1],[-2,2],[-1,3],[-2,4],[1,3],[-1,3],[-1,3],[0,2],[0,5],[1,2],[1,2],[2,4],[-4,3],[-3,4],[1,4],[1,5],[0,17],[0,4],[-4,8],[4,6],[4,7],[12,37],[1,2],[1,1],[3,0],[2,1],[1,2],[-2,2],[-1,1],[-1,2],[3,2],[-1,3],[-3,5],[3,2],[2,2],[2,3],[-1,5],[-1,3],[-5,5],[-1,3],[0,4],[1,5],[1,5],[1,3],[2,3],[2,3],[2,3],[1,5],[0,16],[0,5],[-1,6],[-1,3],[0,4],[-2,4],[0,4],[1,4],[5,10],[3,3],[-2,2],[-3,2],[-2,2],[0,3],[1,3],[1,3],[1,2],[1,4],[2,17],[2,11],[2,6],[4,5],[1,5],[-1,9],[5,11],[5,13],[24,24],[14,11],[7,3],[7,0],[2,-10],[-3,-5],[-2,-5],[2,-5],[0,-5]],[[8376,5869],[5,-5],[-1,-6],[-4,0],[-3,5],[-8,3],[-3,-3],[-2,-3],[-3,5],[-2,4],[3,2],[8,1],[10,-3]],[[6334,5872],[2,0],[5,2],[3,-1],[3,-5],[5,-6],[-1,-1],[-1,-1],[-1,-1],[-1,-2],[7,-8],[2,-3],[2,-2],[3,-2],[1,-2],[0,-3],[4,-5],[1,-2],[-1,-3],[1,-3],[1,-4],[5,-11],[1,-3],[1,-3],[1,-3],[1,-2],[1,-1],[2,-1],[1,-2],[0,-1],[-1,-2],[0,-1],[1,-2],[0,-1],[1,-3],[1,-2],[-2,-3],[-7,-7],[-3,-7],[-1,-1],[-3,-2],[-4,1],[-6,2],[-4,5],[-2,8],[0,1],[0,1],[0,5],[-1,3],[-1,2],[-6,5],[-14,6],[-1,2],[-2,4],[-2,3],[-2,2],[-1,5],[-2,14],[-2,4],[2,6],[-1,6],[-2,6],[-2,4],[0,2],[1,3],[0,2],[-1,3],[3,5],[3,3],[3,2],[5,0],[1,-1],[1,0],[0,-2],[2,-1],[0,-1],[1,0]],[[6332,5882],[-3,0],[-4,0],[-1,2],[1,4],[2,1],[6,1],[2,-2],[0,-2],[1,-2],[-4,-2]],[[6499,5882],[0,-1],[-1,-1],[-1,1],[-1,0],[-1,-1],[0,-1],[-1,0],[-2,1],[-1,2],[2,1],[0,1],[0,2],[1,0],[1,2],[2,2],[0,-2],[0,-1],[1,-3],[1,-2]],[[6490,5899],[3,-2],[1,0],[0,1],[1,0],[2,-2],[0,-1],[0,-1],[-2,-1],[0,-1],[-1,-1],[-1,-1],[-1,0],[-1,-1],[-1,-1],[-1,-1],[0,1],[0,2],[-1,2],[-1,2],[1,3],[0,2],[2,0]],[[6410,5898],[18,-7],[5,0],[2,-1],[1,-2],[1,-1],[2,-2],[2,0],[2,1],[1,3],[2,2],[2,-2],[1,-3],[0,-3],[0,-3],[0,-2],[2,-1],[4,-2],[13,-11],[6,-2],[4,-2],[3,-4],[3,-4],[1,-11],[-4,-7],[-10,-7],[2,-2],[7,-3],[0,-1],[1,-1],[0,-1],[1,0],[3,0],[2,-1],[3,-5],[8,-2],[3,-2],[1,-1],[0,-2],[-1,-3],[0,-2],[1,0],[2,0],[2,3],[2,0],[21,0],[6,-1],[3,-4],[-3,-2],[-2,-2],[-1,-3],[2,-3],[-1,-2],[-1,-1],[-3,-2],[1,0],[-2,-2],[-4,-3],[-4,-1],[-2,0],[-2,4],[-3,2],[-2,2],[-3,1],[-14,0],[-3,2],[1,3],[4,7],[-4,-1],[-4,-3],[-1,-4],[4,-4],[0,-1],[-4,1],[-7,3],[-4,1],[-6,-5],[-3,0],[-1,6],[1,1],[1,1],[2,0],[1,0],[1,2],[0,2],[-1,1],[0,2],[2,3],[4,3],[7,2],[0,2],[-4,2],[-12,-2],[5,5],[1,1],[-1,3],[-2,0],[-8,-3],[-10,-10],[-2,0],[-1,1],[-2,2],[-2,2],[-2,0],[-3,-1],[-6,1],[-2,1],[0,3],[1,3],[1,1],[1,2],[1,3],[-1,5],[-1,5],[-1,5],[1,7],[-2,3],[-2,1],[-3,2],[-1,1],[1,3],[0,3],[-1,3],[-1,1],[-4,1],[-6,6],[-2,1],[-7,-1],[-3,2],[-3,4],[-3,5],[-3,6],[1,0],[-1,2],[1,1],[-1,0],[1,4],[0,3],[-1,3],[0,2],[2,2],[3,0],[3,-1],[7,-6]],[[6546,5944],[-2,-1],[-1,-1],[-1,-2],[2,-2],[6,2],[2,-1],[2,-5],[2,-1],[14,5],[3,-4],[-4,-3],[-2,-3],[-4,-8],[-2,-2],[-4,-1],[-4,0],[-4,3],[-3,4],[-5,4],[-8,2],[-4,4],[2,10],[-2,0],[-1,1],[1,2],[0,2],[2,4],[1,0],[4,0],[0,-1],[3,-5],[2,-1],[4,-1],[1,-1]],[[6231,5976],[-4,0],[-2,3],[0,3],[3,-1],[5,0],[1,-2],[0,-2],[-3,-1]],[[3746,6079],[-4,-3],[-3,1],[6,8],[2,5],[1,7],[1,7],[3,7],[3,4],[2,-5],[0,-5],[-3,-14],[-1,-4],[-3,-3],[-4,-5]],[[3734,6100],[4,-1],[3,1],[-4,-5],[0,-1],[-1,-1],[-2,0],[0,-1],[0,-1],[0,-1],[-2,-2],[-2,-1],[-2,2],[0,5],[-5,11],[0,2],[0,4],[-1,5],[-1,4],[2,-3],[2,-3],[2,-3],[0,-3],[1,-1],[6,-7]],[[8891,6156],[4,-5],[-4,-2],[-20,-15],[-4,-4],[-3,-6],[-2,-2],[-6,2],[-2,-2],[-1,-2],[1,-3],[-1,-2],[-3,0],[-4,1],[-10,10],[9,-1],[1,1],[1,2],[-1,6],[1,2],[1,0],[6,0],[2,2],[2,2],[1,4],[1,1],[6,0],[2,1],[-1,3],[-2,1],[-1,3],[1,1],[2,0],[4,0],[2,1],[3,2],[2,1],[11,0],[2,-2]],[[3803,6131],[-15,0],[-3,1],[-2,4],[-6,15],[-1,5],[0,4],[2,2],[3,-3],[3,-3],[16,-16],[8,-2],[3,-2],[-3,-4],[-5,-1]],[[8758,6193],[-1,-7],[5,0],[2,1],[2,2],[2,3],[1,1],[2,0],[10,-7],[3,-4],[-6,-5],[-6,-1],[-35,2],[-7,3],[1,1],[5,4],[3,2],[8,7],[2,1],[4,1],[3,0],[2,-4]],[[3746,6209],[-1,-4],[-3,-5],[-1,-3],[2,-2],[1,-4],[0,-4],[-2,-1],[-3,1],[-7,10],[0,1],[1,1],[0,1],[0,1],[-4,0],[-5,2],[-5,4],[-2,4],[1,2],[1,0],[1,-2],[3,-1],[6,-4],[0,1],[0,1],[0,1],[1,2],[-5,3],[3,5],[9,2],[2,5],[-2,1],[3,0],[3,2],[2,-1],[2,-4],[0,-4],[0,-7],[-1,-4]],[[3689,6245],[4,-1],[1,1],[1,1],[1,1],[2,0],[2,-1],[1,-1],[0,-2],[0,-3],[-1,-2],[-1,-3],[1,-2],[0,-3],[-2,-1],[-8,-5],[-4,1],[-3,4],[-4,6],[-2,5],[2,0],[1,2],[0,2],[-2,3],[4,2],[4,-1],[3,-3]],[[8735,6234],[-22,-1],[-4,2],[-2,4],[1,6],[0,3],[4,9],[1,3],[1,4],[-1,2],[1,2],[2,1],[2,-2],[1,-2],[4,-8],[12,-12],[4,-8],[-4,-3]],[[3742,6246],[1,-2],[-2,-3],[-2,-1],[-2,2],[-1,2],[-1,-1],[-7,0],[-2,-2],[0,-2],[1,-1],[2,-2],[1,-2],[0,-1],[-1,-1],[-2,-5],[-3,-3],[-2,0],[-1,4],[-1,4],[-1,2],[-1,2],[0,3],[0,5],[2,1],[3,1],[2,1],[-3,3],[-2,2],[-1,3],[0,4],[0,3],[0,5],[1,5],[1,2],[2,0],[8,-2],[2,-2],[2,-3],[3,-12],[1,-2],[1,-1],[1,-3],[1,-3]],[[4940,6287],[0,-8],[0,-4],[-1,-2],[-2,-1],[-8,-7],[-9,-3],[-3,1],[-1,4],[0,3],[1,2],[1,3],[-1,2],[-2,1],[3,1],[8,0],[3,3],[2,3],[3,3],[3,1],[3,-2]],[[3613,6307],[3,-6],[-1,1],[0,-1],[0,-1],[-10,-4],[-5,1],[-3,4],[10,5],[6,1]],[[3039,6325],[6,-1],[13,2],[5,-3],[-5,0],[-2,0],[-2,-2],[-26,2],[-2,1],[-3,2],[-2,0],[0,2],[5,2],[5,0],[8,-5]],[[3136,6337],[4,-4],[-2,-2],[-3,-1],[-27,-3],[-8,3],[2,2],[3,0],[15,-2],[3,1],[-1,1],[-2,1],[-1,0],[-1,0],[0,2],[12,2],[6,0]],[[3391,6319],[-5,-2],[-2,3],[3,3],[14,8],[11,10],[4,2],[0,-1],[0,-1],[-1,-1],[-2,-3],[-3,-8],[-2,-3],[-8,-1],[-9,-6]],[[3162,6333],[-8,-1],[-3,1],[1,4],[15,2],[7,1],[7,3],[8,2],[6,-1],[-6,-4],[-6,-1],[-3,-4],[-5,-2],[-13,0]],[[3726,6328],[-3,-4],[-3,-1],[-8,2],[-4,-1],[-2,0],[-2,2],[3,2],[0,2],[0,2],[0,3],[2,3],[1,2],[4,3],[8,4],[4,0],[2,-6],[-1,-7],[-1,-6]],[[3602,6348],[13,-7],[-2,4],[-1,1],[2,0],[5,-1],[1,0],[1,2],[0,1],[2,0],[5,-3],[4,-3],[2,-1],[0,-1],[-3,-7],[3,-1],[1,-4],[-1,-4],[-3,-3],[-4,1],[-22,14],[-5,6],[-2,9],[4,-3]],[[4756,6369],[0,-7],[-1,-3],[0,-3],[1,0],[4,-3],[-3,-4],[-3,-1],[-3,2],[-3,4],[0,2],[-1,3],[-1,1],[-3,4],[-1,2],[4,-1],[7,6],[3,-2]],[[3583,6364],[2,-5],[1,-5],[-1,-5],[-3,-1],[-5,2],[-16,13],[-2,3],[2,-1],[2,-1],[3,-1],[1,1],[0,3],[-1,3],[-1,2],[-2,1],[1,2],[2,1],[4,0],[3,-2],[1,-1],[0,-3],[1,-3],[2,0],[3,-1],[3,-2]],[[3499,6366],[-3,0],[-6,0],[-3,-1],[-3,-2],[-3,-1],[-3,1],[5,5],[6,4],[6,3],[6,1],[5,2],[5,5],[5,3],[6,-2],[0,-1],[-1,-1],[-4,-8],[-5,-3],[-6,0],[-7,-5]],[[3195,6387],[-3,-3],[-4,2],[-6,0],[-4,1],[1,4],[2,3],[3,1],[3,-2],[10,0],[3,-3],[-5,-3]],[[7549,6371],[2,-3],[11,1],[4,-1],[4,-2],[19,-6],[7,-5],[3,-1],[3,-2],[2,-1],[1,1],[3,3],[2,1],[2,0],[7,-2],[6,2],[9,-1],[9,-4],[3,-6],[-2,-7],[0,-2],[0,-2],[1,-3],[1,-2],[2,-1],[0,-1],[0,-3],[-1,-5],[1,-2],[3,-2],[2,-2],[-1,-3],[-1,-5],[-2,-5],[-2,-2],[-56,1],[-57,0],[-16,7],[-6,5],[-4,7],[-13,22],[-2,4],[-1,5],[0,10],[1,5],[1,4],[3,2],[9,3],[20,-4],[2,0],[1,1],[2,3],[-1,1],[1,2],[0,2],[3,2],[0,1],[2,2],[3,10],[3,3],[3,-2],[7,-6],[3,-2],[1,-1],[0,-2],[-1,-3],[-3,-2],[-1,-2],[-2,-2],[0,-3]],[[3555,6409],[-2,-4],[-2,0],[-3,0],[-2,-1],[-2,-3],[0,-2],[0,-2],[-2,-3],[-10,-7],[-5,-1],[-3,5],[8,5],[16,15],[8,1],[0,-1],[-1,-2]],[[8457,6410],[-2,-4],[-1,-5],[1,-6],[-1,-4],[-5,0],[-30,6],[-4,0],[-2,1],[-2,2],[-1,3],[1,1],[1,1],[2,1],[4,-1],[2,-2],[3,1],[3,4],[3,3],[4,1],[13,-3],[2,0],[1,2],[1,2],[1,3],[0,1],[8,2],[4,0],[1,-3],[-1,-2],[-3,-1],[0,-2],[-3,-1]],[[6180,6337],[-1,-2],[-3,-20],[-6,-13],[-2,-2],[-3,-2],[-7,-9],[-3,-2],[-5,0],[-4,1],[-2,2],[-2,4],[-1,2],[-4,3],[-9,3],[-4,4],[-2,4],[-4,9],[0,3],[-1,3],[-1,5],[-1,2],[0,1],[0,1],[0,1],[1,2],[1,0],[1,-1],[1,0],[1,1],[1,3],[1,1],[1,1],[1,0],[1,1],[1,1],[1,0],[0,2],[1,4],[-1,8],[-3,2],[-8,3],[-3,2],[-2,3],[0,5],[2,5],[1,2],[6,4],[1,1],[0,4],[-2,5],[2,5],[8,8],[3,5],[3,5],[3,4],[3,4],[4,2],[3,0],[9,-5],[9,-6],[8,-11],[1,-1],[1,-2],[0,-3],[-3,-6],[0,-4],[0,-13],[1,-4],[2,-4],[1,-5],[1,-5],[2,-16],[0,-5]],[[3318,6407],[-6,-1],[-5,2],[-1,6],[1,3],[2,1],[4,3],[0,1],[0,2],[0,1],[1,1],[6,0],[15,2],[3,-2],[-1,-2],[-1,0],[-4,0],[-5,-2],[-2,-1],[-2,-4],[-1,-5],[-1,-3],[-3,-2]],[[7399,6428],[12,-7],[40,-12],[3,-3],[2,-5],[2,-4],[1,-1],[3,-2],[1,0],[1,-1],[2,-6],[0,-2],[0,-3],[0,-3],[-3,-3],[-1,-3],[-1,-7],[-2,-6],[-8,-13],[-2,-6],[1,-4],[2,-3],[2,-14],[2,-5],[3,-3],[3,-4],[-2,-10],[-1,-11],[1,-10],[2,-10],[1,-2],[0,-1],[1,-1],[0,-3],[-1,-2],[-2,-6],[-1,-8],[-2,-4],[-7,-12],[-5,-6],[-2,-4],[-3,-3],[-8,-2],[-3,-2],[0,-1],[0,-2],[0,-1],[-1,-1],[0,-1],[-2,0],[-1,-1],[-1,-1],[-2,-5],[0,-1],[-14,1],[-4,-2],[-1,-2],[-1,-5],[-2,-2],[-2,-1],[-10,-1],[-23,-12],[-3,0],[-19,1],[-7,-1],[-11,3],[-3,-1],[-7,-5],[-4,-1],[-19,2],[-7,-4],[-4,-1],[-39,4],[-5,3],[-1,2],[-3,3],[-2,2],[-4,9],[-6,20],[-2,5],[-2,2],[-3,2],[-3,2],[1,5],[1,4],[-1,4],[-2,4],[3,2],[1,1],[0,2],[0,2],[-2,7],[-2,3],[-6,23],[-1,5],[1,4],[2,4],[6,23],[4,8],[26,47],[8,11],[3,8],[1,2],[1,2],[2,2],[13,18],[5,5],[44,17],[3,0],[2,-1],[0,-3],[3,-3],[5,-2],[11,0],[4,2],[7,6],[9,3],[35,-7]],[[7013,6395],[-4,-3],[-6,3],[-6,5],[-3,6],[0,1],[-1,5],[0,5],[0,5],[1,7],[0,1],[0,2],[3,3],[1,1],[23,2],[1,-1],[1,-1],[1,-4],[1,-2],[8,-5],[3,-4],[-2,-8],[-5,-5],[-11,-7],[-5,-6]],[[4628,6422],[-3,-3],[-5,3],[-8,8],[4,1],[5,3],[4,4],[0,6],[1,0],[1,0],[1,-1],[0,-1],[2,-2],[0,-2],[-1,-2],[-1,-2],[1,-9],[-1,-3]],[[6662,6440],[-1,-2],[1,-1],[2,0],[9,1],[2,-1],[0,-1],[0,-1],[-5,-6],[-3,-2],[-1,-1],[-8,7],[-12,2],[-10,5],[2,4],[8,-4],[4,0],[-1,1],[-1,2],[-1,1],[0,1],[8,0],[4,-1],[3,-4]],[[7560,6412],[-2,0],[-3,0],[-4,5],[-2,6],[2,5],[5,1],[0,1],[-10,6],[-4,3],[-4,7],[-2,5],[-1,3],[0,5],[1,3],[1,2],[2,1],[5,0],[2,-1],[6,-5],[9,-10],[6,-10],[3,-4],[1,-3],[-2,-4],[-1,-2],[-2,-2],[-2,-2],[-1,-8],[-3,-2]],[[3428,6470],[-2,0],[-1,0],[-20,-4],[-20,1],[-1,1],[1,3],[6,4],[22,10],[4,0],[5,-4],[4,-5],[2,-6]],[[3437,6490],[-1,-2],[0,-1],[-1,0],[0,-2],[14,0],[-4,-5],[-7,-2],[-8,1],[-5,3],[-1,1],[-2,3],[-1,3],[2,1],[7,-1],[3,1],[3,2],[1,-2]],[[4126,6450],[-14,-2],[-7,4],[-6,2],[-4,4],[-6,2],[-5,3],[-11,12],[-5,7],[-2,3],[-3,0],[11,7],[12,2],[23,-2],[4,-3],[8,-3],[6,-7],[4,-2],[1,-1],[1,-1],[2,-4],[3,-11],[-4,-7],[-8,-3]],[[3082,6496],[-4,-3],[-19,3],[1,2],[-1,2],[1,1],[2,1],[12,-2],[8,-4]],[[7061,6504],[2,-1],[9,1],[3,-2],[0,-2],[0,-1],[0,-1],[0,-2],[-1,-1],[0,-2],[0,-3],[-1,-3],[-3,-3],[-3,0],[-14,4],[-1,-2],[-1,-3],[-2,-3],[-2,-1],[-1,-1],[2,-1],[8,0],[2,-3],[-1,-3],[-3,-2],[-10,-8],[-5,-1],[-6,0],[-5,2],[1,3],[1,0],[-3,2],[-9,0],[1,1],[1,3],[1,2],[-1,2],[2,3],[1,0],[2,-1],[2,0],[2,1],[3,6],[2,1],[9,0],[5,3],[4,5],[-3,4],[-3,1],[-3,1],[-4,1],[1,2],[-14,8],[0,1],[1,0],[2,1],[3,1],[19,-2],[3,-1],[7,-6]],[[7522,6488],[-3,-2],[-28,2],[-4,1],[-3,3],[-1,2],[-4,9],[7,9],[4,3],[13,-3],[4,-4],[0,-6],[0,-2],[2,-2],[6,-2],[1,-1],[6,-7]],[[3050,6504],[-3,-2],[-4,2],[-7,10],[-3,2],[2,2],[1,0],[2,-1],[0,-2],[9,-7],[3,-4]],[[7478,6486],[4,-5],[5,-6],[4,-3],[1,-3],[1,-3],[-1,-3],[-2,-2],[-2,-1],[-6,1],[-2,-1],[-2,-3],[1,-3],[1,-1],[5,1],[5,0],[1,-1],[0,-3],[-2,-2],[-2,-1],[-3,-2],[-2,0],[0,-1],[2,0],[2,-1],[3,-2],[0,-4],[-1,-2],[-4,1],[-1,-1],[3,-1],[1,-1],[1,-3],[-3,0],[-1,0],[-2,2],[-1,2],[-1,1],[-10,-2],[-3,1],[-1,2],[0,5],[-1,4],[-2,3],[-25,13],[-14,12],[-4,5],[2,7],[0,4],[0,3],[0,7],[1,4],[6,9],[1,2],[2,6],[1,1],[2,2],[2,3],[3,1],[2,-4],[2,-1],[4,-3],[17,-3],[3,-2],[2,-7],[1,-7],[2,-5],[1,-5],[4,-4]],[[8214,6547],[8,-2],[14,0],[22,-7],[35,-12],[4,-3],[0,-2],[-29,3],[-30,13],[-29,4],[-2,3],[0,3],[2,2],[1,1],[2,-1],[2,-2]],[[4438,6542],[2,0],[3,1],[3,-1],[1,-4],[0,-4],[-1,-6],[0,-2],[1,-2],[0,-3],[2,-6],[-2,-5],[-1,-2],[-1,-2],[-3,0],[-10,4],[2,-2],[1,-2],[0,-2],[-2,-7],[0,-3],[-1,-2],[-2,-2],[-2,0],[-5,5],[-2,1],[-2,1],[-6,4],[-3,1],[-4,-2],[-1,0],[-1,0],[-3,3],[-1,1],[-2,2],[-1,0],[-1,1],[-5,-1],[-2,1],[-5,4],[-4,1],[-7,6],[4,0],[6,3],[6,4],[5,6],[10,5],[1,1],[1,1],[0,2],[0,2],[0,2],[0,1],[4,6],[5,0],[23,-9]],[[3015,6563],[-3,-1],[-4,1],[-4,2],[-3,3],[2,1],[1,0],[1,0],[10,-1],[3,-2],[-3,-3]],[[4621,6575],[4,-4],[-7,-3],[2,-4],[-1,-3],[-1,-2],[-3,-1],[1,-3],[2,-4],[0,-2],[1,-3],[-1,0],[-1,1],[-2,-1],[-1,-2],[0,-2],[0,-2],[0,-2],[-2,-1],[-6,2],[-2,-1],[-3,-4],[-1,-4],[-2,-4],[-4,-1],[-3,1],[-3,2],[-5,8],[5,4],[-1,3],[-2,2],[-1,0],[-3,0],[-4,-3],[-2,-2],[1,-2],[-3,-4],[-2,0],[-3,5],[-1,6],[0,18],[1,11],[1,1],[2,0],[1,1],[1,5],[-1,4],[-5,7],[6,4],[1,2],[1,5],[3,1],[15,-3],[5,-3],[4,-4],[0,-5],[2,-4],[3,-2],[6,-7],[8,-1]],[[4638,6581],[-2,-2],[-1,0],[-15,7],[-4,3],[-3,4],[-2,6],[-1,7],[1,4],[1,4],[2,3],[1,2],[4,2],[1,2],[2,4],[0,1],[1,1],[1,-3],[1,-1],[3,-3],[2,-1],[3,-2],[2,-2],[-1,-3],[1,-3],[3,-5],[1,-2],[1,-3],[0,-3],[-1,-14],[-1,-3]],[[5742,6623],[2,-1],[2,2],[2,2],[2,2],[3,-1],[-1,-1],[-1,-1],[0,-1],[-2,0],[0,-2],[2,0],[1,-1],[1,-1],[0,-1],[1,-4],[1,-6],[1,-2],[-2,-1],[-1,0],[-2,2],[1,4],[-6,-2],[-2,0],[-3,3],[-1,4],[-1,4],[0,4],[1,4],[1,-2],[0,-2],[1,-2]],[[4451,6615],[-3,-1],[-2,3],[3,5],[7,3],[1,5],[-1,3],[-9,1],[-3,3],[4,7],[7,3],[7,-1],[4,-4],[0,-1],[0,-1],[0,-1],[3,-3],[-1,-5],[-2,-6],[-3,-4],[-1,-2],[-3,-1],[-4,-1],[-2,-1],[-2,-1]],[[5704,6681],[4,-5],[1,-3],[2,-4],[1,-3],[2,-2],[1,-1],[2,-2],[1,-4],[-2,0],[-1,-1],[-1,-2],[0,-2],[1,1],[1,1],[1,-1],[1,-1],[0,-2],[0,-1],[-1,-1],[-2,-3],[-2,-1],[-4,0],[-4,-2],[-2,0],[-1,1],[-1,2],[0,3],[-1,2],[-2,2],[1,1],[1,1],[0,1],[-3,0],[-1,-2],[-2,-3],[-2,-1],[0,1],[1,3],[2,3],[2,1],[0,2],[-4,-1],[-1,1],[-1,3],[1,1],[4,5],[-3,0],[0,2],[2,4],[1,4],[-1,2],[-3,-4],[-5,-9],[-2,-4],[-3,-3],[0,13],[1,4],[1,3],[3,2],[8,3],[5,-1],[4,-3]],[[7076,6688],[5,-1],[11,0],[4,-4],[4,-10],[3,-6],[0,-3],[-2,0],[-3,-2],[-4,-5],[-3,-6],[-1,-5],[-1,-3],[-2,-1],[-2,-1],[-1,2],[-1,3],[-3,1],[-2,1],[-2,1],[-2,-3],[0,-3],[0,-8],[-1,-2],[-7,-5],[-3,-3],[2,-7],[-6,-8],[-13,-12],[0,-3],[0,-5],[0,-3],[-2,-2],[-7,-3],[-1,-2],[-1,-2],[0,-2],[-1,-2],[-2,-3],[-3,-1],[-7,-1],[-4,-3],[-13,-2],[-7,-2],[-2,-2],[-6,-6],[-1,-1],[-5,0],[-2,1],[-7,4],[-4,1],[-1,1],[-1,3],[0,3],[1,2],[0,3],[-1,1],[-1,2],[-1,2],[6,3],[5,7],[9,22],[2,4],[2,2],[3,2],[2,0],[5,0],[2,0],[4,2],[3,1],[7,-1],[2,2],[4,5],[1,1],[4,2],[2,1],[1,3],[0,3],[-1,4],[1,3],[2,3],[8,10],[4,4],[8,-3],[4,1],[-3,4],[-7,6],[-2,5],[1,3],[1,1],[4,1],[2,1],[6,5],[2,1],[7,1],[6,3],[2,0]],[[6891,6682],[-5,-1],[-5,4],[-3,7],[1,5],[6,-7],[2,-2],[6,-2],[-2,-4]],[[6707,6697],[8,-4],[11,1],[5,-2],[1,-9],[1,-6],[-9,0],[-12,2],[-3,3],[1,4],[-10,1],[1,-5],[-7,-5],[-6,4],[-2,8],[1,5],[6,4],[14,-1]],[[5749,6689],[-1,-2],[1,0],[2,0],[1,-1],[1,-3],[0,-3],[-1,-2],[-2,-2],[-4,-2],[-2,-2],[-1,-3],[-1,-4],[0,-4],[-3,-4],[-3,-2],[-1,2],[-1,0],[-1,-6],[-3,2],[-6,7],[0,2],[1,2],[-1,2],[-2,1],[-2,0],[-2,1],[-3,5],[-3,0],[1,4],[-1,2],[1,3],[2,3],[11,6],[2,2],[0,2],[2,2],[3,0],[1,0],[1,1],[1,0],[1,0],[1,-1],[4,-4],[0,-1],[-1,-1],[0,-1],[0,-1],[-1,-1],[3,1],[1,2],[0,3],[-2,3],[1,1],[3,3],[1,1],[-1,-4],[1,-1],[0,-1],[2,-2],[1,-3],[-1,-2]],[[7235,6695],[8,-3],[22,0],[3,1],[2,2],[3,1],[2,-3],[5,-5],[1,-2],[-1,-3],[0,-3],[0,-2],[1,-3],[-3,-2],[-1,-2],[0,-3],[-3,-1],[-7,-5],[-3,-2],[-3,-1],[-4,-7],[-3,-2],[-3,-1],[-2,-2],[-2,-3],[0,-5],[2,-4],[2,-2],[3,-1],[2,-1],[-1,-3],[-2,-1],[-4,-3],[2,-3],[1,0],[0,-2],[-4,1],[-1,2],[-1,2],[0,3],[-4,1],[-10,-1],[-7,-4],[-2,-1],[-12,7],[-7,8],[-5,11],[-2,10],[1,4],[3,6],[1,4],[0,5],[-1,4],[-1,3],[0,3],[4,7],[8,4],[16,4],[3,-5],[4,-2]],[[4498,6726],[1,0],[2,0],[1,1],[3,3],[1,1],[-1,-1],[1,-3],[2,-2],[9,-2],[1,-1],[-1,-2],[0,-2],[1,-1],[1,-1],[2,-2],[5,-2],[2,-2],[2,-1],[0,-2],[-1,-1],[-1,-2],[-1,-1],[0,-1],[-2,0],[-2,0],[-2,2],[-2,0],[-3,-2],[-1,-3],[-1,-2],[0,-3],[1,-2],[-2,0],[-4,-3],[-1,-1],[-2,1],[-2,2],[-1,0],[-1,-3],[1,-1],[-1,-2],[-1,-1],[-1,-1],[-2,1],[-4,3],[2,0],[2,2],[1,2],[-1,4],[-1,3],[-4,1],[-2,3],[5,-2],[3,1],[3,3],[2,6],[-6,5],[-3,0],[1,-3],[-2,-1],[-1,2],[-4,4],[-6,6],[-1,0],[1,1],[2,6],[1,2],[1,1],[3,1],[4,1],[2,-1],[1,-2],[-1,-1],[1,-1],[1,-4]],[[5018,6729],[1,-1],[11,2],[18,-1],[2,-1],[1,-1],[2,-3],[2,-2],[2,0],[3,3],[8,2],[2,4],[2,0],[2,-2],[2,-3],[0,-3],[1,-4],[2,-4],[1,-3],[0,-6],[-1,-4],[-2,-3],[-3,-2],[2,-4],[0,-2],[1,-3],[-1,-2],[-1,-2],[-1,-2],[1,-3],[-10,2],[-5,3],[-4,4],[-6,11],[-2,3],[-11,5],[-6,0],[-3,2],[-6,6],[-4,7],[-10,11],[0,2],[4,0],[1,0],[1,-1],[2,-2],[1,-1],[1,-2]],[[8337,6728],[-5,-6],[-16,1],[-7,-3],[2,-1],[4,-2],[2,-1],[-1,-1],[-1,-2],[-9,4],[-5,-2],[-5,2],[-7,1],[-2,2],[1,1],[0,1],[0,2],[0,1],[2,1],[1,0],[1,-1],[2,0],[3,1],[5,3],[6,1],[2,1],[1,1],[1,2],[0,4],[1,2],[1,0],[1,0],[1,-1],[1,-2],[1,-1],[4,0],[5,0],[2,-1],[4,-4],[2,-2],[2,-1]],[[5127,6740],[2,0],[1,1],[2,1],[1,1],[1,-1],[2,-1],[9,-8],[4,-2],[12,-11],[1,-3],[1,-4],[-1,-2],[-2,-6],[0,-3],[-1,-6],[0,-7],[-1,-2],[-6,-2],[-2,-2],[-2,-6],[-2,-3],[-2,-1],[-20,-2],[-4,4],[1,0],[3,2],[-2,5],[1,5],[3,2],[3,1],[1,2],[-3,11],[-1,5],[1,2],[0,1],[1,2],[1,3],[-1,2],[-1,2],[-1,4],[-3,4],[-5,2],[-5,0],[-3,-2],[2,-12],[2,-5],[2,-4],[-8,-17],[-3,-7],[1,0],[0,-1],[0,-1],[1,-1],[-3,-4],[-4,0],[-5,2],[-3,2],[0,2],[0,4],[0,2],[1,2],[5,5],[-5,4],[-2,3],[0,4],[-1,2],[0,1],[0,1],[1,2],[1,2],[3,2],[2,2],[-1,3],[0,3],[4,12],[1,2],[3,2],[5,0],[2,0],[1,2],[2,4],[2,1],[9,-1],[3,-1],[-2,-4],[1,-1]],[[8272,6765],[2,-2],[3,-3],[2,-2],[4,-2],[3,0],[0,-2],[1,-1],[2,-1],[1,-3],[-1,-2],[-8,-5],[-7,-8],[-4,-6],[-1,-6],[-7,-1],[-38,15],[2,2],[8,2],[-1,0],[2,1],[4,4],[4,2],[7,8],[7,2],[14,-7],[7,3],[-13,4],[-3,4],[0,3],[2,1],[4,-1],[3,1],[1,0]],[[4399,6755],[-5,-4],[-5,2],[-3,6],[0,4],[0,4],[1,2],[8,2],[2,-1],[1,-4],[1,-11]],[[7131,6759],[3,-10],[-2,-3],[-2,-1],[-5,-2],[-3,-3],[-2,-2],[-1,-3],[-2,-2],[-2,-1],[-4,1],[-16,-5],[-9,-7],[-9,-3],[-8,-8],[-3,1],[-5,4],[-4,2],[-3,-1],[-14,-11],[-6,-3],[-4,2],[-2,3],[0,2],[1,3],[2,3],[18,14],[2,2],[7,12],[4,4],[1,2],[2,0],[8,2],[8,-3],[2,1],[1,4],[0,2],[-1,3],[0,3],[2,2],[3,0],[5,-4],[-2,-3],[3,-3],[4,-1],[3,4],[0,4],[-3,7],[-1,4],[6,0],[2,5],[3,0],[2,-1],[2,-1],[1,-2],[2,-1],[3,0],[7,-4],[4,-5],[2,-3]],[[6938,6791],[16,-5],[3,2],[6,-1],[5,-6],[-1,-8],[4,-1],[2,-1],[2,-2],[2,-5],[-2,-3],[-21,-15],[-9,-3],[-27,-2],[-5,-1],[-2,-1],[-3,1],[-3,5],[0,1],[-7,3],[-3,1],[-3,-3],[2,-4],[11,-16],[3,-4],[-3,-7],[-4,-2],[-11,3],[-6,3],[-7,2],[-2,2],[-1,4],[1,5],[2,5],[0,5],[-3,2],[-2,-3],[-6,-12],[-3,-2],[0,4],[0,4],[-1,4],[-1,2],[-1,2],[-4,1],[-2,1],[-2,3],[-7,6],[-7,1],[-23,-1],[-4,1],[-2,5],[1,2],[6,5],[2,2],[1,2],[0,1],[0,2],[1,1],[8,-2],[7,-5],[6,0],[2,-1],[1,1],[1,0],[2,-1],[2,-2],[0,-2],[1,-4],[0,-2],[3,-3],[3,0],[0,3],[-3,5],[2,1],[2,1],[2,1],[-4,0],[-3,2],[-3,3],[-3,3],[-1,3],[-1,4],[1,2],[2,1],[4,3],[4,0],[8,-1],[0,-4],[4,1],[8,5],[3,-1],[4,-3],[4,-3],[1,-4],[0,-2],[-3,-3],[0,-2],[0,-2],[2,1],[6,6],[2,0],[10,-5],[12,3],[4,4],[2,1],[4,-2],[2,-1],[1,2],[0,1],[0,2],[0,2],[0,1],[0,1],[1,0],[1,1],[3,1],[1,0],[0,1],[1,3],[0,1],[2,0]],[[6619,6778],[-7,-5],[-16,4],[-13,5],[-5,3],[2,3],[10,3],[12,2],[16,-9],[1,-6]],[[6501,6783],[-1,-1],[-5,1],[-5,-4],[-5,1],[-2,-1],[-2,-3],[4,-2],[2,-2],[0,-3],[-4,-1],[-6,1],[-10,5],[-3,3],[-4,3],[-2,5],[2,6],[5,3],[32,-7],[5,-3],[-1,-1]],[[5564,6793],[-2,-1],[-1,1],[-2,4],[0,2],[1,1],[0,1],[6,4],[6,1],[6,-1],[6,-4],[-3,-2],[-11,-3],[-6,-3]],[[5606,6795],[-3,-3],[-3,-1],[4,-5],[0,-2],[0,-3],[-1,-4],[-2,-3],[-4,-4],[-3,-1],[-6,4],[-3,1],[-1,1],[-4,5],[-1,2],[-6,2],[0,2],[15,6],[4,3],[2,1],[4,-2],[2,2],[1,2],[0,3],[0,3],[2,2],[2,0],[2,1],[2,0],[2,-1],[-2,-2],[-1,-1],[-3,-2],[2,-2],[-1,-4]],[[4934,6738],[2,0],[0,2],[1,3],[4,13],[1,3],[3,3],[1,0],[2,-2],[5,-2],[11,-13],[2,-4],[2,-5],[1,-2],[1,-1],[3,-1],[17,-18],[4,-3],[7,2],[4,-3],[6,-7],[3,-1],[7,-1],[10,-4],[3,-5],[1,0],[2,-1],[8,-7],[10,-5],[11,-10],[4,-5],[2,-7],[-1,-3],[-3,-5],[-1,-4],[0,-5],[-1,-3],[-1,-3],[-2,-2],[3,-5],[1,-5],[-1,-5],[-2,-6],[2,-4],[2,-4],[3,-2],[3,0],[3,1],[1,2],[-3,6],[-1,4],[0,3],[1,3],[1,4],[-1,4],[-1,3],[0,2],[1,4],[4,4],[1,2],[0,2],[1,3],[2,1],[1,1],[2,-1],[1,-3],[2,-8],[2,-3],[4,-5],[2,-3],[0,-1],[0,-3],[1,-2],[2,-7],[-2,-1],[-2,-2],[-2,-3],[0,-2],[0,-3],[3,-4],[1,-2],[0,-2],[2,-2],[2,-2],[3,-1],[1,-1],[2,-3],[0,-3],[1,-6],[-1,-5],[1,-5],[2,-3],[2,-1],[1,2],[2,2],[2,1],[6,-2],[2,-2],[2,-2],[0,-2],[0,-2],[1,-2],[1,-2],[1,-1],[2,0],[7,1],[20,15],[4,1],[5,-2],[4,-3],[1,-2],[3,-1],[2,-1],[1,-1],[1,-2],[-1,-1],[-1,0],[0,-1],[-2,-4],[-2,-3],[-2,-4],[-11,-10],[-3,0],[-4,-4],[-11,-15],[-4,0],[2,7],[0,5],[-1,4],[-4,3],[-15,-6],[-3,0],[-1,2],[-2,0],[-2,-1],[-1,-2],[1,-4],[1,-3],[-2,-1],[-3,-2],[-2,-4],[0,-6],[1,-6],[-4,-3],[-9,5],[-5,-1],[-1,-1],[-4,-7],[-2,-2],[-2,-2],[-2,-1],[-3,0],[-3,-2],[-16,-20],[-3,-2],[-23,-6],[-4,1],[-8,5],[-9,2],[-19,9],[-7,2],[-5,4],[-4,1],[-2,2],[-2,2],[-1,3],[-2,2],[-2,2],[-5,0],[4,-8],[1,-5],[-1,-2],[-2,0],[-4,-3],[-2,0],[-2,0],[-7,3],[-7,1],[-8,4],[-2,0],[-2,-1],[-2,-2],[-2,-1],[-11,7],[-11,11],[-3,1],[-3,2],[-6,8],[-3,2],[-6,-1],[-17,9],[-6,1],[-2,0],[-3,-5],[-2,-1],[-1,0],[-2,1],[-2,4],[-2,1],[-6,4],[-5,6],[-3,4],[0,4],[3,5],[2,4],[-1,3],[-4,4],[-8,4],[-5,1],[-2,-3],[1,-5],[1,-4],[1,-5],[-2,-5],[-3,-4],[-4,-2],[-5,1],[-3,4],[-3,5],[-1,2],[-7,2],[-1,0],[-2,-2],[-1,-1],[-4,0],[-17,11],[-1,2],[0,3],[0,4],[1,3],[2,2],[3,2],[0,1],[1,1],[0,2],[-1,2],[0,1],[-6,2],[-7,1],[-4,3],[-1,0],[-1,-1],[-1,-1],[-1,-2],[1,-2],[0,-1],[0,-2],[-2,-2],[-2,0],[-2,1],[-2,1],[-5,1],[-1,-1],[1,-3],[6,-7],[2,-3],[-3,-2],[-7,-1],[-3,-2],[-3,-3],[-1,-1],[-1,2],[-5,5],[-2,1],[-4,2],[-13,5],[-3,2],[-2,3],[1,4],[-1,6],[-3,4],[-6,6],[-4,3],[-2,2],[-1,3],[1,3],[2,3],[3,6],[6,11],[4,3],[4,1],[7,-2],[1,1],[2,1],[3,5],[2,0],[2,-3],[2,2],[2,-1],[2,-2],[2,-1],[2,0],[6,2],[14,-2],[5,3],[3,5],[1,1],[3,1],[10,-1],[3,0],[5,3],[3,3],[1,5],[1,0],[-1,4],[0,4],[1,4],[2,4],[3,3],[2,1],[1,-1],[4,-4],[2,-2],[2,0],[1,1],[2,4],[2,2],[2,0],[6,0],[4,1],[2,2],[0,4],[-3,5],[-5,5],[-11,8],[-6,8],[0,2],[0,2],[1,2],[2,2],[1,1],[1,1],[9,-4],[4,0],[4,2],[-14,6],[-6,5],[0,9],[2,5],[3,2],[10,-1],[10,-3],[2,-4],[2,-2],[4,-5],[3,-6],[10,-8],[7,1],[2,-1],[0,-2],[0,-5],[1,-3],[4,0],[4,2],[3,4],[-18,12],[-4,7],[-5,9],[-1,1],[-1,1],[-4,4],[-2,0],[-3,1],[-2,1],[-2,2],[3,5],[1,2],[1,4],[0,4],[0,5],[0,3],[2,7],[3,6],[7,10],[3,2],[6,0],[3,1],[2,2],[4,10],[5,6],[5,3],[5,0],[6,-3],[2,-1],[4,-3],[11,-2],[2,-2],[1,-2],[0,-2],[0,-2],[1,-2],[2,-2],[4,-2],[2,-2],[16,-4],[3,-3],[1,-6],[4,-3],[7,-5],[-4,-4],[-9,0],[-4,-4],[7,-13],[5,-5]],[[4938,6812],[3,-3],[8,0],[4,-3],[-3,-5],[-3,0],[-3,2],[-2,3],[-4,3],[-8,0],[-5,2],[-2,3],[-1,4],[1,5],[2,3],[4,0],[5,0],[5,-3],[2,-4],[-1,-1],[-1,-2],[-1,-4]],[[6109,6864],[23,-7],[17,7],[6,-2],[2,-2],[3,-6],[3,-4],[2,-3],[1,-3],[-1,-3],[1,-1],[4,-1],[-2,-6],[-2,-2],[-31,-3],[-16,5],[-7,5],[-3,1],[-6,-5],[-3,-2],[-4,1],[-8,6],[-7,2],[-2,1],[-1,2],[0,1],[0,4],[0,2],[-3,3],[-7,3],[-1,3],[3,2],[4,1],[4,0],[2,-1],[8,-1],[10,4],[-4,3],[-1,1],[3,5],[5,-3],[8,-7]],[[4552,6897],[-3,-1],[-3,3],[-6,8],[1,0],[1,2],[1,1],[-1,3],[3,1],[4,0],[4,-1],[2,-3],[-1,-1],[-1,-2],[-1,-2],[-1,-2],[0,-2],[1,-2],[0,-1],[0,-1]],[[4561,7000],[12,-6],[3,2],[4,3],[4,0],[3,-5],[5,-5],[6,-2],[2,-1],[-2,-1],[-1,-1],[3,-2],[8,-1],[4,-3],[5,-4],[-2,-2],[-3,0],[-3,1],[-6,4],[-3,1],[-3,-1],[-3,-1],[3,-4],[6,-3],[3,-4],[0,-5],[4,-7],[3,-6],[-4,-2],[-2,1],[-5,5],[-3,2],[-13,1],[-21,14],[-3,6],[0,4],[1,5],[0,4],[0,3],[-2,4],[-3,5],[3,1]],[[7826,7091],[1,-1],[2,0],[33,-10],[16,0],[3,-2],[-1,-6],[-2,-3],[-4,-5],[2,-5],[-3,-3],[-3,-2],[-21,-2],[-31,-12],[-3,-1],[-3,-5],[-2,-2],[-4,-1],[-5,-1],[-5,1],[-3,3],[0,1],[0,2],[0,2],[0,2],[-2,2],[-1,1],[-2,1],[-3,3],[-1,2],[-4,7],[1,1],[0,1],[-1,1],[6,2],[3,0],[2,-2],[-2,-1],[3,-2],[3,0],[4,2],[2,4],[1,4],[1,9],[1,3],[1,2],[-1,2],[0,2],[0,2],[1,2],[17,6],[3,-2],[0,-1],[1,-1]],[[7675,7178],[2,-1],[1,3],[1,5],[0,1],[2,1],[8,6],[2,3],[3,5],[2,1],[2,-1],[2,-2],[2,-2],[1,-3],[2,-3],[2,-2],[5,-2],[2,-3],[-4,-3],[-10,0],[-8,-4],[-2,-2],[0,-3],[0,-2],[2,-5],[0,-2],[0,-1],[0,-1],[-1,-1],[1,-1],[2,-1],[1,-3],[1,0],[-1,-1],[-1,-4],[-1,-1],[-3,-1],[-3,-1],[-2,1],[-1,2],[-1,3],[1,1],[2,1],[-7,3],[-9,1],[-7,5],[1,14],[-3,1],[-1,2],[0,3],[-2,3],[-7,7],[1,3],[-1,2],[0,1],[-1,1],[2,2],[1,2],[21,9],[4,-1],[4,-4],[2,-3],[-1,-3],[-2,-5],[-1,-3],[-1,-3],[-5,-4],[-1,-3],[1,-4],[1,-3]],[[7724,7208],[-4,-4],[-26,6],[-6,7],[3,6],[3,4],[7,7],[22,2],[3,-2],[2,-3],[4,-10],[-8,-13]],[[5175,7277],[-6,-13],[-1,-5],[0,-4],[-2,-1],[-3,0],[-5,-2],[-3,1],[-6,2],[2,5],[9,3],[3,4],[-2,1],[-5,-1],[-2,2],[0,1],[1,2],[-1,1],[0,1],[-1,1],[3,2],[8,3],[3,0],[4,0],[2,-1],[2,-2]],[[4412,4422],[0,1],[0,98],[0,98],[0,98],[0,98],[0,125],[0,125],[0,125],[0,125],[0,9],[0,9],[0,9],[0,9],[0,10],[1,11],[0,11],[1,11],[0,12],[0,12],[-1,12],[0,12],[-5,3],[-9,2],[-10,1],[-5,0],[-49,9],[-49,8],[-48,8],[-49,8],[-69,12],[-68,11],[-69,12],[-69,11],[-59,10],[-58,10],[-59,10],[-59,10],[-28,4],[-27,4],[-28,8],[-26,15],[-8,7],[-7,7],[-7,8],[-7,8],[-15,17],[-15,16],[-14,16],[-15,17],[-12,13],[-11,12],[-12,13],[-11,13],[-4,3],[-4,1],[-5,0],[-4,0],[-31,0],[-31,0],[-30,0],[-31,0],[-14,0],[-14,0],[-14,0],[-14,0],[-4,0],[-4,-1],[-4,0],[-4,1],[-25,17],[-26,17],[-26,17],[-26,17],[-56,37],[-57,36],[-56,37],[-57,37],[-67,44],[-66,44],[-67,43],[-67,44],[-56,36],[-56,37],[-56,37],[-56,36],[-25,17],[-25,16],[-25,16],[-25,16],[-2,1],[-1,0],[-1,2],[-1,3],[0,8],[0,8],[0,9],[0,8],[0,32],[0,32],[0,33],[0,32],[0,52],[0,52],[0,52],[0,54]],[[2299,6728],[5,-6],[41,-30],[13,-5],[9,-6],[10,-4],[23,0],[7,-4],[29,-6],[24,-2],[44,-10],[23,-7],[2,-2],[2,-4],[18,-17],[29,-13],[20,-13],[23,-7],[17,-2],[36,-8],[7,-6],[16,-8],[4,0],[7,3],[2,0],[5,0],[3,0],[2,2],[-1,1],[-3,3],[-2,2],[5,1],[3,-1],[9,-11],[2,-2],[2,0],[1,0],[3,2],[1,1],[1,-1],[1,-1],[1,-1],[33,-5],[5,0],[1,5],[3,0],[7,-2],[9,-6],[3,-5],[1,0],[27,-6],[3,2],[-17,10],[-3,5],[4,0],[0,2],[-25,15],[-2,2],[1,3],[2,2],[3,2],[6,1],[7,-1],[28,-14],[3,0],[2,0],[2,3],[1,0],[8,0],[-3,3],[-8,4],[-2,2],[-4,5],[-3,1],[3,2],[13,-5],[7,-4],[3,-1],[14,1],[3,0],[8,-7],[1,0],[5,0],[1,-2],[1,-1],[5,-3],[19,-6],[11,-9],[6,-1],[0,1],[-1,1],[-1,1],[5,2],[5,-2],[18,-12],[3,-4],[3,-5],[4,-6],[25,-9],[9,-7],[-1,-1],[0,-1],[-2,-3],[4,-4],[2,-2],[0,-3],[2,-3],[3,-2],[19,-9],[2,-2],[0,-3],[1,-3],[1,-2],[1,-1],[2,-1],[1,0],[3,-4],[1,0],[2,-3],[1,-4],[1,-5],[1,-3],[-7,-1],[-1,-2],[1,-4],[3,-3],[3,-1],[6,1],[0,1],[1,4],[1,3],[3,2],[1,-2],[6,-8],[-2,-1],[-2,-1],[-1,0],[-1,-3],[0,-2],[-1,-1],[-9,-11],[-1,-4],[1,-4],[3,-6],[0,-3],[-2,-3],[-5,0],[-14,4],[-2,0],[-1,-1],[-3,-3],[-1,-1],[-8,0],[-2,1],[-3,3],[-1,1],[-2,0],[-3,3],[-2,0],[-20,-4],[-24,3],[-7,5],[-5,1],[-3,-1],[3,-3],[7,-6],[5,-3],[10,-10],[-3,-1],[-3,1],[-3,-1],[-2,-5],[-1,-3],[-4,-1],[-39,8],[1,-4],[3,-3],[3,-4],[-1,-4],[-1,-2],[-3,-3],[-1,-3],[-1,-2],[-1,-7],[0,-2],[3,-4],[2,-1],[2,0],[6,0],[2,-2],[-2,-2],[-4,-2],[-4,-4],[-3,-1],[-8,-1],[-19,-8],[-5,-1],[-3,-1],[-1,-2],[1,-5],[2,-2],[5,-1],[9,-3],[16,2],[-1,-1],[-1,0],[0,-1],[-1,-2],[2,-2],[4,-4],[2,-3],[3,-2],[7,-3],[1,-2],[13,-3],[24,6],[8,-1],[3,-2],[6,-5],[2,-2],[2,0],[3,0],[4,2],[0,1],[0,1],[0,1],[16,-13],[8,-3],[7,0],[7,3],[4,0],[21,-4],[5,-4],[9,-3],[20,0],[41,3],[14,-2],[13,-6],[4,-1],[24,4],[9,-3],[38,2],[7,2],[1,3],[0,3],[-3,4],[5,3],[16,-4],[18,5],[4,-2],[0,-4],[1,-4],[1,-4],[1,0],[5,0],[3,-2],[1,0],[2,2],[-2,3],[-1,2],[-2,1],[-2,1],[1,2],[0,1],[1,0],[1,0],[0,2],[-1,0],[-2,2],[6,3],[38,-10],[0,1],[-4,6],[2,0],[1,2],[1,1],[1,2],[2,0],[6,0],[-1,-6],[2,-3],[3,1],[3,3],[4,9],[1,2],[3,2],[1,2],[3,3],[9,2],[5,3],[2,-1],[0,-2],[-1,-4],[-2,-2],[-2,-2],[-2,-3],[2,2],[2,0],[1,0],[0,-3],[-1,-2],[-3,-1],[-1,-2],[3,0],[4,1],[7,6],[2,-1],[1,-1],[2,-3],[1,-1],[2,0],[13,8],[7,2],[3,2],[3,6],[1,1],[1,1],[3,3],[1,1],[1,0],[6,3],[11,2],[5,7],[13,8],[10,0],[0,1],[-1,4],[3,0],[4,-1],[7,-4],[4,-6],[3,-1],[3,2],[-7,6],[-1,5],[2,6],[3,2],[3,1],[5,-1],[3,-1],[4,-4],[3,-1],[3,-4],[-1,-7],[-2,-12],[0,-8],[3,-5],[4,0],[3,6],[-2,1],[-3,4],[0,2],[9,-1],[5,-2],[5,-9],[3,-3],[7,-4],[-1,-2],[-4,-2],[-1,-4],[11,-5],[3,-3],[-3,-1],[-8,3],[-2,-1],[0,-4],[3,-3],[20,-7],[2,1],[2,3],[1,3],[-2,3],[2,3],[5,1],[2,1],[1,3],[2,2],[1,-1],[1,-3],[-2,-3],[6,-3],[2,0],[1,-1],[1,-1],[2,0],[1,2],[2,1],[1,0],[21,-3],[1,0],[0,-1],[0,-1],[1,-1],[5,-7],[1,-2],[1,-4],[2,-16],[-1,0],[2,-3],[4,-7],[1,-4],[-2,-2],[-5,-4],[-3,-2],[-1,-2],[0,-3],[1,-4],[2,-3],[5,-1],[3,-2],[-1,-4],[8,-1],[2,-5],[1,-6],[2,-3],[2,1],[3,4],[2,9],[2,10],[0,33],[1,6],[2,2],[8,1],[5,-1],[2,-4],[0,-3],[0,-3],[0,-2],[6,-17],[1,-3],[0,-3],[0,-2],[-1,-2],[0,-3],[1,-7],[1,-7],[2,-7],[2,-3],[3,-1],[1,2],[1,3],[0,4],[-1,1],[0,2],[0,2],[0,2],[0,1],[0,1],[0,3],[1,0],[1,0],[2,1],[1,1],[1,1],[2,0],[5,-3],[2,-3],[1,-4],[1,-3],[3,-3],[31,-24],[3,-8],[2,-2],[0,-2],[1,-4],[5,-11],[1,-4],[1,-3],[-5,0],[4,-12],[0,-4],[0,-5],[0,-1],[-2,1],[-2,1],[-4,3],[-4,1],[-4,2],[-2,0],[-1,-1],[-2,-1],[-1,-1],[-2,0],[-5,2],[-2,0],[0,-1],[0,-1],[0,-1],[-1,-1],[-1,0],[-3,-2],[1,-5],[-3,-1],[-10,3],[-7,5],[-6,6],[-7,3],[-3,3],[-6,8],[-3,3],[-4,0],[0,-1],[4,-5],[1,-2],[2,-3],[2,-10],[1,-4],[1,-3],[1,-1],[2,0],[1,1],[1,2],[-1,2],[0,1],[1,3],[1,0],[1,0],[2,-1],[2,-3],[1,-1],[1,-2],[1,-2],[4,-5],[2,-3],[1,-3],[4,-2],[10,-1],[3,-2],[2,-1],[1,-1],[3,-4],[2,-2],[1,-4],[0,-4],[-1,-5],[1,-1],[-3,-5],[-3,0],[-4,0],[-3,-2],[5,-3],[15,3],[6,-5],[1,-1],[1,-3],[2,-2],[1,-3],[2,-3],[-1,-2],[-1,-7],[0,-2],[1,-3],[3,-4],[1,-4],[1,-3],[1,-2],[1,-1],[2,2],[0,3],[-1,3],[0,2],[2,4],[-1,3],[-1,3],[-2,4],[4,-1],[3,-3],[7,-13],[9,-11],[-1,0],[-2,-1],[0,-1],[-1,-2],[3,-2],[7,-13],[5,-5],[1,-2],[2,-2],[2,0],[1,2],[-2,4],[4,-1],[3,-4],[5,-17],[2,-3],[2,-2],[6,-3],[10,-17],[4,-3],[3,2],[1,5],[-4,3],[-1,6],[-5,6],[-14,14],[-2,3],[-1,3],[-1,6],[-2,2],[-3,1],[-4,5],[-3,1],[-1,2],[0,2],[-1,4],[0,2],[-6,9],[-4,10],[-12,8],[-4,7],[7,-3],[3,-1],[3,4],[-1,3],[0,2],[-1,2],[1,1],[-1,7],[-1,12],[-5,12],[-1,4],[0,3],[0,5],[0,4],[1,3],[2,1],[6,-7],[6,-4],[2,0],[1,2],[0,2],[0,2],[-1,2],[-1,1],[0,3],[-3,9],[-2,2],[0,1],[1,1],[2,2],[2,0],[1,-1],[6,-8],[3,-2],[2,-3],[1,-5],[-1,0],[4,-19],[-3,2],[-2,0],[-1,-2],[-1,-1],[-2,0],[-3,2],[-2,1],[-2,4],[-2,2],[-3,1],[-3,-1],[1,-4],[4,-4],[2,-6],[2,-3],[3,-1],[1,1],[-1,4],[2,-1],[4,-3],[1,-1],[-1,-3],[-1,-2],[-3,-3],[1,-2],[4,-3],[2,-1],[1,2],[2,3],[1,1],[1,-3],[2,-4],[2,-2],[3,2],[-5,7],[-2,5],[-2,4],[3,0],[3,-1],[2,0],[2,3],[0,3],[-1,3],[-4,11],[0,2],[2,1],[3,0],[2,-1],[3,-2],[2,-3],[3,-4],[10,-8],[2,-3],[1,-2],[4,-6],[1,-4],[2,-2],[2,-1],[3,-1],[0,2],[-8,12],[3,3],[-15,23],[-6,7],[-6,4],[-4,2],[1,2],[4,4],[13,4],[-1,4],[2,2],[4,4],[-2,2],[-3,-1],[-2,-1],[-2,0],[-3,1],[-3,3],[-6,2],[-4,2],[-12,13],[-2,4],[-1,10],[-4,10],[-7,15],[-5,3],[-2,5],[-1,4],[3,1],[2,-2],[2,-3],[1,-1],[2,1],[1,11],[5,6],[1,3],[-1,3],[-2,2],[-2,1],[-4,2],[-2,2],[-1,2],[0,2],[-3,6],[-4,5],[-8,7],[-6,2],[-8,8],[-3,5],[-3,2],[-1,2],[-1,2],[-1,6],[0,6],[2,5],[4,10],[1,3],[1,3],[0,2],[2,1],[7,1],[14,8],[3,1],[2,1],[1,2],[2,4],[1,4],[-1,2],[-9,3],[-2,-1],[-2,-1],[-2,-1],[-1,2],[-1,3],[-2,2],[-1,1],[-5,0],[-2,2],[-1,3],[1,0],[1,2],[1,3],[0,5],[0,6],[1,5],[2,3],[2,0],[1,-1],[1,-1],[2,-2],[2,0],[5,0],[2,-1],[4,-2],[-2,-4],[-4,-1],[-7,0],[1,-3],[2,-1],[5,1],[-2,-2],[-4,-1],[-3,-2],[-2,-4],[5,-4],[2,-1],[3,2],[0,1],[1,2],[0,1],[0,1],[6,7],[1,1],[4,1],[3,3],[4,0],[9,-1],[8,-4],[7,1],[11,7],[9,10],[5,3],[5,-1],[-2,-4],[-1,-2],[0,-3],[12,4],[6,-2],[2,0],[-2,3],[5,4],[7,-1],[12,-4],[0,1],[2,7],[-2,7],[-3,6],[-3,3],[3,4],[4,1],[8,0],[-2,-5],[0,-2],[2,-2],[2,-3],[2,-1],[2,3],[0,6],[-1,7],[1,2],[3,0],[12,-11],[2,-3],[2,-3],[2,-6],[1,2],[2,4],[2,1],[1,0],[3,0],[2,1],[-16,10],[-1,5],[2,2],[3,4],[2,3],[1,5],[-1,2],[-4,4],[3,5],[3,1],[7,-1],[3,2],[6,7],[3,1],[7,0],[2,-1],[1,-2],[0,-2],[1,-2],[3,1],[5,6],[3,2],[2,-1],[2,-1],[1,0],[2,1],[1,2],[1,2],[0,2],[0,1],[4,0],[3,-2],[4,-2],[4,4],[0,1],[1,3],[0,2],[2,1],[1,-2],[1,-5],[2,-1],[4,0],[2,1],[0,4],[1,3],[1,4],[1,4],[-1,4],[-1,2],[-3,1],[-2,1],[0,2],[-2,5],[2,3],[4,5],[2,3],[1,1],[0,4],[1,1],[1,2],[1,0],[2,0],[1,2],[-6,1],[-3,-1],[-2,-2],[-3,-3],[-2,-1],[-3,0],[-7,6],[-7,1],[-8,-2],[-5,-3],[0,-5],[-4,-2],[-49,-16],[-4,-2],[15,0],[-3,-2],[-10,0],[-2,-5],[3,-1],[2,-3],[0,-3],[-2,-3],[-2,0],[-1,2],[-1,2],[0,1],[-6,1],[-2,-1],[1,-4],[2,-3],[2,-3],[2,-1],[7,-1],[3,-2],[0,-5],[-2,-3],[-3,-2],[-3,-2],[-1,-4],[1,-4],[2,-3],[1,-3],[-3,-1],[-3,0],[-1,1],[-1,1],[-2,5],[-1,1],[-1,1],[-1,3],[-1,1],[-7,0],[-1,1],[0,2],[0,4],[-7,8],[-2,-1],[-1,-3],[2,-3],[-2,-3],[-3,0],[-6,3],[-2,0],[-5,-3],[-6,0],[-2,-2],[0,-4],[-2,-2],[-6,-4],[-1,-2],[-2,-1],[-1,-2],[0,-4],[-1,-3],[-2,-1],[-4,2],[-6,-2],[-3,0],[-3,9],[-4,3],[-4,1],[-11,-1],[-5,1],[-2,5],[-2,2],[-16,-2],[-4,-2],[-2,-1],[-3,1],[-4,2],[-3,0],[-2,-2],[-2,-5],[-1,-6],[-2,-5],[2,-2],[1,-2],[3,-2],[2,0],[3,0],[2,-1],[1,-1],[1,-1],[1,-3],[1,-1],[1,-2],[13,-6],[2,-2],[-2,-1],[-3,-1],[-10,4],[-13,0],[-2,-1],[-1,-2],[-1,-2],[-1,-2],[-4,-1],[-9,2],[2,4],[-18,0],[-4,-1],[-7,-6],[-16,-8],[-4,3],[-2,3],[0,2],[1,3],[1,2],[3,1],[2,-1],[2,-1],[-2,-3],[-1,-1],[0,-1],[4,0],[2,4],[0,5],[1,7],[-1,2],[-1,2],[-1,2],[-1,1],[-2,-1],[-1,-2],[-1,-3],[0,-2],[0,-2],[0,-1],[-1,-1],[-1,-1],[-4,1],[-1,0],[0,3],[1,3],[1,3],[3,7],[1,4],[1,5],[-2,4],[-4,2],[-4,-2],[-4,-4],[-5,-3],[-2,-4],[-2,-1],[-11,0],[-2,-1],[-1,-3],[-1,-1],[-2,1],[-5,4],[-2,2],[0,2],[0,1],[4,2],[2,3],[2,2],[-1,1],[0,1],[-1,3],[0,3],[1,1],[2,1],[15,13],[22,39],[6,9],[8,5],[54,8],[6,-2],[14,7],[16,2],[7,4],[15,0],[13,10],[1,1],[2,1],[8,2],[19,10],[6,2],[4,2],[3,0],[10,-1],[5,1],[11,9],[13,3],[8,5],[8,2],[4,3],[1,2],[2,6],[3,1],[3,0],[22,-11],[21,-3],[2,-1],[5,-4],[5,-6],[3,-2],[8,-10],[10,-9],[3,-5],[3,-3],[-1,-5],[-1,-2],[-1,-1],[-1,0],[-2,-1],[-2,-2],[-1,-2],[1,-4],[2,-4],[4,-5],[2,-4],[7,-16],[2,-2],[3,-1],[4,0],[0,-2],[0,-6],[0,-2],[-1,-1],[-2,1],[-3,3],[-2,1],[-2,0],[-2,0],[-3,-2],[-1,-3],[-1,-1],[-1,0],[-2,-1],[-1,-1],[-2,-2],[-2,-3],[0,-3],[1,-3],[2,-1],[18,3],[3,-2],[0,-2],[0,-2],[-1,-2],[-1,-2],[4,-6],[25,-21],[7,-2],[5,4],[-8,5],[-3,3],[6,0],[2,1],[1,2],[1,1],[2,-1],[1,-2],[2,-2],[3,2],[3,4],[1,5],[2,-3],[2,-1],[1,-2],[1,-4],[0,-4],[-2,-3],[-10,-5],[0,-2],[1,-4],[3,-2],[3,1],[5,4],[2,3],[0,1],[3,-3],[1,-1],[18,0],[3,-3],[-2,-8],[-6,-15],[6,0],[2,-3],[0,-8],[1,-3],[2,-1],[5,0],[3,-2],[1,-1],[0,-1],[-2,-3],[0,-1],[4,-3],[13,1],[2,1],[2,2],[1,2],[3,-1],[7,-6],[10,-1],[17,6],[13,-6],[5,-1],[5,2],[4,2],[1,2],[3,5],[1,1],[3,1],[8,-4],[3,0],[2,0],[1,2],[2,1],[0,1],[0,1],[0,6],[-1,1],[0,2],[2,3],[1,1],[9,9],[3,0],[4,-2],[1,-3],[2,-3],[2,-6],[-1,-2],[-2,-2],[-1,-1],[-1,-3],[1,-1],[1,-7],[0,-3],[0,-3],[0,-2],[1,-1],[4,0],[2,-1],[4,-6],[1,-1],[2,-1],[2,0],[5,0],[3,-1],[2,-2],[3,-4],[3,-3],[4,-2],[3,0],[6,2],[2,0],[3,-6],[0,-2],[2,-3],[8,-9],[8,-7],[22,-5],[13,-3],[13,-8],[4,-7],[6,2],[1,-1],[0,-4],[1,-3],[2,-1],[2,2],[-1,1],[-1,2],[-1,3],[0,2],[1,2],[6,3],[2,4],[2,1],[3,-5],[2,-1],[2,-1],[5,1],[1,1],[1,1],[2,3],[-1,1],[0,2],[1,1],[1,1],[3,-1],[1,0],[1,-3],[2,-4],[2,-2],[5,-1],[2,-1],[0,-2],[0,-2],[1,-2],[1,-1],[1,0],[1,-1],[1,0],[1,1],[1,4],[4,0],[2,-3],[3,-4],[2,-2],[2,1],[4,1],[2,-1],[2,-1],[5,-3],[2,0],[5,4],[1,0],[2,-2],[2,0],[3,0],[1,1],[-1,3],[-4,2],[-1,2],[1,3],[2,1],[2,-2],[3,-3],[2,1],[1,1],[2,1],[7,-2],[2,0],[2,4],[0,4],[-1,4],[2,1],[2,-1],[3,-3],[2,-1],[2,1],[3,4],[2,2],[2,1],[0,-1],[0,-1],[0,-2],[0,-4],[0,-1],[0,-1],[16,1],[4,1],[3,4],[-1,4],[-3,4],[0,4],[2,2],[2,-1],[1,-2],[2,-2],[2,-3],[2,0],[2,1],[5,6],[1,1],[3,-3],[1,2],[1,5],[1,1],[2,0],[3,-3],[2,-1],[4,3],[2,0],[1,-3],[-1,-2],[0,-2],[-1,-2],[1,-3],[5,3],[10,8],[5,2],[28,-1],[2,-1],[5,-5],[3,-1],[6,2],[1,-1],[4,-2],[1,-1],[2,1],[2,2],[1,1],[2,0],[4,-4],[6,-2],[3,1],[0,5],[3,0],[9,-2],[3,-2],[1,-3],[2,-1],[2,1],[4,2],[2,1],[6,-2],[2,-1],[0,-1],[1,-1],[0,-1],[2,0],[1,1],[1,1],[1,1],[2,0],[1,-2],[3,-3],[1,-1],[2,-1],[5,-1],[2,-2],[5,-7],[3,-3],[1,0],[2,2],[1,0],[1,0],[2,-2],[1,0],[4,3],[2,1],[6,-2],[4,0],[4,0],[5,-6],[1,1],[1,3],[1,6],[1,2],[1,1],[2,1],[11,-2],[5,2],[4,8],[-1,3],[1,2],[2,1],[2,-1],[2,-2],[2,-2],[2,-2],[2,0],[21,1],[1,0],[2,1],[2,1],[1,2],[-1,2],[-4,5],[-4,5],[0,1],[-1,1],[-1,2],[-1,0],[-1,0],[-2,-1],[-2,0],[-4,2],[-3,3],[-5,6],[-4,2],[-8,6],[-1,3],[-1,19],[-1,3],[0,3],[1,2],[2,1],[8,-1],[2,1],[3,5],[2,0],[2,-1],[1,-2],[0,-3],[1,-2],[3,-4],[20,-19],[3,-3],[7,-7],[2,-3],[0,-2],[1,-4],[1,-2],[1,-2],[3,-4],[1,-1],[10,-22],[1,-1],[0,1],[3,2],[1,1],[1,-1],[6,-4],[1,-1],[4,-10],[1,-2],[2,-1],[9,0],[4,-2],[4,-4],[2,-4],[2,-1],[2,0],[4,0],[2,0],[2,-3],[2,-4],[3,-4],[4,-1],[7,3],[9,0],[3,3],[3,4],[3,3],[4,3],[3,1],[2,-3],[3,-5],[2,-2],[2,3],[0,6],[5,3],[10,1],[-1,3],[-2,2],[-5,3],[-3,1],[-5,-2],[-2,1],[1,5],[11,11],[-1,3],[-1,5],[-4,8],[-1,1],[-2,2],[-1,2],[-1,2],[0,5],[-1,2],[-3,5],[-2,-1],[-1,-4],[-2,-4],[-4,-3],[-4,1],[-4,2],[-5,8],[-3,4],[-12,9],[-7,8],[-2,1],[-2,2],[-2,1],[-2,-3],[-1,-2],[0,-5],[0,-2],[-1,0],[-2,0],[-1,1],[-2,1],[0,1],[-1,0],[-2,-3],[-4,0],[-2,0],[-2,-1],[-10,-10],[-2,0],[-2,6],[-1,1],[-2,-2],[-5,-8],[0,-4],[0,-6],[1,-6],[1,-3],[-3,-1],[-3,1],[-4,5],[-4,4],[-1,1],[-2,4],[0,1],[-1,2],[-1,3],[-1,3],[-2,2],[-2,1],[-6,7],[-8,6],[-3,2],[-1,4],[-4,11],[-1,1],[-5,6],[-3,5],[-5,5],[-2,3],[2,1],[1,-1],[3,-3],[1,5],[3,5],[3,2],[2,-1],[1,-2],[1,-1],[1,-1],[1,-2],[0,-3],[-1,-5],[-1,-2],[3,-9],[6,3],[4,8],[0,8],[1,2],[-3,3],[-3,2],[-13,6],[-6,5],[1,1],[2,-1],[2,1],[1,2],[-6,5],[-6,6],[-11,14],[-2,2],[-1,2],[0,3],[2,1],[3,0],[3,0],[2,-1],[9,-10],[4,-2],[-1,3],[0,1],[-1,2],[-1,1],[0,1],[5,-3],[2,0],[2,3],[-7,9],[-2,3],[0,3],[2,0],[2,-2],[6,-7],[1,-2],[1,-1],[13,-5],[7,-9],[5,-3],[2,0],[2,3],[2,3],[1,1],[2,-2],[3,-1],[3,0],[1,4],[-4,3],[-1,1],[0,2],[2,1],[22,-6],[-1,12],[7,-1],[8,-4],[4,-1],[-1,5],[-2,3],[-3,2],[-8,2],[-2,1],[-1,1],[-2,1],[0,2],[1,2],[-9,8],[-3,5],[0,2],[0,3],[1,3],[0,2],[2,1],[3,-1],[3,-3],[2,0],[4,3],[2,0],[7,-2],[10,-1],[2,-2],[1,-2],[2,-1],[0,-1],[1,-2],[0,-1],[0,-1],[0,-1],[2,-7],[3,-3],[3,-1],[4,-3],[2,-1],[2,1],[1,2],[0,2],[-1,1],[-1,1],[-1,1],[-5,-2],[-4,1],[-1,1],[0,2],[1,0],[0,1],[1,3],[1,3],[2,2],[2,1],[14,0],[20,-9],[1,-5],[16,-13],[-2,1],[-2,0],[-2,-1],[-1,-2],[1,-2],[2,-1],[15,-5],[2,-3],[-1,-3],[0,-2],[0,-3],[1,-2],[-2,0],[-5,2],[-2,-1],[-1,-4],[-1,-3],[1,-2],[12,-3],[3,-2],[1,0],[1,0],[27,10],[2,0],[1,-5],[1,-1],[1,-1],[1,1],[1,-3],[-6,-2],[-1,-3],[1,-4],[2,-3],[3,-2],[2,-1],[1,2],[-2,5],[1,5],[3,5],[12,11],[5,3],[5,1],[-8,-8],[-1,-3],[0,-7],[0,-4],[-1,-2],[-4,-7],[-14,-12],[3,-4],[-2,-2],[-3,-2],[-3,0],[0,-3],[1,-1],[-4,-2],[-4,0],[-5,-2],[-2,-6],[2,-5],[4,-3],[8,0],[2,1],[1,3],[0,4],[1,2],[2,1],[2,2],[3,0],[1,-2],[3,-5],[5,-1],[5,1],[3,2],[-2,2],[-3,3],[1,5],[-2,1],[-6,-2],[3,7],[4,5],[10,9],[6,3],[2,0],[2,1],[2,2],[9,3],[7,4],[16,6],[2,2],[2,2],[3,4],[1,1],[4,1],[1,1],[-2,-6],[-3,-5],[-12,-15],[-2,-5],[-2,-6],[6,-7],[-2,-9],[-4,-10],[0,-7],[-1,-3],[-4,-6],[-1,-3],[-1,-6],[0,-2],[-1,-2],[-1,-3],[-1,-3],[-1,-6],[-1,-3],[-1,-2],[-2,-3],[0,-6],[0,-7],[1,-6],[1,-3],[1,-1],[-1,-3],[-3,-5],[0,-1],[1,-2],[0,-2],[1,-1],[4,5],[-1,-14],[-2,-4],[-4,0],[-2,1],[-2,2],[-1,2],[-2,4],[0,3],[-1,3],[0,2],[-2,1],[-3,-3],[0,-6],[1,-7],[0,-5],[-2,-7],[-4,-4],[-8,-4],[2,-2],[1,0],[0,-2],[-2,-3],[-1,-2],[0,-9],[2,1],[2,3],[2,1],[2,0],[2,-2],[1,-2],[-1,-4],[1,-2],[2,-1],[1,-2],[1,-1],[6,1],[2,-1],[3,-2],[1,-1],[1,0],[3,1],[2,1],[2,2],[2,4],[2,3],[2,-1],[0,-3],[0,-2],[2,-1],[2,-1],[-2,-3],[-2,-1],[-1,-2],[-1,-5],[-1,-3],[-1,-3],[-4,-6],[-4,-3],[-2,-2],[-1,-4],[0,-4],[-2,-7],[0,-3],[9,2],[1,0],[1,1],[1,0],[1,-1],[1,-1],[-1,-2],[0,-2],[3,-4],[4,1],[4,4],[4,4],[5,8],[2,1],[1,-2],[0,-1],[2,0],[1,1],[13,15],[1,4],[2,2],[3,1],[2,0],[0,-3],[2,-2],[1,-3],[2,-2],[2,0],[2,3],[1,2],[0,3],[1,2],[2,2],[3,0],[2,-2],[3,-5],[-1,-3],[-1,-5],[-4,-5],[-25,-27],[-3,-6],[3,1],[6,3],[3,1],[2,2],[5,7],[3,1],[-1,-1],[-2,-4],[5,-1],[11,4],[4,-5],[-1,-1],[1,-1],[1,-2],[0,-1],[-1,-3],[0,-2],[0,-1],[0,-3],[1,0],[1,1],[1,0],[2,2],[1,3],[-1,5],[1,3],[1,-2],[0,-1],[5,-6],[0,-1],[0,-1],[0,-2],[-1,-2],[0,1],[1,-7],[0,-1],[-3,-6],[-1,-3],[2,-4],[2,-1],[3,-1],[2,-1],[1,-5],[0,-3],[-2,-8],[-1,-3],[-1,0],[-3,2],[-1,1],[-1,-2],[-1,-2],[0,-2],[-7,1],[-14,-2],[-4,5],[2,1],[1,1],[1,1],[0,2],[-6,-1],[-12,-6],[-13,-3],[-4,0],[-3,3],[-1,5],[0,4],[-1,3],[-4,0],[1,5],[2,4],[2,4],[2,2],[-3,1],[-3,-2],[-7,-12],[0,-3],[1,-3],[1,-4],[-3,-3],[-4,-1],[-4,0],[-4,2],[2,1],[1,1],[1,1],[1,2],[-4,1],[-8,-2],[-4,2],[-3,5],[2,9],[-1,5],[-2,1],[-4,-1],[-2,0],[-2,2],[-1,2],[-2,1],[-7,-5],[0,-3],[4,-7],[2,-7],[0,-1],[11,-5],[2,-4],[2,-4],[2,-4],[4,-2],[7,-4],[3,-4],[1,-3],[0,-5],[1,-2],[1,-3],[2,-1],[3,-1],[22,-13],[1,-2],[1,-2],[0,-4],[1,-3],[4,-6],[2,-4],[0,-4],[0,-6],[-1,-4],[-3,-8],[3,3],[6,11],[3,3],[4,0],[2,1],[2,1],[1,3],[0,2],[-1,4],[-2,6],[-1,3],[-1,0],[-1,-3],[1,-6],[0,-2],[-1,-2],[-2,-2],[-2,0],[-1,0],[-3,1],[-1,1],[-1,6],[-2,6],[-3,2],[-2,2],[-3,4],[-3,8],[-3,3],[-2,3],[-6,3],[-2,2],[-3,7],[-1,1],[-1,1],[-3,4],[-1,1],[-1,3],[0,1],[3,3],[4,3],[6,1],[6,-2],[4,-4],[2,0],[5,3],[2,0],[1,0],[3,-1],[1,-1],[0,-1],[1,-4],[0,-2],[2,0],[8,2],[10,0],[8,-3],[2,2],[2,3],[2,2],[3,1],[2,0],[0,-3],[2,-2],[3,-2],[3,-1],[3,0],[2,1],[2,2],[3,5],[0,3],[1,2],[3,5],[-2,3],[-2,2],[-3,1],[-3,2],[1,5],[-1,5],[-5,6],[0,2],[1,2],[2,4],[0,2],[-1,3],[-1,2],[-1,2],[0,2],[0,1],[7,11],[1,1],[1,1],[1,2],[0,3],[0,3],[0,4],[2,2],[3,0],[5,2],[0,3],[-15,9],[-3,4],[-1,6],[-1,2],[-1,4],[-1,2],[0,2],[-1,7],[0,2],[-2,1],[-2,1],[0,3],[1,2],[2,3],[3,2],[1,1],[3,2],[0,5],[-2,5],[-2,4],[-9,9],[-2,2],[-1,6],[-3,2],[-4,0],[-3,2],[-2,4],[-3,3],[-2,2],[-6,1],[-2,1],[-1,3],[-1,2],[-1,3],[1,2],[2,4],[1,2],[5,2],[0,1],[0,1],[0,1],[1,0],[4,0],[5,5],[2,10],[-1,9],[-5,3],[4,8],[2,3],[7,7],[1,3],[1,5],[1,0],[2,3],[-1,3],[-1,3],[1,3],[-3,0],[-2,0],[-1,2],[3,2],[21,1],[3,2],[1,0],[2,-2],[-2,-2],[-1,-3],[-1,-3],[-2,0],[0,-2],[5,2],[9,10],[6,1],[1,-2],[5,-6],[3,-2],[6,3],[3,0],[3,-4],[5,-3],[7,1],[7,5],[4,6],[1,2],[0,1],[0,2],[1,1],[4,3],[10,11],[10,4],[1,2],[0,1],[1,2],[6,2],[2,2],[2,4],[0,-2],[3,4],[10,4],[5,2],[2,5],[-1,4],[-1,5],[0,6],[0,3],[1,2],[3,4],[2,1],[1,0],[1,1],[0,3],[1,2],[1,1],[19,13],[22,7],[4,4],[5,1],[2,1],[4,5],[2,1],[5,1],[2,1],[-1,1],[0,2],[-1,1],[2,0],[0,4],[2,3],[9,2],[8,5],[-2,-1],[-21,-5],[-7,3],[-3,0],[-7,-3],[-2,0],[0,2],[1,3],[1,1],[1,2],[0,-2],[1,4],[1,3],[0,5],[1,6],[2,5],[-1,1],[-1,1],[0,2],[1,3],[4,1],[1,3],[-2,0],[2,2],[1,4],[1,3],[5,4],[1,3],[-1,4],[-1,2],[-5,4],[-2,3],[0,4],[1,3],[0,2],[-2,1],[1,3],[1,1],[1,1],[-3,3],[-3,0],[-3,-1],[-3,0],[1,1],[1,0],[-2,1],[-5,-1],[1,0],[-2,-1],[-4,-2],[-3,0],[-2,3],[0,3],[2,2],[1,1],[1,1],[0,2],[-2,0],[-2,-1],[-4,-3],[1,6],[4,4],[7,7],[-3,1],[-5,-2],[-4,-3],[-3,-4],[1,0],[3,0],[0,-2],[-4,-4],[-1,-1],[1,0],[-1,0],[-1,-1],[0,-1],[1,0],[1,-2],[1,-1],[-1,-1],[-3,-3],[0,-2],[-1,-2],[-1,-2],[-3,-1],[-5,-5],[-1,-3],[-2,-4],[1,0],[0,-3],[0,-2],[1,0],[2,0],[-1,-1],[-4,-2],[1,-1],[3,-2],[1,0],[2,0],[2,2],[3,2],[4,6],[2,1],[0,-2],[0,-1],[-1,-1],[-1,-1],[0,-2],[15,10],[-2,-5],[-4,-2],[-4,-2],[-2,-3],[-3,-5],[-4,-4],[-4,-4],[-5,-2],[-3,-4],[-1,-2],[-2,-1],[-20,0],[-14,-8],[-4,3],[-2,2],[-2,-2],[-1,-2],[-1,-1],[-1,0],[-1,1],[-2,2],[-1,3],[-1,2],[-2,1],[-7,-1],[1,5],[2,11],[2,4],[3,5],[1,3],[0,4],[-1,1],[-1,0],[-2,2],[1,2],[1,6],[0,1],[-2,1],[-1,1],[-1,2],[0,3],[4,3],[4,-1],[8,-5],[0,-4],[2,1],[3,2],[2,2],[1,3],[3,9],[2,4],[3,2],[4,1],[6,-1],[3,1],[4,2],[3,4],[-4,3],[-1,2],[1,3],[1,2],[7,5],[4,1],[3,3],[5,4],[-4,0],[-11,-7],[-13,-1],[-6,4],[-2,3],[0,4],[1,3],[2,7],[2,4],[-1,1],[-1,3],[0,1],[2,2],[2,4],[-5,4],[-1,1],[4,6],[5,4],[14,2],[9,5],[11,0],[2,-1],[2,-3],[1,-1],[5,1],[4,2],[2,2],[3,7],[3,3],[14,8],[10,0],[-2,-3],[-6,-7],[-4,-2],[-1,-2],[-3,-5],[-1,-2],[-2,-1],[-1,0],[-2,-1],[-7,-8],[-3,-1],[-8,-14],[2,0],[4,4],[2,1],[-4,-7],[-2,-2],[-2,-1],[1,-3],[2,0],[2,1],[3,0],[0,-2],[-3,-4],[-2,-3],[0,-2],[2,-2],[2,2],[2,3],[2,2],[3,3],[6,4],[4,5],[7,6],[-1,1],[-1,0],[-1,-1],[-1,-2],[0,2],[2,1],[5,6],[11,4],[3,4],[1,3],[1,3],[9,3],[-3,2],[-10,-5],[-4,0],[-5,3],[-4,-1],[1,2],[6,11],[4,5],[2,3],[-1,1],[0,1],[2,2],[2,1],[1,2],[-1,2],[-1,-2],[-1,0],[-1,0],[-1,2],[0,3],[-3,3],[-3,2],[-2,1],[-15,-1],[-8,-6],[-3,-3],[-2,-1],[-1,0],[0,3],[-2,0],[-5,-2],[-1,-2],[0,-3],[1,-3],[-3,-4],[-4,0],[-7,1],[-17,-2],[-20,5],[-2,3],[-5,7],[-2,2],[-3,2],[-2,2],[-9,13],[0,5],[-1,2],[-1,2],[-1,2],[0,3],[-1,2],[-1,1],[-2,1],[-1,-1],[-1,-2],[0,-1],[-1,1],[-2,3],[0,1],[-1,2],[1,3],[0,3],[-13,-4],[-2,-2],[1,-3],[1,-4],[3,-3],[-3,-6],[-5,-4],[-5,-2],[-4,0],[-9,5],[-2,1],[0,3],[-2,1],[-2,-1],[-1,0],[1,3],[-2,2],[-11,-2],[-2,0],[-3,3],[-3,3],[-1,1],[-3,-1],[-21,10],[-3,3],[-5,2],[-7,6],[-3,1],[-2,2],[-7,10],[-2,1],[-8,0],[-2,-1],[-1,-3],[-1,-4],[-2,-3],[-2,2],[-1,2],[-1,2],[2,4],[1,1],[1,0],[-2,3],[-3,1],[-2,-1],[0,-5],[-2,-1],[-7,3],[-2,0],[1,-1],[-4,0],[-3,3],[-6,8],[-4,4],[-13,4],[2,1],[4,4],[1,0],[3,-4],[2,0],[2,3],[-2,2],[-3,3],[4,3],[0,2],[-2,-1],[0,2],[0,2],[1,3],[-1,2],[-2,1],[-2,1],[-2,1],[-2,1],[-1,0],[-4,-2],[-2,0],[-2,2],[-26,30],[-3,6],[-1,1],[-1,3],[0,3],[0,3],[-1,3],[-3,9],[-1,5],[2,20],[1,6],[4,7],[8,9],[2,2],[3,-1],[2,-1],[2,0],[2,2],[1,2],[1,6],[-1,9],[4,8],[5,7],[3,6],[1,-1],[1,-2],[0,-3],[1,-2],[2,-2],[2,0],[4,3],[-1,5],[2,3],[3,3],[3,1],[3,-2],[6,-5],[16,-6],[4,0],[1,-1],[0,-1],[1,0],[1,2],[0,1],[-1,1],[0,1],[-13,2],[-5,3],[-5,4],[-5,2],[-1,2],[0,4],[0,5],[11,9],[2,0],[5,3],[3,3],[4,1],[2,1],[-10,-1],[-25,-13],[-2,-2],[-2,-3],[1,0],[-4,-3],[-9,4],[-9,6],[-6,6],[-2,4],[-1,4],[-2,4],[-2,3],[-13,5],[-3,5],[-1,8],[2,8],[6,14],[2,4],[2,12],[0,1],[0,3],[0,1],[1,1],[0,1],[1,4],[0,2],[-1,2],[3,0],[4,2],[2,3],[4,6],[1,0],[0,1],[0,2],[-3,3],[-1,1],[-1,-1],[-1,-1],[2,-3],[-1,-2],[-3,-1],[-2,0],[-3,1],[-5,6],[-3,3],[0,3],[1,1],[3,1],[7,5],[3,0],[3,-3],[2,-2],[2,3],[-8,6],[-2,2],[0,3],[1,5],[2,9],[-1,0],[-4,-2],[1,2],[-1,1],[-1,1],[-1,1],[-1,0],[3,3],[4,2],[13,0],[3,1],[3,2],[-1,2],[2,5],[3,6],[3,5],[3,4],[6,3],[17,0],[7,-3],[3,-2],[2,-5],[4,-5],[21,-15],[3,0],[3,1],[2,1],[3,-1],[3,0],[-1,4],[1,1],[1,2],[1,1],[-2,2],[-5,2],[-1,4],[13,0],[0,2],[-1,1],[5,0],[0,2],[-3,0],[-2,2],[-3,2],[-2,2],[-3,4],[1,1],[1,1],[2,6],[1,1],[2,0],[1,0],[2,2],[0,1],[0,2],[0,1],[2,2],[8,4],[-7,-1],[-5,-1],[-1,-4],[-16,2],[-3,2],[-3,2],[-14,0],[-2,2],[-4,5],[-2,1],[-5,1],[-2,1],[1,5],[5,2],[1,3],[-1,0],[-1,0],[-1,1],[-1,2],[3,3],[5,2],[8,2],[4,-1],[1,1],[6,4],[0,2],[0,2],[1,1],[2,1],[3,0],[1,1],[2,3],[2,1],[2,1],[5,4],[3,1],[21,0],[3,1],[2,3],[-2,1],[-1,2],[0,3],[1,4],[0,1],[2,2],[1,2],[1,5],[3,2],[9,1],[6,3],[2,0],[5,-1],[1,0],[3,1],[17,-1],[5,-2],[6,-1],[5,2],[5,3],[5,4],[-4,2],[-13,-8],[-2,0],[-5,2],[-32,3],[-6,-3],[-3,-1],[-4,2],[-4,2],[0,4],[0,1],[1,1],[1,0],[0,2],[-1,0],[-2,1],[1,2],[4,5],[0,1],[0,1],[0,1],[-1,0],[0,2],[38,6],[38,7],[7,-3],[0,-1],[1,-3],[0,-3],[1,-1],[8,-3],[1,-1],[-1,-5],[-3,-3],[-3,0],[-3,-1],[-3,-1],[2,-1],[3,-1],[1,-1],[0,-5],[-1,-2],[-1,-1],[-9,-1],[-4,-2],[-7,-5],[4,-6],[1,-3],[-2,-5],[1,0],[0,-2],[0,-1],[1,-1],[1,-1],[5,5],[1,2],[4,10],[1,2],[1,1],[2,0],[1,-1],[0,-2],[0,-3],[2,-2],[1,1],[1,3],[1,1],[4,0],[1,-1],[2,-2],[0,-2],[-1,-1],[-1,-1],[-1,0],[-1,-3],[0,-1],[-1,-2],[0,-3],[1,-3],[1,-3],[-3,-9],[0,-5],[1,0],[14,18],[1,3],[4,7],[3,3],[3,0],[2,-2],[0,-5],[5,-1],[7,7],[5,-2],[3,-5],[1,-1],[3,-1],[6,1],[3,1],[3,2],[4,3],[6,0],[4,-3],[2,-8],[0,-4],[-1,-3],[-12,-12],[2,-2],[7,-5],[12,-11],[10,-6],[14,-5],[3,-2],[2,-2],[1,-2],[1,-2],[12,-4],[4,-4],[1,-5],[1,0],[1,-1],[0,-2],[0,-2],[0,-1],[0,-2],[1,-2],[2,-3],[2,-3],[2,-1],[9,-1],[4,-2],[3,-2],[1,-2],[0,-3],[-1,-2],[-1,-2],[4,-3],[2,-3],[0,-9],[1,-2],[0,-3],[0,-3],[6,-8],[2,-4],[1,-4],[0,-2],[-1,-2],[0,-3],[0,-2],[0,-2],[0,-1],[0,-2],[-1,-2],[-1,-1],[0,-4],[0,-2],[3,-4],[-3,-6],[-1,-3],[0,-4],[-1,-12],[-1,-3],[1,-2],[-1,-5],[1,-4],[3,-4],[2,-2],[-2,-1],[-5,-2],[-2,0],[-1,0],[-6,6],[-3,2],[-3,0],[1,-4],[2,-3],[9,-8],[14,-2],[6,-3],[3,-2],[5,-3],[2,-3],[-2,-1],[1,-3],[0,-1],[2,0],[1,0],[0,-2],[-6,-1],[0,-2],[4,1],[3,-1],[1,-3],[-1,-1],[-2,-1],[-2,-3],[4,0],[-3,-2],[2,-1],[5,3],[23,-5],[-1,-1],[0,-1],[2,-1],[6,-2],[4,-3],[2,0],[0,-2],[-1,-3],[-1,-3],[-3,-1],[2,-4],[2,-2],[3,2],[2,7],[1,4],[4,-3],[5,-4],[2,-5],[-2,-1],[-3,0],[-4,0],[-2,1],[1,-1],[-1,-2],[-1,-2],[0,-2],[1,-2],[2,-1],[1,0],[1,-1],[2,-2],[1,-3],[-1,-1],[-1,1],[-2,0],[-2,-1],[-5,-4],[5,-1],[8,-3],[4,-1],[2,1],[1,0],[1,-3],[0,-2],[-1,-1],[-1,0],[-1,-1],[0,-1],[0,-1],[10,-4],[-1,-5],[1,-1],[1,-1],[3,1],[1,0],[0,-2],[-1,-1],[-1,-2],[1,0],[1,0],[1,-1],[1,-1],[-2,-2],[0,-2],[0,-3],[1,-3],[-3,-2],[-2,-1],[-6,0],[0,-2],[7,1],[3,-1],[2,-3],[-1,-1],[-2,-1],[5,-3],[0,-1],[0,-1],[0,-1],[1,1],[1,4],[1,1],[1,2],[1,2],[1,1],[2,1],[4,-1],[-4,5],[-2,4],[1,4],[2,2],[3,0],[3,0],[3,-2],[0,-1],[1,-3],[1,-1],[1,1],[2,3],[0,1],[2,0],[2,0],[2,-3],[1,-3],[1,-3],[-1,-2],[-2,-1],[-2,-1],[4,-12],[1,-6],[-1,-5],[0,-2],[1,1],[2,2],[1,2],[1,1],[0,3],[1,-1],[3,-3],[7,-5],[3,-4],[2,-5],[0,-2],[-1,-1],[-3,-1],[-17,-3],[-13,2],[-3,-1],[-5,-2],[-13,-2],[-3,1],[-2,2],[-4,6],[-3,2],[-2,2],[-10,2],[-2,1],[-1,2],[1,3],[-4,7],[-2,3],[-2,2],[0,-4],[-3,-1],[-5,0],[8,-5],[3,0],[1,-1],[0,-1],[-1,-2],[0,-1],[-1,-1],[-1,-1],[-3,0],[1,2],[-2,0],[-5,-3],[-3,-1],[-10,0],[0,-1],[3,0],[3,-1],[2,-2],[3,-4],[-2,-1],[-3,-1],[-2,-2],[-3,-8],[-6,-1],[-7,-3],[2,-1],[5,0],[1,-1],[3,-5],[2,-1],[1,1],[1,4],[17,6],[22,-1],[7,-3],[6,-5],[6,-7],[-2,-2],[-7,-5],[-1,0],[-4,0],[-1,-1],[0,-1],[-1,-2],[0,-1],[-16,-15],[-2,-1],[-3,-1],[0,-1],[1,0],[2,-2],[-2,-4],[-9,-4],[-4,-4],[-1,0],[-1,1],[-2,-1],[-1,0],[-1,-2],[-1,-1],[-2,0],[-4,-2],[-2,-1],[-3,0],[-1,0],[-1,-3],[0,-1],[1,-3],[0,-1],[-1,-3],[-1,-2],[-2,-1],[-2,1],[-5,6],[-1,1],[-2,-3],[-1,-2],[3,-1],[-4,-3],[-17,-11],[-3,-1],[0,-1],[37,3],[0,-2],[-10,0],[-2,-2],[-8,-5],[-3,-1],[-8,1],[-5,3],[-8,-2],[0,-2],[7,-3],[3,0],[8,2],[8,-3],[3,-1],[3,2],[4,5],[15,4],[3,-3],[5,0],[2,0],[0,1],[0,2],[1,1],[0,1],[1,0],[3,-1],[0,-2],[-1,-1],[-1,-2],[-1,-1],[-1,-2],[3,-2],[1,0],[1,0],[0,2],[8,-1],[4,-2],[6,-6],[7,0],[2,-2],[1,-1],[-18,3],[-2,1],[0,-1],[-1,2],[-2,1],[-1,0],[0,-1],[7,-8],[6,-3],[2,-2],[0,1],[0,-1],[0,-2],[2,0],[2,-1],[1,-1],[0,-1],[-1,-2],[3,0],[16,-6],[3,-3],[7,-3],[6,-4],[3,0],[-1,5],[3,3],[5,2],[3,3],[-4,1],[3,2],[8,0],[1,1],[4,5],[2,1],[-1,3],[0,2],[-1,1],[1,2],[4,2],[2,2],[1,1],[2,0],[1,1],[0,1],[1,3],[3,1],[5,1],[8,-2],[2,-2],[2,0],[1,1],[2,1],[2,-2],[1,-2],[2,-1],[6,4],[3,0],[0,-6],[2,0],[6,2],[-1,-4],[-3,-1],[-6,0],[-4,-1],[-12,-13],[-4,-2],[-20,-5],[-4,-2],[2,-3],[2,0],[2,2],[2,1],[18,0],[3,4],[1,1],[13,5],[2,-1],[1,-2],[2,-4],[4,-6],[3,0],[9,4],[3,-3],[9,5],[2,-1],[0,-3],[0,-4],[0,-3],[15,0],[0,7],[3,3],[5,1],[3,0],[17,-7],[4,-3],[4,-1],[1,-1],[1,-2],[1,-2],[1,-1],[0,-1],[2,0],[1,-1],[0,-1],[0,-1],[0,-1],[1,0],[4,2],[2,0],[0,-1],[-5,-3],[-14,3],[-1,-1],[-4,-3],[-2,-1],[-3,0],[-2,0],[-2,1],[-1,1],[0,1],[1,2],[-2,1],[-9,0],[0,-1],[1,0],[3,-2],[3,-3],[1,-1],[3,0],[2,-1],[-1,-1],[-3,-1],[-1,-1],[-2,1],[-3,-1],[-3,-2],[-5,-2],[-6,-6],[0,-1],[0,-2],[-2,-2],[0,-3],[0,-3],[1,-3],[-1,-1],[0,-1],[0,-2],[1,-2],[-2,-2],[1,-1],[1,-4],[-15,-1],[0,1],[4,3],[1,1],[-2,0],[-4,-2],[-1,0],[-1,1],[-1,1],[2,1],[1,0],[0,2],[-1,0],[0,2],[1,-1],[2,0],[1,1],[0,3],[-2,0],[-15,-9],[-2,1],[-1,3],[3,3],[7,3],[0,2],[1,4],[0,1],[1,1],[4,3],[0,1],[-3,0],[-3,-2],[-9,-9],[-8,-5],[-4,0],[-7,2],[-22,14],[0,-1],[6,-7],[16,-10],[8,-2],[2,-2],[2,-3],[1,-2],[2,-2],[10,-5],[20,-18],[4,-1],[2,-2],[4,-5],[1,-1],[3,-1],[7,-3],[2,-3],[-1,-4],[-4,-5],[0,-2],[-1,-6],[2,-2],[8,-1],[-1,0],[2,-2],[3,-9],[-2,0],[0,-2],[1,-1],[1,-2],[-2,0],[-2,-1],[0,-2],[10,-1],[9,-6],[2,-2],[-1,-2],[-3,-2],[-2,-2],[6,0],[1,0],[0,-2],[-3,-1],[-5,1],[-3,-1],[-2,-1],[-2,-2],[-5,-6],[4,-1],[9,7],[5,1],[-1,-3],[0,-1],[-1,0],[-1,-2],[-1,-3],[1,-1],[2,0],[-1,-3],[-2,-2],[-2,0],[-1,3],[-1,0],[0,-3],[-2,-4],[0,-3],[0,-2],[2,-2],[1,-2],[2,-1],[0,-3],[-1,-3],[-1,-5],[-1,-1],[-2,-2],[0,-3],[-1,-1],[-3,-3],[2,-1],[3,-1],[2,-3],[2,-3],[-2,-2],[-3,0],[-1,-1],[0,-2],[0,-1],[1,-2],[3,-1],[2,-3],[1,-2],[-2,-1],[0,-1],[-3,-8],[-1,-1],[-8,-7],[-1,0],[0,-2],[0,-2],[1,0],[4,-2],[8,-5],[20,-8],[0,-2],[-5,0],[-13,-8],[10,3],[6,0],[3,-3],[-1,-1],[0,-1],[1,-2],[-2,-3],[1,-3],[3,-4],[2,-3],[-1,-4],[1,-2],[4,-3],[5,1],[5,3],[3,4],[0,1],[-1,2],[-1,2],[0,2],[-3,2],[0,1],[0,2],[6,5],[2,1],[3,1],[5,7],[3,2],[0,1],[1,1],[0,4],[-1,2],[0,1],[2,2],[-2,2],[2,3],[7,1],[2,2],[-1,3],[0,2],[1,2],[2,-1],[2,-1],[0,2],[1,3],[0,2],[1,3],[1,4],[2,2],[-5,0],[-3,1],[-3,3],[-2,6],[-1,4],[0,3],[2,2],[2,2],[2,3],[-1,4],[6,4],[2,2],[2,5],[1,1],[1,1],[3,-3],[2,-6],[0,-8],[-2,-5],[2,0],[1,1],[3,4],[0,1],[0,2],[0,2],[-1,2],[0,3],[0,5],[1,4],[1,3],[1,4],[0,3],[0,3],[-2,3],[1,1],[1,1],[1,2],[0,2],[0,3],[-2,1],[-3,0],[1,3],[0,2],[0,2],[-1,7],[-1,6],[-1,8],[0,2],[1,3],[2,6],[2,6],[4,6],[15,12],[11,15],[0,2],[4,3],[1,2],[0,7],[2,1],[6,8],[2,1],[5,1],[8,5],[18,-3],[11,-8],[0,-2],[0,-3],[1,-3],[1,-2],[2,-1],[2,-2],[3,-7],[2,-3],[13,-13],[19,-13],[4,-4],[9,-4],[22,-16],[26,-30],[4,-9],[4,-10],[3,-10],[0,-5],[0,-3],[0,-3],[-3,-8],[1,-5],[2,-5],[2,-5],[1,-2],[1,-5],[1,-5],[2,-4],[3,-3],[-1,-3],[0,-3],[1,-2],[4,-7],[0,-5],[0,-4],[-1,-6],[1,-1],[1,-1],[0,-1],[-1,-2],[-1,-2],[0,-3],[-1,-2],[-1,-1],[0,-1],[-1,-5],[-1,-2],[-1,-2],[-5,-3],[-2,-3],[-2,-4],[-1,-1],[-6,6],[-4,3],[-7,1],[-1,0],[-2,1],[-1,0],[-2,-2],[0,-1],[-2,0],[-1,1],[-1,1],[-1,3],[0,3],[1,22],[0,3],[-1,2],[-1,1],[-2,-1],[0,-2],[1,-3],[-1,-1],[-5,-3],[-1,-1],[-3,-5],[-3,-2],[-9,-5],[0,-2],[-1,-1],[2,-5],[6,-7],[1,-5],[1,-1],[0,-2],[0,-1],[0,-1],[2,-4],[2,-10],[0,-3],[0,-2],[-5,-7],[-1,0],[-1,-1],[1,-1],[0,-2],[0,-1],[1,-1],[-1,-2],[-2,-1],[-2,-2],[-1,-1],[0,-2],[1,-1],[2,-3],[-2,-7],[1,-4],[2,-5],[2,-9],[3,-9],[1,-3],[0,-4],[1,-5],[2,-5],[2,-4],[7,-5],[3,-4],[-1,-7],[2,-7],[1,-2],[4,-3],[1,-2],[3,-5],[11,-3],[4,-3],[3,-5],[5,-5],[2,-2],[1,-4],[3,-5],[12,-9],[6,-11],[2,-2],[1,-1],[7,-7],[10,-7],[2,-3],[1,-4],[0,-15],[1,-2],[2,0],[3,2],[3,0],[1,-2],[1,-2],[-2,-4],[-1,-1],[-3,-2],[-5,-5],[-4,-2],[-1,-1],[-1,-3],[0,-1],[0,-1],[0,-2],[0,-3],[0,-2],[0,-3],[-1,-3],[2,0],[2,1],[2,3],[4,7],[1,2],[2,1],[3,0],[5,-1],[1,-1],[0,-4],[1,-4],[0,-4],[1,-4],[2,-1],[1,2],[2,17],[1,2],[2,2],[1,1],[-1,2],[0,1],[-1,1],[0,1],[2,2],[3,-1],[2,1],[0,2],[2,0],[3,-2],[2,-3],[-1,-3],[1,-4],[0,-2],[2,1],[2,6],[1,3],[0,2],[0,6],[0,3],[3,-2],[2,-1],[3,-2],[3,1],[2,2],[-4,4],[-1,3],[1,4],[-2,2],[-4,1],[-2,2],[-1,2],[-2,3],[0,2],[2,1],[4,0],[2,-2],[2,-2],[1,-2],[1,-2],[1,0],[2,2],[-1,3],[0,3],[1,3],[3,2],[4,6],[2,1],[8,3],[3,0],[5,-3],[6,-8],[4,-2],[3,2],[0,7],[1,3],[2,-2],[0,-2],[0,-4],[0,-3],[-1,-1],[2,-4],[4,1],[3,3],[3,3],[-1,3],[-2,6],[-1,3],[0,1],[0,2],[0,2],[4,1],[2,2],[1,4],[-1,3],[-1,2],[0,3],[1,5],[-1,3],[0,2],[1,3],[0,2],[1,2],[2,3],[0,3],[-4,3],[0,3],[0,3],[0,3],[-1,2],[-2,3],[-1,3],[6,7],[9,22],[6,9],[6,5],[2,4],[2,3],[7,4],[1,2],[1,3],[1,1],[3,3],[1,2],[1,3],[-1,3],[0,4],[-1,2],[1,2],[1,1],[1,1],[2,-2],[-2,-3],[0,-3],[1,-2],[2,0],[2,1],[1,3],[1,1],[3,1],[1,0],[13,11],[0,2],[-1,2],[-1,4],[0,3],[0,3],[1,3],[2,1],[3,1],[0,2],[-2,3],[0,3],[-1,3],[0,2],[0,2],[2,2],[2,0],[3,0],[3,0],[1,5],[-3,0],[-2,2],[-1,3],[1,6],[1,2],[1,1],[1,1],[0,3],[0,2],[-3,3],[-1,1],[3,2],[2,1],[10,15],[1,4],[-2,1],[-2,2],[-1,3],[1,1],[4,0],[2,1],[2,2],[-5,1],[-2,1],[-1,4],[0,1],[1,1],[1,2],[0,2],[0,6],[0,2],[0,1],[0,2],[-1,2],[-1,2],[0,2],[1,2],[-1,7],[0,3],[1,3],[1,2],[2,3],[1,2],[1,2],[1,5],[0,2],[1,6],[3,3],[4,2],[5,-4],[4,0],[5,2],[3,4],[-3,2],[-9,4],[17,-2],[5,-3],[-1,-3],[-1,-1],[-1,-1],[4,-2],[15,-3],[1,3],[-2,3],[-3,1],[3,4],[4,0],[9,-4],[18,3],[3,-1],[4,-3],[10,-3],[3,1],[0,1],[3,4],[0,1],[0,4],[-2,1],[-2,1],[0,2],[-2,5],[-3,2],[-8,0],[-2,0],[-5,-3],[-2,0],[-18,6],[-4,3],[-1,5],[2,2],[15,-3],[-1,1],[-1,1],[0,1],[4,0],[1,1],[2,1],[-2,1],[-5,2],[-1,2],[0,3],[6,-1],[2,2],[-17,6],[3,4],[5,0],[14,-3],[1,-2],[3,-1],[4,0],[9,-2],[1,1],[-1,3],[0,3],[-1,1],[-11,2],[-4,3],[0,4],[-1,0],[-2,0],[-1,1],[-1,1],[4,2],[13,-2],[0,2],[-1,1],[4,3],[10,-6],[6,1],[6,-2],[2,-2],[2,0],[2,1],[2,2],[2,3],[-3,2],[-11,0],[-1,1],[-2,2],[-1,1],[-3,-3],[-2,0],[-1,2],[-2,2],[0,1],[1,2],[0,2],[-1,3],[-1,1],[-14,-3],[-4,1],[-3,3],[1,3],[-3,2],[-7,1],[6,4],[1,1],[-5,2],[-9,-7],[-5,2],[3,7],[5,2],[5,0],[4,3],[-3,1],[-17,-2],[-3,-1],[1,-3],[0,-3],[-2,-2],[-2,-1],[-6,2],[-3,1],[-2,3],[3,3],[7,1],[2,4],[-5,2],[-11,1],[-5,3],[2,2],[-6,10],[-1,1],[-3,1],[-1,1],[0,1],[-1,3],[-1,2],[-1,4],[1,2],[3,0],[5,-3],[3,-1],[4,0],[3,2],[-3,1],[-6,4],[-4,1],[-1,1],[-1,3],[1,1],[1,1],[1,0],[-2,3],[-2,1],[-3,2],[-1,4],[3,2],[8,-1],[3,1],[1,2],[-2,3],[-2,3],[8,-2],[-2,3],[-2,2],[-3,1],[-2,1],[-2,-1],[-3,-5],[-2,-1],[-3,1],[-3,4],[0,5],[2,5],[-1,5],[2,4],[4,2],[-1,4],[2,1],[9,-1],[-12,12],[-6,9],[3,7],[5,0],[3,2],[1,5],[-1,7],[-3,2],[-3,1],[-2,4],[10,5],[4,-1],[-2,-7],[2,0],[3,1],[2,2],[2,3],[0,2],[-1,4],[-2,3],[-1,1],[-25,2],[1,5],[2,1],[23,-1],[4,-1],[17,-13],[9,-4],[4,0],[0,5],[3,2],[14,2],[3,1],[3,3],[-1,2],[-4,3],[4,2],[10,-5],[5,1],[4,2],[40,2],[1,-1],[2,-2],[1,-1],[1,-2],[1,-1],[5,2],[9,-4],[15,-14],[25,-8],[7,-5],[2,-1],[5,0],[22,-5],[5,2],[11,-3],[3,1],[5,4],[3,1],[27,-6],[13,3],[24,-3],[3,1],[-1,0],[-1,1],[-1,1],[-1,2],[8,0],[4,-1],[3,-3],[-1,-1],[0,-2],[18,-7],[18,-1],[-17,-5],[-3,1],[-3,4],[-18,1],[1,-1],[0,-1],[1,-1],[1,0],[-2,-1],[-4,0],[-1,-2],[11,-5],[-3,-2],[-19,-3],[15,-2],[3,-2],[1,-3],[12,-11],[-4,-2],[-42,7],[-42,8],[-2,-1],[-2,-2],[0,-1],[2,-1],[9,0],[52,-14],[51,-14],[4,-3],[3,-4],[-5,-2],[-14,6],[0,-2],[3,-2],[6,-8],[8,-3],[4,-3],[4,-4],[3,-5],[-3,-1],[-6,1],[-4,-1],[3,-4],[-1,-2],[-6,-3],[3,-4],[3,-1],[24,4],[-1,1],[0,1],[-1,3],[6,2],[27,-3],[13,-5],[16,-10],[3,-1],[6,2],[1,0],[6,-3],[1,-1],[1,-1],[2,-12],[2,-5],[2,-3],[1,-2],[0,-1],[-10,2],[-3,-2],[-3,-2],[-7,-5],[-1,-1],[-7,-14],[-4,-2],[-3,-1],[-3,0],[-3,1],[-2,0],[-1,-2],[0,-2],[0,-2],[-1,-2],[-1,-1],[-2,-3],[-18,-4],[-14,-7],[-6,-5],[21,6],[5,2],[2,1],[1,-1],[1,-2],[1,-1],[2,0],[4,2],[4,0],[3,-1],[3,-2],[3,-4],[1,0],[1,-1],[1,0],[1,-2],[0,-1],[0,-2],[0,-1],[0,-1],[1,-2],[2,-1],[1,0],[2,2],[2,4],[1,1],[2,1],[4,1],[2,1],[2,0],[2,0],[2,-1],[2,-2],[8,-11],[1,0],[0,-3],[1,-2],[3,-2],[2,-3],[-1,-2],[-1,-1],[-1,-1],[-1,-5],[0,-6],[0,-6],[1,-3],[1,-1],[-1,-3],[-1,-5],[-2,-1],[-6,-3],[-2,-5],[-5,-2],[-5,-2],[-4,-2],[-17,-15],[-2,-1],[-3,0],[-10,4],[-6,1],[2,-4],[0,-2],[-1,-3],[-2,-5],[-1,-1],[-3,-2],[-7,-1],[-4,-6],[-2,0],[-5,1],[-2,3],[-3,12],[-3,3],[-5,1],[-11,5],[-5,1],[0,-2],[3,-4],[3,-2],[4,-2],[1,-2],[-1,-4],[-4,-3],[-5,0],[-19,4],[-4,2],[-12,10],[-4,1],[-3,-5],[0,-3],[2,-1],[3,1],[2,-2],[1,-2],[2,-2],[6,-2],[-2,-2],[-5,-3],[-4,0],[-4,1],[-2,0],[-1,-3],[1,-4],[1,-2],[2,-1],[2,0],[9,2],[2,-1],[2,-2],[1,-2],[1,-2],[11,-7],[3,-4],[-4,-5],[-10,2],[-4,-2],[3,-5],[4,-3],[4,-3],[4,-1],[8,5],[3,1],[2,-7],[-1,-5],[-2,-5],[-7,-11],[0,-2],[1,-3],[2,-2],[13,-6],[3,0],[4,2],[4,4],[0,1],[1,1],[-1,3],[1,2],[1,0],[0,1],[1,1],[3,5],[1,3],[2,1],[4,-2],[0,-4],[-3,-16],[-3,-3],[-7,-2],[-2,0],[4,-4],[1,-3],[-3,-2],[-2,-2],[-2,-3],[0,-2],[1,0],[-1,-3],[-1,-3],[-1,-2],[-2,-1],[1,-3],[4,-5],[1,-3],[-1,0],[1,-1],[0,-1],[-1,-1],[2,-6],[2,-3],[1,-3],[3,-3],[7,-4],[3,-2],[1,-2],[0,-1],[0,-1],[1,-2],[1,-1],[7,-5],[6,-1],[2,-3],[-1,0],[-1,0],[-1,-2],[3,-4],[3,-3],[4,-3],[3,-1],[3,-2],[6,-11],[7,-6],[14,-8],[6,-8],[3,-5],[4,-7],[2,-2],[2,-5],[1,-2],[4,-4],[3,-8],[-2,-7],[-4,-7],[-2,-7],[-1,-8],[-4,-14],[-1,-6],[-1,-13],[0,-4],[-5,-13],[1,-2],[0,-2],[-1,-3],[1,-1],[-1,-1],[-1,-1],[0,-2],[-1,-1],[-1,-2],[0,-1],[-1,-1],[-2,0],[-2,0],[-1,-2],[0,-9],[-1,-4],[-3,-1],[-11,3],[-1,-1],[-2,-4],[-1,-1],[-6,-3],[-12,7],[-6,-2],[-3,-2],[-6,-2],[-3,-2],[0,-2],[-1,-4],[-1,-1],[-1,0],[-2,-1],[-1,-2],[-1,-1],[1,-1],[3,-3],[-9,-11],[-1,-1],[1,-3],[1,-2],[-1,-1],[-5,-4],[-3,-5],[-2,-3],[-1,-3],[0,-5],[2,-3],[1,-2],[0,-3],[-24,3],[-1,0],[0,-2],[0,-3],[0,-2],[-10,-4],[-2,-2],[1,-5],[-7,-4],[-2,-2],[-1,-2],[-2,-7],[-1,-5],[-5,-1],[-24,1],[-7,-3],[-10,0],[-1,-1],[-1,-3],[0,-4],[-1,-7],[-4,-4],[-8,-4],[1,0],[-2,-3],[-14,-13],[-4,-2],[-2,0],[-1,-1],[-5,-8],[-2,2],[-7,-1],[-6,3],[-7,-4],[-3,2],[1,1],[2,2],[-8,8],[-3,4],[1,0],[2,0],[1,0],[-1,4],[1,1],[2,0],[1,-2],[1,-4],[2,-4],[2,-3],[2,0],[1,2],[-1,4],[-3,4],[-3,2],[-6,9],[-1,3],[-2,7],[-1,2],[-6,3],[-13,2],[-5,6],[-1,6],[-2,1],[-2,-2],[-3,-1],[-2,2],[-1,3],[2,3],[-5,3],[1,4],[0,2],[0,2],[-1,4],[1,4],[3,3],[3,1],[2,-2],[2,-3],[0,-3],[0,-3],[1,-4],[2,-1],[1,2],[2,4],[1,3],[-3,6],[-1,1],[-2,1],[-2,0],[-2,1],[-1,3],[1,2],[-1,0],[-1,1],[2,6],[5,5],[2,6],[0,5],[-1,5],[-1,3],[-2,3],[-1,-3],[-1,-8],[-1,-3],[-2,-4],[-6,-2],[-4,-3],[2,-1],[-2,-3],[-6,-6],[1,-7],[-5,-1],[-15,3],[-3,4],[-1,4],[4,5],[-4,5],[-12,1],[-3,6],[6,1],[17,-1],[0,2],[-15,7],[-5,4],[0,6],[0,2],[3,1],[-2,3],[-4,2],[-6,-1],[-2,2],[-6,6],[-2,3],[6,-1],[12,-4],[6,3],[-50,13],[-3,2],[-8,6],[-3,-2],[2,-1],[1,-2],[-1,-1],[-1,0],[-1,0],[2,-2],[8,-3],[0,-3],[-2,-1],[-2,-1],[3,-2],[3,1],[4,3],[3,1],[4,0],[4,-1],[3,-3],[4,-2],[-42,-4],[-23,-11],[1,-2],[4,-1],[2,-1],[-2,-2],[-2,-3],[-3,-1],[-2,1],[1,-4],[4,-4],[3,-2],[3,-1],[3,1],[2,2],[1,1],[2,1],[9,0],[2,1],[2,2],[2,4],[2,3],[0,2],[-1,1],[-8,1],[-1,1],[3,2],[6,-3],[3,1],[1,1],[0,2],[0,2],[1,1],[12,2],[1,0],[1,-1],[2,-4],[1,0],[14,1],[-2,-4],[1,-3],[2,-1],[-18,-1],[6,-1],[9,-5],[8,-6],[4,-7],[-5,2],[-11,6],[-5,-2],[5,-5],[12,-2],[5,-2],[7,-6],[2,-1],[9,1],[2,-2],[-3,-4],[-8,-7],[-3,-2],[2,-2],[7,-2],[2,-1],[2,-4],[2,-1],[1,0],[4,4],[1,1],[1,-1],[1,0],[1,-1],[8,0],[2,-2],[0,-2],[0,-1],[0,-2],[0,-1],[0,-3],[-2,-4],[-1,-5],[-2,-1],[-1,0],[-2,-1],[1,-2],[15,-17],[5,-3],[3,-5],[1,-1],[2,0],[2,-2],[3,-4],[5,-7],[2,-4],[0,-5],[0,-11],[0,-5],[1,-3],[4,-2],[3,0],[2,-2],[1,-7],[2,-4],[7,-6],[1,-4],[0,-3],[-3,-2],[-5,-3],[-5,-4],[-3,-1],[-3,0],[1,3],[1,2],[2,2],[2,2],[0,1],[1,1],[1,2],[-1,0],[-4,0],[-10,-2],[-6,0],[-1,4],[-3,3],[-9,4],[-3,2],[-2,1],[-1,2],[-1,2],[0,5],[-1,2],[-2,3],[-3,1],[-3,-1],[-3,-2],[-5,-2],[-3,-2],[2,-3],[-2,-1],[-3,0],[-2,1],[-2,2],[-1,3],[-1,3],[0,2],[0,3],[-2,6],[-3,3],[-4,2],[-3,1],[1,3],[-4,1],[-3,-4],[-3,-6],[-5,-4],[1,-4],[3,-3],[2,-2],[2,0],[1,-1],[1,-2],[4,-7],[10,-9],[2,-5],[3,-4],[0,-5],[-3,-1],[-9,0],[-1,1],[-5,5],[-8,5],[-16,2],[-4,1],[-4,4],[-4,5],[-2,1],[-2,1],[-4,-3],[-2,-1],[-5,0],[-2,1],[-1,3],[-3,2],[-3,1],[-2,1],[0,4],[0,1],[-1,3],[-2,2],[-1,0],[-2,-1],[0,-2],[1,-2],[0,-3],[-1,-1],[-5,-6],[-2,0],[-2,0],[-3,1],[-1,2],[-2,2],[-2,11],[1,7],[-4,3],[0,3],[2,5],[-2,9],[-1,8],[-3,6],[-3,3],[-1,5],[-3,8],[-6,1],[-7,-2],[-18,-8],[-7,-3],[-4,-5],[-6,2],[-2,-1],[-6,3],[-26,-3],[-2,0],[-4,4],[-2,1],[-6,-1],[-2,1],[-2,2],[-3,5],[-2,1],[-2,-2],[-2,-3],[-1,-3],[-1,-2],[-3,0],[-12,8],[-5,2],[-3,0],[2,-4],[0,-1],[-5,-7],[-6,2],[-7,4],[-5,-2],[4,-7],[13,-13],[-4,-2],[-14,5],[-2,-2],[2,-4],[11,-15],[5,-9],[3,-3],[9,-1],[8,-4],[15,-1],[2,-1],[5,-5],[36,-16],[8,-2],[4,-7],[-5,-4],[-4,-5],[-5,-7],[0,-5],[-1,-3],[0,-2],[0,-2],[1,-3],[-10,-8],[-3,-2],[-8,0],[-3,-2],[-1,-5],[0,-3],[-1,-3],[-2,-1],[-4,-1],[-4,0],[-2,-1],[0,-1],[-1,-1],[0,-1],[-1,0],[-2,-2],[-2,0],[-3,0],[-2,0],[-2,-2],[1,-4],[-2,-3],[-3,-3],[-1,-4],[0,-1],[1,0],[0,-1],[-1,-1],[-2,-2],[0,-1],[0,-3],[2,-1],[1,-2],[1,-2],[-1,-1],[0,-1],[0,-1],[0,-2],[0,-1],[0,-2],[0,-1],[-1,-2],[-6,-7],[-1,-3],[0,-1],[-1,-2],[0,-1],[-1,-1],[-1,-1],[-3,1],[-3,-1],[-6,-3],[-2,-2],[-2,-2],[-3,-6],[-1,-2],[-6,-6],[-2,-2],[-2,-3],[-3,-1],[-5,-1],[-8,-3],[-2,-1],[0,-1],[1,-3],[-1,-1],[-2,-1],[-1,-1],[0,-1],[0,-2],[0,-1],[0,-1],[-3,-1],[-3,1],[-2,-1],[-1,-1],[-1,-2],[-2,-2],[0,-3],[1,-3],[4,-2],[1,-3],[-2,-4],[-8,-2],[-3,-3],[-2,-2],[-1,-1],[-6,-1],[-6,-5],[-8,-3],[-31,0],[-31,1],[-15,6],[-12,10],[-4,1],[-2,2],[-1,2],[-2,4],[-1,2],[-4,4],[-1,1],[0,1],[-3,5],[-1,1],[-2,1],[-7,6],[-5,7],[-10,7],[-4,2],[-9,1],[-3,3],[-3,4],[-5,3],[-8,2],[28,0],[12,-4],[7,-1],[0,2],[-3,3],[-34,7],[-4,2],[-3,1],[-1,1],[-3,3],[-2,0],[-2,-1],[-10,0],[-3,0],[-7,5],[-6,2],[-2,2],[-1,2],[-1,2],[0,2],[-1,2],[-1,2],[-8,5],[-5,3],[-2,0],[-2,1],[-1,2],[-1,2],[-1,2],[-8,7],[-5,2],[-3,-1],[1,-3],[-1,-1],[-2,0],[-2,1],[-1,2],[-1,5],[-1,2],[-1,2],[-10,4],[-4,4],[-2,2],[-2,0],[-6,-3],[-2,2],[-21,3],[-4,-2],[26,-13],[3,-4],[2,-4],[-1,-5],[-1,-3],[-3,-1],[-3,0],[-5,2],[-4,3],[-2,4],[-2,1],[-7,2],[-6,4],[-2,1],[-7,-1],[-4,-1],[-3,-3],[-7,-3],[-8,0],[-15,6],[3,3],[5,1],[8,-1],[8,2],[1,1],[0,2],[-1,1],[-1,1],[-1,0],[-18,-5],[-37,4],[-36,5],[-5,2],[-2,0],[-9,-1],[-14,7],[-4,-1],[-4,-3],[-10,0],[-3,-3],[42,-25],[7,-7],[3,-2],[3,2],[-7,10],[-3,7],[-1,3],[-3,2],[3,4],[4,1],[13,-1],[7,-4],[22,-3],[19,-3],[26,-10],[23,-4],[16,-7],[12,-10],[9,-3],[4,-3],[10,-13],[2,-1],[2,-5],[1,-1],[2,0],[11,-7],[2,-2],[2,-5],[0,-7],[1,-3],[2,-4],[2,-4],[2,-3],[15,-10],[4,-2],[2,-1],[2,-2],[2,-3],[2,-1],[0,-1],[0,-2],[0,-3],[0,-1],[3,-4],[4,-5],[2,-3],[4,-3],[8,3],[4,0],[4,-4],[2,-1],[29,-2],[8,-3],[49,-5],[5,-3],[3,0],[8,4],[4,1],[12,-1],[6,-4],[3,0],[2,4],[-1,1],[4,3],[6,0],[6,-2],[9,-6],[17,2],[37,-9],[1,-2],[0,-3],[-2,-3],[3,-3],[6,-3],[3,-4],[1,-5],[-1,-3],[-3,-2],[-2,-4],[0,-6],[2,-3],[0,-3],[-3,-3],[-9,-7],[-3,-3],[1,-1],[1,-2],[1,-2],[1,-2],[-4,-5],[-1,-2],[-2,0],[-2,0],[-2,-1],[-1,-3],[-3,-4],[-4,-7],[-4,-8],[-3,-9],[-1,-3],[1,-3],[1,-4],[-1,-1],[-1,-2],[-5,0],[-2,-1],[-5,-8],[-3,-3],[-3,2],[0,2],[0,3],[0,2],[-2,1],[-1,-1],[-1,-3],[-2,-3],[0,-2],[0,-3],[1,-9],[1,-4],[-1,-3],[-1,-2],[-2,-3],[-2,-2],[-1,-6],[-1,-2],[-2,-3],[-19,-9],[-5,0],[0,-1],[0,-2],[0,-3],[-1,-2],[0,-2],[-2,-3],[-2,-4],[-1,-3],[0,-4],[-1,-5],[-1,-10],[-2,-6],[-14,-15],[-2,-4],[-1,-5],[6,-7],[1,-4],[-7,-1],[-2,-3],[-1,-2],[-2,-3],[-2,-3],[-2,-1],[-11,2],[-2,-1],[-4,-6],[-2,-1],[-2,-1],[-19,-14],[-2,0],[-7,0],[-1,-2],[2,-3],[-5,-2],[-8,-2],[-1,-2],[0,-2],[-2,-4],[-2,-1],[-3,0],[-5,1],[-5,3],[-2,0],[-1,-1],[-1,-1],[-1,0],[-1,1],[0,3],[1,0],[-4,2],[-7,-5],[-4,2],[0,3],[1,1],[-1,1],[-1,1],[-6,1],[-4,5],[-6,3],[-3,4],[-2,6],[-1,5],[-1,3],[-1,2],[-2,3],[-1,1],[0,1],[-1,0],[-1,-1],[-3,-5],[2,-1],[5,-6],[1,-3],[5,-13],[2,-4],[6,-9],[1,-2],[0,-2],[1,-2],[2,-1],[3,1],[1,-1],[2,-1],[-3,-2],[-12,2],[1,0],[2,3],[-2,1],[-4,0],[-2,1],[-1,1],[-4,6],[-1,1],[-2,2],[-2,0],[0,-1],[1,-2],[2,-1],[1,-1],[-1,-4],[1,-1],[0,-1],[0,-1],[-5,5],[-5,8],[-5,5],[-6,-4],[-3,2],[-4,7],[-3,4],[-3,0],[-2,0],[1,-5],[-1,-8],[1,-4],[2,-4],[7,-9],[-3,0],[-4,5],[-6,11],[-7,10],[-4,4],[-4,0],[1,-2],[1,-1],[0,-2],[1,-3],[-1,-3],[-1,1],[-2,5],[-2,4],[-8,9],[0,2],[1,2],[1,1],[2,1],[3,0],[2,1],[1,2],[0,2],[-1,3],[-1,2],[0,3],[0,6],[0,3],[-1,1],[-1,3],[-2,2],[-1,-2],[-1,-4],[0,-3],[0,-2],[0,-1],[-2,-1],[-5,2],[-2,0],[2,-4],[2,-1],[3,-2],[1,-5],[-2,-4],[-4,2],[-5,4],[-3,1],[-4,0],[-3,-1],[-6,-5],[-8,-3],[6,-4],[2,-2],[1,-2],[2,0],[1,2],[-1,4],[7,0],[4,-1],[2,-4],[-3,-2],[-1,-3],[0,-3],[2,-3],[3,0],[0,-1],[1,-1],[0,-1],[0,-1],[2,-2],[3,-1],[0,-2],[-4,1],[-2,-1],[-2,-1],[1,-2],[0,-1],[0,-1],[3,-1],[5,-2],[3,-3],[0,-3],[0,-4],[-1,-3],[-1,-2],[-2,-1],[-5,1],[-3,0],[-3,-1],[-1,0],[-1,0],[-4,3],[3,1],[2,3],[0,3],[-2,4],[-2,3],[-3,0],[-1,-1],[1,-5],[-2,0],[-3,1],[-2,0],[-1,-4],[-3,3],[-4,7],[-2,3],[-3,1],[-3,1],[-4,-1],[-2,-1],[0,-2],[6,-2],[10,-11],[5,-5],[-1,-1],[-1,-1],[-1,0],[-1,1],[3,-4],[0,-4],[11,-11],[3,-5],[1,-2],[0,-1],[0,-2],[-1,-2],[0,-3],[-1,-1],[-3,1],[-13,0],[2,-1],[4,-3],[1,-2],[0,-3],[-1,-2],[-2,-2],[-1,-1],[0,-4],[-1,-1],[-3,0],[-1,-2],[-3,-8],[1,-3],[1,-4],[-3,-1],[-5,-3],[-4,-1],[-3,3],[2,2],[3,0],[2,2],[-3,2],[-2,1],[-4,0],[-2,-2],[1,0],[1,-1],[-5,-4],[-6,-1],[-6,1],[-4,4],[3,3],[-7,2],[-3,1],[4,0],[2,1],[1,1],[-4,3],[-10,5],[-4,0],[0,-5],[-1,-2],[-3,-1],[-2,-2],[1,-1],[2,-1],[2,0],[2,1],[2,2],[1,1],[0,1],[2,0],[2,-1],[2,-4],[2,-6],[-1,-5],[-1,-3],[-2,1],[-2,2],[-8,-2],[-6,-4],[-1,-2],[-2,0],[-19,-2],[-7,3],[-7,5],[-2,3],[-1,0],[-5,2],[-3,3],[-2,1],[-5,-2],[-5,1],[-4,2],[-5,5],[-2,0],[-4,0],[-2,1],[-2,2],[2,4],[2,-1],[2,-2],[2,-1],[-1,4],[-3,2],[-16,7],[-3,1],[-1,-1],[-4,-2],[-1,-1],[-9,1],[-1,1],[0,2],[0,2],[2,1],[1,0],[-1,4],[-2,3],[-2,1],[-3,-2],[-1,-4],[-2,-2],[-17,0],[-5,2],[-5,6],[2,3],[9,2],[0,2],[-10,3],[-2,0],[0,-3],[0,-2],[0,-1],[-1,-2],[-1,-2],[-1,-1],[-2,-1],[-1,-1],[0,-1],[1,-3],[-2,0],[-5,1],[-5,0],[-2,1],[-3,2],[-12,3],[-9,0],[-2,1],[-8,8],[-5,3],[-27,8],[0,6],[-4,3],[-5,2],[-2,0],[-2,-1],[-6,0],[-2,1],[-1,2],[0,3],[-1,2],[-4,1],[-9,-2],[-3,2],[-4,5],[-4,3],[-17,5],[-6,4],[-5,1],[-4,2],[-4,2],[-3,4],[-4,6],[-2,2],[-5,4],[-2,2],[-1,4],[2,8],[-2,2],[-2,0],[-5,-1],[-3,2],[-3,5],[-2,2],[-2,-2],[10,-9],[1,-2],[1,-1],[0,-2],[1,-2],[0,-2],[0,-5],[0,-1],[3,-2],[4,-8],[2,-2],[2,0],[1,0],[1,-2],[0,-2],[-5,-2],[1,-2],[2,0],[3,0],[0,-2],[-6,-1],[-19,0],[1,-3],[0,-2],[-1,-2],[-1,-2],[2,0],[4,1],[1,-1],[2,-2],[1,-4],[1,-3],[2,-3],[0,-2],[0,-3],[1,-1],[13,-8],[2,-1],[6,1],[4,-2],[7,-6],[4,0],[-1,2],[-4,1],[-1,2],[0,2],[2,0],[7,-3],[3,0],[7,2],[0,2],[-8,0],[-2,1],[-3,3],[-2,1],[-1,1],[-4,7],[-6,7],[1,1],[1,0],[1,0],[-2,3],[-2,2],[6,5],[8,-4],[13,-12],[8,-2],[2,-1],[3,-5],[2,0],[8,-2],[12,-7],[16,-2],[15,-12],[3,-1],[2,2],[3,4],[3,4],[4,0],[4,-2],[3,-4],[2,-10],[1,-2],[13,-1],[4,1],[1,0],[1,-1],[3,-3],[1,-1],[9,-3],[3,-3],[-2,-2],[-5,0],[-3,-2],[-2,-4],[1,-2],[2,-2],[2,-2],[-4,-1],[-9,-4],[-3,2],[2,2],[-2,3],[-2,0],[-2,-2],[-1,-3],[-2,-6],[-2,-2],[-2,-2],[-14,-6],[-2,-2],[9,-7],[1,0],[1,0],[1,5],[2,1],[6,-1],[1,1],[1,0],[1,1],[0,1],[0,1],[0,2],[1,1],[3,0],[6,-8],[3,0],[1,2],[1,5],[1,5],[-1,4],[3,3],[7,0],[3,3],[2,3],[1,0],[4,-1],[1,1],[5,7],[3,2],[7,2],[6,4],[4,2],[3,-1],[13,-9],[4,-5],[1,-1],[0,-1],[0,-3],[0,-3],[0,-2],[0,-2],[1,-3],[1,-2],[2,-1],[5,0],[1,-1],[3,-3],[12,-8],[2,-4],[-1,-1],[-2,-2],[2,-3],[2,-1],[2,1],[2,2],[2,1],[17,-2],[2,-2],[6,-6],[2,-2],[2,0],[1,2],[2,2],[2,1],[3,-2],[3,-2],[4,-6],[16,-8],[-2,-2],[-3,1],[-14,7],[-2,0],[1,-3],[2,-3],[3,-1],[5,-2],[6,-4],[3,-1],[10,0],[2,-2],[1,-5],[-2,-2],[-4,0],[-2,-1],[2,-4],[-1,-3],[1,-2],[3,-1],[1,-2],[-2,-3],[0,-2],[1,-2],[2,-3],[0,-2],[0,-2],[0,-2],[-1,-2],[0,-1],[0,-1],[0,-1],[-2,1],[-1,0],[-1,-1],[1,-3],[4,-1],[2,-1],[0,-3],[-1,-1],[-1,-1],[-1,-2],[1,-3],[4,-9],[-1,0],[-3,-1],[1,-3],[4,0],[2,-2],[-2,-1],[-2,-6],[-3,2],[-1,-1],[-1,-3],[-2,-3],[4,0],[-1,-2],[-2,-1],[-4,0],[-1,-1],[1,-2],[2,-2],[1,-2],[-1,-1],[-2,0],[-4,0],[2,-3],[-2,-2],[-4,-2],[-24,3],[-2,-1],[-5,-6],[-2,-1],[-3,0],[-2,-1],[-5,-4],[-1,-5],[-1,-3],[-10,-8],[-2,0],[-8,0],[-3,-2],[2,-1],[4,-2],[2,-2],[-5,-3],[-5,0],[-25,5],[-5,4],[-4,1],[-10,0],[-17,4],[-12,6],[-5,1],[0,-1],[0,-1],[-1,-4],[2,-3],[3,-2],[1,-3],[-1,-3],[-4,-1],[-3,1],[-8,6],[-6,3],[-6,1],[-11,-3],[-2,-2],[-2,-3],[1,-5],[3,-2],[3,-3],[1,-2],[4,-8],[5,-4],[10,-5],[0,-2],[-2,0],[-2,1],[-1,1],[-2,-2],[1,-2],[9,-2],[26,-3],[8,-4],[1,-2],[0,-3],[0,-4],[0,-3],[-1,-3],[-2,-1],[-3,-3],[-1,-2],[1,-3],[-2,-1],[-2,-1],[-5,0],[-8,4],[-7,-1],[-2,1],[-1,3],[1,3],[4,2],[-1,3],[-2,3],[-2,2],[-4,-4],[-4,1],[-3,-2],[4,-5],[-2,-3],[-3,1],[-3,1],[-3,0],[1,-1],[1,-3],[1,-1],[-9,-2],[-4,0],[-3,2],[-2,2],[0,3],[2,5],[-2,2],[-2,-1],[0,-3],[-1,-2],[-2,1],[-1,0],[0,2],[-5,5],[-2,2],[-2,2],[-1,0],[-1,-3],[2,-3],[2,-2],[2,-2],[-1,-5],[2,-1],[1,-1],[1,-2],[1,-3],[-2,-1],[-1,-1],[0,-2],[0,-3],[0,-2],[2,1],[2,2],[1,2],[2,0],[0,-4],[-1,-2],[-1,-1],[-1,0],[-2,-2],[1,-1],[0,-1],[0,-1],[0,-1],[0,-2],[0,-1],[1,-1],[-2,1],[-5,4],[-3,1],[-6,-1],[-4,1],[-2,-1],[0,-4],[0,-4],[0,-2],[1,-3],[0,-3],[0,-3],[2,-1],[2,0],[0,-1],[-3,-1],[-3,-2],[-2,-1],[-5,0],[-2,0],[-2,-1],[3,-2],[7,0],[9,-4],[3,-1],[2,-1],[2,-3],[1,-3],[0,-1],[0,-1],[-1,-2],[0,-2],[0,-1],[-4,-3],[1,-1],[3,-2],[1,-1],[0,-3],[1,-5],[1,-2],[2,-3],[7,-4],[2,-3],[1,-2],[-3,1],[-7,4],[-5,-2],[-2,2],[-1,5],[0,5],[-7,5],[0,1],[0,1],[1,1],[-1,2],[-1,3],[-1,2],[-2,2],[-2,2],[0,1],[-2,0],[-1,-2],[-1,0],[-2,1],[-2,1],[-2,2],[-2,2],[-2,1],[-5,-1],[-4,3],[-21,5],[-14,6],[8,-6],[4,-2],[8,-2],[10,-8],[4,-2],[4,0],[0,-1],[0,-1],[0,-2],[0,-1],[1,0],[3,0],[4,-1],[4,-2],[3,-3],[2,-6],[-2,0],[-1,-3],[0,-4],[-1,-2],[-3,0],[-2,0],[-2,1],[0,1],[-1,2],[-1,1],[-1,1],[-1,0],[-5,-2],[-4,2],[-2,0],[-2,-2],[5,-3],[-2,-2],[-3,-1],[-7,2],[-1,-1],[-1,-2],[-2,-1],[0,-1],[-1,-2],[-1,-5],[5,-3],[2,0],[6,2],[3,1],[4,-1],[3,-2],[-2,-4],[-2,-4],[-3,-3],[-2,-1],[-4,3],[-3,0],[-1,-3],[1,0],[0,-1],[1,-2],[-1,-2],[-1,-1],[-1,-1],[-3,0],[-2,3],[-1,1],[-1,0],[-2,-2],[1,-1],[0,-2],[1,-2],[-1,-4],[0,-3],[-2,-3],[-2,0],[-2,2],[-4,10],[-2,2],[-3,2],[-3,1],[-2,0],[1,-2],[2,-2],[1,-2],[0,-2],[-13,3],[0,-2],[22,-16],[0,-2],[-17,1],[-2,-1],[5,-4],[2,-1],[2,-1],[1,-2],[1,-3],[1,-2],[-3,-1],[-3,1],[-18,10],[-12,4],[-2,-1],[2,-4],[10,-8],[2,-3],[-10,-1],[-3,-1],[-1,-5],[2,-1],[11,0],[1,-2],[1,-2],[2,-1],[13,-3],[-2,-2],[-3,0],[-5,0],[0,-2],[2,-1],[3,-3],[1,-1],[6,0],[5,2],[2,0],[2,-1],[1,-1],[-2,-5],[0,-5],[-1,-2],[-1,2],[-1,1],[-2,1],[-7,0],[-4,2],[-4,3],[-2,0],[-2,-2],[2,-2],[2,-3],[2,-3],[3,-1],[7,2],[2,-1],[-14,-10],[-3,-1],[-4,0],[-2,-1],[-19,-16],[-3,-3],[-5,-8],[-3,-4],[-1,0],[-2,2],[-1,2],[-2,1],[0,-3],[-5,-10],[-2,-3],[-1,-2],[-2,-1],[-1,-1],[-6,-8],[2,-1],[0,-1],[0,-1],[0,-2],[0,-2],[1,-2],[1,-2],[2,-1],[-3,-1],[-1,0],[5,-3],[2,-1],[1,1],[2,0],[1,0],[2,-1],[3,-6],[1,-3],[-3,-2],[1,-2],[0,-1],[-2,-3],[-8,-3],[-3,-1],[-5,1],[-9,5],[-4,1],[2,-4],[5,-5],[2,-3],[0,-2],[1,-5],[0,-1],[1,-1],[1,-1],[0,-2],[1,-1],[-1,0],[-3,-2],[2,-4],[-3,-3],[-5,-3],[-2,-3],[7,0],[-2,-5],[-1,-2],[0,-3],[1,-3],[2,0],[3,2],[2,-1],[1,-3],[-1,-3],[-2,-1],[-2,-1],[-5,0],[-1,0],[-3,-2],[-1,0],[-1,-1],[0,-2],[-2,-3],[-2,-3],[0,-5],[-1,-5],[-1,-2],[-4,-9],[-2,-4],[0,-3],[0,-2],[-1,-2],[-1,-2],[-2,-1],[-3,-1],[-3,-3],[-2,-2],[-1,-2],[-1,-1],[0,-1],[0,-3],[-1,-2],[-1,0],[-3,-10],[-1,-5],[1,-4],[1,-5],[-2,-5],[-7,-10],[-1,-2],[-1,-2],[0,-3],[0,-2],[0,-1],[-1,-3],[-2,-4],[0,-3],[0,-1],[1,-1],[0,-2],[-1,-3],[0,-2],[-6,-4],[-13,6],[-6,-4],[3,-2],[5,-2],[3,-2],[-3,-4],[-11,2],[-4,-2],[11,-3],[3,-2],[5,-13],[1,-2],[1,-4],[0,-1],[0,-3],[-2,-5],[0,-2],[-1,-3],[-4,-7],[-1,-5],[-3,-6],[0,-2],[1,-2],[1,-1],[0,-1],[1,-5],[0,-5],[0,-5],[-1,-4],[-1,0],[-1,-2],[-1,-1],[0,-2],[1,-1],[0,-3],[1,-2],[0,-2],[0,-3],[-2,-4],[-4,-2],[-2,-4],[1,0],[1,-2],[-1,-1],[0,-1],[-1,-1],[0,-2],[2,-1],[-3,-1],[-2,-1],[-1,-2],[1,-1]],[[5122,7512],[-2,-1],[-3,1],[-3,2],[-6,11],[7,5],[1,0],[2,-2],[3,-4],[2,-2],[1,-4],[-1,-3],[-1,-3]],[[5011,7499],[-3,-6],[-2,-1],[-2,0],[-6,3],[-2,1],[-9,-1],[-3,1],[-3,2],[-4,7],[7,3],[3,3],[-2,6],[3,3],[7,5],[7,6],[3,2],[4,1],[4,-2],[0,-4],[-1,-3],[-3,-7],[6,-4],[3,-4],[0,-5],[-7,-6]],[[5005,7586],[4,0],[3,2],[3,3],[3,2],[1,-4],[7,-10],[1,-8],[-1,-4],[-2,-2],[-2,-3],[-1,-2],[-1,-2],[0,-4],[-1,-1],[-1,-2],[-1,-1],[-6,-6],[-8,-2],[-17,-2],[-5,1],[-10,9],[-4,7],[-1,6],[-3,6],[-1,4],[0,4],[1,4],[2,4],[3,3],[2,4],[3,4],[8,3],[8,5],[5,1],[6,0],[4,-2],[1,0],[2,-2],[2,-3],[-1,-5],[-4,-5],[1,-2]],[[2732,6750],[1,-1],[2,-1],[3,-1],[3,-1],[3,0],[3,0],[3,0],[4,0],[3,-1],[-3,6],[-5,5],[-4,4],[-4,3],[-8,8],[-7,8],[-8,8],[-7,8],[-2,3],[-3,3],[-2,2],[-3,3],[-2,2],[1,5],[1,9],[0,5],[1,8],[12,0],[5,0],[16,0],[15,0],[16,0],[65,0],[65,0],[66,0],[65,0],[28,0],[28,0],[28,0],[28,0],[8,0],[8,0],[9,0],[8,0],[2,0],[2,0],[2,0],[2,0],[1,-1],[1,-2],[-1,-3],[0,-5],[0,-6],[0,-7],[0,-6],[0,-7],[1,-3],[0,-1],[1,0],[2,1],[6,0],[5,0],[6,0],[6,0],[0,5],[2,1],[3,-2],[3,-1],[3,0],[4,0],[3,1],[3,0],[1,0],[2,1],[1,1],[1,1],[2,1],[1,1],[-1,2],[-1,2],[-2,1],[-2,1],[-3,-1],[-2,0],[-5,-1],[-4,1],[-5,1],[-5,1],[3,3],[5,0],[5,0],[5,-1],[0,6],[0,5],[0,5],[0,6],[52,0],[52,0],[53,0],[52,0],[19,0],[18,0],[18,0],[19,0],[-1,45],[0,33],[0,34],[0,55],[0,56],[0,56],[0,56],[0,31],[0,31],[0,50],[0,36],[0,36],[0,29],[0,45]],[[3507,7428],[3,1],[13,-4],[3,-3],[3,-1],[3,1],[-2,4],[-21,9],[-2,2]],[[3507,7437],[0,7]],[[3507,7444],[1,-1],[12,-7],[4,-1],[4,1],[4,3],[-2,2],[-3,5],[-1,1],[-2,1],[-2,1],[-4,3],[-3,4],[-1,4],[-2,4],[-5,5]],[[3507,7469],[0,20]],[[3507,7489],[15,-9],[4,-1],[2,0],[0,2],[-1,2],[-2,2],[-1,2],[2,3],[2,0],[5,0],[-3,4],[-12,0],[-5,2],[-5,3],[-1,0]],[[3507,7499],[0,44],[0,12]],[[3507,7555],[11,-4],[2,-1],[1,-2],[0,-2],[0,-2],[0,-1],[3,-3],[3,0],[8,3],[8,-1],[7,-4],[-2,-4],[-10,-1],[-2,-3],[42,-27],[-1,-1],[-2,-3],[25,3],[2,-1],[4,-4],[1,0],[-1,-1],[-2,-1],[-1,-1],[-1,-2],[12,-9],[13,-3],[0,-2],[-2,-2],[-8,-2],[1,-4],[0,-2],[-2,-2],[-1,-2],[2,-3],[1,-1],[-1,-1],[-1,-2],[-1,-1],[3,-2],[10,3],[8,-2],[3,1],[-2,2],[-1,1],[-3,1],[3,3],[22,-6],[4,-3],[2,-4],[1,-5],[1,-3],[1,-3],[0,-5],[-1,-2],[-2,-1],[-2,-2],[0,-2],[0,-3],[-1,-5],[-1,-7],[-1,-4],[0,-4],[0,-5],[2,-3],[4,-6],[2,-3],[6,-21],[0,-2],[0,-6],[0,-3],[0,-2],[2,-2],[4,0],[3,1],[0,4],[2,1],[1,-1],[2,-1],[1,-2],[-1,-2],[-1,-1],[-1,0],[-1,0],[0,-2],[3,0],[1,-3],[0,-4],[1,-3],[0,-5],[-1,-2],[1,-3],[-1,-4],[-1,-9],[7,0],[3,0],[4,-4],[-4,-8],[-2,-1],[3,0],[7,6],[3,2],[4,-2],[0,-3],[-3,-9],[-3,-3],[-4,-2],[-2,-2],[2,-5],[1,-2],[2,-1],[1,-1],[0,-3],[-1,-2],[-4,-6],[3,-4],[3,-3],[3,-1],[3,-1],[1,0],[0,-2],[0,-2],[0,-2],[-2,-1],[-6,-3],[5,-4],[12,4],[5,-1],[4,-4],[3,-4],[1,-6],[-1,-6],[8,-1],[7,-5],[3,0],[2,1],[1,0],[2,1],[2,-1],[3,1],[2,4],[-7,7],[-2,1],[-2,1],[-1,2],[-1,3],[0,4],[4,6],[5,-2],[6,-4],[4,0],[5,5],[3,6],[4,5],[5,2],[30,5],[-3,3],[-3,2],[-8,2],[-2,1],[-5,6],[-4,2],[1,1],[3,3],[1,0],[10,-4],[4,1],[4,2],[2,4],[-2,2],[-7,2],[-7,3],[-2,0],[0,4],[0,4],[-1,4],[-2,1],[-5,1],[-9,5],[-5,1],[-1,2],[-2,4],[-2,5],[0,4],[1,2],[3,1],[0,3],[0,1],[-4,3],[-1,1],[1,1],[2,1],[-16,3],[0,4],[0,2],[-1,2],[0,1],[0,12],[-1,6],[-2,3],[-3,3],[-3,5],[1,2],[2,0],[3,0],[4,-4],[3,-1],[3,1],[1,3],[-2,6],[-5,3],[-5,2],[-3,3],[4,2],[1,1],[0,4],[-2,2],[-3,1],[-2,3],[-1,5],[7,2],[3,5],[-6,1],[-6,1],[-6,4],[-5,7],[13,2],[-1,3],[-1,2],[0,3],[2,4],[-3,5],[-4,3],[-9,3],[1,2],[2,1],[2,1],[2,1],[-7,5],[-1,4],[2,6],[-1,2],[-3,3],[-1,2],[-3,4],[-1,3],[2,1],[1,2],[-1,6],[-4,10],[-3,5],[-1,2],[0,5],[2,1],[2,2],[0,2],[-2,3],[-4,3],[-1,2],[0,3],[0,3],[2,3],[1,1],[-3,5],[-7,7],[-1,2],[1,1],[4,-1],[2,1],[2,1],[-12,12],[-4,8],[4,10],[11,7],[11,1],[23,-5],[-4,6],[-17,6],[-2,1],[-4,4],[-2,2],[0,2],[0,3],[0,1],[-1,1],[-2,1],[-1,1],[4,3],[12,-1],[-2,3],[-4,4],[-2,3],[8,0],[25,-7],[15,1],[16,-8],[8,-2],[13,-9],[5,-2],[3,-4],[4,-2],[6,-6],[12,-4],[3,1],[4,4],[-3,3],[-11,8],[4,4],[1,3],[0,3],[3,1],[4,3],[2,1],[2,0],[4,-1],[1,0],[3,3],[5,1],[6,-2],[5,-3],[4,-4],[4,-9],[4,-3],[12,-4],[3,2],[0,1],[0,2],[0,1],[1,0],[1,0],[2,-1],[1,-1],[1,-2],[1,-2],[0,-4],[1,-2],[1,-1],[4,-2],[2,-5],[3,-2],[5,-1],[6,-6],[2,-1],[0,-1],[2,-4],[2,-2],[6,-1],[14,-7],[5,-1],[5,1],[2,1],[1,1],[1,1],[3,-1],[6,0],[2,-2],[0,-5],[0,-4],[1,-2],[3,-2],[5,-5],[16,-7],[-3,-3],[2,-2],[4,-3],[2,-2],[1,-1],[1,-2],[1,-2],[2,-1],[-1,-2],[-1,-1],[-1,0],[-1,-1],[0,-1],[3,0],[4,-1],[4,-3],[3,-3],[5,-8],[3,-2],[-1,-1],[-1,-2],[-2,-2],[5,-2],[3,-3],[1,-3],[-3,-2],[-2,0],[-2,0],[-1,2],[-2,3],[-3,3],[-3,3],[-2,3],[-4,-2],[10,-12],[-1,-1],[-1,-1],[-1,-2],[-1,-2],[12,-7],[5,-2],[1,-1],[1,-2],[1,-2],[-1,-2],[-6,-4],[2,-3],[1,-3],[1,-4],[-1,-5],[8,-1],[2,-2],[0,-1],[-1,-2],[0,-1],[1,-2],[1,-1],[-3,-4],[1,-2],[5,-2],[-1,-6],[-1,-1],[-1,-1],[-3,1],[-2,1],[-2,1],[-2,-2],[3,-4],[4,-9],[3,-5],[7,-6],[2,-3],[0,-2],[0,-4],[1,-1],[0,-2],[4,-7],[3,-4],[2,-2],[2,-6],[2,-3],[3,-2],[2,-3],[1,-4],[-1,-4],[-2,-3],[-5,-4],[-1,-4],[1,-2],[2,-2],[0,-3],[-2,-3],[2,-1],[5,-1],[2,-2],[1,-2],[1,-5],[1,-1],[2,-2],[2,-4],[5,-13],[1,-3],[1,-4],[0,-6],[1,-7],[3,-4],[4,-2],[8,-2],[2,-1],[1,-3],[1,-2],[0,-3],[1,-2],[1,-1],[3,-2],[2,-3],[5,-7],[0,-2],[1,-4],[1,-2],[1,-2],[4,-3],[11,-18],[2,-6],[-1,-5],[-4,-11],[2,-4],[2,-3],[6,-3],[-2,-7],[-1,-4],[1,-4],[4,-6],[2,-4],[0,-4],[-2,-1],[-12,6],[-3,-1],[-3,-1],[-3,-3],[-1,-5],[2,-7],[3,-6],[1,-6],[-2,-9],[-2,-3],[-7,-9],[-7,-3],[-3,-3],[-2,-4],[1,-2],[2,-2],[2,-5],[1,-2],[33,-25],[8,-1],[3,-3],[7,-6],[4,-8],[10,-27],[2,-5],[4,-3],[3,-1],[7,1],[2,-1],[2,-1],[7,-7],[3,-2],[4,-3],[3,-4],[4,-2],[1,-2],[6,-10],[3,-3],[4,-2],[25,0],[5,-2],[3,-3],[2,-4],[5,-4],[5,-8],[19,-4],[5,1],[0,5],[-3,3],[-9,1],[-4,3],[-2,3],[-3,1],[-2,3],[-1,5],[0,3],[-1,5],[-1,4],[-1,3],[3,5],[6,-1],[11,-6],[3,-4],[2,-4],[2,-1],[4,-2],[3,-2],[2,-4],[1,-5],[-2,-6],[0,-5],[15,-5],[2,-2],[5,-1],[4,-4],[7,-2],[8,-5],[9,-3],[18,-12],[7,-10],[1,0],[1,0],[2,1],[2,1],[1,1],[2,-2],[2,-3],[1,-1],[5,-2],[10,-10],[4,-2],[7,0],[3,2],[2,7],[2,1],[11,1],[4,-2],[1,-2],[2,-2],[3,-1],[1,0],[2,-2],[-2,-2],[-3,-4],[-2,-5],[-1,-5],[1,-5],[2,-5],[5,-7],[3,-4],[3,-1],[1,1],[1,2],[1,0],[1,1],[8,-1],[2,2],[1,4],[2,6],[5,-1],[9,-7],[2,-1],[1,0],[2,2],[6,2],[1,1],[2,3],[1,3],[-1,3],[3,1],[5,-1],[4,-2],[3,-3],[2,-3],[0,-4],[0,-3],[-2,-9],[0,-2],[1,-2],[4,-9],[3,-6],[-1,0],[-1,-2],[1,-4],[1,-2],[0,-2],[-1,-3],[-2,-3],[-1,-3],[0,-3],[1,-2],[1,-1],[2,0],[2,-1],[1,-1],[1,-2],[1,-3],[-1,-3],[2,-16],[-1,-6],[-1,-5],[-3,-3],[-3,-1],[0,-2],[3,-4],[-1,-5],[-4,-8],[-2,-1],[-8,2],[-3,-2],[-2,0],[-2,2],[-3,3],[-2,2],[-1,-1],[0,-1],[-10,-3],[-5,0],[-4,2],[-3,4],[-2,7],[0,2],[-6,8],[-4,14],[-1,2],[2,4],[3,8],[0,5],[-1,2],[-2,-1],[-2,-3],[-9,-23],[-1,-8],[-1,-5],[-1,-3],[-7,-9],[-1,-3],[-3,-9],[-1,3],[0,2],[0,2],[-1,2],[-1,2],[-17,14],[-1,0],[-2,-1],[-2,-2],[-2,-1],[-2,2],[-5,8],[-3,3],[-1,2],[-1,2],[0,2],[0,3],[-1,6],[-2,2],[-2,-1],[-5,-4],[-2,-1],[-3,0],[-1,1],[-1,2],[-1,3],[0,2],[2,5],[-2,4],[-2,3],[-3,2],[-3,-2],[0,-5],[2,-10],[-7,0],[-3,-1],[-3,-4],[4,-3],[1,-2],[1,-3],[-1,-3],[-2,-2],[-1,-2],[-2,-1],[-3,0],[0,1],[1,2],[0,3],[-1,3],[-1,3],[-1,1],[-1,-3],[0,-3],[0,-2],[-1,-2],[-2,0],[-3,3],[-1,0],[-1,0],[-1,0],[-1,-1],[-4,-6],[-2,-2],[-4,-2],[-1,0],[-2,0],[-2,2],[-2,2],[-2,2],[-2,0],[-1,-3],[8,-10],[4,-4],[10,-2],[0,-5],[-2,-8],[1,-9],[0,-1],[-1,-1],[0,-1],[-1,0],[0,-1],[1,-3],[0,-1],[-3,-3],[-4,-1],[-7,-1],[-7,2],[-2,-1],[-8,-3],[-4,0],[-6,5],[-16,6],[-1,2],[-8,1],[-7,5],[-8,8],[-5,8],[-3,3],[-3,-1],[-3,-2],[-4,0],[0,1],[-1,2],[-1,2],[-2,1],[-2,0],[-6,-3],[1,-4],[0,-5],[-3,-9],[7,-2],[13,-9],[12,-5],[4,-5],[3,-7],[3,-13],[1,-2],[2,0],[2,4],[0,4],[-1,5],[0,5],[2,2],[3,-1],[2,-1],[1,-3],[1,-5],[1,-2],[0,-5],[0,-1],[1,-2],[1,-3],[1,-2],[-2,-3],[-2,-7],[-2,-3],[2,-6],[1,-4],[-1,-11],[-2,-5],[-4,-5],[-14,-7],[-1,-2],[-1,-2],[0,-1],[1,-2],[0,-2],[-1,-3],[0,-2],[0,-3],[0,-2],[2,-3],[2,-1],[1,1],[2,2],[0,2],[1,5],[1,3],[2,1],[4,5],[4,2],[3,5],[8,18],[2,2],[4,3],[2,3],[0,2],[0,3],[0,3],[1,1],[1,0],[1,1],[1,1],[0,2],[1,1],[15,1],[4,3],[4,2],[10,0],[3,4],[4,10],[2,2],[9,1],[4,2],[5,2],[3,1],[7,-3],[15,0],[3,-2],[10,-10],[3,-5],[0,-4],[-11,-11],[-9,2],[-5,-2],[-1,-1],[1,-2],[-1,0],[0,-5],[0,-2],[1,0],[3,3],[1,0],[2,1],[1,-1],[0,-1],[1,-3],[2,-3],[3,-1],[1,-2],[-2,-5],[-2,-5],[-2,-2],[-2,2],[-3,4],[-2,3],[-3,2],[-3,-1],[-3,-4],[1,-1],[1,-2],[0,-3],[-1,-2],[2,-1],[3,-2],[2,-1],[1,0],[2,-5],[2,-2],[3,-1],[3,2],[1,6],[1,4],[3,3],[3,2],[5,2],[3,1],[2,-2],[2,-4],[4,-5],[5,-2],[4,-4],[2,-8],[-1,-9],[-4,-7],[-7,-12],[1,-2],[2,-2],[2,-2],[1,-3],[-1,-3],[-3,-1],[-13,-5],[-3,1],[-2,2],[-2,2],[-2,2],[-2,1],[-2,-1],[-2,-2],[-4,-5],[-2,-3],[-2,-1],[-6,1],[-11,-4],[-3,1],[-5,3],[-3,0],[1,-4],[2,-3],[-1,-3],[-2,-2],[-1,-1],[-3,-2],[-2,-1],[-1,-2],[-2,-2],[-4,-2],[-12,0],[-1,1],[-6,5],[-4,2],[-2,0],[-2,-2],[1,-4],[3,-3],[3,-2],[-1,-3],[-3,-2],[-10,0],[-3,-1],[-7,-6],[-6,-1],[-5,1],[-7,4],[-4,1],[-1,1],[-1,2],[-1,1],[-3,1],[-19,-3],[-2,-2],[-1,-3],[-1,-2],[1,-3],[-3,-2],[-4,0],[-3,2],[-6,4],[-5,5],[-3,1],[-1,-1],[-3,-2],[-2,-1],[-11,3],[-5,3],[-2,0],[-6,0],[-3,1],[-8,4],[-3,1],[-12,-2],[-4,0],[-11,6],[-3,3],[-1,2],[-1,1],[0,1],[-1,4],[0,2],[0,1],[-2,2],[-4,-2],[1,-2],[2,-3],[1,-3],[-4,0],[-7,5],[-9,3],[-2,-1],[0,-1],[-1,-4],[0,-1],[-7,-9],[-7,-2],[-30,4],[-30,3],[-3,2],[-5,7],[-3,2],[-4,3],[-2,1],[3,0],[3,2],[3,2],[2,5],[4,2],[9,-4],[4,4],[-2,3],[-1,1],[10,3],[6,4],[-1,7],[-2,2],[-4,2],[-12,3],[-2,0],[-3,-3],[2,0],[3,-2],[3,-2],[2,-3],[-20,4],[-5,3],[-23,2],[-1,1],[-2,5],[-2,2],[-3,4],[-2,1],[-38,5],[-5,-2],[-4,-4],[-2,-1],[-3,0],[-7,3],[-15,-2],[-14,7],[-10,2],[0,1],[0,3],[1,5],[1,3],[1,2],[11,6],[2,4],[-1,6],[-1,2],[-1,1],[-1,1],[0,3],[-1,11],[0,2],[0,1],[0,1],[-2,2],[-2,2],[-2,2],[-3,1],[-2,0],[-3,2],[-8,12],[-4,2],[-4,1],[-4,-2],[-5,-6],[-4,-3],[-1,-2],[-1,-1],[-2,1],[-1,-1],[0,-2],[0,-1],[0,-1],[0,-4],[0,-3],[0,-3],[-1,-2],[-2,-1],[-17,-1],[-5,-2],[-3,-4],[0,-5],[4,-7],[1,-8],[1,-2],[1,-4],[0,-3],[-1,-3],[-2,-2],[-1,-1],[-7,-1],[-2,-2],[1,-3],[1,-3],[0,-3],[-2,1],[-7,2],[-7,-7],[-12,-19],[-7,-7],[-16,-8],[-46,-9],[-7,-4],[-32,-2],[-11,5],[-24,-1],[-3,-3],[2,-3],[1,-2],[0,-2],[0,-3],[-1,-2],[-2,-2],[-5,-3],[-7,-7],[-15,-10],[-7,-3],[-7,-7],[-3,-3],[-9,-3],[-36,-7],[-31,-14],[-18,-2],[-14,1],[-9,-3],[-19,2],[-8,-1],[-8,-6],[-6,-6],[-3,-1],[-3,3],[-2,4],[-3,2],[-14,2],[-4,-1],[-6,-7],[-4,-1],[-8,0],[-24,-8],[-7,3],[1,2],[3,2],[18,4],[-1,2],[-2,1],[-2,1],[-15,-2],[-7,-5],[-3,-1],[-29,2],[-6,-2],[5,-4],[13,-1],[6,-2],[-5,-7],[-11,0],[-21,5],[-40,-3],[-41,-3],[-2,0],[-1,2],[-2,0],[-4,-3],[-3,-1],[-4,-1],[-22,1],[-2,0],[-1,-1],[-4,-6],[-1,-1],[-2,-1],[-2,1],[-6,-3],[-17,3],[-39,-5],[-5,1],[-2,4],[0,4],[5,3],[5,0],[17,-4],[3,2],[-2,3],[-6,7],[-4,3],[-9,3],[-7,5],[-8,2],[-1,1],[-3,5],[-1,1],[-2,2],[-3,4],[-1,6],[1,4],[-1,3],[-2,3],[-9,8],[-2,2],[-2,3],[-1,3],[-5,3],[-2,2],[-1,4],[-1,3],[1,2],[3,1],[2,1],[-1,4],[-5,5],[-1,4],[1,3],[4,4],[3,4],[2,2],[1,1],[1,2],[0,1],[-1,2],[0,5],[1,5],[1,3],[-8,0],[1,3],[3,1],[6,-1],[-2,3],[-8,5],[-2,3],[0,1],[-1,4],[0,2],[0,1],[-1,0],[0,1],[0,1],[1,0],[3,2],[-6,4],[-1,2],[0,2],[1,1],[0,2],[0,2],[2,0],[4,-2],[15,-2],[-4,3],[-5,3],[-34,7],[-1,2],[-3,3],[-3,2],[-9,-2],[-12,0],[-8,4],[-4,3],[-2,1],[-16,1],[-52,-9],[-5,3],[-3,3],[-1,1],[-8,1],[-2,-1],[-2,-2],[-4,-4],[-4,-2],[-8,-1],[-45,7],[-44,7],[-2,1],[-2,2],[-1,2],[-2,2],[-2,2],[-11,1],[-32,15],[-11,2],[-12,12],[1,4],[3,2],[5,2],[-2,3],[-2,1],[-2,2],[0,4],[2,2],[1,2],[-2,3],[-2,1],[-17,2],[2,-2],[5,-5],[-3,-1],[-3,1],[-6,6],[-11,4],[-2,4],[6,3],[3,3],[-1,4]],[[3778,7698],[-2,-1],[-6,0],[-6,-3],[-30,-4],[-3,2],[-1,1],[-2,4],[-4,2],[0,2],[8,8],[19,2],[19,-2],[11,-6],[-1,-3],[-2,-2]],[[4119,7634],[-9,-10],[-9,-13],[-1,-4],[3,-3],[-3,-4],[-9,-4],[-3,-3],[-6,-9],[-8,-6],[1,-11],[-4,-5],[-11,-6],[-10,-11],[-3,-2],[-4,1],[-3,3],[-4,2],[-4,-3],[4,-1],[3,-3],[0,-2],[-4,-2],[7,-6],[6,-4],[2,-2],[-6,-1],[-1,-1],[-1,-1],[-1,0],[-2,0],[0,2],[-1,2],[-1,1],[-2,0],[-6,-2],[0,2],[1,2],[0,2],[-5,1],[-2,2],[-2,4],[0,4],[-1,2],[-2,3],[-27,22],[-7,8],[-3,5],[-1,2],[-5,2],[-2,1],[-3,5],[0,3],[1,1],[1,1],[-2,1],[-1,-1],[-2,0],[-1,0],[-18,8],[2,1],[3,1],[2,1],[-2,2],[2,1],[-6,6],[-7,4],[-2,1],[-2,4],[2,2],[4,-1],[2,1],[-25,13],[-1,3],[-2,2],[-2,2],[-3,0],[-1,1],[-2,4],[-2,2],[-6,2],[-12,2],[-6,3],[-4,4],[-9,4],[-23,1],[-6,3],[2,5],[4,6],[3,4],[3,2],[4,1],[4,3],[4,4],[11,17],[7,8],[8,5],[50,3],[50,3],[16,6],[22,2],[24,-3],[5,-2],[2,-1],[3,-1],[11,-10],[13,-7],[3,-3],[2,-5],[16,-2],[14,-8],[7,-7],[1,-8],[-1,-1],[-1,-1],[-1,-3],[0,-5],[-3,-3],[-2,-3],[-2,-4],[2,-4],[1,-2],[-1,-3],[-2,-5],[-1,-2],[1,-5],[0,-4],[0,-3],[-2,-5]],[[6863,7742],[24,-15],[3,0],[2,1],[1,1],[2,4],[1,2],[3,1],[3,-2],[5,-4],[34,-7],[5,-4],[17,-7],[54,0],[2,0],[6,5],[7,2],[4,0],[5,-2],[35,1],[36,2],[3,-1],[5,-3],[8,-2],[15,-9],[49,-12],[3,-1],[6,-8],[3,-1],[12,-1],[1,-1],[1,-1],[1,-1],[4,-4],[1,-1],[-1,-1],[0,-2],[-1,-1],[8,-8],[7,-6],[-2,-2],[-1,-2],[-1,-2],[3,-5],[15,-8],[7,2],[5,-1],[5,-2],[0,1],[1,0],[3,0],[1,-1],[1,-1],[0,-3],[-5,-1],[2,-6],[4,-4],[9,-5],[4,-5],[-1,-3],[-5,-9],[2,0],[11,-9],[3,-2],[6,-1],[6,-2],[13,-2],[3,-2],[-2,-3],[-5,-2],[-1,-3],[1,-1],[1,-1],[1,0],[1,0],[-1,-2],[-1,-1],[-3,-2],[1,-2],[4,-3],[2,-3],[-6,-4],[-2,-3],[8,-6],[16,-1],[7,-9],[-2,-1],[-2,-2],[-1,-3],[2,-3],[-4,-4],[-11,-3],[-11,-5],[-39,3],[-8,3],[-5,0],[-5,-2],[-28,3],[-28,3],[-12,6],[-35,2],[-35,1],[-9,-1],[-6,2],[-53,-11],[-8,-5],[-18,-5],[-8,-5],[-26,-9],[-25,-3],[-20,5],[-29,17],[-14,4],[-3,2],[-3,3],[-3,11],[-1,2],[-1,0],[-1,4],[-1,2],[-5,5],[-2,3],[-3,11],[-2,3],[-1,1],[1,3],[4,9],[1,7],[-1,1],[-4,2],[-1,2],[0,2],[0,2],[0,2],[1,0],[4,1],[2,3],[0,4],[-3,5],[-2,3],[-8,4],[-5,0],[-10,-3],[-9,5],[-12,0],[-18,5],[-7,0],[-3,1],[-2,4],[-2,2],[-3,2],[-1,2],[0,3],[0,2],[1,1],[1,3],[-5,4],[3,3],[0,2],[0,2],[-1,4],[0,3],[0,2],[3,7],[2,3],[2,1],[2,0],[9,-3],[2,1],[-2,3],[-7,2],[-1,1],[-8,9],[0,2],[-2,5],[0,4],[-1,0],[0,1],[-2,3],[7,4],[4,11],[0,12],[-6,6],[0,2],[9,0],[2,2],[2,-1],[8,1],[13,3],[13,0],[12,-3]],[[6159,7765],[21,-2],[40,2],[31,-6],[54,-3],[3,2],[3,-2],[13,-2],[25,-15],[5,-1],[-1,-1],[-4,-7],[-1,-2],[-2,-2],[-5,-6],[-29,-18],[-30,-17],[-9,-1],[-3,-2],[-10,-13],[-8,-4],[-33,-34],[-11,-16],[-10,-21],[-3,-4],[-6,-4],[0,-2],[1,-3],[0,-2],[0,-2],[-2,-2],[-3,-2],[0,-4],[-6,-6],[-2,-3],[-8,-12],[-16,-15],[-7,-11],[-2,-6],[-1,-2],[0,-11],[-1,-2],[-2,-5],[-1,-2],[0,-4],[7,-8],[-3,-4],[3,-5],[40,-34],[6,-9],[3,-11],[-1,-7],[-3,-4],[-3,-2],[-3,-4],[-3,-4],[-5,-7],[-2,-4],[-1,-3],[-1,-2],[0,-6],[-3,-10],[1,-2],[2,-4],[0,-2],[-1,-10],[1,-4],[0,-3],[0,-6],[1,-4],[1,-6],[0,-3],[1,-3],[3,-2],[4,-1],[2,-2],[0,-2],[0,-3],[1,-4],[1,-3],[6,-8],[2,-2],[6,-13],[7,-11],[2,-3],[2,-1],[6,-2],[1,-2],[2,-3],[6,-3],[2,-3],[2,-3],[9,-10],[3,-2],[3,-2],[3,0],[1,-1],[3,-3],[0,-3],[1,-2],[0,-1],[5,-5],[12,-6],[5,-5],[1,-3],[1,-3],[1,-2],[2,-1],[6,2],[2,-1],[0,-1],[0,-2],[0,-1],[2,-2],[3,0],[4,0],[2,-1],[2,-3],[1,-1],[5,0],[2,0],[2,-2],[0,-1],[1,-1],[0,-1],[2,-1],[18,-1],[7,-3],[2,-1],[1,-2],[1,-3],[1,-3],[-1,-2],[-2,-3],[0,-3],[0,-3],[1,0],[1,0],[2,1],[2,-1],[2,-2],[2,-5],[2,-7],[-2,-2],[-38,5],[-10,-3],[3,-4],[3,0],[6,2],[0,-2],[-3,-1],[-7,-4],[-4,-1],[-13,2],[-5,-2],[-2,-4],[3,0],[3,0],[3,0],[1,-5],[-2,-3],[-3,0],[0,2],[-3,2],[-2,0],[-1,-3],[-2,-3],[-2,-2],[-3,1],[-18,7],[-2,-1],[-8,-5],[-5,0],[-5,-2],[-4,1],[-3,0],[-4,-2],[-1,-2],[-4,-5],[-16,-9],[-6,-2],[1,-1],[0,-3],[0,-1],[-11,-6],[-4,-1],[-5,1],[-2,-1],[-1,-2],[-1,-1],[-9,0],[-34,-9],[-2,-2],[-1,0],[19,-4],[42,7],[4,3],[3,1],[3,3],[4,2],[5,3],[26,12],[8,9],[6,2],[25,1],[2,1],[3,3],[2,1],[33,-5],[0,-1],[-1,0],[2,-2],[2,1],[1,1],[1,1],[6,0],[-2,3],[-1,2],[0,2],[12,-5],[3,1],[-3,3],[-1,1],[-2,0],[0,1],[8,2],[2,-1],[3,-3],[1,-1],[2,1],[2,0],[1,-2],[2,-3],[0,-2],[0,-2],[-1,-5],[-1,-2],[0,-8],[-1,-2],[-1,1],[-8,7],[-2,1],[-19,-4],[0,-1],[21,-2],[0,-3],[-1,-2],[-1,-1],[-2,1],[-1,-1],[0,-1],[0,-2],[0,-2],[3,-1],[4,-2],[1,-5],[0,-6],[-2,-2],[-1,-2],[-2,-2],[0,-4],[1,-2],[3,-1],[4,2],[12,0],[1,1],[0,3],[1,3],[1,1],[2,1],[1,2],[-1,6],[-1,3],[-1,1],[-1,2],[1,3],[-4,4],[-1,3],[-1,5],[0,3],[2,7],[1,3],[0,8],[1,3],[1,2],[4,2],[6,2],[4,3],[-11,8],[-5,7],[2,8],[3,12],[2,3],[-2,6],[2,4],[5,2],[12,2],[5,2],[2,6],[-1,5],[-2,10],[-1,5],[-2,2],[-3,1],[-4,4],[-2,7],[2,2],[5,6],[-3,5],[-1,1],[-1,1],[-8,1],[-18,-1],[1,-2],[0,-2],[1,0],[1,-1],[-1,-4],[-2,-1],[-2,0],[-42,8],[-7,4],[-15,18],[-8,3],[1,2],[-2,1],[-4,1],[-1,1],[14,2],[2,1],[0,3],[-1,3],[-2,2],[-3,3],[-6,9],[-2,2],[1,1],[-2,2],[-9,3],[-11,7],[-31,8],[-2,3],[-1,3],[0,2],[2,0],[1,0],[0,-2],[1,-2],[2,-2],[2,-1],[2,-1],[-1,3],[-1,2],[-1,0],[0,3],[0,2],[0,2],[2,0],[2,-1],[2,0],[2,1],[-1,0],[6,0],[17,-5],[-2,2],[-4,2],[-1,3],[5,3],[14,1],[5,0],[2,1],[5,4],[1,0],[1,2],[-2,3],[-4,5],[0,2],[9,0],[4,2],[2,1],[2,-1],[-2,3],[-4,4],[-4,2],[-3,1],[1,5],[0,4],[-2,3],[-3,2],[3,1],[8,-5],[3,0],[-1,4],[-3,2],[-2,2],[-3,0],[3,3],[4,0],[8,-3],[3,0],[5,3],[2,0],[5,-2],[2,0],[-3,-1],[1,-3],[2,-1],[2,1],[2,1],[0,2],[-1,0],[1,2],[3,0],[5,-1],[1,0],[1,-2],[0,-2],[-2,-1],[0,-1],[5,0],[1,1],[0,4],[3,1],[2,-1],[2,-1],[4,-2],[5,-3],[3,-1],[1,0],[2,-1],[2,-1],[0,-4],[1,-1],[17,-12],[9,-1],[4,-3],[1,-1],[4,0],[2,-1],[5,-6],[1,0],[1,-1],[2,-5],[1,-2],[2,-1],[5,0],[1,0],[1,-2],[3,-2],[3,-3],[2,-1],[2,-2],[1,-6],[1,-6],[1,-3],[2,-4],[3,0],[1,3],[0,6],[0,1],[1,0],[0,1],[1,0],[-1,2],[-2,4],[-1,1],[3,3],[-2,1],[-7,7],[-8,2],[-2,1],[-1,2],[-5,8],[-1,2],[-2,0],[2,2],[2,2],[-12,2],[-12,4],[-2,2],[-16,15],[5,0],[2,0],[2,1],[-2,2],[-3,1],[-5,-1],[-1,1],[-6,4],[-10,4],[-3,3],[1,1],[5,3],[0,1],[1,2],[7,8],[3,2],[24,-2],[5,0],[11,6],[-3,2],[-9,-5],[-33,3],[-4,-3],[-1,1],[0,2],[-1,5],[3,1],[3,0],[3,1],[2,3],[-2,0],[3,6],[1,3],[-1,1],[-6,-4],[-7,-2],[-15,-9],[1,-1],[0,-4],[1,-2],[-8,1],[-2,-1],[-3,-4],[-1,0],[-2,1],[1,3],[0,1],[1,1],[1,0],[0,2],[-25,6],[-13,9],[-7,3],[-3,5],[-1,1],[-1,4],[-1,3],[-1,2],[-1,0],[-2,2],[-1,1],[0,2],[0,2],[2,1],[11,2],[3,2],[0,1],[-12,1],[-6,2],[-8,10],[0,1],[0,2],[1,3],[0,2],[1,6],[-1,3],[-1,3],[1,0],[-1,2],[-1,3],[1,2],[0,2],[1,2],[-1,2],[1,0],[0,4],[1,3],[1,3],[1,4],[-1,3],[1,3],[1,2],[1,2],[-1,4],[-2,1],[-1,1],[2,3],[1,2],[5,5],[1,3],[1,4],[3,2],[3,1],[26,-3],[18,-4],[27,-12],[52,-14],[10,-7],[7,-2],[6,-4],[14,-2],[16,-7],[4,2],[-14,7],[-18,3],[-43,25],[-12,2],[-22,12],[-3,1],[-17,5],[1,0],[3,1],[-3,7],[-4,3],[-4,1],[-3,-4],[1,-2],[-41,4],[1,3],[7,17],[3,5],[2,2],[3,0],[2,-1],[1,-2],[2,-5],[2,-3],[14,-8],[10,-2],[-3,5],[-8,9],[-1,5],[6,4],[16,-3],[17,-8],[17,-4],[31,-5],[30,-4],[22,-2],[15,-1],[18,-10],[6,-1],[2,-1],[-3,4],[-13,7],[0,2],[-2,2],[-14,4],[-28,1],[-9,5],[-28,6],[-9,0],[-51,21],[-10,0],[-5,2],[-4,5],[2,3],[1,3],[2,4],[0,4],[1,5],[3,3],[31,16],[5,1],[4,-2],[2,-3],[6,-11],[3,-4],[3,-2],[4,-2],[3,0],[7,-4],[6,-7],[7,-5],[6,1],[-5,7],[-21,15],[-3,3],[0,4],[0,3],[-2,3],[-3,3],[-2,2],[7,4],[41,15],[6,1],[46,-10],[2,-2],[3,-4],[1,-3],[-2,-1],[3,-7],[2,-6],[2,-5],[4,-4],[2,-1],[2,0],[2,2],[-1,4],[-1,5],[-2,1],[-2,2],[0,5],[1,5],[0,2],[1,3],[-2,5],[-2,3],[-39,12],[2,5],[39,17],[11,1],[22,11],[21,5],[4,0],[8,-4],[3,2],[4,5],[11,7],[4,5],[1,1],[3,1],[44,-1],[44,1],[20,-3],[39,-2],[6,-3],[3,-4],[0,-5],[0,-1],[2,-3],[1,-3],[2,-1],[7,-5],[10,-11],[1,-2],[2,-4],[2,-6],[0,-5],[0,-2],[1,0],[1,-1],[0,-1],[0,-1],[-1,-2],[0,-1],[1,-7],[1,-2],[0,-3],[1,-10],[-1,-4],[-2,-6],[-1,-1],[0,-1],[-1,-7],[4,-7],[6,-4],[56,-19],[2,-2],[5,-8],[3,-8],[-2,-6],[-3,-5],[2,-5],[-3,-4],[-4,-4],[-2,-4],[1,-2],[1,-2],[0,-2],[0,-3],[0,-3],[1,-2],[4,-3],[3,-4],[4,-8],[3,-4],[5,-3],[11,-2],[5,-3],[1,-2],[4,-7],[0,-2],[1,-3],[1,-2],[1,-1],[-2,-3],[-4,-2],[-7,-2],[-1,0],[-4,-4],[-5,-3],[-1,-2],[-2,-3],[-2,-3],[-2,-2],[-2,-1],[-3,-1],[1,-4],[3,-8],[-27,-13],[-16,-15],[-5,-2],[-4,-3],[-7,-13],[-5,-4],[-2,-4],[-1,-1],[-2,-1],[-2,0],[-1,0],[-2,-2],[-1,-2],[0,-2],[0,-4],[-2,-3],[-3,-2],[-6,-2],[-11,-8],[8,0],[8,4],[40,42],[2,2],[6,0],[2,1],[6,5],[21,12],[3,-1],[1,-1],[2,-3],[0,-2],[1,-5],[0,-2],[0,-2],[0,-3],[-3,-5],[0,-2],[0,-2],[1,-3],[0,-2],[-7,-1],[-7,-4],[-13,-12],[-9,-12],[-3,-7],[3,-5],[-4,-3],[-12,1],[24,-15],[3,0],[2,-1],[5,-4],[6,-2],[6,-5],[0,-2],[-1,-3],[-3,-2],[-9,3],[-8,7],[-31,-2],[1,-1],[1,-1],[1,-1],[1,-2],[-2,-2],[-9,-3],[4,-3],[6,-1],[5,0],[13,4],[4,-2],[1,-7],[-1,0],[-1,0],[-1,-1],[0,-3],[0,-1],[0,-2],[0,-2],[0,-3],[0,-2],[-2,-4],[-4,-4],[-9,-4],[-3,-5],[17,5],[8,7],[0,11],[11,6],[11,1],[2,1],[4,3],[6,2],[7,3],[5,6],[-1,9],[-3,3],[-6,4],[-2,4],[-2,6],[1,3],[2,0],[4,0],[1,0],[3,-4],[2,0],[1,0],[2,1],[3,2],[1,1],[0,2],[1,1],[1,1],[1,-1],[2,0],[1,-1],[1,2],[-4,5],[-3,6],[2,8],[5,3],[4,-1],[5,-4],[11,-16],[4,-3],[2,-4],[1,-1],[1,0],[3,-1],[3,-3],[30,-9],[-2,5],[-4,2],[-12,2],[-11,7],[-12,13],[-1,2],[-2,5],[-10,8],[-2,4],[1,3],[13,14],[14,3],[5,5],[2,4],[0,4],[-1,3],[0,4],[2,3],[4,0],[3,0],[3,-2],[-2,-2],[-1,-1],[2,-2],[1,0],[1,1],[1,2],[2,0],[1,-1],[2,-2],[1,-1],[-2,-3],[-4,-3],[-3,-3],[0,-5],[2,-3],[11,-7],[-1,-1],[0,-1],[0,-1],[2,0],[2,-1],[2,-2],[2,-2],[-1,-2],[-2,-5],[-1,-3],[-3,-2],[-9,-5],[-2,-2],[-2,-3],[-1,-3],[2,-3],[2,-1],[4,0],[2,1],[-2,3],[2,2],[5,3],[3,3],[2,2],[2,0],[5,0],[2,-2],[2,-2],[0,-2],[1,-4],[0,-2],[2,-6],[2,-2],[1,0],[1,2],[-1,3],[0,3],[-1,2],[-1,2],[-1,5],[-1,2],[-2,1],[-3,2],[-1,2],[3,1],[3,1],[0,1],[0,2],[0,2],[1,1],[1,1],[2,0],[3,4],[4,3],[3,3],[0,6],[4,1],[5,-2],[18,-12],[4,-5],[6,-8],[1,-2],[7,1],[1,-1],[1,-1],[-1,-4],[-1,-1],[-1,-1],[-1,-2],[0,-3],[1,-3],[4,-6],[-5,-6],[-2,-3],[-1,-5],[0,-3],[-2,-9],[-1,-3],[-2,-3],[-7,-6],[-3,-7],[-7,-4],[-3,-4],[7,-4],[2,0],[3,0],[4,4],[3,0],[39,-12],[15,-13],[5,-2],[4,3],[-2,2],[-5,3],[-2,2],[-1,2],[0,1],[0,2],[0,2],[-1,2],[-2,2],[-16,2],[-6,4],[-5,4],[-4,3],[-2,3],[0,4],[1,2],[1,2],[1,1],[1,1],[0,2],[0,2],[0,2],[1,3],[0,1],[1,1],[0,1],[-1,6],[-1,5],[0,2],[1,2],[1,1],[4,0],[36,-18],[5,-5],[3,-8],[6,-18],[0,-7],[20,-28],[9,-3],[9,-6],[11,-4],[5,0],[0,2],[-4,1],[-7,4],[-7,2],[-12,10],[-6,10],[-3,2],[-5,4],[-2,3],[1,5],[3,2],[12,4],[2,0],[2,-2],[0,-1],[0,-1],[0,-2],[0,-2],[5,-4],[6,-5],[6,-6],[1,-8],[5,-5],[13,-5],[5,-6],[2,-6],[1,-1],[1,8],[1,3],[2,3],[2,1],[-2,2],[-2,0],[-5,0],[-1,0],[-4,2],[-5,2],[-3,1],[-3,3],[-3,3],[-9,13],[-8,5],[-2,4],[1,2],[-3,3],[-7,1],[-8,-2],[-4,1],[-3,3],[-2,3],[-1,12],[-1,4],[-2,3],[-1,4],[-2,2],[-3,3],[-4,2],[-11,3],[-7,4],[-7,2],[-19,13],[-2,3],[8,5],[4,3],[2,5],[-1,3],[-2,2],[-3,0],[-1,2],[-1,2],[0,2],[1,2],[2,0],[6,-1],[2,0],[10,5],[3,1],[4,-1],[4,-2],[3,-5],[-2,-1],[0,-1],[-1,-3],[1,-4],[0,-2],[-1,-3],[0,-4],[2,-4],[2,-2],[2,-1],[3,-1],[0,-6],[3,-4],[4,0],[0,2],[-1,1],[-1,2],[-1,1],[0,1],[0,1],[1,2],[-1,1],[-1,2],[-5,3],[2,9],[0,4],[-2,5],[7,3],[44,-12],[2,-3],[-2,-4],[9,3],[3,0],[15,-5],[10,-7],[5,-5],[-2,-6],[2,-2],[1,-1],[2,-2],[0,-3],[0,-1],[-3,-5],[-1,-1],[0,-2],[2,1],[3,2],[2,3],[1,4],[-2,4],[-1,2],[-1,3],[4,1],[33,2],[4,-1],[9,-6],[21,-6],[-15,9],[-16,5],[-31,1],[-9,7],[-7,2],[-16,10],[-13,4],[-7,4],[-25,7],[-35,21],[-2,5],[0,4],[1,6],[-2,5],[3,1],[9,10],[1,2],[0,4],[0,2],[2,1],[2,1],[2,-1],[2,0],[21,15],[19,5],[1,1],[3,3],[1,2],[20,1],[9,6],[8,2],[6,5],[40,-2],[40,-1],[12,-4],[11,-8],[1,-1],[2,-3],[0,-3],[0,-2],[-2,-1],[3,-4],[51,-9],[4,-2],[1,-3],[0,-3],[0,-3],[-1,-2],[0,-2],[0,-2],[4,-9],[3,-3],[2,0],[6,3],[-1,2],[-3,11],[0,2],[4,2],[3,4],[2,2],[6,1],[23,-2],[10,-5],[9,-1],[2,-2],[8,-5],[31,-8],[1,-1],[0,-3],[0,-3],[0,-3],[3,-5],[1,-3],[1,-2],[9,-13],[1,-6],[2,-3],[9,-11],[2,-5],[-6,-5],[-2,-4],[-1,0],[-1,-1],[0,-1],[1,-1],[0,-1],[-4,-4],[-21,-14],[-4,-1],[-22,6],[-18,-1],[-44,-13],[-5,-3],[-2,-5],[-1,-5],[-3,-6],[-3,-5],[-3,-4],[-12,-7],[-4,-5],[-5,-9],[-5,-4],[-1,-2],[3,-3],[3,1],[3,4],[6,10],[3,4],[7,7],[9,4],[3,4],[3,5],[3,8],[1,2],[2,2],[39,15],[17,0],[31,-10],[2,-3],[-3,-4],[-6,-3],[-27,-5],[-5,-4],[-3,-3],[-2,-4],[0,-6],[-1,-6],[0,-3],[-1,-2],[-2,-1],[-2,-1],[0,-2],[-1,-4],[-2,-2],[2,-5],[-2,-3],[-5,-6],[-2,-2],[-4,-9],[-8,-8],[-17,-2],[-9,-3],[0,-2],[5,0],[7,3],[7,-1],[8,1],[3,1],[4,3],[2,3],[8,14],[5,5],[2,4],[0,2],[1,4],[0,2],[1,2],[5,7],[2,4],[1,4],[-1,4],[0,3],[3,4],[5,5],[5,1],[15,-1],[4,0],[5,2],[6,3],[3,4],[1,3],[2,5],[0,2],[1,2],[3,3],[1,2],[4,4],[5,1],[5,0],[9,-5],[33,-3],[32,-2],[4,-2],[3,0],[2,-1],[1,-3],[1,-2],[5,-6],[5,-10],[2,-6],[-1,-2],[-8,-7],[-2,-3],[1,-1],[1,-1],[1,-3],[1,-2],[1,-3],[-1,-2],[-2,-2],[-2,-5],[-5,-4],[-1,0],[-6,0],[-11,-4],[-6,1],[-3,1],[-2,2],[-1,0],[-1,1],[0,1],[-2,1],[-2,-1],[-1,-1],[0,-2],[2,-2],[1,-2],[-3,-2],[-6,-2],[-3,-2],[-2,-2],[-2,-5],[-2,-2],[-4,-1],[-8,0],[-4,-1],[-6,-4],[-3,-1],[-14,3],[-34,-5],[-3,-2],[-2,-4],[35,7],[9,-1],[4,-2],[3,-4],[1,-3],[-2,-3],[-24,-12],[-11,-9],[-17,-9],[2,-2],[10,7],[12,2],[18,12],[13,3],[3,2],[2,1],[1,2],[2,5],[2,2],[5,4],[9,4],[2,1],[2,-1],[2,-3],[2,-2],[1,-3],[0,-2],[0,-1],[-3,-7],[-2,-4],[-1,-2],[-3,-3],[-2,-1],[-1,-2],[-5,-1],[-1,-2],[-1,-2],[-3,-2],[-21,-1],[-4,-4],[-3,-3],[-6,-5],[-2,-6],[2,1],[3,3],[2,3],[3,3],[4,3],[21,0],[2,-2],[0,-6],[-2,-3],[-7,-4],[-3,-1],[13,-1],[8,-3],[3,-5],[1,-4],[0,-2],[0,-2],[-2,-2],[-1,-1],[-3,-1],[-2,0],[-1,-1],[-1,-2],[0,-2],[-1,-1],[-4,-6],[-2,-2],[-2,-1],[-2,-2],[-3,-7],[-2,-1],[-1,-1],[-4,-2],[-2,-3],[-1,-1],[0,-1],[-1,-1],[-2,-3],[-5,-3],[-2,-2],[2,-3],[4,1],[17,15],[4,4],[4,6],[8,8],[4,5],[5,5],[5,6],[0,7],[-4,5],[-4,4],[-4,5],[-1,9],[2,7],[3,2],[5,1],[4,1],[2,3],[7,18],[1,2],[1,2],[2,1],[10,1],[11,9],[3,5],[1,1],[17,2],[1,0],[1,0],[1,-1],[1,-1],[0,-1],[-1,-1],[0,-2],[-1,-1],[-1,-6],[-2,-4],[-3,-4],[-7,-4],[-2,-3],[0,-4],[0,-6],[1,-5],[4,-3],[4,-2],[2,-1],[-6,12],[-2,8],[4,3],[5,1],[3,3],[3,6],[3,7],[7,12],[11,5],[21,1],[2,-1],[1,-4],[1,-4],[-1,-3],[0,-2],[1,0],[3,9],[4,3],[3,-2],[-2,-7],[-2,-4],[-2,-2],[-14,-10],[-3,-1],[-1,-1],[0,-2],[1,-2],[0,-2],[-1,-1],[-1,-1],[-7,-2],[-1,-2],[0,-3],[-2,-4],[-1,-2],[-5,-4],[-2,-2],[-1,-4],[-1,-4],[-1,-3],[-2,-3],[1,-1],[1,-1],[1,-1],[1,0],[-2,-4],[-2,0],[-6,2],[-4,-1],[-6,-3],[-3,-1],[-1,-1],[-8,-3],[-6,-4],[3,-2],[4,2],[8,5],[18,2],[0,-2],[-2,-2],[-2,-3],[-1,-4],[1,-4],[-1,-4],[0,-4],[0,-3],[-6,-1],[-7,-5],[0,-1],[6,0],[3,0],[2,-3],[-1,-3],[-1,-2],[-2,-1],[-4,-6],[-6,-6],[-3,-2],[0,-2],[6,4],[21,26],[0,1],[1,3],[-2,4],[0,2],[6,15],[2,1],[2,1],[1,1],[1,3],[0,2],[0,2],[0,2],[1,3],[2,4],[1,2],[0,2],[0,2],[2,2],[4,2],[6,7],[4,2],[4,1],[4,-1],[4,-1],[6,-6],[-1,-4],[-3,-3],[-2,-3],[1,-8],[-1,-4],[-4,-6],[2,-2],[4,-7],[2,-1],[0,-2],[-6,-8],[3,1],[6,3],[4,1],[1,1],[1,7],[0,3],[2,2],[3,2],[3,0],[1,-1],[0,-5],[2,-2],[2,-3],[1,-3],[-5,-2],[-2,-1],[-3,0],[-1,0],[-2,0],[-1,-1],[0,-2],[0,-2],[0,-1],[0,-1],[-2,-3],[-5,-4],[-8,-7],[-3,-5],[-3,-5],[1,-1],[0,-2],[0,-2],[-1,-2],[1,-1],[-1,-2],[-2,-4],[-1,-3],[0,-3],[1,-6],[-1,-4],[-3,-7],[-13,-4],[-5,-4],[2,-1],[2,1],[1,1],[2,1],[9,1],[4,1],[4,3],[3,5],[0,5],[0,6],[0,7],[1,2],[2,4],[1,1],[0,3],[1,2],[1,2],[7,7],[1,2],[0,2],[1,1],[3,1],[4,1],[2,2],[2,2],[1,2],[0,2],[-2,4],[14,-2],[7,-2],[9,-6],[1,-1],[0,-2],[1,-1],[5,2],[4,-2],[2,-2],[1,-3],[-20,-6],[-5,-5],[2,-3],[1,-4],[1,-5],[0,-5],[-3,-6],[-3,-7],[-4,-4],[-11,-7],[-3,-5],[2,-2],[5,-8],[1,-2],[1,4],[-3,6],[0,5],[3,3],[10,3],[2,5],[1,3],[2,4],[4,5],[2,3],[2,4],[0,2],[-2,4],[0,3],[2,5],[5,1],[8,-1],[3,1],[2,2],[2,3],[1,4],[2,4],[0,4],[1,4],[0,4],[-1,5],[-1,4],[-1,4],[-1,4],[6,3],[14,0],[7,4],[-2,6],[-1,3],[-1,1],[-2,1],[-1,2],[-2,2],[0,3],[1,1],[6,1],[15,7],[2,6],[1,4],[0,5],[0,3],[1,3],[2,4],[0,1],[4,2],[4,-1],[5,-3],[31,-9],[31,-9],[16,-9],[23,-5],[4,-2],[2,-4],[1,0],[11,-8],[-1,-4],[-1,-1],[6,-5],[11,-7],[16,-22],[3,-6],[-4,-6],[-10,-7],[-6,-8],[-4,-2],[-1,-2],[-5,-8],[-4,-10],[-3,-4],[-3,-2],[-17,1],[-10,6],[-17,4],[-3,0],[-12,-6],[-6,-1],[-3,-1],[-3,-3],[-1,-2],[-1,-2],[0,-2],[1,-3],[1,-2],[-1,-1],[-1,-1],[-2,-1],[-4,-9],[-1,-1],[-10,-3],[-3,-2],[-3,-4],[1,-4],[-1,-4],[-2,-3],[-2,-1],[-44,-13],[-4,-2],[34,5],[20,8],[3,0],[5,-4],[5,-2],[2,-1],[1,-1],[0,-2],[-1,-1],[-1,0],[0,-1],[-2,-2],[-9,-9],[-5,-4],[-2,0],[1,-4],[1,-2],[0,-1],[-5,-10],[-2,-2],[-12,-7],[-11,-1],[-4,-3],[1,-2],[2,1],[2,1],[2,0],[0,-2],[-1,-3],[2,0],[2,1],[2,2],[2,2],[2,3],[10,2],[4,2],[3,3],[3,3],[3,5],[5,9],[2,4],[30,7],[14,8],[11,3],[16,8],[30,4],[3,-1],[9,-7],[2,-1],[3,1],[-1,3],[-8,8],[-2,1],[-2,1],[0,3],[1,3],[3,3],[2,2],[0,2],[1,2],[0,3],[3,6],[2,4],[3,3],[19,8],[14,9],[8,2],[9,-3],[8,-6],[2,-2],[4,-9],[1,-2],[0,-6],[1,-2],[4,-8],[2,-5],[0,-6],[-3,-4],[-19,-14],[-2,-2],[-1,-5],[-1,-3],[0,-10],[-7,-8],[-29,-11],[-4,-4],[-2,-4],[-1,-5],[-2,-4],[-2,-4],[-6,-2],[-5,0],[-30,9],[-6,-1],[-1,-1],[-1,-2],[0,-2],[1,-3],[1,-3],[-1,-3],[-1,-2],[-1,-2],[-14,-10],[-1,-2],[-1,-2],[1,-5],[-1,-3],[-2,-1],[-14,-1],[-5,-1],[-4,-4],[3,-1],[14,5],[9,-2],[1,-1],[0,-1],[-2,-15],[-3,-7],[-4,-5],[-5,-2],[3,-1],[4,1],[4,4],[9,14],[0,4],[1,6],[3,10],[0,1],[-1,1],[-1,1],[11,11],[6,7],[-1,8],[4,2],[3,0],[4,-1],[3,-2],[-5,-4],[2,-1],[9,3],[3,-1],[7,-3],[3,-1],[2,0],[3,0],[1,-2],[1,-4],[-9,-24],[-2,-8],[-2,-11],[-2,-1],[-2,-1],[-1,-3],[0,-4],[2,-3],[3,-5],[-4,-11],[-1,-2],[-6,-5],[1,-4],[-1,-2],[-2,-2],[-2,0],[-1,-1],[-2,-6],[0,-2],[1,1],[0,1],[3,0],[-2,-4],[-9,-8],[-2,-2],[-2,-9],[-1,-3],[2,0],[2,3],[3,6],[3,4],[11,10],[2,3],[1,3],[1,5],[1,5],[2,4],[5,4],[1,3],[3,7],[1,2],[-1,1],[-1,2],[-1,3],[1,2],[4,7],[-1,0],[0,2],[1,3],[1,3],[2,5],[1,2],[1,1],[0,2],[1,5],[1,2],[6,13],[1,5],[-1,2],[3,1],[5,0],[2,2],[-2,1],[-6,4],[2,4],[1,3],[2,3],[1,5],[2,5],[16,3],[17,13],[6,2],[1,2],[2,2],[0,1],[7,3],[4,0],[3,1],[5,5],[6,4],[18,4],[36,19],[5,1],[6,-2],[2,-1],[2,-1],[1,-3],[0,-2],[-2,-2],[0,-2],[-2,-4],[-7,-9],[-10,-10],[-28,-19],[-8,-3],[-15,0],[-4,-2],[-4,-5],[2,-3],[0,-1],[1,-1],[0,-5],[6,-8],[0,-3],[-2,-4],[-8,-5],[0,-2],[2,-1],[2,0],[11,5],[5,1],[-2,9],[-3,5],[-3,4],[-3,7],[2,2],[2,1],[5,-1],[16,1],[10,-2],[3,1],[1,4],[2,1],[0,-5],[1,-4],[2,3],[5,11],[2,1],[7,2],[5,5],[6,-2],[3,2],[-1,1],[-3,2],[3,4],[3,-1],[5,-6],[4,-1],[1,2],[1,4],[1,5],[2,4],[2,4],[3,2],[13,5],[8,0],[-1,-3],[0,-1],[16,2],[8,-1],[8,-11],[4,-3],[41,-17],[42,-16],[3,-3],[4,-4],[1,-1],[1,-2],[0,-4],[0,-2],[-1,-3],[-1,-2],[-5,-7],[-2,-1],[-1,-1],[-2,-1],[-1,-6],[-1,-1],[-3,-1],[-2,-4],[0,-4],[-1,-4],[-3,-1],[-5,2],[-4,4],[-2,4],[0,2],[3,4],[1,1],[-1,5],[-2,2],[-2,-1],[-2,-1],[-3,-1],[-2,-4],[-2,-4],[0,-5],[2,-5],[0,-5],[-1,-4],[-3,-3],[-4,-3],[-13,-1],[-8,-5],[-12,2],[-9,-5],[-5,-1],[-8,1],[-3,0],[-4,-3],[-5,-1],[-5,-2],[-13,-2],[-1,-2],[-2,-4],[-5,-1],[-8,-1],[-12,-9],[-10,-3],[-1,-1],[-2,-2],[0,-2],[0,-3],[-1,-2],[-2,-4],[-4,-4],[-4,-2],[-18,-4],[-3,-3],[-4,-2],[-3,-3],[0,-3],[3,-1],[4,2],[6,8],[4,1],[9,-1],[8,-4],[3,-2],[3,-5],[1,-5],[-2,-2],[-19,3],[-4,-1],[-8,-5],[-4,-1],[-5,-3],[-2,-2],[-1,0],[-1,0],[0,-2],[0,-1],[0,-1],[1,-1],[0,-1],[-3,-4],[-3,-1],[-6,-2],[-2,-1],[-3,-4],[-2,-2],[-1,0],[-5,0],[-1,0],[-3,-5],[12,0],[3,2],[28,25],[4,2],[14,1],[3,-1],[3,-3],[3,1],[5,2],[3,0],[3,2],[2,3],[0,4],[-1,3],[-2,2],[-2,1],[-1,2],[-1,2],[0,1],[0,5],[1,4],[3,4],[5,6],[4,3],[54,13],[6,-1],[9,-5],[4,0],[3,5],[-11,3],[-2,4],[3,1],[12,-4],[5,-1],[28,2],[3,-1],[0,-2],[1,-2],[0,-2],[2,-2],[0,-2],[1,-4],[-13,-10],[1,-3],[2,-3],[-7,-9],[-8,-6],[-25,-8],[-4,-2],[-4,-6],[-4,-3],[-3,-2],[-4,-5],[-4,-2],[-2,-3],[-8,-11],[-4,-3],[-4,-2],[-4,-1],[-26,6],[-4,0],[-5,-3],[-6,-6],[-2,-3],[-1,-3],[-1,-8],[-1,-3],[-2,-3],[-6,-5],[-2,-3],[-1,-3],[-1,-2],[-1,-2],[0,-2],[0,-4],[0,-2],[3,-2],[2,4],[1,6],[2,3],[3,2],[5,4],[4,5],[2,6],[0,2],[0,2],[0,3],[2,2],[3,5],[2,1],[16,1],[6,-1],[6,-5],[2,0],[10,0],[3,1],[18,24],[5,2],[7,6],[10,3],[7,4],[3,1],[17,-1],[8,-4],[1,0],[1,3],[1,-1],[2,0],[1,0],[2,1],[-1,1],[-1,0],[0,1],[-1,0],[0,2],[0,3],[0,1],[-1,1],[2,1],[2,0],[3,1],[3,5],[4,3],[1,3],[-2,0],[-1,1],[0,3],[2,1],[1,-1],[2,-1],[6,0],[1,0],[0,1],[0,2],[0,1],[2,1],[2,0],[4,-2],[6,1],[6,4],[10,10],[-3,2],[-12,2],[-1,1],[2,2],[3,2],[0,1],[-5,0],[-2,1],[0,4],[1,2],[2,-1],[2,-1],[2,0],[-3,5],[-1,4],[2,1],[2,1],[6,7],[7,5],[3,4],[-1,5],[9,2],[3,-1],[0,-1],[1,-3],[1,-1],[1,-1],[2,-1],[16,-4],[6,-4],[5,-4],[3,-4],[18,-17],[4,-3],[6,-7],[2,-4],[4,-5],[1,-2],[1,1],[1,0],[0,-1],[1,-2],[-1,0],[3,-3],[1,-1],[2,-3],[0,-1],[0,-1],[0,-3],[0,-1],[1,0],[1,-1],[0,-1],[11,-9],[2,-4],[1,-5],[0,-11],[1,-4],[6,-9],[1,-5],[-2,-4],[3,-7],[2,-4],[1,-4],[-26,-3],[-24,6],[-7,4],[-5,1],[-5,3],[-5,2],[-30,-1],[-5,-2],[-2,-1],[-1,-2],[-3,-3],[-1,-2],[-8,-3],[-4,-3],[-1,-6],[-2,-5],[-4,-4],[-4,-3],[-3,-1],[-22,0],[-14,4],[-4,-1],[-2,0],[-1,-1],[-3,0],[-5,-4],[-18,-6],[-6,-4],[-2,-1],[-8,-1],[-5,-2],[-8,-6],[-5,-1],[-50,8],[-10,-1],[-17,-7],[-7,1],[-2,-1],[1,-3],[3,-1],[4,0],[7,-3],[5,3],[5,4],[4,2],[12,1],[42,-10],[6,1],[6,3],[4,0],[2,0],[2,2],[1,1],[7,-1],[3,1],[7,7],[21,6],[20,0],[15,-6],[7,-5],[44,-20],[10,-2],[5,1],[9,5],[23,0],[5,-2],[36,-2],[50,-30],[1,-1],[4,-8],[2,-2],[2,0],[2,-2],[2,-5],[2,-5],[1,-5],[-1,-5],[-2,-4],[-8,-12],[-4,0],[-23,10],[-20,2],[-5,-2],[0,-2],[0,-1],[2,-2],[-3,-2],[-5,2],[-7,7],[-4,2],[-5,-1],[-8,-5],[-5,-2],[-19,4],[-25,14],[-4,3],[-7,2],[-5,5],[-6,1],[-12,7],[-4,1],[-12,-4],[-38,8],[-38,7],[-22,-14],[-6,-6],[3,-3],[4,3],[7,7],[16,7],[5,1],[21,-4],[7,-4],[33,-7],[22,2],[5,-2],[2,-2],[3,-5],[3,-2],[1,-2],[1,-3],[0,-1],[-2,0],[-4,-2],[-2,-1],[-2,0],[-6,3],[-45,6],[-8,-4],[-10,-1],[-20,1],[9,-5],[16,1],[7,2],[17,-1],[2,-2],[-4,-3],[-12,-5],[-4,-6],[-2,-1],[-3,-1],[-8,1],[-14,-5],[-12,1],[-4,-2],[-3,-5],[5,0],[1,0],[1,2],[1,1],[11,0],[0,-2],[0,-1],[0,-1],[0,-1],[-1,-3],[-2,-9],[-1,-1],[0,-2],[-1,-3],[-1,-3],[-4,-7],[4,1],[5,6],[0,6],[1,1],[1,1],[2,4],[0,1],[1,1],[0,3],[0,1],[1,3],[2,2],[11,4],[10,1],[5,6],[19,8],[11,2],[19,-7],[5,-1],[1,0],[4,-4],[43,-22],[3,-4],[-4,-3],[-24,-1],[-3,-3],[2,-3],[-4,-2],[-5,0],[-22,8],[-18,0],[-4,-2],[-4,-1],[-1,0],[0,-1],[-1,-1],[0,-1],[-2,0],[-4,2],[-2,-1],[0,-3],[1,-2],[1,0],[2,2],[24,3],[2,-3],[2,-2],[8,-1],[4,-3],[14,0],[-1,-4],[0,-1],[0,-2],[-1,-2],[-2,-1],[-1,0],[2,-2],[4,0],[2,-1],[2,-2],[3,-5],[-3,-2],[-15,3],[-6,3],[-35,4],[4,-3],[21,-3],[2,-1],[2,-2],[2,-1],[4,-1],[6,-4],[23,-6],[4,0],[2,0],[2,-2],[-1,-1],[2,-2],[6,0],[2,-2],[-5,-2],[-6,-1],[-39,13],[-6,0],[-15,5],[-8,0],[-6,1],[-6,0],[-3,-1],[0,-1],[7,-3],[4,-3],[2,-4],[-12,1],[0,-1],[0,-3],[-2,0],[-3,1],[-2,1],[-3,2],[-3,2],[-3,1],[-6,-3],[-5,3],[-3,0],[-3,-1],[-8,1],[-3,2],[-6,6],[-2,1],[-12,0],[-16,7],[-6,0],[-4,-6],[14,1],[8,-2],[6,-5],[-23,1],[-4,-5],[45,-3],[3,-1],[2,-3],[-1,-2],[-1,-1],[0,-3],[-1,-1],[-1,-1],[-1,0],[-1,1],[0,-2],[3,-2],[2,1],[5,4],[4,2],[14,-2],[10,-5],[13,3],[4,-3],[2,-2],[10,1],[22,-5],[8,-4],[1,-1],[0,-1],[1,-1],[1,0],[7,0],[0,-2],[-5,-3],[0,2],[-2,-1],[-7,1],[3,-2],[6,-2],[2,-3],[-3,-4],[-5,2],[-5,4],[-4,2],[-6,-2],[-9,-6],[-3,0],[-29,9],[-9,-3],[-3,0],[-2,0],[-2,-3],[0,-1],[-2,-1],[0,-1],[0,-2],[-3,-2],[-6,0],[-3,-2],[3,0],[4,-3],[2,0],[4,0],[3,1],[3,3],[3,2],[2,3],[1,1],[6,1],[17,-4],[1,0],[0,-2],[0,-2],[-1,-1],[-1,-1],[-2,0],[-4,-2],[-19,-1],[-5,-3],[34,-2],[2,1],[1,2],[0,2],[1,3],[2,1],[3,0],[3,0],[2,-1],[13,-2],[2,-3],[1,-1],[6,2],[1,-2],[0,-2],[0,-1],[0,-2],[1,-2],[1,-1],[1,1],[2,0],[7,0],[1,-2],[0,-1],[1,0],[2,0],[2,2],[3,4],[1,1],[2,-2],[0,-2],[-1,-1],[0,-2],[1,-6],[1,-6],[0,-2],[0,-4],[-4,-9],[-2,-4],[3,1],[1,2],[2,2],[1,3],[1,3],[1,6],[2,6],[0,3],[-1,3],[1,4],[1,3],[1,0],[2,-2],[2,-1],[13,4],[4,2],[2,0],[3,-1],[0,-1],[1,-2],[2,-1],[5,2],[1,0],[2,0],[2,-2],[1,-3],[-5,-4],[-7,-2],[-3,-3],[3,-9],[0,-4],[-1,-3],[-1,-3],[-1,-3],[1,-3],[1,-1],[2,1],[3,7],[2,2],[0,4],[0,3],[-2,5],[-1,3],[3,1],[10,-1],[3,2],[1,0],[2,-1],[1,-3],[1,-1],[1,0],[0,4],[3,2],[3,0],[2,-1],[1,-2],[0,-4],[2,-3],[1,-1],[1,2],[1,4],[2,3],[2,-3],[1,-4],[0,-3],[0,-2],[2,-2],[0,-2],[-9,2],[-5,-1],[-3,-4],[25,0],[2,2],[0,1],[1,5],[0,2],[-2,3],[-1,1],[-1,3],[8,-2],[6,-4],[2,-4],[2,0],[5,3],[7,3],[8,0],[15,-6],[0,-2],[-7,-1],[-4,1],[-3,-1],[-2,-2],[0,-4],[-3,1],[-4,2],[-3,1],[-4,-2],[1,-5],[-2,-3],[-4,-3],[-6,-2],[-3,1],[-7,2],[-8,5],[-17,2],[-4,-2],[-4,1],[-1,-1],[0,-1],[0,-1],[-3,-2],[-40,-9],[-5,-4],[-2,-2],[-1,-2],[-1,-5],[1,-4],[2,-2],[2,2],[1,2],[0,4],[0,1],[2,0],[6,5],[3,0],[9,0],[9,5],[2,1],[1,0],[1,0],[-1,-2],[-1,-2],[-2,0],[-1,-2],[-1,-4],[0,-2],[1,-2],[0,-2],[-2,-3],[-12,-4],[0,-1],[2,-1],[5,2],[6,1],[2,1],[2,1],[1,2],[1,5],[-1,0],[0,1],[-1,1],[1,2],[0,1],[13,9],[2,2],[6,0],[2,-1],[0,-7],[2,1],[2,1],[3,5],[2,1],[2,1],[2,0],[2,0],[19,-8],[2,-2],[-6,-3],[-19,-2],[0,-1],[1,-1],[0,-1],[-1,0],[-5,-3],[-4,-4],[-2,-7],[1,-10],[-3,-5],[-4,-3],[-7,0],[-4,1],[-5,3],[-4,1],[-4,-1],[4,-4],[19,-6],[2,0],[2,2],[1,2],[4,3],[2,3],[-1,1],[1,2],[2,1],[1,2],[0,3],[-1,1],[-1,2],[0,3],[0,3],[18,10],[1,1],[6,2],[5,0],[2,-1],[2,-1],[1,-2],[1,-1],[3,-1],[6,-3],[12,-3],[2,-1],[-1,-2],[0,-2],[0,-1],[0,-2],[-1,-2],[-3,-2],[-1,-1],[-2,-4],[-1,-4],[1,-4],[1,-3],[-2,-3],[-12,-11],[-2,-3],[0,-6],[1,-2],[1,-2],[1,-1],[2,1],[0,2],[0,3],[0,2],[2,4],[10,9],[3,2],[2,0],[1,1],[2,4],[1,2],[3,2],[3,-1],[1,-3],[1,-5],[-1,0],[0,-1],[0,-1],[0,-1],[0,-1],[0,-2],[0,-9],[-1,-4],[-2,-4],[-2,-4],[-2,-2],[5,1],[3,3],[3,6],[2,18],[2,7],[3,3],[5,1],[22,-4],[4,-2],[-3,-8],[-5,-2],[-11,0],[0,-2],[10,-2],[10,5],[2,0],[4,-1],[1,-1],[0,-2],[0,-2],[0,-2],[1,-4],[1,-2],[4,-2],[0,-2],[-3,-3],[-9,-2],[-4,-5],[-3,-8],[-1,-2],[-1,-1],[-2,-2],[-2,-5],[1,-2],[0,-3],[-2,-2],[-2,-1],[-8,-1],[-15,6],[-7,1],[-5,-3],[2,-2],[6,-1],[5,-3],[6,0],[3,-3],[4,-3],[10,2],[3,-5],[-1,-5],[2,-4],[2,1],[2,6],[0,2],[0,2],[-1,1],[0,1],[-1,2],[1,1],[0,2],[1,1],[1,3],[1,3],[0,2],[-1,4],[3,2],[2,7],[2,4],[2,1],[3,-1],[2,1],[2,2],[1,1],[7,2],[2,1],[5,5],[6,4],[6,1],[1,2],[1,4],[0,6],[0,5],[0,2],[0,2],[-1,2],[-1,2],[0,2],[2,5],[2,0],[2,-3],[2,-2],[3,-5],[0,-3],[-1,-2],[-3,-4],[1,-1],[1,1],[1,0],[1,1],[1,-2],[0,-3],[-4,-4],[-2,-5],[-1,-3],[-1,-1],[-12,-13],[0,-2],[1,0],[1,-2],[6,-7],[2,-4],[0,-6],[-2,-5],[-3,-4],[-2,-5],[0,-2],[1,-5],[0,-3],[-1,-3],[-2,-5],[-1,-3],[1,-5],[1,-5],[1,-4],[-2,-4],[2,-4],[2,0],[1,2],[-2,5],[3,3],[1,5],[2,6],[1,13],[-1,2],[-1,3],[0,4],[1,3],[6,4],[10,5],[1,1],[0,1],[0,2],[1,2],[2,2],[0,2],[0,4],[-1,2],[-1,3],[-3,3],[-1,3],[2,1],[1,0],[2,-1],[2,0],[2,0],[5,5],[5,2],[15,-1],[2,0],[3,2],[1,0],[2,-1],[1,-1],[-1,-11],[-2,-5],[-6,-17],[-1,-5],[-1,-2],[-1,-1],[-1,-1],[-3,-3],[-3,-2],[0,-3],[0,-2],[1,-2],[2,-4],[8,-5],[5,-3],[3,-3],[2,-4],[2,-5],[4,-16],[1,-2],[4,1],[0,2],[-4,19],[0,4],[-2,3],[-15,12],[-3,3],[-1,3],[1,2],[3,2],[2,2],[1,2],[1,5],[6,14],[1,2],[3,4],[2,1],[9,2],[13,6],[3,6],[-4,6],[2,4],[2,0],[3,-2],[2,0],[1,1],[1,1],[0,1],[1,2],[3,2],[3,1],[7,-1],[13,-8],[8,-3],[3,-2],[4,-3],[-5,-5],[-14,-6],[-14,2],[-6,-4],[3,-2],[7,0],[4,-3],[-8,-10],[-4,-5],[-4,-1],[-2,4],[-1,1],[-1,-2],[0,-3],[1,0],[-4,-9],[-1,-3],[2,0],[1,1],[6,6],[1,1],[3,1],[1,-1],[1,-3],[1,-3],[0,-3],[-1,-2],[-2,-1],[-1,-1],[1,-3],[-5,-3],[-5,-1],[-4,-3],[-4,-6],[-1,-3],[0,-3],[0,-7],[1,-3],[2,-1],[1,1],[0,4],[2,4],[0,4],[1,4],[3,3],[2,1],[4,0],[2,1],[1,0],[1,1],[0,1],[1,1],[0,1],[0,2],[0,1],[0,1],[2,2],[6,1],[1,2],[2,1],[1,2],[2,0],[2,0],[5,-3],[2,0],[0,-1],[1,-6],[0,-1],[-1,-5],[-1,-4],[-1,-3],[-3,0],[0,-2],[4,0],[3,1],[3,3],[1,7],[0,6],[1,5],[0,4],[-1,6],[2,1],[2,-1],[2,-1],[2,-1],[1,1],[2,2],[1,1],[4,0],[2,0],[11,-6],[5,-5],[1,-6],[6,2],[2,0],[2,-3],[-1,-2],[-2,-5],[-1,-1],[1,-1],[0,-1],[-1,-1],[-10,0],[-6,-2],[-6,-4],[-5,-5],[2,0],[20,7],[5,0],[3,1],[2,2],[3,5],[2,1],[2,-1],[4,-4],[3,-4],[2,-4],[1,-6],[-2,-5],[-3,-2],[-4,0],[3,-2],[3,-1],[14,2],[4,-1],[0,-3],[0,-4],[-3,-5],[0,-4],[3,-2],[2,0],[2,-2],[-1,-1],[-2,-2],[-1,0],[-3,0],[-1,-1],[0,-3],[-1,-1],[-15,-8],[-2,0],[-20,3],[6,-4],[7,-3],[14,0],[12,4],[6,0],[6,-4],[4,-7],[1,-5],[0,-4],[1,-6],[-2,-4],[-2,-3],[-3,0],[9,-3],[2,-3],[-2,-3],[-4,1],[-7,3],[-42,6],[-42,6],[-4,-2],[50,-12],[11,1],[3,-3],[-6,-5],[-7,-4],[-5,-2],[-15,3],[-30,-10],[-3,-3],[5,-2],[5,1],[10,6],[7,2],[32,0],[32,11],[3,0],[2,-2],[-4,-11],[-2,-4],[-4,-1],[-4,0],[-6,4],[-2,-1],[-5,-3],[-46,-16],[-6,-6],[-4,-8],[1,-15],[-1,-3],[3,-1],[2,0],[1,2],[2,2],[0,1],[2,13],[1,1],[1,3],[2,3],[2,2],[3,3],[4,2],[17,2],[6,3],[6,1],[11,6],[21,0],[1,1],[2,1],[1,2],[3,8],[1,1],[1,0],[4,-1],[7,1],[2,-4],[1,-2],[1,0],[6,5],[14,2],[7,-1],[7,-3],[-1,-3],[0,-3],[0,-6],[-1,-2],[-9,-14],[-14,-12],[-8,-4],[-4,-5],[-3,-8],[0,-3],[3,-5],[0,6],[1,5],[1,3],[2,3],[7,6],[14,4],[7,5],[6,7],[4,9],[0,3],[-1,3],[-1,3],[1,5],[9,16],[4,6],[1,1],[2,0],[6,-2],[1,-1],[0,-2],[-2,-2],[-1,-2],[-2,0],[-5,-4],[0,-2],[4,0],[4,2],[4,4],[2,5],[-2,3],[-2,2],[5,5],[5,-1],[5,-5],[5,-2],[3,-4],[3,-8],[0,-8],[-4,-5],[-4,-6],[-1,-2],[-22,-8],[-2,-2],[-1,-3],[-1,-3],[1,-2],[1,-1],[4,-12],[1,-5],[0,-4],[0,-3],[-1,-3],[-6,-7],[-7,-3],[-14,-2],[-6,-2],[-3,-3],[-2,-3],[1,-1],[1,-4],[0,-1],[0,-3],[-1,-2],[-3,-2],[-3,-2],[-11,-2],[-7,2],[-4,0],[-1,-3],[28,-2],[5,2],[3,2],[1,2],[1,1],[0,2],[0,1],[-2,5],[0,2],[1,1],[0,3],[0,2],[1,1],[2,1],[1,0],[1,-1],[1,0],[1,-7],[1,-10],[0,-8],[-1,-8],[-3,-16],[3,0],[2,1],[1,3],[2,4],[1,3],[3,9],[1,4],[1,8],[-2,7],[0,6],[6,2],[7,1],[3,1],[9,7],[14,3],[1,1],[1,2],[1,1],[3,1],[5,0],[5,0],[3,-3],[-1,-3],[0,-7],[1,-7],[1,-6],[1,-9],[-1,-6],[0,-3],[-1,-1],[-3,-7],[-1,-4],[0,-2],[1,-3],[0,-3],[-2,-4],[-2,-4],[-2,-4],[2,-4],[1,3],[5,4],[2,4],[1,1],[-2,5],[0,3],[1,1],[2,3],[1,2],[-1,0],[2,5],[3,2],[2,3],[0,6],[0,1],[-3,6],[0,2],[2,4],[1,3],[-1,2],[-1,3],[-1,3],[0,2],[1,2],[2,3],[2,2],[4,-1],[1,1],[2,1],[7,0],[3,-1],[3,-3],[2,-3],[2,-4],[1,-4],[1,-9],[0,-2],[3,-4],[4,-1],[3,-2],[1,-8],[8,-12],[2,-2],[1,2],[1,3],[0,3],[1,2],[-1,2],[-5,5],[-1,2],[-2,5],[-2,3],[-3,1],[-2,2],[1,3],[-2,9],[1,2],[1,1],[-1,4],[2,1],[5,0],[5,1],[4,2],[2,4],[-1,4],[-2,10],[1,3],[1,2],[1,1],[1,1],[1,0],[2,-1],[1,-1],[2,1],[1,0],[2,2],[5,2],[5,-1],[6,-2],[4,-3],[-3,-6],[-2,-3],[-1,-4],[0,-2],[-1,-5],[0,-2],[-1,-2],[-2,-3],[-1,-1],[2,-2],[3,-1],[3,0],[2,1],[6,12],[2,2],[2,1],[2,0],[8,0],[-1,-2],[-2,-5],[2,-2],[2,1],[3,2],[3,1],[2,-1],[2,-1],[0,-3],[-1,-4],[-6,-6],[-15,3],[-6,-2],[-2,-4],[1,-2],[2,0],[5,5],[15,-1],[2,-1],[3,-1],[6,3],[11,-5],[-2,-6],[-4,-3],[-8,-1],[8,-5],[17,-1],[7,-9],[-2,-1],[2,-3],[5,-1],[2,-2],[3,-6],[7,-9],[2,-5],[1,-6],[-2,-3],[0,-1],[1,-2],[1,0],[-1,-2],[-4,-2],[-4,-3],[-1,0],[-2,0],[-1,1],[-1,0],[-2,-1],[-1,-1],[-1,-3],[0,-1],[-1,-2],[-16,1],[-2,1],[-2,2],[-1,2],[0,3],[-1,3],[-1,2],[-10,10],[-5,2],[-10,1],[-5,2],[-5,5],[-5,2],[-4,-3],[2,-3],[-2,-3],[-2,-3],[-16,-4],[0,-1],[8,-4],[3,0],[4,3],[3,4],[3,2],[4,0],[17,-5],[3,-3],[2,-4],[12,-17],[3,-3],[4,-2],[2,-2],[1,-3],[0,-2],[-2,-2],[-2,-1],[-4,-1],[-6,0],[-2,0],[-2,-2],[-3,-4],[-1,-1],[-2,1],[-4,2],[-1,0],[-5,0],[-2,-2],[0,-1],[-10,-3],[0,-2],[3,-2],[4,1],[7,3],[34,0],[1,-1],[1,-1],[1,-3],[1,-2],[6,-3],[1,-2],[-3,-1],[-6,-4],[-2,-1],[-2,0],[-3,1],[-1,0],[-2,-1],[-2,-2],[-1,-2],[-3,0],[-8,3],[-3,-1],[-3,-3],[-2,-4],[-3,-3],[-3,-1],[-6,0],[-2,0],[-3,2],[-4,2],[-5,4],[-3,1],[-1,0],[-2,-2],[-2,-1],[-8,0],[-1,2],[-1,4],[-1,4],[-1,3],[2,7],[-4,2],[-8,0],[-2,2],[-4,1],[-5,1],[-8,0],[-13,4],[-2,0],[0,-3],[1,-3],[1,-2],[-2,0],[-3,0],[-2,2],[-3,1],[-2,-2],[2,-3],[5,-2],[4,-1],[3,2],[2,3],[4,1],[4,-1],[3,-2],[0,-7],[6,-6],[7,-6],[4,-4],[-2,-1],[-7,-6],[-9,-2],[-10,-7],[-5,-2],[-3,-1],[-3,-1],[-2,2],[-1,2],[-6,2],[-2,2],[0,1],[1,2],[-1,1],[-1,1],[-1,2],[0,2],[0,1],[0,1],[0,1],[-1,3],[-3,3],[-3,1],[-6,1],[0,-2],[3,-2],[2,-3],[5,-8],[-1,-2],[2,-5],[3,-3],[3,-2],[2,-4],[3,-2],[5,0],[14,7],[1,-1],[3,-4],[1,0],[4,-3],[9,-3],[2,1],[0,-5],[4,-2],[21,-1],[4,-2],[-6,-4],[-2,-2],[2,-4],[3,-1],[7,1],[3,0],[0,-2],[-1,0],[-1,-1],[-1,-1],[-1,-1],[4,0],[2,0],[1,-2],[-3,-1],[-6,-1],[-2,-3],[1,-1],[2,-1],[10,1],[2,-1],[2,-4],[1,-3],[-2,-1],[-6,2],[-4,-1],[-6,-4],[-3,1],[-4,2],[-3,-1],[-12,-7],[-2,0],[-1,1],[-3,3],[-1,1],[-2,1],[-1,3],[-2,1],[-3,-2],[-3,1],[-4,3],[-3,1],[-11,-1],[-2,0],[-4,2],[-1,2],[-3,1],[-2,-1],[-2,-2],[-2,-1],[-2,1],[-3,1],[-2,3],[-2,3],[-1,4],[-2,8],[-1,3],[-5,5],[-5,0],[-11,-6],[2,-2],[3,0],[8,3],[1,0],[1,-1],[1,-2],[3,-5],[1,-6],[2,-5],[2,-3],[2,-3],[1,-1],[33,-3],[3,-2],[1,-4],[3,-3],[3,1],[2,1],[2,-3],[2,-4],[1,-5],[-8,-1],[1,-2],[2,-1],[1,-2],[2,0],[2,2],[2,0],[1,0],[0,-2],[-1,-2],[0,-1],[-1,0],[0,-4],[1,-2],[1,-2],[6,-8],[2,-1],[-2,-3],[-1,0],[-4,1],[-2,0],[-4,-2],[-12,-3],[-3,2],[1,3],[-1,1],[-1,-1],[-1,0],[-1,-1],[-2,-1],[-7,1],[-8,4],[-1,1],[-1,4],[-2,1],[-1,2],[-1,1],[-2,0],[-1,1],[-1,1],[0,2],[-1,3],[-2,2],[-3,1],[-2,1],[1,-3],[-1,-1],[0,-1],[0,-1],[5,-4],[2,-1],[-1,-1],[0,-1],[0,-1],[-1,0],[0,-2],[2,-1],[8,-6],[1,0],[3,0],[3,-1],[7,-8],[4,-2],[1,-2],[0,-3],[-1,-4],[-1,-2],[-2,0],[-5,2],[-4,3],[-16,6],[-2,0],[3,-9],[4,-3],[11,-1],[2,-2],[3,-6],[5,-2],[0,-2],[-1,-4],[-1,-3],[0,-4],[-2,-5],[-3,-2],[-2,3],[3,7],[-3,2],[-4,-2],[-2,-3],[0,-5],[0,-1],[-2,-1],[-2,0],[-2,1],[-2,2],[-2,2],[-1,1],[0,3],[-1,1],[-1,0],[-1,0],[0,1],[-1,2],[0,2],[0,10],[-2,7],[-2,6],[-4,3],[-3,-1],[3,-4],[1,-2],[1,-2],[0,-6],[1,-6],[1,-6],[0,-2],[-1,-3],[-6,-9],[-4,-1],[-1,2],[-1,4],[0,2],[0,2],[0,2],[-17,3],[-1,-1],[1,-3],[-4,0],[-2,4],[-2,6],[-2,5],[-6,7],[-3,4],[-1,6],[-1,4],[-10,16],[-1,6],[-2,8],[-2,7],[-2,1],[-1,0],[0,-1],[-1,0],[0,-2],[1,0],[-1,-2],[1,-3],[-1,-2],[-2,-5],[6,-7],[8,-21],[5,-6],[-1,-2],[-1,-3],[1,-2],[1,-2],[1,-3],[1,-3],[-3,0],[-10,3],[-13,-2],[-2,-2],[-1,-1],[-2,2],[-10,2],[-1,-1],[1,-2],[2,-3],[3,-2],[9,2],[3,-1],[5,-4],[2,-1],[13,1],[2,-1],[2,-3],[3,-6],[-1,-1],[-1,0],[0,-2],[1,-1],[-3,-8],[-1,-2],[-13,-2],[-2,1],[-2,3],[-4,9],[-2,2],[-2,1],[-2,0],[0,-1],[0,-1],[2,-1],[0,-1],[0,-2],[0,-2],[2,-3],[-3,1],[-1,0],[-2,-1],[2,-4],[7,-3],[2,-5],[-1,-3],[2,-1],[3,1],[4,0],[2,0],[16,-11],[0,-2],[-2,0],[-4,2],[-3,0],[-3,-2],[-1,0],[-2,0],[-16,10],[-7,1],[-3,-3],[22,-15],[6,0],[3,-1],[-16,-2],[0,-2],[7,0],[0,-1],[-1,0],[-1,-1],[-2,-1],[3,-3],[3,-1],[8,-6],[-1,-4],[0,-1],[2,0],[2,-1],[1,-1],[-3,-1],[-6,1],[-3,-1],[-1,-3],[1,-1],[2,-1],[2,-2],[1,1],[2,1],[2,0],[2,-1],[1,-2],[0,-1],[-1,-1],[-6,0],[-4,-1],[-2,-2],[2,-2],[10,-2],[3,-1],[1,-3],[-1,-3],[-2,-1],[-1,0],[-2,3],[-1,0],[-1,0],[-3,-1],[-10,0],[-2,-2],[9,-5],[-1,-1],[-1,-1],[-2,0],[-1,0],[4,-1],[8,-1],[4,-3],[-2,-4],[-3,-2],[-3,-1],[-3,-2],[2,-1],[0,-2],[1,-2],[1,-1],[1,0],[2,-2],[1,-1],[-3,-2],[-2,0],[-6,1],[0,-2],[9,-2],[0,-1],[-2,-3],[-2,0],[-3,1],[-3,0],[1,-5],[1,-2],[-2,-2],[-7,-6],[1,-1],[2,-2],[-2,-3],[-1,-2],[-1,-2],[1,-2],[1,-2],[0,-3],[-1,-1],[-6,1],[-8,4],[2,1],[1,2],[-1,2],[-2,1],[4,3],[2,2],[0,4],[0,3],[-1,2],[-2,1],[-2,-2],[-1,-2],[-5,-2],[-4,-3],[-2,-1],[-1,3],[-1,1],[-6,1],[0,2],[1,1],[0,1],[1,1],[-1,0],[-1,0],[0,2],[7,0],[0,1],[-1,0],[0,1],[3,2],[10,3],[-3,2],[-4,1],[-4,-1],[-3,-3],[-3,-2],[-4,0],[-3,3],[0,6],[-2,4],[1,5],[2,4],[7,10],[1,4],[-2,1],[-3,0],[-2,0],[-2,-2],[-2,-8],[-1,-1],[0,1],[-1,0],[-5,-8],[-2,-2],[-3,-1],[-3,2],[-4,4],[-2,-1],[-1,-4],[0,-6],[-1,-4],[-2,-2],[-3,-1],[-2,2],[-7,10],[2,1],[0,1],[0,1],[-2,0],[-3,-2],[-2,-1],[-2,2],[2,3],[6,11],[3,3],[8,3],[3,3],[0,1],[-4,1],[-12,-2],[-2,-1],[-1,-4],[-1,0],[-2,-1],[-2,-1],[-3,-3],[0,-2],[-1,0],[-1,1],[-1,5],[-3,1],[-1,2],[0,2],[-1,5],[0,2],[0,3],[0,2],[0,2],[0,1],[2,2],[2,3],[2,4],[-1,1],[-2,-1],[-2,-3],[-2,-1],[0,2],[-1,0],[2,4],[6,7],[1,3],[9,11],[0,2],[-2,1],[1,1],[1,2],[1,2],[0,2],[-5,-1],[-1,-1],[0,-5],[-2,-1],[-3,-2],[-1,-2],[-1,-3],[0,-2],[0,-2],[-2,-3],[-5,-1],[-2,-2],[-1,-2],[-3,-7],[-1,-2],[0,-2],[3,-8],[-1,-9],[-5,-15],[-3,-8],[-3,-4],[-1,2],[-1,6],[-2,1],[-2,-1],[-3,-1],[0,2],[2,1],[1,1],[-1,2],[-2,1],[0,2],[2,6],[0,2],[0,1],[-2,-1],[-1,-1],[0,-2],[-1,-1],[-4,-1],[-2,-3],[-2,0],[1,4],[1,3],[1,3],[2,1],[-3,4],[-1,1],[2,3],[1,1],[3,1],[2,0],[0,2],[-5,-1],[-1,1],[-1,4],[-1,1],[-1,0],[-1,0],[-1,0],[-1,-1],[-1,-4],[-4,-4],[-4,1],[-4,4],[-4,6],[6,0],[-1,2],[-2,1],[-2,1],[-2,0],[0,2],[2,0],[2,2],[5,1],[2,2],[1,-1],[3,-2],[2,0],[5,3],[4,3],[-12,2],[-12,-3],[-2,1],[3,3],[8,4],[7,5],[9,2],[4,3],[2,3],[2,1],[11,1],[2,-1],[2,-1],[1,-1],[2,1],[1,3],[3,10],[-1,2],[-2,0],[-2,-2],[0,-4],[-1,-3],[-3,-2],[-2,0],[-2,1],[1,1],[2,1],[0,1],[-14,-1],[-2,2],[0,2],[-2,-2],[-2,-3],[0,-1],[-2,-2],[-2,0],[-4,-1],[-6,-3],[-2,-1],[-6,3],[-2,-2],[-8,-3],[-4,-3],[1,-4],[-3,-2],[-4,1],[-3,3],[-2,3],[-5,6],[-1,3],[-1,3],[1,3],[0,3],[-3,3],[3,3],[3,1],[10,2],[11,4],[0,2],[-1,1],[1,1],[2,2],[1,1],[1,-1],[1,1],[0,2],[1,1],[2,3],[1,1],[1,1],[1,0],[3,-2],[2,0],[-1,2],[-4,4],[0,1],[0,3],[0,2],[0,1],[11,2],[3,2],[0,1],[-14,2],[-2,-1],[-2,-3],[-11,-22],[-3,-2],[-4,-1],[-7,1],[-7,-1],[-3,0],[-4,2],[-1,3],[1,2],[2,2],[1,4],[-1,1],[-11,-10],[-2,-1],[-2,1],[-1,5],[1,1],[2,1],[1,2],[0,3],[0,4],[4,2],[5,1],[5,-3],[7,2],[1,2],[0,7],[2,1],[4,2],[9,5],[4,2],[9,-1],[4,1],[4,3],[-4,2],[-20,-3],[-7,-2],[-3,-3],[-6,-6],[-2,-4],[-3,-1],[-30,5],[0,3],[6,14],[2,4],[1,2],[0,2],[-1,2],[-2,-1],[-2,-2],[-3,-4],[-3,-4],[-2,-1],[-2,4],[0,2],[1,3],[2,5],[2,8],[1,3],[9,11],[1,3],[1,3],[2,2],[6,1],[3,3],[5,1],[6,4],[4,2],[-1,0],[3,0],[5,4],[12,4],[2,1],[3,0],[12,-8],[-1,0],[3,0],[2,0],[2,-1],[2,1],[-1,3],[-2,2],[-2,1],[-4,2],[-3,4],[-1,2],[0,1],[3,4],[3,6],[2,7],[2,6],[1,9],[1,8],[3,6],[9,5],[9,10],[5,2],[4,1],[6,3],[4,4],[4,5],[-2,1],[-1,-1],[-1,-2],[0,-1],[-2,-1],[-2,-1],[0,1],[-1,2],[0,1],[-2,0],[-2,-1],[-7,-5],[-1,-1],[-2,0],[-2,-1],[-4,-3],[-11,-4],[-6,-7],[-4,-5],[-2,-5],[1,0],[-1,-1],[-1,-1],[-1,-1],[0,-1],[-1,-2],[-1,-6],[-1,-3],[0,-2],[1,-5],[-2,-3],[-6,-3],[-3,-4],[-2,-1],[-6,1],[-2,-2],[-2,-3],[-17,-7],[-4,-1],[-1,2],[-1,1],[-1,1],[-3,0],[0,-1],[-1,-6],[0,-1],[-2,-1],[-5,-3],[-2,-2],[-1,-1],[-41,-2],[-8,-4],[-4,-1],[-5,1],[-3,1],[-1,4],[-1,8],[0,3],[-1,2],[-1,3],[0,2],[2,2],[2,1],[1,3],[0,3],[2,2],[5,1],[2,1],[19,15],[4,1],[1,5],[1,2],[3,3],[4,5],[1,3],[1,2],[1,5],[0,3],[2,3],[5,4],[-2,1],[-4,-1],[-2,2],[6,4],[2,3],[-1,4],[0,2],[1,1],[0,1],[-1,1],[-2,-1],[-1,-2],[-1,-4],[-1,-4],[-1,-1],[-1,-1],[-1,0],[-1,-2],[-1,-11],[-1,-3],[-2,-3],[-10,-16],[-1,-1],[-5,-1],[-5,-3],[-6,-5],[-1,-2],[-2,-2],[-6,-2],[-3,-2],[-2,-1],[-6,1],[-16,6],[0,1],[0,1],[-1,2],[-4,4],[-1,1],[0,2],[-1,2],[0,2],[-5,5],[0,1],[0,-1],[1,-1],[0,2],[2,4],[2,1],[2,-2],[2,6],[3,6],[-10,-9],[-5,-3],[-3,5],[0,3],[-2,-1],[-4,-4],[-2,2],[-2,3],[-3,1],[-2,-3],[3,-2],[3,-4],[2,-4],[-2,-4],[-4,-2],[-6,1],[-5,3],[-2,7],[-1,5],[0,4],[1,2],[6,-3],[1,0],[0,2],[-1,4],[-1,2],[-1,1],[-1,1],[1,2],[4,4],[0,1],[-8,-1],[-4,2],[0,4],[2,3],[13,5],[0,1],[0,2],[-2,1],[-2,0],[-3,-3],[-1,0],[-2,-1],[-4,-2],[-3,-2],[-3,-4],[-8,0],[-5,1],[-4,3],[4,1],[3,3],[2,6],[1,8],[-3,-1],[-1,-2],[0,-3],[-3,-1],[-2,1],[-3,1],[-2,3],[-2,3],[3,2],[0,6],[-1,7],[-3,4],[0,-2],[-1,0],[-1,0],[-1,2],[13,7],[2,4],[-4,2],[-10,-2],[-5,2],[-9,8],[-4,3],[-6,1],[0,-2],[20,-17],[-4,-3],[-5,-4],[-5,-1],[-3,2],[0,-2],[-1,-1],[0,-1],[0,-1],[-3,3],[-4,1],[-4,-1],[-3,-2],[0,-1],[9,-1],[1,-2],[-2,-4],[-4,-2],[-4,1],[-3,1],[-1,4],[1,3],[0,3],[-10,4],[-3,3],[-3,3],[1,4],[-3,1],[-7,-2],[-2,-3],[-2,-1],[-1,2],[-6,9],[-1,1],[-1,0],[-2,1],[-1,-1],[1,-2],[2,-4],[1,-5],[-2,-2],[-3,2],[-3,2],[-1,1],[-2,1],[-2,1],[-2,0],[1,-1],[0,-3],[1,-1],[-7,1],[-3,0],[-3,-3],[5,-2],[37,-3],[2,-1],[7,-3],[2,-1],[1,-3],[1,-4],[1,-5],[9,-6],[4,-3],[-2,-2],[0,-2],[1,-4],[0,-4],[0,-3],[-1,-2],[-2,-1],[-3,1],[-3,7],[-3,2],[-12,1],[0,2],[1,0],[1,0],[1,1],[-2,2],[-2,1],[-5,-1],[2,-5],[4,-4],[4,-3],[3,-1],[2,-2],[6,-3],[3,0],[2,-1],[5,-6],[1,-3],[-2,-3],[-3,0],[-6,1],[-3,-1],[-3,-4],[-2,-2],[-4,3],[-4,5],[-1,1],[-1,-4],[-1,-1],[-3,2],[-2,4],[-1,2],[1,1],[0,1],[0,1],[0,1],[-5,2],[-1,1],[0,2],[0,3],[0,2],[-2,1],[-2,-1],[-2,2],[-3,6],[-1,-2],[-1,2],[-4,2],[-2,1],[1,3],[1,0],[-3,0],[-5,-1],[-12,4],[-2,2],[-1,4],[-2,8],[-1,2],[-1,-1],[-1,-3],[0,-2],[-2,-1],[-1,1],[-3,4],[-1,2],[-5,-2],[-1,-2],[1,-2],[1,1],[2,-1],[1,-3],[1,-2],[4,-2],[3,-1],[0,-2],[1,-3],[0,-3],[1,-2],[0,-1],[0,-2],[2,-1],[4,-4],[2,-3],[3,-3],[0,-1],[0,-3],[0,-3],[1,-2],[1,-1],[0,-2],[3,-3],[1,-3],[0,-3],[-2,-3],[-3,-3],[2,-2],[5,-1],[2,-2],[-1,-1],[4,-2],[9,-8],[10,-3],[-1,-2],[0,-2],[0,-1],[1,0],[1,0],[0,-1],[4,-3],[13,-4],[3,-2],[1,-4],[0,-4],[1,-4],[2,-2],[1,0],[2,-3],[1,0],[6,0],[0,-2],[-8,-5],[-4,-2],[-5,-1],[2,-1],[3,-1],[2,-1],[0,-2],[1,-3],[1,-2],[1,-2],[2,-2],[2,-2],[3,-1],[-2,-4],[-3,0],[-7,4],[-5,-1],[-2,-1],[-1,-1],[-1,1],[-1,2],[-1,1],[-2,1],[-1,0],[-1,0],[0,1],[0,1],[-1,0],[0,-1],[0,-1],[0,-2],[-4,-7],[-3,-2],[-5,4],[-3,0],[-6,-1],[-2,0],[-1,1],[-1,1],[-1,2],[-2,1],[-3,1],[-2,0],[-3,7],[-1,2],[-1,-4],[0,-3],[2,-4],[3,-4],[0,-2],[-5,0],[-3,-1],[-2,-3],[-2,-1],[-13,6],[-1,3],[0,4],[-1,4],[-2,2],[-2,2],[-1,2],[0,4],[-4,1],[-1,2],[0,2],[2,6],[0,3],[0,1],[-1,0],[0,1],[0,1],[0,2],[0,1],[1,0],[5,3],[-2,3],[-10,6],[-7,1],[-9,5],[-2,3],[-1,4],[-1,6],[0,4],[-2,3],[-4,-1],[-3,-1],[-1,-3],[2,-5],[-2,-1],[-3,0],[-3,1],[-2,2],[0,4],[-2,10],[-2,3],[-1,3],[-2,3],[-2,2],[-2,1],[-2,-2],[1,-3],[3,-7],[2,-2],[0,-2],[0,-2],[-1,-2],[-1,-2],[-2,-1],[-1,0],[-35,-2],[-2,-2],[14,-3],[1,-2],[3,1],[1,1],[1,1],[2,1],[2,-1],[2,-2],[2,-1],[2,1],[1,0],[1,1],[1,2],[1,2],[7,-1],[-1,0],[4,-1],[2,-1],[1,-2],[3,-7],[2,-2],[6,-4],[8,-10],[5,-1],[0,1],[0,1],[-2,4],[-3,3],[1,1],[2,0],[1,-1],[1,0],[0,1],[0,3],[2,0],[6,-3],[2,-2],[3,-6],[1,0],[2,-2],[4,2],[2,-2],[-2,-1],[-1,-1],[-2,-4],[2,-6],[0,-4],[-1,-3],[-3,-4],[-1,1],[-1,2],[0,2],[1,2],[-3,4],[-4,2],[-4,-2],[-2,-4],[6,-3],[3,-3],[2,-4],[-4,0],[-3,1],[-12,11],[-3,3],[-2,4],[-1,1],[-1,-1],[-1,-2],[1,-1],[4,-9],[3,-4],[7,-3],[3,-4],[-3,-1],[-2,1],[-4,4],[-2,0],[-3,-2],[-2,-2],[0,-5],[2,-4],[5,0],[9,3],[0,-1],[0,-2],[1,-1],[1,-1],[0,1],[2,5],[1,0],[1,-1],[0,-3],[-1,-4],[-1,-1],[1,-3],[-1,-2],[-2,-1],[-2,-1],[7,-7],[4,0],[1,-1],[-1,-2],[-2,-1],[-5,-1],[-5,-3],[0,-1],[9,0],[3,0],[6,-4],[4,0],[3,3],[2,1],[7,-1],[1,1],[1,5],[2,2],[3,-1],[3,-2],[-2,-1],[-1,-3],[1,-2],[2,-2],[-4,-5],[-9,-1],[-4,-4],[1,-4],[-2,-3],[1,-3],[2,-1],[2,1],[3,1],[1,1],[1,-1],[1,-1],[0,-2],[1,-1],[-2,-2],[-1,-3],[1,-3],[2,-2],[-1,-2],[-1,-2],[-2,-2],[-1,0],[-2,0],[-4,4],[-3,-1],[2,-2],[6,-5],[-3,-3],[-1,0],[-1,0],[-1,2],[0,1],[-1,0],[-2,-1],[-7,-7],[2,-1],[2,-1],[2,1],[1,2],[1,1],[2,0],[2,-1],[0,-3],[-1,-3],[-6,-10],[-2,-3],[0,-2],[2,0],[2,2],[1,2],[1,3],[8,9],[3,5],[2,2],[2,1],[4,0],[2,1],[0,2],[2,4],[2,4],[3,1],[21,6],[3,-1],[3,-2],[1,-1],[1,2],[0,3],[1,2],[1,1],[3,1],[4,-1],[-1,0],[4,-1],[11,-7],[0,-3],[2,-6],[-1,-1],[-2,0],[-7,-5],[0,-1],[2,-2],[3,-2],[2,0],[1,-1],[0,-1],[1,-3],[1,-1],[-10,2],[-4,0],[-4,-4],[1,-1],[2,-1],[5,0],[1,-1],[1,-2],[-2,-2],[-10,1],[-3,1],[0,-2],[9,-5],[3,-3],[1,-2],[0,-1],[1,-1],[2,1],[1,1],[1,2],[1,1],[2,1],[1,-1],[3,-2],[1,-1],[2,0],[8,3],[3,0],[3,-1],[3,-2],[0,-1],[-5,-2],[-1,-1],[1,-1],[4,0],[1,-1],[0,-2],[-2,-5],[0,-2],[-1,-4],[-3,-2],[-4,-1],[-19,0],[-1,-3],[-10,-1],[1,-3],[2,-2],[3,-1],[2,-1],[1,0],[1,1],[1,0],[0,1],[1,2],[1,2],[1,2],[1,1],[4,-2],[9,-7],[5,-1],[-4,-8],[-1,-4],[1,-2],[1,1],[3,3],[0,-1],[1,-2],[1,-1],[1,0],[2,-1],[0,-2],[-6,0],[-3,-1],[-1,-4],[2,1],[1,0],[2,0],[1,-2],[-1,-3],[1,-1],[2,-1],[3,4],[2,2],[3,0],[1,-1],[0,-3],[0,-1],[3,-1],[2,2],[4,8],[0,-6],[-2,-21],[-1,-5],[-8,1],[-2,-1],[-2,-2],[-2,-6],[-2,-2],[0,-1],[2,0],[8,4],[2,1],[4,0],[1,-1],[-1,-4],[1,0],[5,2],[3,2],[-1,6],[6,10],[2,1],[3,1],[5,6],[3,1],[0,-1],[-1,-3],[-1,-3],[-1,-3],[-1,-3],[0,-4],[-6,-3],[-2,-3],[4,0],[-2,-6],[-9,0],[-3,-4],[6,-2],[2,-1],[2,2],[8,6],[-1,-4],[-1,-2],[-1,-2],[1,0],[0,-1],[1,0],[0,-1],[-1,-1],[0,-2],[0,-2],[1,-1],[1,-1],[1,2],[1,0],[0,-3],[0,-6],[-3,-3],[-7,-3],[9,-3],[3,-3],[-1,-4],[-1,-3],[-1,-1],[-2,-2],[0,-2],[2,-1],[2,-3],[0,-4],[-1,-3],[2,-1],[1,-1],[0,-2],[-1,-3],[2,-1],[1,-1],[0,-1],[-1,-1],[0,-1],[0,-1],[1,-1],[2,0],[5,4],[-1,2],[-3,2],[-1,3],[-1,4],[0,5],[1,4],[3,16],[0,4],[1,5],[-1,4],[-5,8],[0,4],[2,2],[10,-4],[2,-3],[1,0],[2,0],[1,0],[0,-2],[1,-3],[-1,-7],[0,-2],[4,7],[1,0],[1,0],[1,-2],[0,-2],[-2,-6],[2,-2],[3,-2],[9,-2],[1,1],[-1,5],[3,0],[1,-2],[1,-3],[2,-3],[0,-2],[-1,-2],[0,-3],[0,-2],[1,-1],[2,1],[1,3],[0,2],[1,2],[3,1],[4,-2],[7,-7],[-2,-2],[-1,-3],[-1,-3],[-1,-3],[1,-3],[1,-4],[-2,-3],[-1,-2],[-16,0],[3,-3],[4,-1],[8,1],[-1,-3],[0,-2],[1,-2],[0,-2],[0,-3],[0,-2],[-1,-3],[6,-2],[1,6],[1,9],[3,6],[3,9],[1,2],[2,1],[6,9],[1,-6],[0,-6],[-1,-6],[-2,-4],[-1,-6],[0,-5],[2,-3],[3,1],[-1,3],[0,4],[0,3],[1,2],[1,0],[1,1],[2,1],[1,-1],[1,-2],[-1,-2],[0,-2],[0,-1],[5,-14],[5,-4],[2,-2],[1,-2],[2,-2],[2,0],[1,3],[-1,2],[-6,7],[-2,5],[-2,6],[0,4],[4,1],[0,2],[-2,2],[-3,2],[-2,4],[-2,3],[2,0],[2,1],[1,1],[1,3],[-5,8],[-3,6],[2,1],[2,-1],[4,-2],[1,-2],[1,0],[1,0],[1,1],[0,1],[0,1],[1,1],[1,0],[1,-1],[1,-1],[0,-1],[2,-2],[1,0],[1,-1],[0,-4],[-2,-9],[2,-1],[2,3],[1,4],[2,3],[2,0],[3,-4],[2,-4],[0,-3],[-4,-5],[-2,-3],[-1,-4],[0,-4],[0,-3],[3,-7],[1,0],[0,15],[6,5],[7,-4],[3,-11],[-1,0],[-2,-2],[0,-1],[0,-2],[0,-2],[1,-1],[0,-5],[-1,-2],[-3,-4],[-5,-6],[-2,-1],[-5,0],[-2,-1],[-2,-2],[2,-2],[1,-2],[1,-3],[0,-5],[-1,-4],[0,-2],[0,-3],[2,-4],[2,0],[9,3],[12,0],[-1,-1],[0,-3],[-1,-1],[3,-1],[5,3],[4,5],[2,4],[1,0],[4,1],[2,0],[0,-4],[-1,-3],[-6,-7],[-1,-2],[-1,-3],[1,-1],[2,2],[3,4],[1,1],[2,2],[0,1],[1,-1],[2,-2],[1,-1],[2,0],[1,0],[1,1],[-1,3],[0,4],[0,2],[1,2],[1,2],[3,1],[1,1],[1,1],[1,-1],[0,-3],[0,-2],[-3,-5],[-1,-2],[4,1],[7,6],[3,-1],[-1,-2],[-5,-4],[-1,-2],[2,-1],[7,1],[1,2],[1,-2],[1,-4],[0,-3],[-1,-5],[-2,-3],[-6,-3],[-10,0],[-1,-1],[-2,-4],[8,-4],[2,-5],[-2,-8],[-4,-3],[-4,-1],[-29,8],[-3,0],[-1,-1],[-4,-3],[-7,-1],[0,-1],[0,-1],[21,2],[4,-1],[2,-4],[-3,1],[-1,1],[-1,-1],[0,-1],[1,-1],[1,-2],[1,-1],[-4,-1],[-7,-2],[-4,-1],[2,-2],[3,-1],[19,3],[1,-1],[1,-1],[-1,-2],[-2,-1],[0,-2],[8,0],[3,-2],[2,-5],[-4,0],[-3,-1],[-8,-8],[-2,-1],[0,-1],[6,0],[1,1],[4,5],[1,0],[3,1],[2,-1],[6,-6],[0,-3],[1,0],[3,1],[0,-2],[-1,0],[-2,-3],[2,-1],[4,-1],[2,-1],[-9,-8],[9,2],[4,-1],[3,-3],[-2,-1],[-6,-4],[-6,-2],[-5,0],[1,-3],[2,-1],[13,-2],[1,0],[4,1],[13,-1],[-1,-4],[-1,-2],[-3,-4],[5,1],[2,2],[3,2],[3,5],[2,2],[4,2],[1,0],[2,-1],[1,-2],[0,-1],[0,-2],[1,0],[2,2],[2,4],[3,3],[2,-1],[1,-2],[0,-1],[0,-2],[0,-2],[1,-2],[2,-3],[0,-2],[-1,-2],[-17,-9],[-2,-2],[-2,-3],[-7,-5],[1,-4],[-1,-3],[-2,-1],[-2,-1],[-4,-1],[-1,-2],[-2,-4],[2,-6],[2,-4],[3,-2],[19,-7],[6,3],[1,-1],[1,-2],[2,-2],[15,-13],[2,-5],[2,-6],[-1,-5],[-1,-2],[-1,-1],[-4,2],[-1,-1],[-1,-3],[2,-1],[3,-3],[0,-4],[-2,-1],[2,-2],[1,-2],[1,-3],[0,-4],[0,-4],[0,-3],[-1,-2],[-2,-2],[3,-3],[2,-4],[0,-5],[-3,-5],[-3,0],[-6,5],[-3,-1],[4,-8],[9,-2],[2,-8],[0,-2],[0,-3],[-1,-2],[-1,-2],[-3,1],[-3,3],[-3,1],[-3,-1],[4,-4],[9,-2],[3,-3],[-1,-1],[-2,-1],[-1,-1],[-1,-1],[0,-2],[0,-1],[-2,-1],[-2,1],[-1,1],[-2,0],[-1,0],[-1,-1],[-1,-2],[-1,-2],[-1,0],[-1,1],[-1,1],[-1,2],[0,2],[-1,1],[-4,2],[-1,1],[0,2],[0,1],[0,2],[-1,2],[-5,4],[1,4],[-1,3],[0,3],[1,3],[-3,3],[-1,1],[0,1],[-1,4],[-4,16],[0,1],[1,1],[0,1],[0,2],[-1,0],[0,1],[-1,2],[-2,5],[-3,3],[-4,5],[-1,4],[0,2],[0,4],[0,2],[0,3],[-1,1],[-6,5],[-6,3],[-3,3],[-1,3],[-3,10],[-2,3],[-6,3],[-3,4],[2,1],[5,-2],[-2,4],[-3,3],[-4,1],[-3,0],[-3,1],[-2,0],[0,-2],[1,-2],[9,-10],[3,-4],[1,-2],[2,-1],[3,-2],[1,-4],[2,-6],[2,-4],[5,-6],[5,-4],[1,-1],[1,-3],[0,-2],[0,-1],[-8,-3],[-5,-6],[1,-8],[4,-9],[-1,-7],[-2,-1],[-6,-1],[-2,-3],[6,0],[6,2],[-1,-3],[-3,-3],[-5,-3],[5,-1],[4,1],[4,0],[4,-7],[2,-2],[1,-1],[1,-2],[0,-3],[0,-2],[1,-3],[2,-5],[0,-3],[-1,-2],[0,-1],[2,-8],[-3,1],[-3,2],[-4,8],[-2,3],[-3,3],[-3,1],[-1,-2],[2,-1],[6,-7],[-3,-6],[-5,3],[-5,6],[-4,0],[5,-10],[1,-1],[11,0],[3,-3],[0,-4],[-2,-3],[-2,1],[-2,2],[-3,2],[-2,0],[-2,-3],[2,-1],[3,-3],[-1,-1],[-1,0],[0,-2],[15,1],[5,-2],[3,-1],[-1,-2],[-2,-1],[-1,0],[-1,-1],[1,-2],[1,-1],[8,0],[1,-1],[2,-2],[1,-2],[0,-3],[1,-2],[2,1],[-3,-2],[-7,1],[-3,-1],[3,-3],[7,-2],[2,-5],[-1,-3],[-1,0],[-1,2],[-2,1],[-1,-1],[1,-2],[2,-2],[1,-1],[0,-2],[-1,-1],[0,-2],[1,-2],[1,-2],[0,-1],[-2,-2],[-4,0],[-3,-2],[3,-1],[5,-2],[2,-2],[3,-5],[2,-2],[1,3],[-1,4],[1,1],[1,1],[1,0],[7,-11],[-1,-4],[-3,0],[-4,2],[-2,-1],[-2,-2],[0,-3],[-2,-3],[-3,-1],[-12,1],[-4,3],[-3,4],[-7,4],[-5,6],[-4,2],[-1,3],[-1,3],[0,3],[-1,2],[-1,2],[-3,-5],[1,-3],[1,-4],[1,-6],[-2,1],[-4,6],[-7,3],[-1,3],[-1,4],[-2,3],[-1,-6],[4,-6],[5,-5],[2,-4],[0,-3],[-1,-1],[-1,-2],[2,-5],[1,-2],[6,-8],[4,-8],[1,-2],[2,-1],[4,-9],[1,-2],[0,-2],[1,-1],[1,0],[1,-1],[1,-3],[0,-2],[0,-2],[0,-1],[1,-4],[1,-3],[2,-3],[2,-2],[-1,-3],[-2,0],[-2,2],[-2,1],[0,-4],[-2,-3],[-8,-7],[-3,-2],[-4,-2],[-3,0],[-5,2],[-4,7],[-6,17],[4,5],[-4,0],[-2,0],[-2,-1],[1,-4],[-1,-1],[-1,1],[-1,2],[0,3],[1,2],[3,9],[3,4],[5,5],[-3,2],[-8,-3],[-3,3],[1,3],[0,3],[1,3],[2,1],[5,-1],[2,0],[3,2],[-3,1],[-3,0],[-3,1],[-1,3],[-1,3],[0,5],[1,5],[1,2],[-2,2],[-3,-8],[-3,-15],[-1,-1],[-1,-1],[-1,0],[-1,0],[-1,1],[0,3],[0,2],[-2,1],[0,-1],[-1,-4],[-1,-2],[-6,2],[-3,-2],[-5,1],[-4,4],[-2,7],[-1,5],[0,2],[0,3],[-1,-1],[-2,-3],[-2,1],[-1,1],[-4,5],[1,4],[2,4],[4,7],[-1,1],[1,1],[1,1],[-1,2],[-2,2],[0,1],[0,2],[0,3],[0,2],[-2,1],[-1,-1],[-2,-4],[-1,-5],[1,-4],[1,-5],[-2,-3],[-6,-3],[-1,-5],[1,-3],[3,-3],[1,-4],[-1,-4],[-2,3],[-4,7],[-4,4],[-1,0],[-5,-2],[-2,1],[-2,3],[1,3],[5,5],[3,8],[1,1],[1,0],[3,3],[1,2],[0,1],[-4,0],[-3,-2],[-7,-6],[0,-2],[1,-2],[0,-1],[-6,0],[-3,2],[-3,3],[-1,2],[-1,1],[0,2],[0,2],[0,1],[2,2],[0,2],[-1,1],[-2,-1],[-2,-3],[-2,-1],[-3,3],[-1,3],[-1,5],[-1,4],[-1,2],[-4,1],[-5,6],[-3,1],[-1,-2],[3,-4],[5,-6],[1,-2],[0,-2],[0,-5],[1,-2],[1,-2],[1,-1],[4,-4],[5,-4],[2,-4],[-1,0],[-1,-1],[0,-1],[0,-5],[-3,-2],[-4,1],[-3,1],[-2,4],[-1,2],[-2,1],[-1,0],[-1,-1],[-1,-1],[-1,1],[0,2],[0,2],[-1,0],[-3,2],[-7,-1],[-3,1],[-2,2],[-5,6],[-2,3],[1,4],[-4,1],[-2,3],[1,2],[2,0],[-1,3],[-2,6],[-5,6],[-1,4],[3,5],[0,3],[-2,4],[-2,3],[-4,4],[-2,3],[-1,2],[-3,1],[-2,3],[0,5],[1,2],[4,7],[1,4],[-1,0],[1,2],[0,1],[-1,1],[-1,1],[-1,-1],[-1,-1],[1,-3],[-1,-2],[-2,-3],[-4,-7],[-2,-3],[1,-4],[2,-3],[7,-6],[2,-2],[1,-1],[2,-3],[1,-1],[-2,-7],[0,-4],[1,-3],[2,-3],[1,-1],[0,-1],[0,-5],[0,-2],[0,-1],[2,-1],[1,-1],[0,-2],[-1,-1],[-4,-3],[-3,2],[-4,5],[-2,2],[1,-4],[1,-3],[2,-3],[2,-3],[7,-5],[1,-2],[1,-4],[0,-2],[-2,1],[-4,5],[-6,5],[0,-2],[1,0],[0,-1],[-3,0],[-3,2],[-7,11],[-1,2],[0,2],[0,5],[0,3],[0,1],[-1,1],[0,3],[1,2],[0,1],[1,4],[-1,2],[-4,11],[-1,4],[1,5],[-3,0],[-2,-5],[0,-6],[1,-2],[3,-2],[1,-6],[-1,-5],[-2,-4],[-3,1],[-3,2],[-3,4],[-2,4],[-2,3],[-7,5],[-1,3],[0,5],[3,3],[3,1],[1,3],[-1,3],[-2,2],[-3,2],[0,6],[3,8],[1,4],[-3,2],[-2,0],[0,-1],[0,-2],[-1,-4],[-1,-7],[-1,-2],[1,-3],[2,-9],[-2,-1],[-9,1],[-2,2],[-4,5],[-2,2],[0,-2],[2,-2],[2,-3],[-1,-2],[-2,-1],[-3,2],[-4,8],[-12,12],[-2,4],[-1,4],[-1,4],[-1,7],[0,2],[-1,1],[-2,1],[-1,-1],[0,-2],[1,-5],[0,-2],[0,-1],[0,-2],[-1,0],[-1,2],[-2,1],[-3,2],[-1,1],[-2,1],[0,2],[0,3],[0,2],[-4,5],[-1,3],[2,4],[-3,3],[-4,2],[-3,4],[-1,6],[-2,5],[-3,3],[-2,4],[2,6],[-3,-1],[-5,-4],[-3,0],[-2,2],[-1,4],[0,5],[-1,4],[-2,-2],[-1,0],[-1,1],[0,2],[0,4],[-3,5],[-7,6],[1,-5],[5,-8],[1,-3],[0,-3],[1,-4],[5,-5],[1,-5],[-2,-9],[1,-5],[1,-3],[1,-4],[1,-3],[8,-14],[3,-9],[2,-10],[-3,-6],[-4,0],[-6,7],[-4,1],[-1,1],[-1,3],[-1,4],[-1,5],[-1,2],[-9,10],[-6,5],[-2,3],[-2,2],[-5,0],[-2,3],[-1,3],[-1,4],[-1,3],[-2,2],[1,-5],[-1,-2],[-3,1],[-1,2],[-1,3],[-5,-2],[-2,3],[2,2],[1,3],[-2,4],[-2,8],[-2,3],[-2,2],[-3,0],[0,-2],[3,-7],[1,-4],[-1,-2],[-3,0],[-5,4],[-3,1],[-2,1],[-1,4],[-1,4],[-1,3],[-1,2],[-5,5],[-1,1],[-2,0],[-2,1],[-1,0],[0,-2],[0,-1],[-1,0],[-3,0],[-18,10],[-2,1],[0,-3],[1,-1],[3,-2],[0,-2],[0,-3],[-1,0],[-2,0],[-1,2],[-3,-3],[-12,5],[-3,2],[-2,1],[-8,0],[0,-3],[1,-2],[8,-9],[4,-7],[6,-6],[2,-3],[0,-3],[-4,-1],[-2,-3],[2,-1],[2,-1],[4,-1],[1,-2],[1,-2],[1,-4],[1,-2],[5,-7],[3,-3],[5,-3],[7,-7],[3,-6],[5,-1],[7,0],[-1,-2],[-1,-1],[-4,0],[1,-2],[5,-8],[5,-3],[3,-4],[5,-9],[4,-3],[1,-2],[1,-2],[1,-2],[-1,-2],[-1,-1],[0,-2],[1,-1],[2,0],[2,-2],[4,-3],[1,-2],[-1,-3],[-1,-2],[-1,0],[-2,0],[2,-3],[4,-2],[2,-2],[2,-4],[1,-1],[2,-1],[2,0],[6,2],[5,-3],[8,-2],[3,0],[-1,-3],[-1,-1],[-1,-1],[-1,-5],[-4,-7],[2,-2],[3,1],[2,2],[2,2],[-1,5],[3,3],[3,0],[8,-8],[4,0],[9,4],[5,0],[3,-2],[1,-3],[-1,-4],[-3,-1],[-9,0],[-2,-2],[-1,-2],[-1,-3],[-4,-2],[-2,-3],[4,0],[0,-3],[0,-2],[2,-1],[1,0],[0,-2],[-1,0],[-1,-1],[-1,-2],[-1,-2],[4,0],[2,0],[1,1],[0,2],[0,3],[-2,2],[-1,2],[7,7],[2,3],[3,0],[2,-1],[3,0],[2,3],[2,1],[2,-2],[2,-3],[2,-3],[-1,-3],[-2,-2],[-3,-1],[-3,-1],[-2,0],[-3,-3],[-3,-2],[1,-3],[-2,-2],[-1,-1],[-1,-2],[0,-2],[1,-1],[1,0],[1,0],[0,-2],[1,4],[6,6],[3,3],[2,-2],[2,1],[6,5],[2,0],[2,-1],[2,-1],[3,-3],[0,-1],[0,-2],[-2,-2],[0,-1],[-1,-1],[1,0],[0,-2],[3,2],[2,1],[2,-1],[7,-9],[2,-2],[3,-1],[0,-1],[-14,-5],[0,-2],[5,0],[3,0],[2,2],[-1,-2],[5,5],[3,1],[2,-1],[3,-5],[-2,-4],[-3,-3],[-3,-3],[2,1],[4,2],[1,0],[1,-3],[2,1],[4,3],[2,-4],[1,-2],[1,-2],[-1,-2],[-2,-5],[-1,-2],[2,0],[3,3],[4,0],[0,-1],[0,-1],[0,-1],[2,-3],[1,-2],[0,-2],[-1,-3],[-4,-5],[1,-1],[4,2],[4,0],[8,-1],[-1,-1],[-4,-2],[-2,-1],[-1,-2],[-3,-4],[3,0],[25,7],[4,-3],[-2,-1],[-1,-2],[-2,-2],[0,-4],[9,4],[2,-2],[-1,-1],[-4,-2],[1,-2],[1,-1],[2,-1],[3,2],[2,0],[1,-1],[2,-1],[0,-1],[-1,-1],[0,-1],[2,-5],[6,-5],[1,-6],[2,-4],[3,-1],[2,-2],[-2,-7],[0,-4],[-2,-6],[0,-3],[2,-1],[2,2],[2,3],[2,1],[9,2],[1,-1],[-1,-3],[-4,-3],[8,-8],[-1,-2],[-1,-1],[-1,0],[-4,-1],[-1,-1],[-1,-3],[-1,-1],[-3,1],[-2,-1],[-1,1],[-1,0],[0,-1],[0,-2],[0,-1],[0,-1],[5,-4],[10,-12],[4,1],[-4,9],[-2,5],[2,2],[5,2],[2,0],[1,-1],[2,-4],[1,-2],[1,5],[1,2],[1,0],[1,-2],[0,-1],[1,-2],[0,-1],[2,-1],[2,-2],[1,-1],[0,-1],[-1,-1],[0,-1],[0,-2],[1,-3],[0,-1],[1,-1],[0,-3],[-1,-1],[-1,2],[-2,2],[-3,0],[-6,0],[10,-6],[13,-3],[3,3],[-1,0],[3,2],[3,-2],[7,-9],[0,-2],[-9,-5],[-3,-4],[-1,-2],[-1,-1],[0,-2],[0,-3],[0,-1],[2,-6],[1,-1],[-1,-3],[-1,-1],[-2,0],[-2,2],[-1,0],[-1,1],[-2,0],[-2,0],[-3,9],[-2,2],[-3,-4],[0,-1],[1,-1],[1,0],[1,2],[1,-1],[0,-1],[1,-2],[6,-12],[0,-2],[1,-3],[-1,-2],[-3,-1],[-1,-1],[3,-2],[3,-1],[3,-1],[0,-3],[2,-5],[1,-1],[1,0],[3,2],[1,-2],[1,-2],[-1,-2],[0,-2],[3,-2],[1,-2],[1,-2],[0,-3],[-3,-3],[-7,1],[-1,-2],[-2,-1],[-12,0],[-12,-4],[-7,4],[-8,0],[-4,2],[-5,3],[-3,1],[-3,0],[-2,1],[-3,5],[-1,1],[-1,-1],[-2,-1],[-1,0],[-2,0],[-2,4],[1,2],[4,4],[-2,2],[-5,-2],[-2,0],[-1,1],[0,2],[-2,3],[0,1],[-3,2],[1,3],[-3,1],[-10,1],[-7,5],[-3,1],[2,-2],[1,-1],[-2,-5],[-4,2],[-6,8],[-2,-1],[-3,-3],[-2,-1],[-2,1],[-2,1],[-12,13],[-1,1],[-1,0],[-1,-4],[0,-1],[-7,-2],[-1,2],[0,2],[-1,3],[-1,3],[-1,2],[2,1],[1,2],[-1,2],[-7,2],[-11,-1],[-3,1],[-3,4],[-4,3],[-3,-1],[-4,-3],[-3,-1],[-10,1],[-9,4],[1,2],[1,1],[0,1],[2,0],[0,2],[-2,1],[-1,0],[-2,-1],[-1,-3],[-1,0],[-1,0],[-2,1],[-10,4],[-3,0],[-3,-1],[-1,0],[-2,1],[0,1],[-1,1],[-1,2],[-1,1],[0,-1],[-2,-2],[-1,-1],[-50,8],[-6,4],[-2,2],[-4,2],[-2,2],[-4,3],[-1,1],[0,2],[1,1],[1,0],[0,2],[-12,3],[-1,2],[-1,4],[-1,3],[-3,1],[-2,-4],[-1,-1],[-1,0],[-2,1],[-2,2],[-1,1],[-2,2],[-5,-2],[-2,0],[-20,16],[-10,12],[-2,11],[-1,0],[4,19],[0,1],[-1,1],[-2,1],[-1,-8],[0,-1],[-1,-2],[-2,-3],[-1,-1],[-1,-3],[0,-2],[-1,-2],[-1,-2],[-1,2],[0,2],[0,1],[-1,2],[0,1],[-1,1],[-1,2],[0,2],[0,2],[1,2],[0,1],[-1,2],[-1,1],[-4,3],[0,-3],[1,-2],[1,-1],[1,-3],[0,-2],[-1,-2],[0,-2],[2,-2],[-3,-2],[-1,3],[-2,4],[-1,2],[-2,1],[-2,3],[-4,6],[-4,6],[0,3],[2,2],[3,1],[8,-1],[0,2],[0,1],[-1,2],[3,0],[2,1],[1,3],[-7,0],[-5,-2],[-2,1],[-2,2],[4,5],[4,2],[9,0],[2,0],[1,1],[1,2],[-1,2],[-1,1],[-7,-3],[-2,1],[-1,2],[-2,1],[-4,-3],[-3,1],[-5,4],[0,1],[-1,1],[-1,-1],[-1,-2],[1,-4],[-1,-1],[-1,-1],[-4,-2],[-1,0],[2,8],[-5,1],[-5,2],[-4,4],[-4,5],[-1,-1],[0,-1],[0,-2],[3,-6],[1,-2],[2,-1],[2,-1],[1,-3],[-1,0],[-1,-2],[1,0],[1,-3],[1,-1],[-3,-1],[-1,1],[-2,5],[-2,2],[-2,0],[-4,-1],[-2,1],[-2,2],[-3,7],[-4,-6],[-2,-2],[-8,-5],[-2,0],[-2,0],[-5,2],[-2,1],[-9,-1],[-3,1],[-8,8],[-3,0],[-2,2],[-5,7],[-3,2],[-2,1],[-1,2],[-1,2],[0,2],[-2,0],[-3,0],[-17,5],[-13,1],[-3,3],[0,-2],[-7,4],[-1,1],[3,3],[8,-1],[4,1],[-4,1],[-8,3],[-5,1],[-5,-1],[-8,2],[-1,1],[0,2],[2,1],[7,-1],[2,1],[0,4],[-2,3],[0,2],[3,3],[-2,3],[-2,0],[-2,-1],[-2,-2],[-1,-2],[0,-2],[0,-2],[-1,-1],[-1,0],[-2,-1],[-7,-1],[-4,0],[-1,2],[0,3],[1,3],[2,3],[1,1],[10,-1],[1,0],[1,5],[1,1],[1,1],[3,1],[1,1],[0,2],[1,2],[0,2],[1,2],[3,1],[2,1],[4,2],[3,3],[1,0],[0,1],[0,1],[0,2],[-4,-1],[-3,-3],[-3,-2],[-1,-1],[-1,0],[0,1],[-1,2],[0,2],[-1,2],[1,1],[4,2],[1,2],[-3,2],[-5,-3],[-3,-5],[0,-5],[0,-3],[-1,-3],[-2,-2],[-1,-2],[-9,-1],[-2,1],[-1,2],[-1,3],[-1,1],[-1,-1],[-4,-8],[0,-3],[-1,-9],[-1,-2],[-2,0],[-3,0],[-2,1],[0,2],[3,2],[-1,2],[-3,1],[-3,1],[-7,1],[-3,1],[-3,5],[2,2],[2,4],[0,4],[-2,2],[-1,-1],[-3,-2],[0,-1],[-2,-1],[-4,5],[-7,1],[-7,4],[-7,7],[-3,1],[1,3],[1,1],[4,-1],[0,2],[-6,5],[-5,2],[-1,1],[2,3],[1,1],[8,0],[2,1],[0,3],[-3,2],[-11,-3],[0,1],[1,1],[3,1],[-3,3],[0,2],[0,3],[-1,4],[-3,3],[-3,3],[6,0],[3,0],[2,2],[-1,0],[-1,2],[0,4],[-2,5],[-2,3],[-2,-3],[0,-3],[-3,0],[-13,1],[-3,2],[-1,6],[-1,1],[-1,0],[-1,-2],[0,-5],[0,-1],[-1,0],[-8,1],[0,1],[0,1],[1,1],[1,1],[0,1],[-3,1],[-4,3],[-2,2],[2,1],[3,0],[2,-1],[1,-1],[0,-3],[2,1],[2,4],[2,0],[6,-2],[11,0],[21,-5],[8,1],[4,2],[2,4],[-2,3],[3,2],[15,4],[2,3],[-1,2],[0,1],[3,3],[7,3],[5,7],[0,1],[0,1],[-1,2],[0,1],[1,2],[2,1],[4,0],[-3,4],[-4,0],[-4,-2],[-2,-6],[1,-2],[0,-1],[-1,-1],[-5,0],[-2,0],[-2,2],[5,6],[1,2],[0,4],[-2,2],[-2,1],[-2,0],[1,-4],[1,-1],[-2,-1],[-1,-1],[-4,0],[1,-2],[1,-1],[1,0],[0,-2],[-3,-1],[-6,-1],[-5,0],[-3,3],[2,7],[2,1],[4,0],[3,1],[-2,3],[-7,0],[-2,3],[1,1],[1,3],[0,2],[0,3],[1,1],[2,2],[1,0],[0,2],[-2,0],[-2,-3],[-2,0],[-1,0],[0,2],[1,2],[0,1],[-2,1],[-3,0],[-2,-1],[-2,-3],[1,-3],[3,-5],[1,-3],[1,-3],[-1,0],[-4,2],[-2,4],[-5,4],[0,2],[2,0],[1,3],[0,4],[2,3],[-3,3],[-2,-4],[-4,-11],[-1,1],[0,1],[-1,1],[0,2],[1,4],[-1,4],[-6,8],[-1,1],[-2,1],[-5,0],[-1,2],[1,1],[1,1],[0,1],[-4,3],[-4,-3],[-3,-6],[-3,-7],[1,-1],[1,-1],[2,2],[3,4],[3,1],[-1,-2],[0,-1],[2,-1],[5,2],[3,-1],[-1,-3],[-2,-6],[-1,-1],[-1,-1],[-1,-3],[1,-3],[1,-2],[-2,-2],[-2,-3],[-2,-2],[-2,1],[-2,-1],[-1,0],[0,2],[1,2],[1,1],[2,2],[1,1],[1,2],[-2,3],[-1,3],[-2,1],[-6,-12],[-2,-1],[-2,3],[1,3],[-1,3],[0,4],[-1,2],[-1,2],[-1,-2],[-1,-2],[-3,-1],[-2,1],[-2,2],[-3,6],[-1,4],[-1,1],[-2,0],[0,-2],[0,-3],[0,-3],[1,-1],[-1,-5],[-1,-2],[0,-3],[1,-6],[-2,0],[-10,1],[0,1],[0,15],[-2,2],[-3,0],[0,2],[2,3],[2,1],[1,0],[2,0],[5,-4],[1,1],[-1,3],[-1,4],[-1,3],[2,2],[1,2],[3,11],[2,2],[2,1],[4,0],[2,1],[5,6],[-8,-3],[-3,-1],[-2,1],[-2,2],[-1,3],[1,3],[-2,6],[-3,-4],[0,-2],[0,-1],[-1,0],[0,-1],[0,-1],[1,-2],[1,-3],[0,-1],[0,-3],[-2,0],[-2,2],[-2,3],[-2,-7],[-3,-4],[-3,-3],[-3,-4],[3,-3],[2,-1],[0,-3],[0,-4],[-2,0],[-3,1],[-8,0],[-4,1],[-2,2],[2,2],[5,0],[1,3],[-3,1],[-10,7],[-2,3],[-3,-1],[-2,-2],[-3,1],[3,5],[0,4],[-2,3],[-1,6],[4,2],[2,2],[1,3],[0,3],[-1,2],[0,2],[0,2],[2,7],[-2,1],[-5,-4],[-2,0],[0,4],[0,3],[1,4],[1,4],[-1,1],[-1,2],[-1,0],[0,-3],[-1,-4],[0,-5],[-1,-2],[0,-2],[-1,-2],[1,-1],[1,-2],[1,0],[2,-1],[1,-3],[-2,-2],[-1,-1],[-2,-2],[-2,0],[-2,1],[0,2],[-1,3],[0,2],[-4,4],[-8,2],[-4,3],[1,1],[1,0],[1,0],[0,2],[-2,1],[-5,-1],[-1,2],[-1,3],[0,2],[2,2],[2,1],[1,0],[1,0],[1,1],[1,3],[-1,2],[-1,1],[-1,0],[0,1],[-1,4],[0,3],[0,3],[-2,3],[-2,2],[-3,1],[-6,0],[-5,-2],[-2,0],[0,4],[0,3],[2,2],[1,2],[-1,5],[-5,2],[-2,2],[-1,1],[1,2],[0,1],[1,0],[1,0],[-3,3],[-9,1],[-1,7],[-1,2],[-1,0],[-2,-1],[1,-2],[-1,-2],[-2,0],[-4,0],[0,-1],[2,-1],[2,-3],[1,-4],[-2,-4],[-3,-1],[-3,1],[-4,2],[-2,3],[0,2],[1,5],[-2,5],[1,1],[1,2],[1,1],[2,0],[1,1],[0,2],[0,2],[1,1],[0,2],[1,1],[1,-1],[2,-3],[1,-1],[1,0],[1,0],[1,1],[0,1],[-1,1],[-5,6],[-2,1],[-3,-1],[-1,-6],[-2,-1],[-3,0],[-7,8],[3,9],[3,3],[2,-1],[3,-3],[3,1],[3,3],[3,4],[-3,2],[-2,-1],[-2,-3],[-3,-1],[-3,1],[0,3],[2,4],[12,8],[2,1],[1,1],[1,4],[0,2],[1,1],[5,2],[3,1],[1,1],[0,3],[0,1],[0,3],[-1,2],[-1,-1],[-2,-3],[-2,-2],[-3,-1],[-2,2],[-1,2],[0,1],[0,1],[0,3],[-1,4],[0,2],[-1,1],[-1,2],[0,1],[-1,0],[0,-1],[-1,-1],[-1,0],[-1,-2],[-1,-1],[-1,-2],[3,-4],[2,-7],[1,-6],[-1,-7],[-2,-2],[-11,-3],[-2,-1],[-2,-1],[-2,1],[1,4],[0,4],[-1,3],[-1,3],[0,1],[1,0],[2,-1],[1,-1],[3,-4],[2,1],[0,1],[1,1],[0,1],[-5,4],[-1,2],[0,2],[1,1],[2,0],[-3,3],[-2,0],[-5,-5],[-3,-1],[-3,0],[-2,1],[1,4],[-1,3],[-2,4],[-2,4],[-3,2],[-3,-1],[0,-1],[1,-3],[0,-4],[0,-4],[-1,-2],[-1,-2],[-1,-3],[3,-1],[2,-1],[-2,-3],[-10,-2],[1,-1],[0,-2],[-1,0],[-1,0],[-1,1],[-1,1],[-1,0],[-3,4],[-3,2],[-4,7],[-2,2],[-3,-1],[-1,-1],[-1,-3],[1,-2],[5,-2],[-1,-5],[2,-3],[2,-3],[1,-3],[1,-3],[1,-1],[3,-1],[1,-1],[-1,2],[1,-1],[1,-1],[0,-1],[1,-2],[-2,-2],[-1,-1],[-1,0],[-3,5],[-2,2],[-2,1],[-1,-2],[0,-2],[1,-2],[0,-3],[-2,-1],[-2,1],[-4,3],[-2,1],[1,-5],[-1,-4],[0,-4],[3,-4],[-2,-1],[-1,-2],[-3,-1],[0,-2],[0,-2],[0,-4],[0,-5],[-1,-2],[-2,-1],[-2,2],[-3,1],[-2,-2],[1,1],[-2,0],[-2,0],[-1,2],[0,2],[-2,1],[-2,0],[-1,1],[-2,3],[5,-1],[2,1],[2,1],[-1,2],[0,2],[-1,3],[4,4],[-3,3],[-5,4],[-3,4],[9,-2],[3,-4],[2,-2],[1,3],[-1,5],[-8,18],[5,-1],[9,-5],[5,-1],[0,2],[-2,3],[-3,2],[-10,6],[-1,1],[-6,-2],[-2,2],[0,1],[4,4],[15,-2],[-2,4],[-6,5],[-1,5],[0,1],[1,6],[1,5],[1,6],[0,2],[-1,3],[-2,3],[-2,1],[-2,-1],[-1,-4],[2,-7],[-1,-4],[-2,-1],[-2,1],[-2,0],[-2,-5],[-2,0],[-1,3],[-2,-3],[0,-2],[0,-3],[2,-6],[0,-3],[-3,-6],[-4,6],[-2,3],[0,3],[2,3],[1,1],[0,3],[-1,2],[-2,1],[-1,2],[-1,1],[-2,-2],[0,-9],[-1,-3],[-3,1],[-4,6],[-3,2],[2,-5],[2,-5],[3,-4],[2,-3],[-3,-5],[-3,0],[-2,4],[-2,5],[1,0],[1,2],[0,2],[-1,0],[-2,0],[-4,-2],[-2,-1],[-3,1],[-2,1],[-1,1],[-1,2],[0,5],[-1,2],[-2,2],[-5,2],[-2,3],[-1,3],[-6,8],[1,2],[1,1],[0,1],[1,1],[1,0],[1,0],[1,2],[1,3],[0,2],[4,3],[4,9],[2,2],[5,-3],[3,0],[1,3],[-3,3],[-7,1],[-1,3],[1,1],[1,0],[1,2],[0,2],[-2,0],[-9,1],[-2,-1],[1,-2],[1,-3],[1,-3],[-3,1],[-7,-1],[-2,-2],[0,-3],[2,-5],[2,-5],[0,-3],[-1,-3],[-2,-1],[-1,0],[-6,7],[-1,1],[-2,0],[0,-1],[-1,-2],[-1,-2],[-1,-1],[-2,0],[-4,4],[-2,1],[-6,0],[-2,0],[3,-7],[4,-2],[5,-1],[4,-3],[1,-1],[0,-3],[0,-2],[1,-1],[1,-1],[1,-2],[2,-2],[5,-1],[3,-2],[15,-14],[14,-16],[-4,-4],[-2,-1],[-3,0],[-6,2],[-3,0],[-2,-4],[7,0],[3,-2],[2,-4],[-2,-2],[-2,1],[-1,3],[-2,1],[-2,-1],[0,-3],[1,-3],[1,-1],[3,-2],[5,-1],[2,-2],[1,-1],[-2,-2],[-2,-2],[0,-4],[-2,-1],[-4,1],[-2,-1],[0,-4],[-6,-1],[-2,-2],[0,-3],[0,-2],[-4,1],[-5,5],[-3,1],[0,-1],[0,-3],[0,-1],[-3,0],[-4,3],[-4,4],[-3,4],[-2,1],[-2,-1],[-1,-2],[-2,-1],[-2,1],[-3,5],[-11,6],[-3,4],[-2,2],[-2,0],[-1,-2],[0,-2],[2,-1],[2,-1],[0,-4],[-1,0],[-2,0],[-2,-1],[1,-2],[-1,-1],[-2,-1],[-1,1],[-1,1],[-1,4],[-1,1],[0,1],[-2,0],[-5,4],[-2,2],[-4,6],[-1,0],[-3,1],[-7,5],[-3,1],[-6,0],[-3,2],[-1,6],[-5,-2],[-5,0],[-5,3],[-10,9],[-3,1],[-2,-2],[0,-3],[2,-3],[2,-2],[2,-1],[0,-1],[-3,0],[-2,-1],[-1,-1],[1,-1],[1,-1],[0,-3],[1,-3],[1,-1],[3,-2],[4,-1],[5,-6],[4,-3],[2,-1],[1,-3],[0,-1],[-1,-1],[-1,0],[-1,0],[1,0],[-3,1],[-5,4],[-2,-1],[2,-3],[3,-2],[-1,-1],[-2,1],[-3,3],[-13,7],[-3,1],[-3,-1],[1,-1],[1,0],[2,-1],[3,-3],[1,0],[1,-1],[3,-5],[2,-1],[-1,-1],[-1,0],[-1,-1],[1,-2],[11,-9],[-2,-3],[-4,1],[-7,2],[-1,-1],[0,-2],[-1,-1],[-2,2],[-4,3],[-2,0],[-2,-1],[-1,-2],[-1,8],[-3,-1],[-5,-5],[-2,-1],[-2,-2],[-1,-1],[-3,1],[-1,1],[0,2],[-1,2],[-2,0],[1,0],[-6,-4],[1,0],[1,0],[1,-1],[1,0],[-3,-4],[-8,3],[-6,-4],[-7,3],[-3,-2],[2,-2],[4,-6],[3,-2],[7,2],[3,-1],[2,-4],[-14,-5],[-4,3],[-3,4],[-5,3],[-4,0],[-4,-2],[2,-1],[3,0],[2,0],[2,-2],[2,-3],[2,0],[-3,-4],[-6,-1],[-4,2],[-1,6],[-6,2],[-2,-2],[-1,1],[-1,1],[0,1],[-1,0],[-1,-1],[-2,-5],[-2,-1],[-5,-1],[0,3],[1,0],[-4,3],[-2,-2],[0,-5],[2,-5],[0,-4],[1,-2],[2,-1],[2,0],[1,-1],[0,-3],[-1,-3],[-2,-2],[-2,1],[-4,2],[-3,1],[-9,0],[2,2],[2,1],[-1,2],[-1,0],[0,2],[-4,0],[-4,2],[-7,7],[-2,-4],[-4,1],[-8,5],[-3,4],[-2,1],[-1,-2],[-2,-2],[-8,-3],[-14,-8],[-3,2],[0,2],[1,5],[0,1],[-1,1],[-2,2],[-1,0],[-1,0],[-1,-1],[-2,0],[-1,1],[2,2],[4,1],[2,2],[-3,3],[-1,1],[-1,-1],[-1,-1],[-1,-2],[-12,0],[0,2],[3,3],[3,4],[-2,1],[-5,1],[-10,7],[-1,1],[-1,-4],[1,-5],[0,-6],[-3,-2],[-1,1],[0,2],[-1,0],[-3,2],[-1,0],[-1,0],[-1,-1],[-1,1],[-1,1],[0,3],[-1,1],[-9,2],[-2,1],[-4,7],[1,1],[3,-1],[1,0],[1,1],[2,1],[1,2],[-1,1],[-5,0],[-5,-2],[-3,-1],[-1,-1],[-1,1],[-2,2],[-2,1],[-1,1],[0,1],[-1,2],[1,1],[1,0],[4,3],[0,2],[-9,7],[-3,5],[1,4],[0,2],[-2,2],[-7,2],[-3,3],[-1,2],[0,2],[1,1],[5,0],[0,1],[-2,2],[-2,1],[-1,3],[1,4],[-1,2],[-1,1],[-1,1],[-1,-1],[1,5],[1,3],[3,4],[-4,2],[-12,0],[3,2],[10,2],[-1,1],[0,1],[1,0],[-1,5],[2,4],[2,2],[3,1],[1,0],[0,2],[1,2],[-1,1],[0,1],[1,2],[4,0],[0,3],[0,3],[1,2],[-1,3],[-1,2],[-2,1],[-2,1],[-1,2],[-1,4],[-1,11],[0,1],[0,1],[1,2],[2,2],[3,2],[3,4],[2,2],[3,3],[4,1],[1,1],[0,1],[0,2],[0,1],[1,1],[1,1],[8,4],[1,0],[7,6],[7,1],[6,3],[3,3],[2,3],[3,3],[5,1],[8,-2],[4,1],[6,4],[6,1],[3,2],[2,3],[2,3],[-7,11],[-7,7],[-3,5],[-4,5],[-2,2],[1,3],[4,9],[2,2],[16,-5],[3,4],[-3,5],[-10,9],[-4,5],[5,4],[18,0],[-1,-1],[-1,-1],[-1,0],[-1,0],[0,-2],[9,-1],[8,-3],[0,-4],[4,-2],[13,-1],[2,1],[0,3],[2,1],[12,-1],[2,-1],[4,-4],[1,-1],[10,2],[32,-15],[31,-15],[3,-1],[6,1],[3,-1],[3,-5],[1,0],[5,-1],[3,0],[5,-4],[5,-1],[3,-1],[2,-3],[2,-2],[0,-1],[-1,-4],[1,-1],[1,-3],[1,-3],[0,-1],[10,-4],[6,-4],[4,-1],[5,-3],[8,-8],[1,-3],[1,-4],[2,-4],[0,-4],[-3,-2],[1,-1],[2,-5],[-1,-2],[-2,-2],[-2,-1],[-3,-1],[-2,-3],[-2,-1],[-4,0],[-9,3],[-4,-2],[2,-2],[2,-2],[3,-2],[0,-3],[1,-6],[0,-3],[2,-3],[2,-1],[8,1],[2,-1],[1,-1],[1,-1],[0,-1],[0,-1],[0,-2],[-1,-4],[-2,-1],[-4,0],[0,-1],[-1,-1],[0,-1],[-1,-2],[1,0],[-1,-1],[3,-2],[4,0],[3,-1],[2,-4],[0,-1],[2,0],[1,-2],[1,-2],[-1,-2],[3,-1],[5,1],[4,2],[1,4],[-2,2],[-3,4],[-1,2],[-1,3],[0,2],[-1,5],[1,1],[-2,2],[0,4],[1,3],[1,3],[2,2],[-2,1],[0,3],[0,3],[1,2],[-3,0],[-2,-1],[-7,-9],[-1,-1],[-1,0],[-3,1],[-2,1],[-3,-2],[-2,0],[-2,2],[3,2],[7,2],[0,4],[2,5],[3,3],[3,1],[3,1],[-1,5],[3,1],[6,-3],[4,-3],[2,-1],[-1,4],[-1,2],[-1,2],[0,3],[2,2],[1,0],[2,1],[0,6],[1,4],[2,2],[2,1],[1,1],[0,5],[-1,3],[-1,1],[-5,0],[-1,-3],[0,-3],[0,-5],[-1,-4],[-2,-3],[-2,-2],[-3,-2],[-2,-2],[-1,-1],[-2,2],[-1,2],[-1,7],[-1,3],[-4,5],[-7,10],[-3,5],[-4,5],[-9,4],[-4,4],[-2,-2],[-1,-1],[-1,1],[-2,2],[1,0],[1,1],[0,1],[1,1],[-5,6],[-6,3],[-12,2],[1,3],[0,2],[-1,1],[-3,2],[0,2],[-1,3],[0,2],[26,-3],[1,0],[3,3],[2,1],[1,2],[1,1],[0,1],[3,0],[2,-2],[0,-3],[-1,-3],[3,-5],[4,-3],[27,-2],[11,-3],[2,0],[5,3],[2,0],[3,3],[2,6],[1,6],[-1,3],[-1,9],[-1,2],[-1,1],[1,1],[2,1],[2,-1],[10,-4],[1,0],[1,0],[2,2],[1,1],[7,2],[2,-1],[4,-2],[2,-1],[2,1],[2,1],[2,2],[0,2],[1,4],[1,2],[2,2],[1,-2],[2,0],[2,0],[0,-2],[0,-1],[-1,-1],[-3,-3],[3,-1],[1,-1],[1,-3],[-1,1],[-1,-1],[-1,0],[-1,-1],[4,-7],[6,-3],[7,-1],[5,2],[8,9],[2,3],[6,2],[2,3],[2,9],[2,5],[3,2],[12,2],[3,2],[-3,2],[-1,2],[0,2],[1,2],[4,0],[1,1],[2,1],[1,0],[3,0],[3,0],[1,0],[1,-1],[1,-3],[5,-3],[3,0],[5,6],[4,1],[11,-3],[5,-2],[-1,-1],[0,-3],[1,-3],[2,-3],[3,-2],[2,-2],[3,1],[6,3],[2,-1],[2,-3],[4,-1],[3,0],[2,1],[-3,5],[1,2],[0,2],[0,1],[-1,0],[-4,7],[-6,15],[-4,5],[0,2],[1,1],[1,1],[0,3],[0,3],[-2,2],[-2,1],[-2,-1],[-2,1],[-1,1],[0,2],[-1,3],[0,2],[0,4],[2,1],[3,0],[-2,3],[-2,1],[-1,3],[-1,4],[-1,2],[-7,6],[-5,4],[-3,1],[-6,2],[-5,3],[-3,1],[-4,3],[-8,12],[-6,4],[-6,7],[-2,7],[-2,3],[-2,0],[-21,22],[-4,7],[-2,4],[1,5],[1,3],[4,6],[9,9],[30,18],[3,5],[2,1],[1,1],[2,0],[1,1],[3,5],[1,1],[2,0],[5,0],[-1,2],[0,1],[1,2],[2,2],[3,1],[5,5],[4,2],[7,10],[30,22],[2,3],[3,10],[2,6],[3,4],[2,3],[2,1],[6,3],[4,3],[6,2],[7,6],[3,1],[10,0],[7,2],[-3,6],[-3,5],[-1,9],[5,12],[6,10],[4,4],[-1,0],[0,1],[0,1],[3,4],[4,16],[2,6],[8,6],[7,3],[22,2],[8,3],[7,5],[7,10],[0,2],[1,2],[0,2],[-1,6],[0,2],[2,6],[2,4],[6,3],[-13,8],[-7,6],[2,8],[-2,2],[-3,10],[-3,6],[-1,4],[0,5],[-8,6],[2,3],[6,4],[1,3],[-1,3],[-1,0],[-1,1],[-2,1],[0,2],[-2,5],[-1,1],[2,5],[-2,3],[-3,2],[-8,0],[0,1],[-1,2],[0,2],[-1,1],[-4,2],[-2,3],[-1,4],[1,5],[2,2],[6,4],[0,2],[0,3],[-1,2],[-1,3],[-1,2],[-1,2],[-2,1],[-2,0],[-1,1],[-2,2],[-3,4],[-2,3],[-1,2],[1,2],[-3,3],[-3,-2],[-2,-3],[-3,-2],[-3,3],[-3,8],[-3,3],[1,2],[3,4],[-2,1],[-3,-1],[-3,0],[1,4],[3,2],[1,1],[0,2],[0,2],[-1,2],[0,2],[0,2],[1,2],[1,3],[-1,0],[-1,1],[0,1],[-1,2],[0,1],[1,1],[0,1],[0,1],[-1,3],[-3,7],[-1,3],[-2,7],[-2,8],[0,4],[0,9],[0,2],[-1,2],[-2,0],[-2,-1],[-1,-1],[0,-2],[1,-2],[0,-2],[-2,-2],[-2,1],[-3,4],[-2,2],[-3,-3],[-1,0],[-2,4],[-1,2],[-1,-1],[-2,-1],[-1,0],[-2,2],[2,0],[1,1],[1,2],[0,2],[-1,3],[-1,0],[-2,0],[-1,1],[-2,1],[-4,1],[-1,0],[-1,-1],[-2,-1],[-1,-2],[0,-1],[2,0],[-1,-2],[-1,-2],[-2,-1],[-1,0],[-1,0],[-2,1],[-3,5],[-3,1],[-6,0],[2,2],[2,2],[2,1],[2,0],[1,1],[0,3],[0,1],[1,0],[3,1],[4,0],[2,0],[3,2],[1,1],[0,3],[-1,2],[-2,1],[2,4],[3,1],[7,0],[-5,4],[-2,1],[-5,0],[2,-3],[1,-1],[-2,-1],[-3,0],[-2,-1],[0,-3],[0,-5],[0,-2],[-2,-1],[-7,1],[-4,0],[-1,-3],[1,-1],[-7,-7],[-4,-3],[-5,0],[0,-2],[1,0],[0,-1],[1,-2],[-3,-2],[-1,1],[0,1],[-1,1],[0,1],[-6,3],[4,4],[2,1],[2,-1],[1,0],[2,0],[1,3],[-6,1],[-7,0],[-3,1],[-12,7],[-1,2],[-3,6],[-2,3],[-2,3],[-5,3],[2,3],[2,2],[2,3],[2,9],[2,3],[2,2],[3,1],[6,0],[3,2],[1,6],[-2,1],[-8,6],[6,3],[2,1],[2,6],[1,1],[-2,2],[-4,0],[-6,-1],[0,1],[10,5],[3,4],[-8,1],[-2,0],[1,2],[1,1],[3,2],[-10,2],[-5,4],[-8,-2],[-3,1],[-1,2],[-1,2],[-1,1],[-3,-2],[-6,0],[-1,1],[-3,2],[-2,1],[-3,-2],[-2,0],[2,-5],[1,-2],[1,-1],[6,0],[2,-1],[4,-6],[0,-2],[0,-1],[2,-1],[1,-3],[-1,-1],[3,-2],[2,-2],[5,-8],[5,-3],[1,-2],[0,-2],[-1,-1],[0,-1],[-2,-6],[-3,-3],[-5,-3],[-4,0],[-1,3],[1,3],[2,4],[-17,-2],[-5,-3],[-3,0],[0,3],[-4,2],[-13,1],[-2,1],[-1,3],[0,2],[-1,1],[-1,1],[-2,3],[-2,1],[-6,2],[-2,3],[-1,6],[0,4],[-1,3],[-1,1],[-3,1],[-2,1],[-8,9],[-5,2],[-2,3],[0,5],[0,3],[1,1],[11,2],[3,2],[2,3],[3,5],[-3,4],[-4,2],[-5,1],[-4,-2],[1,-2],[0,-1],[-2,0],[-2,-1],[-2,-1],[-2,-2],[4,0],[3,1],[4,0],[2,-4],[-2,-3],[-19,-3],[-1,0],[0,2],[0,2],[-1,1],[-6,0],[-8,3],[-2,0],[3,2],[9,1],[3,2],[0,2],[7,3],[7,1],[3,2],[2,3],[0,4],[-2,3],[1,2],[1,1],[1,0],[-4,4],[-5,1],[-16,-6],[-17,-2],[2,2],[18,4],[2,2],[-5,2],[-12,-5],[-5,2],[21,8],[-9,-2],[-5,-3],[-2,0],[-3,2],[2,2],[8,8],[1,1],[23,5],[-7,-4],[-2,-3],[2,1],[5,2],[2,1],[5,-1],[3,1],[2,3],[-2,4],[-2,2],[-4,1],[-2,1],[-7,5],[-3,2],[-2,-1],[-2,-1],[-5,-2],[-5,-3],[-1,-1],[-2,-1],[-1,-1],[-2,-4],[-1,-2],[-4,-1],[-2,-1],[1,-4],[0,-2],[-1,-2],[-2,-1],[1,-4],[-1,-5],[-4,-6],[-1,-4],[-3,-1],[-5,1],[-2,1],[-5,5],[-6,4],[-7,9],[-2,0],[-2,2],[-3,6],[-2,3],[-3,1],[-2,0],[-3,-1],[-2,-2],[-5,-6],[1,-4],[8,-8],[-6,-7],[-7,-5],[-14,-6],[-8,-6],[-15,-3],[-2,-1],[-5,-8],[-3,-2],[-34,-19],[-24,-6],[-10,2],[-6,11],[1,9],[11,13],[4,8],[0,7],[-2,1],[-4,0],[-4,2],[-2,5],[-1,4],[1,4],[2,5],[-1,3],[-1,4],[1,4],[1,2],[2,1],[5,2],[16,6],[30,-4],[3,-1],[5,-6],[3,-1],[8,0],[1,1],[2,2],[8,6],[7,2],[2,7],[3,1],[8,1],[7,-2],[4,1],[3,3],[0,2],[-1,4],[-1,2],[1,2],[3,2],[0,1],[-3,15],[1,3],[-1,1],[-10,13],[-7,4],[-6,5],[-8,4],[-4,1],[-1,0],[-3,4],[-1,2],[-1,1],[-10,2],[-13,7],[-6,1],[-16,-4],[-4,1],[-1,3],[-1,5],[-2,4],[-2,4],[-17,14],[-3,4],[-1,6],[2,5],[3,3],[4,3],[10,10],[2,1],[7,-4],[6,-1],[12,-5],[2,0],[0,2],[-2,2],[1,3],[4,5],[-20,0],[-3,2],[-2,2],[1,2],[-1,2],[-1,0],[-4,-2],[-15,0],[-5,-1],[-2,-4],[11,0],[2,0],[1,-1],[-2,-5],[-2,-4],[-2,-2],[-4,-1],[-1,0],[-1,1],[-1,1],[-1,-1],[0,-1],[0,-1],[0,-2],[0,-1],[-4,-8],[-3,-1],[-15,2],[-5,4],[0,2],[1,3],[1,2],[4,1],[1,2],[-3,-2],[-7,-1],[-5,-5],[-4,0],[-3,0],[-3,2],[0,2],[-1,2],[0,1],[-1,1],[-6,-1],[-3,0],[-7,4],[2,7],[5,3],[11,0],[1,1],[2,4],[1,0],[1,0],[1,-2],[2,-1],[2,1],[4,4],[2,0],[-1,-2],[-1,-3],[-3,-3],[8,1],[4,2],[5,7],[0,1],[0,1],[0,2],[1,2],[-4,2],[-4,0],[-3,0],[-2,5],[1,6],[5,1],[6,0],[4,3],[-2,4],[-3,0],[-4,-3],[-4,-1],[-23,0],[0,1],[5,4],[2,2],[2,3],[-6,-1],[-3,-1],[-3,-2],[-3,-4],[-2,-2],[-6,3],[-11,1],[-2,2],[3,6],[19,9],[12,3],[4,6],[-10,-2],[-17,-7],[-2,-2],[-2,-1],[-6,0],[-2,-2],[-3,-3],[-16,-2],[-2,-5],[-14,-5],[3,-1],[2,0],[6,3],[2,0],[2,-2],[-2,-1],[2,-2],[4,-2],[3,-3],[-3,0],[-6,2],[0,-6],[-3,-5],[-4,-2],[-4,0],[-3,3],[-1,6],[-1,5],[-3,5],[2,8],[0,10],[-2,11],[0,6],[-1,6],[1,6],[1,5],[1,6],[0,6],[-1,6],[-2,10],[1,7],[-2,4],[-4,4],[-4,4],[-2,4],[-1,0],[-1,0],[-2,-1],[-2,0],[-4,3],[-1,1],[-5,-1],[-9,-7],[-5,-2],[-1,1],[-6,6],[-4,2],[-5,0],[0,-2],[2,-1],[3,-3],[2,-1],[0,-1],[0,-2],[-1,-2],[-1,-1],[-7,0],[-3,2],[1,4],[-7,-2],[-1,0],[-2,-4],[-1,0],[-2,0],[-5,3],[-2,0],[1,1],[1,2],[-4,7],[-4,5],[-8,8],[1,1],[2,0],[1,1],[1,0],[1,2],[1,1],[5,-1],[3,1],[2,1],[-11,4],[-6,1],[-5,-3],[-1,-2],[1,0],[2,-4],[-2,-1],[-9,9],[0,2],[2,3],[1,1],[-6,0],[-2,1],[-4,11],[-1,3],[0,4],[-3,0],[-7,-2],[-3,2],[-1,4],[-1,2],[-1,-1],[-1,-1],[-1,-4],[0,-2],[1,-1],[1,-3],[1,-2],[1,-2],[-3,0],[-2,3],[-3,4],[-3,3],[-2,1],[-5,-1],[-1,0],[-4,5],[3,3],[-2,0],[-1,0],[-2,1],[-1,2],[2,0],[2,1],[1,1],[2,2],[-1,2],[-1,1],[-1,0],[-2,0],[0,2],[18,0],[5,3],[-3,1],[-6,-1],[-1,1],[-1,0],[-1,0],[-1,0],[0,2],[4,1],[9,8],[4,-1],[2,-1],[1,-2],[1,-2],[1,-3],[2,-2],[2,-1],[5,0],[-5,3],[-2,3],[-2,8],[-2,3],[-3,3],[-2,1],[-1,-1],[0,-2],[-2,0],[-5,0],[-3,1],[-2,2],[-2,3],[-1,4],[1,1],[-2,2],[-2,0],[-11,-8],[-1,-3],[-1,-1],[-1,0],[-1,-1],[1,-2],[3,-1],[12,4],[1,0],[1,-1],[0,-2],[-1,-1],[-1,-1],[-1,-4],[-1,-2],[-2,-2],[-5,-5],[-1,-2],[-1,-1],[-1,-4],[0,-1],[-1,-1],[-3,1],[-2,-1],[4,-2],[0,-1],[0,-1],[-1,-1],[1,-6],[0,-2],[0,-2],[-2,-3],[-1,-2],[-2,-1],[-1,-2],[-2,-1],[-2,1],[-4,3],[-2,2],[-5,0],[-2,1],[2,2],[1,-1],[1,-1],[1,0],[1,2],[0,1],[-1,1],[0,2],[1,5],[2,3],[3,2],[3,4],[0,1],[-1,1],[-1,1],[1,1],[0,2],[-2,0],[-7,-3],[1,-1],[1,-1],[1,-1],[1,-2],[-2,-2],[-1,-4],[-1,-4],[-3,-1],[-2,0],[-4,4],[-2,1],[-2,-2],[1,-2],[2,-3],[1,-2],[-2,-5],[-1,-1],[-2,1],[-5,4],[-3,1],[-3,-4],[-5,-2],[-1,-2],[-1,-4],[2,-3],[1,-2],[11,-5],[2,-2],[0,-2],[2,0],[2,2],[0,1],[5,2],[5,-1],[5,-3],[3,-5],[4,-4],[5,-2],[6,-1],[5,3],[0,5],[4,0],[9,-4],[3,-3],[2,-2],[6,-12],[12,-15],[1,-2],[1,-3],[-1,-7],[1,-2],[0,-2],[1,-2],[1,-1],[0,-1],[-1,-4],[0,-2],[0,-2],[1,0],[2,-4],[2,-1],[0,-1],[1,-1],[0,-1],[0,-1],[2,-4],[0,-2],[0,-3],[1,-6],[-1,-2],[-4,-9],[-1,-2],[-3,-2],[-3,-1],[-2,-1],[0,-1],[-1,-4],[-1,-1],[-3,0],[-2,0],[-3,2],[-2,1],[-31,-1],[-31,-1],[-28,-9],[-3,4],[-2,2],[-3,2],[-2,1],[-1,3],[-2,6],[-1,3],[-1,2],[-2,2],[-2,2],[-2,1],[-1,1],[-2,2],[-1,2],[-2,-1],[-7,-7],[-3,-1],[-3,2],[-1,3],[-1,0],[1,3],[1,4],[-3,2],[-16,1],[-5,0],[-2,-2],[-2,-2],[-1,-1],[-8,1],[-21,13],[-31,2],[-8,4],[-5,0],[-3,-1],[-5,-1],[-4,2],[-2,6],[-1,1],[-7,-4],[-3,1],[-5,2],[-41,3],[-5,3],[-10,3],[-4,0],[-2,-1],[-1,-2],[1,-2],[2,-4],[5,-5],[13,-2],[5,-4],[8,-6],[1,-2],[0,-2],[1,-1],[1,-1],[4,2],[7,7],[3,-1],[13,-8],[4,-4],[0,-3],[2,-5],[0,-3],[2,-2],[4,-3],[4,-4],[8,-4],[18,-20],[5,-2],[1,-3],[0,-3],[-1,-3],[-2,-2],[-4,-4],[-11,-4],[-2,0],[-2,1],[-1,1],[1,2],[2,1],[-15,7],[-5,6],[-4,7],[-2,2],[-5,2],[-6,2],[-3,2],[-5,6],[-1,1],[-3,1],[-7,7],[-5,3],[-8,0],[-2,1],[-2,3],[0,3],[3,3],[-4,2],[-14,-2],[-3,1],[-18,16],[-3,1],[-1,0],[0,1],[-1,1],[0,1],[0,1],[-18,14],[-6,1],[-3,1],[-5,5],[-2,0],[-3,-1],[-2,0],[-4,4],[-2,1],[-10,3],[-3,1],[-2,1],[-2,1],[-6,-1],[-2,2],[-3,5],[-28,11],[-8,8],[-5,3],[-5,1],[-5,1],[21,-18],[3,-1],[0,-1],[1,-1],[1,-3],[1,-1],[2,-1],[6,1],[3,0],[6,-4],[47,-19],[14,-10],[7,-3],[10,-12],[1,-1],[-1,-1],[1,-1],[1,0],[1,0],[1,0],[5,-5],[2,-1],[11,-2],[5,-3],[3,-4],[-7,-7],[0,-1],[1,-1],[3,-3],[0,-1],[-3,-1],[-5,4],[-3,-1],[-3,-1],[0,-1],[0,-4],[-12,-5],[-1,6],[-3,2],[-3,-1],[-2,-2],[-4,-5],[0,-1],[-4,-1],[-2,-1],[-1,-2],[1,0],[1,-1],[-1,-7],[-1,-1],[-13,5],[-5,0],[0,1],[1,0],[1,1],[1,1],[2,1],[-2,2],[-2,0],[-2,0],[-2,-2],[0,2],[3,1],[8,-2],[3,1],[-14,7],[-8,1],[-3,-4],[-2,-1],[-23,12],[-2,4],[-4,3],[-4,1],[-1,-3],[-4,1],[-8,6],[-7,3],[-10,8],[-1,0],[-1,0],[-1,0],[-1,1],[-1,1],[0,2],[0,1],[-1,0],[-10,-2],[-1,1],[-1,2],[-1,1],[-1,0],[-7,-4],[-1,1],[-1,2],[-1,1],[-10,-8],[-15,-3],[-10,-4],[-11,-2],[-40,7],[-3,3],[-3,-1],[-33,1],[-19,5],[-18,-1],[-5,4],[-9,2],[-2,1],[-1,4],[-3,7],[6,2],[3,3],[1,3],[-3,1],[-3,-2],[-6,-6],[-7,-4],[-22,4],[-1,1],[-2,2],[-1,0],[-1,0],[-4,-1],[-5,0],[-8,4],[-3,0],[-9,-3],[-8,2],[-32,-1],[-6,-3],[-2,-4],[2,-5],[3,0],[4,2],[21,5],[8,-4],[5,-1],[10,-5],[6,-2],[13,-8],[-2,-2],[-38,7],[-16,-6],[-6,1],[-27,14],[-28,14],[-28,28],[-4,2],[0,2],[0,6],[-1,7],[-2,5],[3,2],[0,3],[-2,3],[-2,1],[0,2],[1,2],[7,11],[2,2],[15,6],[5,2],[3,4],[0,4],[-2,3],[-3,3],[-3,2],[-3,-1],[5,-5],[1,-3],[-3,-3],[-2,-1],[-2,-1],[0,-1],[-1,-1],[-7,-3],[-3,-3],[-7,-2],[-2,0],[-6,-8],[0,-1],[-1,-1],[0,-1],[-3,-1],[-1,-2],[1,-3],[2,-5],[1,-3],[-25,0],[-2,3],[-2,5],[4,3],[9,1],[5,3],[-6,2],[-13,-2],[-4,5],[8,0],[-3,4],[-8,0],[-3,3],[2,1],[5,2],[1,2],[0,1],[-6,3],[-6,-1],[-12,-7],[0,-6],[-4,-3],[-9,-2],[1,-3],[2,-1],[3,2],[17,-1],[4,-2],[4,-5],[-1,-3],[0,-3],[1,-2],[1,0],[-1,-2],[-1,-3],[0,-1],[1,-1],[-3,-2],[-48,8],[-7,3],[-33,1],[-1,-1],[-1,-2],[0,-1],[1,-1],[17,-3],[3,-2],[0,-2],[-18,-8],[-9,-3],[-9,3],[0,1],[0,1],[1,0],[0,1],[-2,1],[-10,7],[-14,1],[-5,4],[-5,2],[-2,2],[3,4],[2,1],[2,0],[0,-1],[0,-2],[0,-1],[3,0],[10,0],[2,-1],[7,-7],[3,-1],[4,1],[7,4],[-3,2],[-6,1],[-21,15],[-22,7],[-8,7],[-1,-1],[-1,-1],[-2,-1],[-28,6],[-9,4],[-15,15],[-3,4],[-8,18],[0,2],[0,2],[-1,2],[-1,2],[-5,3],[-1,1],[0,2],[0,2],[-2,0],[-5,4],[-4,2],[-15,12],[-3,5],[-4,5],[-6,15],[-4,7],[3,3],[3,1],[3,0],[6,3],[13,1],[0,2],[-1,0],[0,1],[2,2],[1,0],[-2,3],[-2,2],[-12,5],[-15,12],[-3,1],[-2,1],[-4,6],[-2,1],[37,-3],[3,-3],[-1,-1],[0,-1],[-1,-2],[5,-3],[3,-1],[3,0],[23,5],[2,-1],[4,-2],[3,-1],[4,0],[7,3],[12,1],[2,-1],[6,-2],[12,-2],[6,-1],[4,-4],[1,-2],[1,-5],[0,-3],[2,-2],[1,-1],[49,-6],[9,5],[3,0],[7,0],[2,1],[3,1],[7,-1],[4,-2],[7,1],[19,-1],[-3,0],[3,0],[1,2],[2,7],[3,3],[2,2],[7,1],[19,-5],[3,0],[3,1],[-5,3],[-32,8],[0,1],[0,1],[-1,0],[-1,0],[-2,-1],[-1,0],[-2,1],[-1,1],[-1,2],[-2,2],[-9,2],[-1,1],[-3,3],[-12,5],[-4,1],[-1,1],[-6,4],[-11,11],[-1,2],[0,2],[3,4],[0,3],[-1,2],[-1,1],[2,2],[1,1],[-5,4],[-7,-1],[-11,-8],[-5,-6],[-2,0],[-3,0],[-8,4],[-38,4],[-11,-2],[-46,8],[-45,7],[-45,7],[-4,3],[-4,3],[-2,2],[-1,4],[-1,6],[-1,1],[-3,1],[-1,1],[0,1],[0,2],[-2,4],[-1,2],[-1,1],[0,2],[1,1],[1,0],[0,6],[-1,3],[-2,3],[4,0],[2,1],[0,3],[-4,7],[0,3],[-5,2],[-1,3],[1,2],[4,1],[1,2],[0,1],[-1,2],[0,1],[3,6],[8,5],[3,6],[0,1],[5,3],[1,3],[-1,2],[0,2],[1,0],[3,2],[0,1],[-4,1],[-3,2],[-6,6],[1,0],[3,3],[-2,1],[-5,0],[-2,1],[0,3],[0,2],[0,1],[0,2],[-1,1],[-4,2],[-1,3],[-1,3],[-1,1],[-1,0],[-8,5],[-1,3],[2,4],[1,1],[1,0],[2,0],[1,0],[0,1],[0,1],[1,1],[4,6],[1,1],[-1,4],[0,1],[1,1],[0,2],[3,4],[-3,6],[1,1],[26,13],[3,1],[2,-1],[3,-2],[2,0],[2,3],[0,3],[6,3],[3,2],[-1,4],[-4,2],[-7,-1],[1,0],[-11,-5],[-3,1],[-4,3],[-8,3],[0,1],[0,4],[0,2],[-1,3],[-2,2],[-4,4],[3,2],[1,1],[-3,9],[0,2],[1,5],[1,3],[4,6],[1,1],[5,2],[1,3],[1,1],[0,1],[-1,0],[-7,3],[-1,2],[1,4],[0,-1],[2,-2],[1,-1],[3,-1],[7,1],[0,1],[-9,0],[-2,1],[0,1],[0,3],[1,1],[7,1],[2,2],[1,2],[0,2],[1,6],[1,3],[0,2],[-1,2],[-1,3],[0,3],[0,3],[4,9],[2,2],[3,2],[3,1],[2,0],[6,-3],[3,-1],[2,2],[2,4],[0,2],[-2,3],[2,2],[6,-2],[3,2],[-3,3],[-8,4],[-2,6],[1,2],[0,3],[1,3],[2,2],[-3,5],[0,1],[2,-1],[6,0],[3,-1],[6,-4],[2,-1],[5,1],[2,0],[4,-3],[2,0],[2,1],[-1,1],[-1,4],[2,0],[1,1],[1,1],[1,1],[-3,1],[-6,3],[-5,0],[-2,1],[-2,3],[-1,2],[1,1],[1,1],[1,2],[0,-1],[1,3],[0,1],[5,3],[2,2],[-1,3],[3,4],[0,2],[0,4],[-1,3],[-3,2],[-2,3],[-1,3],[1,3],[2,2],[1,2],[3,2],[3,1],[2,0],[3,-1],[0,-1],[-1,-1],[2,0],[1,1],[-1,1],[-1,2],[-9,5],[1,0],[-1,1],[1,2],[3,3],[1,2],[1,2],[1,4],[1,2],[2,4],[4,2],[7,2],[-2,0],[-4,2],[1,3],[1,2],[1,2],[4,2],[1,1],[1,2],[1,1],[5,2],[12,-2],[6,2],[-2,2],[-6,-1],[-3,1],[-6,4],[-2,2],[10,9],[6,3],[6,1],[1,0],[2,-2],[1,-1],[8,0],[12,-3],[3,1],[-3,3],[-21,6],[-3,3],[2,3],[7,5],[2,3],[4,6],[4,4],[12,8],[8,2],[2,1],[1,2],[0,1],[1,2],[1,2],[2,2],[4,5],[48,32],[16,6],[10,8],[20,5],[17,7],[16,1],[18,8],[29,4],[19,7],[17,2]],[[4658,7775],[3,-5],[2,-2],[1,2],[2,-1],[3,-5],[4,-1],[21,-3],[2,-2],[3,-5],[2,-2],[2,0],[2,0],[2,0],[5,-5],[15,-9],[2,-2],[-1,-3],[-1,-2],[-2,-2],[-1,-1],[2,-1],[4,-1],[2,0],[1,1],[1,3],[0,4],[1,4],[2,3],[3,0],[6,0],[16,3],[8,-1],[5,-6],[1,0],[1,4],[3,3],[4,2],[3,0],[1,1],[9,1],[7,3],[50,6],[6,4],[8,3],[2,4],[3,5],[3,4],[15,4],[4,-2],[-3,-3],[-6,-4],[-2,-3],[3,-2],[4,1],[8,4],[10,3],[5,0],[12,-7],[23,-1],[7,-3],[4,-3],[10,-16],[3,-2],[7,-3],[3,-3],[1,-5],[1,-5],[0,-2],[-4,-6],[1,-5],[-3,-4],[-20,-8],[-1,-4],[-5,-4],[-6,-1],[-17,7],[-3,1],[-1,-2],[1,-3],[2,-5],[0,-3],[-6,-3],[-12,5],[-6,0],[2,-4],[-1,-2],[-1,-2],[-1,-2],[-1,-2],[0,-1],[0,-1],[0,-2],[2,-2],[2,-1],[1,-1],[2,0],[6,2],[6,5],[6,4],[5,-3],[-1,-2],[3,-2],[4,-1],[14,2],[4,-2],[2,-5],[-2,-1],[-8,-2],[3,-2],[5,-2],[2,-3],[0,-1],[2,-2],[-1,-1],[-1,-1],[-1,-3],[1,-4],[-3,-2],[-3,0],[-5,4],[-3,0],[-34,-11],[-8,0],[1,-2],[-3,-1],[-6,-1],[-11,-6],[-2,-4],[0,-1],[-1,-2],[0,-2],[-1,-2],[-36,-28],[-5,-7],[-13,-11],[-7,-2],[-10,-4],[-2,-3],[4,-1],[2,-2],[2,-2],[-1,-1],[-3,-4],[4,-3],[1,-2],[-1,-3],[-1,-2],[0,-3],[0,-6],[1,-3],[1,-3],[2,0],[1,2],[0,4],[0,2],[1,2],[2,2],[7,8],[6,8],[5,4],[11,5],[4,4],[2,-2],[2,1],[4,3],[4,1],[12,1],[4,-2],[20,-1],[46,-18],[4,-6],[-2,1],[-1,-1],[-1,-2],[1,-3],[-2,-6],[-1,-3],[-2,-1],[-14,-2],[0,-1],[17,-2],[5,-3],[2,-2],[3,-5],[1,-2],[0,-4],[2,-1],[5,1],[2,-1],[1,-1],[5,-9],[1,-3],[0,-1],[-10,-6],[-1,-2],[-7,-5],[1,-4],[1,-1],[-1,-1],[-1,-1],[-1,0],[1,0],[1,-2],[-2,-4],[4,-2],[6,0],[4,1],[16,15],[25,10],[6,-1],[2,2],[4,4],[1,2],[-2,2],[8,0],[1,-3],[1,-3],[1,-1],[-2,-2],[-4,-3],[-2,-2],[1,-2],[14,-11],[-1,-5],[1,-6],[3,-5],[3,-2],[-2,-2],[-2,-3],[0,-2],[1,0],[3,2],[1,0],[1,-2],[1,-1],[1,-2],[0,-2],[3,-5],[1,-3],[1,-4],[1,-5],[1,-2],[0,-3],[-2,-3],[-2,0],[-5,0],[-2,-1],[-1,-1],[-1,-2],[0,-1],[-1,-1],[-2,-3],[-2,-2],[-1,0],[-3,0],[-1,-1],[-1,-2],[-1,0],[-16,-9],[-3,0],[-5,2],[-16,0],[4,-3],[14,-4],[8,-5],[5,-1],[2,3],[2,-1],[0,-1],[1,-1],[1,-2],[-2,-1],[-5,-2],[2,-2],[2,-1],[2,0],[3,1],[0,-5],[1,-6],[1,-7],[2,-3],[-1,0],[2,-3],[0,-4],[0,-4],[-1,-4],[-4,-3],[-38,-9],[3,-2],[2,0],[5,1],[4,-1],[6,-4],[3,-1],[17,6],[2,-3],[0,-8],[-2,-6],[-3,-4],[-4,-1],[-1,1],[-3,2],[-2,0],[-1,0],[-3,-2],[-4,-2],[-8,-6],[0,-1],[6,-3],[6,2],[6,4],[6,2],[5,0],[2,-1],[0,-4],[-7,-18],[-3,-5],[-4,-3],[-11,-3],[-2,1],[-1,4],[1,3],[2,1],[5,2],[-4,3],[-3,0],[-4,-3],[-3,-4],[-4,-3],[-4,-1],[-8,1],[-3,-1],[-2,-4],[-3,-8],[-5,-6],[-3,-4],[0,-3],[-15,-9],[-10,-3],[-15,-8],[-3,-1],[-25,0],[-3,1],[-2,3],[-4,3],[-16,1],[-5,2],[-3,1],[-14,-4],[-5,0],[-9,3],[-4,3],[-9,10],[-2,2],[-1,3],[1,1],[1,2],[1,11],[2,5],[11,15],[1,2],[0,3],[-2,0],[-2,0],[-1,1],[0,1],[0,1],[-4,-2],[3,-5],[1,-3],[-3,0],[-5,0],[-2,-1],[1,0],[-12,-14],[-4,-9],[-3,-11],[1,0],[0,-1],[1,0],[2,2],[1,-1],[1,-1],[4,-1],[2,-1],[1,-2],[-2,-1],[1,-4],[1,-4],[2,-3],[2,-1],[4,0],[1,-1],[1,-3],[0,-3],[1,-1],[2,0],[1,-1],[4,-4],[1,-1],[3,0],[2,-1],[2,-6],[1,-2],[1,-1],[1,-1],[1,0],[2,1],[0,-2],[5,-1],[2,-1],[-2,-2],[-4,-7],[-4,-7],[-4,-6],[-2,-4],[-3,-3],[-12,-6],[-9,-10],[-1,-1],[-4,-1],[-8,-8],[-3,-3],[-14,2],[-4,-2],[0,-2],[1,-1],[-3,-2],[-2,1],[-1,1],[-5,1],[-5,4],[-13,18],[-2,2],[-3,1],[-3,0],[-8,-7],[-4,-2],[-10,0],[-3,0],[0,2],[-1,3],[-1,4],[0,1],[-3,3],[-1,1],[0,4],[-1,1],[-2,5],[-1,3],[1,4],[-6,8],[-5,9],[2,-1],[2,2],[1,2],[0,4],[-2,2],[-14,5],[-6,4],[-4,5],[-1,6],[0,3],[-1,2],[0,3],[-2,2],[-1,1],[-2,2],[-2,1],[0,2],[0,1],[0,1],[1,0],[-1,4],[-2,2],[-1,2],[-5,2],[-2,0],[-2,1],[-2,4],[-2,0],[-1,2],[-2,9],[-2,2],[-19,2],[-3,1],[-4,4],[-3,2],[-3,7],[-4,4],[-10,6],[-4,5],[-5,9],[-4,4],[1,0],[-2,0],[-6,4],[2,2],[1,2],[0,2],[0,3],[-2,1],[-4,-1],[-3,1],[-2,5],[1,1],[3,1],[-7,2],[-2,1],[-2,2],[-1,3],[-2,2],[-2,0],[-1,4],[-19,6],[-2,-1],[-2,-2],[-3,-1],[0,1],[-2,2],[0,1],[-2,0],[-3,-2],[0,-1],[1,-2],[-1,0],[-2,-1],[-1,-2],[-2,0],[-1,0],[-6,6],[0,2],[0,2],[-1,2],[-2,2],[3,3],[1,1],[0,1],[-2,1],[-3,-1],[-2,2],[3,2],[1,1],[2,0],[-4,3],[-8,1],[-3,3],[-4,6],[-2,1],[-1,0],[-8,-2],[-7,-5],[-5,-6],[0,-7],[-2,-2],[-3,0],[-3,0],[-2,2],[1,0],[1,0],[0,2],[1,2],[-1,1],[-4,3],[1,0],[-3,3],[-8,3],[-4,1],[-2,-2],[-3,-2],[-2,-1],[-2,1],[1,2],[-2,1],[-1,-1],[-1,0],[-4,0],[-1,1],[-6,4],[-2,2],[-1,2],[-1,2],[-1,1],[-1,1],[0,1],[0,1],[1,1],[1,2],[-1,2],[-1,2],[3,1],[8,-2],[2,1],[-2,4],[-5,0],[-9,-1],[1,2],[0,1],[-1,3],[-4,6],[-1,3],[6,0],[-3,4],[-4,2],[-4,1],[-3,1],[-4,2],[-8,7],[-3,-1],[1,-2],[-1,0],[-4,1],[0,1],[-1,1],[-1,1],[-3,0],[-2,0],[0,2],[-9,6],[-5,0],[-1,0],[-1,1],[-1,2],[0,1],[0,1],[-23,15],[-5,5],[-3,5],[-5,3],[-1,3],[0,1],[1,2],[2,3],[-2,1],[-2,0],[0,1],[-1,4],[0,3],[0,2],[1,2],[-1,1],[3,5],[7,7],[7,12],[1,3],[0,3],[0,3],[0,1],[1,3],[1,5],[1,2],[1,2],[4,3],[1,2],[4,4],[10,2],[5,3],[-4,0],[-1,1],[4,1],[7,-1],[3,2],[-1,3],[8,-2],[11,3],[3,0],[4,-2],[11,3],[0,-2],[-1,0],[-2,-5],[2,-3],[2,0],[4,2],[1,-5],[1,-3],[3,-1],[2,-1],[3,0],[2,0],[2,1],[1,0],[1,-1],[0,-2],[-1,-2],[-1,-2],[-1,-2],[4,0],[1,-1],[1,-3],[-7,2],[-2,-2],[9,-10],[6,-3],[3,-3],[4,-3],[1,3],[0,2],[2,3],[3,1],[3,-1],[-2,0],[-1,-1],[-2,-1],[-1,-2],[2,-1],[5,-3],[2,-2],[3,-7],[3,-5],[1,-4],[-1,-2],[3,-3],[5,-3],[4,-1],[4,2],[-1,1],[-3,2],[-1,2],[3,0],[3,-2],[2,-2],[2,-3],[-1,-3],[-1,-2],[-2,-2],[-2,-1],[-1,0],[-2,3],[-1,0],[-5,-1],[2,-1],[7,-6],[4,-1],[18,-2],[7,-5],[2,0],[2,0],[2,2],[2,1],[2,-1],[3,-2],[2,0],[4,0],[4,1],[4,3],[3,4],[-2,1],[-5,2],[0,2],[1,0],[1,0],[0,1],[0,2],[1,1],[1,1],[1,-1],[1,-1],[-1,0],[8,-2],[19,2],[9,-1],[1,0],[1,0],[0,-1],[1,-1],[0,2],[1,-2],[1,-1],[1,0],[1,-1],[2,1],[2,1],[2,4],[5,4],[3,2],[1,5],[-1,8],[-1,2],[-2,3],[-6,1],[-2,2],[3,4],[0,3],[-6,5],[2,2],[4,4],[-3,2],[-7,-1],[-2,3],[6,10],[-2,2],[0,1],[3,4],[2,0],[2,-1],[-2,2],[1,0],[2,-3],[1,0],[1,1],[0,1],[1,2],[0,1],[1,0],[1,1],[1,0],[1,-1],[2,-2],[1,-2],[-2,-2],[-2,-2],[-2,-1],[-2,0],[-1,0],[-1,1],[-1,1],[1,-1],[0,-1],[1,0],[1,0],[-4,-6],[0,-2],[2,-3],[3,-3],[3,-2],[3,-2],[4,0],[-3,4],[2,2],[2,0],[2,-7],[-1,-2],[-6,-1],[-2,-3],[0,-3],[1,-2],[1,-2],[1,-1],[12,2],[7,3],[6,-2],[2,1],[1,1],[-1,2],[0,1],[-1,1],[0,-1],[0,2],[-1,0],[0,1],[1,2],[1,1],[4,1],[0,2],[-3,1],[1,1],[0,1],[0,2],[-15,6],[2,3],[2,1],[4,-2],[-1,2],[-1,2],[1,2],[1,2],[-4,0],[0,1],[2,3],[-3,1],[-5,-1],[-3,1],[3,4],[3,3],[-4,4],[-9,-1],[-4,2],[2,1],[3,1],[2,1],[-4,2],[2,1],[4,1],[2,1],[-4,2],[-6,0],[-5,-2],[-5,-6],[-2,0],[-2,-2],[-1,-3],[-1,-1],[-3,0],[-5,2],[1,5],[-3,4],[-7,4],[1,2],[1,4],[1,2],[2,1],[5,0],[0,2],[1,1],[5,1],[-2,3],[-3,1],[-3,1],[-2,0],[0,2],[9,2],[3,1],[0,1],[0,2],[-1,1],[3,2],[6,5],[3,1],[-2,-2],[1,-2],[2,-1],[4,3],[2,0],[2,-1],[3,-4],[4,-4],[1,-2],[2,0],[9,1],[-2,-2],[-2,-3],[11,-7],[5,-1],[21,2],[5,3],[-6,2],[-12,-1],[-8,5],[-6,3],[-5,5],[-7,6],[-2,2],[-3,6],[-1,2],[-6,5],[-3,4],[-6,6],[-2,1],[-3,0],[-1,-1],[2,-4],[-2,-2],[-1,-1],[2,-6],[1,-1],[3,-2],[2,-1],[-16,-1],[-21,-10],[-22,-4],[-13,4],[-7,5],[-1,3],[1,2],[0,1],[-4,0],[-25,8],[-3,4],[0,6],[-4,3],[-9,1],[-2,1],[-3,4],[-1,1],[-9,2],[-3,2],[2,1],[1,0],[1,-1],[-1,2],[-2,3],[-5,3],[2,1],[4,-1],[2,0],[2,1],[1,1],[1,2],[5,8],[2,2],[2,1],[2,1],[5,0],[8,-1],[4,2],[2,7],[3,4],[28,-3],[7,2],[2,-1],[2,-1],[4,-7],[2,-1],[7,-4],[5,-8],[1,-2],[1,-4],[1,-2],[5,-4],[11,-2],[4,-4],[0,-3],[4,-3],[5,-1],[2,3],[-2,4],[-5,5],[-12,6],[-2,4],[1,4],[2,5],[1,1],[3,4],[0,2],[0,3],[0,1],[-1,1],[1,3],[-8,0],[-4,0],[-3,5],[-1,1],[-3,0],[-2,-1],[-2,-2],[-1,-2],[-3,1],[-2,1],[-2,1],[-6,1],[-3,1],[-2,3],[3,0],[2,1],[1,3],[1,4],[-3,0],[-6,2],[-9,0],[-4,1],[-1,3],[1,3],[4,1],[0,1],[-2,1],[-2,2],[-2,1],[-4,-2],[-2,0],[3,5],[9,2],[4,5],[-3,0],[-1,0],[-2,1],[3,6],[4,2],[10,1],[0,4],[2,0],[5,-3],[7,-1],[4,1],[3,3],[3,4],[3,2],[26,-3],[13,-8],[14,-3],[12,-6],[1,-1],[0,-2],[0,-2],[2,-2],[2,-2],[1,2],[0,3],[-2,2],[2,2],[2,2],[1,2],[1,1],[1,0],[0,2],[0,1],[0,1],[0,1],[3,2],[7,3],[2,2],[1,3],[1,4],[0,4],[-2,1],[-2,1],[-1,-1],[2,-6],[-5,5],[-4,-2],[-9,-10],[-6,-2],[-20,2],[0,1],[1,1],[0,1],[1,1],[-2,1],[-1,0],[-4,0],[3,7],[4,5],[4,2],[6,2],[5,3],[2,1],[8,1],[3,1],[22,-4],[-1,-1],[-1,-2],[-1,-2],[-1,-3],[1,-1]],[[4905,7817],[-5,-9],[-13,-14],[-11,-5],[-10,-3],[-2,-1],[-3,-3],[-1,-1],[-1,0],[0,-1],[-1,-3],[-1,-2],[-1,0],[-1,-1],[-4,-2],[-2,-2],[-29,-5],[-29,-5],[-7,7],[-3,-2],[-1,-5],[4,-2],[-3,-2],[-2,-1],[-9,2],[-6,-1],[-7,-3],[-21,4],[-4,3],[3,1],[37,-2],[0,2],[-2,3],[-4,1],[-42,0],[-12,2],[-7,4],[-3,7],[6,3],[33,8],[11,-1],[3,1],[-2,1],[-1,2],[-1,2],[-2,2],[5,3],[16,2],[8,6],[2,1],[25,0],[40,15],[51,6],[4,-1],[4,-4],[1,-7]],[[5459,7820],[3,0],[1,1],[5,3],[3,1],[14,-8],[10,-2],[7,-4],[6,-5],[5,-8],[-1,-4],[-6,-6],[1,0],[-1,-2],[23,2],[2,1],[-1,0],[0,2],[1,5],[1,2],[1,0],[4,0],[2,0],[3,2],[23,7],[4,-3],[23,4],[23,-5],[43,-2],[40,-13],[32,-9],[2,3],[0,1],[5,-2],[3,0],[3,-1],[5,-4],[-5,-10],[-3,-4],[-3,1],[-1,2],[-1,2],[0,2],[-2,1],[-2,-1],[-2,-1],[-2,-2],[-1,-3],[2,-2],[1,-3],[0,-4],[-1,-4],[-8,-12],[-4,-3],[-4,-8],[-9,-10],[-5,-4],[-17,-20],[-7,-3],[-10,0],[-1,1],[-3,2],[-1,0],[-2,-2],[3,-4],[6,-8],[-2,-4],[-11,-9],[0,-1],[0,-3],[-1,-1],[0,-1],[-1,-1],[1,0],[-2,-1],[-2,-1],[-1,-3],[-1,-3],[-1,-2],[-8,-5],[-3,-5],[-4,-11],[-2,-4],[-3,-2],[-3,-2],[-3,0],[-5,1],[-9,-4],[-15,0],[-5,-3],[29,-4],[0,-3],[-11,-16],[0,-1],[-2,-7],[-4,-5],[-7,-6],[-4,-7],[-1,0],[-1,-1],[0,-4],[0,-3],[-1,-2],[-1,-2],[-12,-13],[-3,-4],[-5,-10],[-2,-4],[-18,-12],[-2,-3],[-4,-8],[-3,-2],[-8,-4],[-25,-5],[-9,3],[-13,1],[-5,4],[-48,7],[-5,3],[-2,1],[-11,0],[-3,0],[-5,6],[-2,2],[-2,-1],[-4,-3],[-6,-2],[-11,1],[-11,-4],[-32,-1],[-31,-2],[-10,3],[-5,-1],[-4,-3],[-1,-2],[0,-3],[0,-3],[0,-2],[2,-2],[2,1],[3,2],[5,2],[4,0],[19,-6],[17,-1],[0,1],[3,3],[1,1],[1,0],[0,1],[2,2],[1,1],[2,-1],[0,-3],[-1,0],[-1,-3],[1,-6],[-1,-3],[-1,-2],[-2,-3],[1,-3],[4,-4],[14,-4],[4,-3],[7,-8],[7,-12],[4,-13],[-1,0],[1,-2],[-1,-3],[-2,-2],[-2,-1],[-3,0],[-1,-1],[-2,-2],[-1,-5],[-3,-6],[-3,-3],[-7,-6],[-5,-2],[-5,-1],[-4,-1],[-4,-6],[-2,-5],[-1,-2],[-2,-2],[-3,-3],[-1,-1],[-1,-3],[-1,-4],[-10,-13],[0,-3],[-1,-1],[-1,-5],[0,-1],[0,-2],[0,-2],[0,-1],[-1,-2],[-1,-1],[-2,-4],[-15,-3],[-2,-5],[3,-2],[5,-1],[4,1],[3,3],[2,3],[3,3],[3,1],[-5,-10],[5,-5],[-2,-1],[-2,0],[0,-1],[0,-2],[1,-1],[-3,-1],[-5,2],[-7,4],[-11,0],[-7,5],[-26,-1],[-3,-2],[0,1],[-2,0],[-14,0],[-45,-12],[-3,2],[1,2],[6,2],[0,2],[-10,-2],[-2,1],[1,3],[1,5],[1,3],[0,2],[0,3],[0,2],[4,2],[7,5],[-12,0],[0,3],[1,1],[7,0],[2,1],[3,3],[33,7],[-6,1],[-30,-6],[-8,4],[2,1],[-1,2],[-3,2],[-2,5],[0,2],[0,3],[1,3],[1,2],[1,3],[0,4],[0,2],[2,1],[0,2],[0,2],[-1,3],[1,3],[1,5],[0,3],[-1,3],[-1,2],[-2,2],[-2,3],[-2,5],[0,3],[0,2],[2,1],[8,-2],[-1,2],[-5,3],[4,0],[1,1],[2,1],[-4,4],[-2,3],[-1,7],[-2,3],[-2,1],[-6,0],[-2,0],[-1,2],[-1,1],[-1,5],[0,1],[-1,0],[0,1],[-1,2],[0,1],[0,2],[1,1],[3,1],[-2,2],[-4,6],[-2,2],[-6,2],[-1,1],[0,2],[0,2],[-1,1],[-10,2],[-6,2],[1,6],[-1,0],[-2,1],[3,4],[2,0],[6,-2],[-2,2],[-5,6],[2,3],[0,1],[-7,0],[-3,1],[-2,3],[-1,5],[0,7],[1,8],[2,7],[3,3],[-2,2],[-1,3],[-1,2],[-1,4],[-1,2],[-3,2],[-2,2],[0,3],[2,2],[2,0],[2,1],[2,4],[0,4],[-1,3],[-2,2],[-2,2],[5,8],[2,2],[3,1],[8,-1],[2,2],[-1,1],[-1,1],[0,2],[-1,1],[-3,1],[-1,1],[0,5],[-1,6],[0,5],[3,0],[0,2],[-2,0],[-1,2],[-1,3],[1,3],[-1,2],[0,2],[1,2],[-1,1],[-1,3],[-4,2],[-1,3],[1,0],[1,1],[1,1],[1,1],[-1,3],[0,2],[2,5],[-2,1],[-3,3],[-3,9],[-1,5],[1,5],[3,3],[2,2],[-1,1],[-1,1],[-1,0],[2,4],[-1,3],[-6,4],[3,0],[2,1],[2,1],[-1,1],[-1,1],[-1,3],[4,2],[2,0],[2,-1],[1,-2],[2,-2],[2,0],[1,2],[-1,3],[-5,5],[-5,4],[-10,2],[5,7],[-1,2],[-3,3],[2,1],[4,1],[1,1],[-1,1],[-2,2],[2,4],[3,1],[7,0],[15,9],[2,1],[2,0],[2,-1],[1,-1],[1,-2],[1,-1],[1,-1],[6,4],[3,0],[4,-2],[17,-17],[8,-5],[25,-5],[20,2],[-7,7],[-18,1],[-7,5],[-2,2],[-7,5],[-2,4],[-2,1],[-6,4],[-3,4],[-2,5],[18,5],[0,1],[-2,1],[-5,4],[-3,0],[-4,-1],[-3,-2],[-1,-1],[-1,-3],[0,-1],[-3,0],[-5,5],[-6,3],[-3,2],[-2,3],[-3,3],[-1,6],[0,7],[2,6],[2,5],[5,4],[24,4],[10,5],[10,2],[5,3],[9,7],[2,1],[8,-2],[21,1],[3,1],[1,2],[2,3],[2,2],[2,1],[52,1],[4,-2],[6,-10],[4,-2],[4,-1],[4,3],[-3,1],[-1,4],[0,3],[-1,3],[4,6],[7,3],[23,2],[22,0],[29,-9],[24,-3],[4,-1],[2,-4],[-2,-4],[2,-2]],[[4947,7946],[2,-2],[-1,-4],[-3,-4],[-10,-13],[-2,-1],[-6,1],[-4,-1],[-17,-11],[-5,-1],[-1,1],[-4,2],[-1,0],[-4,1],[-2,2],[-1,2],[4,7],[14,7],[2,2],[0,4],[9,4],[3,3],[-2,0],[3,5],[5,2],[10,-1],[11,-5]],[[5163,7942],[11,-11],[1,-6],[-6,-5],[-4,-1],[-42,9],[-6,5],[-8,3],[-3,4],[6,4],[2,0],[5,-2],[2,-1],[-2,7],[5,4],[6,2],[11,1],[8,-3],[7,-4],[7,-6]],[[4215,8110],[0,-11],[3,-3],[6,-2],[0,-1],[0,-2],[0,-1],[1,0],[2,-1],[5,-4],[1,-1],[-2,-6],[-4,-4],[-14,-11],[-12,-7],[-33,-6],[-8,-5],[-3,0],[-9,4],[-15,0],[-6,3],[-8,1],[-13,5],[-9,7],[-13,6],[-3,4],[2,4],[5,3],[6,3],[2,3],[2,7],[1,1],[2,1],[0,1],[0,2],[1,1],[0,1],[2,1],[3,1],[1,1],[-2,0],[-2,1],[-3,1],[-2,1],[0,2],[2,1],[3,1],[1,2],[-1,3],[3,4],[3,3],[22,17],[9,4],[30,1],[7,-5],[12,-1],[2,-2],[6,-6],[9,-3],[1,-2],[0,-5],[7,-12]],[[5273,8187],[39,-32],[3,-5],[4,-3],[11,-6],[12,-10],[11,-2],[1,-1],[1,-1],[1,-5],[1,-2],[-3,-2],[-5,-1],[-9,0],[0,-2],[15,-1],[19,-8],[0,-3],[-1,-4],[-2,-2],[-6,-7],[2,-3],[9,-9],[2,-4],[0,-4],[-3,-3],[-2,-3],[0,-1],[2,-4],[-2,-1],[-5,-2],[-6,-2],[-2,-1],[-2,-2],[2,-2],[8,3],[2,0],[2,-2],[2,-5],[6,-7],[2,-5],[-1,-6],[2,-6],[1,-3],[0,-4],[0,-3],[-1,-6],[1,-2],[-3,-13],[0,-5],[-2,-4],[-4,-2],[-8,0],[7,-1],[3,-1],[2,-4],[-1,-4],[-3,-3],[-5,-5],[-8,-3],[-33,-3],[-34,-2],[-4,2],[-4,3],[-2,0],[-16,-4],[-3,-3],[-30,1],[-6,5],[-2,5],[-4,3],[-5,0],[-6,-5],[-2,0],[-6,3],[-6,0],[-2,1],[-3,2],[1,3],[2,4],[1,3],[-28,18],[-2,0],[-2,0],[-1,-2],[-1,-1],[-1,-2],[-2,-1],[-3,-2],[-4,0],[-2,2],[9,5],[0,1],[-6,1],[-3,-1],[-2,-3],[0,-3],[1,-3],[2,-6],[-6,0],[-4,2],[0,5],[6,7],[-1,2],[-2,1],[-13,0],[-5,1],[-4,6],[1,0],[-2,1],[-2,-1],[-1,-2],[1,-2],[0,-1],[-18,1],[-3,2],[2,0],[2,0],[1,2],[1,3],[-5,0],[-5,2],[-5,3],[-4,5],[-6,2],[-3,3],[-2,5],[2,2],[9,1],[0,3],[-1,2],[-1,0],[-1,0],[-2,0],[-2,-3],[-1,0],[-4,0],[2,4],[2,4],[3,3],[3,2],[-1,3],[-3,0],[-7,-2],[1,-2],[-1,-3],[0,-3],[-5,-5],[0,-1],[-1,-2],[0,-1],[0,-2],[0,-2],[-1,-2],[-2,-2],[-4,-1],[-9,0],[-5,2],[-1,5],[1,2],[1,3],[0,3],[-2,3],[5,-2],[3,1],[1,2],[-1,4],[-2,1],[-20,-1],[-2,-1],[-3,-2],[-2,-2],[-3,2],[1,2],[1,3],[1,5],[-2,4],[0,4],[1,2],[2,2],[0,2],[-1,1],[6,7],[3,4],[3,7],[0,2],[1,0],[0,2],[0,2],[1,1],[3,-1],[2,-1],[3,-3],[2,1],[0,4],[-1,0],[-1,1],[-1,1],[0,1],[5,3],[25,5],[6,-2],[11,4],[-3,2],[-2,-1],[-5,-1],[-2,0],[0,2],[0,2],[2,1],[4,2],[10,0],[5,3],[-3,2],[-27,-4],[1,3],[0,1],[-1,1],[4,0],[12,5],[8,12],[4,3],[4,-5],[-2,-1],[-4,-2],[0,-2],[16,5],[2,3],[-29,6],[-3,-1],[-1,0],[-1,0],[0,-3],[-5,-3],[-3,0],[-3,1],[3,6],[4,3],[9,1],[2,-1],[1,0],[1,2],[0,2],[2,1],[9,-1],[3,-2],[2,-3],[0,-2],[2,-1],[7,-1],[2,0],[2,1],[5,0],[3,3],[-1,2],[-3,1],[-7,1],[-3,1],[-3,3],[-2,4],[6,9],[23,11],[7,2],[9,4],[2,0],[2,-1],[3,-3],[1,0],[2,2],[1,4],[1,2],[1,1],[2,2],[2,1],[1,0],[6,-3],[5,-1],[4,1],[5,2],[2,1],[2,4],[2,0],[8,0],[2,1],[5,4],[3,0],[28,-4],[29,-5]],[[5059,8198],[17,-10],[7,1],[3,0],[2,-3],[0,2],[3,-3],[7,-4],[3,-5],[-3,-3],[-1,-1],[-9,-2],[-2,-2],[1,-2],[-1,-2],[-3,-2],[-4,-2],[-2,-1],[-3,-4],[-1,-2],[-5,-2],[-3,2],[-3,4],[-4,3],[-8,4],[-5,3],[-2,4],[1,3],[2,1],[2,1],[2,1],[0,3],[-2,1],[-5,0],[1,-2],[0,-1],[-7,-2],[-4,-2],[-3,-4],[-1,-4],[0,-3],[3,-7],[1,0],[1,-2],[0,-2],[-3,-1],[-3,-3],[-2,-1],[-11,-1],[-4,-3],[2,-2],[3,-1],[2,-1],[3,-1],[-4,-3],[-14,-3],[-7,-5],[-3,0],[0,5],[-3,1],[-7,1],[-3,3],[1,0],[1,0],[1,1],[1,1],[0,2],[-1,1],[-3,1],[-2,6],[-2,2],[-2,0],[0,2],[3,2],[-1,2],[-4,4],[39,17],[4,-3],[16,6],[11,2],[2,1],[0,3],[-1,5],[0,2],[6,6],[10,-2]],[[4405,8275],[-3,-2],[1,-1],[9,0],[3,-1],[-1,-1],[0,-1],[0,-1],[0,-1],[-12,-5],[-33,-6],[-6,-5],[-2,-3],[-2,-9],[-2,-3],[-2,-2],[-11,-7],[-7,-1],[-5,3],[-2,0],[0,-5],[-4,0],[-3,0],[-1,2],[0,2],[-8,-2],[1,-2],[-4,-2],[-49,0],[-6,4],[13,6],[2,4],[-7,0],[7,5],[7,2],[16,-1],[-3,2],[-4,1],[-3,1],[0,5],[3,4],[4,3],[3,2],[32,7],[31,8],[10,0],[21,5],[17,-5]],[[5282,8225],[-39,-4],[-10,4],[-5,5],[-4,6],[-1,9],[1,10],[-3,2],[-3,2],[-2,3],[-1,5],[6,4],[21,3],[14,6],[1,2],[2,-1],[4,-4],[3,-3],[0,-3],[0,-4],[0,-5],[4,-7],[0,-3],[0,-1],[3,-2],[8,-10],[3,-5],[0,-6],[-2,-3]],[[4375,8298],[1,-5],[0,-5],[-4,-3],[-24,-5],[-42,-8],[-43,-8],[-42,-9],[-12,4],[-10,-1],[-3,1],[-2,6],[-1,1],[-3,1],[-1,2],[18,1],[10,7],[26,1],[20,10],[10,-1],[3,1],[8,6],[3,0],[9,0],[33,5],[33,5],[11,-4],[2,-2]],[[7013,8313],[13,-1],[3,-3],[7,-3],[1,-3],[-1,-3],[-2,-3],[-4,-3],[-32,-17],[-2,-2],[-2,-3],[1,-5],[3,-3],[15,-9],[7,-2],[9,-6],[-3,-3],[-3,0],[-5,2],[-9,-2],[-3,2],[3,3],[3,2],[-3,1],[-7,-1],[-19,1],[-6,-1],[-11,-5],[2,-3],[3,-1],[2,0],[3,-1],[-10,-8],[-1,0],[-2,1],[0,1],[0,2],[-1,1],[-2,1],[-13,0],[1,2],[1,1],[1,1],[0,2],[0,2],[-2,2],[-3,1],[-4,1],[-4,-2],[-4,0],[-4,5],[4,0],[10,2],[3,3],[2,3],[0,2],[0,2],[-1,1],[1,3],[2,1],[8,2],[8,9],[2,1],[10,3],[4,3],[2,1],[0,3],[0,1],[2,2],[8,3],[4,3],[2,2],[2,4],[5,4],[6,1]],[[6751,8318],[-4,0],[-8,2],[-3,2],[1,1],[9,5],[7,2],[6,-1],[6,-2],[-3,-3],[-4,-3],[-7,-3]],[[4340,8348],[2,-4],[9,-6],[0,-3],[-5,-8],[5,-5],[1,-2],[-10,0],[0,-1],[1,-2],[-8,-7],[-17,-8],[-46,-8],[1,0],[-33,-2],[-32,-2],[-15,2],[6,2],[2,1],[-22,3],[-2,-1],[0,-2],[-32,6],[-3,1],[-2,2],[-5,9],[-3,3],[0,2],[2,3],[2,3],[3,1],[7,0],[2,1],[1,6],[2,1],[12,4],[9,-2],[2,1],[3,2],[19,3],[6,3],[8,0],[9,1],[14,0],[7,3],[3,-1],[2,1],[4,1],[41,5],[40,4],[9,-5],[1,-5]],[[5824,8398],[2,-2],[1,2],[3,-3],[10,-5],[-3,-2],[-21,1],[-4,2],[8,5],[4,2]],[[6452,8386],[-2,-1],[-8,4],[-7,2],[2,6],[0,2],[-1,1],[-2,4],[-1,2],[5,0],[3,-2],[6,0],[3,-2],[2,-2],[7,-4],[-5,-7],[-2,-3]],[[4194,8432],[-14,-13],[3,-1],[9,3],[7,1],[4,7],[3,3],[3,1],[4,-2],[10,-10],[4,-1],[9,-1],[-2,-3],[-1,-2],[-1,-3],[2,-2],[22,-7],[9,1],[5,2],[1,-1],[1,-1],[0,-1],[0,-1],[-1,0],[1,-1],[0,-1],[0,-1],[1,-2],[2,0],[16,-2],[6,-3],[1,-7],[-1,-2],[-36,-20],[-30,-2],[-31,-1],[-8,2],[-48,0],[-4,2],[2,2],[5,3],[2,3],[-2,3],[-1,1],[-1,1],[-3,1],[-5,-1],[-3,2],[3,2],[4,4],[2,5],[-2,6],[-2,3],[-2,3],[-3,1],[-2,0],[-2,-3],[-3,0],[-3,0],[-2,2],[0,2],[-1,1],[-1,1],[-1,3],[1,0],[1,0],[1,1],[-2,3],[-7,2],[-2,2],[3,7],[-3,1],[-1,2],[-1,1],[3,0],[34,15],[37,2],[7,-2],[5,-5],[-1,-5]],[[4819,8443],[0,-2],[-3,-4],[0,-3],[1,-1],[10,-5],[12,-1],[6,-2],[3,-1],[-14,-1],[-2,0],[-6,3],[-7,0],[4,-6],[6,-2],[5,0],[5,-1],[4,-3],[1,0],[1,1],[0,1],[1,1],[1,1],[3,-1],[4,-5],[2,-1],[31,-4],[16,-8],[-2,-2],[-2,-2],[1,-3],[2,-2],[1,-3],[0,-3],[-3,-2],[-2,-2],[-3,-2],[0,-3],[0,-2],[-1,-4],[0,-2],[-1,-2],[-3,-3],[-1,-2],[2,-1],[4,-1],[2,-2],[-1,-1],[0,-2],[-1,-2],[3,-3],[17,-11],[3,-4],[3,-5],[-1,0],[2,-8],[-1,-3],[-1,-4],[-2,-3],[-4,-4],[-2,-2],[0,-3],[-1,-6],[0,-2],[0,-3],[-1,-1],[-1,-2],[-1,-1],[-1,-5],[-1,-1],[-1,-1],[-1,-1],[0,-2],[1,-2],[0,-2],[-1,-1],[1,-3],[1,-2],[2,-5],[5,-7],[1,-4],[-2,-4],[-7,-6],[-1,-2],[-1,-2],[0,-1],[-8,-5],[-8,-2],[-10,-4],[-5,-3],[5,-2],[25,-3],[2,-1],[4,-3],[2,-1],[20,-2],[5,-4],[1,-1],[0,-4],[-1,-1],[-1,-1],[-1,-2],[-1,-4],[0,-4],[0,-9],[0,1],[0,-1],[0,-1],[0,-1],[0,-1],[0,-1],[-1,0],[-1,-6],[0,-3],[2,-11],[4,-8],[10,-9],[-2,-2],[-3,1],[-16,6],[-3,4],[0,1],[0,3],[0,2],[0,3],[0,2],[1,1],[1,1],[-28,19],[-4,0],[1,-6],[-3,0],[-6,2],[-3,-2],[1,-2],[6,-4],[3,-3],[1,-2],[1,-3],[1,-3],[-2,-3],[-1,-1],[-2,-1],[-2,1],[-2,1],[-1,2],[-1,3],[-1,1],[0,3],[0,1],[-1,2],[-7,1],[-3,-1],[-1,-2],[1,-3],[-8,0],[-3,-2],[3,-1],[6,-1],[2,-1],[-3,-2],[-6,-1],[-3,-1],[4,-2],[9,2],[15,-5],[4,-3],[-1,0],[-1,0],[0,-1],[-1,-1],[-1,0],[-1,0],[-1,0],[-24,0],[-3,-3],[4,-2],[16,-3],[1,-2],[-1,-1],[-3,-2],[-9,-1],[0,1],[1,0],[0,2],[-3,-1],[-6,-2],[-2,0],[2,5],[-4,-1],[-3,-1],[-2,-3],[-3,-5],[2,-2],[1,0],[3,0],[-3,2],[2,1],[2,1],[2,-2],[1,-2],[-1,-2],[-2,-1],[-1,0],[-2,0],[2,-4],[3,-2],[4,-1],[2,2],[-1,0],[4,5],[4,4],[5,2],[4,-3],[-7,-3],[1,-2],[-2,-1],[-4,-6],[7,-3],[3,0],[0,2],[-1,3],[0,3],[3,2],[2,-3],[-1,-2],[4,-5],[10,-7],[3,-6],[-4,0],[-2,0],[-2,-3],[4,-2],[9,0],[4,-2],[-2,0],[-1,-3],[8,-4],[2,1],[-3,1],[0,2],[2,0],[2,-1],[1,-2],[1,-2],[-9,-6],[-9,-3],[-10,0],[-22,11],[-3,5],[0,2],[2,4],[0,2],[-2,2],[-3,1],[-5,0],[3,-4],[-1,-2],[-7,-4],[1,0],[2,-1],[-2,-2],[-1,0],[2,-4],[3,-3],[9,-5],[4,-1],[2,-1],[1,-2],[1,-3],[0,-2],[1,-3],[0,-4],[0,-5],[-1,-4],[-2,-1],[-7,3],[-40,-4],[-41,-5],[-4,3],[5,3],[-3,1],[-7,-1],[-3,1],[2,2],[-1,1],[-1,2],[-1,1],[2,2],[0,1],[1,0],[-2,3],[-5,0],[-4,-1],[-3,-4],[2,-4],[1,-3],[-2,-2],[-47,-5],[-4,3],[1,0],[2,3],[-1,4],[1,4],[9,12],[2,3],[0,3],[-3,3],[-3,-3],[-5,-9],[-1,-2],[-7,-4],[-4,-5],[-2,-1],[2,-4],[5,-3],[2,-3],[-6,1],[-2,-1],[-4,-2],[-13,-2],[-39,5],[-39,4],[-4,3],[-3,4],[0,4],[0,4],[1,3],[0,4],[0,4],[-1,2],[-4,5],[2,2],[1,0],[1,0],[0,1],[-7,5],[-7,1],[-2,2],[-1,2],[7,6],[8,2],[45,-2],[3,2],[-2,3],[-6,-1],[-6,2],[-7,0],[-9,2],[-11,-1],[-6,4],[4,1],[5,1],[5,2],[4,6],[-29,-5],[-4,1],[-16,9],[-8,2],[-4,3],[2,1],[13,5],[3,0],[2,-1],[-2,-1],[-6,-1],[-2,-2],[3,-2],[5,-1],[5,1],[1,4],[8,-1],[5,1],[4,3],[-4,2],[-7,1],[-2,4],[2,0],[-6,4],[-5,1],[-12,0],[0,2],[39,2],[38,2],[4,4],[-34,1],[10,3],[20,-2],[10,4],[-37,5],[0,2],[51,5],[-3,2],[-9,-2],[-9,2],[-3,0],[0,1],[6,0],[22,7],[5,0],[5,2],[5,5],[-23,0],[0,1],[2,1],[4,2],[-1,4],[0,1],[49,4],[48,3],[6,2],[2,2],[-2,2],[-2,1],[-42,-4],[-41,-5],[2,3],[10,2],[-38,-3],[-37,-4],[-6,-4],[-3,0],[-6,1],[-14,-3],[-9,2],[-29,-4],[-28,-5],[-5,-3],[-2,0],[-4,0],[-8,-3],[-7,1],[-8,-4],[-4,3],[3,1],[-4,3],[-5,1],[-35,-8],[-35,-7],[-16,2],[-22,-4],[-26,-10],[-4,2],[0,5],[-2,2],[-3,1],[-5,0],[-2,1],[1,3],[4,4],[-3,3],[-10,4],[-3,2],[-1,2],[1,2],[3,2],[3,1],[2,-1],[3,-3],[4,-1],[12,0],[-1,2],[-5,1],[-2,1],[1,2],[1,1],[1,0],[1,1],[-2,6],[2,1],[3,2],[2,1],[6,-1],[2,1],[2,1],[-1,2],[-1,1],[-2,1],[-2,0],[3,3],[4,1],[32,-1],[16,-7],[3,0],[7,2],[0,2],[-3,-1],[-3,-1],[-3,0],[-3,2],[2,1],[1,0],[-3,5],[-10,0],[-3,2],[1,3],[15,7],[-3,3],[-23,-4],[-3,1],[-2,3],[-1,2],[1,2],[3,5],[1,1],[2,1],[4,5],[2,1],[48,8],[2,0],[2,-2],[2,-5],[2,-1],[1,-1],[6,-3],[8,-2],[6,-4],[10,-4],[4,0],[-1,-1],[0,-1],[0,-1],[0,-2],[0,-1],[0,-2],[-1,-1],[3,-3],[4,-1],[15,-3],[2,0],[2,2],[1,1],[1,2],[9,4],[18,2],[8,5],[-15,-1],[2,-2],[-4,-2],[-9,-1],[-9,-2],[-15,0],[1,3],[2,5],[1,1],[-3,5],[-2,3],[-5,3],[-11,-1],[-5,3],[1,3],[-1,3],[-2,3],[-2,1],[0,3],[2,3],[10,5],[6,2],[3,2],[4,5],[2,1],[1,0],[2,1],[1,2],[1,2],[-4,0],[-8,-4],[-34,-3],[-8,3],[-7,6],[-6,9],[-1,5],[0,5],[2,3],[3,3],[2,2],[4,2],[2,2],[4,5],[2,2],[22,6],[18,10],[-45,-8],[-3,0],[-5,3],[-28,-3],[-7,5],[3,2],[4,1],[4,2],[1,5],[-2,3],[-5,1],[-9,-1],[3,2],[10,3],[4,2],[0,2],[-6,4],[4,2],[9,0],[4,3],[-3,2],[-6,2],[-3,3],[3,4],[3,1],[4,1],[3,1],[11,7],[4,1],[45,-5],[2,-1],[3,-4],[5,-1],[2,-1],[10,-9],[15,-7],[3,-4],[-1,0],[-2,-2],[-1,-2],[0,-2],[-4,-10],[0,-4],[3,-1],[16,-2],[13,-5],[11,-7],[4,-2],[1,-2],[1,-2],[1,-2],[4,-1],[2,-2],[1,-3],[-10,-4],[-3,-3],[36,7],[-4,-5],[-19,-4],[0,-1],[31,-4],[5,-5],[1,-2],[10,-10],[2,-2],[1,-4],[2,-2],[2,-1],[9,-1],[-2,-4],[-2,-2],[-3,-1],[-2,-2],[2,-1],[4,-1],[3,1],[6,3],[2,0],[1,0],[1,-1],[1,-2],[0,-1],[-1,-2],[0,-1],[0,-2],[1,-1],[1,-2],[2,0],[3,2],[6,6],[16,2],[1,1],[3,4],[2,0],[4,0],[4,1],[1,1],[0,2],[1,2],[1,1],[2,0],[4,-2],[3,0],[6,3],[-6,3],[-37,-8],[-5,-3],[0,3],[-1,1],[-2,-1],[-1,1],[0,3],[1,1],[1,1],[1,1],[-2,3],[-3,0],[-2,-1],[-3,1],[-1,1],[0,1],[0,1],[-1,2],[-1,0],[-5,0],[1,1],[2,2],[1,1],[0,3],[-7,0],[-3,3],[-2,5],[-1,3],[0,2],[0,2],[-2,2],[-8,4],[-2,2],[11,0],[0,2],[0,1],[4,3],[5,0],[10,-4],[16,-1],[10,-4],[5,0],[14,5],[11,1],[5,4],[-32,-6],[-3,1],[-4,3],[-2,1],[-12,3],[-10,7],[-3,1],[-19,-4],[-35,11],[-4,3],[2,5],[5,4],[42,-3],[0,2],[-2,0],[-4,1],[1,2],[-8,2],[-2,1],[4,3],[4,-2],[4,-4],[5,-2],[18,-2],[6,2],[-17,3],[-3,2],[3,1],[6,1],[2,2],[-3,2],[-9,-1],[-3,0],[-14,9],[1,1],[4,4],[-3,-1],[-6,-2],[-3,-1],[-9,5],[-40,-2],[-32,24],[-3,6],[29,7],[4,2],[-2,0],[0,1],[-1,2],[-1,2],[3,0],[6,5],[8,1],[18,10],[4,0],[6,4],[7,1],[48,-10],[2,1],[7,5],[3,1],[11,0],[6,-2],[5,-5],[-4,-2],[-8,2],[-4,-2],[19,-2],[26,-21],[4,-1],[0,1],[-1,0],[0,2],[8,1],[3,-1],[3,-3],[-9,-4],[-4,-4],[2,-5],[9,-8],[5,-2],[5,0],[0,1],[-4,5],[-3,3],[-1,3],[2,2],[9,-2],[2,1],[-1,0],[0,2],[2,1],[4,0],[3,-1],[0,-4],[3,-3],[3,-1],[6,1],[0,2],[-1,-1],[-2,1],[-2,1],[4,4],[-2,3],[-3,3],[-4,2],[-3,1],[-1,1],[-1,2],[-1,2],[0,2],[-2,1],[-4,0],[-1,1],[1,2],[1,3],[-1,2],[-1,3],[1,-1],[1,1],[1,0],[1,1],[0,1],[-1,2],[6,4],[8,2],[8,-1],[6,-3],[0,-5],[3,-1],[3,2],[-1,6],[14,-2],[7,2],[5,5],[-3,1],[-1,0],[1,2],[2,0],[3,0],[0,2],[-9,2],[-13,-4],[-11,0],[-6,2],[-3,5],[46,1],[3,-1],[2,-3]],[[4601,8455],[-47,-16],[-46,-16],[-21,-5],[-39,6],[2,2],[8,1],[8,7],[11,3],[21,1],[6,2],[6,7],[5,1],[4,1],[1,0],[1,0],[0,2],[0,2],[1,2],[1,1],[6,4],[39,5],[43,-4],[-10,-6]],[[5803,8408],[4,-4],[-4,0],[-9,-5],[-4,0],[-7,6],[-4,2],[-3,-3],[1,-5],[-4,-2],[-10,2],[-11,5],[-5,5],[-7,12],[-34,34],[-3,4],[0,4],[2,6],[5,4],[23,8],[15,3],[24,1],[5,-3],[10,-4],[17,-14],[3,-5],[-2,-1],[0,-2],[0,-1],[2,-1],[-2,-5],[-4,-3],[-10,-3],[-2,-2],[-2,-4],[-1,-6],[0,-4],[1,-4],[3,-8],[4,-4],[9,-3]],[[3507,8005],[0,17],[0,53],[0,53],[0,44]],[[3507,8172],[55,-5],[56,-6],[10,-5],[3,0],[1,4],[0,1],[-3,8],[-3,3],[-7,1],[-2,2],[4,3],[6,0],[10,-2],[0,2],[-1,0],[-2,1],[-1,2],[-1,2],[2,3],[4,3],[1,2],[-8,11],[-3,2],[0,1],[0,1],[1,0],[6,-1],[3,1],[2,2],[-3,2],[-12,1],[-2,3],[-2,0],[-2,1],[-6,-1],[0,1],[0,3],[1,1],[-5,3],[-35,7],[-6,4],[2,-1],[5,2],[-33,2],[2,2],[4,2],[1,2],[-2,2],[-4,0],[-2,2],[2,2],[3,3],[-2,1],[-3,1],[-3,1],[-2,-1],[-5,-4],[-1,0],[-4,2],[-6,-4],[-7,2],[-6,4]],[[3507,8253],[0,8]],[[3507,8261],[3,2],[33,7],[4,2],[7,6],[6,3],[12,2],[5,4],[-4,3],[-2,0],[4,2],[12,0],[3,3],[-4,1],[-6,2],[-3,1],[0,2],[0,1],[0,1],[7,3],[2,1],[-2,3],[-2,2],[-3,1],[-6,4],[-8,2],[-3,3],[0,1],[0,1],[0,1],[-1,2],[-2,1],[-18,1],[-2,3],[1,0],[1,0],[2,1],[0,2],[-1,1],[-15,4],[-5,0],[-4,-2],[1,-2],[1,-4],[1,0],[-1,-2],[-2,0],[-2,0],[-9,7]],[[3507,8337],[0,4],[0,53],[0,5]],[[3507,8399],[10,0],[4,3],[2,0],[6,-1],[4,2],[8,6],[-3,2],[-13,0],[0,2],[2,1],[3,1],[3,3],[2,3],[-1,1],[-1,1],[-1,3],[15,-2],[0,2],[-1,3],[0,2],[9,3],[3,0],[-1,6],[1,5],[2,4],[3,2],[2,6],[6,4],[12,4],[-1,1],[0,2],[-1,1],[3,3],[2,2],[6,2],[6,4],[3,0],[-1,0],[20,2],[8,-3],[4,0],[9,4],[1,2],[0,3],[2,2],[2,1],[2,-1],[1,-1],[3,-4],[6,-4],[2,-2],[-2,-4],[-2,-4],[0,-3],[2,-1],[16,0],[5,-2],[4,-6],[1,-7],[-3,-4],[-8,-6],[-4,-4],[-3,-1],[-6,3],[-3,0],[-6,-3],[5,-8],[0,-2],[3,-1],[4,-4],[2,-1],[5,-1],[3,-2],[0,-4],[-1,-2],[-7,-2],[-2,-2],[3,-2],[6,2],[2,-2],[-4,-3],[-3,-2],[0,-2],[2,-3],[7,-1],[3,-2],[-4,-2],[-8,-2],[-3,-5],[3,-3],[3,0],[4,1],[3,0],[3,-2],[15,-1],[3,-1],[6,-6],[2,-3],[1,-1],[2,0],[3,0],[2,-2],[-2,0],[-1,0],[-1,-1],[-2,-2],[17,-12],[0,-1],[-6,0],[0,-2],[2,0],[-3,-3],[-8,2],[-2,-4],[2,-3],[14,-3],[-4,-4],[-24,-6],[-5,-3],[-4,-6],[2,-4],[-2,-3],[-3,-2],[-2,-2],[2,-1],[4,-1],[2,-2],[-3,-1],[-4,-3],[-4,-3],[-2,-4],[17,3],[21,-1],[14,5],[17,0],[4,-1],[14,-6],[2,-2],[2,-3],[1,-1],[4,-2],[1,-4],[0,-4],[-4,-2],[-3,-1],[-10,-8],[2,-1],[3,-1],[1,-1],[-5,-3],[-13,-1],[-5,-6],[0,-1],[1,0],[0,-1],[0,-1],[-4,-1],[-3,-3],[-6,-9],[1,-1],[3,-2],[-1,-2],[5,-2],[7,3],[6,5],[5,5],[6,9],[2,2],[3,2],[40,8],[8,-2],[9,1],[8,-4],[8,0],[4,-3],[-4,-2],[-2,-2],[0,-2],[-1,-4],[0,-1],[3,-6],[5,-2],[4,-1],[5,-2],[-2,0],[-3,-1],[-3,-2],[-1,-2],[1,-2],[9,-4],[-2,-4],[0,-6],[2,-6],[2,-3],[1,-3],[3,-2],[1,0],[1,3],[-1,2],[-1,1],[-1,2],[0,3],[-1,0],[2,5],[0,3],[1,4],[3,3],[-3,2],[7,4],[8,3],[15,1],[-5,3],[-23,3],[-2,3],[-1,5],[2,7],[1,4],[-2,4],[-1,5],[2,6],[4,4],[14,8],[2,2],[5,6],[3,2],[2,1],[31,0],[32,-9],[33,-8],[17,-13],[11,-11],[5,-8],[3,-9],[-2,-5],[0,-4],[4,-9],[-4,-3],[2,-2],[3,-3],[2,-5],[0,-4],[1,-2],[0,-2],[0,-1],[-1,-3],[-1,-3],[-4,-1],[-2,-2],[-2,-5],[-1,-4],[-1,-4],[-2,-2],[-3,-1],[-6,0],[-11,-6],[0,-2],[1,0],[1,-1],[1,-2],[-2,0],[-5,-2],[-3,-1],[4,-2],[11,-1],[3,-4],[-2,-1],[-4,-5],[-2,-2],[-5,-3],[0,-2],[3,-4],[2,-2],[1,-2],[0,-2],[-1,-3],[-2,-2],[-3,-1],[-2,-2],[1,-2],[-2,-3],[-9,-4],[-6,-7],[-2,-1],[-1,-2],[-5,-10],[1,-2],[10,-6],[-4,-2],[-1,-2],[0,-3],[-1,-2],[-13,-3],[-4,-2],[-2,-2],[-1,-3],[-1,-3],[2,-1],[4,-1],[3,-2],[0,-4],[-1,0],[-42,-7],[-43,-6],[-1,-1],[-1,-1],[0,-1],[-1,-1],[0,-1],[-25,-14],[-25,-1],[-10,3],[-22,2],[-20,10],[0,4],[-4,4],[0,1],[1,3],[3,2],[1,2],[-1,6],[-3,4],[-3,2],[-3,0],[-2,-1],[1,0],[1,-2],[-1,-3],[3,-2],[1,-3],[-4,-5],[-9,-5],[-9,-2],[1,1],[1,0],[0,1],[-2,1],[-2,1],[-3,0],[-1,-2],[-1,-3],[1,-1],[3,-2],[2,-4],[-2,-2],[-6,-3],[2,-1],[4,-1],[2,-1],[-3,-2],[-10,2],[-40,-5],[-9,4],[-3,1],[-2,1],[-4,4],[-27,6],[0,2],[34,-3],[-4,8],[-6,6],[-6,4],[-19,4],[-12,-4],[-2,-2],[-1,-1],[-2,-6],[1,0],[1,0],[1,-1],[1,-2],[-11,0],[4,-3],[-9,-1],[-32,-10],[-2,-2],[-3,-4],[-1,-3],[1,0],[2,-1],[-5,-1],[-12,-8],[-5,-2],[-34,3],[-13,-4],[-1,-1]],[[5177,8527],[6,-3],[21,-1],[6,-4],[2,-1],[48,-2],[1,-1],[1,-1],[-1,-2],[0,-1],[0,-1],[1,-2],[2,-2],[2,-2],[2,0],[15,1],[4,-3],[1,-1],[0,-1],[1,-1],[1,-2],[1,0],[14,-1],[8,5],[4,5],[2,1],[3,1],[5,1],[11,-4],[13,-1],[4,-1],[3,-3],[3,-6],[3,-4],[9,-3],[5,-5],[4,-1],[2,-1],[0,-1],[2,-3],[1,0],[6,-2],[5,-4],[-1,-1],[0,-2],[3,-3],[1,-1],[2,-1],[4,2],[1,0],[2,-3],[0,-5],[1,-10],[-2,-3],[-8,-10],[0,-1],[0,-4],[0,-2],[-2,-2],[-3,-2],[-1,-2],[2,-4],[-5,-4],[-9,-5],[-5,-4],[-1,-3],[-1,-1],[-2,-1],[-15,-6],[-3,-2],[5,-8],[6,-4],[6,-2],[6,5],[-8,0],[-4,1],[-2,4],[8,6],[15,3],[2,1],[1,1],[2,4],[1,2],[7,4],[2,2],[1,1],[1,5],[1,1],[4,1],[3,2],[7,7],[-1,1],[-1,4],[4,3],[14,0],[4,-2],[12,-1],[10,-4],[3,-1],[19,6],[7,-1],[8,-4],[3,0],[4,0],[11,6],[9,-1],[16,9],[1,1],[32,4],[32,4],[34,-6],[5,-3],[10,0],[10,-4],[5,-3],[4,-6],[4,-4],[26,-6],[5,-3],[4,-6],[-1,-1],[-1,-2],[-1,-1],[-1,-1],[1,-2],[2,-1],[4,-2],[-3,-4],[-18,-3],[-31,7],[-2,3],[-3,4],[-4,2],[-2,-5],[1,-1],[1,-1],[-7,-3],[-9,0],[-23,9],[-24,-3],[0,-1],[5,-1],[6,-3],[2,-1],[4,-4],[2,-1],[46,-2],[3,-3],[1,0],[17,-1],[9,-3],[10,-1],[7,-6],[2,0],[5,3],[7,1],[44,-9],[43,-8],[43,-9],[1,-1],[0,-2],[1,-1],[7,-5],[3,-4],[-1,-5],[-2,-1],[-3,1],[-2,0],[-1,-1],[-1,-3],[-1,-1],[-2,-2],[-8,-3],[-40,-1],[-41,0],[-41,-1],[4,-4],[37,-3],[5,-3],[-5,-3],[-5,1],[-11,4],[-52,6],[-52,7],[-9,7],[-43,10],[4,-3],[6,-1],[3,0],[4,-2],[22,-8],[2,-2],[-2,-3],[-9,-3],[0,-2],[-6,-3],[0,-2],[4,-1],[14,5],[52,-8],[5,-2],[6,0],[2,-2],[-1,-1],[-4,0],[-2,-1],[0,-1],[-1,-1],[0,-2],[0,-1],[-3,-2],[-4,-1],[-7,0],[0,-1],[17,-3],[3,2],[4,3],[4,2],[8,2],[41,-6],[-5,-7],[-7,-2],[-46,0],[-46,1],[-6,-4],[5,-2],[14,2],[5,-3],[-3,-3],[-8,1],[-3,-2],[2,-1],[6,-3],[2,-1],[3,-1],[1,-3],[0,-3],[-2,-1],[-5,0],[-11,-4],[-5,-4],[1,-3],[2,-2],[0,-3],[-1,-6],[0,-2],[3,3],[0,-2],[2,2],[2,6],[2,2],[-1,2],[-1,1],[3,2],[13,5],[9,7],[1,3],[3,6],[2,2],[3,1],[4,-1],[4,-2],[3,-3],[-4,0],[-2,-1],[-1,-2],[4,-2],[4,1],[13,6],[4,0],[5,-1],[5,-3],[-3,-2],[-4,-2],[-4,-3],[-1,-6],[1,-4],[3,2],[7,7],[8,3],[2,1],[1,4],[2,1],[8,-1],[2,-2],[4,-4],[2,-2],[7,0],[1,4],[3,6],[0,3],[4,5],[4,2],[5,-1],[4,-2],[-2,-2],[-1,-3],[0,-2],[2,-2],[10,-1],[2,-1],[2,-3],[1,-2],[1,-2],[2,-2],[10,-5],[-2,-5],[-7,-10],[-2,-5],[1,-1],[3,-2],[-1,-1],[-2,-2],[2,-1],[2,-1],[2,1],[1,1],[1,1],[3,5],[2,1],[5,4],[2,0],[3,0],[2,0],[1,3],[-8,0],[1,1],[1,1],[1,0],[9,-1],[8,-2],[14,-10],[14,-3],[3,-3],[1,-4],[-1,-4],[-3,-6],[-1,-4],[-4,-11],[-2,-6],[0,-1],[-2,-1],[-3,1],[-12,-4],[-27,1],[-2,-4],[-4,-3],[-4,-2],[-3,-1],[6,-2],[12,5],[7,-5],[-5,-2],[-10,1],[-4,-2],[4,-2],[15,1],[4,1],[6,6],[4,1],[24,-1],[2,-2],[2,-3],[3,-9],[1,-1],[3,-1],[0,-2],[0,-3],[1,-2],[2,-2],[10,-2],[2,-1],[-1,-5],[2,-2],[3,-2],[2,0],[18,2],[0,2],[1,5],[0,3],[-4,11],[-3,6],[-10,14],[11,4],[6,5],[0,7],[4,2],[4,0],[8,-5],[4,-1],[11,-12],[17,-12],[12,-4],[2,-2],[1,-4],[-2,-1],[-3,0],[-5,-3],[-2,-1],[0,-1],[7,-4],[1,-1],[0,1],[2,-1],[1,1],[1,2],[2,2],[21,10],[-1,0],[1,1],[1,1],[2,1],[1,0],[23,8],[10,-3],[4,-3],[0,-5],[2,-3],[10,-6],[3,-1],[-2,-4],[-3,-3],[-4,-1],[-3,-1],[0,-1],[10,0],[6,1],[3,4],[1,7],[-3,4],[-4,2],[-2,4],[0,4],[1,2],[4,0],[2,1],[3,3],[2,1],[0,1],[1,2],[1,2],[2,2],[11,1],[18,-7],[1,-1],[1,-2],[1,-2],[1,-3],[2,-2],[32,-18],[22,0],[7,-2],[14,-9],[3,0],[-3,-4],[-3,-2],[-22,-9],[24,2],[23,9],[23,2],[20,-4],[34,-1],[0,2],[-3,1],[-1,0],[0,-1],[-3,2],[-24,3],[-5,2],[-8,7],[-23,4],[-2,1],[-2,2],[-1,3],[2,1],[5,1],[9,4],[34,5],[14,10],[7,1],[-4,-5],[-1,-2],[3,-1],[10,-2],[8,3],[18,-3],[0,2],[-8,2],[-3,-1],[1,2],[1,2],[2,0],[1,0],[-1,1],[-1,0],[0,2],[9,0],[2,1],[1,2],[0,2],[1,2],[2,1],[8,-1],[2,0],[5,3],[2,0],[0,2],[-2,1],[-1,0],[27,0],[26,-7],[8,2],[-4,2],[-9,0],[-5,2],[2,2],[0,3],[-2,5],[42,5],[9,10],[4,2],[18,2],[9,8],[2,2],[-7,2],[-2,2],[18,-4],[12,3],[4,-1],[-2,-1],[-4,-2],[8,-4],[18,2],[10,-11],[4,-1],[15,-2],[7,3],[3,0],[6,-4],[4,1],[52,19],[10,0],[4,2],[10,0],[10,3],[45,-5],[46,-5],[46,-5],[2,-1],[0,-4],[-1,-1],[-2,-3],[-1,-1],[-2,0],[-2,-2],[-8,-8],[-5,-8],[4,-2],[8,1],[4,-2],[5,-2],[9,-2],[29,3],[29,4],[5,-2],[1,-2],[1,-1],[3,-2],[9,1],[13,-5],[5,-4],[8,-2],[3,-2],[2,-4],[-1,0],[-1,-1],[0,-2],[-1,-2],[4,-1],[9,2],[3,-3],[-5,-4],[-39,-10],[-4,-4],[41,1],[41,0],[6,-2],[2,-2],[-6,0],[-3,-1],[-3,-2],[7,-7],[3,-2],[4,-1],[4,-1],[3,-1],[1,-2],[0,-3],[-2,-2],[-1,0],[-2,-1],[1,-1],[1,-1],[-2,-5],[-8,-2],[-1,-4],[3,-3],[16,-1],[-1,-3],[-5,-3],[-2,-2],[0,-6],[-10,-7],[-3,-5],[-14,-2],[-5,2],[-2,0],[-1,-1],[-1,0],[0,-1],[0,-2],[-2,-2],[-1,0],[-5,-1],[-2,-1],[-1,-1],[-2,-2],[-1,-3],[-1,-2],[0,-2],[0,-2],[-36,-8],[-4,-4],[-3,-1],[-11,0],[11,-7],[3,-2],[4,-3],[8,1],[4,-3],[-9,-2],[-2,-2],[0,-3],[3,0],[6,1],[7,3],[3,1],[3,-2],[3,-1],[1,0],[2,2],[1,3],[-1,0],[28,12],[4,0],[12,-4],[5,1],[10,-5],[-3,0],[-1,-1],[-2,-1],[2,-4],[6,-5],[6,-8],[12,-4],[-3,-5],[-4,-2],[-48,-10],[-7,-4],[-3,-1],[-5,1],[-24,10],[-1,1],[0,1],[0,2],[1,4],[2,3],[0,4],[-3,4],[-3,1],[-1,-2],[0,-4],[2,-5],[-6,-5],[-2,1],[-3,4],[1,0],[-2,5],[-1,4],[-1,2],[-10,-7],[-1,-3],[2,-4],[4,-3],[29,-11],[0,-1],[-1,0],[-1,-1],[-1,-1],[1,-1],[1,-3],[1,-1],[-4,-2],[-9,-2],[-4,-4],[11,-7],[3,-3],[-3,-3],[-3,-1],[-3,-1],[-1,-5],[3,-3],[1,0],[0,-1],[-2,-4],[5,-2],[-1,-3],[-4,-3],[-4,-2],[2,-4],[-6,-2],[-18,2],[-2,0],[-3,-1],[-5,-3],[-3,-1],[-32,1],[-33,2],[-23,-4],[-13,-8],[-9,-2],[-5,-4],[-7,0],[-13,-9],[-6,-2],[-7,2],[-4,0],[-10,3],[-11,0],[-14,5],[-4,3],[-22,2],[2,1],[2,2],[1,4],[-2,4],[-3,2],[-3,0],[-2,-3],[1,-2],[-1,-1],[0,-1],[-1,-1],[-11,-6],[-4,-1],[-4,1],[-4,2],[-2,1],[-5,-1],[-20,7],[-4,3],[-15,18],[-3,7],[-2,9],[1,4],[1,4],[1,3],[1,2],[4,1],[3,3],[-1,3],[-4,1],[0,6],[-2,2],[-4,1],[-7,-1],[-19,7],[-7,8],[-8,5],[-7,-2],[-1,-2],[1,-1],[0,-2],[4,-9],[2,-3],[4,-3],[6,-4],[7,-2],[2,-3],[1,-4],[-1,-4],[-6,-10],[0,-2],[-1,-3],[-4,-3],[-2,-2],[1,-2],[-2,-6],[-1,-5],[0,-4],[-3,-4],[-2,-2],[-6,-1],[-4,-4],[-2,-1],[-3,0],[-5,2],[-40,-2],[-12,-6],[-12,-3],[-28,1],[-13,-2],[-2,1],[-3,3],[-2,1],[-2,1],[-5,-1],[-12,-5],[-8,2],[0,1],[-1,2],[-1,3],[-1,5],[-2,1],[-2,1],[-1,2],[0,3],[1,4],[1,2],[-1,3],[-2,6],[4,4],[0,4],[-2,4],[-3,2],[0,-6],[-2,-3],[-3,-2],[-3,-4],[2,-3],[1,-3],[0,-3],[-1,-5],[1,-1],[1,-4],[-1,0],[-1,-1],[1,-1],[1,-3],[1,-1],[-4,-7],[-7,-3],[-6,-1],[-17,0],[-3,3],[-1,2],[0,2],[1,2],[-1,4],[-1,3],[-4,4],[-2,2],[-5,19],[-1,2],[-1,3],[-2,1],[-3,0],[-1,-1],[1,-4],[-1,0],[-1,-1],[2,-2],[1,-2],[0,-4],[0,-4],[1,-3],[4,-12],[0,-2],[-1,-2],[-4,-5],[-3,-2],[-6,-2],[-29,-2],[-30,-1],[5,12],[0,3],[-2,4],[-6,4],[-3,3],[2,1],[1,2],[-1,2],[-2,2],[-2,0],[-3,-2],[-1,-3],[0,-4],[1,-7],[-3,-7],[-4,-5],[-3,-2],[-7,-3],[-7,1],[-25,11],[-6,5],[-1,6],[-2,0],[-2,1],[-1,3],[2,4],[-2,1],[-1,1],[-2,0],[-1,0],[0,-2],[1,-4],[-1,-6],[0,-6],[2,-2],[5,-3],[10,-10],[5,-2],[0,-2],[-35,-1],[-2,1],[-8,7],[0,1],[-1,2],[0,1],[-2,-2],[-1,-1],[0,-2],[0,-1],[1,-1],[-2,-2],[-3,-1],[-5,-1],[1,0],[-17,0],[-8,3],[-5,11],[5,1],[2,1],[2,3],[-8,-1],[-4,-2],[-4,-4],[0,-2],[-1,-3],[-1,-2],[-1,-1],[-10,-4],[1,0],[-11,-3],[-7,1],[-4,3],[-1,2],[-1,2],[1,4],[-1,3],[-2,0],[-8,-7],[-4,-2],[-21,-1],[-6,2],[-4,5],[1,1],[-1,0],[-2,-1],[-4,-5],[-4,-1],[-31,5],[-2,2],[-2,3],[-1,4],[-1,4],[0,2],[0,2],[3,10],[1,4],[2,1],[1,1],[4,7],[3,5],[1,3],[1,3],[-1,3],[1,0],[1,1],[1,1],[-1,2],[1,2],[0,2],[-3,1],[16,2],[0,2],[-3,1],[-13,0],[-2,1],[-3,1],[-1,0],[1,3],[1,3],[1,3],[2,1],[-2,3],[-8,13],[-1,1],[-4,0],[0,-2],[1,-4],[3,-6],[-2,-3],[-2,-3],[-5,-2],[1,2],[0,2],[-1,2],[-1,2],[-2,1],[-2,-2],[-2,-7],[0,-1],[-1,-1],[0,-2],[-2,-1],[-2,-1],[-1,-1],[-1,-2],[-1,-2],[0,-3],[0,-2],[1,-5],[-2,-5],[-5,-6],[-1,-3],[-2,-2],[-2,-1],[-2,0],[-2,1],[0,2],[0,2],[1,1],[-1,2],[-2,3],[-1,2],[-1,2],[0,1],[1,1],[0,1],[0,4],[0,2],[0,3],[-1,2],[-2,3],[-9,4],[-7,6],[-2,1],[2,-6],[2,-4],[3,-3],[3,-1],[-3,-11],[-3,-4],[-2,-1],[-3,2],[-6,1],[-9,6],[-3,1],[2,-2],[5,-7],[10,-6],[2,-3],[-1,-4],[-1,-6],[-2,-5],[-2,-4],[-5,-4],[-6,-2],[-15,1],[-1,0],[-2,-2],[-1,-4],[-1,-1],[-1,-1],[-16,-1],[1,1],[-3,1],[-3,0],[-5,-4],[-4,-1],[-12,1],[-13,-3],[-3,1],[-1,3],[-2,7],[-1,3],[-3,1],[-2,-1],[-4,-5],[-3,-2],[-3,-1],[-29,12],[5,4],[2,2],[-18,-5],[-11,1],[-15,9],[-4,4],[-1,3],[0,3],[-1,4],[-1,3],[3,1],[1,1],[0,3],[-1,-1],[-1,0],[-2,1],[-1,-2],[-6,0],[-3,-1],[-1,-2],[-1,-2],[0,-2],[-1,-1],[-3,-3],[-5,1],[-9,4],[4,8],[11,5],[5,6],[4,8],[1,1],[4,2],[1,1],[1,4],[0,3],[-1,2],[0,3],[-1,1],[-2,0],[-4,0],[1,0],[-3,-1],[-2,-3],[-5,-7],[-22,-22],[-3,-1],[-2,1],[-1,1],[-2,1],[-2,-1],[-4,-3],[-1,-1],[0,-3],[2,-2],[2,-2],[1,-1],[0,-5],[7,-9],[3,-5],[-21,2],[0,2],[-3,5],[3,3],[-2,7],[-3,4],[-5,-4],[2,-2],[1,-2],[1,-3],[-1,-3],[-3,-4],[-3,-1],[-13,0],[-14,5],[-6,5],[2,1],[4,3],[2,2],[-5,4],[-6,1],[-11,-1],[1,-5],[-2,-1],[-4,1],[-3,1],[2,3],[7,4],[-6,3],[0,-2],[-6,4],[-12,4],[-5,7],[-1,3],[1,3],[3,4],[1,1],[1,0],[0,1],[0,2],[0,2],[1,2],[0,2],[-1,2],[0,1],[0,1],[0,1],[-1,3],[-2,2],[-4,2],[3,2],[-1,4],[-15,15],[-4,2],[0,2],[2,0],[1,1],[0,2],[0,3],[-2,0],[-1,0],[0,2],[0,1],[2,1],[1,0],[5,3],[12,-2],[4,3],[-9,5],[-2,4],[6,2],[0,2],[-2,1],[-18,-4],[-5,0],[-5,2],[0,4],[-2,1],[-3,1],[-2,1],[0,1],[1,2],[-4,1],[-7,5],[-3,1],[1,2],[3,3],[1,3],[0,3],[-1,3],[0,4],[3,9],[1,2],[3,3],[6,3],[3,3],[-4,0],[-8,-1],[-4,1],[10,19],[9,14],[5,3],[2,2],[3,4],[2,1],[12,4],[5,2],[4,5],[-2,2],[-5,2],[-1,3],[1,2],[6,6],[-2,1],[-1,2],[-2,4],[-11,9],[-2,5],[1,2],[5,8],[1,3],[1,2],[0,4],[-1,8],[-2,3],[-2,3],[-4,2],[-9,1],[-17,6],[-3,4],[-1,4],[-1,1],[-2,1],[-4,0],[-2,1],[-2,2],[0,3],[-1,3],[-2,2],[-2,0],[-3,1],[-1,1],[-1,8],[0,2],[-1,3],[-1,2],[1,1],[1,3],[0,1],[-1,4],[-3,4],[-2,2],[-6,2],[-2,3],[-2,4],[-1,5],[-2,3],[-4,2],[-10,3],[-9,11],[-5,3],[-2,3],[-1,4],[3,3],[-1,1],[0,1],[0,1],[-1,0],[1,0],[-8,5],[-9,2],[-9,-2],[-34,-17],[-9,1],[1,1],[3,1],[1,1],[-1,3],[-5,0],[-2,-1],[-3,-2],[-4,-6],[-1,-1],[4,-3],[7,-1],[4,-2],[-8,-2],[-16,4],[-11,-2],[-7,1],[-7,-1],[-14,6],[-47,-3],[-7,3],[2,2],[1,0],[-2,3],[-1,0],[0,2],[-9,0],[-1,-1],[-5,-6],[-5,-4],[-18,-4],[-2,-2],[-2,-3],[-2,-1],[-3,0],[-9,3],[-26,0],[1,2],[0,1],[0,1],[-1,3],[1,4],[1,3],[2,2],[8,-4],[21,4],[3,2],[0,1],[0,1],[0,1],[5,2],[17,0],[4,-3],[-1,-1],[0,-1],[-1,-2],[2,0],[2,1],[2,1],[1,3],[-5,3],[-32,7],[-3,-3],[-3,-1],[-2,1],[-4,4],[-2,1],[-2,0],[-4,0],[-1,-1],[-3,-2],[-7,-1],[-4,0],[-2,2],[1,3],[3,2],[3,1],[2,1],[-5,1],[-15,-4],[-31,3],[-2,2],[-6,4],[-9,3],[-7,7],[-8,9],[3,4],[3,2],[30,-1],[4,1],[4,3],[13,14],[2,3],[-4,0],[-3,-2],[-12,-10],[-3,-1],[-2,2],[0,1],[-3,0],[-10,-3],[-13,0],[1,3],[1,1],[1,2],[1,1],[-4,1],[-9,-2],[-4,1],[-2,2],[-1,2],[-1,1],[-2,3],[-6,1],[1,1],[2,3],[-17,0],[-5,3],[-3,2],[-2,3],[0,2],[-2,3],[-2,1],[-21,4],[-10,-3],[-11,1],[-6,3],[-5,6],[3,0],[4,1],[4,3],[3,4],[-2,0],[-3,1],[-2,2],[0,4],[2,2],[2,0],[6,-1],[2,-3],[1,0],[2,0],[2,0],[2,0],[7,-5],[-2,-3],[1,-1],[2,-1],[2,0],[8,3],[5,1],[3,-3],[0,2],[11,-5],[7,0],[4,3],[-1,1],[-6,-1],[-2,1],[-4,2],[-8,2],[-2,0],[-6,-1],[-2,0],[0,1],[3,1],[6,1],[0,2],[-2,2],[1,1],[2,0],[2,0],[3,-3],[1,0],[11,1],[4,2],[-33,8],[-1,2],[-8,4],[-3,1],[-9,-1],[-3,2],[-4,4],[-2,6],[2,3],[9,4],[7,-1],[6,3],[-5,3],[-5,1],[-9,1],[1,2],[1,1],[4,0],[33,-3],[-1,-3],[2,0],[1,1],[0,1],[1,1],[6,2],[2,2],[4,1],[5,-2],[4,0],[2,5],[-7,0],[0,2],[1,0],[-2,2],[-5,1],[-3,2],[44,4],[12,4],[14,2],[12,-3],[27,-1],[3,-1],[5,-3],[5,-1],[8,-6]],[[6330,8634],[-13,0],[-2,1],[1,3],[4,6],[-2,3],[-2,1],[-3,0],[-5,4],[-34,2],[2,2],[7,1],[2,1],[4,4],[4,1],[4,2],[2,1],[35,-2],[3,-3],[5,-8],[1,-2],[4,-2],[3,0],[3,-1],[3,-4],[-6,-4],[-20,-6]],[[5736,8675],[8,-4],[1,-1],[1,-1],[0,-1],[1,-2],[2,-1],[5,-1],[41,-23],[7,-7],[-1,-4],[1,-4],[1,-5],[2,-3],[-1,-1],[-1,0],[-1,-2],[0,-2],[1,-3],[2,0],[2,0],[1,-1],[2,-3],[0,-2],[-1,-2],[-15,-13],[-35,-16],[-8,2],[-4,-1],[-4,-1],[-14,-2],[-5,1],[-1,2],[0,1],[2,2],[1,3],[-30,9],[-31,8],[-14,12],[-6,2],[-1,2],[-2,3],[-5,3],[-3,3],[0,3],[1,3],[1,3],[2,2],[-3,13],[-2,3],[2,3],[0,1],[1,1],[0,2],[-3,6],[1,1],[1,2],[1,3],[0,2],[2,2],[5,2],[4,-2],[4,1],[11,8],[8,1],[33,-3],[33,-4]],[[4071,8623],[2,0],[24,2],[5,-1],[2,-6],[3,-2],[18,-10],[4,-4],[2,-1],[6,-2],[2,-1],[-2,-1],[-2,-1],[-1,-1],[-2,-2],[4,-3],[3,0],[2,-1],[2,-4],[1,-5],[0,-1],[-2,-9],[-7,-10],[-7,-3],[-15,0],[-12,-7],[-4,0],[-4,1],[-4,3],[-6,7],[-5,3],[-6,2],[-10,0],[-10,-3],[-4,0],[-5,3],[2,5],[1,2],[-4,2],[-9,0],[-5,1],[0,2],[2,-2],[3,0],[2,0],[2,2],[-4,5],[-4,3],[-9,4],[2,1],[1,0],[4,0],[0,2],[-9,2],[-6,2],[-3,4],[-1,3],[0,1],[1,4],[2,3],[1,1],[-1,0],[-3,2],[-3,1],[-7,0],[1,2],[1,1],[2,2],[-1,2],[-6,3],[4,4],[-3,3],[-9,3],[1,3],[1,2],[2,0],[2,1],[2,4],[1,2],[-1,1],[-3,0],[-4,3],[-10,2],[-2,2],[4,3],[0,1],[-2,2],[2,5],[-2,6],[-4,5],[-3,4],[-17,9],[-4,5],[-1,3],[2,3],[19,7],[30,-4],[5,-3],[11,-5],[29,-22],[29,-22],[3,-5],[2,-7],[-3,-4],[-4,-2],[0,-5],[0,-2],[0,-3],[-1,-2]],[[5159,8712],[-1,-5],[1,-3],[2,-2],[3,0],[3,2],[2,3],[2,2],[6,-6],[5,0],[13,3],[11,7],[4,1],[34,-2],[33,-2],[-1,-3],[8,0],[-2,2],[0,1],[15,-3],[5,0],[8,3],[5,-2],[2,-2],[4,-2],[3,-3],[1,1],[1,1],[2,1],[7,-3],[2,1],[1,1],[1,3],[2,2],[2,1],[19,3],[4,-1],[13,-6],[4,1],[2,0],[4,-3],[2,-1],[9,0],[4,-2],[3,-2],[4,-4],[1,-3],[7,-7],[-12,-5],[-5,0],[-9,0],[-5,-3],[-4,-6],[1,0],[1,-1],[1,-2],[1,-1],[-11,-8],[-1,-1],[-1,-1],[0,-1],[1,-4],[0,-2],[-1,-2],[-2,-1],[4,-5],[-3,-6],[-5,-4],[-4,-1],[-7,3],[-4,0],[-4,-2],[-3,0],[-8,3],[-4,0],[2,-1],[5,-2],[2,0],[-2,-2],[-3,0],[-5,2],[-12,-2],[0,1],[1,1],[0,1],[0,1],[-3,1],[-3,1],[-10,-3],[-15,2],[-12,5],[-47,0],[-47,0],[-46,0],[-47,0],[-10,5],[-2,1],[-4,6],[-20,16],[-3,3],[2,1],[3,1],[1,2],[-2,1],[1,2],[0,2],[0,1],[0,2],[3,0],[1,2],[-1,3],[0,3],[3,4],[12,1],[10,6],[7,2],[9,4],[20,3],[3,1],[2,1],[4,-2],[2,1],[13,8],[7,0],[5,-7]],[[7154,8715],[-10,-4],[-3,2],[-4,5],[-5,7],[8,4],[27,5],[6,-2],[3,-2],[-2,-3],[0,-2],[1,-2],[-2,-3],[-8,-4],[-11,-1]],[[4533,8703],[-3,-4],[-3,0],[-6,1],[-12,-3],[-22,3],[-6,-3],[-6,0],[-6,2],[-6,1],[-8,-7],[-19,-6],[-43,4],[-13,6],[-18,3],[0,2],[1,0],[1,1],[1,1],[1,3],[-3,0],[-4,2],[-3,3],[-2,3],[2,4],[0,2],[-4,4],[4,7],[6,4],[5,2],[34,1],[34,0],[9,3],[3,0],[6,-4],[48,-12],[29,-19],[3,-4]],[[3507,8748],[0,19],[0,22]],[[3507,8789],[35,-1],[10,-5],[3,-5],[-2,-6],[-7,-8],[-2,-5],[-1,-1],[-11,-5],[-25,-5]],[[4294,8795],[-4,-1],[-17,2],[-4,2],[-2,6],[2,4],[2,4],[21,16],[5,2],[14,1],[7,-2],[4,-2],[2,-5],[0,-3],[0,-1],[0,-2],[0,-1],[-2,-2],[-8,-3],[-20,-15]],[[5990,8874],[1,-7],[-4,-6],[-9,-9],[1,-5],[-1,-6],[-3,-5],[-2,-3],[-5,-5],[-4,-4],[-6,-1],[-5,1],[-3,3],[0,5],[1,5],[2,5],[5,4],[5,3],[10,2],[0,1],[-1,1],[-1,1],[-1,0],[-2,1],[0,1],[2,3],[0,3],[0,3],[0,2],[1,2],[2,1],[2,1],[3,0],[1,0],[2,2],[5,2],[2,0],[2,-1]],[[3507,8844],[0,29],[0,53],[0,4]],[[3507,8930],[5,-2],[8,0],[4,-3],[0,-4],[2,-3],[3,-1],[6,-1],[11,-6],[-1,-1],[4,-3],[15,1],[1,-1],[1,-3],[2,-1],[2,0],[5,-2],[6,-6],[2,-2],[0,-3],[0,-2],[0,-1],[1,-2],[2,0],[3,0],[1,0],[1,-3],[0,-1],[-2,-2],[0,-1],[2,-2],[-6,-4],[-2,-3],[2,-4],[-4,-4],[-1,-1],[4,-4],[-2,-5],[-7,-7],[-1,-2],[1,-1],[-1,-1],[-1,0],[-10,2],[-35,-5],[-6,2],[-10,6],[-5,0]],[[7553,8931],[-10,-3],[-41,11],[-2,1],[0,2],[1,3],[1,1],[2,0],[1,0],[1,2],[1,3],[0,1],[2,2],[8,-1],[5,-2],[22,-4],[17,-7],[-3,-5],[-5,-4]],[[4990,8934],[22,-4],[12,2],[6,-2],[23,-14],[3,-2],[3,1],[5,2],[2,0],[7,-2],[0,-1],[-1,-2],[-2,-2],[-3,-2],[0,-3],[0,-2],[-1,-3],[-1,-1],[-2,-1],[-2,-1],[-1,-3],[42,-12],[4,1],[14,7],[5,2],[27,-4],[23,-10],[16,-4],[21,-12],[7,-2],[2,-2],[3,-6],[-3,-4],[-3,-3],[-55,-21],[-4,-3],[8,-4],[17,-2],[8,-4],[2,-7],[23,-13],[-4,-5],[1,-2],[3,-3],[0,-1],[-2,0],[-2,-1],[-2,-2],[-1,-2],[1,0],[-5,-6],[-5,-7],[-1,-2],[-2,-2],[-6,-3],[-6,0],[-12,3],[-12,0],[-3,-1],[-6,-5],[-2,-1],[-6,1],[-5,-3],[-15,-2],[-12,-6],[-32,-5],[-31,-4],[-3,1],[-2,2],[0,1],[0,1],[0,1],[0,2],[-1,1],[-9,1],[-4,-1],[-7,-4],[-4,-2],[-9,1],[-3,-3],[5,-4],[8,0],[8,2],[6,4],[0,-2],[-9,-3],[7,0],[-1,-2],[-2,-2],[-3,0],[-4,2],[-14,-3],[-8,-5],[-5,-2],[1,0],[1,0],[1,-1],[-3,-3],[-2,0],[-6,1],[-15,3],[-5,0],[-2,0],[0,2],[1,2],[0,1],[0,2],[1,1],[0,1],[0,1],[-1,2],[-1,1],[-1,1],[-1,3],[4,1],[4,3],[4,4],[3,4],[-40,8],[-10,8],[-27,8],[-6,0],[-3,1],[-2,3],[3,0],[7,3],[3,0],[0,1],[-2,3],[-1,1],[3,5],[41,-2],[40,-2],[5,3],[4,4],[-1,0],[-2,0],[-1,1],[-1,2],[1,1],[1,0],[1,1],[1,2],[-3,2],[-9,2],[-7,-1],[-16,6],[-6,5],[-18,4],[-16,-1],[-34,4],[-7,-2],[-3,1],[-1,3],[1,3],[4,0],[7,-3],[0,2],[1,-2],[1,2],[3,0],[1,1],[-8,7],[-8,4],[-18,4],[1,3],[2,2],[2,1],[1,1],[-1,2],[-5,5],[-2,1],[1,3],[1,2],[2,1],[2,1],[-4,1],[-17,3],[-16,9],[-1,1],[-1,2],[-3,5],[-1,2],[1,4],[3,2],[6,0],[-3,4],[2,2],[2,0],[3,-2],[24,0],[3,1],[3,1],[0,2],[-2,1],[0,1],[2,2],[-3,3],[-10,5],[-3,0],[-6,0],[-3,2],[2,3],[-3,2],[-7,4],[-1,2],[0,7],[0,2],[1,3],[-4,3],[-2,2],[2,1],[3,3],[1,1],[-3,2],[-2,3],[1,1],[23,11],[40,-2],[39,-3],[12,-6],[44,-9],[7,-8]],[[4225,9088],[32,-9],[10,2],[19,-5],[5,-4],[7,-6],[2,-2],[3,-1],[7,1],[3,-2],[-1,0],[-2,-1],[-1,-2],[-1,-2],[1,-2],[1,-6],[1,-2],[2,-1],[4,0],[2,0],[4,-5],[2,-1],[1,-1],[1,-2],[1,-1],[1,-1],[2,-1],[1,-1],[3,-1],[7,-1],[1,-2],[1,-5],[-1,-4],[-3,-4],[-1,-6],[-2,-7],[-9,-5],[-3,-5],[5,-2],[2,1],[1,1],[4,1],[9,0],[2,-1],[0,-1],[-1,0],[0,-1],[1,-2],[-6,-6],[0,-2],[1,-1],[1,-1],[2,-1],[-1,-2],[7,0],[1,1],[3,4],[11,9],[3,5],[-1,1],[0,1],[1,1],[-2,2],[-2,6],[-2,2],[2,1],[-1,0],[14,2],[2,1],[-1,2],[19,2],[18,11],[6,1],[29,-3],[1,0],[1,-1],[0,-1],[1,-1],[1,-1],[4,-1],[22,-13],[4,-2],[5,-5],[16,-4],[1,0],[1,1],[0,1],[0,2],[1,1],[1,0],[13,-7],[3,-5],[-3,-2],[-11,-12],[-6,-8],[-5,-4],[3,-4],[3,-1],[14,1],[3,-2],[0,-4],[11,-2],[8,5],[8,2],[19,0],[9,5],[14,2],[5,-2],[3,-2],[0,-4],[1,-3],[5,-3],[32,-9],[11,-9],[-4,-5],[-12,-5],[-3,-4],[7,-3],[7,-1],[12,4],[4,-2],[1,-2],[1,-2],[1,-2],[1,-1],[7,0],[6,2],[5,-1],[5,-2],[3,-4],[-3,-4],[-8,-1],[-3,-4],[3,0],[2,-1],[1,-1],[-1,-2],[-4,-3],[-2,-2],[-1,-2],[0,-2],[-1,-2],[-2,-1],[-15,-5],[-5,-4],[9,-5],[4,-4],[1,-5],[-1,-4],[-3,-11],[-2,-3],[4,-3],[22,0],[4,-2],[4,-4],[4,-7],[4,-7],[3,-2],[12,-5],[29,-25],[6,-3],[5,0],[-2,-5],[0,-2],[0,-4],[0,-3],[-1,-1],[-3,-4],[-8,-3],[-4,-3],[4,-6],[-1,-2],[1,-2],[3,-4],[2,-2],[1,-1],[0,-2],[-1,-1],[-6,-1],[-5,-3],[-4,-4],[-3,-1],[-34,-3],[-35,-3],[-3,-1],[-2,-2],[-3,-3],[-2,-2],[-6,-1],[-20,8],[-19,1],[-12,6],[-25,5],[-2,1],[2,4],[-1,4],[-3,4],[-16,13],[-3,6],[-2,4],[-1,3],[0,3],[1,4],[2,1],[3,-1],[2,2],[1,5],[-5,3],[-19,7],[-4,3],[-2,5],[0,2],[0,3],[0,2],[-1,3],[-3,1],[-4,0],[-19,-5],[-3,1],[-24,12],[-37,6],[-38,6],[-43,-11],[-6,1],[-19,7],[-4,3],[-2,6],[2,4],[15,8],[-9,4],[-33,-3],[-33,-4],[-6,-3],[-7,-1],[-6,-2],[-18,-1],[-10,-3],[-15,-8],[-4,0],[1,-5],[-5,-3],[-29,4],[-29,4],[-24,12],[-6,5],[-3,1],[-2,0],[-3,1],[-2,2],[-6,8],[-14,11],[-1,1],[-5,12],[0,2],[1,2],[2,3],[5,4],[10,4],[8,5],[20,1],[34,-7],[34,-8],[36,1],[15,-5],[5,-1],[5,2],[4,4],[1,4],[0,2],[-1,1],[-2,1],[-2,2],[2,2],[6,2],[1,-1],[1,0],[3,3],[-1,2],[-1,1],[-2,0],[-2,0],[2,2],[2,1],[-4,2],[-33,0],[-32,-1],[-4,3],[7,6],[8,3],[49,1],[-4,6],[-2,2],[4,3],[4,1],[13,0],[2,1],[1,2],[0,3],[-4,0],[-3,7],[-5,2],[-8,1],[-3,-2],[-3,-5],[-3,-3],[-4,1],[1,1],[1,3],[-2,1],[-3,0],[-13,-6],[-7,-2],[2,5],[4,3],[8,6],[-3,2],[-4,0],[-3,-2],[-3,0],[-6,3],[-3,0],[-2,-3],[0,-1],[1,-2],[0,-1],[-5,-4],[-6,-2],[-6,0],[-5,3],[1,0],[-2,2],[-10,0],[-4,3],[8,9],[34,14],[2,2],[0,3],[-2,2],[-20,7],[-2,3],[0,2],[0,2],[-1,2],[-3,2],[-19,2],[-2,-1],[-2,-3],[0,-3],[-2,-2],[-8,2],[-5,0],[-4,-2],[-4,-6],[-5,-10],[-2,-4],[-3,-3],[-13,-5],[-11,-8],[-5,-2],[-14,-1],[-6,2],[-2,5],[2,5],[15,11],[2,1],[0,2],[2,1],[1,1],[2,6],[6,8],[9,10],[-3,4],[-4,2],[-24,4],[-50,-10],[-19,3],[-3,3],[-2,4],[2,5],[0,2],[0,2],[-1,2],[-1,1],[1,2],[0,2],[-3,8],[0,3],[1,3],[4,6],[3,8],[3,3],[9,5],[2,1],[1,3],[0,2],[-2,2],[-5,0],[3,6],[6,2],[28,-9],[55,5],[54,5],[54,6],[7,-6]],[[4701,9274],[3,0],[34,5],[7,-2],[11,-7],[10,-3],[3,-3],[-1,-1],[-2,-2],[2,-3],[8,-5],[1,-3],[0,-4],[3,-3],[5,-4],[-2,-1],[-2,-2],[-2,-3],[-1,-4],[-1,-5],[1,-2],[5,-3],[3,-4],[3,-5],[2,-7],[0,-6],[-1,-3],[-2,-3],[-2,-2],[-2,-1],[-1,-2],[-1,-3],[-2,-3],[-2,-2],[-1,-1],[-2,-6],[-1,-2],[-2,-2],[-2,1],[-4,2],[0,4],[-1,3],[-8,5],[-39,7],[-3,2],[2,4],[1,1],[-1,1],[-2,3],[-1,1],[2,1],[2,1],[0,4],[0,2],[-1,2],[-31,12],[-4,0],[-10,-4],[-41,-1],[-3,1],[-7,6],[-3,1],[1,3],[3,4],[1,2],[0,2],[0,1],[0,1],[0,2],[0,1],[-1,2],[0,1],[0,2],[-2,3],[-2,2],[-1,3],[1,3],[2,1],[3,1],[2,2],[0,1],[2,3],[1,1],[-1,-1],[5,4],[32,14],[23,-3],[14,-7]],[[5150,9418],[7,-3],[29,1],[11,-3],[1,-1],[2,-2],[1,-1],[0,-2],[5,-4],[2,-2],[-25,-6],[-33,4],[-17,7],[-25,-1],[-34,3],[5,5],[6,2],[38,0],[-1,1],[22,0],[6,2]],[[5456,9565],[8,-4],[25,-4],[13,-5],[15,-2],[46,-23],[4,0],[-2,-2],[-3,-4],[-1,-2],[4,-3],[2,-3],[1,-3],[-3,-2],[-12,1],[0,-2],[24,-9],[21,-15],[-1,-3],[0,-3],[0,-2],[2,-2],[3,-2],[6,-5],[8,-2],[4,-4],[8,-10],[2,-2],[5,-1],[3,-1],[2,-3],[0,-1],[0,-2],[-1,-2],[3,-6],[5,-4],[43,-11],[11,-12],[2,0],[-3,-5],[-15,-9],[-2,-5],[4,-2],[6,-1],[5,1],[7,-1],[1,3],[1,1],[7,-4],[55,-5],[7,-6],[3,-2],[2,0],[15,-11],[3,2],[0,3],[-8,7],[-2,4],[13,2],[7,-2],[3,0],[0,3],[34,-1],[31,-17],[-5,-5],[-1,-1],[-14,-7],[-3,-4],[5,-1],[10,4],[5,-3],[-5,-7],[-1,-3],[3,-3],[-3,-3],[-4,-3],[-10,-4],[0,-3],[3,-4],[8,-10],[14,-9],[49,-18],[4,-3],[3,-2],[5,0],[6,1],[3,2],[-6,6],[-1,2],[10,-1],[13,-4],[4,-4],[11,-2],[0,1],[-4,2],[-16,12],[-9,3],[-3,5],[5,0],[6,-2],[4,-3],[2,0],[2,3],[-6,6],[-22,2],[-10,5],[-3,2],[-1,1],[-1,3],[-1,1],[-3,3],[-4,2],[6,8],[1,4],[-4,5],[0,4],[4,5],[6,4],[6,2],[5,4],[3,0],[44,-3],[43,-3],[3,-2],[2,-4],[4,-10],[1,-5],[-1,-9],[1,-5],[2,-5],[2,-2],[1,-3],[-1,-7],[1,-1],[0,-1],[0,-2],[-1,-1],[-1,0],[-2,1],[-3,-1],[-4,-4],[-41,-5],[-4,-4],[2,-1],[4,-1],[2,-1],[0,-3],[1,-1],[1,0],[2,-1],[0,-1],[1,-1],[0,-2],[2,-1],[4,-3],[4,0],[17,5],[53,-2],[5,-7],[1,-4],[2,-4],[5,-5],[10,-5],[3,-3],[-3,-4],[-7,0],[-7,-3],[-4,0],[-4,-1],[-6,-9],[-3,-2],[-14,-3],[-4,-2],[-2,-4],[6,0],[10,1],[36,17],[4,1],[4,-2],[-1,-6],[-9,-16],[-2,-5],[0,-4],[1,-4],[0,-6],[-1,-3],[-7,-13],[-1,-1],[1,-3],[-1,-1],[-2,-3],[-4,-3],[-25,-9],[-4,-5],[0,-2],[0,-2],[-1,-2],[-1,-2],[7,-4],[7,-2],[7,0],[6,5],[2,3],[4,6],[1,2],[4,2],[15,4],[17,-2],[3,-2],[-1,-5],[-1,-3],[2,-1],[3,1],[2,1],[0,1],[0,2],[1,1],[8,-1],[-2,6],[-4,3],[-8,3],[11,6],[11,2],[36,-1],[5,-2],[11,-8],[9,-5],[5,-4],[-2,-1],[-1,0],[1,-2],[1,-1],[1,-1],[-2,-2],[-3,-1],[-6,-2],[0,-1],[2,0],[4,-2],[2,-2],[0,-1],[0,-2],[0,-1],[1,-1],[1,0],[1,-1],[1,0],[-4,-2],[-8,0],[-3,-3],[4,-4],[7,-2],[6,1],[5,4],[3,6],[1,2],[4,2],[1,3],[-1,6],[4,14],[9,6],[10,0],[8,-3],[7,-6],[2,-3],[0,-3],[1,-2],[0,-2],[2,-2],[7,-6],[4,-4],[2,-6],[1,-3],[8,-3],[2,-3],[4,-5],[3,-2],[16,-6],[5,-4],[7,-6],[7,-6],[2,-3],[1,-6],[-5,-2],[-28,-7],[-5,-3],[-6,-6],[-2,-1],[-10,0],[-6,-4],[-6,-2],[-14,-1],[-5,-3],[-47,-10],[-46,-10],[-7,-4],[1,-2],[-1,-3],[-3,-5],[0,-2],[0,-3],[-1,-2],[-2,-1],[-15,-4],[-3,1],[-4,2],[-12,12],[-1,3],[-2,6],[-1,1],[-2,0],[-2,0],[0,-3],[0,-5],[-2,-7],[0,-2],[2,-6],[0,-2],[0,-1],[0,-1],[1,-2],[3,-4],[1,-2],[-11,-10],[-21,-13],[-10,-3],[-3,-4],[-6,-10],[-12,-10],[-3,-4],[-1,-1],[0,-2],[0,-1],[0,-1],[-2,-2],[-8,-5],[-3,0],[-3,1],[-21,11],[-3,3],[-3,5],[-8,16],[-1,4],[-1,5],[1,6],[4,12],[0,6],[-4,7],[1,1],[1,1],[2,1],[8,3],[10,6],[-1,2],[-1,1],[-1,1],[-1,1],[3,3],[4,2],[3,3],[2,5],[-4,1],[-5,-2],[-8,-8],[-2,-3],[0,-2],[-1,-2],[-3,-2],[-22,-1],[-5,-2],[-4,-6],[-1,-4],[1,-4],[1,-4],[1,-4],[0,-5],[-3,-9],[-2,-4],[0,-6],[1,-5],[2,-5],[7,-14],[1,-5],[-3,-2],[-18,-1],[-4,-4],[2,0],[1,0],[0,-1],[6,3],[24,0],[5,-2],[5,-4],[5,-6],[3,-5],[1,-5],[-2,-5],[-11,-14],[-4,-4],[-21,-3],[-5,3],[-30,27],[-1,1],[0,3],[-1,1],[-1,0],[-5,0],[-18,2],[-2,-1],[-2,-1],[5,-3],[1,-2],[-3,-3],[2,-1],[1,-1],[0,-2],[0,-3],[18,-22],[2,-3],[3,-3],[1,-4],[-2,-6],[-1,-3],[-12,-13],[0,-2],[-1,-5],[-1,-2],[0,-1],[-1,-2],[-1,-2],[-1,-7],[-2,-5],[-4,-8],[-5,-4],[-4,-2],[-17,2],[-6,4],[-22,19],[-4,8],[-4,3],[-1,3],[-1,3],[-2,1],[-3,3],[-12,6],[-2,2],[-3,5],[-2,2],[-8,5],[-3,0],[-1,1],[-17,16],[-1,3],[-2,6],[-2,2],[-6,3],[-6,9],[1,1],[-3,2],[-4,1],[-3,-1],[-4,-1],[-2,-3],[1,-6],[-2,-2],[1,-3],[1,-2],[3,-3],[-1,-2],[0,-2],[4,-6],[2,-1],[1,-1],[0,-1],[0,-1],[0,-1],[2,0],[-1,-4],[2,-2],[21,-10],[21,-28],[1,-2],[3,-9],[3,-4],[8,-8],[2,-6],[-5,-2],[-4,-1],[-5,1],[-4,2],[-2,1],[-1,2],[-1,2],[0,3],[-1,3],[-1,2],[-4,2],[-2,0],[-21,-1],[-4,2],[-3,3],[1,6],[-10,9],[-11,5],[-22,3],[-23,-3],[-19,3],[-5,-3],[5,-5],[13,-3],[25,-1],[5,-3],[-3,-3],[-2,-2],[0,-3],[1,-3],[2,-3],[5,-3],[3,-4],[-2,-5],[-3,-4],[-4,-1],[-28,2],[-32,-2],[-34,7],[-47,4],[-19,9],[-6,2],[-4,2],[-13,-4],[-2,-3],[-2,-1],[-2,-1],[-5,2],[-5,2],[-9,9],[1,3],[-2,2],[-12,0],[-2,1],[3,4],[10,1],[4,2],[-5,4],[-7,-1],[-12,-5],[-13,1],[-13,6],[1,4],[1,1],[-3,2],[-6,1],[-16,8],[-6,0],[-9,8],[-4,7],[2,6],[4,2],[7,-1],[-1,1],[0,2],[29,-3],[-1,0],[8,0],[5,1],[3,4],[-8,1],[2,2],[3,0],[6,-2],[35,4],[35,3],[14,-2],[3,-2],[5,-1],[2,4],[-2,5],[-3,1],[-7,-2],[-51,6],[-52,6],[-9,5],[-9,2],[-4,-1],[-7,-3],[-2,0],[-1,0],[0,-1],[0,-1],[-1,-1],[-4,-2],[-27,-3],[-7,1],[-8,3],[-14,11],[0,1],[3,0],[8,3],[3,2],[-3,3],[-17,-3],[-1,0],[-2,1],[-3,4],[-8,3],[-12,11],[-6,3],[4,3],[24,1],[45,-11],[11,1],[2,1],[1,1],[-1,2],[1,2],[-5,2],[-25,7],[-7,4],[-20,4],[-26,-2],[-11,1],[-4,3],[-3,5],[-3,4],[-4,2],[-8,2],[-23,16],[-3,5],[10,5],[21,4],[10,4],[14,1],[14,3],[5,-1],[2,0],[2,1],[1,1],[2,5],[12,9],[1,4],[4,5],[5,2],[6,1],[12,0],[16,-4],[10,1],[18,-3],[2,1],[5,3],[3,1],[25,0],[27,-3],[46,2],[35,4],[22,4],[16,-1],[30,3],[31,4],[19,0],[16,5],[3,2],[-1,0],[-5,3],[-39,-3],[-32,1],[-35,-6],[-44,-3],[-16,1],[-23,-3],[-31,4],[-30,5],[-5,2],[-3,3],[3,2],[8,7],[4,2],[49,-2],[0,2],[-3,1],[-2,0],[-1,0],[2,1],[3,0],[2,0],[4,-4],[3,-1],[16,1],[8,3],[2,0],[19,5],[19,0],[37,4],[2,3],[0,-2],[0,1],[1,1],[1,0],[-1,1],[0,1],[-1,1],[2,3],[1,0],[-5,1],[-18,-4],[-15,0],[-32,-3],[-7,-5],[-3,0],[-25,2],[-13,0],[-26,4],[-11,-1],[-6,1],[-4,4],[2,1],[7,2],[9,5],[3,0],[5,0],[8,3],[6,0],[-3,4],[-4,2],[-36,-1],[-27,-10],[-9,1],[-4,2],[-10,14],[-2,1],[-2,0],[-4,0],[-2,-1],[-1,-1],[0,-3],[1,-1],[3,-3],[3,-3],[7,-7],[2,-4],[-10,-5],[-33,-6],[2,3],[6,5],[6,7],[3,3],[-1,3],[-1,3],[-2,2],[-3,1],[-3,-1],[-2,-2],[-4,-3],[-4,-3],[-2,-2],[-1,-3],[-1,-3],[-5,-6],[-2,-1],[-1,0],[-6,0],[-6,2],[-3,0],[-3,-2],[2,-2],[-1,0],[-2,-1],[2,-1],[2,-1],[1,-1],[2,-2],[-4,-3],[-7,-2],[-7,-5],[-17,-6],[-13,1],[-10,4],[-2,1],[-6,-2],[1,2],[3,2],[1,2],[0,3],[0,3],[2,3],[15,9],[16,4],[3,0],[0,2],[-28,-2],[-44,9],[-3,-2],[1,-1],[1,-2],[0,-3],[-1,-2],[2,-2],[1,1],[1,2],[2,0],[2,0],[1,-1],[4,-4],[-6,-1],[-10,-5],[-49,-13],[-3,-4],[-2,-2],[-13,4],[-10,0],[-4,2],[2,2],[1,1],[1,1],[2,1],[0,1],[-4,0],[-4,1],[-4,1],[-3,3],[3,1],[2,0],[3,0],[2,-2],[2,0],[2,1],[4,3],[-2,2],[-11,0],[-1,1],[1,2],[-1,3],[0,1],[-1,1],[-1,0],[0,1],[3,3],[-8,0],[-4,-2],[-1,-5],[-20,3],[-9,1],[-2,2],[-4,1],[-6,0],[-3,3],[1,0],[2,2],[-2,1],[-1,0],[3,2],[7,0],[3,2],[-5,4],[-1,1],[-2,0],[-3,2],[-2,1],[2,0],[3,1],[2,2],[2,3],[-2,4],[-2,2],[0,3],[5,0],[9,4],[50,6],[50,6],[11,6],[33,12],[1,3],[0,2],[-3,5],[0,2],[5,3],[2,3],[0,4],[-3,2],[-3,1],[-2,-1],[-2,-2],[2,-2],[-6,-3],[-17,-3],[-2,-1],[-2,-5],[-1,-1],[-9,-2],[-3,-1],[-6,-5],[-42,-6],[-5,-2],[-38,1],[-37,0],[-32,21],[-5,1],[-5,-1],[-2,1],[-1,3],[0,3],[0,3],[0,2],[-2,3],[-2,1],[-14,7],[-10,1],[3,3],[6,1],[3,2],[-5,2],[-12,-5],[-5,2],[0,1],[0,2],[-1,2],[-1,1],[-1,0],[1,3],[3,2],[2,0],[6,1],[3,1],[8,0],[4,-2],[-2,-5],[3,1],[6,3],[4,1],[12,0],[4,2],[-8,5],[-40,4],[-4,3],[-3,3],[-4,7],[-2,3],[3,3],[29,5],[-20,1],[-6,-2],[-5,0],[-6,2],[-6,5],[-3,6],[2,0],[1,4],[1,3],[2,3],[4,2],[39,-2],[39,-16],[36,-4],[4,-2],[-1,0],[24,0],[33,-4],[24,3],[13,0],[48,-15],[4,1],[-5,5],[-2,1],[-6,2],[-7,4],[-10,3],[-12,9],[-6,1],[0,2],[30,14],[52,6],[-3,1],[-43,5],[-1,3],[-5,2],[-5,0],[-3,-1],[-9,-8],[-30,-12],[-50,-3],[-29,12],[-5,1],[3,3],[13,4],[25,0],[7,2],[5,5],[-3,3],[-5,-2],[-8,-4],[-21,0],[-2,1],[-2,3],[-2,1],[-30,-12],[-10,1],[-40,14],[-28,4],[1,1],[9,7],[2,3],[-3,2],[-10,0],[-12,3],[6,5],[6,-1],[7,-3],[6,-1],[1,3],[3,0],[11,-4],[9,-1],[3,2],[-5,4],[-1,1],[2,2],[11,3],[2,-1],[5,2],[17,1],[16,-6],[16,1],[12,-5],[14,-2],[-1,2],[-2,3],[-1,2],[1,0],[1,1],[-3,3],[-9,0],[-14,4],[-5,2],[-7,9],[-30,7],[-5,3],[15,4],[1,1],[2,2],[1,3],[1,2],[2,0],[-8,3],[-4,2],[-2,5],[46,2],[37,-1],[27,3],[8,1],[25,-9],[21,-3],[33,3],[29,-4],[2,1],[1,1],[2,0],[2,-1],[1,-1],[-1,-2],[4,-5],[6,0],[11,2],[0,2],[-25,13],[-42,6],[-16,-3],[-3,1],[-2,2],[-2,3],[-2,3],[-7,2],[-2,4],[3,4],[3,2],[31,2],[31,2],[-6,5],[-14,-1],[-7,1],[0,1],[0,3],[-2,1],[-7,0],[-9,-4],[-2,0],[-3,-2],[-20,2],[-37,9],[-2,3],[16,3],[5,4],[-4,2],[-5,0],[-23,-6],[-41,3],[-4,4],[2,2],[-1,1],[5,3],[33,6],[4,3],[-32,0],[-6,5],[22,10],[3,3],[4,13],[7,3],[15,2],[15,7],[5,0],[36,-8],[2,-1],[2,-1],[4,0],[2,-2],[-6,-4],[0,-2],[0,-3],[3,-1],[4,-1],[4,0],[5,1],[4,3],[4,4],[0,3],[2,1],[3,0],[4,-3],[4,0],[4,1],[2,1],[3,4],[19,-2],[0,1],[-16,4],[4,2],[15,-2],[-7,3],[-23,2],[-1,0],[-3,3],[-3,1],[-4,2],[-8,2],[-2,2],[2,1],[4,3],[17,-6],[37,2],[38,-9],[0,2],[-1,0],[-1,1],[-1,2],[23,-1],[18,3],[-2,1],[-1,1],[2,1],[3,1],[2,1],[-2,2],[-1,1],[-2,1],[-2,-1],[3,3],[4,1],[2,2],[-1,5],[-4,4],[-8,-1],[-4,3],[3,3],[1,0],[-41,2],[-22,-4],[-17,1],[-27,3],[-19,5],[-7,5],[2,4],[2,3],[12,7],[1,2],[-1,2],[-2,1],[-1,0],[4,4],[6,2],[30,-2],[29,-5],[8,-5],[4,-2],[0,-1],[1,1],[3,2],[2,0],[1,1],[1,1],[1,2],[-1,1],[-3,1],[-2,2],[-2,3],[-1,4],[2,1],[2,1],[30,-4],[19,-6],[36,-7]],[[8068,9999],[3,-3],[-2,0],[-2,0],[-4,-3],[2,-1],[3,-1],[2,-1],[-3,-2],[-7,-2],[-2,-3],[9,0],[-1,0],[3,-1],[22,0],[4,-2],[-6,-1],[-12,-7],[-6,0],[5,-4],[45,6],[20,8],[5,-1],[2,-4],[-1,-7],[4,-3],[31,0],[5,1],[11,5],[41,-2],[4,-3],[-3,-4],[-3,-6],[7,-3],[7,1],[8,4],[45,4],[47,-9],[8,3],[0,3],[2,1],[18,-2],[-4,-3],[-1,-2],[10,-2],[5,0],[7,8],[5,-1],[10,-3],[28,0],[5,-2],[5,1],[2,-1],[-2,-2],[-8,-1],[1,-2],[2,0],[2,-1],[-2,-4],[-50,-15],[0,-1],[-5,-3],[-34,-5],[-11,-6],[-27,-2],[-6,-2],[-7,0],[-9,-3],[-12,0],[-9,-4],[-32,-5],[-32,-4],[-6,-4],[-17,-4],[-4,-3],[6,-2],[44,2],[45,2],[44,2],[20,9],[13,1],[5,3],[28,3],[21,-3],[4,3],[-1,1],[-1,1],[-1,0],[-2,0],[2,4],[4,3],[44,10],[44,10],[40,0],[4,-3],[-1,-2],[-6,-4],[3,-5],[6,-2],[33,-2],[-6,4],[-12,1],[-5,4],[4,1],[14,9],[4,1],[7,0],[-5,4],[-13,-1],[-5,2],[3,2],[8,0],[8,5],[49,2],[5,-2],[-3,-4],[-4,-1],[-21,1],[-4,-3],[6,-3],[20,0],[-1,-2],[-4,-5],[5,-4],[15,-5],[10,-6],[5,-2],[5,2],[-1,1],[-1,2],[-1,1],[4,2],[7,-1],[22,12],[39,-2],[39,-1],[4,-2],[3,-3],[-1,-2],[0,-2],[0,-3],[1,-2],[-6,-4],[-6,-3],[-41,-4],[23,-1],[8,-4],[4,0],[0,-1],[-1,-3],[-1,-1],[2,-1],[7,1],[2,-1],[3,-2],[13,-2],[10,-3],[-3,-5],[-5,-1],[-9,1],[1,-4],[47,-4],[3,-6],[-3,0],[-5,-1],[-4,-3],[-3,-4],[0,-1],[0,-2],[0,-2],[-2,-4],[-1,-2],[-1,-2],[-2,0],[0,-2],[3,0],[0,-1],[-25,-2],[-4,-2],[-3,-3],[38,4],[6,3],[10,10],[3,1],[3,1],[1,-1],[1,-1],[0,-3],[1,-1],[2,-1],[8,0],[-1,2],[-1,1],[-1,1],[-1,1],[3,3],[32,1],[32,0],[0,-1],[-15,-1],[-3,-2],[1,-2],[-1,-1],[-1,-1],[-1,-3],[10,2],[3,3],[-1,0],[-2,2],[18,0],[0,-1],[0,-3],[29,-1],[30,0],[1,-1],[26,-9],[23,-16],[4,-5],[2,-7],[0,-6],[-2,-7],[-2,-5],[-2,-2],[-3,-1],[-17,-12],[-40,-7],[-41,-7],[4,-2],[10,-1],[4,-3],[-5,-4],[-10,-2],[-9,-7],[-20,-9],[-15,-2],[-9,2],[-3,-1],[-1,-1],[0,-2],[2,-1],[-3,-3],[-30,-9],[-30,-9],[1,0],[-9,-1],[-45,-13],[-46,-14],[-23,1],[-18,7],[-5,0],[2,-3],[22,-12],[-3,-4],[-5,-2],[-13,-1],[-8,-4],[-46,2],[-11,5],[-5,1],[-14,-2],[-37,3],[1,0],[1,-1],[1,-2],[1,-2],[-3,-1],[-38,-5],[-39,-5],[4,-3],[16,3],[18,-4],[42,4],[-4,-3],[-55,-13],[-8,3],[-2,3],[-6,0],[-17,-6],[-27,-2],[-27,1],[-50,-3],[-34,-2],[-34,-2],[-31,-4],[-30,-4],[-42,13],[-41,13],[-41,13],[6,-6],[49,-16],[49,-16],[3,-6],[-56,2],[0,-2],[31,-6],[50,4],[49,5],[49,4],[35,-5],[34,-4],[6,-4],[-5,-4],[-33,-6],[-19,-3],[-19,-5],[-43,-8],[-43,-9],[-12,-5],[-12,-1],[-21,-8],[-15,-2],[-6,-3],[-12,0],[-10,-4],[-7,1],[-6,-3],[-10,-1],[-45,3],[-6,-3],[4,-3],[10,0],[4,-2],[-6,-1],[-24,-6],[-14,0],[-25,-6],[-27,1],[-6,-3],[16,-5],[50,5],[-3,-2],[-9,-3],[-9,-5],[-20,-6],[-3,-2],[44,8],[11,9],[6,2],[48,8],[48,8],[48,8],[48,8],[1,1],[32,6],[33,7],[41,5],[35,6],[35,6],[30,4],[25,7],[55,5],[35,5],[44,4],[2,-1],[1,-2],[1,-2],[2,-2],[4,-4],[2,-2],[1,-3],[-2,-3],[-2,-2],[-25,-17],[-48,-14],[-32,-16],[-20,-4],[-11,1],[-22,-6],[-14,-1],[-13,-7],[-16,-5],[-3,-3],[-3,-3],[-18,-7],[-2,-2],[-4,-6],[-2,-2],[-16,-10],[-23,-6],[-41,-18],[-37,1],[-5,-3],[2,-2],[7,-6],[-4,-4],[-5,-3],[-8,-3],[-11,0],[-6,-1],[-4,-4],[2,-2],[-1,-1],[-8,-3],[-7,-4],[-33,-9],[-14,-9],[-8,-2],[-12,-7],[-8,-2],[-14,-9],[-22,-5],[2,-2],[2,-1],[5,0],[-4,-5],[-23,-11],[-4,-4],[-6,-8],[-6,-4],[-1,-2],[-1,-3],[0,-3],[-1,-2],[-4,-5],[-5,-3],[-33,-10],[-36,0],[-37,1],[2,4],[7,6],[2,4],[-6,9],[-7,5],[-52,23],[-2,0],[-2,-1],[0,-3],[1,-1],[5,-3],[6,-1],[4,-2],[12,-9],[8,-4],[3,-3],[3,-6],[1,0],[2,-1],[7,-4],[-8,-4],[-2,-3],[-1,-3],[-1,-3],[1,-3],[2,-1],[-1,0],[50,-14],[6,-6],[-4,-4],[-10,-3],[-8,-9],[-6,-1],[-31,2],[-30,2],[-40,-5],[-29,-8],[-16,-8],[-27,0],[-12,5],[-1,1],[-6,7],[-3,2],[-8,2],[-18,1],[-19,9],[-6,-1],[-4,-3],[45,-15],[10,-7],[4,-1],[0,-2],[-24,-6],[-32,1],[0,-2],[8,0],[-2,-2],[-1,-1],[-1,0],[2,-2],[19,-1],[37,1],[15,3],[21,-1],[23,5],[36,8],[40,5],[5,-4],[2,-2],[3,-2],[3,-1],[2,-1],[2,-1],[0,-6],[1,-3],[-4,-4],[-12,-1],[-5,-6],[0,-2],[-3,-4],[-38,-6],[-38,-6],[-5,-2],[-3,-4],[-2,-4],[4,-3],[28,7],[30,-6],[-2,-3],[-6,-6],[-2,-3],[-2,-5],[-8,-1],[0,-4],[-6,-3],[-27,-6],[-11,-6],[-52,-9],[-9,-4],[-12,-1],[-12,-4],[-18,6],[-27,-2],[-33,7],[-3,3],[0,5],[-2,2],[-4,3],[-2,1],[-1,3],[-1,2],[-1,3],[-2,0],[0,2],[14,0],[4,2],[-3,2],[-29,0],[-22,3],[-49,-4],[-6,4],[3,2],[4,1],[7,0],[-3,4],[-52,7],[-11,-5],[-29,-3],[-30,-3],[6,-1],[3,-2],[0,-3],[8,-6],[3,-1],[36,3],[44,-6],[43,-6],[12,0],[11,4],[3,-2],[-3,-3],[-1,0],[1,-2],[2,-3],[1,-2],[-1,-2],[3,-2],[5,-2],[0,-2],[3,-2],[7,-3],[4,-3],[1,-4],[0,-4],[-1,-4],[-2,-3],[5,-3],[1,-1],[1,-2],[0,-1],[0,-2],[-1,-1],[-2,-1],[-3,-3],[-1,-1],[-2,0],[-13,-6],[-16,-2],[-15,1],[-16,-2],[-7,3],[-12,11],[-7,2],[-4,-5],[2,-3],[1,-3],[3,-7],[-1,-4],[-4,-3],[-4,-3],[-29,-2],[-12,3],[-35,-1],[-10,7],[-14,3],[-4,3],[2,1],[-3,2],[-5,0],[-4,-2],[-3,-3],[1,-2],[4,-2],[1,-1],[0,-2],[0,-1],[2,-1],[2,-2],[6,-3],[1,-2],[2,-2],[0,-1],[-2,-2],[-6,-6],[-7,-2],[-55,5],[-13,6],[-2,3],[-3,1],[-4,-4],[1,-2],[-7,1],[-3,1],[-2,3],[1,2],[-1,1],[-1,1],[-1,1],[-2,0],[-9,-5],[-5,-1],[-5,2],[-21,14],[-48,3],[-48,4],[-3,1],[-5,4],[-3,1],[-1,0],[-2,-2],[-2,-2],[-1,-3],[0,-2],[3,-2],[4,-2],[21,0],[10,-3],[11,3],[20,-4],[15,1],[29,-3],[6,-5],[-6,-3],[-2,-1],[2,-2],[2,-2],[1,-3],[2,-2],[16,-6],[11,-1],[3,-2],[-4,-2],[-27,-3],[-9,3],[-34,1],[-34,1],[-3,-2],[-1,0],[-2,1],[-2,2],[-5,1],[-15,10],[-2,2],[-1,2],[0,3],[0,5],[-1,3],[-3,0],[-5,-3],[-6,-2],[-12,4],[-6,-3],[3,-4],[7,-9],[4,-2],[2,-2],[2,-5],[3,-3],[4,-2],[2,-2],[-2,0],[-5,-2],[-32,4],[-31,3],[-11,4],[-5,0],[-7,0],[-8,-3],[0,-2],[25,-7],[23,0],[30,-9],[5,-4],[-5,-1],[-10,0],[-5,-3],[47,4],[34,3],[31,-4],[23,0],[12,4],[5,-1],[24,-9],[34,0],[34,-1],[33,0],[29,3],[37,-4],[-3,-4],[-11,-4],[-22,0],[-4,-4],[5,-4],[13,1],[6,-4],[-2,-1],[-1,0],[-3,0],[1,-3],[3,-1],[7,-2],[1,-2],[2,-2],[2,-2],[5,-2],[3,-3],[1,-3],[-1,-3],[-2,-2],[-3,-1],[-34,2],[-33,-3],[-47,10],[-29,2],[0,1],[-1,2],[-1,1],[-1,1],[-12,0],[4,3],[-1,1],[-2,2],[0,2],[1,1],[1,1],[1,0],[0,2],[0,-1],[-4,5],[-22,8],[-29,0],[-29,0],[-43,-3],[-26,2],[-55,0],[-55,-1],[-5,-2],[-4,-4],[5,-3],[6,1],[12,4],[40,-2],[40,-3],[39,-2],[30,0],[28,-4],[41,-2],[-1,-1],[0,-1],[0,-1],[5,-2],[1,0],[-3,-5],[-6,-1],[-25,-1],[-36,4],[-29,-5],[-22,0],[-43,-11],[-9,2],[-9,0],[-3,2],[-5,6],[-3,1],[-34,3],[-27,0],[-32,-1],[-32,-2],[4,-2],[49,-1],[29,0],[29,-1],[17,-4],[4,-3],[3,-4],[-4,-1],[-25,-11],[-2,-4],[-18,-16],[-9,-11],[-7,-6],[-2,-2],[3,-6],[5,0],[5,3],[3,3],[3,6],[7,9],[28,21],[11,6],[56,8],[56,8],[54,-7],[54,-6],[6,-4],[-2,-1],[-6,-8],[-4,-1],[-50,-5],[-19,-8],[-2,-3],[0,-2],[3,-1],[4,1],[16,7],[46,0],[12,4],[21,-2],[28,-1],[26,-5],[18,-2],[5,-4],[-1,-1],[0,-3],[8,3],[4,0],[2,-4],[-2,-3],[-11,-3],[-4,-1],[4,-1],[10,-1],[3,-3],[-6,-4],[-2,-1],[2,-3],[7,-5],[3,0],[-3,-3],[-7,0],[-4,-1],[-2,-2],[0,-3],[2,-2],[2,-3],[-1,-1],[-1,-1],[-2,-1],[-1,0],[4,-3],[4,-1],[3,0],[4,1],[3,-1],[3,-2],[6,-7],[-19,0],[-23,-13],[-3,0],[-17,1],[-24,-6],[-42,3],[-41,2],[-42,3],[-9,-4],[-6,0],[-3,-1],[-3,-2],[0,-2],[1,-1],[45,3],[15,-3],[2,-1],[2,-3],[1,-3],[1,-2],[2,-1],[55,-11],[55,-11],[2,-3],[2,-3],[2,-2],[3,-1],[-5,-5],[-6,-3],[-11,0],[-11,2],[-6,-1],[-5,-10],[-4,-5],[-8,-6],[-6,-4],[-6,-1],[-32,6],[-32,5],[-19,-3],[-28,5],[-3,0],[-6,-3],[-23,-4],[-3,-4],[5,-4],[6,-2],[11,0],[1,0],[2,-2],[1,-1],[4,0],[5,-2],[35,-2],[12,-5],[30,0],[29,1],[6,-2],[4,-4],[-1,-4],[-11,-10],[1,-2],[-18,-11],[-7,-8],[-4,-2],[-4,1],[0,4],[-2,1],[-12,3],[-4,2],[-5,3],[-7,1],[-26,-8],[-1,-1],[-3,-5],[-4,-4],[-24,-3],[-15,-7],[-21,2],[-10,8],[-41,0],[-41,-1],[-38,14],[-6,-3],[0,-2],[5,0],[3,-1],[2,-3],[-22,-7],[-4,-2],[-2,-4],[2,-2],[6,-3],[1,-1],[2,-2],[1,-2],[3,0],[-1,0],[3,-1],[14,0],[2,-1],[0,-4],[18,-7],[3,-5],[-1,0],[0,-2],[2,-6],[-2,-1],[-1,-1],[1,0],[-5,-2],[-5,-3],[2,0],[2,-2],[3,-2],[0,-3],[-2,-3],[-6,0],[-3,-3],[7,-2],[12,1],[6,-4],[1,-1],[0,-3],[1,-2],[2,-1],[8,0],[2,-2],[3,-3],[-3,-2],[-19,-7],[-5,-1],[-3,-1],[-1,-3],[1,-2],[4,-7],[-9,-10],[-20,-4],[-9,-5],[5,-7],[-2,-4],[-4,-2],[-24,-3],[-22,-12],[-8,-2],[-9,2],[10,10],[4,4],[-3,2],[-3,0],[-5,0],[-3,-1],[-6,-5],[-4,-2],[-5,-6],[-3,-2],[-6,-1],[-9,-5],[-29,3],[-29,4],[-22,-2],[-12,-7],[-2,-1],[-26,2],[-12,3],[-10,-1],[-8,2],[-10,1],[-10,4],[-16,2],[-22,11],[-4,1],[-1,1],[-1,3],[-1,1],[-20,4],[-9,4],[-2,1],[-2,4],[-4,4],[-4,3],[-4,2],[-7,2],[-5,2],[-3,0],[-2,1],[0,2],[1,2],[1,2],[-2,3],[-10,4],[0,1],[4,0],[2,1],[1,1],[-3,4],[-21,6],[-3,3],[2,5],[-3,4],[-3,3],[-4,1],[-2,-1],[0,-2],[0,-1],[0,-1],[-1,-1],[0,-3],[2,-6],[2,-3],[2,-2],[2,-1],[8,-1],[2,-3],[2,-2],[3,-5],[6,-4],[2,-4],[0,-5],[-4,-5],[-3,-3],[-1,-3],[0,-2],[1,-4],[2,-1],[5,-2],[11,-7],[11,-4],[31,0],[3,-3],[1,-7],[-3,-3],[-37,-4],[-37,-4],[-34,7],[-5,-4],[20,-8],[9,-2],[2,-1],[2,-3],[1,-4],[1,-2],[-1,-2],[-1,-1],[-1,0],[2,-5],[4,-3],[4,-2],[8,1],[15,8],[11,2],[5,3],[42,16],[42,-7],[41,-7],[5,-3],[1,0],[20,-2],[3,-1],[-3,-4],[-11,-5],[-4,-11],[-11,-5],[-4,-5],[4,-2],[6,2],[33,26],[35,13],[32,-1],[11,-5],[10,0],[5,-3],[12,-7],[5,-2],[3,-3],[-1,-1],[-1,-2],[-1,-1],[-1,0],[5,-5],[2,-3],[-3,-4],[-7,-3],[-29,-24],[-4,-5],[-1,-1],[0,-3],[0,-2],[2,-2],[29,2],[2,-1],[2,-1],[7,4],[5,0],[8,-1],[4,-3],[0,-1],[-1,-2],[-1,-1],[-2,-1],[0,-2],[0,-2],[0,-1],[-4,-4],[-2,-2],[-1,-4],[28,-3],[3,3],[-2,5],[-1,5],[11,0],[3,2],[3,2],[1,1],[-1,2],[0,5],[4,5],[12,2],[5,3],[-1,0],[-2,2],[-1,1],[2,3],[2,1],[2,1],[2,0],[2,1],[23,4],[4,-1],[20,-14],[3,-4],[0,-3],[-1,-2],[0,-2],[0,-2],[5,-3],[2,-2],[-7,-5],[3,-2],[8,-3],[10,-7],[-5,-3],[-2,-2],[-1,-3],[-1,-3],[0,-1],[1,-2],[1,-4],[1,-3],[0,-1],[-2,-1],[-1,0],[-1,-2],[-1,-1],[0,-2],[0,-4],[1,-2],[3,-4],[-6,-5],[-7,-2],[-14,1],[-2,-1],[-6,-5],[-2,-2],[-2,-3],[-6,-4],[-2,-3],[3,-3],[2,-2],[1,-4],[-25,-7],[-3,-3],[2,-4],[-15,-2],[-7,3],[-7,7],[2,2],[1,1],[1,2],[-1,1],[-1,2],[0,3],[-1,2],[5,2],[0,1],[-13,0],[-7,2],[-3,0],[-3,-2],[1,0],[-2,-4],[-5,-5],[-2,-4],[-5,-12],[-2,-3],[-4,-4],[-5,-2],[-16,-3],[-2,-1],[-1,-1],[-2,-2],[0,-2],[2,0],[0,-1],[-3,-2],[-4,-4],[-3,-4],[2,-5],[-5,-5],[-6,-1],[-17,5],[-6,0],[-31,-11],[-31,-11],[-10,5],[-6,-1],[-16,-6],[-5,-4],[-2,-1],[-16,1],[-3,-1],[-1,-2],[-3,-5],[-2,-1],[-14,0],[-7,2],[-4,0],[-6,-7],[-15,-4],[-1,2],[0,4],[2,3],[2,3],[10,3],[0,2],[-4,1],[-8,1],[-4,3],[5,3],[1,1],[0,2],[0,2],[1,2],[1,1],[8,3],[1,1],[6,13],[2,2],[3,1],[3,2],[3,4],[2,3],[1,8],[-6,4],[-8,3],[-5,4],[-3,4],[-4,1],[-3,1],[-8,3],[-14,0],[2,2],[3,6],[-3,7],[-5,-3],[-5,-7],[-3,-4],[-3,-1],[-6,-4],[-16,-3],[-45,6],[-13,5],[-3,2],[4,5],[4,7],[1,1],[1,0],[1,1],[-1,3],[-8,5],[5,1],[13,7],[5,0],[3,1],[2,3],[1,1],[1,0],[1,1],[1,1],[0,2],[-1,1],[-1,0],[-11,-5],[-32,-10],[-11,1],[-6,3],[-13,11],[-2,2],[-4,8],[-1,2],[-6,4],[-1,2],[-1,2],[-2,2],[-6,3],[-5,4],[-5,2],[-4,-1],[3,-2],[5,-3],[4,-4],[3,-5],[-2,-1],[3,-2],[6,-2],[2,-3],[0,-8],[5,-7],[15,-10],[10,-2],[24,-11],[-15,-6],[-2,-5],[2,-5],[9,-11],[-1,-4],[-4,-3],[-12,-6],[-29,-1],[-30,-2],[-2,2],[-3,4],[-17,5],[-12,8],[1,2],[1,3],[3,3],[-2,2],[-1,2],[0,2],[1,2],[-3,10],[-1,3],[-2,2],[-7,3],[-1,0],[0,1],[0,2],[-2,3],[-17,9],[-2,2],[0,1],[-2,6],[0,3],[1,2],[2,2],[3,4],[-3,2],[-3,0],[-2,-2],[-3,-3],[-8,-5],[-1,-3],[1,-4],[2,-3],[11,-6],[1,-1],[1,-4],[1,-2],[3,-2],[7,-4],[3,-3],[1,-3],[1,-2],[0,-7],[1,-5],[2,-1],[3,-1],[1,-3],[-1,-2],[-4,-5],[-2,-2],[1,-1],[0,-1],[1,0],[2,-1],[3,-4],[-4,-5],[-7,-1],[-47,6],[-2,1],[-1,1],[0,1],[-1,2],[0,1],[0,1],[-5,4],[-11,2],[-3,5],[-11,6],[-6,6],[0,5],[-3,11],[-3,3],[-15,7],[-6,0],[-5,-3],[3,-2],[4,-3],[7,-2],[1,-1],[1,-2],[-1,-4],[-5,-11],[-1,-2],[2,-3],[5,-6],[2,-4],[-2,-1],[0,-2],[0,-2],[-1,-1],[-2,-2],[-2,-1],[-35,-3],[-10,1],[-9,4],[-9,6],[-1,6],[-5,7],[-6,7],[-10,6],[-3,0],[-2,-3],[-1,-3],[-1,-7],[-1,-3],[3,-2],[1,-3],[1,-4],[-2,-3],[2,-3],[1,-2],[2,0],[1,-2],[2,-4],[2,-1],[31,-10],[31,-10],[1,-3],[0,-3],[-3,-1],[-45,-3],[-44,-3],[-3,1],[-1,1],[0,2],[0,1],[0,1],[-2,1],[-20,-2],[-4,2],[-6,4],[-4,1],[-3,0],[-16,5],[-16,0],[-15,5],[-9,-1],[-16,1],[-7,2],[-8,1],[-3,2],[1,0],[-4,2],[-1,1],[-1,2],[1,1],[0,2],[0,2],[-1,5],[0,3],[2,3],[19,6],[3,4],[-3,6],[-19,8],[-13,10],[-7,4],[-6,-1],[37,-25],[0,-2],[-2,-1],[-30,-10],[-3,0],[-3,-1],[-2,-2],[3,-2],[0,-2],[-6,-10],[-3,-11],[-1,-2],[0,-1],[0,-1],[-4,2],[-4,0],[-6,3],[-28,3],[-4,2],[-3,4],[1,2],[3,1],[0,4],[-4,3],[-28,2],[-4,3],[4,4],[1,3],[0,3],[-3,3],[-7,3],[-2,3],[2,1],[1,3],[0,3],[1,3],[-3,3],[-4,3],[-4,2],[-3,-3],[0,-1],[1,-2],[1,-1],[0,-1],[-1,-5],[-1,-3],[-1,-2],[0,-2],[2,-2],[8,-4],[1,-2],[0,-2],[-5,-7],[-1,-4],[5,-4],[8,-3],[0,-1],[0,-2],[0,-1],[-1,-2],[-6,-3],[2,-1],[3,-2],[5,-4],[-2,-2],[-3,1],[-10,5],[-2,0],[-3,-1],[1,-1],[0,-2],[1,-2],[-7,-3],[-9,1],[-8,6],[-4,10],[1,0],[-1,3],[-2,0],[-5,-2],[1,-2],[0,-1],[-1,-1],[-2,-4],[0,-2],[-2,-1],[-55,7],[-4,3],[2,3],[2,2],[2,1],[3,2],[-2,0],[-1,1],[-1,3],[-1,1],[-2,1],[-3,1],[7,3],[3,2],[-1,3],[1,4],[-2,3],[-3,2],[-2,2],[-1,2],[0,5],[0,2],[-2,2],[-1,1],[-2,1],[-4,4],[-2,1],[0,3],[1,1],[3,3],[1,3],[-1,5],[-6,6],[-1,3],[0,5],[7,7],[1,2],[0,5],[-1,4],[-2,2],[-4,-2],[-2,-2],[-1,-2],[-4,-7],[-3,-4],[-7,-5],[-2,-5],[4,-3],[10,-15],[-1,0],[0,-3],[-7,-6],[4,-3],[5,0],[5,-2],[4,-8],[-2,-2],[0,-2],[1,-3],[2,-3],[-11,-9],[-4,-7],[3,-7],[-3,-1],[-2,-1],[-3,0],[-3,1],[0,2],[0,3],[1,3],[2,5],[1,4],[2,2],[3,0],[-2,3],[-6,7],[1,3],[1,3],[1,1],[2,2],[-2,4],[-3,3],[-3,2],[-3,-1],[-1,-1],[0,-2],[0,-3],[-1,-2],[-5,-2],[2,-5],[-1,-3],[-5,-5],[4,-4],[1,-2],[0,-4],[-2,-3],[-9,-4],[-1,-1],[0,-2],[0,-1],[-2,-1],[-9,1],[-3,1],[-2,2],[1,3],[-2,3],[-2,-1],[-5,-4],[-3,-2],[-4,1],[-4,1],[-4,2],[-1,4],[-1,1],[-18,8],[-2,2],[1,2],[1,2],[1,3],[0,3],[-2,4],[-2,3],[-2,1],[-2,-1],[-5,-6],[-2,-1],[-7,4],[-6,2],[-3,2],[2,1],[3,1],[2,1],[1,2],[-1,2],[0,2],[0,1],[2,3],[18,7],[2,3],[1,4],[-1,2],[-2,3],[-1,4],[2,0],[3,2],[2,0],[0,1],[-1,0],[-1,1],[1,0],[-3,0],[-3,2],[-2,3],[0,4],[-2,5],[-1,2],[-1,2],[0,3],[-1,2],[-3,5],[8,6],[53,18],[3,4],[-2,1],[0,1],[31,8],[9,7],[12,7],[4,1],[4,-1],[5,1],[3,6],[-5,-1],[-12,1],[5,3],[12,1],[11,4],[18,-1],[0,-2],[-1,0],[-1,0],[-2,-2],[5,-1],[30,7],[10,-3],[11,3],[3,-2],[3,-4],[1,-3],[3,-1],[13,3],[10,-1],[5,2],[-14,4],[0,2],[8,7],[3,2],[32,3],[7,-3],[-1,-1],[2,-1],[4,0],[2,-1],[1,-3],[4,-3],[5,-2],[5,0],[4,2],[-8,6],[-3,0],[0,2],[32,1],[-8,4],[-17,-3],[-8,2],[-4,2],[-23,4],[-3,-2],[2,-2],[-4,1],[-2,1],[-2,2],[2,0],[6,6],[2,1],[13,4],[9,0],[2,2],[-4,2],[-5,1],[-20,-1],[-5,2],[-4,6],[6,1],[12,7],[6,2],[12,-2],[12,3],[-3,2],[-31,1],[-25,-8],[-38,5],[-4,4],[2,6],[-2,1],[-4,2],[-2,3],[-1,4],[3,5],[10,2],[2,5],[0,2],[-2,2],[-2,2],[-1,2],[0,3],[-1,2],[-2,2],[-7,0],[-4,1],[-2,1],[-2,3],[1,0],[1,0],[2,0],[1,1],[-2,2],[-2,0],[-4,0],[-3,0],[-12,8],[-20,6],[-3,2],[-1,4],[1,4],[2,3],[5,6],[-1,2],[1,1],[0,3],[-1,2],[-1,2],[-2,1],[-2,-1],[-2,0],[-1,3],[5,1],[2,1],[-1,5],[6,2],[2,1],[-1,1],[-12,9],[0,2],[3,-2],[7,0],[0,-1],[-1,-1],[3,-3],[4,-2],[4,-1],[49,10],[48,10],[39,-4],[20,-9],[24,-4],[27,-10],[16,-15],[11,-4],[2,-6],[5,-8],[3,-4],[1,-2],[0,-3],[1,-2],[0,-3],[2,-1],[1,-2],[1,-1],[-1,-2],[4,-11],[2,-4],[1,-2],[5,-3],[2,-4],[-2,-4],[-6,-8],[5,0],[17,9],[5,0],[3,1],[0,1],[0,1],[0,1],[1,0],[1,0],[1,-1],[1,-1],[1,-1],[-1,-2],[-2,-1],[-1,-1],[-2,1],[1,-3],[1,-1],[2,-1],[2,0],[1,-1],[4,-5],[4,-2],[46,-4],[7,-2],[20,-13],[33,-5],[-2,5],[-7,4],[-3,3],[1,3],[-1,3],[-3,1],[-2,1],[5,6],[9,0],[15,-4],[34,3],[33,-7],[34,-6],[-1,3],[-1,5],[0,2],[-36,9],[-4,7],[4,3],[20,0],[6,3],[8,5],[9,1],[4,1],[2,2],[2,2],[4,7],[2,1],[5,1],[26,26],[4,1],[2,1],[1,3],[1,3],[0,2],[2,0],[6,3],[12,16],[-1,0],[-2,2],[4,3],[10,4],[3,6],[1,3],[0,4],[1,2],[2,3],[9,5],[-5,5],[2,1],[4,0],[2,2],[-2,1],[-3,4],[-1,2],[1,0],[1,0],[0,1],[-4,5],[-1,2],[5,4],[7,1],[13,0],[0,3],[2,2],[2,2],[2,3],[-22,-1],[-16,-4],[-13,-9],[-2,-2],[-1,-7],[3,-5],[3,-5],[1,-5],[-3,-3],[-20,-8],[-2,-3],[0,-1],[-1,-2],[-2,-2],[-1,-3],[-1,-2],[-8,-4],[-4,-3],[-2,-3],[-7,-15],[-2,-3],[-4,-4],[-10,-6],[-7,-11],[-4,-4],[-29,-18],[-27,-10],[-1,1],[1,2],[1,1],[1,1],[-21,-1],[-28,6],[-48,-3],[-3,1],[-7,4],[-2,3],[1,2],[-1,3],[1,2],[2,2],[21,8],[2,1],[5,6],[9,6],[4,3],[5,8],[0,1],[-1,2],[-1,2],[-2,1],[-1,1],[-2,0],[-1,1],[-1,-1],[2,0],[2,-2],[0,-1],[-1,-5],[-1,-1],[-2,-3],[-1,-2],[-3,-3],[-8,-3],[-11,-7],[-26,-9],[-2,1],[-4,4],[-32,9],[-3,2],[1,3],[2,4],[1,4],[-8,4],[-1,2],[16,4],[1,3],[0,4],[1,3],[3,2],[12,-1],[10,4],[3,2],[-3,4],[-4,0],[-5,-2],[-5,-4],[-6,1],[-13,-5],[-7,0],[-6,3],[-2,1],[-1,3],[2,4],[7,6],[8,3],[8,1],[17,-3],[8,2],[7,3],[4,0],[3,-2],[-1,0],[-5,-7],[-2,-5],[2,0],[24,16],[36,-4],[5,2],[-26,7],[-39,-2],[-39,-2],[-13,-6],[-14,-3],[-2,0],[-2,3],[0,2],[0,2],[1,2],[1,0],[-18,2],[-3,4],[2,4],[34,13],[34,14],[5,-1],[10,-4],[25,-2],[2,1],[10,6],[7,3],[23,-1],[5,3],[-34,-1],[-17,-5],[-13,0],[-27,11],[6,5],[5,8],[0,2],[-1,1],[-2,0],[-1,0],[0,2],[21,-3],[25,9],[26,-3],[21,-8],[21,5],[0,1],[-22,0],[-23,7],[-37,1],[-14,-4],[-9,0],[-2,3],[2,8],[2,2],[6,6],[1,2],[2,5],[5,4],[25,5],[2,2],[-1,3],[-2,1],[-3,0],[-1,-1],[-4,-3],[-3,-2],[-4,-1],[-3,0],[-3,2],[-7,4],[-3,1],[1,1],[1,1],[0,1],[2,3],[1,4],[1,5],[0,4],[2,4],[2,3],[2,2],[-2,5],[8,6],[6,10],[3,2],[2,2],[-3,2],[-4,-2],[-7,-8],[-10,-8],[-3,-4],[0,-2],[-1,-4],[-1,-4],[-2,-2],[-1,0],[-2,-4],[-2,-2],[-3,-1],[-1,-1],[-1,-3],[1,-1],[1,0],[1,-2],[-1,-2],[-1,-4],[0,-3],[1,-3],[-2,-3],[-6,-4],[-2,-1],[0,-3],[1,-1],[0,-1],[1,0],[0,-2],[-3,-1],[-3,-1],[-13,-12],[-4,-7],[-14,-10],[-3,-3],[-1,-2],[-11,-8],[-4,0],[0,1],[1,0],[1,0],[2,2],[-41,-7],[-42,-6],[-2,0],[-3,2],[-2,3],[0,4],[2,6],[2,2],[2,1],[0,2],[-1,0],[-1,2],[0,1],[3,2],[12,1],[3,1],[8,9],[9,6],[2,3],[1,4],[1,2],[2,2],[1,3],[-1,0],[-2,2],[3,4],[3,2],[2,3],[-2,7],[1,0],[2,2],[-2,1],[-2,1],[-2,-1],[-1,-1],[1,0],[-1,-2],[-1,-1],[0,-2],[1,-2],[-2,-2],[-5,-3],[-3,0],[-1,-1],[-1,-1],[-1,-1],[-5,-2],[-1,-1],[-1,-3],[0,-5],[0,-2],[-2,-3],[-7,-3],[-10,-8],[-6,-2],[-8,3],[-9,0],[-5,2],[-3,0],[-2,-2],[-5,-7],[-18,-12],[-5,-2],[-40,1],[-41,0],[-3,2],[-2,3],[5,6],[6,4],[39,5],[0,1],[-12,2],[-9,3],[-8,-3],[-8,-1],[-8,0],[-4,4],[0,3],[3,5],[0,3],[0,3],[-2,6],[0,2],[1,7],[1,5],[0,5],[-3,4],[2,6],[3,4],[37,23],[33,0],[0,2],[0,3],[-5,3],[-17,-3],[-5,1],[-2,2],[0,3],[15,13],[2,5],[1,7],[2,4],[38,23],[3,1],[3,-1],[5,-2],[56,9],[29,-2],[30,11],[12,2],[16,8],[3,1],[6,-2],[15,2],[37,-12],[54,-4],[55,-4],[22,-11],[25,-5],[7,-6],[12,-6],[42,-7],[3,-3],[6,-3],[2,-1],[-2,-2],[2,-2],[5,-1],[2,-2],[0,-1],[1,-2],[1,-1],[3,-3],[9,-3],[7,-2],[7,-3],[4,-1],[9,4],[3,1],[0,1],[0,1],[-1,1],[-30,16],[-9,9],[11,3],[3,2],[-1,2],[26,-4],[3,2],[-5,6],[-8,2],[-49,-6],[-4,1],[-10,7],[-4,1],[-8,0],[-4,2],[-5,4],[-2,1],[-13,1],[-2,0],[-1,2],[-1,4],[-2,2],[6,3],[26,1],[38,-5],[28,-1],[5,2],[3,2],[35,5],[28,0],[4,-3],[-8,-2],[6,-1],[5,2],[0,5],[2,0],[1,1],[3,4],[-3,2],[-9,4],[2,3],[4,5],[1,7],[2,2],[2,2],[1,0],[2,2],[18,11],[-2,2],[-4,1],[-2,2],[-1,-2],[-37,-11],[-6,-3],[-4,-6],[2,0],[-2,-2],[-7,-4],[-8,-8],[-2,-1],[-46,-8],[-3,1],[-6,3],[-40,10],[-42,-3],[-38,1],[-33,7],[-36,-2],[-26,15],[-27,5],[-4,4],[0,5],[2,3],[8,2],[5,4],[17,8],[6,1],[12,-4],[20,-1],[13,-6],[-1,-1],[-1,-4],[2,0],[2,-1],[2,-1],[2,-1],[0,-1],[0,-1],[0,-1],[0,-1],[2,-1],[30,-2],[30,-3],[6,2],[4,4],[-5,3],[-10,-5],[-5,0],[-17,6],[-3,1],[2,2],[1,0],[-4,3],[-18,7],[-9,1],[-4,2],[1,1],[0,1],[1,1],[-1,0],[-2,0],[-1,1],[-1,1],[2,3],[3,2],[5,3],[-3,7],[-4,1],[-10,-7],[-6,-3],[-22,2],[2,3],[-1,4],[-1,3],[-2,4],[2,0],[-1,3],[-3,2],[-2,1],[-3,4],[-1,1],[-1,1],[0,4],[-1,1],[0,1],[-1,2],[0,7],[0,2],[-1,2],[-2,0],[-1,0],[0,3],[0,5],[-3,3],[-9,4],[-11,2],[-22,9],[-2,3],[2,4],[-2,1],[-4,1],[-4,4],[0,1],[0,1],[0,1],[0,1],[-1,3],[-3,2],[-5,3],[1,4],[2,3],[-17,8],[-17,7],[-34,7],[-27,-3],[-33,9],[-33,5],[-3,1],[-2,3],[-1,1],[0,2],[1,7],[1,2],[1,1],[11,0],[-2,1],[-5,2],[-2,3],[-1,2],[0,6],[-1,2],[1,4],[1,4],[1,3],[2,3],[4,3],[9,3],[39,-5],[10,-3],[39,-1],[13,-8],[10,-1],[5,2],[4,4],[-19,2],[-11,8],[-40,3],[-9,3],[-55,4],[-11,6],[-5,4],[-1,0],[0,1],[0,2],[0,1],[-2,3],[-3,5],[-2,3],[0,6],[1,4],[6,9],[4,4],[0,1],[1,2],[0,3],[1,1],[1,1],[0,2],[1,2],[0,3],[1,5],[-1,1],[0,1],[0,1],[2,2],[3,0],[4,2],[31,3],[31,2],[18,-6],[47,-11],[39,1],[39,1],[33,-1],[33,-1],[29,-4],[50,-26],[26,-17],[49,-20],[29,-15],[29,-14],[7,-7],[4,-4],[6,-12],[4,-4],[2,-3],[-1,-1],[0,-1],[-1,-3],[8,-5],[9,-3],[18,0],[-2,-7],[1,-4],[7,-5],[-2,-2],[-2,-1],[-3,-1],[-3,1],[-2,2],[-3,1],[-2,0],[-2,-4],[15,-4],[8,2],[15,10],[8,0],[45,-9],[45,-9],[-2,2],[-1,2],[-2,0],[-1,1],[2,4],[30,5],[31,6],[15,-1],[3,0],[19,14],[-36,-2],[-37,-2],[-5,-3],[-49,-5],[-50,16],[-21,-2],[-2,1],[-1,1],[2,6],[1,2],[-7,3],[-7,0],[5,5],[2,5],[0,4],[-4,9],[-3,4],[-3,2],[-1,3],[0,2],[2,1],[9,3],[14,0],[4,4],[-6,3],[-30,5],[-31,4],[-8,5],[-11,2],[-40,23],[-48,31],[-6,9],[-23,9],[0,1],[5,3],[35,5],[35,5],[31,3],[31,4],[43,1],[47,5],[48,5],[47,5],[7,4],[-2,1],[-4,1],[-2,3],[35,4],[31,2],[31,1],[9,4],[44,-1],[13,3],[13,-2],[12,4],[34,-1],[33,-1],[7,1],[2,2],[-1,1],[0,1],[1,0],[0,1],[1,0],[0,1],[0,1],[-43,2],[-43,3],[-56,-2],[-39,0],[-39,1],[0,1],[6,3],[39,8],[20,9],[35,3],[26,8],[24,2],[12,7],[22,0],[12,2],[5,4],[52,2],[25,4],[42,0],[32,2],[32,1],[6,3],[4,5],[-12,6],[-21,0],[-28,-4],[-41,4],[-35,1],[-34,0],[-14,-5],[-41,-3],[-41,-4],[-3,1],[-3,1],[-4,3],[2,5],[4,3],[2,3],[-2,8],[-2,3],[-2,2],[-1,2],[2,4],[2,2],[47,23],[6,4],[3,6],[-9,1],[3,5],[8,1],[13,1],[11,3],[12,6],[8,7],[16,9],[20,5],[5,4],[29,4],[19,9],[15,0],[11,7],[24,7],[2,2],[-6,2],[-7,1],[-52,-12],[-53,-11],[-10,-6],[-36,-7],[-33,-19],[-13,-3],[-10,-5],[-3,-3],[-2,-4],[-3,-6],[4,-3],[1,-1],[2,1],[-5,-3],[-23,-2],[-32,12],[-32,11],[5,-6],[35,-18],[7,-1],[3,-2],[-5,-4],[-12,0],[-5,-2],[-7,-9],[-1,-2],[-2,-4],[4,-2],[4,-2],[4,0],[7,2],[1,-2],[0,-3],[-3,-3],[-9,-8],[-4,0],[-7,0],[-4,-1],[-3,-4],[-1,-5],[-4,-5],[-15,-9],[-36,-7],[-14,-9],[-16,-3],[-18,-2],[-31,-9],[-28,-10],[-38,-8],[-38,-2],[-30,-6],[-30,-3],[-29,-4],[-5,-3],[-1,1],[-38,-1],[-21,-4],[-24,2],[-5,4],[0,6],[2,6],[3,5],[4,3],[4,2],[12,-1],[6,2],[7,6],[38,5],[39,6],[14,6],[10,2],[20,13],[-2,2],[-9,0],[-13,3],[-6,-1],[-12,-6],[-27,-5],[-6,-3],[-6,0],[-3,-1],[-2,-2],[-2,-2],[-2,-2],[-2,-1],[-48,-4],[-34,-6],[-23,0],[-3,1],[-2,2],[2,2],[-1,2],[-3,3],[4,4],[20,3],[23,11],[3,4],[-17,3],[-6,-2],[-5,-5],[-2,-1],[-16,-7],[-6,0],[-2,-1],[-7,-6],[-5,-1],[-18,3],[3,-2],[7,-3],[-4,-1],[-2,-3],[0,-4],[4,-4],[2,-2],[3,-4],[2,-4],[1,-4],[-2,-4],[-4,-10],[-3,-4],[-5,-4],[-6,-1],[-20,-1],[-36,-6],[-33,5],[-27,0],[-22,-6],[-49,5],[-11,4],[-17,-3],[-7,3],[8,8],[18,6],[8,6],[-25,-6],[-30,-15],[-8,0],[-36,8],[-35,8],[7,11],[15,16],[43,28],[42,27],[20,9],[13,1],[7,2],[9,0],[15,4],[3,2],[1,1],[23,1],[-2,2],[2,2],[2,0],[3,0],[5,-3],[3,-1],[37,4],[38,3],[9,4],[52,5],[52,5],[53,5],[6,3],[27,3],[11,5],[-3,1],[-51,-5],[-51,-6],[-52,-5],[-51,-6],[-6,2],[-52,-4],[-52,-4],[-12,-4],[-13,2],[-11,-2],[-33,-4],[-37,-10],[-29,-16],[-8,-8],[-3,-1],[-11,-4],[-7,-7],[1,-2],[-51,-25],[-2,-3],[-2,-7],[-7,-7],[-13,-7],[-21,0],[-28,3],[-21,7],[-23,1],[-31,10],[-28,12],[-35,7],[-29,13],[-23,6],[-7,7],[-1,2],[0,1],[3,2],[10,2],[8,4],[28,1],[19,6],[51,4],[32,0],[33,0],[24,-4],[55,2],[12,4],[37,0],[28,5],[20,10],[30,4],[18,-2],[3,2],[-6,2],[-2,2],[-2,2],[3,3],[4,3],[12,2],[8,4],[8,3],[8,4],[40,11],[39,6],[8,5],[4,1],[-2,3],[-3,1],[-3,-1],[-2,-1],[-3,-1],[-10,5],[-8,-1],[-27,-1],[-36,-8],[-35,-7],[-37,-12],[-11,-9],[-26,-6],[1,0],[-47,-4],[-52,-10],[-53,3],[-55,-4],[-56,-4],[-56,-4],[-55,-4],[-14,3],[-6,3],[-9,2],[-12,5],[-2,3],[-5,7],[-8,8],[-2,3],[-2,4],[2,1],[4,4],[36,14],[2,-1],[5,-2],[9,2],[5,0],[22,-9],[5,0],[35,5],[36,4],[-7,5],[-45,-4],[-21,8],[-29,5],[-14,6],[4,3],[35,5],[18,10],[24,2],[28,14],[12,-4],[4,0],[4,2],[-4,0],[-6,4],[-3,1],[2,3],[3,1],[44,6],[21,-1],[42,4],[0,-2],[-1,0],[-1,0],[-1,0],[-1,-2],[4,-2],[5,0],[12,3],[8,-2],[22,0],[27,-7],[4,2],[-1,0],[-1,1],[-1,1],[-1,1],[-3,2],[-42,7],[-41,6],[-10,-3],[-13,1],[2,2],[-6,4],[-7,1],[-6,-1],[-12,-6],[-12,-2],[-41,-3],[-38,-9],[-23,-4],[-31,-10],[-53,-12],[-11,-5],[-8,-2],[-8,1],[-6,3],[7,4],[-42,11],[3,4],[42,13],[42,13],[13,-1],[2,1],[1,1],[2,4],[2,1],[31,2],[6,7],[-29,0],[-28,-2],[1,3],[2,1],[2,0],[2,1],[-3,2],[-6,0],[-3,2],[-12,8],[-7,2],[-4,0],[-2,-3],[0,-3],[2,-3],[5,-5],[-3,0],[-9,4],[-10,-1],[-22,5],[-4,-1],[-7,-8],[-6,-2],[-11,0],[-5,-2],[-2,-1],[-2,-3],[0,-2],[5,-9],[-3,0],[-3,0],[-3,-2],[-4,-3],[-3,0],[-1,0],[2,2],[-5,3],[-37,-8],[6,4],[1,1],[-1,1],[-3,8],[-5,2],[-30,2],[-7,4],[-8,0],[-7,3],[1,3],[3,1],[5,-1],[-2,2],[-3,1],[-5,1],[5,2],[10,-2],[6,1],[-5,3],[2,2],[4,1],[2,1],[-1,1],[3,4],[1,1],[-2,2],[3,2],[10,0],[5,4],[10,5],[10,1],[22,-4],[10,2],[7,-1],[2,1],[-10,5],[-4,3],[8,7],[8,3],[32,-1],[4,2],[1,1],[0,1],[0,1],[1,1],[3,0],[5,3],[42,5],[34,-3],[-2,3],[-3,1],[10,0],[15,5],[4,0],[5,-2],[-2,-2],[-3,0],[-2,-2],[4,-2],[7,-6],[13,-5],[4,1],[2,-1],[2,-1],[-1,-1],[-1,-3],[3,1],[9,-2],[4,-2],[-2,2],[2,3],[8,3],[10,8],[3,2],[-3,4],[-23,1],[-3,1],[-2,2],[1,3],[3,2],[5,2],[11,0],[19,-8],[10,3],[-1,3],[1,1],[0,1],[-7,10],[5,4],[37,13],[50,5],[8,3],[5,0],[1,0],[2,2],[1,1],[2,1],[8,-1],[24,-7],[23,4],[24,-4],[20,-11],[3,-4],[-17,-9],[4,-3],[31,-9],[32,-8],[-3,5],[-7,4],[-31,6],[-4,5],[32,3],[0,2],[-3,0],[-14,7],[-1,1],[2,3],[2,2],[38,2],[42,-3],[11,0],[6,-4],[15,-3],[31,-19],[31,-19],[3,-1],[4,2],[-4,5],[-27,15],[-10,10],[5,5],[47,0],[8,-3],[20,-19],[13,-6],[13,-2],[-17,11],[-14,14],[4,2],[19,-1],[3,-2],[2,-4],[-1,-7],[6,-5],[6,-4],[14,-2],[-3,2],[-7,5],[-1,2],[-1,6],[-1,3],[-4,6],[-56,8],[-56,9],[-56,9],[-56,9],[-15,13],[-2,4],[4,3],[32,5],[23,-3],[39,2],[39,2],[12,7],[6,1],[12,-1],[6,4],[-12,5],[-4,3],[2,2],[4,2],[2,2],[-11,1],[-2,3],[4,8],[-5,4],[-18,-1],[-9,3],[-13,1],[-2,1],[2,3],[2,1],[3,0],[10,4],[40,2],[40,1],[40,1],[5,-2],[4,-5],[-2,-2],[-29,-2],[-5,-2],[48,-7],[15,-6],[0,-2],[4,-3],[17,5],[30,-3],[30,-3],[28,-11],[4,-5],[2,-3],[0,-1],[0,-1],[0,-3],[2,-3],[2,-2],[34,-14],[2,-2],[5,-7],[2,-2],[-3,-6],[-5,-4],[-12,-2],[0,-1],[39,-1],[39,-1],[38,-1],[18,-6],[4,0],[4,2],[-2,1],[-6,6],[-19,7],[-53,-1],[-5,2],[-24,20],[-19,10],[-1,2],[0,2],[0,4],[-1,5],[1,2],[2,1],[3,1],[45,-13],[20,0],[30,-13],[31,-9],[31,-9],[55,-12],[55,-13],[0,-3],[5,-1],[6,0],[8,3],[48,-10],[46,-26],[5,-7],[5,-2],[33,-1],[-51,28],[-3,5],[1,1],[-6,3],[-13,1],[-6,3],[2,2],[6,3],[2,1],[-3,3],[-41,6],[-40,7],[-40,6],[-4,3],[-1,3],[0,1],[1,1],[9,4],[2,1],[-48,1],[-53,18],[-54,17],[-53,18],[-2,2],[-1,3],[0,2],[6,7],[21,17],[56,-2],[57,-1],[2,1],[-3,2],[-8,-2],[-38,5],[-1,1],[-42,14],[-12,8],[3,7],[5,3],[29,3],[37,-5],[37,-5],[30,-12],[30,-11],[35,2],[-3,3],[-28,14],[-28,14],[-4,4],[1,1],[-4,6],[-12,2],[-11,7],[-13,2],[-11,7],[-1,2],[2,3],[3,2],[8,2],[43,-4],[44,-3],[44,-4],[4,-4],[-1,0],[0,-2],[-1,-1],[13,-5],[4,-3],[-2,-3],[-1,-1],[-1,-1],[3,-5],[6,-3],[20,-3],[5,-2],[4,-1],[3,0],[2,2],[-16,6],[-5,4],[4,4],[5,2],[44,-6],[44,-6],[10,3],[11,-3],[45,2],[4,3],[-52,6],[-52,6],[-53,6],[-4,4],[1,4],[-9,2],[9,4],[28,1],[0,1],[-38,4],[-39,4],[-5,4],[-3,5],[4,1],[9,1],[6,2],[4,1],[2,0],[5,4],[47,5],[46,4],[2,-1],[4,-3],[2,-1],[9,0],[-1,-1],[-1,-1],[-1,-1],[0,-2],[4,0],[6,-3],[5,-4],[1,-6],[7,-1],[3,0],[22,16],[10,1],[11,-1],[9,-4],[2,-1],[0,-3],[0,-3],[-6,-7],[5,-5],[5,2],[6,4],[5,0],[1,-4],[4,-1],[8,0],[10,-5],[7,0],[0,2],[-3,0],[-5,1],[-4,2],[-3,4],[5,3],[16,-5],[4,0],[0,2],[-1,2],[-2,2],[-7,2],[3,3],[26,5],[41,-7],[40,-7],[21,-15],[3,-4],[-3,-2],[-1,-1],[4,-2],[16,-2],[6,-4],[20,-16],[14,-3],[4,-2],[2,-2],[2,-1],[4,1],[5,-3],[13,-1],[23,-8],[8,0],[-3,-3],[-3,-2],[-6,-2],[-9,-7],[-8,-2],[-5,-4],[-4,-5],[-4,-6],[1,0],[4,-4],[5,1],[29,16],[28,17],[24,2],[7,4],[-5,3],[-38,7],[-37,7],[1,1],[0,2],[-6,0],[-7,2],[-7,4],[-4,7],[9,4],[19,-1],[10,5],[-46,0],[-18,8],[-15,14],[-40,13],[-41,13],[3,4],[3,2],[22,6],[-1,-2],[-1,-1],[-1,-1],[-1,1],[5,-4],[50,3],[51,3],[11,5],[40,-2],[40,-1],[5,-2],[10,-1],[1,0],[0,-1],[12,-1],[4,2],[-1,0],[11,3],[49,-5],[48,-5],[17,-9],[4,0],[0,-3],[0,-2],[2,-1],[0,-1],[3,0],[2,-1],[6,-4],[1,-2],[3,-5],[4,-5],[47,-13],[47,-13],[46,-13],[-1,1],[-1,1],[0,1],[12,2],[0,2],[-34,7],[-35,8],[-24,12],[-5,1],[2,3],[-7,8],[-4,2],[-14,3],[-9,3],[4,5],[5,3],[16,3],[21,12],[36,6],[5,3],[-14,0],[4,4],[21,2],[10,3],[12,2],[3,-1],[5,-5],[2,0],[26,7],[37,-1],[36,0],[5,-3],[-5,-2],[-15,-10],[-4,-5],[4,-5],[22,-9],[0,-1],[-1,-1],[0,-1],[-1,-1],[43,-7],[42,-8],[-4,5],[-50,20],[-16,3],[-5,2],[2,3],[24,9],[18,3],[2,2],[-1,0],[-1,1],[-1,2],[-1,2],[13,0],[8,-3],[4,-1],[4,0],[4,2],[-2,1],[-3,1],[-1,0],[8,3],[43,0],[42,1],[43,0]],[[559,6624],[5,-4],[2,-5],[-4,-1],[-6,1],[-3,-2],[-6,1],[-6,5],[0,4],[5,-1],[0,4],[-5,5],[-3,4],[4,2],[8,-5],[9,-8]],[[620,6692],[7,-4],[8,2],[4,-1],[4,-4],[-2,-4],[-1,-1],[2,-1],[6,2],[1,-3],[-1,-5],[-2,-3],[-1,-2],[-5,6],[-3,2],[-22,11],[2,5],[3,0]],[[600,6716],[13,-10],[-1,-4],[-5,0],[-8,6],[-6,2],[-9,0],[0,3],[6,2],[10,1]],[[809,6740],[5,0],[-2,-3],[-12,-2],[0,-2],[5,-2],[-4,-1],[-3,0],[-8,3],[0,1],[1,1],[0,1],[2,1],[2,2],[9,6],[1,0],[3,-2],[0,-1],[0,-1],[1,-1]],[[621,6732],[-4,-1],[-3,2],[-2,4],[2,3],[5,7],[3,3],[4,1],[8,0],[-1,-3],[-6,-12],[-3,-3],[-3,-1]],[[1812,6872],[-1,-2],[1,0],[-2,-2],[-7,2],[-3,-1],[5,-4],[-2,-1],[-2,0],[-2,0],[-2,1],[0,4],[2,4],[2,1],[12,2],[2,-1],[-1,0],[-1,-1],[-1,-2]],[[3171,6903],[-21,0],[-5,4],[1,0],[1,1],[1,1],[1,1],[13,0],[9,-3],[3,-2],[-1,-1],[-2,-1]],[[3318,6910],[-49,-3],[-23,11],[-1,3],[1,2],[3,1],[22,-4],[16,-7],[8,-2],[23,-1]],[[2762,6952],[2,-4],[-4,2],[-16,0],[-4,1],[-3,3],[-2,4],[4,0],[15,-3],[6,-1],[2,-2]],[[2801,6963],[-4,-3],[-2,0],[-6,0],[-12,-2],[-2,1],[-2,2],[2,3],[4,2],[3,-1],[3,-1],[4,-1],[6,6],[4,-1],[12,-1],[4,-2],[-12,0],[-2,-2]],[[2767,6971],[4,-3],[-3,-2],[-23,0],[-4,2],[3,1],[1,1],[2,-1],[5,3],[8,0],[7,-1]],[[2842,6966],[-7,0],[-13,2],[6,2],[2,1],[9,2],[3,2],[2,1],[3,-1],[3,-2],[-1,-3],[-7,-4]],[[2729,6968],[-3,-2],[-11,2],[-24,-2],[-7,1],[-3,6],[8,5],[2,0],[18,-5],[16,-2],[4,-3]],[[1944,4422],[-2,6],[-3,3],[-3,1],[-4,-2],[-3,-2],[-5,-2],[-3,1],[0,5],[3,7],[2,3],[0,4],[-2,7],[-1,2],[0,2],[-1,2],[-1,2],[-1,3],[-2,2],[-2,2],[-2,3],[-2,4],[-2,4],[-1,5],[-2,6],[-2,4],[-2,4],[-1,4],[-2,5],[0,5],[0,5],[0,4],[0,5],[-1,8],[-3,4],[-4,1],[-4,-1],[-3,0],[-3,0],[-3,0],[-2,0],[-1,1],[0,3],[-1,3],[0,2],[0,5],[-2,4],[-1,3],[-2,3],[-5,6],[-7,9],[-4,10],[2,9],[3,3],[4,3],[4,4],[3,6],[-1,3],[-2,5],[-2,4],[-2,4],[-1,11],[-2,10],[-4,6],[-5,-1],[-2,-2],[-2,1],[-2,0],[-2,0],[-2,0],[-2,0],[-1,0],[-2,1],[-3,2],[-3,-2],[-2,-4],[-3,-4],[0,-6],[0,-5],[0,-4],[-2,-2],[-3,0],[-3,0],[-3,0],[-3,1],[-3,0],[-3,-1],[-2,-1],[-3,0],[-2,0],[-3,0],[-2,0],[-3,-1],[-1,-1],[-2,-2],[-2,-3],[-2,-2],[-4,-4],[-5,-3],[-5,-1],[-5,1],[-5,3],[-5,3],[-5,3],[-5,1],[-5,0],[-5,0],[-4,1],[-5,4],[-4,4],[-4,4],[-4,3],[-4,0],[-5,-2],[-2,-4],[-2,-6],[-2,-8],[-2,-2],[-5,1],[-4,2],[-2,3],[0,2],[0,2],[1,2],[-1,1],[-1,2],[-3,0],[-3,0],[-2,0],[-1,0],[-2,-1],[-2,-1],[-1,-3],[1,-2],[1,-2],[1,-1],[-1,-4],[-2,-5],[-5,-2],[-5,0],[-4,0],[-3,0],[-2,0],[-3,1],[-2,2],[-1,2],[-2,2],[-2,1],[-1,1],[-4,-1],[-3,-3],[-3,-4],[-4,-4],[-1,-2],[-2,-2],[-2,-1],[-2,1],[-2,2],[-1,2],[-2,2],[-2,2],[-2,0],[-2,0],[-1,-1],[-2,-2],[0,-1],[0,-1],[0,-1],[-1,-1],[-1,0],[-1,1],[-1,1],[-1,1],[-1,2],[-2,2],[0,2],[-1,4],[1,2],[1,2],[0,2],[-1,2],[-3,3],[-1,3],[1,4],[0,5],[-1,3],[1,2],[1,1],[1,3],[0,1],[-1,3],[-1,2],[0,1],[-2,4],[0,4],[0,3],[-1,5],[1,2],[0,2],[0,2],[-2,1],[-2,1],[-3,-1],[-2,-1],[-2,-2],[-2,-2],[-1,0],[-1,0],[-2,3],[-1,1],[-1,3],[0,2],[1,2],[2,2],[2,0],[1,1],[2,2],[2,4],[0,2],[-2,2],[0,3],[1,2],[0,1],[1,2],[1,3],[-1,2],[0,2],[0,3],[-1,2],[0,5],[-1,4],[-1,5],[-1,4],[-1,2],[0,2],[-1,2],[0,2],[0,2],[0,2],[0,2],[0,3],[-1,2],[-2,1],[-2,1],[-1,2],[-1,3],[0,3],[0,2],[0,3],[-2,6],[-3,2],[-3,1],[-3,2],[-3,4],[-3,3],[-4,1],[-4,-1],[-5,-2],[-5,-1],[-5,1],[-4,0],[-2,1],[-2,0],[-1,1],[-1,2],[-3,2],[-2,1],[-3,1],[-2,1],[-1,1],[-1,2],[-1,2],[0,2],[-2,4],[-3,1],[-3,1],[-2,1],[-4,2],[-4,4],[-3,5],[-3,5],[-2,2],[-2,1],[-3,0],[-2,2],[-1,1],[0,1],[-1,2],[-1,2],[0,3],[0,3],[0,3],[0,3],[-1,2],[-1,3],[-1,2],[-1,2],[-2,4],[-1,4],[-2,4],[-3,2],[-3,0],[-4,-2],[-3,-1],[-3,3],[-1,2],[0,3],[0,3],[-1,3],[-1,3],[-1,2],[-1,3],[-1,2],[-1,1],[-2,2],[-2,2],[-1,2],[-2,1],[-2,1],[-2,1],[-1,2],[0,3],[1,1],[0,2],[0,2],[-2,3],[-2,2],[-3,1],[-2,2],[-2,2],[-1,1],[-2,2],[-1,2],[-1,3],[-1,2],[-2,2],[-1,1],[-5,-1],[-5,1],[-4,-1],[-5,-4],[-1,-2],[-1,-3],[-1,-3],[-1,-3],[-2,-1],[-3,1],[-3,1],[-2,1],[-1,1],[0,3],[0,2],[-1,3],[-1,2],[-2,1],[-2,0],[-2,0],[-2,1],[-2,1],[-1,2],[-2,1],[-3,0],[-2,0],[-2,-1],[-2,0],[-2,-2],[-2,-2],[-3,0],[-2,1],[-3,2],[-2,2],[-2,2],[-3,1],[-1,0],[-3,0],[-1,1],[-1,2],[1,2],[3,2],[3,2],[2,2],[1,4],[0,1],[-1,0],[-2,2],[-1,2],[-1,2],[0,2],[-2,2],[-1,2],[-1,2],[0,2],[0,3],[-1,1],[-1,2],[-1,2],[0,1],[1,5],[4,4],[3,3],[3,2],[-2,2],[-3,1],[-3,2],[-2,2],[0,4],[1,1],[1,1],[1,2],[0,3],[1,1],[1,0],[2,0],[2,1],[3,0],[2,2],[1,2],[-3,7],[-6,4],[-6,2],[-4,1],[-3,3],[-3,4],[-3,4],[-4,2],[-3,-1],[-4,-3],[-4,-2],[-2,4],[1,3],[4,1],[3,2],[0,5],[0,1],[-1,1],[-1,0],[0,1],[-1,1],[0,1],[0,1],[0,1],[0,1],[1,2],[0,1],[-1,1],[-1,1],[-1,0],[-1,1],[-1,1],[0,1],[-1,1],[0,1],[-2,0],[-1,0],[-1,0],[-1,0],[-1,0],[0,1],[-1,1],[0,2],[0,2],[0,2],[-1,0],[0,1],[0,1],[-1,0],[0,1],[0,1],[1,0],[0,1],[1,0],[-1,1],[0,1],[-1,1],[-1,2],[-2,2],[-1,4],[-1,3],[-2,3],[-2,1],[-2,2],[-2,3],[-1,4],[1,4],[2,1],[3,0],[2,1],[-2,2],[0,2],[0,2],[1,3],[0,1],[0,1],[0,2],[0,1],[1,2],[1,1],[1,1],[1,2],[1,3],[0,2],[0,2],[1,2],[2,2],[2,2],[1,2],[1,2],[-1,6],[-5,1],[-6,-1],[-3,-2],[-2,1],[-2,2],[-2,2],[-3,1],[-2,0],[-1,1],[-1,2],[2,4],[-2,2],[-2,5],[-1,5],[-2,4],[-1,3],[-2,2],[-1,1],[-3,0],[-1,-1],[-1,1],[-1,1],[-1,3],[-3,5],[-2,4],[-3,2],[-3,0],[-2,-1],[-1,0],[-2,-1],[-1,1],[-1,1],[0,2],[1,2],[0,3],[1,3],[0,1],[3,2],[2,2],[3,2],[2,1],[3,1],[3,0],[2,1],[3,2],[1,1],[2,2],[2,3],[0,2],[-1,5],[0,3],[0,3],[2,4],[3,2],[3,1],[2,2],[-2,5],[-1,4],[-2,4],[-1,4],[-1,4],[-2,2],[-2,1],[-2,0],[-2,2],[0,1],[0,2],[1,2],[-1,3],[-2,2],[-3,0],[-4,0],[-2,0],[-1,0],[-1,0],[-1,0],[-1,1],[-1,1],[1,1],[0,1],[0,1],[-1,2],[-1,2],[0,1],[1,2],[1,3],[-1,2],[-2,2],[-2,-1],[-3,-2],[-3,-3],[-4,-2],[-3,-1],[-2,1],[-1,1],[0,2],[-2,1],[-2,1],[-1,3],[0,3],[2,3],[2,2],[3,0],[2,0],[2,1],[1,0],[0,1],[0,1],[0,1],[2,1],[1,0],[2,0],[1,-1],[3,1],[3,2],[1,4],[-2,5],[-1,2],[-2,-1],[-1,-1],[-2,-1],[-2,0],[-2,0],[-2,1],[-2,2],[-5,2],[-4,2],[-5,3],[-4,4],[-1,2],[-1,2],[-1,2],[-1,2],[-2,2],[-2,1],[-2,2],[-2,1],[-1,2],[-1,1],[-1,1],[-2,0],[-1,-1],[-1,-1],[-2,0],[-1,1],[-1,2],[-2,1],[-4,2],[-5,2],[-5,4],[-3,5],[-1,4],[1,2],[1,1],[1,3],[-1,3],[-2,1],[-2,0],[-2,0],[-1,-1],[-2,-1],[-3,-1],[0,3],[1,3],[0,3],[-1,2],[-2,1],[-2,3],[-3,3],[-2,3],[1,5],[3,5],[3,3],[1,2],[-5,3],[-2,2],[-1,2],[0,3],[-1,4],[-2,4],[-4,2],[-3,1],[-3,1],[2,4],[3,3],[1,4],[-3,4],[-1,1],[-1,0],[-1,1],[-1,2],[0,1],[0,1],[0,1],[0,1],[-1,2],[-1,1],[-2,1],[-1,3],[-4,5],[-3,1],[-5,0],[-4,1],[-2,1],[-2,1],[-2,1],[-3,1],[-2,2],[-2,1],[-2,1],[-2,-1],[-1,-2],[0,-1],[0,-3],[0,-2],[-1,-3],[-2,-2],[-3,0],[-1,-1],[-1,-1],[-1,-1],[-1,-1],[-1,-1],[-1,0],[-1,0],[-1,0],[-1,0],[-2,-1],[-2,-2],[-2,-2],[-2,0],[-2,1],[-2,2],[-1,2],[-2,1],[-3,0],[-2,-1],[-3,0],[-3,-1],[-1,-1],[-2,-1],[-2,0],[-1,3],[1,1],[1,3],[0,3],[1,2],[4,8],[6,7],[4,8],[-6,6],[-3,1],[-2,1],[-2,0],[-2,-1],[-2,0],[-2,1],[-2,2],[-2,2],[-1,2],[-2,1],[-2,2],[-1,1],[-2,2],[-1,2],[0,3],[-1,2],[-2,2],[-2,2],[-1,2],[-1,3],[-1,3],[-1,2],[-1,1],[-2,1],[-5,0],[-5,2],[-5,3],[-5,5],[-4,4],[-5,3],[-4,1],[-5,0],[-2,0],[-2,1],[-3,1],[-2,0],[-2,0],[-2,-2],[-2,-1],[-2,0],[-2,1],[0,2],[-1,2],[-1,3],[-1,2],[-2,1],[0,2],[1,3],[3,3],[5,2],[4,2],[2,4],[0,6],[-2,4],[-1,5],[0,5],[2,2],[2,1],[3,0],[2,1],[1,2],[0,2],[0,2],[2,2],[1,1],[3,0],[2,0],[2,3],[0,1],[-1,3],[-1,3],[0,2],[-2,3],[-2,1],[-3,0],[-2,-1],[-3,0],[-2,1],[-3,0],[-3,2],[-3,2],[1,4],[1,5],[1,4],[-1,3],[-2,3],[-2,2],[-2,1],[-1,-2],[-2,-1],[-1,-2],[-2,-1],[-2,1],[-2,0],[-2,-1],[0,-3],[-2,0],[-3,2],[-4,3],[-1,3],[1,0],[3,0],[3,1],[1,1],[-1,3],[-2,1],[-2,0],[-1,1],[-2,2],[2,1],[2,0],[2,0],[2,0],[2,1],[1,2],[1,3],[2,3],[2,2],[2,1],[2,0],[1,0],[3,0],[2,1],[0,3],[0,7],[3,7],[4,4],[3,3],[1,1],[2,2],[1,1],[1,1],[2,1],[1,0],[1,1],[1,1],[1,1],[0,1],[1,2],[0,1],[1,0],[1,0],[1,0],[1,1],[0,2],[1,2],[0,1],[0,3],[1,2],[0,2],[2,2],[0,1],[1,2],[1,3],[-1,2],[1,3],[1,2],[2,1],[2,0],[1,2],[2,1],[1,1],[1,2],[1,3],[0,2],[-1,2],[-1,2],[-1,3],[-1,2],[0,2],[-1,2],[-2,3],[-1,1],[-2,2],[-2,2],[0,2],[-1,1],[0,2],[-1,2],[-1,2],[-1,1],[-1,2],[-1,1],[-4,7],[-5,5],[-5,3],[-5,3],[-2,1],[-2,1],[-2,2],[-2,2],[0,3],[1,3],[1,2],[1,2],[4,4],[4,3],[4,2],[4,2],[2,1],[2,1],[3,0],[1,2],[0,2],[0,3],[-1,3],[-1,2],[-4,2],[-6,-3],[-5,-2],[-4,3],[-1,2],[0,2],[-1,2],[-1,2],[-2,2],[-2,0],[-3,0],[-2,0],[-2,-1],[-3,-2],[-2,-3],[-2,-3],[-2,-3],[-4,-1],[-3,-2],[-2,-4],[0,-2],[0,-2],[-1,-2],[0,-1],[-2,-1],[-1,-1],[-2,-1],[-2,0],[-3,0],[-4,-1],[-4,1],[-3,3],[3,4],[3,5],[3,5],[1,5],[-1,6],[-4,1],[-5,-1],[-4,-1],[-5,1],[-6,-1],[-5,-1],[-6,-3],[-2,-1],[-2,-2],[-2,-2],[-2,0],[-1,0],[-2,1],[-1,0],[-1,-1],[0,-1],[-1,-2],[0,-1],[0,-1],[-4,-4],[-4,-1],[-5,2],[-4,1],[-4,0],[-4,0],[-5,2],[-2,5],[1,7],[2,6],[0,3],[-4,1],[-2,0],[-2,0],[-2,2],[-1,2],[0,3],[2,4],[1,3],[2,2],[3,2],[4,2],[3,4],[2,6],[-1,3],[-1,2],[0,2],[-1,3],[1,3],[0,2],[1,2],[0,3],[-2,7],[-6,3],[-6,0],[-5,-1],[-3,-1],[-5,1],[-2,3],[2,6],[1,3],[1,2],[1,3],[1,3],[0,3],[0,3],[0,4],[1,3],[3,4],[4,-2],[5,-2],[2,2],[0,4],[-3,4],[-3,5],[-1,5],[2,4],[5,3],[4,3],[1,2],[-2,2],[-3,2],[-3,0],[-2,1],[-2,2],[0,2],[-1,2],[-1,3],[-1,2],[-2,1],[-2,0],[-2,2],[-1,2],[1,2],[1,1],[1,3],[-2,4],[-4,3],[-3,4],[1,3],[2,2],[2,2],[1,2],[2,4],[0,4],[0,3],[-2,1],[-2,2],[-2,2],[-1,2],[0,2],[-2,4],[0,1],[-1,1],[-1,0],[-2,1],[0,1],[-1,1],[0,2],[0,1],[-4,4],[-4,2],[-5,2],[-4,2],[-2,0],[-2,2],[-1,1],[-1,3],[1,3],[2,1],[2,1],[2,1],[5,-1],[7,0],[6,1],[5,4],[-11,0],[-11,0],[-10,0],[-11,0],[-10,0],[-14,0],[-13,0],[-13,0],[-13,0],[-13,0],[-14,0],[-13,0],[-13,0],[-7,0],[-15,0],[-14,0],[-7,0],[-10,0],[-11,0],[-11,0],[-11,0],[-11,0],[-2,0],[-2,0],[-2,0],[-2,0],[-1,0],[0,2],[-1,1],[0,2],[-1,3],[-1,1],[-1,2],[-1,2],[-1,2],[0,2],[1,2],[0,2],[0,1],[-1,1],[0,1],[0,2],[0,2],[1,0],[0,1],[1,2],[0,3],[0,1],[-2,1],[-1,3],[-1,3],[1,2],[2,2],[1,1],[2,2],[2,1],[2,1],[2,2],[1,2],[0,4],[0,3],[1,4],[1,2],[1,1],[0,2],[0,3],[-1,2],[0,1],[-1,2],[-1,2],[-1,5],[1,1],[1,3],[-1,1],[-2,2],[-2,1],[-1,1],[-2,2],[-1,0],[-1,2],[1,3],[2,4],[0,4],[0,4],[1,4],[0,1],[1,1],[0,1],[0,1],[-1,1],[-1,1],[0,1],[-1,3],[0,4],[0,3],[-1,4],[-3,5],[-3,5],[-3,3],[-4,1],[-2,0],[-2,1],[-2,0],[-2,1],[-1,0],[-2,1],[-1,0],[-1,2],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[-1,1],[0,1],[-1,1],[0,4],[0,5],[0,5],[0,4],[0,22],[0,21],[0,21],[0,21],[0,24],[0,24],[0,24],[0,23],[0,12],[0,12],[0,12],[0,12],[0,6],[0,6],[0,6],[0,14],[0,12]],[[516,6572],[3,-2],[20,-4],[20,0],[21,-9],[3,-3],[8,-4],[1,-2],[2,-3],[2,-4],[4,-2],[7,-3],[14,-2],[3,-4],[1,-1],[2,-4],[2,-2],[2,-1],[1,-1],[1,0],[0,-2],[0,-2],[0,-1],[4,-1],[9,3],[4,-1],[8,-5],[5,0],[-2,2],[-4,1],[-2,3],[1,0],[2,2],[-3,0],[-6,5],[-6,2],[-2,3],[-2,8],[-1,2],[0,1],[-4,1],[-3,2],[-4,1],[-2,3],[-2,4],[-1,4],[19,-1],[-8,3],[-18,2],[-7,8],[2,1],[1,3],[-25,-2],[-5,1],[-15,10],[1,4],[2,1],[6,-2],[0,2],[-6,6],[-2,5],[1,6],[3,2],[5,-2],[7,-7],[9,-5],[10,0],[10,4],[9,6],[-20,-4],[-2,1],[0,2],[-1,3],[0,2],[-3,3],[-20,8],[-2,3],[-1,13],[-2,7],[0,4],[0,4],[4,8],[4,-1],[9,-9],[3,-1],[1,-1],[3,-5],[19,-14],[13,-3],[6,-4],[-2,5],[-4,3],[-14,3],[-3,2],[-24,26],[-2,8],[2,4],[3,3],[5,2],[7,1],[2,-1],[2,-2],[1,-1],[1,-1],[0,-1],[1,-2],[0,-3],[0,-1],[1,-4],[5,-8],[2,-1],[1,2],[-1,5],[-4,9],[-1,4],[0,2],[0,2],[1,1],[2,2],[2,1],[6,-2],[5,0],[4,-1],[7,-8],[4,-3],[18,-6],[2,0],[0,1],[0,1],[0,1],[0,2],[-1,0],[-2,0],[-2,2],[-1,1],[-2,2],[-1,2],[0,2],[1,2],[-1,2],[0,5],[-1,1],[-1,2],[-2,7],[-1,1],[0,2],[0,3],[1,2],[1,2],[6,3],[7,6],[3,1],[2,-5],[1,2],[1,2],[2,1],[2,0],[-1,0],[3,1],[7,-2],[3,-1],[6,4],[4,0],[5,-2],[1,1],[0,4],[2,-1],[0,-2],[1,-1],[0,-1],[4,-1],[12,1],[12,-8],[7,-1],[5,5],[1,7],[-3,5],[-6,0],[-4,-5],[1,-2],[0,-1],[-2,-2],[-2,1],[-3,4],[1,3],[1,0],[-3,1],[0,4],[1,4],[2,1],[1,0],[3,-2],[1,-1],[2,0],[3,2],[1,0],[-2,5],[0,1],[3,3],[3,2],[1,3],[3,8],[-1,5],[-3,6],[-7,10],[2,1],[2,-1],[2,-1],[2,-1],[2,1],[4,2],[4,-1],[1,0],[1,0],[0,-4],[0,-2],[-1,-2],[-1,-1],[-2,-1],[3,-2],[9,2],[3,0],[2,-3],[2,-3],[2,-3],[3,-1],[0,-2],[-6,-3],[-2,-2],[-2,-5],[-1,-3],[2,0],[2,2],[2,1],[1,-1],[1,-4],[1,-1],[1,-1],[3,-3],[1,-1],[3,1],[3,2],[3,1],[4,-1],[-1,-3],[-1,-1],[-3,-3],[6,0],[1,0],[1,-1],[1,-1],[1,-1],[2,-1],[3,0],[3,1],[3,2],[-1,2],[-1,1],[2,2],[5,0],[2,1],[-1,1],[-1,1],[-1,0],[1,3],[1,1],[2,1],[2,0],[-1,-1],[0,-1],[1,-2],[0,-1],[1,-1],[2,-1],[-1,-2],[-3,-1],[-1,-2],[0,-2],[1,-3],[1,-1],[-1,-2],[0,-1],[-1,0],[-3,-6],[-3,-3],[-21,-17],[-7,-9],[-1,-12],[-8,0],[-1,0],[-3,-4],[-1,-1],[-1,-1],[0,-2],[0,-2],[-2,-1],[-1,-1],[-1,-1],[-1,-3],[-1,-2],[0,-2],[0,-1],[0,-1],[0,-3],[-1,-2],[0,-1],[-2,-2],[-1,-4],[1,0],[3,1],[2,0],[2,2],[1,3],[1,7],[3,7],[4,4],[5,3],[13,1],[6,4],[5,1],[3,2],[1,4],[0,2],[0,2],[2,2],[1,1],[5,0],[2,0],[3,3],[1,0],[0,-1],[0,-3],[0,-1],[5,0],[2,1],[3,3],[-2,1],[-2,2],[3,5],[4,3],[19,4],[23,-1],[2,0],[1,-2],[1,-1],[1,0],[1,1],[0,1],[1,1],[13,8],[1,-2],[1,0],[1,-1],[1,0],[0,4],[-1,3],[1,4],[2,1],[3,1],[1,2],[1,3],[2,2],[-2,2],[1,3],[2,4],[1,4],[-1,2],[-3,3],[1,6],[4,5],[6,6],[14,0],[-1,-1],[-1,-1],[-1,-2],[-1,-1],[9,-1],[2,2],[2,3],[2,0],[7,-1],[6,-5],[3,0],[16,3],[4,4],[2,6],[-2,1],[-13,-7],[2,2],[1,1],[1,2],[0,3],[-1,2],[-1,1],[-2,0],[-3,-4],[-2,-1],[-4,-1],[-1,-1],[-2,-2],[-1,0],[-3,0],[-2,1],[1,2],[1,1],[1,2],[1,0],[3,5],[2,7],[17,2],[6,3],[4,2],[3,0],[3,-1],[-3,0],[-9,-5],[-10,-1],[-4,0],[0,-2],[22,0],[-1,-2],[0,-1],[-1,-1],[-1,-1],[6,-2],[1,1],[1,1],[1,0],[0,-1],[1,-2],[7,-4],[3,0],[2,5],[0,3],[2,2],[12,3],[3,2],[5,5],[10,0],[2,2],[5,5],[2,3],[-1,4],[2,2],[5,2],[10,8],[19,5],[3,4],[-1,0],[-1,0],[-1,0],[1,4],[2,5],[2,2],[2,0],[4,-2],[7,-1],[4,-1],[2,-2],[2,-4],[1,-4],[1,-4],[0,-2],[-2,-4],[0,-2],[0,-2],[1,-2],[3,-3],[3,8],[0,2],[-2,1],[-1,-1],[2,5],[2,2],[6,-2],[1,2],[1,3],[2,7],[4,7],[1,4],[1,5],[1,3],[6,2],[2,2],[-3,2],[-5,-2],[-3,0],[2,1],[7,11],[4,4],[2,2],[0,-3],[-1,-2],[-2,-1],[0,-1],[2,-2],[1,2],[2,3],[2,1],[2,0],[4,3],[4,1],[1,0],[0,3],[1,2],[2,1],[1,0],[2,-2],[-3,-6],[-2,-3],[1,-2],[2,0],[4,2],[3,2],[1,3],[-1,4],[0,3],[2,1],[2,-1],[3,-1],[3,2],[2,2],[0,3],[2,3],[3,0],[6,0],[-3,-2],[-3,-1],[-3,-2],[-1,-5],[1,-5],[2,-2],[3,0],[1,4],[-4,1],[2,3],[3,1],[5,-2],[2,1],[1,1],[1,2],[-1,3],[2,1],[4,-8],[3,2],[0,-5],[0,-3],[2,-1],[2,0],[-4,-3],[-1,-2],[2,-1],[8,1],[2,1],[6,4],[2,0],[-3,-5],[-1,-3],[0,-2],[1,-3],[2,2],[2,3],[5,9],[1,1],[2,-1],[0,-2],[-1,-3],[-1,-2],[5,1],[9,4],[2,0],[2,-1],[-4,-3],[-2,-2],[1,-1],[6,1],[5,2],[0,1],[-1,1],[0,1],[3,2],[1,0],[-1,7],[2,5],[7,6],[-1,2],[-1,1],[-2,1],[-1,0],[0,2],[2,-1],[1,2],[1,2],[1,2],[1,1],[5,-1],[2,1],[-3,3],[2,4],[3,2],[3,1],[0,-2],[-1,0],[0,-1],[2,0],[2,-2],[0,-3],[-1,-4],[2,1],[1,-1],[2,-1],[1,-2],[1,-2],[0,-1],[-2,-5],[-1,-1],[-1,-1],[3,-1],[7,1],[3,-1],[-1,-2],[2,-2],[4,2],[2,-2],[-2,-3],[-2,-2],[-1,-2],[0,-4],[3,-2],[2,3],[2,5],[2,2],[1,-3],[0,-5],[-2,-5],[-2,-3],[-2,-3],[-2,-1],[-1,0],[-4,0],[-2,0],[-2,-2],[6,-2],[1,-1],[-2,-2],[-6,-3],[-4,-4],[-12,-5],[-5,-4],[-41,-21],[-9,-2],[-8,-5],[-4,0],[0,3],[-2,-1],[-3,-2],[-2,-2],[-4,-6],[-2,-1],[-4,-1],[-1,-1],[-1,-1],[-1,-2],[-6,1],[-1,-1],[-2,-4],[-2,-1],[0,-2],[1,-2],[-1,-5],[-1,-2],[-1,-2],[-9,-4],[-11,-1],[-2,-2],[-2,-5],[-4,-6],[-1,-3],[1,-2],[-1,-1],[-3,-2],[-3,-3],[-2,-2],[-1,-2],[-1,-1],[-4,5],[-5,1],[-2,2],[-1,3],[0,1],[1,2],[0,2],[-2,3],[-1,-1],[-1,-1],[-2,-1],[-9,1],[-2,-2],[-4,-4],[-5,-2],[-9,1],[0,-2],[1,-4],[0,-1],[-7,0],[-1,0],[-1,3],[-1,0],[-2,0],[-5,-4],[-2,-1],[-1,1],[-3,1],[-1,0],[-1,0],[-2,-1],[-1,-1],[-10,2],[1,-2],[1,-2],[0,-2],[0,-2],[-1,-2],[-1,1],[-3,4],[-3,3],[-4,0],[-7,-3],[-8,-5],[-3,-1],[-5,1],[-2,0],[-4,-2],[-7,-11],[3,-2],[0,-2],[0,-2],[-5,-2],[-1,-2],[0,-3],[1,-2],[-3,0],[-1,0],[-1,-2],[5,-4],[2,-2],[-4,-6],[-25,-11],[-1,-2],[0,-3],[3,-7],[0,-3],[-1,-1],[-2,2],[-2,3],[-1,4],[0,1],[-1,2],[0,2],[-1,0],[-5,-1],[-1,-3],[0,-2],[0,-3],[1,-5],[-2,-1],[-2,-1],[-2,-2],[1,-3],[-1,-4],[0,-8],[-2,-3],[0,8],[-1,9],[-1,9],[-3,6],[-2,0],[-2,0],[-4,0],[-2,-2],[-3,-4],[-1,-1],[-6,0],[-5,-2],[-1,-5],[-2,-1],[-4,-2],[-2,-3],[2,-1],[-1,-4],[-4,-9],[-1,-2],[-2,-1],[-1,-2],[0,-3],[1,-2],[3,0],[5,1],[-2,-2],[-4,-3],[-1,-2],[-1,-2],[-1,-8],[1,-2],[1,-3],[1,-4],[0,-3],[-1,-3],[-3,1],[-1,3],[0,6],[0,1],[-3,4],[3,5],[0,2],[0,2],[-2,0],[-3,-4],[-2,-2],[-4,0],[-2,0],[-5,4],[-2,0],[0,-1],[0,-1],[-1,-1],[0,-2],[-1,-1],[-1,0],[-3,1],[-1,-1],[-3,-1],[-1,-1],[-1,-3],[0,-1],[-1,-2],[0,-1],[0,-2],[1,-2],[1,-1],[-1,-3],[-1,-1],[-2,0],[-2,-1],[1,-2],[4,-3],[-2,-7],[-1,-1],[-2,-2],[-3,-2],[-2,-1],[-3,-5],[-1,-1],[-2,-1],[-2,1],[-2,1],[-1,2],[-1,3],[3,0],[2,2],[0,2],[-3,1],[-10,-5],[1,-3],[1,-2],[2,-2],[1,-2],[-3,-2],[-3,0],[-1,-2],[1,-6],[1,-3],[3,-3],[4,-3],[8,-4],[2,-1],[1,-2],[1,-1],[8,-2],[3,-2],[0,-3],[4,0],[12,-5],[12,1],[4,-1],[0,1],[-2,4],[-3,1],[-8,0],[4,-1],[0,-2],[-16,4],[-1,1],[0,2],[-1,1],[-1,2],[-2,1],[-2,0],[-2,1],[-3,1],[-1,4],[1,3],[2,-1],[2,-3],[3,-2],[3,-1],[3,0],[-2,3],[-10,6],[-3,1],[-7,-2],[-3,2],[0,2],[2,1],[-1,3],[-1,1],[-3,3],[2,1],[6,1],[7,3],[2,0],[2,0],[1,-2],[4,-3],[0,-1],[1,-3],[1,-1],[1,0],[3,-1],[1,-1],[-1,-1],[1,-1],[2,-1],[3,0],[0,3],[-1,0],[4,2],[9,-5],[4,0],[-3,6],[-1,4],[0,3],[2,1],[5,-1],[4,-3],[5,-5],[6,-3],[5,-5],[1,1],[0,4],[-3,3],[0,2],[4,1],[22,-8],[3,1],[2,2],[0,4],[1,1],[2,1],[2,0],[1,1],[1,0],[1,2],[0,2],[0,1],[-11,10],[-6,3],[-4,-3],[11,-5],[-3,-2],[-3,1],[-5,3],[-2,0],[-2,-1],[-2,-1],[-2,-2],[1,-5],[-4,-2],[-5,-1],[-5,1],[-2,1],[-1,1],[2,4],[2,1],[2,-1],[2,0],[1,3],[-3,0],[-4,2],[-5,3],[-1,3],[2,4],[5,0],[10,-2],[-2,4],[-5,4],[-5,3],[-4,0],[-3,3],[-2,6],[1,5],[3,3],[-1,3],[-1,1],[-3,-1],[1,4],[3,3],[5,2],[15,0],[2,1],[7,8],[3,1],[5,1],[3,2],[2,2],[2,4],[0,2],[0,2],[-1,1],[1,4],[2,5],[2,2],[1,-3],[-2,-13],[0,-2],[-1,-1],[0,-1],[0,-1],[0,-2],[-1,0],[-1,-1],[-1,-2],[-1,-1],[0,-1],[0,-2],[7,4],[4,4],[1,7],[0,3],[1,7],[1,2],[2,2],[2,0],[1,-1],[0,-4],[0,-2],[-1,-2],[-2,0],[-2,-1],[5,-3],[2,0],[0,1],[3,6],[2,1],[1,1],[2,1],[1,-1],[1,-2],[-1,-1],[-2,-4],[-2,-7],[0,-3],[2,2],[2,2],[4,12],[-1,3],[0,2],[0,1],[2,2],[0,1],[1,0],[1,0],[1,0],[1,-1],[0,-1],[0,-1],[0,-1],[4,0],[6,5],[4,0],[1,0],[3,1],[1,2],[0,4],[2,1],[3,6],[3,1],[-1,-3],[-2,-2],[0,-2],[2,-1],[1,0],[1,1],[1,1],[1,1],[4,3],[1,3],[6,12],[5,7],[-2,1],[-2,0],[-11,-6],[-1,-1],[0,2],[0,3],[0,1],[-1,1],[-3,2],[-3,0],[-4,-3],[-3,2],[8,5],[4,1],[3,-1],[-1,-2],[3,0],[5,3],[3,0],[13,-1],[-4,5],[1,3],[3,3],[4,0],[2,0],[5,-3],[8,-1],[2,-2],[4,-4],[-2,-1],[-3,-2],[-2,-5],[-1,-2],[1,-5],[0,-3],[-1,-2],[-2,-1],[-3,-2],[3,0],[4,2],[4,3],[3,3],[1,6],[0,4],[1,2],[4,0],[-2,8],[1,0],[2,0],[1,-1],[1,0],[0,1],[0,2],[-1,2],[1,2],[1,2],[1,2],[1,-1],[1,-3],[1,-1],[3,-13],[2,-3],[0,5],[-3,14],[0,3],[2,1],[2,-5],[2,-7],[1,-11],[0,-2],[-1,-3],[-2,-2],[-9,-4],[-5,-7],[-4,-3],[-1,-2],[-1,-4],[2,2],[6,3],[2,2],[5,8],[2,2],[3,1],[2,0],[2,0],[2,4],[1,0],[1,0],[1,1],[0,1],[-1,1],[0,1],[0,1],[0,2],[0,1],[0,2],[1,1],[1,2],[0,2],[-1,1],[-1,3],[0,4],[1,3],[-8,11],[-2,6],[-1,2],[0,2],[1,1],[2,-1],[1,-2],[1,-4],[1,-1],[4,-4],[0,-1],[1,-1],[0,-2],[1,0],[0,1],[1,0],[1,-1],[1,0],[1,-1],[0,-2],[-1,-12],[0,-2],[0,-1],[0,-2],[1,-2],[0,-2],[-1,-3],[-2,-3],[-1,-2],[0,-4],[1,0],[2,1],[1,2],[1,2],[2,6],[0,2],[0,3],[1,2],[0,2],[2,6],[0,2],[-1,7],[-1,2],[0,2],[-2,1],[-1,1],[-3,5],[-1,1],[-2,0],[-2,1],[-1,2],[-1,3],[1,4],[3,2],[3,1],[3,0],[1,-2],[1,-3],[0,-3],[-2,-2],[3,-5],[3,-6],[2,-8],[0,-9],[0,-5],[-1,-4],[-1,-5],[-2,-4],[0,-1],[-1,-5],[0,-2],[-4,-5],[-1,-2],[3,2],[3,3],[3,5],[1,4],[1,5],[6,16],[0,1],[0,1],[0,3],[-2,4],[-1,3],[1,1],[0,1],[0,3],[1,2],[2,0],[2,-3],[0,-1],[0,-2],[0,-4],[0,-4],[1,-8],[0,-2],[0,-3],[0,-3],[-1,-2],[-1,-1],[-1,-2],[0,-1],[-4,-8],[-1,-1],[-2,-1],[-2,-2],[-1,-3],[1,-2],[1,-3],[1,-2],[1,1],[1,5],[3,6],[3,5],[2,3],[3,1],[1,-3],[0,-4],[-1,-5],[-1,-5],[-2,-4],[-6,-9],[0,-3],[0,-3],[-1,-4],[-2,-3],[-1,-3],[1,-1],[1,-1],[0,-2],[-1,-2],[-1,-1],[1,-6],[1,-2],[1,-1],[1,0],[1,1],[1,0],[0,1],[2,0],[2,-2],[1,0],[-3,6],[0,4],[2,4],[1,4],[-1,4],[-2,4],[0,4],[1,3],[9,12],[5,3],[3,4],[-2,1],[-1,3],[0,3],[1,1],[2,0],[3,4],[2,1],[7,2],[3,4],[2,7],[-3,0],[-9,-2],[25,21],[3,5],[1,1],[4,3],[1,2],[1,2],[-1,2],[-1,2],[1,3],[1,2],[4,2],[2,1],[1,3],[1,3],[1,2],[3,1],[0,1],[-1,0],[-2,0],[-1,1],[-1,1],[5,2],[19,2],[24,10],[6,1],[23,8],[11,8],[2,1],[9,-1],[6,3],[37,5],[5,3],[4,5],[2,3],[1,2],[3,8],[3,5],[5,3],[4,0],[3,-5],[-4,4],[0,-4],[-1,-6],[-1,-5],[-4,-2],[-1,-2],[0,-2],[2,-5],[0,-2],[1,-3],[-21,1],[-7,-4],[5,-13],[0,-2],[0,-3],[-1,-2],[-2,-8],[1,-3],[1,-1],[3,-1],[8,0],[6,-4],[3,0],[3,1],[2,1],[-4,5],[-2,3],[1,3],[3,3],[3,1],[9,1],[3,1],[3,3],[5,6],[5,4],[6,9],[4,5],[20,16],[10,4],[3,3],[-1,1],[0,1],[0,2],[1,1],[0,1],[0,2],[0,4],[-1,3],[0,1],[-2,1],[-1,1],[-1,2],[0,4],[-1,4],[-1,4],[1,5],[3,3],[8,3],[3,2],[2,2],[2,1],[4,1],[2,1],[3,3],[2,1],[13,-2],[8,6],[4,1],[8,0],[26,7],[8,-4],[-3,5],[-4,2],[-19,3],[-14,2],[-18,6],[-4,3],[0,4],[-2,2],[-1,2],[-1,3],[-1,3],[3,0],[9,-2],[2,-2],[3,0],[3,9],[3,2],[-4,2],[-14,-6],[-9,-1],[-4,1],[-2,5],[0,12],[4,11],[5,10],[5,5],[7,5],[0,1],[0,1],[0,1],[1,1],[1,0],[2,-1],[1,-1],[4,-4],[26,-17],[29,-22],[24,-24],[5,-6],[6,-7],[1,-3],[1,-4],[7,-11],[7,-17],[7,-11],[2,-5],[2,-5],[0,-1],[1,-2],[2,-2],[5,-2],[3,-2],[2,-1],[1,-1],[-2,0],[0,-6],[1,-2],[2,-2],[-4,-3],[-1,-3],[0,-3],[4,-9],[5,-14],[1,-4],[1,-2],[1,-2],[3,-4],[3,-6],[42,-43],[2,-4],[2,-2],[2,-2],[2,-1],[4,-1],[1,-2],[4,-4],[21,-17],[15,-6],[4,0],[7,2],[3,-1],[1,0],[1,-1],[0,-1],[1,-2],[2,-1],[4,0],[0,-1],[1,-1],[0,-2],[1,0],[16,-9],[3,0],[1,1],[4,5],[1,3],[-1,0],[-2,3],[0,1],[-2,2],[-5,2],[-2,3],[1,2],[1,1],[3,-3],[30,-3],[2,1],[1,2],[1,3],[1,2],[0,1],[-2,1],[-1,1],[2,3],[5,2],[-7,0],[0,2],[1,0],[1,0],[0,1],[1,1],[-12,-1],[-3,-1],[1,-3],[-2,-1],[-2,1],[-3,1],[1,1],[0,1],[0,1],[1,1],[-2,1],[-18,-5],[-9,-5],[-4,0],[-1,0],[-1,-2],[-1,-1],[-1,0],[-1,0],[1,3],[3,4],[2,3],[-1,1],[0,1],[0,1],[4,1],[5,-1],[0,2],[-1,1],[0,1],[0,1],[3,4],[2,1],[3,-2],[15,-1],[0,1],[-1,1],[-2,2],[0,1],[2,1],[3,-2],[5,-4],[7,-3],[7,0],[-4,3],[-10,3],[-3,5],[0,2],[-2,1],[-1,0],[0,2],[1,0],[2,1],[1,1],[0,2],[5,3],[-3,2],[-19,2],[-3,2],[4,0],[2,1],[1,3],[-2,0],[0,1],[-1,0],[-1,1],[-1,1],[4,11],[2,2],[3,2],[9,-2],[3,0],[10,8],[2,0],[3,0],[2,0],[0,1],[0,1],[0,1],[0,2],[1,0],[1,1],[4,-1],[7,-4],[0,-1],[0,-3],[0,-2],[2,-3],[1,-2],[0,-2],[0,-3],[1,-4],[2,1],[2,3],[2,3],[2,5],[0,4],[1,1],[4,-1],[-6,6],[-6,3],[-17,2],[-7,3],[-4,1],[-5,-2],[-3,0],[-2,5],[1,2],[-1,2],[-1,1],[-1,0],[7,8],[2,2],[2,1],[2,0],[-1,-4],[1,-3],[0,-6],[1,-1],[9,0],[1,0],[1,1],[0,1],[3,4],[1,2],[0,3],[0,3],[0,2],[5,5],[5,10],[2,2],[2,2],[22,8],[1,1],[2,-2],[2,0],[2,0],[1,3],[0,3],[-2,3],[-2,3],[-2,1],[-2,-2],[-2,-7],[-2,-3],[-3,-1],[-4,1],[-4,1],[-2,2],[3,3],[4,2],[-2,2],[-10,0],[-1,-1],[-1,-1],[0,-2],[-1,-1],[-1,0],[-1,0],[-3,3],[-3,0],[-1,0],[-2,-1],[-4,-4],[1,-1],[0,-2],[-2,0],[-3,0],[-2,-1],[-2,-2],[1,-1],[1,0],[1,-3],[-2,-1],[-2,0],[-2,1],[-2,0],[-1,-1],[-2,-2],[-1,0],[-1,3],[2,3],[2,3],[-1,3],[-2,4],[1,2],[4,2],[4,5],[8,3],[3,3],[1,1],[0,4],[1,0],[1,0],[1,0],[2,-1],[0,-3],[-1,-5],[-2,-1],[-3,-1],[-2,-2],[5,-2],[6,-1],[6,2],[5,3],[-12,-2],[0,2],[2,0],[2,2],[1,1],[6,2],[1,1],[1,1],[1,1],[2,0],[0,-1],[1,-2],[1,0],[2,1],[13,4],[2,-1],[2,-3],[4,-6],[0,-4],[3,-2],[3,1],[2,5],[-4,0],[4,5],[4,-1],[5,-3],[4,2],[-4,7],[-2,2],[-2,-1],[-3,-2],[-3,-1],[-3,1],[-2,2],[2,1],[3,1],[2,0],[1,3],[-9,3],[-4,-1],[-10,-5],[-3,0],[0,1],[2,1],[1,0],[3,3],[-1,1],[-2,1],[-1,0],[-3,-2],[0,1],[0,1],[-1,1],[1,2],[1,0],[1,0],[0,1],[3,3],[4,1],[8,-2],[1,1],[3,2],[1,1],[2,-1],[2,-2],[1,-1],[8,2],[4,0],[2,-3],[-5,-5],[-1,-2],[1,-2],[2,0],[2,-2],[3,-3],[1,-3],[0,-4],[-1,-3],[-1,-3],[-1,-2],[-3,-2],[-7,2],[-4,-3],[3,-2],[5,0],[3,-1],[-2,-1],[-5,0],[0,-2],[-1,-2],[-2,-1],[-3,0],[-1,-1],[6,-2],[6,-1],[0,-1],[0,-2],[-1,-2],[0,-2],[-1,-2],[-1,0],[-3,1],[-3,-1],[1,-3],[3,-2],[2,-1],[1,-1],[0,-1],[0,-3],[1,-1],[1,-1],[1,-1],[2,0],[-2,-2],[-3,-4],[-3,-5],[0,-4],[-2,0],[0,-2],[0,-3],[1,-3],[0,-1],[0,-2],[-3,-4],[3,-4],[8,1],[2,-1],[3,-4],[5,-1],[5,1],[3,2],[3,4],[3,1],[14,-1],[1,-1],[1,-1],[1,-2],[1,-1],[0,-1],[-1,-4],[1,-1],[1,0],[-4,-4],[-13,-8],[-3,-4],[-1,-5],[0,-5],[-2,-6],[-4,-3],[-8,-5],[-6,-5],[-4,-5],[-1,-3],[0,-3],[1,-3],[2,-6],[-10,1],[-2,-2],[2,-3],[7,-6],[2,-1],[5,0],[4,-2],[2,-1],[19,-2],[7,3],[5,8],[-1,-3],[2,-5],[2,-3],[2,0],[1,2],[0,2],[-3,4],[2,4],[1,2],[2,0],[2,-1],[0,-1],[0,-1],[1,-1],[8,-3],[2,-1],[0,1],[2,2],[1,1],[1,0],[2,0],[2,-2],[-1,-3],[6,2],[3,1],[3,-2],[1,0],[0,-1],[1,-1],[2,-1],[2,1],[3,2],[8,1],[19,9],[-5,2],[-2,1],[0,3],[-1,6],[1,4],[2,1],[1,0],[7,5],[10,1],[-1,-1],[11,-2],[1,1],[0,3],[0,3],[0,3],[1,2],[1,1],[1,2],[0,1],[1,1],[0,2],[0,4],[-3,4],[-1,2],[1,3],[1,3],[2,1],[2,2],[1,3],[1,2],[-1,12],[-1,5],[-2,4],[-3,2],[9,5],[3,2],[5,6],[3,3],[3,1],[13,-9],[8,-2],[3,7],[0,2],[1,0],[4,-3],[2,0],[4,1],[2,0],[5,-3],[6,1],[6,-3],[14,3],[21,-2],[15,3],[44,-7],[10,-5],[15,-1],[35,-17],[35,-17],[14,-16]],[[1462,6983],[-1,-2],[-1,-1],[-2,0],[-1,1],[1,-5],[-2,-2],[-3,-1],[-3,1],[-4,5],[-2,0],[-2,-3],[1,-1],[0,-1],[1,-1],[-3,-1],[-9,-6],[3,7],[7,18],[3,3],[3,0],[6,-3],[5,-4],[3,-4]],[[1708,7309],[-5,-1],[-2,1],[-2,1],[-1,2],[2,1],[2,0],[6,-4]],[[3507,7437],[-4,4],[-25,11],[3,3],[4,0],[11,-3],[11,-8]],[[3454,7456],[-8,-1],[-4,2],[-2,5],[5,0],[3,0],[2,-2],[1,0],[2,-1],[0,-1],[1,-2]],[[3454,7468],[9,-1],[1,0],[2,0],[1,-1],[1,-2],[-16,3],[2,1]],[[3473,7470],[-4,0],[-8,2],[3,2],[2,1],[6,0],[4,-1],[-3,-4]],[[3507,7469],[-6,5],[-5,3],[-4,-1],[-4,-3],[-6,0],[-5,2],[-3,5],[11,2],[3,2],[-3,3],[-1,3],[1,3],[9,0],[9,-2],[4,-2]],[[3507,7499],[-5,2],[-7,0],[-4,-2],[-5,-3],[-4,-1],[-3,0],[-2,1],[1,2],[2,2],[5,1],[2,2],[1,2],[0,3],[-3,2],[-1,2],[-3,4],[-28,4],[-5,2],[-5,4],[-4,9],[-3,3],[-6,3],[-4,4],[-3,5],[-1,5],[2,4],[4,2],[35,-2],[36,-2],[8,-2]],[[2732,6750],[0,1],[-2,2],[-5,2],[-6,4],[-10,2],[-4,2],[-6,7],[-7,5],[-3,1],[-2,2],[-1,4],[-1,5],[-1,3],[0,4],[-6,9],[-4,12],[-1,8],[-3,4],[-4,5],[3,7],[3,4],[3,3],[7,2],[12,9],[26,9],[25,4],[26,4],[54,14],[45,7],[45,3],[21,4],[33,3],[28,6],[37,0],[17,-8],[44,-5],[19,5],[29,-6],[26,-6],[3,1],[-2,2],[0,1],[2,1],[11,-2],[4,-3],[28,-8],[4,0],[3,2],[-1,2],[-6,1],[3,4],[4,2],[7,0],[1,-1],[4,-2],[2,-1],[5,2],[1,0],[0,2],[-5,1],[2,3],[4,1],[19,0],[4,3],[-15,1],[-5,4],[6,2],[36,-11],[12,3],[13,-3],[9,4],[10,-2],[3,0],[6,3],[3,2],[-4,2],[-9,-1],[-3,4],[3,1],[6,2],[2,2],[-3,5],[-6,2],[-27,-2],[-8,3],[-10,2],[-2,1],[-3,4],[-5,3],[-2,2],[0,1],[-1,4],[0,2],[-5,8],[-2,4],[-3,2],[-3,1],[-7,-1],[-3,1],[-5,4],[-3,0],[-4,-3],[-2,0],[-2,2],[0,2],[-1,2],[-2,1],[-2,-1],[-3,-4],[-2,0],[-3,0],[-5,2],[-6,6],[-5,2],[-5,2],[-11,1],[-2,-1],[-3,-6],[-2,1],[-4,4],[-15,6],[-4,5],[-2,2],[-3,-1],[-5,-2],[-2,1],[-1,2],[-1,3],[1,2],[0,2],[-18,0],[-6,2],[-8,6],[-2,1],[-1,-1],[-1,-1],[0,-2],[0,-1],[0,-2],[-4,-3],[-5,2],[-4,3],[-6,3],[-3,4],[-31,5],[-5,-1],[-12,-9],[-5,-1],[-18,4],[-6,-1],[-12,-9],[-42,-4],[-19,-6],[-22,2],[-29,-3],[-2,0],[-4,-4],[-2,-1],[-2,1],[-17,6],[-35,-7],[-16,5],[-3,2],[1,2],[4,1],[-3,4],[-5,1],[-39,-2],[-8,-4],[-3,-2],[-35,-1],[-28,4],[-11,-4],[-2,3],[1,1],[-4,2],[-15,-2],[-1,0],[0,-1],[0,-1],[0,-1],[-2,-1],[-2,0],[-4,1],[-12,7],[-5,7],[3,7],[-2,5],[-5,4],[-4,2],[-7,2],[-36,22],[-10,10],[-5,6],[-10,17],[-1,4],[4,7],[33,18],[33,18],[37,4],[43,14],[22,-1],[5,2],[4,4],[-1,1],[-1,0],[4,5],[55,15],[5,3],[19,3],[3,-2],[-11,-4],[2,-1],[3,-1],[3,1],[6,4],[32,1],[4,4],[-6,2],[-13,0],[-19,6],[-16,2],[-1,2],[3,2],[16,0],[35,16],[-2,-3],[1,-3],[2,0],[14,3],[3,-2],[-2,-1],[4,-3],[9,-1],[7,-4],[4,2],[4,2],[3,2],[14,2],[6,5],[10,5],[-5,2],[-22,-9],[-15,-2],[-7,1],[-7,4],[0,2],[11,3],[0,2],[-17,4],[-24,-2],[-7,-4],[-11,-1],[-7,-3],[-11,-1],[-5,-2],[-42,-5],[-9,-6],[-11,0],[-10,-6],[-12,-1],[-14,2],[-2,-1],[-2,-1],[-5,-3],[-4,0],[-5,-3],[-2,0],[-7,1],[-9,-4],[-3,-1],[-2,1],[-1,1],[1,2],[1,6],[2,4],[1,2],[-2,2],[-9,-1],[1,2],[1,1],[1,0],[-2,4],[-4,3],[-4,1],[-3,1],[0,-2],[1,-1],[0,-1],[-3,-2],[-4,-1],[-2,-1],[0,-2],[1,-3],[2,-1],[8,0],[4,-2],[2,-4],[-1,-4],[-2,-3],[-3,-4],[-2,-1],[-18,3],[1,2],[-1,2],[-2,1],[-1,-2],[-3,-4],[-2,-1],[-2,-1],[-4,1],[-5,2],[-2,0],[-30,-2],[-9,4],[-7,9],[0,9],[6,8],[14,9],[22,2],[17,-4],[3,2],[-7,3],[-2,2],[8,0],[8,-1],[3,1],[2,3],[-5,2],[-9,1],[-4,2],[-2,3],[-8,3],[-1,-1],[-2,0],[0,-1],[0,-1],[-1,-1],[-1,0],[-2,1],[-1,1],[0,3],[1,3],[2,2],[9,0],[19,6],[-3,2],[-40,-3],[-6,-3],[-6,-6],[1,-4],[-4,-3],[-10,-1],[-7,0],[-2,-1],[-4,-2],[-2,0],[1,2],[2,4],[1,2],[-1,3],[-1,1],[-5,3],[-1,2],[-3,2],[-6,1],[-5,2],[-2,-1],[-1,-1],[-2,-3],[-1,-1],[-2,1],[-2,2],[-1,1],[-17,0],[-3,-1],[1,-3],[-1,-2],[-2,-2],[-2,-1],[-1,-1],[1,-3],[1,-1],[2,-3],[-2,-2],[-4,1],[-8,7],[-2,1],[-2,0],[-2,1],[-1,3],[-4,7],[-1,5],[-2,12],[-1,4],[-1,5],[1,5],[2,11],[1,10],[1,4],[2,5],[2,4],[12,13],[14,12],[9,3],[3,5],[1,2],[1,2],[-1,2],[-1,2],[2,5],[3,4],[11,8],[16,1],[35,11],[3,5],[-3,15],[-5,7],[-8,2],[-21,0],[-8,4],[-2,1],[-3,1],[-2,4],[-3,5],[-1,5],[3,11],[6,8],[26,19],[4,1],[2,1],[1,2],[2,5],[2,2],[2,2],[5,2],[10,5],[4,1],[16,6],[6,9],[24,15],[6,8],[7,5],[3,3],[6,10],[4,3],[18,1],[11,5],[25,6],[28,16],[6,6],[11,0],[4,2],[2,2],[14,4],[13,6],[22,5],[27,11],[22,4],[20,8],[2,2],[1,2],[2,2],[2,1],[2,0],[10,0],[2,1],[10,7],[24,6],[19,11],[7,3],[11,1],[20,-9],[16,-3],[8,-6],[19,-23],[5,-9],[2,-5],[1,-4],[-1,-4],[-4,-12],[-1,-2],[0,-3],[0,-2],[1,-2],[-1,-2],[-1,-2],[-2,-3],[-1,-2],[0,-3],[0,-3],[-1,-1],[1,-3],[2,-3],[0,-2],[0,-5],[-1,-1],[-1,-1],[-1,-2],[0,-2],[0,-3],[0,-2],[2,-2],[5,-5],[2,-4],[-1,-4],[-5,-2],[-24,-2],[-1,0],[-4,-6],[-6,-3],[-2,-2],[1,-4],[0,-2],[1,-5],[0,-2],[2,-1],[6,2],[2,-1],[-3,-1],[-5,-3],[-10,-4],[-2,-2],[-4,-5],[-1,-1],[-15,-4],[2,-1],[1,-1],[3,-3],[-1,-4],[1,-1],[4,2],[12,-4],[5,0],[3,1],[2,7],[2,2],[5,0],[15,9],[27,6],[-2,-3],[5,-3],[7,1],[27,14],[6,1],[-6,-8],[-14,-8],[-6,-6],[3,-1],[3,0],[3,1],[2,2],[1,1],[0,2],[1,1],[1,1],[3,1],[12,5],[4,5],[0,5],[2,3],[1,2],[-3,6],[-13,0],[-5,2],[0,4],[3,5],[10,10],[11,17],[22,20],[20,5],[9,-2],[9,0],[2,-1],[4,-3],[2,-1],[29,-6],[18,-11],[11,-3],[23,-1],[7,-4],[10,-3],[5,-3],[19,-6],[7,-5],[11,-1],[5,-3],[3,-3],[2,-3],[0,-1],[0,-3],[1,-1],[3,0],[4,-2],[9,-1],[11,-3],[-1,-4],[0,-1],[-1,-2],[2,-3],[1,-2],[-1,-4],[-3,-7],[-1,-4],[-1,-6],[-1,-2],[-2,-3],[-23,-14],[1,0],[2,-1],[2,-1],[1,-2],[-1,-2],[-13,-10],[-33,-15],[-4,-3],[-2,-5],[2,0],[2,-1],[3,-3],[2,-2],[3,0],[6,3],[6,5],[4,1],[1,-5],[-1,-4],[-7,-5],[0,-3],[0,-4],[-3,-4],[-3,-4],[-2,-4],[2,-3],[2,-1],[2,2],[3,4],[0,2],[0,2],[1,1],[2,4],[2,6],[2,0],[3,0],[2,0],[2,1],[3,3],[9,4],[7,5],[-2,2],[-3,-1],[-5,-3],[-5,0],[-3,1],[-2,3],[-1,2],[-1,1],[1,1],[2,2],[1,2],[5,3],[6,1],[4,2],[4,3],[3,5],[4,4],[6,2],[6,0],[4,-2],[-1,-2],[-1,-1],[-2,-2],[-11,-5],[-3,-3],[2,-1],[5,1],[3,-2],[-9,-7],[-2,-4],[3,-2],[2,1],[2,3],[2,2],[2,1],[4,-1],[2,-1],[1,-2],[1,-4],[1,-1],[6,0],[3,-2],[3,-3],[3,-4],[2,-3],[2,-1],[0,7],[-2,5],[-3,4],[-2,5],[1,9],[3,3],[5,2],[4,5],[-3,3],[1,2],[3,2],[3,1],[17,-2],[-3,6],[-1,2],[-1,4],[12,11],[3,2],[3,0],[6,-5],[4,-1],[7,-1],[5,2],[0,-1],[0,-2],[-1,-2],[-11,-6],[-2,-2],[1,-3],[3,-2],[3,-2],[2,-1],[1,-1],[7,-2],[3,-2],[4,-2],[3,-2],[4,-2],[3,2],[-20,11],[-6,6],[2,3],[4,2],[3,1],[2,-1],[1,1],[1,1],[1,4],[2,1],[1,1],[4,0],[3,-1],[4,-4],[5,-2],[9,-12],[6,-5],[3,-1],[1,0],[2,-4],[1,0],[4,4],[1,1]],[[2197,7934],[18,-6],[17,0],[2,0],[5,-5],[9,0],[14,-10],[3,-5],[0,-1],[1,-5],[1,-1],[4,1],[44,-18],[44,-17],[19,-2],[5,-2],[9,-6],[33,0],[6,-2],[-4,-1],[-9,2],[-4,-2],[2,-7],[-4,-7],[-15,-15],[-1,-3],[-1,-5],[1,-4],[3,-3],[4,-2],[3,0],[1,3],[-1,2],[-2,3],[0,2],[1,5],[3,3],[11,6],[5,5],[4,7],[4,7],[3,3],[4,0],[8,-2],[16,3],[10,-5],[2,-2],[2,-4],[1,-5],[0,-3],[-4,-5],[5,-1],[2,-1],[1,-3],[0,-4],[-2,-3],[-4,-1],[-2,-2],[-1,-2],[-2,-6],[-1,-3],[-1,-2],[-1,-2],[0,-3],[5,1],[9,5],[5,0],[3,-2],[1,-1],[3,2],[1,1],[3,4],[2,5],[3,5],[2,2],[5,1],[1,3],[2,6],[-3,3],[-3,2],[-3,4],[-1,2],[0,2],[0,1],[2,2],[3,1],[5,1],[7,5],[12,1],[8,4],[11,1],[6,3],[38,5],[37,-5],[37,-6],[31,-18],[31,-17],[27,-23],[27,-22],[3,-4],[4,-4],[9,-2],[7,-8],[23,-15],[6,-8],[22,-12],[10,-1],[3,-2],[1,-1],[3,-6],[2,-1],[4,-3],[4,-5],[11,-9],[5,-6],[2,-4],[1,-4],[1,-5],[0,-2],[-2,0],[-2,-1],[-6,-2],[-1,-1],[-4,-7],[-1,-1],[-9,-1],[-2,-1],[-4,-4],[-18,-11],[-26,-8],[-27,-10],[-27,-6],[-5,-5],[-24,-9],[-6,-5],[-2,-2],[-8,-1],[-6,-6],[-8,-6],[-19,-6],[-6,-6],[-2,-1],[-20,-2],[-33,-18],[-8,-2],[-13,-11],[-13,-5],[-6,-5],[-22,-7],[-22,-12],[-4,-3],[-3,-5],[-4,-4],[-5,-2],[-17,0],[-5,-2],[-16,-13],[-25,-12],[-2,-1],[-2,-1],[-1,-3],[-1,-3],[-1,-3],[0,-3],[1,-7],[0,-3],[-2,-2],[-4,-4],[-2,-2],[1,-5],[-2,-3],[-3,-2],[-2,-3],[-1,-4],[-2,-6],[0,-6],[2,-3],[1,-1],[-1,-2],[-2,-2],[-1,-1],[-5,-3],[-6,-4],[-24,-11],[-14,-10],[-4,-2],[-3,0],[-8,2],[-2,0],[-6,-2],[-23,3],[-2,2],[6,3],[2,2],[-3,2],[-4,0],[-5,0],[-3,-3],[0,-4],[2,-3],[5,-3],[2,-4],[0,-3],[1,-3],[2,-6],[0,-3],[-1,-3],[-1,-3],[-3,-7],[-12,-17],[-5,-4],[-5,-4],[-5,-4],[-2,-9],[5,-7],[2,-4],[0,-2],[-1,-3],[-1,-6],[0,-6],[0,-5],[-1,-7],[0,-3],[1,-4],[2,-5],[1,-3],[0,-2],[-1,-2],[-2,-5],[-2,-7],[-10,-22],[-14,-12],[-14,-5],[-26,-11],[-24,-4],[-25,-7],[-21,5],[7,4],[1,3],[-3,5],[-4,3],[-6,1],[-5,0],[-4,-2],[-14,-13],[-8,-11],[-6,-3],[-9,-12],[-7,-7],[-4,-2],[-8,-2],[-10,-5],[-9,-2],[-26,-12],[-5,-6],[-3,-2],[-3,-7],[-3,-3],[-5,-3],[-34,-1],[-6,2],[-12,9],[-11,13],[-9,16],[-14,36],[-8,15],[-5,8],[-5,7],[-2,1],[-7,3],[-2,1],[-1,2],[-2,5],[-11,20],[-15,11],[-33,12],[-34,13],[-24,12],[-11,9],[-30,12],[-2,4],[26,1],[1,-1],[4,-3],[1,-1],[3,1],[0,1],[0,2],[-2,2],[-5,3],[-30,1],[-33,-2],[-26,-6],[-10,4],[8,2],[3,3],[3,4],[2,5],[1,8],[1,3],[0,2],[0,4],[-1,4],[-2,1],[-5,0],[5,8],[7,5],[7,2],[6,0],[0,1],[-7,1],[-3,2],[0,4],[-1,1],[0,1],[-1,3],[1,1],[1,4],[1,1],[9,-2],[3,2],[5,5],[1,0],[1,2],[0,4],[-1,2],[-1,-2],[-2,-2],[-1,0],[-1,1],[1,4],[1,2],[1,3],[2,3],[2,1],[4,2],[2,2],[1,1],[2,7],[-1,1],[-3,1],[-1,1],[3,2],[8,0],[-2,4],[1,2],[5,4],[-1,4],[2,2],[4,3],[5,1],[5,3],[13,12],[3,0],[7,0],[2,1],[-3,5],[1,4],[-8,0],[-3,2],[-4,4],[2,1],[3,1],[2,0],[2,3],[-1,1],[-2,1],[-1,0],[2,3],[2,-1],[2,0],[2,0],[2,1],[1,2],[2,5],[-2,1],[-7,-1],[2,3],[2,4],[2,3],[-1,4],[-1,2],[-2,2],[-1,3],[-1,2],[0,4],[0,3],[-2,1],[-5,2],[-3,3],[-1,3],[1,0],[1,1],[2,2],[2,2],[4,-3],[5,-1],[1,-1],[3,-2],[20,9],[8,-2],[15,5],[5,4],[4,0],[2,2],[0,2],[-6,4],[1,1],[2,3],[1,1],[-1,2],[-2,0],[-3,0],[-2,0],[-3,2],[-3,3],[-2,3],[1,2],[-11,0],[2,-3],[1,-1],[-3,0],[-4,2],[-10,11],[-2,3],[0,3],[2,1],[3,3],[1,1],[0,3],[1,2],[1,2],[1,1],[13,3],[3,3],[-1,1],[-1,0],[-1,1],[0,2],[0,2],[0,1],[1,1],[1,0],[6,9],[1,1],[2,2],[4,12],[5,10],[3,3],[5,2],[2,3],[0,4],[-4,4],[-1,3],[2,2],[2,2],[12,5],[2,2],[-3,3],[4,1],[11,0],[3,5],[-2,0],[2,4],[8,2],[2,3],[0,1],[-1,6],[1,2],[2,2],[1,2],[-1,1],[-2,3],[-1,3],[0,3],[0,4],[2,2],[5,2],[11,7],[6,0],[3,1],[1,3],[-2,3],[2,4],[6,5],[-2,4],[-3,8],[-2,3],[-5,3],[-30,3],[-4,3],[-3,4],[-2,2],[1,3],[2,5],[-1,1],[-3,3],[-1,2],[-2,3],[0,1],[-2,1],[-3,2],[-5,2],[-1,1],[2,1],[-1,4],[-4,5],[0,2],[-2,3],[-5,3],[-2,3],[2,1],[-2,3],[-1,4],[3,0],[1,0],[1,3],[-1,3],[-1,3],[-2,2],[-1,1],[-2,2],[0,2],[-1,3],[0,3],[-1,2],[-2,2],[0,1],[1,2],[0,2],[-3,7],[-4,1],[-9,-1],[-4,2],[3,4],[8,5],[-6,3],[-11,-2],[-4,4],[3,4],[8,1],[31,5],[21,2],[12,-2],[2,2],[0,4],[2,1],[40,2],[6,3],[23,1],[20,5],[35,1],[42,2],[25,4],[32,5],[37,11],[23,3]],[[3507,8253],[-6,4],[6,4]],[[2253,8218],[-9,0],[-10,3],[-4,3],[5,4],[15,7],[3,2],[2,0],[2,0],[2,1],[2,2],[0,2],[-2,2],[2,1],[2,6],[-1,2],[-4,4],[0,3],[2,2],[3,3],[4,2],[5,1],[4,-1],[2,-5],[-1,-1],[0,-3],[0,-2],[1,-2],[-1,-2],[-2,-9],[-1,-3],[-1,-4],[-9,-11],[-8,-6],[-3,-1]],[[2641,8313],[4,-3],[4,0],[5,-1],[9,-6],[-2,-1],[-1,-2],[-1,-6],[-2,-4],[-2,-3],[-5,-3],[1,-4],[-1,-3],[-2,-2],[-2,-2],[-2,-2],[-4,-5],[-1,-2],[-9,-4],[-3,-3],[-5,-7],[-1,-3],[1,0],[0,-1],[0,-1],[1,-1],[-2,-2],[-5,-1],[-2,-2],[1,-2],[-1,-2],[-4,-2],[-1,-2],[-1,-3],[0,-2],[1,-1],[0,-3],[1,-1],[0,-2],[0,-1],[-4,-1],[-2,-1],[-3,-3],[-29,-23],[6,-2],[2,-2],[-6,-1],[-41,-23],[-5,0],[-6,2],[-9,5],[-17,1],[1,3],[2,0],[3,0],[2,2],[-7,2],[-4,-1],[-3,-1],[-2,0],[-5,3],[-4,1],[-12,-2],[-24,9],[0,1],[0,1],[1,2],[-1,0],[9,10],[31,22],[7,3],[3,2],[5,8],[9,7],[5,3],[5,1],[2,6],[6,6],[7,5],[34,12],[13,0],[4,0],[2,2],[2,2],[2,7],[4,5],[3,1],[4,0],[15,7],[-1,3],[11,7],[16,4]],[[3507,8337],[-16,12],[-27,6],[1,3],[4,4],[1,3],[-1,1],[-2,0],[-1,2],[-1,2],[1,2],[0,2],[-2,2],[-1,2],[1,1],[0,2],[0,3],[0,2],[2,0],[1,0],[4,0],[24,6],[7,5],[5,2]],[[3507,8005],[-2,-1],[-4,-4],[-1,-2],[-2,1],[0,2],[0,2],[-8,-4],[-17,6],[-8,-2],[1,-1],[1,0],[0,-1],[-1,0],[-2,0],[-3,-2],[0,-1],[8,-3],[3,-1],[-3,-2],[-30,-3],[-5,2],[-2,0],[0,-5],[1,-3],[3,-4],[5,-4],[-3,-5],[-18,-5],[-7,-7],[-3,-1],[-10,-1],[-1,-2],[-1,-2],[-1,-2],[-3,-2],[-8,0],[-9,-5],[-8,0],[-3,-2],[-6,-5],[-2,-1],[-10,2],[-19,-9],[-9,-7],[-33,-7],[-33,-6],[-8,-4],[-9,-2],[-5,-3],[-23,0],[-44,-4],[-22,2],[-26,4],[-27,6],[-8,6],[-47,23],[0,2],[1,0],[-3,1],[-22,15],[-5,4],[-1,3],[0,4],[2,4],[3,3],[10,4],[8,7],[30,12],[16,1],[9,5],[14,2],[15,-1],[8,4],[10,2],[9,-4],[5,0],[2,4],[-7,3],[-8,2],[2,1],[2,0],[1,0],[9,1],[2,1],[1,1],[1,4],[7,6],[11,3],[6,3],[18,4],[11,-1],[17,5],[32,0],[23,2],[18,-3],[14,-3],[12,5],[10,0],[4,2],[6,6],[2,2],[-3,3],[-5,-3],[-3,3],[4,4],[4,-1],[7,-3],[5,1],[13,6],[-2,3],[-2,1],[-3,1],[-2,0],[3,2],[5,-2],[3,0],[1,1],[0,1],[0,1],[8,4],[3,0],[1,0],[2,3],[4,1],[8,6],[-4,3],[1,1],[4,2],[2,2],[-1,2],[2,1],[7,0],[2,2],[-1,3],[-3,2],[-17,5],[-16,-2],[-1,-1],[-3,-7],[-1,-3],[-3,-2],[-7,-6],[-6,-2],[-7,-3],[-4,-1],[-5,2],[-3,0],[-3,-2],[5,-1],[2,-2],[-13,0],[-3,2],[-4,7],[-2,1],[-1,0],[-1,-1],[-1,-1],[-13,-10],[-3,-1],[-15,3],[-18,-5],[-15,-2],[-8,2],[-6,5],[2,2],[3,2],[7,2],[6,5],[2,2],[-3,2],[-8,0],[0,2],[3,0],[2,0],[1,1],[-6,5],[-2,0],[-1,-1],[-3,-2],[-2,-2],[-1,-3],[1,0],[1,-1],[2,-1],[0,-1],[0,-3],[-2,-1],[-15,0],[-4,2],[-1,2],[-1,2],[1,3],[1,1],[2,1],[1,2],[0,3],[-3,4],[-4,2],[-3,1],[1,0],[1,-1],[0,-2],[-4,-1],[-2,-3],[-1,-5],[-2,-6],[4,-4],[2,-2],[0,-1],[-1,-5],[0,-3],[-3,0],[-15,-8],[-18,-3],[-4,0],[-3,3],[-5,8],[-1,2],[-3,-9],[-2,-4],[-3,-2],[-16,-2],[-11,1],[-20,-4],[-3,1],[-1,-1],[0,-1],[-1,-1],[-25,0],[-8,4],[0,11],[2,4],[1,2],[2,1],[30,10],[-4,2],[-4,-1],[-17,-6],[-4,0],[-4,3],[7,16],[5,13],[2,5],[3,4],[12,5],[9,1],[8,5],[18,7],[6,0],[0,2],[-3,-2],[-8,2],[-5,-1],[-2,1],[1,4],[-5,-5],[-41,-11],[-3,1],[-2,3],[-5,9],[-8,9],[-4,3],[-2,-5],[1,0],[0,-3],[-1,-2],[0,-3],[0,-3],[1,-2],[1,-1],[2,-3],[1,-5],[-2,-5],[-3,-7],[-3,-10],[-1,-2],[-4,-6],[-4,-4],[-7,1],[-6,3],[-5,5],[0,3],[0,1],[-1,2],[-15,4],[-2,3],[-4,-1],[-5,-3],[-3,-4],[0,-2],[2,-1],[2,0],[4,1],[11,-2],[3,-2],[2,-2],[2,-7],[4,-4],[3,-2],[3,-3],[-1,-6],[-2,-5],[-6,-7],[-3,-4],[-4,-6],[-29,-11],[-29,-12],[-12,-1],[-11,4],[-2,2],[-2,3],[-1,3],[-1,6],[-2,5],[0,1],[2,4],[1,1],[2,2],[1,1],[-1,2],[0,4],[-2,6],[-4,6],[-3,2],[1,-4],[-2,-4],[-15,-11],[-5,-1],[-8,6],[-15,6],[-2,0],[0,-2],[2,-3],[2,-1],[3,-1],[2,-1],[-2,-3],[2,-5],[4,-5],[2,-5],[-1,0],[0,-6],[-4,-5],[-5,-4],[-4,-2],[-6,0],[-24,6],[-11,7],[-16,4],[-10,4],[0,2],[13,4],[6,4],[5,7],[-5,0],[-21,-7],[-4,1],[-4,3],[7,6],[4,5],[0,4],[0,3],[0,2],[0,2],[-1,1],[-24,-7],[-6,1],[-3,0],[-6,-4],[-1,-2],[-7,-8],[-2,-2],[-9,0],[-14,5],[-7,5],[-1,0],[-3,-3],[-3,0],[-56,12],[-9,7],[-6,2],[-3,2],[-2,4],[2,2],[1,1],[1,1],[-1,1],[-2,3],[3,3],[3,1],[7,1],[-2,5],[3,5],[28,24],[12,6],[42,0],[43,-1],[9,3],[25,-3],[12,2],[51,25],[37,3],[-2,4],[0,2],[1,3],[2,3],[16,9],[10,1],[3,1],[-6,3],[-36,-12],[-37,-3],[-9,-5],[-23,-4],[-14,-6],[-1,-2],[-41,0],[-41,0],[-41,0],[-4,2],[-2,6],[3,6],[30,31],[9,6],[41,4],[40,4],[41,3],[2,2],[1,3],[2,2],[2,-1],[2,-1],[16,-5],[40,4],[40,4],[5,3],[3,3],[-1,3],[-2,1],[-3,1],[-48,-3],[-47,-3],[-7,2],[-50,-7],[-2,1],[0,1],[-3,-1],[-52,7],[-2,2],[-1,3],[-1,4],[1,4],[2,2],[11,3],[7,1],[8,-2],[3,2],[-2,3],[-5,3],[-1,2],[1,1],[3,3],[-6,2],[-13,-4],[-5,5],[0,2],[0,2],[2,4],[5,7],[0,1],[1,3],[3,1],[2,0],[2,1],[2,3],[2,3],[1,2],[41,10],[25,0],[12,-4],[19,-1],[25,2],[33,-6],[13,0],[0,-2],[11,2],[13,-4],[14,1],[5,3],[-5,2],[-10,-1],[-9,2],[-11,0],[-8,7],[-27,7],[-28,0],[-28,0],[-11,4],[-2,2],[-3,5],[-1,1],[2,3],[7,6],[-2,2],[-4,3],[-1,1],[4,4],[41,22],[55,6],[5,1],[9,8],[3,1],[20,0],[-2,-3],[-3,-2],[-2,-2],[35,2],[28,-7],[5,-3],[3,-3],[2,-4],[3,-10],[-3,-5],[1,-1],[-2,-2],[-2,0],[-4,0],[1,-3],[2,-1],[4,1],[0,-1],[-3,-4],[1,-1],[1,-1],[-1,0],[6,-5],[8,-5],[3,-4],[-2,-3],[-4,-3],[-3,-3],[4,-2],[4,-3],[3,-2],[10,4],[18,0],[17,7],[5,1],[6,0],[3,0],[5,4],[5,2],[15,0],[5,-2],[2,-1],[19,5],[9,-2],[5,-3],[2,-6],[2,-2],[9,1],[0,-3],[0,-1],[15,0],[9,-4],[9,-2],[3,-3],[1,0],[-3,-2],[-3,-1],[-3,-3],[1,-5],[-3,-1],[1,-2],[8,-8],[1,-2],[0,-2],[2,-3],[6,-4],[28,-5],[30,-14],[9,-2],[3,-2],[2,-5],[-6,-4],[-33,-6],[-13,-6],[3,-2],[3,-2],[-8,-3],[-2,-2],[3,-1],[4,0],[8,3],[2,1],[1,2],[1,1],[2,1],[2,0],[3,-2],[2,0],[-2,-1],[-3,-1],[-2,-1],[10,-3],[2,0],[5,4],[2,1],[46,3],[2,-1],[-1,-2],[-2,-3],[-2,-1],[-10,2],[-2,-2],[15,-6],[4,-5],[-1,-1],[-1,-1],[-1,-2],[1,-2],[1,-2],[5,1],[3,-1],[0,-5],[-1,-2],[-3,-2],[-1,-5],[1,0],[1,-2],[1,-1],[1,-3],[-1,-2],[-3,-6],[2,-1],[1,-1],[4,-7],[1,-1],[5,-1],[0,-1],[-1,-1],[-1,-1],[-1,-3],[3,-1],[2,-1],[0,-1],[0,-1],[0,-1],[3,-1],[2,0],[3,1],[22,1],[18,6],[38,4],[10,-3],[0,-1],[0,-1],[0,-2],[1,-1],[23,-1],[7,3],[4,-2],[6,2],[7,-1]],[[3085,8495],[17,-9],[5,-1],[3,-1],[3,-4],[3,-5],[1,-6],[-4,-2],[-8,-7],[-5,-2],[-4,-4],[-2,-2],[-2,0],[-18,5],[-33,1],[-34,0],[-10,4],[-35,3],[-2,0],[-1,2],[-2,-1],[-1,1],[2,9],[2,1],[24,13],[22,5],[22,-1],[5,3],[39,2],[13,-4]],[[2815,8637],[4,-3],[22,-4],[5,-3],[5,1],[3,-1],[13,-10],[10,-2],[9,-6],[5,-1],[0,-1],[0,-1],[-1,-1],[0,-1],[2,-1],[4,-3],[1,-2],[-7,0],[-1,-1],[-3,-2],[-1,-1],[-1,0],[-1,-3],[0,-1],[-2,-2],[-16,-1],[-6,-3],[-5,1],[-4,-3],[-4,-2],[-6,-4],[-3,-1],[-36,-4],[-3,-2],[-1,-3],[0,-1],[0,-2],[-2,-4],[-8,0],[-4,-2],[3,-3],[8,-2],[2,-2],[0,-1],[0,-3],[1,-3],[3,-6],[4,-4],[44,-16],[6,-1],[3,-1],[2,-4],[-2,-4],[-18,-8],[-13,0],[-26,10],[-8,1],[-5,-3],[1,-6],[5,-4],[6,-3],[10,-9],[15,-3],[5,-8],[0,-3],[0,-2],[0,-2],[4,-3],[6,-10],[2,-3],[-2,-1],[-8,-1],[-2,-2],[-1,-1],[-1,-1],[0,-2],[0,-1],[-4,-5],[-4,-3],[-27,-11],[-14,-1],[-5,-2],[-3,-1],[-27,1],[-3,-1],[-2,-1],[-3,-4],[-2,-1],[-13,3],[-5,-1],[-8,-6],[0,-1],[1,-4],[1,-2],[2,-2],[2,-1],[2,-1],[-1,-2],[3,-2],[0,-3],[-1,-3],[0,-4],[5,-5],[1,-3],[0,-3],[0,-1],[1,-3],[0,-1],[-2,-2],[-2,-2],[-2,-1],[-2,0],[-2,-1],[-13,-10],[-16,-2],[-6,-5],[-2,0],[-25,3],[-8,4],[-2,1],[1,6],[-3,2],[-16,0],[-4,1],[-3,4],[2,0],[-16,13],[-6,2],[0,1],[1,2],[2,2],[-1,1],[-1,3],[7,5],[-3,3],[-1,1],[-1,0],[2,5],[12,7],[-1,3],[-4,5],[1,1],[3,1],[1,2],[-6,2],[-2,2],[5,5],[4,5],[-1,1],[-1,2],[-1,3],[0,3],[1,3],[2,2],[4,2],[11,12],[3,5],[-4,8],[-5,3],[-5,0],[-5,-4],[-6,-6],[-2,-2],[-4,-2],[-4,0],[-33,1],[0,-1],[3,-3],[1,-3],[0,-5],[-1,-2],[-2,-1],[-19,3],[1,-2],[1,-1],[1,-2],[1,0],[0,-1],[-1,-2],[0,-1],[5,-3],[5,-2],[4,-4],[3,-14],[2,-5],[0,-3],[-3,-1],[-12,1],[-5,-2],[-1,-1],[-1,-2],[1,-1],[-1,-2],[-1,-2],[-3,-2],[-16,1],[-2,3],[-4,5],[-3,2],[-4,2],[-4,0],[-3,-1],[2,-2],[2,-1],[5,-2],[-6,-1],[-7,0],[-7,-2],[-3,-7],[4,-2],[11,-1],[1,-1],[1,-2],[1,-1],[0,-1],[2,-1],[8,1],[3,-1],[3,-2],[3,-3],[6,-11],[0,-3],[-2,-2],[5,-4],[0,-2],[-1,-2],[-3,-3],[-1,-3],[-4,-5],[-5,-1],[-19,0],[-4,-2],[-4,-6],[0,-5],[0,-6],[2,-4],[3,-1],[-1,-3],[-5,0],[-2,-2],[-1,-3],[1,-2],[0,-2],[-3,-2],[-6,0],[-2,0],[-3,-3],[0,-1],[1,-2],[0,-1],[0,-1],[-1,-1],[-1,-1],[-3,2],[-5,3],[-2,1],[-5,0],[-2,0],[-3,2],[-2,2],[-2,5],[0,9],[-2,3],[-1,2],[-4,5],[-2,2],[-7,4],[-2,3],[-1,3],[-1,19],[-1,1],[-1,1],[-3,-1],[0,-1],[0,-2],[-1,-2],[-5,-4],[-11,4],[-5,-1],[0,-2],[2,0],[3,-1],[2,-2],[0,-4],[0,-5],[-2,-1],[-4,-1],[2,-3],[11,-13],[2,-1],[0,-2],[1,-1],[0,-1],[1,-2],[-1,-2],[-6,-1],[-6,1],[-3,0],[3,-6],[5,-1],[5,-1],[4,-2],[-3,-3],[-26,1],[2,-2],[3,-2],[9,-1],[18,-10],[4,-4],[1,-7],[-1,-8],[-2,-2],[-4,2],[-6,5],[-2,1],[-2,-1],[-3,-2],[-5,-10],[6,-2],[3,-2],[2,-3],[-2,0],[-7,2],[-2,-1],[-1,-1],[-3,-4],[-4,-4],[-10,-6],[-21,-4],[-4,3],[0,2],[-1,3],[-1,2],[-1,1],[-2,1],[-2,2],[-2,0],[-2,-3],[-3,-5],[-3,-5],[-3,-3],[-4,-3],[-5,-2],[-6,-1],[-5,0],[-5,5],[-1,4],[0,4],[3,8],[0,3],[0,2],[0,2],[1,2],[2,3],[4,4],[1,2],[-6,2],[-1,1],[1,2],[1,2],[0,2],[-3,2],[-13,-2],[0,-1],[1,-2],[1,-1],[-4,-1],[-5,1],[-5,4],[-4,5],[-2,6],[3,5],[7,7],[-3,1],[-7,-3],[-2,2],[1,5],[3,2],[4,2],[2,1],[-3,4],[-12,4],[-6,5],[-2,1],[-1,-1],[-8,-4],[-5,-4],[-2,-6],[1,-9],[-1,-1],[-1,-3],[-1,-3],[0,-3],[5,-7],[1,-2],[1,-3],[1,-4],[1,-4],[1,-2],[0,-1],[-2,-2],[-2,-2],[-2,-1],[-2,1],[-1,4],[1,3],[3,2],[-6,4],[-9,-1],[-10,-4],[-6,-4],[-1,-3],[-2,-8],[-2,-2],[-1,0],[-5,5],[-7,1],[-2,2],[1,0],[1,3],[-4,2],[-2,2],[-1,2],[-2,1],[-6,1],[-15,7],[-16,2],[-3,4],[-22,-3],[-6,2],[-3,0],[-3,-2],[1,0],[1,-1],[-1,-5],[-1,-3],[1,-1],[1,0],[0,-1],[0,-1],[0,-1],[-1,-1],[-6,1],[-20,-10],[-6,-1],[-22,2],[-13,9],[5,1],[5,1],[5,4],[5,6],[-2,-1],[-4,-2],[-2,-1],[-8,2],[3,2],[8,1],[8,7],[0,2],[3,3],[2,5],[0,3],[-4,2],[-21,-2],[2,2],[22,5],[-2,1],[-2,1],[-5,-1],[0,5],[-4,1],[-7,1],[-11,-6],[-4,-1],[-3,1],[-1,0],[0,-3],[-1,-2],[-5,-6],[-4,-2],[-4,-2],[-9,-1],[2,2],[4,1],[1,2],[-2,3],[-1,1],[6,2],[0,2],[1,3],[1,1],[2,0],[1,0],[-1,2],[-1,1],[-1,2],[-1,0],[2,3],[4,-2],[4,0],[3,13],[4,1],[6,-2],[4,0],[-2,0],[2,4],[2,2],[5,2],[1,2],[0,3],[-1,2],[0,2],[1,6],[2,4],[3,3],[16,6],[5,6],[3,1],[47,7],[21,-3],[26,3],[4,2],[1,2],[0,1],[0,1],[0,1],[1,0],[8,8],[2,3],[-1,10],[5,4],[6,2],[1,4],[6,3],[3,3],[2,5],[0,5],[1,4],[2,2],[3,1],[2,0],[4,-4],[2,-1],[6,3],[13,3],[12,9],[30,6],[3,3],[1,3],[17,8],[-1,0],[-1,2],[-1,1],[1,1],[1,2],[1,0],[-2,1],[-2,2],[-2,2],[-1,2],[6,6],[16,2],[7,7],[-1,1],[-1,1],[0,2],[1,1],[7,3],[3,0],[-2,6],[2,4],[3,2],[3,1],[8,-1],[4,1],[3,5],[-2,2],[2,1],[6,1],[2,2],[0,3],[0,3],[1,2],[12,4],[8,1],[2,1],[0,5],[4,2],[19,7],[5,3],[1,3],[4,8],[1,2],[1,1],[4,9],[2,2],[4,2],[4,3],[4,2],[4,1],[18,-2],[10,4],[2,1],[3,4],[2,1],[4,-2],[3,0],[6,2],[30,0],[29,-1],[9,5],[28,3],[0,-2],[-3,0],[-6,-3],[-3,0],[3,-2],[15,2],[2,0],[1,-2],[2,-4],[1,-1],[2,0],[6,0],[2,0],[5,-3],[16,1],[1,0],[0,-3],[1,-1],[2,-1],[7,1],[6,-3],[7,-4],[3,-1],[12,4],[10,-2],[3,2],[-6,8],[-3,2],[-5,-1],[-2,0],[-2,1],[-1,2],[1,2],[2,1],[14,-2],[19,-9],[6,0],[4,3],[-1,0],[-3,3],[5,3],[10,4],[4,5],[-4,1],[-8,1],[-3,-1],[-6,-3],[-3,0],[0,5],[-10,-2],[-6,1],[-2,3],[-2,4],[-14,7],[3,2],[2,2],[5,1],[0,-3],[1,-1],[2,0],[4,2],[3,1],[5,-1],[2,1],[18,9],[-2,2],[-17,2],[3,2],[56,6],[-1,-1],[-2,-2],[10,-2],[-1,-3],[-1,0],[13,0],[3,-2],[13,-7],[5,-1],[-6,-2],[-1,-2]],[[3019,8768],[7,-6],[16,3],[6,-7],[-1,0],[9,-8],[2,-3],[3,-2],[24,-4],[4,-3],[1,-6],[2,-4],[8,-4],[3,-4],[-3,-1],[-14,-10],[-49,-15],[-16,1],[-20,10],[-16,1],[-4,3],[4,1],[12,-3],[-3,5],[-5,4],[-9,2],[-15,11],[-7,2],[-29,24],[2,1],[3,1],[3,0],[6,-2],[2,-1],[3,1],[5,3],[12,3],[3,1],[3,3],[7,8],[20,3],[13,7],[7,1],[5,-4],[-1,-3],[-7,-4],[2,-2],[8,-3],[-6,0]],[[3507,8748],[-14,-2],[-11,-6],[-35,-3],[-35,-4],[-7,-4],[3,-7],[12,-5],[2,-4],[4,-5],[35,2],[8,3],[24,-1],[2,-1],[3,-3],[0,-3],[-1,-1],[0,-3],[1,-1],[-3,-6],[-1,0],[1,-4],[1,-2],[4,-3],[-1,0],[0,-2],[-1,-1],[1,-1],[1,-1],[2,-1],[1,0],[-2,-3],[-2,-2],[-5,-2],[-3,-2],[-2,-2],[0,-2],[6,-1],[2,-1],[1,-3],[-1,-4],[-2,-3],[-6,-2],[-8,-7],[-38,-12],[-16,0],[-4,-2],[-5,0],[-1,-1],[-3,-4],[-6,-3],[-35,2],[-7,3],[-3,0],[-14,-7],[-55,-12],[-12,-6],[-13,-1],[-41,11],[-3,2],[-1,2],[-3,6],[-8,10],[-4,2],[-5,0],[-5,-3],[-6,0],[-17,4],[-3,1],[-1,3],[-1,5],[-2,2],[-5,1],[-15,0],[-3,3],[-1,4],[-1,4],[-1,3],[-2,2],[3,4],[10,4],[4,2],[-2,2],[-8,4],[1,0],[-2,2],[-5,2],[-3,1],[2,1],[2,2],[2,3],[-1,4],[-2,3],[0,1],[0,2],[2,1],[3,0],[-9,5],[-4,4],[-2,7],[1,8],[3,5],[8,7],[-3,4],[11,4],[7,0],[4,-3],[2,-3],[2,0],[3,0],[3,1],[1,2],[1,3],[2,2],[2,0],[-1,0],[36,10],[37,10],[5,-1],[6,0],[13,3],[12,0],[14,7],[3,0],[1,-3],[2,-2],[3,-1],[4,1],[2,1],[6,3],[13,5],[15,1],[13,4],[8,-1],[23,3],[0,-1],[0,-2],[1,-1],[2,-2],[13,-1],[4,2],[13,7],[40,-1],[40,0]],[[3507,8844],[-1,0],[-36,-10],[-29,3],[-29,4],[-2,2],[0,3],[-3,2],[-6,1],[-10,6],[-14,4],[-7,-2],[-8,-1],[-3,-2],[2,-3],[2,-2],[2,-2],[2,-1],[-4,-4],[-15,2],[-2,-5],[0,-1],[2,-1],[1,-2],[-1,-2],[-1,-2],[-2,0],[-39,-1],[-6,3],[-4,5],[-1,1],[-1,5],[-3,2],[-6,2],[-1,1],[-6,1],[-17,6],[-32,-6],[-33,-7],[-3,-1],[-1,-3],[2,-3],[-4,3],[-4,0],[-25,-8],[-14,0],[-7,2],[-5,8],[1,2],[-6,3],[13,6],[0,3],[0,2],[0,2],[-1,2],[2,2],[7,1],[2,1],[4,3],[36,11],[11,6],[22,3],[15,7],[23,4],[13,-1],[13,2],[12,-3],[45,14],[5,4],[-7,5],[14,1],[2,3],[2,2],[13,7],[3,-1],[18,6],[15,3],[6,3],[4,2],[2,2],[1,-1],[1,0],[1,-1],[32,4],[4,-4],[3,-2],[15,-4],[11,-7],[12,-1],[3,-2]],[[220,4422],[-6,8],[-8,10],[0,6],[4,15],[4,16],[5,18],[0,5],[-1,5],[-3,1],[-18,-1],[-17,-1],[-12,-1],[-12,0],[-6,-2],[-6,-4],[-11,-13],[-10,-12],[-6,-4],[-5,-1],[-17,10],[-18,10],[-11,6],[-3,0],[-2,-1],[-1,-3],[-4,-10],[-1,-3],[-2,-1],[-14,5],[-12,5],[-16,6],[-8,3],[-2,2],[0,4],[-1,35],[0,35],[0,35],[0,35],[0,36],[0,35],[0,35],[0,35],[0,35],[0,35],[0,36],[0,35],[0,35],[0,35],[0,35],[0,35],[0,36],[0,35],[0,35],[0,35],[0,35],[0,35],[0,36],[0,35],[0,35],[0,35],[0,35],[0,35],[0,36],[0,35],[0,35],[0,35],[0,35],[0,35],[0,36],[0,35],[0,35],[0,35],[0,35],[0,35],[0,36],[0,35],[0,35],[0,35],[0,35],[0,35],[0,36],[0,35],[0,35],[0,35],[0,35],[0,35],[0,36],[0,35],[0,35],[0,35],[0,35],[0,35],[0,36],[0,35],[0,35],[0,35],[0,35],[0,35],[2,-1],[10,-1],[0,-1],[-1,-1],[2,-1],[2,-1],[4,1],[0,1],[-1,0],[3,1],[20,-5],[27,-3],[21,-1],[7,5],[27,0],[35,-9],[1,-1],[-13,2],[-7,3],[3,-3],[6,-3],[38,-7],[5,-6],[3,-1],[17,-1],[4,-3],[6,-11],[2,-3],[5,-5],[4,-3],[21,-12],[5,-7],[7,-7],[3,-4],[3,-9],[1,-1],[1,0],[1,1],[1,1],[1,1],[2,1],[5,-1],[0,-1],[-3,0],[2,-2],[3,-1],[4,-1],[1,0],[0,1],[0,1],[0,1],[0,1],[3,1],[1,3],[0,4],[1,3],[-2,0],[-1,-1],[-3,-2],[3,4],[4,2],[4,-1],[1,-4],[1,-7],[1,-4],[2,-3],[9,-11],[15,-11],[26,-11],[40,-18],[10,-1],[5,-3],[-6,0],[0,-2],[15,-5],[9,-5],[4,-1],[14,2],[8,-2],[1,-1],[1,-3],[1,-1],[7,-2],[12,-7],[6,-3],[9,-1],[3,1],[1,1],[0,2],[2,2],[1,0],[9,-1],[-1,1],[-1,0],[0,2],[5,2],[4,-1],[2,0]],[[242,6736],[-4,-3],[-10,2],[-4,0],[-3,-3],[-4,-6],[-3,-3],[-3,0],[-4,1],[-4,3],[-3,3],[-1,1],[0,3],[-2,1],[-1,-1],[-2,-2],[-1,-1],[-2,0],[-2,0],[2,4],[11,11],[6,4],[6,1],[4,-1],[7,-4],[7,-2],[4,-2],[3,-2],[3,-4]],[[6955,2243],[0,2],[-2,5],[-14,15],[-4,8],[-3,7],[-2,9],[0,9],[2,9],[4,12],[2,8],[-1,6],[-1,3],[-1,4],[0,4],[1,3],[5,5],[2,1],[3,6],[2,1],[6,2],[1,2]],[[6955,2364],[0,-25],[0,-27],[0,-27],[0,-27],[0,-15]],[[6955,2237],[0,-7],[0,-27],[0,-27],[0,-27],[0,-27],[0,-28],[0,-27],[0,-27],[0,-27],[0,-27],[0,-27],[0,-28],[0,-27],[0,-27],[0,-27],[0,-27],[0,-28],[0,-27],[0,-27],[0,-27],[0,-27],[0,-27],[0,-28],[0,-27],[0,-27],[0,-27],[0,-27],[0,-27],[0,-13],[1,-14],[0,-6],[0,-6],[0,-6],[0,-7],[0,-7],[0,-8],[0,-7],[-2,-7],[-2,-6],[-2,-7],[-1,-8],[1,-8],[2,-6],[3,-7],[2,-6],[3,-5],[1,-3],[2,-4],[1,-4],[1,-4],[0,-8],[0,-7],[-1,-8],[0,-7],[2,-11],[4,-9],[4,-8],[4,-9],[3,-12],[5,-10],[5,-8],[5,-7],[8,-29],[2,-7],[5,-5],[2,-4],[1,-5],[0,-5],[1,-6],[3,-10],[4,-8],[5,-7],[15,-16],[2,-4],[1,-5],[1,-4],[3,-3],[31,-10],[5,0],[3,-1],[2,-4],[1,-2],[3,0],[6,2],[15,-3],[7,-6],[14,0],[23,-10],[4,-2],[3,-1],[22,-16],[6,-9],[2,-2],[2,-2],[2,-1],[1,-3],[2,-2],[3,-4],[2,-5],[0,-4],[0,-5],[1,-4],[1,-4],[2,-5],[3,-5],[3,-4],[3,-3],[4,-5],[5,-4],[4,-3],[5,-3],[2,-2],[4,0],[3,1],[2,4],[0,3],[-1,4],[0,4],[0,3],[0,4],[2,1],[1,1],[2,1],[3,-1],[4,-2],[3,-2],[1,-4],[0,-4],[1,-11],[0,-4],[1,-5],[1,-5],[2,0],[2,-2],[1,-2],[1,-4],[0,-4],[1,-7],[1,-3],[2,-4],[0,-3],[2,-7],[1,-2],[1,-2],[1,-2],[3,-2],[11,-5],[5,-2],[2,-1],[1,0],[2,-3],[1,0],[0,-2],[2,-1],[2,-4],[1,-1],[2,-1],[3,-1],[2,0],[1,1],[3,2],[1,2],[1,1],[1,3],[0,1],[1,1],[1,1],[7,2],[3,0],[9,-3],[3,-2],[2,-5],[1,0],[3,-3],[1,-1],[3,-4],[1,-3],[0,-2],[2,-1],[3,-7],[2,-2],[1,-1],[2,0],[7,2],[2,1],[1,2],[2,7],[3,4],[3,4],[3,1],[7,1],[26,16],[6,6],[3,2],[26,4],[3,3],[4,4],[3,2],[3,1],[18,-1],[7,2],[2,0],[1,1],[1,0],[1,2],[1,1],[2,0],[3,-2],[4,-5],[4,-5],[5,-2],[5,-1],[7,-5],[0,-6],[-1,-6],[-2,-10],[-2,-13],[-2,-11],[-1,-9],[-1,-7],[3,-5],[3,-4],[4,-7],[4,-6],[1,-2]],[[7541,856],[-2,-3],[-1,-1],[-1,0],[-3,-3],[-1,-1],[-3,-1],[-1,-1],[-3,-3],[-2,-4],[-2,-5],[0,-5],[-2,-3],[-2,-3],[-12,-9],[-7,-11]],[[7499,803],[-3,0],[-14,3],[-2,0],[-2,-1],[-4,-4],[-7,-4],[-2,-2],[-5,-6],[-2,-1],[-6,-2],[-16,-15],[-8,-12],[-13,-18],[-14,-19],[-13,-19],[-6,-13],[-5,-8],[-4,-8],[-2,-6],[-1,-4],[-2,-3],[-3,-2],[-19,-13],[-9,-10],[-3,-6],[0,-2],[1,-6],[0,-1],[-2,-4],[-3,-2],[-12,-8],[-1,-1],[-1,-2],[-1,-2],[0,-2],[-1,-1],[-9,-12],[-3,-4],[-5,-14],[-4,-13],[-5,-13],[-5,-13],[-5,-14],[-5,-13],[-5,-13],[-4,-13],[-1,-1],[-1,0],[0,-1],[-1,-1],[-1,0],[0,-1],[-1,0],[0,-1],[-27,0],[-26,0],[-26,1],[-26,0],[-26,0],[-26,0],[-26,0],[-26,1],[-7,-4],[-23,-17],[-15,-12],[-10,-8],[9,-29],[4,-16],[0,-4],[1,-22],[0,-2],[0,-3],[-1,-3],[-1,-1],[0,-3],[0,-3],[0,-3],[1,-2],[2,-3],[1,-2],[1,-2],[0,-2],[-1,-4],[0,-5],[1,-2],[1,-3],[3,-4],[3,-2],[3,-2],[2,-3],[1,-4],[-1,-10],[-5,-11],[-7,-10],[-17,-13],[-18,-13],[-17,-13],[-17,-13],[-15,-11],[-3,-2],[-17,-14],[-17,-13],[-18,-13],[-14,-5],[-15,-4],[-2,-1],[-12,-4],[-15,-5],[-14,-5],[-15,-4],[-15,-5],[-14,-5],[-16,-16],[-17,-16],[-16,-16],[-16,-16],[-16,-16],[-17,-16],[-16,-16],[-16,-16],[-3,-1],[-24,-1],[-5,3],[-18,17],[-13,14],[-10,9],[-3,7],[-3,11],[-1,4],[0,1],[-3,14],[0,11],[2,17],[1,20],[0,10],[2,8],[4,7],[4,7],[11,6],[13,10],[3,5],[12,24],[6,9],[4,1],[4,3],[3,4],[2,7],[2,7],[3,17],[1,11],[1,4],[0,3],[-1,1],[-1,5],[0,4],[2,7],[0,5],[2,9],[2,7],[2,8],[0,8],[1,8],[3,15],[4,16],[3,15],[4,15],[4,15],[3,15],[4,15],[4,15],[1,6],[0,6],[-1,3],[-5,50],[-6,51],[-5,51],[-5,50],[-6,51],[-5,51],[-6,51],[-5,50],[-2,10],[-5,9],[-19,18],[-29,28],[-21,19],[-22,21],[-13,13],[-9,9],[4,12],[5,16],[5,14],[1,4],[-2,3],[-2,4],[-2,2],[-6,11],[-6,6],[-5,1],[-12,-3],[-6,2],[-2,1],[-2,-1],[-2,-2],[-6,-10],[-2,-2],[-2,1],[-2,2],[-2,7],[-1,3],[-1,1],[-1,8],[-4,5],[-3,7],[-1,6],[0,3],[0,3],[-1,7],[-1,4],[-1,5],[1,3],[1,5],[-3,6],[-2,6],[-1,8],[5,23],[-4,3],[-6,2],[-5,-5],[-7,-6],[-5,5],[-4,-1],[-4,-2],[-3,-5],[-2,-3],[-1,-3],[-2,-2],[-2,-1],[-3,0],[-3,2],[-3,3],[-12,22],[-9,17],[-1,5],[-4,22],[-2,14],[-2,14],[-1,5],[-2,3],[-1,2],[-2,2],[-2,2],[-2,1],[-2,2],[-2,2],[-2,2],[-1,1],[-14,13],[-15,12],[-14,13],[-14,12],[-14,13],[-14,12],[-14,13],[-14,12],[-12,11],[-12,10],[-12,10],[-12,10],[-12,10],[-12,11],[-11,10],[-12,10],[-21,18],[-22,18],[-21,17],[-21,18],[-21,18],[-22,18],[-21,18],[-21,17],[-3,2],[-4,0],[-19,-8],[-7,-4],[-16,-13],[-14,-11],[-22,-18],[-17,-15],[-10,-9],[-3,-1],[-3,1],[-8,3],[-4,0],[-11,0],[-7,3],[-3,0],[-4,-1],[-1,1],[-2,2],[-2,0],[-2,0],[-3,0],[-11,-7],[-2,0],[-7,6],[-1,2],[-2,2],[-6,15],[-1,3],[-6,3],[-6,0],[-33,-6],[-1,0],[-1,1],[0,1],[-2,-1],[-3,-2],[-1,0],[-3,1],[-1,2],[-1,2],[-3,-1],[-10,-4],[-6,0],[-4,4],[-1,4],[-1,7],[-1,3],[-1,2],[-1,1],[-1,1],[-1,3],[0,2],[1,3],[0,2],[-4,4],[-5,-2],[-10,-8],[-2,-2],[-5,-3],[-15,-16],[-4,-6],[-4,-4],[-10,-2],[-8,-5],[-2,1],[-2,2],[-2,1],[-10,-1],[-2,1],[1,2],[0,2],[0,3],[-1,2],[-1,0],[-2,0],[-8,2],[-3,3],[2,6],[-3,8],[-2,3],[-3,2],[-10,2],[-11,7],[-2,2],[-2,3],[-2,4],[-2,6],[-1,6],[-1,5],[-3,3],[-10,1],[-12,-3],[-2,-2],[-3,-5],[1,-5],[1,-4],[0,-5],[-2,-3],[-2,-2],[-3,-2],[-2,0],[-2,2],[-4,12],[-3,8],[-2,4],[0,6],[0,9],[-1,4],[-2,6],[-4,3],[-5,0],[-10,-2],[-2,1],[-3,1],[-2,3],[1,6],[3,1],[5,2],[2,2],[-1,4],[-2,4],[-8,3],[-18,7],[-8,8],[-3,1],[-17,2],[-15,1],[-14,-3],[-4,-3],[-3,-2],[-1,-2],[0,-4],[0,-3],[-4,-4],[-8,-1],[-4,-3],[-2,1],[-16,-3],[-2,1],[-4,5],[-3,18],[-4,5],[-18,2],[-23,2],[-2,1],[0,1],[0,5],[-1,2],[-1,2],[-1,1],[-1,1],[-1,1],[-2,1],[-19,-4],[-4,1],[-9,6],[-10,12],[-2,3],[-1,4],[0,4],[0,5],[0,4],[0,13],[-5,27],[-4,24],[-1,5],[-1,5],[0,10],[0,5],[-1,5],[-1,10],[-1,4],[-2,2],[-8,7],[-7,2],[-13,0],[-7,4]],[[5889,3663],[3,0],[2,2],[1,0],[2,0],[0,-1],[1,-4],[1,0],[2,0],[2,-1],[1,-2],[-3,-4],[4,-5],[9,-9],[4,-6],[3,-6],[2,-2],[5,-2],[7,-6],[2,0],[3,-3],[3,-5],[4,-6],[3,-4],[1,0],[1,-1],[1,-1],[1,-2],[1,-1],[6,-7],[24,-15],[6,-2],[6,-4],[2,-2],[0,-3],[0,-1],[2,-8],[2,-3],[10,-14],[15,-31],[0,-1],[1,-4],[0,-3],[2,-2],[2,-2],[3,-4],[3,-8],[2,-2],[1,0],[1,-1],[-1,-4],[0,-2],[-3,-6],[0,-3],[0,-4],[2,3],[1,4],[2,3],[3,2],[3,-1],[3,-1],[2,-2],[-1,-5],[3,-1],[9,0],[3,-2],[1,-4],[-2,-9],[6,3],[29,-8],[9,1],[3,-1],[-1,-1],[0,-1],[0,-1],[-1,0],[2,-2],[6,2],[3,-1],[6,-6],[4,-2],[5,-1],[2,-1],[4,-5],[12,-8],[23,-13],[10,-9],[27,-10],[9,-2],[7,-3],[4,-4],[5,-2],[3,-3],[4,-2],[4,-2],[4,-1],[1,-4],[4,-5],[4,-4],[4,-4],[3,-7],[3,-4],[3,-4],[7,-11],[13,-9],[7,-4],[12,-3],[3,-3],[1,-5],[-1,-6],[-2,-2],[-3,-1],[-3,-1],[-1,-3],[0,-4],[-1,-2],[-2,-2],[-3,-2],[-1,-2],[-2,-4],[-2,-9],[-2,-4],[-7,-10],[-1,-4],[0,-1],[1,-4],[0,-2],[0,-1],[-2,-6],[-2,-7],[0,-2],[2,1],[2,4],[2,4],[4,16],[1,5],[14,29],[5,6],[6,4],[6,2],[12,0],[37,-12],[7,1],[8,5],[5,1],[2,2],[22,4],[4,-1],[7,-3],[4,-1],[2,0],[14,11],[3,1],[3,-1],[4,-4],[11,-6],[4,-5],[1,-4],[-1,-3],[0,-2],[2,0],[0,1],[1,4],[1,1],[1,0],[2,-1],[1,-2],[1,-2],[2,0],[1,1],[2,0],[1,-2],[1,-2],[0,-2],[1,-2],[1,-1],[1,-1],[-1,-2],[-2,-3],[0,-3],[1,-2],[1,-2],[1,-2],[1,2],[-2,3],[1,3],[1,2],[0,3],[-2,4],[-1,3],[0,4],[-1,2],[-3,1],[-5,1],[-1,1],[-2,2],[-3,4],[3,2],[13,-4],[8,-6],[2,-1],[5,1],[2,-1],[3,-2],[13,-2],[5,0],[1,0],[2,-2],[2,-3],[1,0],[3,0],[8,6],[5,3],[8,0],[6,-5],[2,-11],[0,-1],[-1,-1],[-1,-2],[-1,-3],[0,-2],[-3,-6],[3,1],[3,8],[3,3],[1,0],[4,-3],[2,-1],[6,0],[2,0],[8,6],[3,1],[3,-1],[6,-5],[3,-1],[0,2],[-1,2],[0,2],[2,-1],[2,-1],[1,-3],[1,-1],[3,0],[1,-1],[1,-2],[2,-2],[1,-1],[1,-1],[1,-1],[0,-1],[-1,-1],[0,-1],[1,-2],[1,-2],[5,-1],[2,0],[2,-2],[-1,3],[-2,5],[4,4],[-1,3],[-2,4],[-2,5],[3,-2],[3,-7],[5,-4],[1,-4],[-1,-5],[-3,-8],[0,-4],[1,-4],[2,-7],[0,-2],[0,-1],[0,-3],[-1,-1],[-1,-1],[0,-1],[0,-2],[0,-1],[1,-1],[1,-9],[0,-1],[2,-1],[2,-1],[1,-1],[2,-5],[-1,-5],[-6,-29],[-7,-33],[-7,-24],[-2,-12],[-2,-7],[0,-2],[-1,-2],[-2,-3],[0,-1],[0,-7],[0,-1],[1,-1],[1,-3],[-1,-5],[0,-16],[1,-4],[0,-3],[2,-3],[4,-9],[0,-3],[-1,-3],[1,-1],[1,0],[2,1],[1,0],[1,-1],[2,-1],[6,-9],[2,-4],[7,-35],[2,-5],[1,-2],[1,-13],[1,-3],[1,-3],[-1,-6],[-1,-10],[-1,-3],[-4,-12],[-2,-3],[0,-5],[-1,-6],[-1,-5],[2,-3],[0,-4],[-1,-4],[0,-3],[0,-4],[1,-5],[1,-4],[4,-9],[1,-7],[4,-28],[0,-5],[0,-3],[-3,-4],[-5,-6],[-2,-1],[-2,-1],[-2,-1],[-1,-2],[0,-3],[-3,-14],[-2,-8],[-1,-19],[0,-3],[1,-3],[1,-3],[0,-4],[-2,-2],[3,-3],[4,-7],[5,-3],[1,-1],[1,-2],[1,-1],[5,-2],[2,-1],[7,-10],[6,-11],[2,-4],[-1,-3],[3,-5],[4,-17],[4,-3],[4,-2],[4,-6],[12,-24],[3,-5],[4,-3],[9,-11],[1,-3],[0,-3],[-2,-4],[1,-4],[1,-11],[2,-2],[1,-1],[5,-6],[1,-3],[-12,4],[-2,-1],[-13,-9],[-2,0],[-1,-4],[-1,-2],[-2,2],[-2,1],[-4,-1],[-3,-3],[-2,-3],[2,1],[1,1],[2,1],[0,-2],[-3,-4],[-4,-7],[3,0],[3,1],[3,3],[4,6],[3,2],[26,-1],[6,-5],[3,-7],[5,-15],[4,-6],[10,-3],[4,-5],[8,-6],[3,0],[2,2],[2,0],[2,-3],[3,-3],[6,-3],[2,-3],[1,-2],[2,-7],[3,-6],[0,-3],[1,-5],[3,-3],[5,-3],[1,0],[2,1],[0,-1],[1,0],[0,-1],[0,-1],[0,-1],[4,-4],[1,-2],[3,-5],[5,-5],[1,-3],[4,-9],[1,-2],[1,0],[2,-2],[2,-3],[1,-3],[0,-2],[0,-2],[0,-2],[1,-5],[3,-11],[1,-7],[4,-14],[1,-3],[2,-4],[2,-3],[2,-1],[1,-3],[1,-5],[0,-11],[1,-9],[0,-4],[-2,-1],[-3,-1],[-13,-8],[-5,-5],[-4,-6],[-4,-7],[-1,-2],[-1,-5],[-1,-2],[-1,-2],[-1,-1],[-2,-3],[-9,-5],[-2,-3],[-2,-3],[-3,-8],[-1,-2],[-6,-4],[-2,-2],[-2,-3],[-1,-4],[4,1],[2,1],[2,2],[1,0],[0,-2],[0,-2],[0,-1],[-3,-4],[1,-2],[4,5],[4,16],[3,7],[3,2],[6,0],[3,2],[4,4],[1,0],[1,1],[1,2],[1,2],[0,2],[8,8],[1,3],[1,4],[2,4],[5,6],[8,6],[5,6],[3,0],[15,-6],[3,-2],[3,-2],[7,1],[3,-2],[1,-1],[2,-4],[2,-1],[6,-2],[3,-1],[7,-12],[5,-6],[11,-8],[4,-6],[5,-14],[1,-3],[2,-1],[12,-15],[5,-11]],[[7587,878],[-1,-2],[-2,-2],[-2,-4],[-1,-1],[-1,-1],[-3,-1],[-2,-1],[-4,-1],[-7,-1],[-1,1],[-1,1],[-1,3],[0,2],[1,2],[1,2],[1,1],[1,2],[1,0],[9,-1],[1,1],[4,2],[1,1],[0,1],[1,0],[7,2],[0,-1],[0,-2],[-1,-1],[-1,-1],[0,-1]],[[7591,892],[-1,0],[-2,0],[-2,1],[-1,0],[0,2],[-2,3],[-1,2],[0,2],[0,1],[0,1],[1,0],[6,-2],[1,0],[1,-1],[1,-1],[1,-1],[1,0],[2,-1],[0,-1],[0,-1],[-1,-1],[-1,0],[-3,-2],[0,-1]],[[7589,922],[-1,0],[-1,0],[-1,1],[0,1],[0,1],[0,1],[1,1],[5,3],[1,0],[0,-3],[-2,-3],[-2,-2]],[[7629,971],[-1,-2],[-2,-2],[-2,-2],[-3,-5],[-1,-1],[0,-1],[0,-2],[-4,-11],[-3,-4],[-1,-2],[-2,-1],[-1,-1],[-4,-1],[-7,-1],[-2,1],[-1,1],[0,2],[0,2],[2,3],[2,5],[3,2],[5,10],[2,3],[3,3],[5,3],[1,1],[1,-1],[1,0],[1,0],[6,3],[1,0],[1,-1],[0,-1]],[[7632,930],[0,-8],[0,-1],[0,-1],[0,-1],[-1,0],[-2,-2],[-1,-1],[0,-1],[-1,-3],[-2,-2],[-1,0],[-1,0],[-11,6],[-9,0],[-2,-1],[-4,-3],[-8,-5],[-3,1],[-1,-1],[0,1],[-1,1],[-1,0],[0,1],[0,1],[1,1],[8,8],[2,3],[1,0],[1,5],[1,1],[1,1],[2,0],[2,0],[1,0],[1,0],[1,0],[1,1],[6,4],[3,3],[3,4],[2,6],[0,3],[2,5],[1,2],[1,2],[8,9],[2,3],[1,1],[1,1],[1,1],[2,1],[1,0],[0,-2],[-1,-4],[0,-3],[-2,-12],[-1,-5],[0,-2],[0,-1],[-2,-4],[-1,-2],[-1,-3],[0,-2],[0,-3],[0,-3]],[[7916,1253],[-5,-1],[-5,0],[-1,4],[5,7],[10,11],[8,13],[4,5],[5,2],[2,-1],[1,-1],[1,-3],[-1,-3],[-1,-3],[-4,-10],[-1,-2],[-2,-1],[-7,-9],[-9,-8]],[[7988,1376],[-2,-1],[-2,1],[-1,1],[0,2],[0,3],[1,2],[2,2],[2,2],[3,1],[3,0],[2,0],[-1,-2],[-5,-8],[-2,-3]],[[9007,1437],[-2,-2],[-3,-2],[-2,-2],[-1,-4],[-1,-4],[-4,-3],[-4,-1],[-2,2],[3,2],[3,4],[2,4],[2,5],[-4,1],[-3,-1],[-3,-2],[-3,-3],[-5,-8],[-2,-4],[-3,-1],[-3,-3],[0,-2],[-5,-10],[-3,-3],[-3,-2],[-5,-8],[-3,-2],[5,-2],[4,5],[13,25],[4,4],[5,1],[-4,-2],[-4,-5],[-6,-11],[-6,-18],[-1,-1],[-5,1],[-2,-2],[-1,-5],[-1,-4],[-1,-2],[-4,-1],[-2,-1],[-2,-2],[-1,-3],[-2,-7],[-1,-4],[1,-2],[2,-3],[3,0],[1,5],[1,-4],[2,-2],[2,-2],[4,-1],[2,0],[1,2],[2,3],[-1,-6],[-3,-3],[-3,-1],[-3,0],[0,3],[-3,-1],[-5,-3],[-5,-1],[-3,2],[0,5],[3,7],[2,8],[1,3],[1,4],[0,11],[1,5],[2,3],[3,2],[3,1],[2,1],[2,1],[15,23],[1,2],[15,21],[3,3],[3,1],[14,0],[0,-1],[0,-3]],[[8161,1621],[-2,0],[-1,0],[0,1],[0,1],[1,2],[2,1],[1,1],[1,1],[1,-1],[-1,-1],[0,-1],[0,-2],[-2,-2]],[[8390,1527],[-3,0],[-3,-2],[-3,-2],[-2,-3],[-1,-5],[0,-4],[1,-4],[-1,-3],[-2,-2],[-3,2],[-3,3],[-1,3],[-3,0],[-2,0],[-3,-2],[-2,-2],[-1,-1],[-1,-1],[-1,-2],[-2,-1],[-2,-1],[-1,0],[-2,0],[-2,1],[-2,1],[-1,0],[-1,-1],[-2,0],[-2,0],[-2,1],[-1,0],[-2,-1],[-1,-5],[-1,-5],[-1,-2],[-3,2],[-1,2],[0,1],[-1,2],[0,2],[-1,1],[-1,0],[-1,1],[-1,0],[-2,2],[-1,1],[-2,2],[-1,1],[-3,2],[-3,1],[-2,2],[-3,0],[-2,1],[0,2],[0,1],[0,3],[0,2],[0,3],[0,2],[0,2],[-2,0],[-7,0],[-7,0],[-7,0],[-7,0],[-6,0],[-7,0],[-7,0],[-7,0],[0,-2],[0,-2],[0,-3],[0,-2],[0,-2],[0,-2],[0,-2],[0,-2],[-4,0],[-4,0],[-3,0],[-4,0],[-4,0],[-4,0],[-3,0],[-4,0],[0,-9],[0,-9],[0,-9],[0,-9],[0,-9],[0,-9],[0,-9],[0,-9],[0,-5],[0,-5],[-1,-7],[-2,-6],[-4,-4],[-3,-4],[-3,-5],[-3,-6],[-4,-5],[-3,-3],[-5,-4],[-6,-3],[-5,-4],[-5,-3],[-8,-5],[-8,-3],[-8,-4],[-9,-5],[-1,-1]],[[8138,1356],[0,2],[0,8],[1,14],[1,7],[-2,3],[-17,7],[-2,0],[-2,-2],[-2,-3],[-10,-21],[-10,-22],[-10,-22],[-10,-21],[-11,-22],[-10,-22],[-10,-21],[-10,-22],[-2,-4],[-2,-11],[-1,-12],[-2,-24],[-1,-16],[-2,-6],[-9,-11],[-1,-1],[-3,-3],[-1,-5],[0,-5],[0,-2],[-3,-2],[-1,-3],[-2,-9],[-3,-9],[-1,-5],[3,-2],[1,-5],[1,-5],[-1,-5],[-1,-3],[-4,-5],[-2,-2],[0,-2],[1,-1],[0,-1],[1,-1],[0,-1],[0,-1],[0,-1],[-1,-2],[0,-1],[-1,-8],[0,-4],[1,-4],[3,-2],[2,-1],[1,-2],[-1,-6],[-3,-8],[-3,-5],[-9,-10],[-2,-4],[0,-1],[-1,-3],[1,-1],[0,-2],[1,-2],[0,-6],[-1,-4],[-1,-3],[-2,-3],[-1,-1],[-3,0],[-1,-1],[-4,-4],[-6,-5],[-2,-2],[-11,-19],[-5,-16],[0,-4],[1,-5],[1,-3],[5,-5],[2,-2],[1,-5],[-1,-3],[-3,-2],[-2,0],[-3,2],[-4,6],[-2,2],[-2,0],[-2,-1],[-2,-3],[-1,-5],[0,-4],[0,-11],[0,-5],[-1,-2],[-2,-2],[-1,-2],[0,-2],[0,-5],[0,-2],[-2,-3],[-2,3],[-5,21],[-2,3],[-3,0],[-2,-1],[-2,-2],[-1,-2],[-2,-4],[-1,0],[-2,-1],[-1,-1],[-2,-3],[-4,-9],[-2,-1],[-3,1],[-2,1],[-10,9],[-2,1],[-2,-2],[-2,-2],[-4,-7],[-1,-2],[-5,-2],[-1,-2],[3,-2],[1,-3],[0,-5],[-4,-8],[0,-3],[-1,-4],[0,-3],[-5,-10],[-1,-4],[0,-3],[0,-6],[-11,0],[-11,0],[-11,0],[-12,-1],[-11,0],[-11,0],[-12,0],[-11,0],[-11,0],[-11,0],[-12,0],[-11,-1],[-11,0],[-12,0],[-11,0],[-11,0],[-11,0],[-12,0],[-6,0],[-5,0],[-11,0],[-12,-1],[-11,0],[-11,0],[-11,0],[-12,0],[-11,0],[-11,0],[-12,0],[-11,-1],[-11,0],[-11,0],[-12,0],[-2,0]],[[7499,803],[9,4],[18,13],[5,4],[3,7],[0,2],[0,3],[1,2],[1,1],[1,1],[1,1],[2,1],[3,5],[2,1],[10,3],[2,2],[4,6],[3,1],[11,0],[5,3],[4,5],[9,17],[1,1],[2,0],[1,1],[2,2],[6,7],[1,1],[3,6],[1,2],[2,0],[1,1],[2,2],[10,-2],[2,1],[2,0],[0,-2],[1,-1],[2,0],[1,1],[1,1],[2,6],[0,3],[-2,10],[0,3],[0,3],[0,3],[4,10],[2,4],[1,2],[1,3],[0,4],[1,2],[-2,7],[1,8],[2,6],[2,5],[18,25],[5,10],[1,4],[0,5],[0,4],[0,5],[0,2],[3,6],[1,2],[0,3],[0,2],[1,2],[0,2],[2,2],[12,4],[1,2],[1,2],[2,3],[3,3],[2,3],[3,4],[2,-2],[4,0],[4,1],[9,7],[4,1],[7,10],[0,3],[-1,4],[2,3],[5,5],[6,7],[7,13],[2,2],[3,0],[6,6],[14,6],[4,6],[3,9],[1,5],[1,3],[1,3],[10,8],[5,1],[3,0],[2,2],[2,3],[2,3],[2,4],[6,6],[4,7],[3,0],[2,-2],[3,-2],[4,-4],[6,-1],[46,28],[2,2],[5,5],[2,1],[2,2],[3,9],[3,4],[4,3],[11,-1],[5,3],[6,8],[2,2],[2,-1],[2,-1],[2,0],[1,1],[2,3],[1,1],[1,0],[3,-1],[1,1],[1,1],[2,6],[2,3],[15,9],[5,2],[2,2],[17,25],[11,23],[7,18],[3,7],[14,13],[3,8],[2,16],[3,8],[2,1],[2,1],[1,1],[1,2],[1,2],[1,2],[4,2],[11,18],[13,26],[11,20],[3,4],[1,1],[0,3],[0,1],[0,2],[0,4],[1,4],[11,24],[2,2],[2,1],[6,2],[2,2],[2,3],[8,15],[11,13],[11,20],[11,14],[13,13],[2,6],[3,1],[6,0],[3,1],[7,9],[1,2],[2,3],[8,4],[5,5],[6,14],[5,4],[5,1],[3,1],[4,8],[12,13],[3,2],[6,1],[2,1],[3,7],[3,2],[3,1],[6,0],[2,1],[28,22],[0,2],[5,4],[3,1],[5,2],[6,7],[14,3],[7,7],[8,5],[20,10],[27,20],[29,18],[10,3],[35,18],[19,4],[19,0],[23,4],[8,4],[5,2],[19,-1],[14,-4],[10,-2],[25,-5],[10,-5],[6,-2],[3,-2],[4,-6],[6,-4],[2,-2],[9,-2],[1,-1],[3,-3],[3,-2],[6,-9],[1,-1],[1,0],[1,-1],[2,-4],[2,-1],[4,-5],[2,-2],[4,-6],[1,-2],[3,-1],[1,-1],[1,-1],[1,-4],[0,-1],[7,-10],[2,-6],[-1,-10],[3,-7],[1,-3],[3,-3],[-2,-3],[-2,3],[-3,4],[-1,3],[-3,1],[-6,6],[-26,12],[-3,0],[2,-1],[7,-9],[0,-1],[-7,-1],[0,-3],[2,-1],[7,3],[5,0],[2,-2],[1,-2],[1,-1],[1,-2],[0,-2],[-1,-1],[-3,2],[-1,-1],[-1,-2],[1,-2],[1,0],[1,1],[1,1],[2,-1],[2,-4],[3,-1],[6,-8],[5,-11],[1,-2],[2,-1],[2,-2],[1,-3],[2,-4],[-4,-1],[-3,0],[-4,-1],[-2,-8],[-1,-4],[1,-2],[3,-3],[3,-4],[2,-1],[2,-1],[-1,-1],[-1,-2],[-1,-2],[-1,-2],[-1,-1],[-4,-1],[-2,-2],[-2,-2],[-2,-4],[-1,-4],[2,-2],[-2,-2],[-3,-2],[-10,-2],[-6,-3],[-5,-1],[-14,-9],[-2,-3],[-3,0],[-2,-1],[2,-1],[1,-2],[0,-1],[0,-3],[-1,-1],[-1,-2],[0,-1],[-1,0],[0,-1],[-1,-4],[-2,-7],[-1,-5],[-2,-3],[-15,-7],[-1,0],[-2,0],[-1,1],[-1,2],[-1,-1],[0,-2],[1,-4],[1,-3],[-4,-5],[-8,-6],[-14,-14],[-12,-8],[-11,0],[-3,-2],[-3,0],[-3,2],[-2,2],[0,2],[-1,3],[0,2],[-2,1],[-4,2],[-4,1],[-10,7],[-14,5],[-4,4],[-4,7],[-4,2],[0,5],[0,5],[0,4],[-1,-4],[-1,-2],[-1,-1],[-2,0],[-2,-1],[-1,-1],[-9,-18],[-3,-3],[-3,-1],[-10,4],[-10,0],[-1,0],[-2,-2],[1,0],[1,-1],[2,0],[-1,-7],[-1,-1],[-1,-1],[0,1],[-1,2],[-1,1],[-1,0],[-1,0],[0,1],[0,1],[-1,1],[0,1],[-1,1],[-4,3],[-11,1],[-4,-3],[-2,-9],[-1,-3],[-5,1],[-2,-4],[-1,1],[-3,1],[-1,0],[-1,-2],[0,-1],[-1,-2],[-4,-2],[-6,-1],[-2,-3],[-2,0],[-3,0],[-2,-1],[-2,-1],[1,0]],[[8696,1998],[12,-7],[3,-1],[1,0],[6,-5],[27,-7],[11,-4],[10,-1],[16,-4],[19,-10],[9,-1],[9,-8],[10,-3],[2,-2],[4,-7],[1,-1],[6,0],[8,-7],[2,-1],[3,0],[2,-1],[3,-2],[8,-8],[3,-2],[6,-2],[2,-3],[5,-7],[2,-1],[1,-1],[0,-1],[0,-3],[1,0],[5,-10],[1,-1],[1,0],[3,1],[1,-1],[3,-4],[2,-1],[2,0],[2,-1],[1,-2],[3,-6],[1,-1],[0,-2],[1,-2],[2,0],[1,-1],[5,-3],[13,-2],[2,-1],[4,-4],[2,-3],[1,-1],[4,1],[3,0],[2,-4],[5,-7],[-1,-3],[1,-2],[3,-3],[1,-2],[0,-3],[1,-2],[0,-1],[3,-2],[1,-3],[3,-9],[3,-4],[3,-4],[-1,-2],[-3,-3],[-1,-2],[0,-3],[0,-2],[0,-1],[-3,0],[-2,0],[-1,0],[-6,-7],[-2,0],[-4,3],[-2,1],[-19,-2],[-9,1],[-14,-3],[-3,1],[-3,2],[-2,3],[-2,2],[-9,3],[-10,6],[-20,2],[-23,12],[-23,9],[-8,7],[-6,2],[-7,11],[-6,2],[-4,4],[-2,1],[-2,2],[-7,1],[-14,8],[2,3],[-2,4],[-2,3],[-1,4],[0,4],[-1,4],[-1,4],[-1,3],[-4,7],[-10,15],[-18,17],[-15,10],[-14,6],[-14,9],[-2,3],[-2,6],[-3,3],[-3,-2],[2,-6],[-3,1],[-3,3],[-2,3],[-1,4],[-2,3],[-3,2],[-1,2],[2,5],[4,5],[12,4],[6,1],[3,0],[1,0],[2,2],[0,1],[1,2],[2,0],[9,0]],[[8728,2059],[-4,-1],[-3,1],[-1,4],[0,5],[2,3],[1,-1],[1,-1],[3,-7],[1,-3]],[[9239,2165],[2,-10],[1,-1],[1,-6],[0,-2],[0,-1],[-1,0],[-1,-1],[-1,-2],[-2,-2],[-1,-1],[-1,4],[1,3],[3,11],[-1,1],[-1,0],[0,-2],[-1,-1],[-1,1],[-1,1],[-1,2],[0,5],[0,3],[2,1],[3,-3]],[[9294,2200],[-1,-2],[-3,1],[0,2],[3,10],[1,-5],[0,-6]],[[9326,2275],[-2,-5],[-4,3],[4,7],[4,7],[10,7],[4,-3],[-4,-6],[-4,-5],[-8,-5]],[[9349,2300],[-5,0],[-1,5],[3,4],[9,3],[0,-4],[-2,-4],[-4,-4]],[[9343,2319],[0,-3],[1,-1],[0,-2],[1,-1],[-1,-1],[-6,-6],[-2,-1],[-10,3],[1,3],[2,3],[4,4],[3,1],[5,0],[2,1]],[[8312,4011],[-6,-4],[-6,3],[-3,9],[0,9],[3,9],[5,5],[4,-3],[3,-16],[0,-12]],[[8137,4172],[-5,-1],[1,3],[3,6],[3,4],[7,1],[1,-3],[2,-3],[-5,-5],[-7,-2]],[[8123,4217],[4,-1],[1,-2],[5,1],[1,-2],[2,-5],[2,-4],[1,-5],[-5,-2],[2,-1],[-4,-4],[-3,-2],[-1,-3],[-2,-2],[0,-1],[-5,-4],[-2,-4],[2,-4],[-2,-2],[-2,1],[-5,6],[-1,-5],[-2,-3],[-2,0],[-2,3],[1,3],[2,4],[1,3],[0,1],[0,5],[1,1],[0,1],[1,1],[1,5],[1,1],[0,1],[-1,3],[-4,8],[-2,6],[1,1],[4,0],[1,2],[7,0],[4,-1]],[[8036,4652],[-1,-5],[-3,1],[0,8],[-1,7],[3,5],[3,4],[4,-1],[0,-4],[2,-4],[-5,-3],[0,-5],[-2,-3]],[[7861,4736],[-5,-1],[-3,1],[0,6],[3,2],[8,1],[1,-4],[-1,-5],[-3,0]],[[7805,4869],[-5,0],[-6,5],[-3,4],[2,4],[3,-3],[8,-4],[3,-1],[2,-3],[-4,-2]],[[7769,4909],[-3,-4],[-4,3],[-6,3],[-2,3],[6,0],[3,-1],[6,-4]],[[8651,4496],[0,-1],[-3,0],[-3,0],[-2,-1],[-3,-1],[-1,0],[-2,0],[-2,0],[-3,0],[-3,0],[-4,-1],[-5,-2],[-4,-4],[-2,-7],[2,-4],[4,-5],[4,-4],[3,-2],[4,2],[5,0],[3,-2],[3,-7],[1,-4],[0,-2],[0,-2],[-2,-1],[-3,0],[-3,0],[-3,0],[-3,0],[-2,-1],[1,-3],[2,-2],[2,-1],[-1,-3],[-1,-2],[-2,0],[-1,-1],[-2,-1],[-2,-1],[-2,-1],[-2,0],[-2,1],[-3,2],[-2,2],[-2,-1],[-1,-2],[0,-2],[2,-2],[1,-2],[1,-3],[0,-3],[1,-2],[0,-3],[2,-2],[1,-1],[1,0],[2,1],[2,0],[1,0],[2,-2],[1,-1],[3,-1],[4,-1],[3,-2],[1,-6],[-2,-8],[-3,0],[-4,2],[-4,-3],[0,-2],[2,-3],[1,-2],[0,-2],[-2,-1],[-2,-1],[-2,-1],[-2,-2],[1,-2],[1,-2],[0,-1],[0,-2],[-1,0],[0,-1],[0,-2],[-1,0],[0,-1],[1,-1],[0,-1],[1,-2],[0,-1],[0,-2],[1,-3],[0,-2],[0,-2],[1,-2],[0,-1],[1,-1],[-6,-5],[-5,-2],[-2,-2],[-2,-5],[0,-9],[1,-10],[5,-4],[3,-2],[0,-1],[3,-4],[1,0],[1,0],[1,-1],[0,-1],[0,-1],[0,-1],[0,-1],[1,-3],[1,-2],[1,-3],[2,-2],[3,-2],[3,1],[4,2],[3,2],[4,2],[3,3],[4,4],[3,3],[4,2],[4,1],[5,-2],[3,-4],[0,-3],[-1,-2],[-2,-1],[-2,-2],[-2,-1],[-3,-2],[-2,-2],[-3,-2],[-1,-2],[-2,-1],[0,-2],[1,-3],[1,-3],[0,-1],[-2,-2],[-2,0],[-2,-3],[2,-4],[2,-4],[0,-4],[-1,-2],[-2,-2],[0,-2],[-1,-3],[1,-2],[1,-2],[1,-2],[1,-3],[-1,-3],[-1,-3],[-1,-3],[0,-4],[1,-2],[1,-2],[1,-1],[1,-2],[1,-5],[0,-5],[0,-4],[2,-5],[3,-1],[3,-1],[3,0],[3,-1],[2,-2],[2,-2],[1,-3],[2,-3],[1,-6],[-3,-4],[-4,-2],[-4,0],[-3,0],[-2,0],[-3,1],[-3,2],[-2,1],[-2,0],[-2,1],[-2,1],[-2,2],[-2,1],[-3,0],[-2,1],[-2,1],[-1,2],[-1,2],[-1,2],[-4,3],[-4,0],[-4,-3],[-4,-4],[-1,-2],[-1,-4],[-2,-3],[0,-2],[0,-1],[1,-2],[1,-1],[0,-2],[0,-3],[-1,-2],[-2,-1],[-2,-2],[1,-2],[2,-2],[3,-1],[1,0],[4,0],[3,3],[3,4],[4,0],[4,-3],[3,-4],[4,-4],[4,-2],[3,0],[2,1],[3,1],[3,0],[3,-1],[2,-1],[3,-1],[3,0],[3,0],[4,0],[4,-2],[2,-6],[0,-4],[-1,-3],[0,-3],[0,-4],[1,-3],[1,-1],[1,-2],[2,0],[4,-3],[3,-1],[4,0],[4,4],[4,5],[4,5],[4,3],[5,1],[3,0],[3,0],[3,0],[2,1],[2,2],[1,2],[1,3],[2,2],[3,3],[4,0],[4,-2],[4,-3],[2,-2],[3,0],[2,-2],[3,-2],[2,-2],[1,-3],[2,-3],[2,-3],[3,-7],[-2,-5],[-4,-4],[-5,-2],[-6,-1],[-6,-1],[-7,-1],[-6,-1],[-2,0],[-3,1],[-2,0],[-2,1],[-2,-1],[-2,-2],[-2,-2],[-3,0],[-4,0],[-4,-1],[-3,-3],[-3,-7],[-2,-6],[0,-5],[1,-5],[1,-5],[3,-5],[3,-1],[3,0],[3,1],[3,3],[3,4],[4,2],[3,-5],[0,-3],[1,-2],[0,-3],[1,-2],[2,-2],[1,-3],[0,-3],[-2,-3],[-3,-6],[-5,0],[-4,0],[-4,-1],[-3,-3],[-2,-4],[-1,-5],[-2,-2],[-3,-1],[-4,-1],[-3,-2],[-2,-2],[-2,-3],[0,-2],[-1,-2],[0,-3],[-1,-3],[0,-4],[0,-4],[-1,-3],[-1,-3],[-2,-2],[-2,-2],[-2,-1],[-5,-1],[-5,-1],[-5,-5],[-2,-9],[-1,-6],[-1,-7],[0,-5],[3,-3],[2,0],[2,1],[2,0],[2,-1],[2,-2],[2,-1],[2,-1],[2,0],[4,0],[1,-4],[0,-6],[0,-7],[1,-5],[3,-4],[2,-5],[1,-5],[3,-9],[4,-13],[4,-9],[4,2],[0,3],[0,4],[1,2],[2,-1],[2,-1],[2,0],[2,-1],[2,-2],[1,-3],[1,-4],[0,-5],[1,-3],[2,-1],[2,-1],[2,0],[1,-1],[0,-3],[1,-3],[1,-1],[2,0],[1,4],[1,4],[1,2],[3,-4],[1,-5],[2,-5],[3,-4],[2,4],[1,3],[0,3],[0,2],[1,3],[1,3],[2,2],[2,0],[2,-1],[1,-7],[-1,-6],[-3,-6],[-2,-4],[-3,-4],[-4,-2],[-3,-3],[-3,-4],[-1,-5],[0,-7],[0,-6],[0,-6],[0,-6],[1,-5],[1,-5],[2,-3],[2,-6],[-1,-3],[-3,-2],[-3,-2],[-2,-1],[-1,-1],[-2,-2],[-1,-2],[-1,-1],[-1,-2],[0,-2],[0,-3],[-1,-6],[-2,-5],[-1,-6],[1,-6],[2,-2],[2,6],[1,9],[1,5],[2,0],[3,-3],[3,-5],[0,-4],[-1,-4],[-1,-3],[-2,-3],[-1,-4],[0,-2],[0,-2],[1,-3],[0,-3],[-2,-6],[-3,-3],[-3,-1],[-2,-4],[-1,-2],[0,-4],[-1,-3],[0,-3],[1,-3],[1,-2],[0,-1],[-1,-2],[-2,-3],[-2,-4],[0,-5],[1,-5],[1,-1],[1,-2],[2,-3],[0,-3],[-2,-1],[-2,0],[-3,1],[-1,-1],[-3,-2],[-3,-3],[-2,-4],[1,-6],[1,-5],[-3,-3],[-4,-2],[-3,-1],[-2,-3],[-2,-5],[-1,-6],[0,-4],[2,-5],[3,-3],[3,-3],[2,-4],[1,-4],[1,-3],[1,-3],[1,-2],[2,-2],[2,-1],[2,-2],[1,-3],[0,-5],[-1,-5],[0,-5],[3,-3],[3,-2],[2,-3],[0,-3],[-3,-3],[-6,-2],[-6,0],[-6,2],[-6,2],[-1,0],[-2,0],[-2,0],[-1,-2],[1,-3],[1,-1],[2,-1],[1,-2],[2,-5],[0,-7],[-1,-6],[-2,-4],[1,-2],[0,-1],[1,-1],[1,-1],[1,-1],[0,-2],[1,-1],[0,-1],[2,-2],[2,0],[2,1],[2,1],[4,-1],[5,-2],[5,-3],[2,-5],[-1,-6],[-4,-3],[-5,-1],[-3,-1],[-2,-4],[-1,-5],[0,-5],[0,-6],[1,-5],[4,2],[4,5],[3,3],[1,1],[3,1],[2,1],[1,-1],[1,-2],[0,-1],[-2,-2],[-1,-1],[-1,-2],[0,-2],[1,-1],[1,0],[2,0],[0,-2],[0,-1],[1,-2],[3,-2],[4,0],[4,0],[4,1],[2,-1],[2,-2],[2,-2],[2,-2],[3,0],[2,0],[2,0],[3,0],[2,1],[3,1],[4,1],[2,-2],[-1,-3],[-1,-2],[-2,-1],[-2,0],[-2,-1],[-2,0],[-2,0],[-3,-1],[-1,-1],[-2,-2],[-2,-2],[-2,-2],[-3,-2],[-3,-3],[-4,-1],[-4,-1],[-3,0],[-3,-3],[-3,-2],[0,-4],[2,-6],[5,-5],[2,-5],[0,-6],[0,-3],[2,-1],[4,-1],[2,-2],[0,-5],[-1,-6],[-1,-4],[-2,-6],[-2,-3],[0,-5],[2,0],[3,0],[2,-1],[2,-7],[-1,-6],[0,-10],[-1,-9],[-3,-4],[-4,-3],[-3,-1],[1,-6],[3,-5],[11,-3],[7,-1],[8,0],[6,-2],[7,-2],[4,1],[2,0],[1,-5],[-1,-7],[-4,-1],[-9,-3],[-4,1],[-4,0],[-3,0],[-3,-2],[0,-3],[2,-1],[3,0],[-1,-4],[-3,-1],[-3,0],[-6,-4],[-1,-4],[1,-6],[6,-3],[5,-1],[6,3],[4,2],[6,1],[-1,-6],[0,-1],[-4,-5],[-5,0],[-4,-2],[-2,-2],[-2,-7],[-2,-9],[-2,-12],[2,-5],[0,-7],[1,-4],[0,-6],[-2,-5],[0,-3],[0,-7],[-1,-4],[-6,0],[-4,4],[-5,4],[-4,2],[-5,0],[0,-6],[-3,-9],[1,-8],[1,-4],[-1,-4],[-6,-4],[-2,-4],[1,-4],[3,-4],[9,-9],[4,-7],[3,-8],[-1,-6],[-7,5],[-7,-3],[-2,-4],[-2,-3],[-8,1],[-10,0],[-7,6],[-3,9],[-2,5],[-2,5],[-7,6],[-8,7],[-9,6],[-8,2],[-5,-8],[-4,-6],[-4,-3],[-6,2],[-1,0],[-4,0],[-2,0],[-3,-1],[-6,2],[0,3],[-1,4],[2,5],[0,7],[-2,4],[-4,1],[-4,3],[0,5],[-2,4],[-8,10],[-7,2],[-6,3],[-5,2],[-7,-7],[-4,-12],[-3,-7],[-6,-3],[-2,-3],[-3,3],[-4,-3],[-3,0],[-2,0],[-3,2],[-5,2],[-1,-6],[2,-5],[2,-4],[0,-8],[-4,-4],[-8,-1],[-3,3],[-6,3],[-3,-5],[-1,-3],[-2,-1],[-2,2],[-1,4],[-3,5],[-3,5],[-3,4],[-4,5],[-2,6],[-1,3],[0,3],[0,4],[0,3],[-1,3],[-2,1],[-2,1],[-1,0],[-3,1],[-2,1],[-3,1],[-2,-1],[-2,-1],[-1,-1],[-1,-1],[-2,0],[-2,1],[-2,3],[-2,4],[-2,3],[-2,2],[-3,1],[-2,1],[-2,0],[-7,6],[-5,8],[-5,10],[-5,9],[-1,2],[-1,2],[-2,2],[-1,2],[-2,1],[-2,1],[-1,2],[-2,2],[-2,6],[-3,6],[-2,5],[-4,4],[-3,0],[-4,-1],[-3,1],[-1,5],[-1,7],[-4,5],[-4,3],[-4,0],[-2,-1],[-1,0],[-1,-2],[0,-2],[1,-3],[2,-3],[2,-3],[2,-2],[2,-3],[4,-5],[3,-5],[1,-4],[0,-3],[-2,0],[-2,-1],[-1,-1],[-1,-3],[-1,-2],[0,-4],[0,-3],[-1,-3],[0,-2],[0,-2],[0,-3],[1,-1],[1,-2],[1,-2],[0,-2],[1,-3],[1,-2],[1,-1],[1,-3],[1,-2],[2,-2],[1,-3],[0,-3],[-1,-2],[-4,-1],[-2,-3],[0,-5],[4,-8],[6,-13],[3,-11],[-4,-6],[-2,1],[-2,0],[-3,0],[-2,-2],[1,-2],[1,-2],[1,-3],[0,-2],[-4,-1],[-6,3],[-5,4],[-4,4],[-4,2],[-4,3],[-3,3],[-3,6],[-1,3],[-1,4],[-1,4],[-1,3],[-2,2],[-3,1],[-2,1],[-2,2],[-4,4],[-3,5],[-3,4],[-4,3],[-2,1],[-3,2],[-3,1],[-1,3],[0,2],[1,2],[1,2],[1,1],[1,2],[0,2],[0,1],[-2,1],[-2,2],[-3,1],[-2,1],[-3,0],[-3,-2],[-3,-4],[-1,-3],[0,-4],[2,-9],[5,-11],[7,-14],[6,-14],[10,-13],[10,-12],[0,-12],[0,-1],[-2,-4],[-2,-3],[-4,-8],[-3,-1],[-1,0],[-5,-5],[-3,-5],[1,-5],[2,-8],[-3,-7],[-6,4],[-3,2],[-6,10],[-3,4],[-9,6],[-1,-3],[0,-2],[2,-8],[-2,-12],[-4,-1],[-7,-4],[-3,4],[-4,-6],[-3,-3],[-6,-5],[0,-11],[0,-9],[7,-9],[2,-10],[4,-9],[0,-10],[-4,-5],[-4,-3],[-4,0],[-4,-7],[-2,-6],[-1,-9],[-1,-7],[4,-7],[3,-5],[4,-3],[5,-7],[6,-5],[2,-9],[1,-6],[1,-1],[2,-2],[5,-2],[0,-2],[1,-4],[-4,-3],[-3,-1],[-2,-4],[2,-4],[5,-1],[2,-7],[7,-5],[0,-1],[4,-3],[2,-14],[6,-12],[2,-4],[0,-1],[16,-6],[6,-3],[2,-1],[9,-2],[7,-9],[6,-10],[3,-5],[-5,-5],[-3,-1],[-3,-2],[-2,-7],[1,-7],[4,-7],[0,-12],[0,-2],[-1,-12],[0,-1],[0,-1],[-2,-3],[1,-6],[0,-9],[-2,-6],[-6,-2],[-1,5],[1,7],[-5,0],[-3,0],[-5,8],[-5,0],[-3,3],[-7,-3],[-3,-5],[-4,-1],[-3,-6],[2,-22],[3,-9],[1,-8],[-1,-7],[5,-11],[4,-2],[7,-8],[4,-2],[4,9],[4,4],[3,0],[1,0],[1,-15],[1,-9],[6,-3],[0,-1],[5,-5],[4,-10],[3,-5],[2,-3],[1,7],[1,9],[0,1],[3,-2],[1,-8],[2,-3],[3,0],[1,6],[0,1],[-1,14],[2,2],[5,-1],[6,6],[1,8],[0,15],[2,11],[4,1],[5,0],[4,4],[1,0],[5,9],[0,9],[1,0],[3,1],[7,-8],[5,-16],[3,-8],[0,-1],[0,-8],[0,-4],[-2,-1],[-2,2],[-3,2],[-4,3],[-1,-7],[2,-11],[5,-15],[5,-24],[0,-7],[-2,0],[-7,15],[-4,-5],[0,-14],[2,-6],[0,-2],[2,-4],[-1,-9],[0,-9],[2,-6],[1,-14],[0,-6],[-4,2],[-5,2],[-5,-9],[0,-1],[0,-1],[1,-6],[1,-5],[0,-4],[1,-4],[2,-6],[3,-2],[3,-11],[0,-3],[1,-4],[2,-3],[6,0],[4,6],[-1,16],[-2,7],[-1,8],[2,5],[3,-2],[1,-2],[4,-9],[2,-7],[3,-2],[5,-3],[5,-8],[-1,-11],[0,-8],[7,-2],[4,-5],[2,2],[3,3],[1,3],[4,-3],[4,4],[5,2],[3,-1],[4,-3],[3,-1],[4,4],[0,1],[5,0],[0,-3],[-1,-7],[-2,-10],[2,-9],[3,7],[3,4],[7,0],[2,6],[4,10],[1,-8],[3,-10],[3,-5],[2,-5],[3,-1],[2,1],[1,-8],[-1,-7],[-2,-5],[3,-12],[3,-7],[2,8],[5,3],[3,-5],[3,-5],[4,-1],[1,-7],[-1,-4],[6,-4],[6,-2],[5,3],[5,-6],[4,-4],[4,2],[0,8],[7,3],[10,-3],[7,-7],[2,-3],[4,-7],[5,-10],[1,-3],[3,-6],[4,-4],[1,-1],[4,2],[2,2],[5,7],[3,4],[7,3],[3,7],[0,1],[5,10],[-1,6],[-1,4],[-5,9],[0,12],[0,1],[-3,22],[3,11],[4,-5],[0,-1],[3,-5],[5,4],[-2,7],[-3,7],[2,11],[1,7],[4,3],[5,-3],[2,4],[-3,8],[-4,14],[-1,11],[2,9],[5,4],[2,8],[2,8],[0,10],[-1,8],[-1,6],[-2,10],[-2,5],[-1,5],[3,6],[0,10],[0,9],[4,5],[2,6],[5,2],[7,1],[24,9],[8,2],[6,8],[1,6],[5,6],[1,-10],[0,-8],[1,-2],[0,-2],[2,0],[3,-2],[2,-3],[2,-2],[2,-2],[2,-1],[3,-1],[2,-2],[3,-4],[3,-6],[0,-6],[-3,-3],[-3,1],[-2,0],[-3,1],[-2,0],[-4,0],[-3,-1],[-3,0],[-3,-1],[-6,-1],[-5,-1],[-6,-1],[-6,-1],[-2,-1],[-1,0],[-4,-2],[-3,-2],[-3,-3],[-3,-2],[-1,-1],[-3,-4],[-3,-4],[-2,-4],[-1,-1],[-1,-3],[-1,-3],[-1,0],[-1,-5],[1,-5],[2,-3],[0,-1],[1,-2],[1,-3],[1,-5],[3,-5],[0,-2],[3,-3],[8,-4],[2,-1],[1,1],[3,-1],[5,1],[4,-1],[3,-3],[-1,-2],[-1,-1],[-1,-1],[-1,-1],[0,-2],[1,-3],[2,-2],[2,-5],[3,-6],[2,-9],[1,-11],[1,-11],[1,-4],[2,-4],[1,-3],[0,-4],[-3,0],[-5,3],[-6,3],[-3,3],[-3,-1],[1,-6],[4,-7],[4,-4],[4,0],[3,0],[4,0],[4,0],[24,0],[11,0],[11,0],[23,0],[23,0],[23,0],[23,0],[23,0],[23,0],[34,0],[34,0],[35,0],[34,0],[35,0],[34,0],[34,0],[35,0],[28,0],[28,0],[28,0],[28,0],[28,0],[29,0],[13,0],[14,0],[29,0],[12,0],[12,0],[13,0],[12,0],[3,0],[0,-15],[0,-11],[0,-6],[0,-18],[0,-18],[0,-18],[0,-10],[0,-15],[0,-13],[0,-11],[0,-5]],[[9492,2352],[-5,2],[-2,0],[-1,-3],[-2,1],[-4,5],[-3,7],[2,6],[-2,4],[-4,-1],[-9,-3],[-4,-3],[-1,0],[-2,1],[-1,0],[-2,-1],[1,-3],[0,-3],[-1,-2],[-2,0],[-1,4],[-1,1],[0,-1],[-1,-1],[0,-1],[-1,-1],[-11,-5],[1,3],[1,3],[1,3],[-2,-1],[-4,-5],[-5,-2],[-1,0],[0,1],[-1,2],[0,3],[0,3],[1,2],[-3,-2],[-3,0],[-2,-2],[-1,-6],[-1,-5],[0,-2],[-1,-1],[-1,0],[-1,2],[0,3],[0,3],[-2,-2],[-3,-5],[-4,-2],[-6,-8],[-4,-8],[-1,-1],[-8,-4],[-1,1],[-1,-2],[-7,-4],[-1,1],[-2,1],[-1,-1],[-3,-3],[-5,-2],[-2,1],[-2,3],[1,5],[0,3],[0,2],[-4,0],[2,-2],[-3,-9],[1,-4],[-5,1],[-9,7],[-4,2],[-14,-14],[-2,-1],[0,4],[-2,2],[-1,0],[-1,-4],[0,-1],[1,-1],[0,-3],[0,-2],[-1,0],[-2,0],[-1,1],[-1,2],[0,3],[-1,0],[-1,-3],[1,-4],[2,-4],[2,-3],[5,-3],[-1,-1],[-1,-2],[-1,-3],[-1,-2],[-1,-2],[-4,-1],[-1,-2],[-3,-4],[-1,-2],[-7,-3],[-1,0],[-1,-2],[-2,-2],[-1,0],[-1,3],[-1,-2],[-1,-3],[0,-4],[1,0],[1,-1],[0,-2],[-2,-2],[-2,-3],[-2,-5],[-1,-1],[-2,1],[0,3],[3,7],[-1,3],[-2,0],[-3,-1],[-1,-2],[-1,-1],[-1,-3],[-1,-1],[-1,0],[-1,1],[-1,-1],[-1,-1],[1,-3],[1,0],[2,0],[1,1],[1,-1],[0,-3],[-1,-5],[-1,-1],[0,-2],[0,-1],[0,-1],[2,-1],[0,-2],[0,-2],[0,-3],[-1,-2],[0,-2],[1,-1],[2,-3],[-3,-2],[-1,-4],[2,-4],[0,-3],[-1,-2],[-1,-1],[-1,-2],[-1,-3],[0,-1],[-2,-2],[-2,-1],[0,-2],[0,-1],[1,-1],[1,-1],[-3,-2],[-4,3],[-2,5],[-1,7],[0,3],[3,6],[1,3],[0,3],[-1,3],[0,2],[-2,-2],[-1,-4],[-2,-11],[-1,-6],[-7,-10],[0,-3],[-2,-4],[-2,2],[-2,4],[-2,1],[-1,-1],[0,-2],[1,-3],[-1,-2],[-1,-2],[-1,0],[-2,-1],[-2,-3],[-3,-6],[-2,-3],[-2,0],[-3,1],[-2,-1],[0,-1],[0,-2],[1,-1],[1,-1],[-1,-3],[-1,0],[-2,0],[-1,-1],[-1,-2],[-1,-6],[2,-2],[3,0],[1,-3],[0,-4],[-1,-4],[-3,-1],[-5,0],[-2,3],[-1,-1],[-1,-2],[0,-1],[-2,-1],[-1,-2],[-1,-2],[-1,-4],[-1,-2],[-1,-2],[-7,-1],[-5,-3],[-3,-1],[-1,-1],[-1,-2],[-2,-1],[-1,1],[-3,-1],[-1,1],[-2,-3],[-5,-10],[-2,-3],[0,-2],[1,0],[2,-1],[-1,-3],[0,-3],[1,-2],[2,-1],[0,2],[1,3],[1,2],[0,-3],[-1,-4],[-2,-4],[-6,-4],[-2,-2],[-2,-1],[0,-1],[-1,-4],[-1,1],[-2,2],[-1,-1],[-2,-4],[-2,-2],[-3,-1],[-2,3],[-1,-2],[-1,0],[-1,0],[-1,-1],[-1,-1],[-1,-2],[0,-1],[-1,-1],[-1,-2],[-1,1],[1,4],[-5,0],[1,2],[1,2],[0,3],[0,6],[-2,0],[-3,-3],[-2,-5],[-3,-5],[-3,2],[1,1],[2,2],[1,2],[0,3],[-2,-1],[-1,0],[-1,2],[0,4],[-2,-2],[0,-8],[0,-2],[-5,2],[-2,0],[-2,-2],[1,-1],[1,-2],[0,-1],[-2,-1],[-1,1],[0,3],[-1,1],[-1,1],[-1,1],[-1,0],[0,-1],[-1,-1],[-1,-1],[-1,0],[-1,1],[1,4],[-3,-2],[-4,0],[-1,-1],[-2,-5],[-1,-1],[-7,-2],[-3,2],[1,7],[-3,-4],[-1,-1],[-2,-2],[-6,0],[0,1],[1,1],[0,1],[0,1],[-3,0],[-1,-2],[-2,-2],[-2,-4],[-1,-1],[-3,4],[-2,0],[1,4],[2,3],[2,2],[2,1],[-3,1],[-4,-2],[-11,-13],[0,-1],[-1,0],[0,1],[0,2],[-1,1],[0,1],[-1,0],[-2,-2],[-2,1],[-2,2],[-1,1],[-2,-1],[-1,-1],[0,-1],[0,-2],[0,-1],[-1,0],[-2,2],[-3,-2],[-1,-1],[0,-1],[-1,-1],[-3,3],[-2,-1],[1,-4],[-1,1],[-5,3],[-2,1],[-10,-1],[-13,-8],[-9,-2],[-9,-7],[-6,-2],[-6,-4],[-3,-1],[-7,1],[0,1],[2,1],[3,3],[2,1],[2,1],[10,6],[2,0],[2,1],[1,3],[-1,3],[-1,1],[-2,-4],[-3,-2],[-7,-2],[-6,-5],[-3,-1],[-3,3],[1,6],[-3,5],[-2,3],[-1,4],[-2,2],[-2,1],[-5,0],[-1,1],[-2,1],[-1,0],[-1,-1],[-2,-2],[-1,0],[-3,-1],[-13,4],[-9,0],[-2,1],[-5,4],[-2,3],[-2,2],[-1,2],[-2,-1],[-1,-1],[-2,-1],[-1,0],[-1,0],[0,2],[0,1],[-1,1],[-1,0],[-1,-1],[-1,-2],[-1,-2],[-1,-2],[-1,0],[-1,1],[-1,2],[-2,1],[-13,-1],[-17,2],[0,2],[4,3],[0,2],[-5,-3],[-3,0],[-1,1],[-1,-2],[-1,-1],[-3,-2],[-2,2],[-9,-2],[-3,0],[-1,1],[-1,2],[-2,3],[-1,1],[-2,0],[-1,-1],[-2,-5],[-1,-1],[-1,0],[-5,2],[-6,0],[-2,-1],[1,-3],[1,-3],[0,-1],[-2,-3],[-3,-1],[-5,-1],[-9,2],[-3,-1],[-2,-1],[-1,-4],[-1,-1],[-3,0],[-2,0],[-1,1],[0,4],[-2,3],[-2,2],[-3,1],[-3,0],[-8,-3],[-1,1],[-1,3],[-2,1],[-1,0],[-1,-1],[-1,0],[-2,1],[-1,1],[-1,4],[-1,2],[-1,0],[-5,0],[-2,0],[-2,2],[-1,1],[-21,-2],[-5,-3],[-7,-1],[-1,-1],[-3,-4],[-1,-1],[-12,1],[-11,4],[-3,3],[-2,3],[-3,2],[-3,1],[-3,0],[-2,-2],[-5,-4],[-11,-4],[-27,-2],[-1,0],[-1,0],[-2,2],[0,1],[-1,0],[-3,-1],[-11,0],[-9,4],[-4,0],[-2,-2],[-3,-1],[-2,3],[-5,6],[-1,-1],[-3,-2],[-1,-1],[-18,-1],[-8,-5],[-10,-1],[-13,-5],[-12,1],[-2,1],[-4,4],[-4,2],[-4,1],[-3,-4],[-5,-10],[-1,-3],[-2,-1],[-2,0],[-1,2],[1,4],[-3,0],[0,-4],[1,-5],[1,-4],[-2,1],[-6,0],[-2,2],[-2,2],[-2,1],[-5,1],[-4,-1],[-4,-3],[-3,-1],[-6,4],[0,1],[0,2],[0,2],[0,2],[0,3],[-1,2],[-2,2],[-2,0],[-2,-1],[-3,-4],[-4,-6],[-1,-3],[-1,-4],[1,-4],[3,-4],[2,1],[3,1],[3,0],[1,-1],[1,-2],[-1,-1],[-2,-2],[-1,-1],[-1,-1],[-3,1],[-4,3],[-1,0],[-3,0],[-1,0],[-2,1],[-3,4],[-2,1],[0,-2],[0,-2],[1,-2],[1,-2],[-3,-1],[-2,-3],[-4,-6],[-5,-13],[-3,-3],[-5,-4],[-11,-3],[-5,-6],[-1,-5],[-1,-3],[-1,-2],[1,-3],[0,-2],[1,-3],[-3,-3],[-2,-4],[-1,-6],[-1,-4],[-2,-4],[-1,1],[-2,3],[-2,1],[-3,-1],[-2,-3],[-2,-4],[-2,-6],[-1,-4],[-1,-4],[0,-4],[0,-4],[1,-3],[1,-4],[0,-4],[-1,-3],[-4,-7],[-2,-3],[0,-3],[-2,-10],[1,-6],[1,-7],[-1,-8],[-1,-6],[-2,-6],[-13,-23],[-2,-2],[-17,0],[-4,0],[-4,-3],[-1,1],[-2,0],[-3,-1],[-7,0],[-1,-1],[-4,-2],[-1,-2],[-1,-2],[-1,-1],[-1,-1],[-6,1],[-6,2],[-3,1],[-6,-2],[-3,0],[-3,2],[-1,-3],[-1,-2],[-2,-4],[-4,3],[-2,-4],[0,-6],[-3,-6],[-1,0],[-2,0],[-1,0],[-1,-1],[-1,-2],[-1,0],[-6,0],[-2,1],[0,-1],[5,-6],[2,-1],[4,2],[1,-1],[1,-4],[0,-2],[0,-4],[-4,-3],[-12,-4],[-2,-2],[-4,-5],[-2,-1],[-2,-1],[-4,0],[8,10],[1,3],[0,6],[-4,6],[0,5],[-3,3],[-2,0],[2,-3],[1,-6],[1,-6],[1,-3],[-1,-1],[-4,-5],[-4,-3],[-1,-2],[-3,-2],[-6,-2],[-3,-1],[-2,-4],[-2,-5],[-1,-7],[1,-6],[-7,-5],[1,-3],[4,0],[1,-3],[-16,-2],[-1,-1],[-1,-2],[-1,-2],[0,-2],[-8,-8],[-3,-2],[-2,-2],[-3,-1],[0,-2],[0,-2],[-1,-2],[-1,-4],[-2,-1],[-6,0],[-2,-2],[-1,-4],[-6,-31],[-3,-7],[-5,-3],[-1,1],[-1,1],[-1,0],[-1,-1],[-5,-12],[-1,-7],[-1,-3],[0,-9],[-4,-9],[-9,-15],[-3,-11],[-2,-5],[-2,-2],[-3,-2],[-2,-3],[-2,-3],[-2,-2],[-1,-1],[-2,1],[-1,-2],[0,-1],[0,-1],[0,-1],[-8,-15],[-3,-4],[-2,-1],[-2,0],[-4,3],[-2,1],[-4,-1],[-2,1],[-2,3],[-3,8],[-2,4],[-2,2],[-5,7],[-5,4],[-6,1],[-6,-2],[-6,0],[-2,3],[-5,7],[-3,2],[-3,1],[-3,2],[-9,10],[-3,0],[-2,-2],[-3,-1],[-22,8],[-5,-1],[-2,0],[-2,3],[-4,5],[-2,2],[-3,1],[-1,0],[-2,-1],[-1,-1],[-1,1],[-5,4],[-12,4],[-3,-1],[-4,-3],[-3,-1],[9,2],[2,-1],[10,-7],[6,-2],[3,-1],[2,-3],[0,-4],[-2,-3],[-3,-1],[-6,0],[1,-4],[1,-3],[1,-2],[2,-1],[2,1],[5,5],[4,2],[17,0],[6,2],[7,-2],[2,-1],[2,-2],[6,-1],[4,-2],[1,-1],[2,-1],[1,-5],[1,0],[2,0],[1,1],[6,-8],[8,-7],[8,-2],[7,4],[3,-2],[6,-3],[3,-2],[2,-4],[4,-12],[3,-3],[3,-2],[7,-1],[2,-2],[0,-4],[-4,-8],[-1,-4],[-1,-12],[0,-4],[-1,-2],[-2,-1],[-1,-2],[0,-2],[0,-2],[0,-3],[-1,-6],[-1,-4],[-6,-11],[-1,-5],[-1,-10],[-2,-4],[-1,-2],[-3,-3],[-1,-2],[-4,-9],[-2,-3],[-5,-3],[-1,-2],[-2,-2],[-2,0],[-1,0],[-2,-3],[1,-3],[-2,-4],[-3,-4],[-2,-3],[-1,-4],[0,-3],[0,-4],[0,-3],[0,-2],[-1,-3],[-2,-5],[-2,-2],[-3,-2],[-3,-3],[-1,0],[-7,-2],[-4,-2],[-3,-2],[-2,-3],[-1,1],[-2,1],[-1,1],[-1,-3],[-1,-12],[-1,-3],[-2,-2],[-2,-5],[-2,-11],[-3,-11],[-2,-5],[-2,-2],[-6,-17],[-3,-8],[-4,-6],[-5,-4],[-15,-9],[-4,-4],[-26,-39],[-1,-3],[-3,-8],[0,-1],[-1,-3],[-1,-2],[-7,-7],[-2,-1],[-2,-3],[-2,-1],[-12,-1],[-6,-2],[-9,-6],[-5,-2],[-6,-4],[-3,-2],[-6,0],[-1,1],[-2,2],[-2,1],[-3,1],[-2,1],[-2,-1],[-11,-14],[-4,-6],[-5,-3],[-10,-3],[-8,-5],[-2,-2],[0,-4],[-2,-13],[-1,-3],[-4,-4],[-5,-2],[-2,-2],[-4,-4],[-6,-2],[-2,-2],[-16,-24],[-5,-4],[-2,-1],[-8,1],[-4,-1],[-8,-4],[-8,-6],[-9,-7],[1,-4],[-2,-13],[-2,-8],[-6,-5],[-4,0],[-6,-3],[-2,-5],[0,-5],[-1,-4],[-2,-8],[-7,-20],[-3,-6],[-5,-5],[-2,-3],[-2,-8],[-5,-11],[-9,-9],[-20,-5],[-9,-7],[-12,-20],[-4,-9],[-1,-4],[-1,-1],[-3,-2],[-1,-1],[-1,-2],[-1,-2],[-2,-2],[-2,-2],[-5,-2],[-7,1],[-2,1],[-1,0],[-2,1],[-3,6],[-1,2],[-2,1],[-1,0],[-11,-2],[-1,1],[-2,2],[-3,6],[-2,0],[2,-2],[1,-4],[1,-3],[2,-3],[2,-2],[4,0],[17,-9],[3,-1],[2,-1],[3,-3],[2,-2],[2,3],[1,-3],[-1,-4],[2,-6],[2,-6],[2,-4],[0,-1],[-1,0],[0,-1],[-5,-4],[-10,0],[-5,-2],[-2,-3],[-4,-9],[-1,-2],[-2,0],[-9,-7],[0,-1],[-1,-1],[-2,0],[-1,-1],[0,-1]],[[6955,2237],[12,-21],[1,-3],[1,-6],[1,-3],[4,-6],[0,-2],[0,-4],[0,-3],[1,-2],[2,-2],[0,10],[-1,6],[-2,2],[-1,3],[-5,14],[-4,8],[-2,4],[-2,1],[-2,2],[-3,8]],[[6955,2364],[1,0],[0,4],[-2,6],[-1,7],[-2,4],[5,1],[5,5],[14,20],[3,1],[5,-6],[2,-1],[2,-2],[-2,-6],[-2,-8],[-1,-5],[1,-5],[1,-1],[10,0],[1,0],[2,2],[0,1],[1,1],[2,0],[2,-3],[2,-3],[1,-3],[6,-6],[2,-3],[1,-3],[0,-6],[0,-3],[1,-3],[2,-4],[1,-3],[2,-16],[1,-4],[1,-3],[-1,-2],[-2,-4],[0,-2],[0,-4],[1,-3],[4,-9],[2,-1],[6,-3],[-2,3],[-1,4],[-1,3],[-4,3],[-1,3],[1,4],[0,10],[1,1],[0,2],[2,4],[3,6],[2,2],[2,0],[-2,3],[-4,2],[-1,4],[1,3],[1,3],[2,2],[2,2],[3,7],[2,4],[4,2],[1,1],[6,0],[-2,4],[-8,0],[-3,2],[-2,6],[1,7],[1,7],[2,3],[-1,1],[-3,0],[-2,1],[0,2],[-3,5],[-1,1],[-3,1],[-3,1],[-1,4],[1,6],[-4,3],[-1,1],[-2,2],[-2,7],[-1,2],[-2,1],[1,5],[2,2],[2,1],[2,1],[1,-1],[2,-1],[2,0],[1,1],[3,2],[2,2],[3,3],[1,3],[1,4],[0,3],[-4,5],[0,4],[-2,6],[0,3],[2,2],[5,1],[9,5],[2,2],[1,7],[1,0],[1,0],[1,1],[0,2],[0,3],[-1,2],[-1,2],[5,0],[2,0],[2,3],[-1,1],[-1,2],[-1,0],[3,3],[2,1],[3,2],[2,4],[0,2],[1,6],[0,2],[1,2],[2,4],[0,2],[-2,9],[1,3],[3,2],[3,0],[9,-4],[0,2],[-2,2],[-1,2],[-1,2],[-2,1],[-5,0],[-3,0],[-2,-2],[-1,5],[2,4],[4,6],[1,6],[-1,3],[-2,2],[-1,5],[1,1],[1,2],[1,3],[-1,1],[-2,1],[-1,2],[0,3],[3,2],[1,2],[-1,2],[0,1],[-6,2],[1,2],[1,1],[1,1],[0,3],[-1,2],[-1,2],[-2,1],[-1,1],[-5,-2],[-2,0],[0,4],[-2,1],[-2,0],[-2,-2],[-2,-1],[1,1],[0,3],[0,1],[-1,0],[-3,1],[-2,1],[0,2],[2,0],[4,4],[2,0],[-1,2],[-1,2],[-1,1],[-1,0],[0,2],[2,0],[-1,3],[0,4],[-2,-2],[-2,0],[-2,1],[-1,2],[2,6],[0,3],[-2,1],[-4,0],[0,2],[2,0],[2,1],[1,3],[-1,4],[-2,1],[-3,1],[-3,2],[1,4],[2,2],[8,0],[2,2],[1,2],[0,3],[0,5],[-2,-3],[-2,0],[-1,-1],[-1,1],[1,2],[9,14],[-1,0],[-9,-3],[-2,-1],[-3,1],[-2,1],[-1,2],[-2,2],[-1,4],[-1,3],[-1,1],[3,3],[0,2],[0,5],[2,-2],[4,2],[1,-2],[1,1],[1,1],[0,1],[1,2],[-3,0],[-2,1],[-1,1],[-1,3],[-1,2],[0,2],[-2,-1],[0,-1],[-3,-2],[-3,1],[-4,2],[-3,3],[2,1],[3,2],[2,2],[1,4],[-2,2],[-1,0],[-2,-1],[-2,0],[7,24],[-3,3],[0,2],[2,1],[3,2],[-1,2],[-1,1],[-1,1],[-1,1],[1,1],[3,1],[1,0],[0,3],[-1,1],[-1,1],[-2,-1],[2,3],[1,1],[0,1],[-1,1],[-2,0],[-1,2],[-1,1],[-1,2],[0,1],[1,2],[0,3],[-1,3],[-1,2],[-1,2],[1,3],[1,3],[1,3],[0,2],[-1,3],[-1,1],[-1,1],[-1,2],[-1,6],[-1,2],[-1,0],[-2,0],[-1,-1],[-1,-1],[-2,-1],[-1,0],[0,2],[1,1],[0,2],[-2,2],[-2,2],[-1,4],[-2,3],[3,4],[2,2],[4,1],[1,0],[-1,-1],[-1,-1],[1,-1],[-1,-2],[2,-4],[3,0],[3,1],[2,3],[-1,1],[-3,2],[-1,1],[-1,4],[-1,1],[-1,1],[1,3],[2,-1],[3,-2],[2,-2],[3,1],[1,3],[-1,3],[-3,2],[0,1],[1,2],[-2,2],[-2,2],[-1,1],[-3,-4],[-2,0],[-2,0],[-1,-1],[0,3],[1,2],[1,3],[1,2],[2,-2],[1,1],[1,3],[-1,1],[-1,0],[-1,0],[-1,2],[-1,3],[1,0],[2,0],[1,1],[0,2],[-1,2],[-1,0],[-3,0],[-3,1],[-5,2],[-3,0],[1,3],[2,1],[8,-1],[1,2],[1,3],[-3,1],[0,3],[2,8],[-1,0],[-1,1],[-1,2],[6,5],[-2,2],[-2,0],[0,2],[5,2],[4,-3],[4,-3],[5,-1],[-4,6],[-4,3],[-11,0],[1,6],[-1,3],[-2,2],[-3,1],[3,3],[4,0],[3,-1],[3,0],[-3,5],[-9,4],[-4,2],[7,-3],[4,0],[2,3],[-2,0],[-4,2],[4,0],[-1,3],[-2,1],[-4,-1],[1,2],[1,0],[1,0],[2,0],[-1,1],[0,1],[0,1],[-1,0],[3,2],[9,-2],[-1,4],[-5,4],[-1,3],[-2,-1],[-7,5],[1,1],[1,1],[-5,3],[-15,0],[-3,2],[3,2],[11,-2],[0,5],[2,0],[3,-1],[2,1],[-2,2],[0,2],[1,1],[1,-2],[1,1],[0,1],[0,1],[-1,1],[-1,1],[-1,0],[-2,0],[2,2],[1,1],[4,0],[1,1],[2,3],[1,1],[-2,2],[-4,0],[-7,-2],[-6,-4],[-3,0],[0,4],[-2,-1],[-2,0],[-1,0],[-1,3],[-3,2],[-4,1],[-3,-1],[1,3],[2,2],[3,3],[0,1],[-1,2],[1,2],[1,-1],[2,-1],[1,1],[1,2],[-1,2],[-13,2],[-4,3],[2,3],[1,0],[-1,3],[-1,1],[-3,0],[1,5],[-1,5],[-2,4],[-3,1],[1,0],[0,1],[0,1],[1,1],[0,1],[-1,3],[0,1],[1,0],[2,0],[2,2],[0,1],[-4,2],[-2,0],[1,5],[1,3],[-1,2],[-1,1],[-1,2],[0,3],[0,2],[-1,0],[-1,0],[-1,1],[-1,1],[3,2],[-2,4],[-3,4],[-4,1],[0,2],[6,1],[2,2],[1,3],[-2,4],[-2,1],[-2,-1],[-3,-2],[1,-1],[1,-1],[1,0],[-3,-2],[-2,-1],[-1,0],[1,4],[2,3],[1,4],[0,2],[-11,0],[4,3],[1,2],[0,2],[-1,2],[-1,1],[-2,0],[1,-3],[-3,-2],[-7,0],[0,2],[2,1],[8,7],[1,1],[1,0],[2,1],[0,4],[-1,2],[1,3],[1,1],[-1,-2],[0,-1],[0,-3],[1,-1],[2,0],[2,3],[3,1],[2,2],[4,1],[1,1],[1,1],[2,1],[8,2],[2,2],[1,1],[5,0],[2,1],[3,4],[37,13],[2,2],[6,10],[3,2],[11,4],[5,5],[6,1],[5,4],[3,0],[3,2],[6,7],[2,0],[7,5],[6,1],[2,1],[1,2],[6,4],[5,6],[1,1],[3,1],[9,12],[12,10],[5,2],[15,14],[8,4],[2,2],[12,23],[2,1],[3,1],[45,52],[8,12],[3,4],[3,4],[2,6],[1,3],[-1,0],[-2,-2],[-3,-2],[-4,-3],[-2,0],[3,6],[14,18],[16,28],[7,8],[2,5],[0,2],[1,4],[0,2],[2,2],[1,1],[4,1],[4,4],[4,6],[2,7],[4,16],[1,6],[0,3],[3,15],[1,4],[4,9],[1,6],[1,6],[0,6],[0,5],[1,6],[0,6],[-1,6],[1,2],[0,2],[-1,3],[0,3],[0,1],[0,6],[0,15],[2,28],[0,4],[1,9],[0,4],[-1,4],[0,4],[-1,10],[-1,10],[-2,10],[0,10],[-1,8],[0,4],[0,5],[2,9],[1,3],[-1,3],[0,3],[-1,1],[-1,3],[-1,3],[0,2],[0,7],[-3,19],[-8,32],[0,3],[-2,1],[-1,2],[-2,6],[0,2],[1,0],[0,1],[0,1],[-2,1],[-2,6],[-4,14],[-3,10],[-1,4],[-1,14],[-6,11],[-4,11],[-1,1],[-1,2],[-1,3],[-1,4],[-1,2],[-1,3],[-3,5],[0,1],[-2,1],[-1,2],[-5,16],[-2,3],[0,2],[-1,1],[0,1],[-1,1],[-2,0],[0,1],[-1,1],[-1,1],[-1,2],[-1,5],[-1,3],[-6,9],[-7,8],[-2,1],[-5,1],[-2,1],[-2,3],[-3,6],[-2,2],[1,0],[-3,1],[-8,4],[2,2],[2,1],[1,1],[-1,5],[-2,2],[-7,4],[-7,10],[-3,2],[-8,2],[-2,1],[-7,7],[-21,10],[-6,6],[-2,0],[2,6],[0,3],[-2,1],[-2,1],[-2,5],[-1,1],[-4,-1],[-2,0],[-1,3],[-1,3],[-6,7],[-8,3],[-1,2],[0,2],[-4,9],[-7,10],[-4,4],[-2,-2],[1,-5],[4,-6],[7,-7],[-2,-2],[-4,1],[-3,2],[-2,4],[-1,1],[-1,1],[-1,1],[-1,4],[0,2],[-1,1],[-1,1],[-6,-2],[-3,1],[-2,3],[0,4],[1,3],[0,3],[-1,3],[2,1],[2,0],[2,1],[0,3],[2,1],[1,-1],[2,0],[1,2],[-1,3],[-1,3],[-1,2],[-5,5],[-1,1],[0,2],[0,4],[1,2],[1,-1],[2,-2],[0,-1],[1,1],[1,2],[0,3],[0,4],[-1,2],[0,3],[-2,4],[-2,5],[-1,5],[0,4],[0,2],[0,1],[1,3],[-3,1],[0,3],[1,2],[4,-1],[2,-3],[2,-4],[2,-4],[2,0],[-1,5],[2,4],[3,2],[3,0],[3,-1],[1,-2],[-1,-3],[-1,-5],[1,-1],[1,1],[4,2],[0,2],[0,2],[0,1],[1,3],[2,4],[5,6],[-4,-1],[-2,-1],[-2,-2],[0,1],[-1,1],[2,2],[2,2],[1,2],[1,4],[-1,0],[0,1],[-1,2],[0,2],[1,1],[1,2],[1,1],[1,0],[1,-2],[2,-2],[2,0],[1,2],[-1,3],[-2,8],[3,1],[2,-5],[3,1],[3,3],[0,4],[-1,4],[-2,3],[2,2],[4,-2],[2,2],[-1,3],[-1,6],[-1,3],[4,4],[9,4],[3,4],[2,2],[5,0],[3,3],[7,3],[-2,2],[-4,1],[-3,2],[3,1],[7,0],[3,2],[2,3],[0,3],[-5,4],[4,3],[11,5],[-1,1],[-3,1],[-1,1],[-1,-2],[0,-1],[-1,0],[-2,1],[-1,0],[-1,1],[0,3],[0,1],[-2,1],[-3,-1],[-4,-4],[-2,-1],[-2,-1],[-2,1],[-1,3],[1,4],[2,2],[3,2],[2,0],[0,1],[-4,7],[4,6],[12,7],[1,2],[0,4],[0,3],[2,11],[-1,2],[-4,0],[-1,2],[-2,5],[-1,3],[-1,3],[2,0],[2,0],[1,1],[1,2],[-1,2],[-2,1],[8,-1],[1,-1],[2,-4],[1,-2],[4,0],[3,1],[2,2],[1,2],[1,-1],[1,-2],[1,-2],[7,-7],[3,-5],[-1,-8],[4,-4],[4,-2],[10,0],[0,1],[-8,3],[-5,3],[-2,4],[0,3],[0,5],[0,4],[-2,3],[-2,2],[-7,1],[0,2],[1,0],[-1,0],[1,1],[2,1],[2,0],[1,3],[-2,0],[-1,2],[-2,3],[-1,2],[1,3],[2,0],[1,-2],[2,-1],[3,0],[1,2],[0,3],[1,3],[2,2],[2,1],[10,0],[3,0],[1,2],[-7,16],[-1,5],[-1,3],[-3,2],[-3,1],[0,2],[1,1],[5,0],[2,1],[-1,2],[-1,1],[-1,1],[-1,0],[2,3],[2,2],[3,1],[3,0],[-2,2],[-2,2],[2,1],[6,2],[3,0],[0,1],[-2,0],[-3,1],[-2,2],[-2,2],[4,1],[4,1],[4,3],[2,4],[-3,-1],[-10,-5],[-3,0],[-2,1],[0,5],[-1,1],[-1,0],[-2,1],[-1,-1],[-1,-1],[-1,-3],[-1,-1],[-6,2],[-9,0],[-1,0],[-2,3],[-1,0],[-5,0],[1,2],[4,3],[1,2],[-1,1],[0,1],[-1,3],[5,4],[2,1],[2,-2],[1,-1],[2,1],[1,0],[1,1],[1,0],[6,-1],[0,2],[-8,4],[-2,2],[-4,6],[-2,3],[-1,3],[5,-1],[3,1],[2,2],[-6,9],[-3,2],[-1,1],[0,1],[3,3],[-3,0],[-4,-4],[-2,-1],[-1,2],[1,4],[3,8],[-2,3],[-1,1],[-2,1],[0,1],[0,2],[0,1],[-2,2],[-5,2],[-2,2],[-3,5],[0,5],[2,5],[3,4],[12,10],[3,3],[0,2],[0,3],[1,1],[1,1],[15,0],[-3,3],[-2,2],[-10,2],[-3,0],[-1,-1],[-3,-2],[-1,-1],[-6,0],[-2,0],[-10,8],[-2,3],[-4,9],[0,2],[2,1],[6,7],[4,4],[2,2],[4,6],[5,5],[2,1],[-3,4],[-4,0],[-9,-2],[3,4],[24,13],[0,1],[-6,0],[-38,-21],[1,3],[1,2],[2,4],[-2,2],[-10,-2],[10,9],[-4,2],[-5,-2],[-10,-5],[-4,0],[-13,-4],[3,16],[1,3],[3,1],[2,3],[3,7],[8,9],[1,2],[1,3],[13,15],[2,4],[1,1],[2,0],[3,0],[1,0],[0,2],[0,3],[0,3],[2,14],[0,2],[1,1],[2,5],[1,1],[6,2],[1,4],[-1,2],[-5,3],[0,3],[2,2],[0,2],[1,3],[-1,2],[-1,3],[-1,1],[0,13],[0,6],[-1,6],[-3,13],[1,2],[3,-1],[1,-1],[2,-4],[3,-1],[2,1],[-1,7],[9,0],[4,1],[3,4],[-10,5],[-4,5],[2,6],[1,2],[2,1],[2,0],[4,-4],[11,-2],[-3,5],[-5,3],[-10,2],[1,3],[2,0],[2,-1],[2,1],[0,2],[-3,1],[-10,0],[-3,2],[-3,8],[-2,2],[-1,2],[-3,9],[-1,0],[-2,-3],[-14,1],[-3,2],[-2,3],[-2,4],[0,3],[0,2],[0,3],[-1,3],[0,1],[-1,1],[0,2],[-1,2],[0,4],[-1,2],[-1,3],[-2,4],[-1,4],[-1,4],[0,4],[1,4],[-1,4],[0,5],[-1,4],[-4,4],[-1,5],[0,6],[-1,17],[0,5],[-1,4],[-1,5],[1,6],[1,5],[1,5],[-2,10],[0,5],[3,3],[1,0],[1,2],[0,2],[0,3],[1,2],[9,11],[1,1],[4,0],[5,3],[2,3],[1,1],[2,1],[12,10],[2,1],[6,0],[3,1],[1,3],[2,6],[5,4],[9,4],[1,2],[3,6],[1,2],[2,1],[8,-1],[0,-1],[1,-4],[1,-2],[1,-1],[1,0],[0,1],[1,0],[9,-2],[3,0],[1,-1],[9,0],[0,-1],[1,0],[1,-2],[1,-1],[0,-1],[1,0],[2,1],[4,1],[1,-1],[1,-1],[1,-1],[1,-1],[3,4],[2,1],[3,1],[2,-1],[2,-1],[1,-2],[2,-1],[17,-3],[3,-1],[1,-3],[2,-1],[5,-1],[2,-2],[-1,-1],[3,-3],[3,-1],[7,1],[8,-6],[7,-1],[11,-6],[10,-2],[3,-3],[5,0],[2,-1],[1,-2],[1,-5],[0,-1],[2,-1],[1,1],[2,1],[2,1],[1,-1],[2,-2],[1,-1],[2,1],[4,2],[2,1],[1,-1],[2,-4],[7,-3],[3,-3],[19,-4],[8,-5],[0,-2],[-14,-7],[-4,-5],[-2,-2],[-5,-1],[-2,-2],[-1,-2],[-6,-6],[7,0],[2,1],[4,5],[5,2],[6,5],[1,1],[11,6],[9,12],[3,2],[16,3],[17,-5],[3,-3],[1,-3],[0,-4],[-1,-5],[2,0],[3,7],[2,1],[-1,0],[5,2],[5,-1],[13,-9],[6,-7],[6,-5],[0,-1],[1,-1],[1,-4],[1,-2],[10,-6],[8,-4],[5,0],[0,2],[-3,1],[-6,6],[-5,2],[-7,7],[-2,3],[1,4],[1,5],[2,3],[2,2],[33,0],[3,1],[1,2],[1,1],[1,2],[2,1],[3,1],[3,3],[2,6],[3,2],[2,1],[1,0],[3,-3],[2,0],[3,0],[1,1],[6,6],[8,2],[2,1],[2,3],[3,7],[2,3],[4,2],[1,4],[2,3],[0,2],[14,2],[6,2],[3,0],[3,-2],[2,-3],[4,-11],[2,-5],[3,-1],[2,0],[3,0],[2,-1],[1,-2],[0,-1],[1,-1],[2,-1],[2,1],[1,1],[2,1],[3,-2],[3,-3],[2,-4],[2,-3],[8,-2],[4,-2],[1,-3],[-1,-6],[2,-3],[13,-11],[1,-3],[0,-2],[1,-2],[2,-1],[4,-1],[5,-2],[4,-3],[4,-4],[-2,-2],[-1,-1],[-1,-1],[-2,-1],[14,-1],[5,-2],[0,2],[-5,5],[2,1],[6,2],[2,0],[2,-1],[3,-5],[7,-2],[1,-4],[-2,-4],[-3,-5],[4,0],[2,-1],[2,-3],[-2,-1],[0,-1],[0,-2],[2,-2],[-1,-4],[0,-4],[0,-4],[-2,-2],[-6,-1],[-1,-2],[-1,-1],[1,-5],[0,-4],[-1,-2],[-12,-16],[5,0],[3,3],[4,7],[3,2],[0,-8],[4,-7],[3,-2],[0,7],[-2,6],[-1,7],[0,6],[3,5],[1,0],[3,-2],[9,-3],[15,-8],[14,-2],[3,-4],[1,-2],[0,-2],[1,-2],[1,-3],[4,-4],[1,-2],[-2,0],[-6,-2],[1,-1],[1,-1],[0,-1],[-3,-1],[-1,-3],[0,-4],[2,-2],[1,-1],[2,1],[1,1],[2,3],[1,0],[8,-3],[2,-2],[1,-2],[1,-3],[0,-2],[0,-2],[2,-1],[2,-1],[0,-1],[1,-2],[-1,-4],[-3,-5],[-4,-6],[-2,-2],[-3,-1],[-2,0],[-9,3],[-4,0],[-4,-3],[-3,-9],[7,8],[2,1],[12,-3],[4,0],[1,0],[4,3],[2,0],[3,0],[2,0],[0,2],[1,2],[0,3],[0,2],[1,2],[1,0],[1,2],[1,3],[-1,2],[0,1],[-1,2],[-2,3],[-1,1],[2,1],[11,-3],[6,-4],[5,-1],[1,-1],[3,-2],[6,-2],[3,-2],[8,-8],[1,-3],[-2,0],[1,-1],[2,-3],[1,-3],[0,-3],[-2,-1],[-4,2],[-2,0],[-1,-1],[0,-2],[-1,-2],[-6,-1],[-13,0],[-6,-2],[1,-2],[2,-1],[5,2],[0,-1],[1,-3],[0,-2],[-2,-1],[0,-2],[3,-2],[1,-1],[2,0],[-3,-4],[-4,-3],[-9,-3],[0,-2],[4,-1],[6,0],[7,-4],[13,2],[4,-2],[-5,-3],[-2,-1],[-1,-1],[-1,-1],[-1,-2],[-4,-2],[-13,2],[2,-1],[1,0],[0,-2],[-2,-2],[-1,0],[2,-2],[3,0],[5,0],[2,0],[4,-3],[2,0],[-1,-4],[-2,-2],[-4,-4],[0,-1],[4,-3],[11,-4],[-3,-6],[0,-3],[-1,-4],[2,1],[2,3],[1,1],[2,-1],[0,-2],[1,-2],[1,-2],[-3,-6],[0,-3],[1,-1],[2,0],[2,1],[1,1],[2,2],[-2,3],[-1,3],[0,3],[1,4],[3,1],[3,-1],[1,-3],[1,-5],[1,-2],[6,-8],[0,-2],[2,-3],[2,-1],[-2,6],[11,1],[5,3],[2,0],[-1,-4],[2,-3],[1,-3],[1,-1],[1,4],[1,1],[7,-4],[2,0],[2,2],[4,-1],[9,-3],[3,-3],[1,-3],[-2,-5],[11,1],[2,-2],[1,-2],[2,-1],[1,0],[2,0],[1,0],[0,2],[-1,1],[-1,1],[2,3],[4,-1],[8,-6],[0,-2],[-1,-1],[-2,-3],[2,-1],[7,1],[6,-4],[2,-1],[0,2],[-1,1],[-1,1],[-1,2],[-1,3],[1,0],[2,0],[1,0],[0,1],[-1,1],[0,1],[2,1],[2,0],[2,-1],[2,-1],[0,-1],[0,-3],[1,-2],[1,0],[3,-1],[0,3],[-1,3],[-1,2],[2,2],[0,-1],[1,-1],[2,0],[-1,3],[-1,1],[-1,1],[0,2],[3,-1],[4,-4],[3,0],[-1,4],[1,2],[2,2],[2,-1],[1,-1],[3,-1],[2,-3],[1,-1],[2,0],[1,1],[2,2],[1,1],[4,0],[1,-1],[-1,-3],[2,-1],[1,0],[2,0],[0,-2],[-1,-1],[-3,-3],[-1,-1],[-1,-4],[2,-3],[2,0],[3,2],[1,-2],[-2,-1],[0,-2],[1,-5],[0,-1],[1,-1],[0,-2],[0,-1],[0,-2],[0,-1],[-1,-3],[0,-1],[0,-1],[1,-1],[0,-1],[0,-2],[0,-1],[-1,-2],[-6,-1],[0,-2],[9,0],[2,-1],[-1,-5],[1,-2],[2,0],[5,0],[1,0],[1,-1],[2,-3],[0,-1],[3,-3],[2,-1],[2,1],[1,3],[4,3],[1,3],[-3,0],[-4,-4],[-2,0],[-2,1],[2,3],[3,2],[1,1],[3,5],[5,4],[5,4],[4,1],[0,-9],[4,-3],[5,3],[1,7],[-1,3],[-2,4],[-2,4],[1,3],[0,2],[1,5],[0,2],[1,3],[1,2],[1,2],[0,2],[0,2],[0,2],[2,1],[1,1],[3,3],[1,1],[4,-1],[2,-1],[2,-3],[1,-3],[4,-13],[1,-2],[2,-2],[2,-5],[1,-6],[0,-4],[1,0],[5,-2],[-1,-3],[-1,-5],[-1,-3],[1,-4],[1,-5],[1,-2],[-2,-6],[-3,-4],[-4,-2],[-7,-1],[-7,-4],[-4,-4],[-4,-7],[-1,-1],[-2,0],[-4,-3],[-2,-2],[2,-2],[1,-3],[1,-5],[0,-4],[-1,-4],[1,-1],[2,-1],[1,-4],[-1,-4],[-2,-3],[-14,-5],[-2,-2],[1,-2],[3,-6],[4,-5],[1,-2],[-1,-4],[-2,-3],[0,-2],[1,-2],[2,0],[1,-2],[1,-6],[1,-2],[-1,-3],[-1,-4],[-2,-3],[-1,-3],[0,-1],[1,-3],[1,0],[3,-1],[1,0],[5,-9],[0,-2],[1,-3],[5,-2],[1,-3],[-1,-2],[-3,-5],[-1,-2],[6,0],[0,-2],[0,-2],[-1,-5],[0,-4],[0,-4],[0,-3],[-3,-2],[1,-4],[-2,-2],[-6,-1],[-1,0],[-1,-4],[-1,-1],[-2,1],[-4,4],[-3,2],[-3,0],[-1,-4],[-1,-7],[2,-1],[4,-2],[2,-3],[-2,-3],[-3,0],[-9,3],[-45,5],[-5,2],[0,-2],[1,0],[3,-3],[-3,-1],[-27,-3],[-3,1],[-2,1],[-4,5],[-39,13],[-5,-3],[13,-2],[5,-5],[2,-2],[6,-1],[4,-2],[6,-2],[16,-7],[11,-2],[8,5],[3,0],[2,0],[3,-3],[2,0],[11,2],[1,0],[6,-5],[6,-3],[7,-1],[12,4],[10,-1],[10,2],[3,0],[2,-2],[0,-6],[-1,-4],[-1,-1],[-3,0],[5,-4],[15,-6],[3,-2],[0,-2],[-3,-3],[-1,-3],[0,-2],[0,-2],[0,-2],[-1,-3],[-1,-3],[1,-1],[2,0],[2,-1],[1,-1],[2,-1],[-2,-2],[-3,-5],[-4,-3],[-2,-4],[-2,-1],[1,-1],[1,-1],[3,0],[1,0],[1,-1],[0,-1],[1,-1],[6,-1],[2,-1],[0,-6],[-2,-7],[-4,-5],[-2,-2],[-4,-1],[-4,-3],[-4,-4],[-6,-10],[-1,-2],[0,-3],[0,-1],[0,-1],[1,-1],[1,-1],[5,1],[-1,-3],[1,-1],[1,-1],[2,-2],[0,-1],[1,-3],[1,-3],[1,-3],[-1,0],[-1,-2],[-1,-2],[0,-2],[3,0],[1,-1],[0,-2],[-1,-3],[-12,-5],[-1,-3],[0,-3],[2,-3],[2,0],[11,0],[1,0],[0,2],[1,1],[1,1],[2,2],[2,1],[4,-1],[9,7],[3,0],[3,-1],[2,-2],[2,-3],[-1,-4],[1,-1],[2,0],[5,5],[3,0],[2,-3],[1,-5],[-1,-1],[-1,-2],[0,-3],[1,-2],[1,-1],[1,-2],[0,-4],[-1,-3],[-3,-2],[-3,-1],[-2,1],[-3,5],[-1,1],[-2,0],[0,-1],[0,-2],[0,-5],[0,-1],[0,-1],[-1,-2],[-1,0],[-2,0],[-1,1],[-1,1],[0,2],[0,2],[-1,1],[-1,2],[-1,-1],[-9,-10],[-2,-4],[1,-4],[0,-1],[-1,0],[0,-2],[1,-1],[-1,-2],[1,-3],[2,-2],[2,0],[1,2],[0,1],[1,1],[0,1],[1,1],[1,1],[1,-1],[1,-3],[1,-1],[0,-1],[1,0],[1,-1],[0,1],[2,2],[1,0],[2,0],[1,-1],[2,-2],[1,-3],[-6,-6],[-2,-4],[-2,-7],[-2,1],[-1,2],[-1,2],[-1,3],[-1,2],[-1,1],[-1,1],[4,-34],[2,-8],[0,-1],[-2,-4],[-2,-2],[-1,2],[-1,1],[-1,2],[-2,-1],[0,-2],[0,-5],[-1,-6],[-2,-5],[-1,-2],[-9,-3],[-3,0],[-2,3],[-4,15],[3,4],[2,3],[1,3],[0,5],[-1,1],[-1,-1],[-1,-3],[-1,0],[-1,1],[0,2],[-1,2],[2,6],[-3,5],[-3,1],[-1,-4],[-3,2],[-5,16],[-3,3],[-2,-1],[1,-4],[1,-8],[0,-4],[-1,-3],[-1,-2],[-2,-1],[4,0],[5,-4],[2,-7],[-2,-9],[1,-2],[1,-4],[0,-4],[0,-3],[-2,0],[-2,2],[-2,5],[-1,2],[-2,2],[-1,1],[-2,0],[1,-7],[1,-2],[0,-1],[-8,-5],[-2,0],[-5,0],[-1,0],[-1,-1],[-3,-3],[-1,-1],[-7,-2],[-8,1],[-3,-1],[-2,-3],[6,1],[3,0],[3,-4],[1,-1],[9,0],[1,-3],[1,-1],[2,0],[3,2],[3,3],[2,3],[1,1],[2,1],[1,-1],[-1,-3],[-1,-5],[0,-4],[1,-6],[1,-4],[2,2],[2,4],[2,1],[2,0],[5,-2],[0,-1],[0,-3],[-1,-2],[-1,-1],[-2,-1],[-1,-1],[-1,-1],[-2,-4],[-1,-1],[1,-2],[1,-2],[2,-3],[1,-3],[3,-1],[2,0],[0,5],[1,3],[5,5],[1,5],[2,4],[1,1],[0,2],[1,0],[2,-1],[1,0],[2,1],[1,3],[1,2],[0,4],[3,5],[5,4],[9,4],[7,9],[1,3],[2,4],[2,3],[2,1],[1,0],[2,1],[3,1],[3,3],[3,1],[7,-1],[8,1],[16,0],[-1,-1],[0,-2],[-1,-2],[12,0],[12,10],[1,0],[1,-1],[1,-1],[1,-1],[2,-1],[10,-2],[3,-1],[9,-8],[3,-5],[1,-1],[10,-2],[1,-2],[1,-2],[1,-2],[2,-3],[1,-1],[0,-3],[0,-3],[-2,-4],[-1,-2],[0,-6],[1,-2],[1,-1],[4,-4],[1,-1],[0,-5],[-3,-8],[0,-5],[2,-3],[5,2],[2,-3],[0,-4],[-2,-5],[0,-3],[2,-3],[2,1],[5,5],[2,0],[-2,-10],[-1,-5],[0,-5],[1,-2],[1,-3],[-1,-3],[-1,-2],[-1,-2],[0,-5],[-2,-10],[-8,-25],[-3,-15],[0,-3],[-1,-1],[0,-3],[1,-2],[0,-1],[0,-3],[-1,-2],[-3,-3],[-11,-17],[-5,-3],[-23,-4],[-14,-9],[-14,-4],[-1,-2],[-1,-2],[-1,-2],[-5,-4],[-1,-1],[-10,-4],[-10,-9],[-16,-22],[25,24],[2,4],[2,2],[8,4],[10,8],[22,7],[8,5],[9,1],[16,6],[6,4],[1,2],[0,2],[1,2],[1,2],[1,2],[3,1],[1,1],[3,5],[1,3],[0,3],[0,10],[1,6],[1,5],[1,2],[3,2],[2,3],[0,5],[0,4],[0,5],[2,5],[4,9],[0,1],[-1,3],[0,2],[1,1],[0,1],[1,2],[0,2],[0,2],[0,14],[0,2],[1,2],[3,6],[2,2],[1,-2],[3,-1],[2,0],[2,0],[3,1],[0,1],[0,2],[0,4],[2,7],[4,-3],[2,-6],[-1,-5],[0,-1],[1,-1],[2,-1],[3,0],[2,-3],[0,-3],[-2,-4],[-1,-4],[-1,-10],[0,-2],[1,-1],[0,-1],[1,-1],[0,-2],[0,-1],[0,-2],[0,-1],[1,-1],[4,-8],[0,-5],[0,-4],[-2,-5],[-2,-4],[-9,-11],[-3,-1],[-9,-12],[-4,-14],[-1,-3],[1,-3],[2,3],[4,7],[2,2],[4,3],[2,2],[1,2],[2,9],[2,6],[3,3],[4,2],[3,1],[0,5],[2,5],[5,5],[-2,5],[0,4],[0,15],[0,2],[1,2],[3,7],[0,1],[0,4],[0,1],[1,2],[2,-1],[2,-1],[3,-2],[0,-1],[0,-1],[0,-2],[1,0],[2,-1],[1,-4],[-2,-5],[-1,-5],[2,-5],[-5,-7],[0,-3],[2,-5],[-1,-1],[-1,-2],[2,0],[2,0],[2,-2],[2,-3],[1,-3],[1,-4],[-1,-4],[-5,-12],[0,-7],[1,-8],[2,-9],[-2,-11],[-1,-3],[-2,-3],[-2,-4],[-1,-5],[0,-5],[1,-1],[1,-2],[1,-1],[1,1],[1,2],[-1,3],[-1,4],[1,6],[1,3],[5,6],[1,5],[1,3],[-1,4],[-2,4],[0,5],[2,5],[4,9],[1,6],[3,6],[1,1],[0,1],[2,-1],[1,1],[4,7],[2,3],[2,3],[3,0],[5,-1],[4,1],[3,9],[5,3],[7,1],[1,1],[2,1],[2,5],[3,2],[2,-1],[2,-2],[1,-1],[2,0],[1,0],[1,2],[1,3],[1,3],[-1,1],[0,2],[-1,4],[4,0],[2,1],[2,2],[4,10],[1,2],[2,3],[4,0],[3,-3],[-1,-9],[2,-1],[2,-1],[0,-2],[1,-2],[0,2],[1,3],[0,1],[3,4],[1,3],[0,5],[11,0],[5,-2],[3,0],[2,2],[-5,7],[-1,3],[4,2],[1,1],[1,3],[-1,3],[0,4],[1,3],[2,3],[1,3],[0,3],[0,3],[-1,3],[1,2],[1,-1],[1,-1],[1,3],[1,7],[3,2],[4,0],[3,1],[2,5],[-3,2],[-1,1],[-1,2],[2,2],[2,4],[0,3],[-1,3],[1,3],[2,2],[2,1],[2,0],[0,1],[1,1],[1,1],[1,1],[1,-1],[1,-1],[2,0],[0,-1],[-1,-1],[1,-1],[6,-7],[4,-2],[2,-3],[3,-7],[2,-2],[6,-3],[1,-1],[2,-1],[1,-3],[0,-3],[1,-4],[1,-3],[1,-3],[2,-2],[2,0],[2,1],[2,0],[1,-2],[-1,-3],[2,-1],[4,-1],[1,-1],[1,-3],[0,-3],[-1,-4],[-2,-3],[0,-1],[-1,0],[0,-2],[-1,-2],[0,-1],[-1,-2],[-2,-2],[-1,-4],[0,-3],[0,-2],[0,-2],[2,-6],[1,-6],[-1,-6],[-2,-3],[-6,-4],[-3,-4],[3,-6],[1,-2],[1,1],[1,3],[0,4],[1,4],[2,2],[2,1],[2,1],[1,1],[2,2],[0,2],[0,2],[0,2],[0,1],[3,2],[1,0],[-1,4],[-2,1],[-3,1],[-1,2],[0,3],[2,3],[5,6],[1,1],[1,2],[0,2],[0,3],[0,1],[1,2],[1,2],[1,1],[4,-3],[2,2],[-4,3],[0,1],[-1,0],[-1,6],[0,1],[-2,3],[-3,1],[-2,1],[-1,5],[0,2],[-2,3],[-6,0],[-2,2],[0,3],[0,5],[0,4],[-1,2],[-6,4],[-1,2],[2,2],[6,14],[1,3],[1,1],[1,-1],[4,-8],[3,4],[10,-2],[4,1],[2,2],[1,2],[0,2],[-1,1],[-9,1],[-6,2],[-3,3],[-2,3],[0,3],[4,2],[3,2],[6,2],[1,0],[1,5],[-2,4],[-2,3],[0,4],[2,0],[13,-8],[5,-6],[2,-4],[2,-1],[0,2],[0,3],[0,2],[-1,2],[-1,2],[-1,1],[-11,6],[1,3],[2,1],[4,-1],[-1,2],[-4,2],[3,0],[3,1],[5,4],[0,-2],[3,0],[6,-8],[11,-6],[2,1],[-2,3],[-1,2],[-2,1],[-2,0],[-1,1],[-4,6],[1,2],[5,3],[1,0],[2,0],[2,-1],[2,-2],[4,-2],[11,-1],[3,1],[0,2],[-3,3],[-9,-1],[-5,5],[-5,2],[-2,1],[0,1],[-1,1],[0,1],[-1,-1],[-1,-1],[0,-2],[0,-1],[-1,-2],[-2,-1],[-2,-1],[-3,0],[-2,1],[-1,2],[-1,7],[-5,9],[-1,2],[0,4],[1,3],[1,1],[2,1],[-2,3],[-2,1],[-2,1],[-1,4],[1,2],[-1,3],[0,2],[0,2],[2,0],[5,-1],[4,-5],[6,-13],[0,4],[-2,10],[1,2],[1,-1],[1,-1],[0,-1],[2,0],[0,1],[1,1],[0,2],[2,1],[3,-3],[3,1],[0,2],[-3,2],[-3,5],[-2,1],[-1,2],[-1,5],[0,5],[-1,4],[3,3],[5,-5],[16,-18],[2,0],[-6,14],[-2,1],[-2,2],[0,3],[0,6],[1,3],[2,1],[4,-1],[-3,4],[-10,4],[0,2],[3,0],[0,2],[-3,2],[-3,1],[-3,2],[-1,4],[4,0],[7,-4],[4,0],[5,3],[2,0],[13,-3],[5,-2],[1,0],[1,0],[0,-1],[1,-2],[0,-2],[1,-1],[12,-12],[1,-2],[1,0],[5,-1],[2,1],[-6,2],[-5,5],[-5,6],[-6,10],[-3,1],[-5,2],[-7,5],[-9,1],[-2,1],[-2,3],[-7,23],[-4,19],[-1,3],[-2,3],[3,5],[4,2],[2,3],[-3,5],[1,2],[3,-2],[1,-1],[1,0],[0,1],[1,4],[2,2],[2,-1],[2,-2],[2,-1],[-1,2],[-1,3],[0,2],[2,1],[1,1],[2,2],[2,1],[1,-1],[11,-6],[2,-2],[3,-5],[6,-2],[4,-3],[3,-1],[8,0],[0,1],[-8,2],[-1,1],[-3,3],[-6,2],[-6,7],[-3,2],[0,2],[2,1],[0,2],[-2,3],[-1,2],[0,2],[1,1],[1,0],[0,-2],[2,0],[3,-2],[3,-2],[2,4],[-1,3],[-3,2],[-6,2],[0,2],[4,-1],[1,1],[2,1],[1,0],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[1,1],[1,-1],[2,-1],[1,-1],[2,-4],[1,4],[-1,3],[-5,5],[1,1],[2,0],[2,-1],[2,0],[-1,3],[-2,1],[-2,1],[-1,2],[-1,3],[0,3],[0,2],[0,1],[6,0],[3,2],[2,7],[-3,1],[-1,2],[0,2],[1,2],[2,1],[2,1],[1,0],[0,-2],[1,1],[1,2],[0,1],[0,1],[0,1],[-1,3],[1,1],[1,1],[1,-1],[1,1],[0,4],[2,5],[3,1],[3,-1],[2,3],[0,3],[-9,10],[6,3],[2,2],[3,4],[1,4],[2,4],[-1,3],[-2,3],[0,1],[1,1],[1,0],[1,0],[2,-1],[4,0],[2,0],[4,-4],[2,0],[3,-2],[1,0],[-1,2],[0,1],[-2,0],[0,2],[3,-1],[5,-1],[2,-2],[2,-2],[6,-6],[1,0]],[[8398,754],[1,-4],[2,1],[1,-3],[0,-3],[-2,2],[-1,-3],[1,-3],[1,-7],[2,-7],[0,-5],[2,-3],[2,-2],[0,-3],[0,-2],[-2,-2],[-2,3],[-1,4],[-1,3],[-1,1],[-2,-1],[-2,2],[-1,-1],[-2,-1],[-3,-1],[-2,-3],[-1,-2],[-1,-5],[-2,-1],[-1,1],[0,5],[1,6],[1,9],[2,5],[1,4],[1,6],[3,8],[3,3],[3,-1]],[[8654,1497],[0,-4],[0,-3],[-1,-4],[-3,-5],[-6,-10],[-6,-8],[-1,3],[-1,2],[-2,0],[-2,1],[5,6],[1,2],[-1,3],[-1,0],[-2,-1],[-1,-1],[1,2],[0,1],[1,2],[1,1],[-2,0],[-2,0],[0,2],[0,3],[1,2],[1,3],[2,2],[1,1],[7,3],[3,-1],[-1,-5],[2,2],[2,0],[2,1],[2,4],[-1,3],[1,3],[3,7],[0,-2],[0,-1],[-1,-5],[-1,-5],[-1,-4]],[[8647,1505],[-3,-1],[0,3],[2,5],[4,8],[2,12],[1,3],[3,-3],[1,-5],[1,-4],[0,-5],[-2,-5],[-2,-3],[-7,-5]],[[8706,1043],[-1,-4],[-2,-1],[-3,1],[-2,0],[-3,-1],[-2,-2],[-2,-3],[-1,-5],[-1,-4],[-2,-2],[-2,-3],[-1,-3],[-1,-3],[-1,-3],[-1,-3],[-1,-2]],[[8680,1005],[0,2],[-5,8],[-2,2],[-1,1],[-2,-1],[0,-2],[1,-2],[2,0],[0,-2],[-3,-3],[-6,-14],[-3,-3],[-2,-1],[-1,-3],[0,-5],[-1,-3],[-2,-1],[-2,-1],[-2,1],[-1,2],[2,3],[2,5],[1,7],[1,6],[-1,2],[-3,3],[-1,2],[0,3],[0,3],[0,7],[-2,-4],[-1,-1],[-1,1],[-7,17],[-2,4],[-3,3],[-1,2],[0,2],[0,6],[0,3],[-1,2],[-1,4],[-3,4],[-2,3],[-2,-1],[6,-9],[1,-4],[-1,-4],[0,-2],[1,-2],[1,-3],[3,-4],[0,-2],[1,-3],[2,-1],[1,-4],[2,-7],[1,-3],[3,-4],[1,-3],[-2,-8],[-3,-6],[-4,-6],[-3,-6],[1,-1],[2,-4],[1,-2],[-2,-1],[-2,2],[-2,-1],[-1,-2],[-1,-6],[0,-2],[-2,-2],[-3,-11],[-2,-3],[-2,0],[0,1],[-1,2],[-1,2],[-1,0],[-8,-1],[-2,-1],[-2,-2],[-2,-3],[-6,-12],[-3,-1],[-2,-1],[-17,-9],[-15,-13],[-19,-21],[-4,-1],[-1,-2],[-1,-2],[0,-3],[-1,-2],[-1,-1],[-7,-2],[-1,-1],[-4,-4],[-4,-3],[-4,-7],[-2,-2],[-1,0],[-2,4],[-2,0],[-1,-1],[-2,-4],[-3,-2],[-4,-5],[-3,-1],[-2,1],[-4,2],[-2,1],[-1,1],[-2,4],[-1,6],[-2,3],[-11,7],[11,14],[4,7],[0,2],[0,2],[0,6],[0,2],[-1,2],[0,2],[0,-1],[-1,-3],[-5,-6],[-1,-6],[-2,-6],[0,-1],[0,-1],[0,-2],[-1,-1],[-2,1],[-1,1],[-5,-3],[-2,-1],[-1,-3],[0,-1],[2,-5],[1,-3],[2,-2],[1,-1],[1,-1],[3,-1],[2,1],[2,-1],[1,-4],[-2,-2],[-4,-3],[-3,-5],[0,-4],[-5,-6],[-1,-1],[-1,-3],[-1,2],[-2,7],[-2,2],[-4,1],[-3,-1],[-2,-4],[3,0],[3,0],[3,-1],[-1,-5],[-3,-3],[-7,-2],[-1,-3],[0,-3],[-2,-1],[-1,1],[-1,-2],[-3,-4],[-2,-2],[-1,-1],[-1,-1],[-1,0],[-1,2],[0,3],[1,2],[0,1],[0,1],[-4,10],[-1,-2],[-2,-1],[-2,0],[-1,-1],[-13,-13],[-3,-2],[-3,1],[1,2],[-1,1],[-1,0],[-1,-4],[-1,-3],[-3,-1],[-2,0],[-1,1],[1,4],[1,4],[-3,3],[0,-4],[-1,-1],[-2,0],[-1,0],[-4,-6],[-2,-1],[-2,9],[0,4],[1,2],[2,0],[2,1],[2,1],[-1,2],[-1,1],[-2,1],[-2,0],[-2,2],[-1,3],[-1,3],[-1,2],[-1,0],[-2,-2],[-1,-2],[0,-3],[-1,1],[-2,1],[-2,0],[-2,-6],[0,-2],[0,-4],[-1,-5],[1,-5],[-3,1],[-2,4],[-4,11],[-1,3],[0,2],[-1,8],[0,2],[-1,0],[-2,-2],[0,4],[0,2],[-1,2],[-2,1],[0,-4],[1,-4],[0,-3],[1,-2],[-3,2],[-2,1],[-3,2],[-1,1],[-1,-1],[-1,-1],[-1,-2],[0,-2],[0,-2],[-4,-4],[-4,3],[-4,6],[-4,11],[-5,7],[-1,4],[1,4],[1,4],[3,8],[0,9],[-6,16],[-1,10],[3,3],[3,4],[2,2],[0,9],[-1,7],[-2,3],[-3,0],[-3,0],[-6,0],[-6,2],[-6,4],[-3,4],[-2,3],[0,3],[-1,2],[-1,2],[-1,-1],[-1,-2],[-1,-1],[-1,1],[-3,2],[-2,2],[0,4],[-1,12],[1,4],[1,4],[2,4],[0,3],[0,3],[-2,5],[-1,4],[2,3],[2,4],[1,3],[0,3],[-1,1],[-1,2],[-1,2],[0,2],[0,33],[0,32],[-1,33],[0,32],[0,32],[-1,33],[0,32],[0,33],[-1,9],[0,2],[-5,4],[-2,2],[-3,8],[-5,6],[-1,2],[0,3],[-1,3],[-3,6],[-6,5],[-7,10],[-1,2],[-4,2],[-1,1],[-3,6],[-2,2],[-5,5],[-6,2],[-6,-2],[-3,-8],[-1,-3],[0,-2],[0,-1],[-3,-1],[-1,-1],[-1,-1],[-4,4],[-3,0],[-3,0],[-2,-2],[-5,-4],[-3,-3],[-2,-3],[-2,-3],[-3,0],[-5,0],[-5,-2],[-15,-11],[-2,0],[-15,14],[-2,5],[-1,5]],[[8390,1527],[1,-1],[2,-1],[3,0],[4,-1],[2,0],[5,5],[9,1],[3,3],[3,4],[19,8],[4,-1],[-1,-7],[2,-5],[3,-2],[3,-1],[3,0],[1,-1],[1,-3],[1,-1],[3,2],[1,-1],[3,-3],[23,-13],[12,-2],[6,-2],[5,-6],[3,-8],[6,-30],[1,-4],[2,-3],[4,-6],[-4,-3],[-1,-1],[1,-2],[1,-2],[1,-2],[1,-2],[4,8],[1,3],[3,3],[8,0],[2,1],[5,5],[8,14],[9,5],[8,9],[21,7],[6,-2],[-9,-9],[-2,-6],[1,1],[3,1],[5,1],[10,5],[5,1],[4,0],[3,-2],[1,-3],[0,-6],[-2,-2],[-6,-1],[0,-1],[2,-1],[4,-1],[2,-2],[1,-1],[0,-4],[0,-1],[2,-1],[1,2],[0,2],[-1,0],[1,2],[-1,3],[0,2],[1,1],[2,1],[1,1],[1,-1],[4,-5],[2,-2],[-2,-3],[-6,-5],[-2,0],[0,-3],[-2,-1],[-2,0],[-3,-1],[1,-1],[1,-3],[-1,-1],[-1,-1],[1,-2],[1,-1],[-2,-5],[-2,-4],[-4,-7],[-1,-1],[-1,-1],[-1,-1],[0,-3],[0,-2],[0,-1],[-2,-5],[4,0],[-1,-4],[-4,-8],[-1,-4],[2,3],[1,2],[2,1],[2,2],[-3,-15],[-2,-16],[0,2],[0,3],[1,2],[0,2],[-1,2],[-2,-2],[-1,-4],[-1,-4],[-1,-4],[-1,-3],[1,-5],[0,-4],[-2,-2],[-5,-1],[-3,-2],[-2,-5],[-4,-4],[-1,-3],[-2,-5],[-23,-25],[-4,-1],[0,-2],[9,-1],[4,0],[5,3],[0,-4],[-1,-3],[0,-3],[1,-2],[6,2],[10,9],[3,1],[2,-2],[2,-5],[2,-1],[2,0],[1,2],[1,2],[1,2],[3,1],[10,0],[3,-2],[-3,-5],[0,-5],[2,-6],[0,-7],[-1,-2],[-4,-4],[-5,-11],[-1,-7],[0,-7],[1,-6],[2,-7],[1,-5],[3,-5],[6,-9],[-3,0],[-3,-2],[-1,-4],[0,-6],[2,6],[4,-1],[5,-7],[0,5],[1,3],[1,-1],[3,-2],[0,-4],[0,-5],[0,-9],[-1,-9],[0,-5],[2,-2],[1,-2],[1,-5],[1,-4],[-2,-2],[-1,-1],[-1,-1],[0,-1],[0,-1],[1,0],[1,-2],[0,-1],[1,0],[1,0],[0,1],[1,0],[1,-1],[2,-3],[2,-4],[1,-4],[0,-9],[-1,-3],[-1,-4],[-2,-3],[-2,-1],[-2,0],[-2,-2],[2,0],[1,0],[1,-1],[1,-1],[1,1],[2,2],[5,1],[1,1],[0,2],[2,5],[3,-4],[1,-8],[-1,-3],[-3,-2],[0,-5],[0,-13],[3,3],[2,1],[12,-1],[1,0],[1,-2],[1,-1],[1,0],[1,0],[1,1],[1,1],[11,0],[1,0],[6,-5],[3,-4],[-1,-3],[1,-3],[1,-1],[3,1],[0,1],[1,2],[1,1],[0,-3],[1,-2],[2,-2],[0,1],[4,4],[1,1],[2,1],[5,-1],[3,-1],[6,-5],[9,-4],[3,-3],[2,-3],[-2,-4],[-3,-2],[-5,-1],[-1,-1],[-2,-4],[-1,-2],[-1,0],[-18,1],[-1,-1],[0,-1],[0,-2],[-1,-2],[-1,-1],[1,-3],[3,-5],[1,-2]],[[8529,424],[-2,-1],[1,3],[-1,2],[-1,2],[-1,2],[-1,1],[1,2],[1,1],[3,2],[-1,6],[2,1],[5,-1],[-1,-4],[-1,-4],[-1,-3],[-2,-5],[-1,-4]],[[9185,548],[-5,-4],[-6,-2],[-6,0],[-6,0],[-7,3],[-4,4],[-2,4],[1,0],[2,-2],[1,-1],[2,-2],[9,-1],[14,1],[4,2],[2,1],[3,3],[6,7],[4,5],[-6,-12],[-6,-6]],[[8451,631],[-1,-8],[-2,2],[5,16],[3,7],[2,5],[1,3],[2,2],[1,-1],[0,-3],[-2,-3],[-2,-7],[-7,-13]],[[9058,943],[2,-3],[1,3],[1,0],[1,-1],[1,-2],[0,-2],[-1,-1],[-1,0],[-2,0],[-4,-5],[-1,-4],[2,-4],[-3,-3],[-2,-4],[-2,-2],[-2,2],[1,2],[1,2],[0,2],[-1,1],[-3,0],[-1,0],[-2,1],[-1,2],[-1,3],[-2,1],[0,1],[0,1],[-1,3],[1,1],[1,0],[2,2],[3,2],[2,2],[3,1],[5,0],[3,-1]],[[8877,1004],[2,-1],[1,0],[0,-1],[0,-2],[-2,-1],[-3,-2],[0,-1],[-2,1],[-1,1],[-2,-1],[-1,1],[0,1],[1,1],[3,2],[1,0],[1,0],[1,1],[1,1]],[[8706,1043],[1,0],[0,1],[2,1],[0,1],[1,0],[12,-6],[4,-4],[0,-7],[3,0],[2,-2],[2,-4],[0,-5],[2,0],[4,-1],[3,-2],[1,-2],[-1,-2],[-2,-1],[-1,-2],[2,-1],[5,2],[3,-1],[-1,-3],[0,-2],[1,-1],[1,1],[1,2],[-1,2],[0,2],[0,3],[0,2],[1,1],[0,-1],[1,-2],[1,0],[4,3],[12,2],[6,-2],[3,-5],[-2,0],[-4,1],[-2,0],[1,-2],[4,-3],[1,-3],[-2,-1],[-4,1],[-3,0],[-2,-2],[1,-2],[2,-1],[2,2],[1,-1],[1,-1],[1,0],[1,0],[1,0],[1,-1],[1,2],[0,1],[2,-1],[4,-2],[1,0],[4,0],[3,2],[4,1],[3,-2],[-3,-4],[-5,-3],[-5,-3],[-4,2],[1,-4],[2,-2],[5,-3],[1,-1],[1,-1],[1,-1],[0,3],[1,1],[0,2],[2,3],[0,-2],[0,-2],[0,-2],[5,6],[2,1],[0,1],[0,1],[1,1],[1,0],[0,-1],[0,-2],[-1,-1],[-1,-2],[-1,-1],[0,-3],[2,0],[6,6],[2,1],[2,0],[2,0],[2,-1],[-1,4],[-1,3],[-4,4],[2,1],[14,-3],[6,-4],[17,-4],[3,0],[5,2],[3,0],[-2,-3],[-3,0],[-2,1],[-2,-1],[2,-3],[1,-1],[3,0],[2,0],[2,-2],[2,-2],[2,-2],[-8,-8],[-5,-1],[-3,-2],[-1,-3],[-1,-4],[1,2],[2,2],[2,1],[1,-2],[-1,-7],[0,-4],[1,2],[2,6],[2,3],[1,-1],[3,-5],[1,-1],[1,4],[-2,1],[-1,2],[0,3],[1,3],[1,0],[2,0],[2,0],[1,2],[2,-1],[2,0],[2,-1],[0,-1],[1,-3],[1,-1],[2,-1],[2,1],[2,0],[2,0],[-3,-7],[-1,-4],[2,-2],[2,2],[4,7],[2,1],[2,1],[3,2],[2,3],[-1,2],[-2,0],[-2,-2],[-4,-4],[2,5],[17,6],[24,25],[8,14],[4,4],[3,2],[2,0],[1,-2],[0,-2],[-1,-2],[-1,-1],[-1,-2],[1,-5],[1,-11],[2,-11],[0,-4],[-1,-5],[3,-2],[4,-3],[2,-2],[1,-3],[2,-2],[8,-6],[2,-1],[2,0],[2,1],[2,2],[1,1],[2,-1],[-1,0],[-2,0],[0,-2],[1,-2],[3,-1],[0,2],[0,2],[1,2],[1,1],[1,0],[1,2],[0,3],[1,2],[2,3],[2,1],[5,0],[2,-2],[4,-8],[2,-1],[2,-3],[2,-5],[2,-7],[1,-1],[1,-2],[1,-4],[2,-3],[7,-7],[2,-1],[1,0],[0,-4],[0,-6],[-1,-3],[-3,-2],[-9,-8],[-3,-1],[0,3],[-1,-1],[-2,-2],[-2,-2],[-1,-2],[-6,-4],[2,-5],[2,-1],[35,1],[2,0],[4,-3],[10,-2],[2,-2],[1,-3],[0,-3],[0,-3],[0,-4],[-1,0],[-1,1],[0,2],[-1,2],[-1,1],[-1,-1],[-1,1],[-2,1],[1,-4],[1,-1],[1,-2],[-1,-2],[-1,0],[-1,-1],[-1,-1],[0,-2],[-1,-2],[0,-1],[-2,1],[1,-3],[-6,-3],[-2,1],[-2,5],[2,2],[1,2],[0,2],[1,2],[-5,-2],[-1,-1],[-1,-4],[1,-6],[0,-4],[-1,2],[-1,2],[0,1],[-1,3],[0,3],[-1,2],[-1,-2],[-3,-2],[-2,0],[0,3],[-2,1],[-2,-2],[-4,-4],[-2,-3],[-2,-4],[0,-3],[3,0],[-1,-3],[-2,-2],[-2,-1],[-21,-4],[-4,1],[-4,6],[-1,-5],[-2,2],[-2,3],[-2,-2],[2,-1],[2,-3],[2,-4],[1,-3],[-1,0],[-1,0],[-2,-2],[4,-5],[0,-1],[-4,0],[-2,1],[-1,1],[0,2],[-1,0],[0,-1],[-5,0],[0,-2],[0,-1],[0,-1],[0,-1],[-3,3],[-3,2],[-2,-1],[2,-4],[-1,-1],[-3,1],[-2,0],[0,-2],[1,0],[1,-2],[1,-2],[0,-2],[-1,-1],[-4,2],[-1,-1],[1,-3],[-2,-1],[-2,1],[-2,4],[-1,-4],[0,-2],[-2,2],[-2,0],[-1,-2],[-2,-4],[0,2],[0,1],[-1,1],[0,1],[-1,-1],[-1,-2],[-1,-1],[-1,-1],[2,-1],[6,-4],[-1,-1],[-2,-1],[-2,1],[-1,1],[-1,-2],[-2,-1],[-2,-1],[-1,1],[-2,-1],[-2,-1],[-2,0],[-1,1],[-1,1],[-2,-4],[-1,-1],[-6,1],[-2,-2],[-2,-6],[-1,2],[-1,-2],[-2,-2],[-2,0],[-1,1],[-1,0],[0,-1],[0,-1],[0,-1],[-1,0],[1,-1],[0,-1],[0,-2],[-2,2],[-2,2],[-2,0],[-2,-4],[0,-1],[1,-1],[0,-1],[-2,-1],[-2,-2],[-1,-4],[0,-5],[-2,2],[-1,5],[0,5],[0,5],[-2,-5],[-2,-4],[-2,-2],[-3,3],[-1,-7],[1,-2],[1,-2],[1,-4],[-2,-2],[-3,8],[-2,-1],[-1,2],[-2,-1],[-4,-6],[1,-3],[-4,2],[-14,-4],[1,-3],[0,-3],[-1,-2],[-2,0],[1,-2],[0,-2],[1,-1],[1,-1],[-2,-2],[-1,-1],[-2,0],[-1,2],[-1,2],[0,1],[-1,-3],[-2,2],[-2,1],[-3,1],[-1,-2],[-4,-4],[-1,-1],[-1,1],[0,2],[-1,1],[0,1],[0,1],[0,7],[0,1],[-2,2],[-3,-3],[-1,3],[0,-4],[1,-3],[3,-5],[1,-4],[-1,-4],[-4,-5],[-1,3],[0,4],[0,4],[-1,1],[-2,1],[-1,3],[0,8],[0,2],[-1,2],[-1,0],[-1,-1],[0,-1],[1,-7],[-1,-4],[-1,-7],[-1,-4],[-1,1],[-1,3],[0,2],[1,3],[-2,-2],[-1,-7],[-3,-2],[-1,0],[0,2],[0,3],[-2,6],[0,2],[-2,-3],[0,-3],[0,-3],[2,-2],[-2,-4],[-3,-7],[-1,-4],[-3,2],[-9,1],[-3,3],[2,3],[1,0],[1,1],[-2,3],[-2,0],[-4,-3],[2,-3],[2,-2],[0,-4],[0,-5],[-2,-4],[-2,1],[-2,3],[-4,3],[-4,6],[-4,4],[-6,11],[-1,1],[-2,0],[1,-7],[1,-3],[1,-1],[1,-1],[3,1],[1,-1],[1,-5],[1,-1],[1,-2],[1,-7],[1,-3],[0,-4],[-1,-3],[1,-3],[2,-8],[0,-3],[-1,-4],[-2,-4],[-2,-2],[-1,0],[-1,1],[0,2],[1,1],[0,1],[-2,1],[-1,-3],[-1,-3],[-1,-2],[0,-2],[-1,-1],[-1,0],[0,2],[0,2],[1,2],[0,1],[-1,1],[-2,0],[-4,-1],[-1,-1],[0,-1],[0,-1],[-2,-1],[0,1],[-1,3],[0,1],[-1,-1],[-1,1],[-1,1],[-1,3],[0,3],[-1,-3],[-1,-3],[-1,-3],[-2,2],[0,4],[0,3],[3,7],[-1,-2],[-2,-3],[-1,0],[-1,2],[-1,1],[-1,-2],[0,-3],[-1,0],[-2,-1],[-2,-1],[-1,0],[-2,1],[-1,1],[-1,2],[-1,5],[0,5],[2,9],[-1,2],[-1,5],[2,2],[0,1],[1,2],[-1,0],[-1,2],[3,4],[2,3],[0,4],[-2,-3],[-2,-2],[-8,-4],[-1,-1],[-6,-4],[-1,-1],[0,-3],[0,-8],[1,-2],[2,-2],[1,-5],[2,-9],[-3,2],[-2,-3],[-1,-5],[-1,-3],[-3,0],[-1,1],[-1,2],[-1,7],[0,1],[1,4],[-1,4],[-2,4],[-2,2],[-2,1],[-2,-1],[-2,-2],[0,-4],[-1,-4],[-3,2],[-4,3],[-2,2],[0,-1],[1,-4],[-1,-1],[0,-1],[0,-3],[0,-2],[0,-3],[-1,-3],[-1,-2],[-1,-2],[1,0],[2,-1],[-1,-2],[-5,-2],[0,-2],[1,-2],[1,-2],[0,-4],[5,1],[2,0],[2,-2],[-2,0],[-1,-2],[-2,-2],[-1,-1],[0,-2],[6,0],[3,-1],[2,-2],[-2,-2],[-2,0],[-4,2],[-7,0],[0,-2],[1,-1],[0,-1],[0,-2],[0,-2],[1,-1],[2,-2],[1,0],[1,1],[0,1],[-2,2],[1,1],[1,0],[3,-2],[-3,-7],[1,-2],[5,0],[-3,-7],[-2,-2],[-2,3],[-1,-2],[-2,3],[-1,2],[-1,2],[1,1],[1,0],[-9,8],[-2,2],[1,-3],[2,-3],[2,-3],[2,-1],[1,0],[0,-2],[0,-2],[0,-2],[-1,-2],[-7,-7],[-1,-2],[0,-5],[0,-2],[-2,-1],[-2,-1],[-1,-2],[0,-2],[-2,-8],[-2,2],[-1,1],[-1,1],[-1,1],[-1,0],[-1,-2],[0,-1],[-4,2],[-1,1],[0,-3],[0,-1],[-1,0],[0,-1],[-1,0],[2,-1],[3,0],[3,-1],[1,-3],[-1,-5],[-1,-4],[-2,-3],[-2,2],[-3,-3],[-3,-1],[-8,0],[1,-4],[3,-1],[2,-1],[0,-5],[0,-4],[-2,-1],[-2,-1],[-1,-2],[-2,-4],[-2,-2],[-7,0],[-1,-1],[0,-2],[-1,-2],[0,-1],[-1,-1],[-2,0],[2,-3],[4,-5],[1,-5],[-1,0],[-2,0],[0,-2],[0,-3],[-2,-2],[0,-3],[-1,-1],[-1,3],[-1,1],[-1,4],[-1,1],[-1,2],[-1,1],[-1,0],[-1,1],[0,-3],[2,-4],[1,-2],[0,-2],[0,-4],[1,-2],[-3,-2],[-2,5],[-2,6],[-1,4],[-2,2],[0,-3],[3,-12],[1,-1],[-1,-2],[-1,-6],[-1,-2],[-1,-2],[-2,0],[-2,3],[-2,11],[-2,4],[0,-4],[2,-11],[0,-3],[0,-4],[0,-3],[-1,-3],[-1,-1],[0,1],[0,1],[-1,1],[-1,-1],[0,-2],[-1,-2],[-2,2],[1,8],[-2,3],[-1,-5],[-3,-10],[0,-5],[-3,3],[1,5],[1,4],[-1,6],[-2,-8],[-1,-2],[-1,0],[-1,2],[-1,2],[-1,6],[-1,5],[-1,5],[-2,3],[0,-1],[0,-1],[0,-1],[0,-2],[-2,-2],[1,-5],[3,-9],[-2,-5],[-1,-5],[-2,-1],[-3,1],[-4,5],[0,3],[1,6],[1,2],[-1,5],[0,1],[-1,-1],[-1,-1],[-1,-2],[0,-2],[-1,-1],[-1,1],[1,-6],[2,-6],[0,-5],[0,-7],[0,-2],[0,-2],[0,-2],[-1,-1],[-1,-1],[0,-1],[0,-2],[0,-1],[1,-1],[1,-10],[-3,7],[-1,1],[-1,-1],[-1,-2],[-1,-1],[-2,2],[-3,5],[-2,2],[0,-2],[5,-13],[1,-2],[0,-3],[1,-3],[-2,-1],[-1,2],[0,9],[-2,3],[-1,0],[0,-1],[0,-2],[0,-3],[-1,-2],[-1,-1],[-2,1],[1,-5],[1,-3],[0,-3],[-1,-2],[-1,0],[-1,2],[0,3],[-1,2],[-4,6],[-1,1],[0,2],[0,1],[0,3],[0,2],[-1,0],[0,1],[-1,2],[-1,-3],[-4,-7],[-4,-4],[-1,-1],[0,-2],[0,-1],[-2,-1],[1,1],[-1,1],[-2,1],[-3,0],[-1,1],[-1,4],[0,4],[0,4],[-1,3],[-1,3],[0,-3],[0,-2],[0,-1],[1,-1],[0,-1],[-1,-1],[-1,0],[-2,4],[0,5],[1,13],[0,6],[0,3],[-1,2],[-1,1],[-1,-2],[0,-4],[-1,-3],[1,-2],[0,-3],[0,-3],[0,-4],[-2,6],[0,7],[0,6],[-1,6],[0,2],[-2,5],[-1,3],[0,3],[-1,10],[-1,0],[0,-4],[0,-2],[1,-2],[-2,0],[-1,3],[-2,5],[-1,4],[-1,2],[-2,-1],[0,-1],[1,-2],[0,-2],[-3,-4],[-2,-2],[0,4],[-1,2],[0,4],[-1,5],[0,2],[-1,1],[0,-1],[0,-4],[-1,-2],[0,-2],[1,-2],[0,-1],[3,-13],[1,-3],[-1,-6],[-1,-2],[-1,0],[1,4],[0,2],[-1,0],[-2,-1],[0,-2],[0,-2],[0,-2],[-1,-1],[-1,0],[0,2],[0,1],[0,2],[-1,0],[1,5],[-2,2],[-1,-1],[-1,-6],[-1,3],[-1,3],[1,6],[-1,1],[-2,-1],[-1,-3],[-1,-3],[-1,0],[-1,16],[1,4],[1,3],[0,3],[0,1],[-2,-1],[0,-2],[-1,-2],[-1,-2],[0,4],[0,3],[-1,2],[-1,0],[0,2],[0,2],[0,2],[0,2],[0,2],[2,5],[0,2],[0,4],[-1,3],[0,3],[1,7],[0,2],[-1,3],[-3,15],[-1,2],[-2,-2],[0,5],[2,9],[0,5],[1,2],[3,7],[1,2],[1,5],[3,27],[2,8],[4,4],[2,3],[1,5],[0,3],[8,12],[4,4],[5,7],[2,2],[2,4],[1,4],[-1,3],[-2,0],[-5,-2],[-1,1],[-1,0],[-1,0],[-10,-15],[-18,-30],[-1,-1],[0,6],[2,4],[20,35],[13,12],[1,3],[10,13],[2,-2],[1,-3],[1,-4],[0,-4],[0,-2],[-1,-3],[0,-2],[1,-1],[3,0],[2,1],[2,2],[1,2],[3,5],[3,4],[2,3],[1,4],[1,-1],[6,11],[2,2],[3,1],[5,5],[3,2],[-2,1],[-6,-6],[-4,-2],[-9,-10],[-5,-2],[-2,-1],[-5,-5],[-3,-2],[-2,4],[4,11],[12,12],[18,18],[7,10],[2,2],[3,1],[14,12],[2,3],[4,6],[18,16],[18,19],[16,13],[8,2],[8,5],[7,2],[5,3],[3,3],[3,4],[3,7],[-3,5],[-5,2],[-4,1],[2,2],[3,-1],[4,-3],[8,-1],[2,-3],[1,-3],[0,-3],[-1,-3],[-2,-2],[-1,-4],[0,-8],[-1,-4],[1,-3],[0,-3],[-2,-3],[-2,-3],[3,-5],[0,-2],[0,-2],[1,-1],[1,0],[1,2],[1,3],[0,2],[0,2],[1,1],[2,0],[9,-9],[8,-14],[0,-3],[0,-13],[1,3],[1,1],[1,0],[2,-1],[-1,3],[0,5],[0,4],[2,1],[0,2],[-2,1],[-2,2],[-6,10],[-1,5],[0,3],[3,3],[1,2],[-1,3],[1,1],[0,2],[3,2],[0,1],[2,3],[11,6],[6,2],[7,6],[2,1],[5,-1],[2,1],[2,2],[0,3],[0,1],[1,2],[1,0],[3,0],[2,-1],[1,-2],[1,1],[1,4],[0,1],[1,0],[4,-1],[11,0],[4,3],[2,0],[2,-1],[2,-3],[1,-3],[1,-3],[1,8],[3,5],[5,2],[4,2],[-4,2],[-13,1],[-2,1],[-5,3],[-3,1],[-25,0],[-2,-1],[-2,-2],[-3,-4],[0,-1],[-1,-1],[0,-1],[-2,0],[-1,1],[0,2],[0,2],[1,2],[-4,4],[-8,-2],[-5,1],[1,1],[1,1],[-3,2],[-5,1],[-8,-2],[-7,-3],[-4,-1],[-4,3],[0,-5],[-1,-1],[-1,2],[-2,0],[-1,0],[-2,-1],[-1,-2],[-1,-2],[-1,2],[-4,2],[-1,1],[-1,2],[-1,1],[-13,4],[-5,-2],[-6,-5],[-4,-6],[1,-9],[-2,0],[-2,-2],[-1,-3],[-2,-1],[-1,-1],[0,2],[0,2],[0,1],[1,2],[-1,1],[0,1],[-2,0],[-4,4],[-4,1],[-4,-2],[-4,-5],[-1,6],[1,8],[2,7],[1,4],[6,7],[1,0],[3,2],[2,1],[-4,3],[14,13],[3,1],[2,4],[2,0],[0,7],[4,4],[4,3],[4,4],[3,6],[3,5],[1,0],[2,1],[1,0],[1,2],[1,3],[1,2],[-1,4],[0,3],[0,2],[3,1],[1,1],[2,3],[0,3],[0,3],[9,8],[1,-3],[0,-2],[-2,-5],[-1,-1],[-1,0],[0,-1],[0,-2],[0,-1],[0,-1],[0,-1],[0,-3],[1,3],[2,1],[2,1],[2,0],[0,2],[-1,1],[-2,1],[-1,-1],[1,2],[1,2],[1,3],[1,2],[0,4],[-1,1]],[[9197,1047],[0,-2],[0,-1],[-1,0],[-1,1],[-4,0],[-2,1],[0,2],[6,4],[3,1],[1,-1],[-2,-2],[0,-3]],[[9107,1286],[2,0],[3,1],[4,4],[3,1],[1,-2],[-1,-5],[-4,-8],[-4,-9],[-3,-5],[0,-2],[2,0],[3,-1],[1,-2],[-2,-2],[0,-2],[3,-1],[4,0],[4,1],[2,2],[1,-1],[1,-1],[1,-1],[1,-2],[0,-5],[0,-7],[-2,-14],[-2,-9],[0,-4],[-1,-2],[-5,-4],[0,-2],[2,-1],[1,0],[1,0],[0,-1],[-1,-1],[-1,-1],[-1,0],[1,-3],[2,-2],[1,-2],[0,-3],[-1,-2],[-2,-3],[-5,-14],[-4,-24],[-2,-4],[-4,-9],[-1,-3],[-1,-8],[-6,-14],[-3,-13],[1,-3],[0,-1],[5,16],[2,3],[1,1],[6,10],[4,3],[1,0],[1,-1],[1,-1],[-1,-2],[0,-3],[-1,-5],[-1,-3],[-4,-4],[-16,-30],[-2,-3],[-5,-4],[-1,-2],[-2,-1],[-1,5],[-1,-1],[-3,-5],[-1,-1],[-2,-1],[-5,-3],[-6,-2],[-1,2],[1,1],[1,1],[1,1],[-2,1],[-4,-7],[-8,-14],[-2,-2],[-2,-2],[-10,-4],[-2,-2],[-1,-5],[11,1],[3,2],[1,3],[1,4],[2,3],[8,6],[2,6],[2,1],[5,0],[5,5],[3,2],[2,-3],[-1,-4],[-2,-4],[-3,-3],[-1,-4],[-1,-4],[0,-5],[-1,-4],[-3,-2],[-1,0],[-1,-1],[-1,-2],[-1,-1],[-2,-1],[-1,0],[-2,2],[-5,-1],[-2,-1],[-1,-3],[-1,0],[-7,0],[-1,0],[2,-2],[1,0],[-2,-2],[-2,-1],[-1,-2],[-2,-3],[2,0],[3,0],[2,2],[1,2],[1,1],[3,0],[2,0],[-1,3],[2,-1],[6,-7],[-1,-2],[-1,1],[-2,1],[-1,1],[-1,-1],[3,-5],[1,-2],[-2,0],[-1,1],[-1,1],[-1,-3],[-2,-1],[-1,1],[-1,0],[-1,-2],[-9,-9],[-2,-3],[-2,-5],[1,1],[2,1],[0,-2],[0,-2],[0,-2],[0,-1],[-2,0],[-1,-1],[0,-2],[-1,-1],[0,-1],[2,-1],[5,-2],[4,2],[12,13],[3,1],[5,-1],[4,-3],[3,-5],[1,-4],[-1,-3],[-1,-3],[-3,-5],[0,-1],[0,-1],[1,-1],[1,-1],[0,1],[0,1],[0,2],[2,1],[-1,2],[4,0],[1,1],[1,5],[1,0],[2,-2],[1,0],[0,1],[-1,3],[1,3],[0,2],[-1,3],[-1,2],[0,4],[1,3],[2,3],[1,2],[2,2],[3,2],[2,2],[1,3],[1,3],[1,3],[2,1],[1,2],[3,5],[4,3],[2,6],[5,3],[3,4],[4,5],[2,5],[-3,-1],[-5,-4],[-3,-5],[-8,-3],[-7,-6],[-2,-1],[-2,-4],[-4,-1],[-2,-1],[-1,2],[-1,2],[-1,4],[-1,2],[-1,0],[-2,0],[-1,0],[-1,2],[1,4],[8,7],[3,5],[2,2],[3,1],[2,2],[7,10],[6,6],[2,2],[7,12],[2,1],[1,2],[3,8],[3,2],[2,0],[2,1],[2,3],[0,4],[-2,-2],[-3,-1],[-5,0],[-2,-2],[-5,-10],[-18,-24],[-3,-2],[-1,0],[-1,1],[0,3],[0,2],[1,1],[2,1],[3,4],[3,8],[8,12],[1,4],[2,4],[14,19],[8,4],[0,-3],[2,-9],[7,-9],[-1,-4],[-9,-17],[1,1],[1,2],[1,2],[1,0],[1,0],[1,0],[2,-3],[1,-4],[1,-1],[1,3],[0,3],[0,2],[-1,2],[0,3],[1,4],[2,4],[3,4],[1,2],[3,-1],[3,-1],[3,-2],[3,-3],[-7,-3],[12,-1],[5,-3],[0,-8],[1,-1],[4,3],[2,1],[8,-1],[-3,-4],[-3,-4],[-2,-6],[0,-8],[1,4],[3,2],[2,2],[2,1],[0,-2],[-7,-8],[-2,-6],[-2,-2],[-5,-2],[0,-2],[0,-2],[2,-4],[2,-1],[2,0],[5,2],[2,0],[1,0],[1,-2],[-1,1],[0,-2],[0,-3],[1,-2],[2,-3],[0,-1],[0,-2],[0,-3],[-1,-1],[-1,1],[-5,0],[-8,-7],[-4,1],[0,-1],[-1,-1],[-1,-1],[0,-2],[1,-1],[0,-1],[1,-1],[-18,-2],[-3,-5],[0,-2],[1,0],[2,-1],[2,-3],[1,0],[2,-1],[1,-1],[1,-2],[-1,-2],[0,-1],[-2,-1],[-9,-6],[-3,-2],[-1,-4],[-1,-5],[-2,-3],[-3,-2],[-2,0],[-5,-3],[-8,-10],[-5,-2],[-3,0],[0,1],[-1,2],[-3,-4],[-1,-1],[-13,-8],[-4,-4],[-1,-1],[-4,0],[-1,-1],[-1,0],[0,-1],[-1,0],[-1,2],[0,2],[0,2],[-1,1],[-9,1],[-2,1],[0,3],[0,3],[-1,3],[-2,1],[-1,-2],[-2,-3],[-1,-2],[-1,1],[-1,2],[-1,0],[-1,0],[0,-4],[-3,-1],[-6,2],[2,-2],[-1,-3],[-1,-1],[-3,1],[-1,-1],[-3,-4],[-5,-3],[-3,-1],[-2,0],[2,2],[1,1],[-1,2],[-1,2],[-2,1],[-2,0],[-1,1],[0,3],[-1,-2],[0,-2],[-1,-2],[-1,0],[-1,1],[-1,-2],[-2,-2],[-1,-1],[1,-4],[-2,0],[-4,2],[-1,2],[-3,11],[-7,11],[-2,6],[-2,7],[-1,16],[-1,8],[-2,4],[-1,2],[1,13],[-1,4],[-2,3],[0,4],[0,4],[0,2],[0,5],[-1,3],[0,1],[-1,2],[-1,2],[-1,2],[1,5],[3,3],[3,2],[2,3],[0,4],[1,7],[1,6],[1,2],[2,2],[5,8],[11,11],[1,1],[0,2],[8,19],[2,8],[2,5],[9,16],[1,3],[1,4],[0,8],[1,3],[3,6],[1,4],[-1,3],[-1,3],[-1,2],[1,3],[1,3],[1,2],[2,2],[-2,-8],[0,-2],[1,-2],[1,2],[0,3],[1,3],[0,2],[4,4],[1,3],[2,8],[5,17],[3,8],[5,5],[5,3],[2,2],[7,15],[6,19],[1,5],[1,2],[1,1],[3,4],[1,0],[2,-2],[4,-1],[2,0],[1,-2],[0,-1],[1,-2]],[[9752,1345],[-1,-3],[-2,3],[0,6],[4,2],[1,-1],[-1,-3],[-1,-4]],[[9632,1353],[1,-1],[1,0],[-3,-5],[-1,3],[-1,-1],[-2,1],[-1,1],[-2,1],[-1,0],[-1,-1],[0,2],[3,3],[4,1],[3,0],[0,-2],[0,-2]],[[9822,1373],[-3,0],[-1,2],[0,3],[1,1],[1,2],[2,6],[3,-1],[1,-3],[-1,-8],[-3,-2]],[[9958,1430],[-3,-1],[-3,2],[1,4],[8,9],[4,0],[0,-5],[-3,-4],[-4,-5]],[[9829,1441],[-4,-13],[-1,-8],[1,-5],[-3,-7],[-7,-24],[-2,-2],[-1,0],[-9,1],[-2,2],[2,3],[10,15],[5,13],[0,5],[-3,-1],[0,2],[7,13],[1,1],[-1,-5],[0,-1],[1,-2],[1,4],[3,14],[1,2],[1,-2],[0,-5]],[[9625,1440],[3,-3],[2,-2],[-3,-2],[-3,-1],[-3,0],[-9,4],[-3,2],[-4,0],[-1,2],[0,4],[3,3],[13,5],[3,-1],[2,-1],[-2,-5],[2,-5]],[[9835,1420],[0,-3],[0,-3],[-2,-1],[1,-2],[0,-2],[-1,-3],[0,-5],[-2,4],[0,3],[1,5],[1,5],[-1,-1],[-1,-1],[0,6],[1,13],[0,3],[0,2],[1,2],[0,1],[1,4],[-1,6],[1,-2],[1,-2],[1,-5],[0,-3],[-1,-3],[-1,-3],[-1,-2],[0,-2],[1,-2],[0,-6],[0,-2],[1,-1]],[[9858,1570],[2,-1],[9,1],[2,-1],[5,-5],[3,-1],[3,2],[9,8],[3,4],[3,-1],[0,-4],[-2,-2],[2,-5],[-1,-3],[-1,-3],[-2,-6],[2,0],[-3,-6],[-6,0],[-12,5],[-10,-2],[-2,1],[-5,2],[-1,3],[0,3],[-1,3],[-2,4],[-1,3],[-1,3],[-1,6],[1,2],[3,-3],[4,-7]],[[9770,1883],[-2,-2],[-1,0],[-1,-1],[-1,-1],[0,-2],[1,-3],[-1,-2],[-1,-3],[-1,-2],[-1,-1],[-2,2],[-2,3],[-1,5],[2,3],[4,4],[1,2],[2,2],[2,-1],[2,-3]],[[9653,1894],[-4,-3],[-3,1],[0,3],[0,3],[1,4],[0,2],[2,1],[6,-1],[0,-5],[-2,-5]],[[9655,1920],[0,-1],[2,0],[1,0],[1,-2],[-1,-3],[3,0],[1,0],[1,0],[0,1],[1,1],[1,-1],[0,-1],[0,-1],[-1,-1],[-2,0],[-5,-2],[-3,-4],[-3,0],[-2,0],[-2,3],[1,1],[1,4],[1,5],[1,1],[2,2],[3,0],[-1,-2]],[[9783,1919],[-1,-2],[0,-2],[-1,-3],[0,-2],[-3,-3],[-4,-3],[-11,-5],[-2,-1],[-1,-2],[-1,-1],[0,-2],[0,-3],[-1,-2],[-1,0],[-3,1],[-5,-2],[-3,-1],[-3,2],[2,2],[2,2],[2,0],[2,0],[0,1],[0,1],[-1,1],[-3,1],[0,2],[-2,4],[-2,0],[-1,2],[0,3],[-1,2],[1,3],[4,-1],[2,3],[2,0],[2,0],[2,-2],[-1,-4],[0,-3],[2,-2],[3,-1],[2,0],[4,3],[2,3],[1,4],[1,4],[1,3],[2,2],[2,1],[-1,-2],[-1,-1],[2,-2],[3,4],[3,5],[2,2],[0,-2],[1,-1],[0,-1],[1,-5]],[[9834,1946],[2,-3],[0,-1],[1,0],[1,-1],[1,-1],[0,-1],[-1,-1],[0,-1],[0,-2],[1,-3],[2,-2],[2,-1],[2,0],[-1,-3],[-1,-1],[-4,2],[-1,-1],[-1,-1],[-1,-2],[-3,-1],[-1,-1],[-1,-2],[-2,-2],[-1,-1],[-5,0],[-4,-3],[-2,-1],[-4,-6],[-1,1],[-1,0],[-2,3],[0,7],[-1,8],[0,6],[1,1],[3,-1],[2,1],[-1,1],[-1,1],[-1,1],[0,2],[0,1],[1,1],[0,1],[-1,1],[-1,-1],[2,4],[3,0],[3,-3],[2,-2],[-1,-2],[2,-5],[1,3],[0,4],[-1,3],[1,2],[1,2],[1,-1],[2,-1],[1,1],[0,2],[-2,3],[5,1],[2,-1],[2,-4]],[[9635,2057],[-1,-1],[-3,0],[-2,1],[0,2],[4,0],[0,3],[1,0],[1,-1],[1,-2],[-1,-2]],[[9651,2057],[-2,-1],[-2,2],[-3,0],[-4,2],[-1,3],[0,2],[4,1],[1,0],[3,-1],[2,-2],[1,-4],[1,-2]],[[9675,2197],[1,-1],[1,-1],[-4,-4],[-1,-2],[0,-2],[-2,-3],[-6,-6],[-1,3],[-1,2],[-1,0],[-1,-1],[-3,2],[-1,3],[3,6],[0,2],[0,1],[-1,2],[1,3],[1,1],[1,-1],[1,-2],[0,-2],[2,1],[6,4],[2,2],[2,1],[3,-1],[1,-2],[-1,-2],[-1,-1],[0,-1],[-1,0],[0,-1]],[[9479,2213],[1,-2],[1,-1],[1,-2],[-1,0],[-1,-1],[-2,-2],[0,-1],[-2,-2],[-1,0],[-1,1],[1,0],[1,1],[-1,1],[-1,0],[-1,0],[-1,2],[0,2],[1,3],[2,1],[2,0],[0,-1],[1,0],[1,1]],[[9666,2228],[1,-3],[-2,0],[-2,1],[-4,4],[1,2],[0,2],[-2,5],[-1,3],[1,1],[3,0],[5,5],[2,0],[2,-4],[-1,-3],[-1,-2],[-1,-1],[-2,-3],[0,-3],[0,-1],[1,-3]],[[9629,2400],[2,-1],[2,1],[1,1],[2,1],[-6,-13],[0,-5],[4,1],[-2,-7],[-3,-6],[2,-1],[2,0],[3,1],[8,0],[4,-2],[4,-3],[5,-2],[3,2],[-1,1],[-5,2],[-1,2],[-1,3],[0,3],[0,3],[-6,4],[1,5],[2,2],[-2,-7],[3,3],[5,8],[3,2],[-1,-2],[-1,-2],[0,-1],[1,-1],[0,-1],[1,-2],[2,-1],[3,-1],[1,1],[0,1],[0,1],[-1,0],[6,6],[2,1],[4,-3],[0,-3],[1,0],[1,1],[0,1],[4,0],[2,-2],[1,-4],[-2,-3],[-2,1],[-2,2],[-2,0],[1,-2],[3,-4],[1,0],[-1,-3],[0,-3],[-1,-2],[-1,-1],[-4,2],[-5,0],[1,-3],[3,-3],[2,-2],[3,1],[0,-4],[-2,-6],[0,-3],[-1,-3],[-1,-6],[-1,-3],[-2,-2],[-1,1],[-1,2],[-3,2],[-1,0],[0,-2],[0,-4],[0,-1],[1,-1],[1,0],[-1,-1],[-1,-2],[-3,0],[-1,0],[-1,2],[0,-3],[1,-3],[0,-3],[1,-1],[-2,-2],[-3,3],[-3,3],[-2,2],[-14,4],[-7,-2],[-2,0],[-1,1],[-1,1],[-2,3],[-2,1],[-1,0],[0,-1],[1,-3],[-3,-1],[-2,2],[-3,4],[-3,3],[0,-2],[0,-2],[0,-1],[0,-2],[-2,2],[-1,1],[-1,1],[-2,-1],[0,-1],[0,-2],[0,-5],[-1,-3],[2,0],[5,-2],[2,-1],[0,-8],[1,-1],[-1,-1],[-1,-1],[-1,0],[-1,1],[1,-2],[3,-3],[-2,-1],[-4,-1],[-1,-2],[4,-2],[2,-1],[1,-2],[0,-1],[0,-2],[3,-7],[2,-3],[2,0],[-1,4],[0,1],[0,1],[0,1],[0,1],[0,2],[5,1],[3,2],[2,3],[2,2],[9,-3],[1,0],[1,-2],[3,-4],[1,-2],[0,-2],[-1,0],[-1,-1],[0,-2],[0,-3],[0,-3],[2,-1],[0,-1],[-1,-3],[-1,-4],[0,-2],[0,-3],[-1,-3],[-2,-2],[-2,-2],[-1,1],[-1,2],[-1,1],[0,-3],[0,-2],[1,-2],[2,-1],[1,0],[-4,-7],[-1,-2],[0,-3],[-1,-2],[-1,-2],[0,-1],[1,-1],[0,-2],[1,-5],[-2,-2],[-1,2],[0,3],[-1,2],[-1,-2],[-1,-3],[0,-4],[0,-3],[1,1],[0,1],[1,-1],[1,-1],[-2,-1],[-1,-2],[-1,-3],[0,-4],[-2,8],[-1,0],[-2,-3],[-2,-3],[-2,-9],[-6,-11],[-2,-4],[0,-1],[-2,-3],[-2,-5],[-2,-3],[-2,1],[0,2],[1,2],[0,2],[0,2],[-2,-2],[-1,1],[-2,5],[1,5],[0,1],[-1,7],[1,6],[2,6],[1,6],[-3,-1],[-2,-2],[-2,-3],[-1,-6],[2,0],[1,-3],[0,-5],[-1,-5],[0,-1],[-2,-5],[0,-1],[1,-2],[0,-1],[0,-1],[0,-2],[0,-2],[0,-4],[0,-4],[1,-2],[3,-3],[2,0],[1,-1],[0,-3],[-3,-6],[-1,-1],[-1,-1],[-5,-9],[-4,-3],[-1,-4],[-1,-9],[-1,-1],[-8,0],[0,-2],[3,-1],[2,-1],[1,-1],[-2,-5],[-3,-6],[-4,-5],[-2,-4],[-4,-9],[-1,-2],[-2,-1],[-1,2],[-1,2],[-1,2],[-5,-2],[2,-4],[3,-1],[3,-2],[-1,-8],[-1,-6],[-2,-6],[-3,-6],[-2,-3],[0,-5],[-2,-6],[-9,-14],[-2,-2],[-1,0],[-2,1],[-1,1],[-2,-2],[3,-3],[1,-2],[-1,-2],[-2,-4],[-6,-8],[-1,-2],[-1,-4],[-4,-18],[-1,-4],[0,-4],[1,3],[3,5],[2,2],[1,-3],[-1,-4],[-2,-9],[0,-2],[0,-2],[-1,-1],[-1,-1],[-1,-1],[0,-3],[0,-1],[1,0],[0,-1],[-1,-3],[-1,-3],[-1,-2],[-1,-2],[-9,-7],[-3,-5],[2,-1],[7,-1],[1,0],[1,-2],[1,-1],[1,0],[1,0],[0,-8],[-1,-2],[0,-2],[-3,-14],[-1,-7],[-1,-3],[-3,-8],[2,2],[2,3],[3,8],[6,13],[2,8],[14,29],[3,4],[1,1],[0,2],[1,2],[1,2],[4,2],[3,7],[2,1],[5,-12],[1,1],[-1,5],[0,4],[0,3],[2,4],[2,-1],[3,-4],[-1,6],[-1,3],[0,3],[5,17],[2,5],[3,3],[3,2],[3,12],[7,8],[2,2],[1,1],[2,1],[2,0],[0,-1],[-1,-3],[0,-1],[2,0],[1,-1],[1,-1],[-2,-1],[1,-3],[2,-2],[2,-2],[0,-3],[-1,-3],[-3,-6],[-1,-3],[-2,-11],[-1,-4],[-2,-3],[-4,-8],[4,2],[3,5],[6,13],[2,2],[5,3],[2,0],[0,-3],[-1,-4],[-4,-5],[4,-2],[2,5],[2,8],[1,4],[4,-1],[3,-2],[4,-3],[2,-6],[-1,0],[1,-2],[4,-7],[1,0],[1,1],[1,0],[0,-1],[1,-1],[0,-2],[1,-1],[2,-2],[3,0],[1,1],[1,2],[1,2],[1,2],[-1,2],[3,2],[6,-1],[4,0],[1,1],[9,9],[0,-1],[2,-7],[1,-4],[0,-4],[-2,-4],[-2,-3],[-2,-1],[-2,-1],[-5,-2],[-8,-10],[-5,-3],[-2,-6],[-11,-5],[-17,-15],[-2,0],[-1,1],[-1,2],[-1,0],[-1,-3],[1,-3],[-2,-3],[-6,-4],[1,-4],[-3,-5],[-7,-7],[-1,-2],[0,-2],[-1,-3],[0,-4],[1,-3],[1,3],[3,7],[3,5],[7,9],[8,7],[1,1],[2,0],[2,1],[2,1],[1,1],[0,-1],[1,-1],[0,-1],[-2,-1],[-4,-3],[-2,-1],[0,-2],[2,0],[2,1],[1,2],[2,2],[1,-2],[1,0],[5,0],[-1,-1],[0,-2],[-1,-1],[-1,0],[-1,-1],[0,-2],[0,-2],[-1,-2],[-1,-3],[-6,-5],[1,-3],[2,-1],[4,0],[-3,-8],[-4,-6],[-5,-4],[-4,-2],[-2,0],[-2,-4],[-1,0],[-1,-1],[-1,-1],[0,-2],[0,-2],[-4,-7],[-2,-3],[1,0],[3,2],[4,2],[8,10],[5,3],[10,5],[4,0],[6,-2],[-1,-3],[-1,-1],[2,0],[2,0],[1,-1],[0,-1],[2,-1],[1,-2],[-1,0],[-2,0],[-1,0],[-1,0],[0,-1],[-1,-1],[-1,-2],[-1,-1],[3,-1],[3,1],[7,5],[-1,-4],[-1,-4],[-1,-4],[3,0],[0,-2],[0,-2],[0,-3],[0,-2],[2,-1],[1,2],[1,3],[1,3],[1,3],[5,11],[3,4],[3,1],[-3,-11],[-1,-6],[0,-3],[-1,-3],[0,-3],[0,-3],[0,-3],[3,5],[1,3],[1,3],[0,4],[0,3],[2,1],[2,1],[-1,1],[0,1],[0,1],[0,3],[1,2],[2,-4],[1,1],[1,4],[1,3],[2,1],[7,2],[-1,-3],[1,0],[1,1],[1,-1],[-7,-13],[-2,-2],[0,-2],[3,2],[7,6],[3,-1],[-2,-3],[-7,-13],[-4,-4],[-2,-4],[3,2],[3,2],[2,-2],[-1,-7],[2,5],[2,6],[2,6],[2,0],[-2,-3],[2,-1],[3,3],[0,-2],[-1,-5],[-2,-1],[-1,-1],[-2,-2],[1,-1],[1,-3],[0,-3],[1,-2],[1,0],[0,1],[0,3],[0,3],[1,2],[3,2],[2,3],[1,2],[1,2],[0,1],[-4,7],[0,3],[-1,4],[1,4],[0,2],[-5,14],[3,0],[1,1],[2,2],[4,-3],[1,-2],[-2,-2],[1,-1],[1,-1],[1,-1],[1,0],[-1,1],[0,4],[-1,2],[3,1],[2,1],[2,1],[2,-2],[-2,0],[0,-1],[2,-3],[1,-3],[0,-3],[0,-4],[0,-4],[-2,-3],[-1,-2],[-2,-3],[1,-4],[-1,-4],[-2,-2],[-1,3],[-2,-3],[-1,-6],[-1,-5],[2,-7],[0,-10],[-1,-3],[-1,-2],[-1,-1],[-3,-1],[-3,-3],[-1,-2],[-1,-3],[-1,-3],[-1,-2],[-4,-5],[0,-2],[0,-1],[3,0],[2,-1],[-2,-8],[2,0],[1,-2],[0,-3],[-1,-3],[-2,-3],[-3,-4],[-2,-3],[2,1],[3,4],[2,2],[2,0],[3,0],[2,1],[9,10],[0,2],[-3,-1],[-6,-5],[-2,0],[-1,5],[0,6],[2,5],[2,3],[0,2],[-4,-2],[0,2],[0,3],[2,1],[1,1],[1,0],[2,2],[4,4],[16,14],[-1,1],[1,2],[-2,6],[-1,3],[0,4],[2,-3],[2,-4],[2,-1],[2,5],[2,-12],[0,-4],[-1,-4],[-1,-3],[-4,-5],[2,-1],[2,2],[2,2],[1,3],[3,9],[1,1],[3,0],[1,0],[0,-2],[0,-2],[2,-1],[1,1],[1,1],[1,2],[0,2],[1,8],[0,1],[-2,1],[-1,1],[0,3],[1,3],[0,3],[1,2],[2,3],[0,1],[2,-1],[1,-1],[-1,-5],[0,-14],[0,-2],[0,-3],[1,-4],[1,-3],[3,4],[1,2],[0,2],[0,4],[0,3],[1,2],[1,2],[1,0],[0,-1],[0,-3],[0,-2],[0,-2],[1,0],[1,1],[1,4],[0,2],[1,3],[1,2],[1,1],[2,1],[1,1],[0,1],[1,2],[1,4],[1,0],[0,1],[1,1],[0,2],[-1,1],[1,3],[6,11],[3,6],[2,3],[2,2],[2,0],[4,2],[2,1],[1,-1],[-6,-8],[0,-2],[-1,-7],[0,-1],[-3,-6],[2,3],[4,3],[3,2],[3,-1],[-2,-3],[-1,-3],[0,-3],[1,-3],[-3,-5],[-1,-1],[0,-5],[-1,-3],[1,-1],[2,-2],[1,-4],[-1,-5],[-2,-9],[0,-3],[1,1],[0,3],[1,4],[3,3],[2,3],[3,2],[-3,4],[1,5],[2,5],[2,7],[3,2],[5,1],[0,-4],[1,-1],[4,3],[0,-4],[1,0],[4,2],[-1,-3],[-1,-3],[0,-3],[0,-4],[1,3],[1,4],[1,3],[1,2],[2,-5],[0,-2],[1,0],[1,4],[1,5],[1,4],[1,4],[6,2],[1,-2],[1,2],[1,2],[0,1],[1,1],[0,1],[1,0],[1,-3],[1,-1],[3,-1],[1,-1],[-2,0],[-4,-5],[3,-1],[7,4],[3,1],[2,-2],[4,-6],[1,-1],[2,0],[4,-3],[2,-2],[2,1],[2,-2],[4,-4],[5,-2],[2,-2],[-2,-2],[2,-4],[4,-2],[3,0],[2,-2],[0,-2],[1,-1],[0,-1],[1,0],[1,0],[1,0],[0,-2],[0,-1],[1,-1],[2,-1],[1,0],[2,2],[2,-5],[1,-1],[1,-1],[0,-1],[0,-3],[-4,-6],[0,-5],[-1,0],[-2,-1],[-1,0],[-1,-1],[0,-2],[0,-4],[0,-2],[-2,-4],[-1,-1],[0,-2],[-2,0],[-3,3],[0,-1],[-1,-2],[-1,-2],[0,-2],[1,-1],[1,-1],[2,-1],[1,-2],[0,-5],[-1,-4],[-2,-2],[-21,-4],[-5,2],[-1,-1],[1,-2],[2,-1],[1,-1],[8,0],[2,0],[-3,-4],[-3,-2],[-4,-2],[-3,0],[-2,0],[-2,3],[-1,0],[-2,-3],[1,-2],[4,-3],[5,-6],[0,-1],[-2,-1],[-2,-3],[-2,-1],[-7,2],[-2,-2],[-1,-2],[-1,-1],[-2,0],[2,-3],[3,0],[6,3],[0,-2],[-9,-5],[-2,-3],[1,-4],[-1,-1],[-2,0],[-4,0],[-1,-1],[-4,-3],[-7,0],[-1,0],[-3,-4],[-1,0],[0,-3],[0,-2],[-1,-1],[-1,-1],[0,-2],[4,1],[16,11],[2,0],[2,-2],[0,-3],[-1,-2],[-2,-3],[3,-1],[2,1],[1,2],[-2,1],[4,9],[5,4],[6,0],[5,-6],[-3,-3],[-3,-1],[-7,1],[1,-3],[1,-3],[0,-3],[1,-1],[1,0],[1,3],[2,1],[1,-1],[-4,-6],[-2,-2],[-3,0],[-5,3],[-3,0],[-3,-2],[0,-1],[6,-1],[3,-2],[0,-4],[1,-2],[-1,-2],[-1,-3],[0,-3],[1,-2],[2,-1],[2,-1],[1,-1],[0,-2],[-2,-1],[-2,-2],[5,1],[11,11],[0,2],[-9,-3],[-5,0],[-1,3],[2,4],[4,5],[4,6],[0,-2],[-2,-9],[12,2],[0,-3],[3,0],[1,-1],[1,-1],[-1,-2],[-1,-1],[-1,-1],[0,-2],[0,-1],[2,-1],[1,1],[6,5],[1,3],[1,1],[2,0],[1,-1],[1,-2],[-1,-2],[0,-1],[-1,-1],[-1,-1],[-2,-3],[-2,-2],[-2,0],[-4,-1],[-8,-4],[-5,-7],[-2,-1],[-4,0],[-3,-1],[-2,-1],[-1,-3],[-1,-3],[-2,-3],[2,-1],[1,2],[2,3],[1,1],[22,-1],[-2,-2],[-5,-1],[-2,-2],[1,-1],[1,0],[1,0],[0,1],[2,-3],[0,-2],[-1,-1],[-3,-3],[-2,-1],[-4,-1],[-3,-1],[-4,-2],[-3,-3],[-6,-7],[-4,-3],[-3,-1],[-4,1],[-2,0],[-2,-1],[-4,-7],[-2,-1],[1,-2],[1,-2],[1,-2],[0,-3],[6,11],[2,2],[2,0],[5,-1],[3,1],[9,12],[5,4],[5,-2],[-2,-4],[-6,-10],[0,-5],[1,-4],[2,-1],[2,2],[-2,2],[1,3],[4,5],[0,2],[0,2],[1,2],[1,1],[2,0],[0,2],[0,2],[1,3],[3,6],[4,7],[5,4],[4,-1],[-1,-2],[-2,-3],[-1,-2],[1,0],[2,0],[1,-1],[1,-2],[-10,-11],[-1,-2],[-1,-4],[0,-1],[3,2],[2,2],[3,2],[2,0],[1,3],[4,15],[1,0],[0,-3],[0,-2],[-1,-6],[2,2],[1,1],[2,0],[-8,-20],[2,3],[4,6],[2,5],[2,-2],[1,1],[1,6],[1,1],[0,4],[0,1],[1,1],[1,-1],[1,0],[0,4],[1,2],[1,1],[0,2],[-1,3],[0,2],[0,4],[0,1],[2,2],[1,1],[0,2],[0,1],[2,2],[1,0],[3,-2],[4,0],[1,-1],[4,-6],[1,-2],[-2,-6],[1,-1],[1,1],[2,2],[0,-3],[1,-3],[0,-2],[1,-1],[2,0],[1,2],[0,3],[0,2],[2,3],[5,5],[1,3],[-1,2],[2,4],[0,2],[1,1],[2,1],[2,0],[2,1],[-1,1],[2,12],[1,1],[1,-1],[-1,-2],[1,-2],[1,-1],[2,-1],[1,-1],[1,-2],[1,-5],[0,-3],[4,-6],[0,-3],[-3,-4],[1,-1],[1,-3],[0,-1],[-1,-2],[-2,-1],[-2,-1],[0,-3],[0,-2],[-1,-4],[-1,-2],[0,-2],[0,-5],[-1,-7],[-3,-4],[-7,-5],[-1,-2],[-3,-7],[-1,-1],[-3,0],[-2,1],[-4,6],[-2,0],[-3,0],[-1,-2],[-1,0],[0,1],[-1,1],[0,1],[-2,0],[0,-1],[0,-6],[-3,-13],[2,-3],[-2,-2],[-7,0],[-1,-2],[-1,-5],[-3,-2],[-6,-2],[0,-1],[-3,-6],[-4,-5],[-1,-2],[-3,-3],[-3,-1],[-2,1],[-8,6],[-2,1],[-3,0],[-5,-2],[-1,2],[-1,2],[-1,3],[-1,2],[-3,2],[-2,0],[-2,-2],[0,-6],[0,-6],[1,-4],[1,-4],[1,-5],[0,-7],[1,-2],[12,-3],[9,2],[4,0],[3,-5],[-7,-5],[-7,-3],[-8,0],[-6,3],[-2,-2],[2,-3],[2,-1],[5,-1],[3,-1],[2,-1],[3,-1],[4,4],[6,1],[2,2],[4,5],[3,1],[-1,-3],[-1,-2],[0,-6],[-2,-10],[-6,-17],[-3,2],[-7,0],[0,-2],[3,0],[4,-1],[-1,-5],[0,-7],[-1,-6],[-3,-2],[-1,-2],[-1,-3],[-1,-3],[-1,0],[-2,4],[-1,2],[-1,3],[-1,3],[-3,5],[-3,3],[-2,1],[-1,-2],[5,-6],[0,-5],[3,-2],[0,-4],[0,-4],[1,-2],[1,-2],[1,-3],[-1,-4],[-2,-1],[1,-2],[4,-7],[2,-4],[3,-9],[3,-1],[2,6],[2,7],[2,4],[2,-2],[0,-3],[-1,-4],[-1,-3],[-2,-5],[0,-1],[-1,-2],[0,-3],[1,-2],[1,1],[1,1],[1,0],[1,0],[0,-2],[0,-3],[1,-1],[0,1],[0,1],[1,-1],[0,-1],[0,-3],[0,-2],[-2,-6],[1,2],[3,5],[1,1],[1,0],[0,-1],[1,-1],[0,-2],[0,-1],[1,1],[1,1],[1,0],[3,-2],[0,1],[1,4],[0,3],[-1,2],[0,2],[2,2],[-2,4],[-1,3],[1,1],[1,0],[1,-1],[3,-2],[-1,4],[1,2],[0,3],[-1,4],[2,0],[1,3],[1,2],[0,1],[-1,2],[-1,3],[0,4],[0,3],[2,3],[1,2],[1,1],[0,5],[1,3],[2,4],[-1,2],[2,5],[1,3],[1,2],[2,1],[1,0],[3,-1],[0,1],[-1,3],[1,3],[1,3],[1,4],[1,3],[2,8],[1,4],[3,6],[1,1],[3,0],[1,0],[1,2],[1,1],[0,2],[4,1],[0,1],[0,1],[1,2],[3,1],[2,-2],[1,-3],[3,-1],[1,1],[3,2],[2,2],[1,2],[5,9],[4,11],[2,4],[1,1],[3,-3],[4,-9],[3,-3],[0,-1],[-1,-2],[-1,-1],[-5,0],[-2,-1],[0,-2],[-1,-2],[0,-1],[-2,-4],[-1,-2],[-1,-2],[0,-4],[-1,-4],[-1,-4],[-2,-4],[-4,-5],[-2,-4],[-1,-5],[0,-6],[0,-4],[-1,-4],[-2,-5],[-5,-6],[-1,-4],[-1,-5],[-1,-2],[-1,-1],[-1,-2],[-1,-1],[-2,-1],[0,-1],[-1,0],[1,-2],[1,-1],[3,-2],[-1,-3],[-1,-3],[1,1],[1,0],[0,-1],[1,-2],[-2,-2],[-2,-8],[-3,-1],[-2,-1],[-2,-2],[-1,-3],[1,-3],[1,2],[2,1],[1,0],[1,1],[-1,-3],[-3,-3],[-1,-3],[3,1],[2,1],[4,5],[-1,-4],[-3,-4],[-3,-2],[-2,-2],[-1,0],[1,-1],[1,-2],[2,0],[1,0],[4,3],[0,-9],[0,-2],[-1,-3],[-1,-2],[0,-2],[-1,-3],[2,2],[2,3],[1,2],[2,0],[-1,-3],[0,-1],[1,-2],[-4,-6],[-1,-3],[0,-2],[3,1],[1,1],[1,1],[3,-1],[-1,-4],[0,-1],[2,3],[4,9],[4,4],[8,10],[7,7],[2,4],[2,4],[1,5],[2,5],[1,5],[2,22],[2,11],[3,3],[1,-1],[1,-4],[1,-1],[4,-3],[1,-2],[1,-10],[0,-2],[-2,-8],[1,-2],[1,-1],[3,1],[1,-1],[1,-2],[-1,-11],[0,-4],[-3,-8],[1,1],[1,-3],[1,-2],[2,-2],[2,-1],[1,-1],[0,-2],[-1,-7],[-2,1],[-2,-2],[-3,-4],[3,-4],[0,-2],[0,-4],[-2,-3],[-3,-4],[-1,-2],[0,-4],[-2,-9],[-1,-3],[-1,-1],[-4,-1],[-2,-1],[1,-1],[1,-1],[2,0],[1,0],[-1,-2],[-2,-2],[-3,-2],[1,-4],[0,-1],[-1,-2],[-1,-6],[-1,-4],[-1,-5],[0,-1],[-1,-1],[0,-2],[-1,-8],[0,-4],[-2,-3],[-1,-2],[-4,0],[-2,-2],[5,-3],[3,0],[2,-1],[0,-2],[-1,-1],[-2,1],[0,-3],[1,-1],[0,-2],[1,-1],[0,-2],[-4,-4],[0,-3],[1,-4],[-2,-1],[0,-2],[1,-1],[1,-1],[0,-2],[-3,-9],[-1,-5],[-1,-6],[-1,-5],[1,-5],[-1,-2],[0,-3],[-1,-3],[-2,-2],[-1,-4],[-1,-2],[-3,-1],[0,-1],[-3,-4],[-4,-17],[-3,-3],[-6,-5],[-3,-1],[-2,0],[-2,2],[-3,5],[-2,7],[-2,7],[1,6],[-2,-1],[-2,-3],[-3,-7],[-1,0],[-1,2],[0,3],[-1,1],[-1,-1],[-1,-2],[-2,-5],[0,3],[3,10],[-2,-1],[-1,-3],[-4,-6],[-1,-2],[-1,-2],[-1,-4],[-1,-1],[-2,-1],[-1,-1],[-4,-6],[-4,-2],[-4,2],[-4,7],[0,2],[-1,5],[0,2],[0,4],[2,8],[1,3],[1,2],[-1,3],[0,3],[0,2],[1,4],[4,8],[1,4],[1,4],[5,2],[2,4],[-2,-1],[-4,-2],[-1,-1],[-8,-18],[-2,-3],[-1,-1],[0,1],[0,7],[1,2],[0,2],[2,3],[0,2],[1,7],[1,2],[6,6],[3,4],[0,4],[-1,-2],[-2,-2],[-1,-1],[-2,1],[1,2],[0,2],[0,2],[-1,2],[-1,-2],[-1,-4],[0,-1],[-1,0],[-1,1],[-1,1],[0,2],[2,5],[-1,1],[-4,-1],[2,6],[2,6],[3,5],[1,4],[-1,2],[2,4],[6,6],[1,3],[-2,0],[-1,2],[0,3],[1,3],[-2,-1],[-2,-2],[-2,0],[0,1],[1,7],[0,3],[-4,-14],[-2,-6],[-4,-3],[2,9],[0,3],[-1,-1],[-1,-1],[-1,-2],[0,-3],[-2,-9],[-1,0],[-1,-2],[-1,-1],[1,-2],[-1,-2],[-2,-4],[-2,0],[-2,-1],[-1,0],[-1,-2],[-5,-17],[-1,-1],[-4,-3],[-1,0],[0,-2],[-1,-5],[0,-2],[-1,-1],[-3,-3],[-2,-1],[-1,-2],[0,-3],[-1,-2],[-1,-1],[-4,-2],[-1,-1],[-3,-5],[-1,-2],[-1,-1],[-1,-1],[-2,0],[-1,0],[-1,-1],[-1,0],[-1,3],[-1,0],[-6,2],[-1,1],[-1,1],[0,1],[-1,7],[0,6],[1,5],[1,5],[2,8],[0,3],[1,5],[0,2],[5,10],[1,3],[7,29],[2,5],[0,3],[1,3],[0,2],[1,2],[6,-2],[2,1],[-1,1],[-3,3],[-2,2],[-1,3],[0,2],[0,5],[0,4],[1,0],[2,-1],[0,-1],[-1,-1],[2,-1],[2,1],[2,2],[0,3],[3,3],[2,2],[0,3],[-3,0],[-1,1],[1,2],[2,5],[2,3],[2,2],[5,2],[0,2],[-2,0],[-1,0],[-1,2],[-1,1],[1,0],[-6,-3],[-1,1],[-1,2],[0,1],[-1,2],[0,1],[2,3],[0,2],[0,2],[1,4],[1,3],[0,2],[-2,2],[2,10],[0,6],[-2,4],[-1,-2],[-1,0],[0,1],[-1,2],[0,1],[-1,2],[0,4],[0,3],[0,3],[-1,2],[-2,14],[0,2],[0,1],[1,1],[-1,1],[-1,1],[0,-1],[-1,-1],[-1,0],[-1,3],[0,6],[-1,3],[1,3],[-4,-4],[-2,-1],[-2,3],[0,2],[1,1],[0,1],[1,0],[-1,1],[0,2],[-1,2],[0,2],[-1,1],[0,3],[-1,-3],[-1,-6],[-1,-2],[-2,0],[-1,2],[-3,4],[0,1],[0,3],[-1,1],[-1,1],[-3,1],[-7,5],[2,-4],[2,-1],[2,-1],[2,-2],[1,-4],[-1,-5],[-3,-15],[-3,-10],[-3,-8],[-4,1],[-2,2],[0,-3],[1,-7],[0,-5],[-3,-7],[-1,-3],[-1,-3],[-3,-8],[-2,-7],[-2,-3],[-3,-4],[0,-2],[0,-1],[1,-1],[1,4],[1,-4],[1,-3],[0,-3],[2,-2],[-4,-5],[-2,-2],[1,4],[0,2],[0,2],[-2,-1],[-1,-2],[-2,-7],[-2,-3],[1,-1],[1,0],[1,1],[0,2],[1,-2],[-4,-5],[-2,-1],[0,2],[-6,-6],[-3,-2],[-1,4],[0,2],[4,5],[16,38],[3,11],[0,1],[-1,1],[-2,-4],[-4,-10],[-4,-5],[-4,-15],[-7,-14],[-6,-6],[-1,-2],[0,1],[-1,1],[0,2],[0,2],[-1,-1],[-1,-1],[-1,-2],[-1,-2],[1,-1],[0,-1],[0,-1],[0,-1],[-1,-1],[-1,0],[-1,1],[0,4],[-2,-2],[-1,-1],[-2,0],[0,5],[-3,-3],[-1,-1],[-1,0],[3,6],[1,4],[-1,3],[-1,0],[-2,-4],[-1,-7],[-2,2],[0,3],[1,3],[2,3],[-2,-1],[-2,-3],[-1,-6],[-2,-3],[-4,-5],[-1,-3],[0,-2],[0,-2],[-1,-2],[-1,-2],[-3,-3],[-1,-1],[-1,-1],[-2,-4],[-3,-6],[-1,-5],[1,-2],[1,-1],[1,-3],[-1,-2],[-1,-3],[-1,-3],[-2,-1],[-2,0],[0,3],[1,4],[-1,3],[-2,2],[-2,-2],[-2,-4],[-1,-3],[4,1],[1,-1],[1,-3],[0,-2],[-1,-3],[0,-2],[2,-8],[0,-3],[-3,0],[0,-1],[0,-3],[1,-1],[-5,-3],[-1,-3],[-1,-5],[0,-1],[-1,1],[-1,1],[-1,1],[0,3],[1,6],[0,1],[-1,2],[-1,0],[-1,-1],[0,-2],[-1,-1],[1,-6],[1,-2],[0,-6],[-1,-5],[-2,-3],[-2,-2],[0,-5],[-2,-5],[-4,-2],[-3,-1],[-5,1],[-1,0],[0,-3],[1,-3],[0,-2],[-3,-2],[-4,0],[-4,2],[-2,5],[1,8],[-5,-2],[-2,0],[0,6],[-1,-3],[0,-2],[0,-2],[-1,-2],[-1,-2],[-1,2],[-2,0],[0,-1],[-1,0],[-1,-2],[-2,-3],[-4,-4],[-2,-1],[-2,0],[-4,2],[-1,0],[-1,-4],[-2,2],[-3,1],[-5,0],[-3,1],[-6,4],[-5,4],[-2,3],[-2,4],[0,3],[1,6],[1,4],[5,8],[4,9],[2,4],[3,2],[2,1],[5,4],[2,1],[3,0],[6,-3],[2,1],[2,2],[8,2],[6,4],[3,1],[1,1],[1,2],[1,4],[1,2],[0,2],[1,1],[1,0],[2,1],[3,7],[4,2],[3,5],[2,2],[2,3],[2,4],[0,3],[0,5],[0,2],[0,3],[2,5],[0,3],[0,6],[2,3],[8,12],[2,2],[4,1],[2,3],[2,0],[4,0],[1,0],[2,1],[3,3],[2,1],[3,0],[1,1],[0,5],[1,1],[4,0],[2,0],[4,6],[1,3],[0,4],[-2,-4],[-1,0],[-1,2],[1,2],[2,4],[1,3],[1,1],[6,5],[3,1],[2,1],[2,3],[1,1],[1,1],[-2,1],[-4,-1],[-2,0],[1,3],[-7,-9],[-8,-7],[-8,-3],[-12,-2],[-2,1],[1,4],[4,6],[2,4],[0,4],[1,2],[8,16],[1,4],[0,5],[-5,-9],[-8,-22],[-5,-9],[1,-5],[-2,-4],[-2,0],[-3,4],[0,3],[1,4],[2,8],[-2,-3],[-3,-3],[-3,0],[-4,4],[-3,1],[-6,1],[0,1],[-1,0],[0,1],[-1,0],[0,-1],[0,-1],[-1,-1],[0,-1],[-3,0],[0,1],[0,2],[0,1],[0,1],[-1,1],[0,2],[0,1],[1,1],[0,1],[1,1],[1,2],[-2,1],[1,1],[0,3],[-1,0],[-1,-1],[-1,-1],[-1,-2],[1,0],[0,-2],[1,-1],[-1,-1],[-1,-1],[-1,0],[0,1],[-1,2],[-1,2],[-1,1],[0,-2],[0,-7],[0,-3],[-2,-3],[-2,-5],[-1,-3],[1,-2],[2,0],[1,3],[2,1],[2,-4],[-3,-5],[0,-2],[0,-1],[4,3],[0,-2],[-1,-1],[0,-1],[0,-2],[0,-1],[-2,-8],[2,-3],[-1,-5],[-1,-4],[-2,-3],[-3,-2],[-7,0],[0,-1],[-3,-4],[-1,-2],[-2,-6],[0,-2],[-2,-1],[-1,2],[1,4],[-1,4],[1,2],[1,1],[0,1],[-3,0],[-3,-4],[-3,0],[1,1],[1,2],[1,1],[3,11],[1,2],[2,1],[-1,3],[-3,4],[-2,1],[-1,-5],[1,1],[1,-1],[1,0],[-1,-2],[-1,-3],[-2,-2],[-1,-1],[-1,0],[-1,2],[-1,1],[-2,0],[0,-3],[1,-1],[0,-1],[-2,-2],[-2,-2],[-2,-2],[-2,-1],[1,5],[0,3],[0,1],[-1,3],[1,2],[0,3],[1,3],[-2,-2],[-2,-4],[0,-5],[0,-4],[-2,-3],[-2,0],[-1,2],[-3,1],[-1,-1],[-2,-3],[-2,-3],[-1,-2],[0,4],[3,5],[6,9],[7,14],[3,4],[-2,0],[-2,-3],[-1,0],[-1,-2],[-1,-2],[-1,-1],[0,3],[-2,0],[-2,-2],[-3,-6],[-2,2],[-2,-2],[-2,-4],[-2,-1],[-3,-1],[-5,-2],[-10,-8],[-1,0],[-5,6],[-1,3],[2,3],[2,2],[10,2],[12,8],[1,0],[2,0],[1,1],[0,1],[1,1],[2,1],[2,3],[1,1],[3,1],[2,1],[2,2],[5,7],[3,1],[2,1],[8,6],[-2,1],[-4,-1],[-6,-4],[-9,-9],[-3,-1],[-3,2],[-3,3],[-2,4],[0,6],[1,4],[9,9],[-1,1],[0,2],[-1,1],[1,1],[2,3],[0,1],[0,5],[1,1],[1,0],[0,2],[-3,3],[1,5],[3,4],[2,3],[-3,2],[2,4],[2,4],[3,3],[-2,5],[-1,2],[-2,1],[1,-6],[-1,-3],[-3,-2],[-2,-4],[1,-2],[0,-2],[-2,-4],[-1,-6],[-1,-6],[-1,-5],[-3,-3],[-9,-5],[-3,-2],[-2,-3],[-4,-8],[-1,-1],[-2,-1],[-3,1],[-1,1],[1,2],[0,1],[-1,2],[2,4],[2,5],[2,4],[-4,-5],[-1,0],[-2,5],[-2,11],[-2,4],[0,-4],[1,-3],[1,-4],[1,-5],[-1,-5],[0,-3],[-4,-6],[1,0],[1,-1],[1,-1],[1,-3],[-2,-2],[-1,-1],[-2,0],[-1,-1],[1,-2],[1,0],[-1,-1],[0,-1],[0,-1],[-1,-1],[3,-3],[-1,-1],[-1,-2],[-2,0],[0,2],[-1,3],[-1,1],[-1,0],[0,-2],[-1,-3],[-3,-2],[-5,1],[1,4],[-1,5],[0,4],[2,3],[-2,2],[-1,4],[0,8],[0,-4],[0,-4],[-1,-3],[-2,-1],[0,-2],[3,-2],[0,-6],[-1,-7],[-2,-4],[-3,-2],[-3,-1],[-8,2],[-4,2],[-2,0],[-6,-1],[-12,2],[-3,-1],[2,-2],[4,-3],[1,-3],[-3,-2],[0,-1],[-1,0],[-4,3],[-1,2],[-3,7],[-1,3],[-1,-2],[-1,-2],[2,-3],[1,-3],[2,-4],[2,-1],[0,-2],[-2,-1],[-6,-7],[-1,-1],[-2,2],[-1,-1],[-1,0],[-3,-1],[-1,2],[1,4],[-2,-2],[-3,0],[-3,3],[-2,3],[-2,3],[-3,1],[-16,-5],[-1,0],[-1,1],[-2,1],[-1,2],[0,1],[2,3],[1,2],[-2,6],[-1,1],[-3,-10],[-1,-1],[-2,1],[-2,1],[-2,0],[-4,-3],[-2,-1],[-2,0],[-1,3],[-1,3],[1,2],[0,2],[1,2],[-1,2],[-2,-1],[-1,-1],[-1,-1],[-1,0],[-1,0],[-2,1],[-1,1],[-10,-1],[-3,1],[0,1],[0,1],[0,1],[-1,-1],[-2,-1],[-1,-2],[-1,-2],[-6,-6],[-2,1],[-1,1],[-2,6],[-1,1],[-4,4],[-2,0],[-2,-1],[0,-3],[-1,-3],[-1,-2],[-2,4],[0,1],[-1,-1],[-1,-1],[-1,0],[-2,2],[-1,2],[-1,1],[-2,1],[-1,0],[-5,-1],[2,7],[1,3],[1,1],[-3,0],[-2,0],[0,-2],[0,-1],[-1,-1],[-2,-2],[-6,2],[0,1],[-2,2],[-2,2],[-2,-6],[-1,0],[-1,1],[1,3],[0,1],[-3,0],[-10,-6],[-3,-3],[-2,-1],[-1,1],[-5,4],[-1,0],[-1,-2],[-3,-5],[-2,-1],[-1,2],[0,3],[1,3],[0,2],[1,1],[3,10],[1,2],[1,2],[2,1],[1,1],[-7,-1],[-3,-2],[-2,-6],[-2,-10],[-1,-5],[-3,-4],[-1,0],[-2,2],[-2,1],[-2,0],[-3,-2],[0,3],[-1,2],[-1,1],[-1,-1],[1,-3],[-1,-1],[-4,-1],[-1,-2],[-2,-3],[-3,-2],[-1,3],[-1,0],[-7,-6],[-7,-1],[-17,1],[-4,-2],[-1,-3],[-1,0],[-1,1],[-1,1],[-1,-1],[-3,-2],[-1,0],[-9,-4],[-4,0],[-2,1],[-3,4],[-1,-1],[-2,-1],[-1,1],[-1,2],[-1,3],[-1,2],[-2,1],[-2,0],[-1,1],[-1,2],[1,4],[-1,6],[0,9],[1,9],[2,5],[-3,1],[-2,3],[-1,5],[0,5],[0,2],[-3,2],[0,1],[-1,3],[0,1],[-3,6],[-1,2],[0,2],[0,4],[1,1],[1,1],[1,2],[1,1],[2,0],[2,0],[2,1],[0,2],[1,2],[1,3],[7,12],[8,10],[19,16],[3,5],[7,14],[6,4],[2,2],[3,6],[2,3],[6,7],[2,3],[5,10],[0,2],[1,4],[1,3],[1,1],[2,0],[1,1],[3,7],[3,5],[4,3],[5,1],[0,-2],[-3,-2],[-3,-2],[-3,-3],[-3,-4],[2,0],[5,5],[6,2],[1,1],[2,3],[3,7],[2,3],[1,1],[7,3],[4,1],[2,0],[2,2],[0,2],[-2,1],[-2,0],[-7,-2],[-4,-3],[-3,-3],[-2,3],[-2,0],[-6,-1],[-2,1],[-4,6],[-3,2],[-12,2],[-2,-1],[-13,-6],[-11,-2],[-11,1],[-5,-1],[-18,-11],[-3,0],[1,2],[1,3],[1,2],[0,3],[0,3],[1,4],[2,3],[11,13],[16,16],[19,23],[4,3],[-2,-5],[-3,-6],[-3,-4],[-3,-2],[-2,-2],[-4,-6],[-3,-7],[-2,-5],[1,-3],[0,-2],[3,-3],[3,-2],[0,-2],[0,-5],[1,-1],[2,2],[2,6],[1,2],[0,3],[0,7],[1,0],[0,-7],[0,-3],[0,-2],[2,-2],[1,-2],[2,-1],[4,-1],[3,0],[2,1],[3,8],[1,10],[0,11],[2,9],[4,12],[2,9],[2,4],[4,7],[1,2],[0,2],[1,4],[1,6],[1,3],[1,1],[1,18],[5,9],[2,3],[3,14],[2,6],[3,3],[1,0],[2,-1],[0,-4],[-1,-3],[-2,-2],[3,0],[0,-2],[-1,-2],[-2,-2],[0,-3],[2,-3],[1,0],[4,3],[2,1],[9,-1],[4,-1],[4,-5],[5,-12],[3,-4],[17,-5],[4,3],[-3,2],[-11,0],[-4,4],[-2,3],[-4,4],[-1,3],[-2,9],[-1,3],[0,1],[-1,3],[1,5],[1,2],[2,0],[5,-2],[7,3],[7,-3],[3,1],[1,2],[6,11],[-1,2],[-4,-7],[-2,-2],[-3,-1],[-3,1],[0,3],[2,2],[2,2],[0,2],[-11,-9],[-6,-2],[-3,6],[1,3],[9,-1],[3,2],[3,4],[4,10],[-2,-1],[-4,-4],[-3,-2],[-3,0],[-12,3],[-6,3],[-2,3],[-2,3],[-1,4],[0,5],[1,16],[2,9],[3,8],[6,10],[5,8],[3,3],[4,6],[3,2],[4,0],[3,-3],[2,-4],[1,-5],[0,-7],[0,-6],[2,-4],[3,-2],[-3,10],[0,4],[3,4],[3,0],[3,-3],[3,-4],[3,-3],[3,-1],[4,-1],[0,1],[-2,2],[-9,10],[-3,5],[0,7],[-1,-2],[-1,-4],[-2,-2],[-1,0],[-1,1],[-1,2],[0,3],[-1,1],[0,2],[-1,3],[-3,3],[1,1],[0,1],[0,1],[-3,3],[-1,7],[1,8],[1,7],[3,11],[1,4],[1,1],[3,2],[0,2],[1,1],[0,3],[0,2],[1,2],[0,3],[3,6],[2,9],[3,8],[2,4],[3,14],[4,9],[5,16],[5,14],[1,4],[0,5],[1,5],[3,10],[3,10],[2,6],[0,5],[0,1],[-1,1],[0,1],[1,1],[0,1],[1,0],[0,4],[2,8],[3,6],[1,2],[0,4],[0,2],[2,3],[4,6],[2,7],[2,8],[2,6],[4,3],[9,-4],[5,-1],[4,4],[1,3],[-2,0],[-8,-2],[-2,0],[-2,1],[0,2],[1,0],[1,0],[1,1],[1,1],[-2,3],[-5,-1],[-1,5],[-1,5],[-4,0],[-3,1],[-2,1],[0,3],[1,1],[3,2],[2,1],[1,0],[0,-3],[1,0],[5,2],[5,3],[3,4],[1,1],[1,0],[3,-1],[2,2],[2,3],[5,4],[2,3],[1,2],[1,4],[1,3],[4,3],[2,4],[1,2],[1,1],[0,4],[0,4],[1,0],[1,-1],[2,1],[2,1],[-1,3],[-1,1],[-2,1],[-2,-1],[-1,0],[0,2],[-1,2],[1,1],[3,3],[-2,2],[-4,-1],[-3,2],[0,1],[-1,5],[0,1],[-1,2],[-1,1],[-2,0],[-1,0],[2,2],[7,-2],[1,-2],[1,-4],[1,-2],[3,2],[5,3],[1,2],[0,5],[-2,1],[-5,1],[-2,2],[5,0],[4,2],[3,4],[2,8],[1,2],[5,3],[1,1],[1,3],[0,3],[-2,7],[3,4],[1,-1],[1,-1],[2,0],[-1,3],[-1,3],[-4,4],[1,2],[2,3],[1,2],[2,6],[1,3],[2,3],[6,8],[19,14],[6,3],[6,1],[2,1],[4,7],[3,2],[8,3],[2,2],[1,3],[3,2],[9,5],[2,1],[3,5],[5,4],[2,1],[12,12],[1,-2]],[[9689,2464],[-6,0],[-2,2],[1,7],[4,10],[1,1],[3,2],[2,0],[2,2],[3,4],[2,2],[-1,-7],[-1,-4],[-5,-15],[-3,-4]],[[9638,2669],[-2,-1],[-4,0],[-1,4],[-2,1],[-1,3],[1,3],[4,0],[2,-1],[3,0],[0,-2],[0,-4],[0,-3]],[[9641,2734],[-1,-2],[2,1],[1,-2],[-2,-3],[-1,-4],[0,-3],[-1,-1],[-3,4],[-1,4],[-2,3],[0,3],[5,1],[1,2],[1,1],[2,-2],[-1,-2]],[[9642,2755],[2,-3],[0,-4],[-2,-3],[2,-3],[-2,-2],[-3,1],[-2,-2],[-3,0],[-3,1],[-1,3],[0,4],[2,4],[1,2],[3,0],[4,3],[2,-1]],[[9629,2854],[1,-4],[4,-1],[2,0],[1,2],[2,-1],[2,-1],[2,-1],[1,-3],[1,-2],[0,-2],[0,-1],[-2,2],[-2,1],[-2,0],[-1,-3],[-1,-2],[2,-2],[2,-1],[1,-2],[0,-2],[-2,1],[-2,2],[-1,-1],[3,-3],[0,-1],[-1,0],[-4,3],[-3,1],[-5,2],[-2,2],[-5,1],[-2,2],[0,4],[-1,3],[2,3],[5,2],[2,0],[1,1],[2,1]],[[9500,2906],[-3,-3],[-1,-4],[-2,-1],[-1,-4],[1,-2],[-1,-2],[-3,-1],[-4,1],[-2,3],[0,4],[2,4],[2,3],[3,0],[3,-1],[2,2],[2,1],[2,0]],[[9507,2929],[5,-3],[4,2],[6,0],[2,-4],[-4,-3],[-4,-3],[-4,0],[-3,-3],[-4,2],[-3,7],[1,5],[4,0]],[[9469,3035],[-1,-2],[-3,1],[-3,3],[0,3],[2,2],[2,2],[4,-1],[-1,-2],[-2,-1],[1,-2],[1,-3]],[[9479,3088],[2,-5],[-3,1],[-3,4],[-3,-2],[-2,2],[1,3],[4,1],[2,1],[1,-2],[1,-3]],[[9486,3091],[-3,0],[-1,2],[4,1],[1,1],[2,0],[1,-1],[-1,-3],[-3,0]],[[9470,3096],[-5,0],[-3,1],[0,4],[2,4],[0,4],[5,4],[3,2],[0,-3],[-1,-3],[2,-3],[1,-2],[0,-2],[-1,-6],[-3,0]],[[9393,3187],[-1,-2],[-1,-5],[0,-3],[0,-2],[1,-1],[0,-1],[0,-2],[-1,-2],[2,0],[4,5],[1,1],[1,-3],[2,-2],[2,-1],[2,3],[-1,3],[1,1],[1,0],[2,-2],[-2,-5],[-3,-2],[-5,0],[-3,-1],[-5,-6],[-3,-3],[-3,-1],[-12,3],[-1,1],[-1,2],[0,3],[0,3],[-1,1],[0,1],[-2,2],[-1,4],[-1,5],[-1,4],[2,2],[1,-1],[2,-3],[1,-1],[1,1],[3,3],[1,1],[1,0],[3,0],[2,0],[1,1],[2,3],[1,1],[2,1],[5,3],[3,1],[1,0],[0,1],[1,0],[0,-8],[-1,-1],[-1,-1],[-2,-1]],[[9070,3445],[3,-1],[7,1],[-1,-3],[0,-1],[0,-3],[2,0],[3,1],[2,1],[-2,-6],[-5,-4],[-13,-5],[-3,0],[-1,3],[1,2],[3,0],[5,0],[0,1],[-4,2],[-2,2],[-1,3],[0,4],[1,3],[1,2],[1,1],[3,-3]],[[9046,3486],[9,-3],[2,-3],[-1,-1],[-1,-2],[0,-2],[-1,-2],[-1,-2],[-1,-1],[-1,-1],[-1,-1],[0,-2],[5,-2],[2,-2],[-1,-2],[-2,-1],[-4,1],[-19,5],[-3,3],[-2,3],[-2,6],[0,4],[3,2],[0,-2],[-1,0],[0,-1],[2,-3],[2,0],[4,3],[-1,2],[-1,1],[-1,-1],[-1,-2],[-1,3],[2,2],[4,1],[1,2],[0,3],[0,2],[1,1],[1,0],[1,1],[0,2],[0,2],[2,-1],[1,0],[4,-2],[0,-2],[-3,-2],[0,-2],[2,-2],[1,-2]],[[8960,3559],[25,-7],[4,0],[-1,0],[3,1],[2,2],[3,1],[2,-2],[0,2],[1,-2],[4,-5],[1,-2],[-1,-2],[-4,-2],[-1,-2],[2,-1],[2,-2],[2,-1],[0,-1],[-6,-2],[-1,0],[-3,1],[-11,3],[-13,7],[-2,2],[-2,5],[-1,2],[-1,1],[-3,1],[-1,1],[0,2]],[[9010,3583],[3,-1],[2,-5],[-4,-3],[-2,0],[0,-2],[1,0],[3,-1],[1,0],[1,0],[2,2],[8,2],[4,-1],[3,-2],[-2,-1],[-2,-1],[-2,0],[-1,-3],[2,-1],[2,1],[3,0],[2,-3],[-26,2],[-10,5],[-3,0],[1,-1],[0,-2],[1,-1],[-1,-2],[-2,-1],[-1,1],[-2,2],[1,0],[-3,4],[-3,0],[-4,-1],[-3,1],[0,1],[2,0],[0,2],[-1,1],[-1,0],[-1,2],[22,-4],[2,1],[0,2],[-3,2],[-20,2],[1,4],[2,0],[2,-1],[2,1],[-1,1],[-1,2],[-2,3],[0,2],[1,0],[5,0],[2,-1],[8,-1],[2,-1],[4,-4],[3,-1],[3,-1]],[[9041,3618],[0,-1],[2,0],[-1,-3],[-3,-3],[0,-3],[4,2],[1,0],[0,-2],[-1,0],[-4,-3],[-6,-1],[-1,-1],[-3,-3],[-1,-2],[-2,1],[0,1],[-1,4],[-1,1],[0,1],[0,1],[0,1],[4,5],[1,1],[1,3],[2,1],[3,-1],[0,1],[-2,2],[-1,0],[0,2],[0,2],[1,1],[1,-1],[1,-1],[3,0],[2,-1],[1,-1],[0,-3]],[[9002,3683],[2,-2],[2,1],[2,1],[2,1],[1,0],[1,-1],[-3,-7],[-3,-4],[-1,-2],[1,-1],[2,-1],[1,-1],[2,-2],[-2,-3],[-4,-4],[-3,-2],[-3,0],[1,2],[0,2],[-1,1],[-1,0],[-2,0],[-1,-2],[-1,-2],[1,-2],[-1,-2],[-1,-2],[-1,-1],[-1,0],[-4,-2],[-1,-1],[-1,-2],[-1,-2],[0,-1],[1,-2],[2,-1],[1,1],[2,2],[2,2],[2,3],[2,1],[1,0],[8,-3],[2,-1],[1,-4],[0,-6],[0,-10],[0,-3],[-1,-1],[-2,0],[0,-2],[0,-2],[2,-3],[1,-3],[-1,-2],[-2,-1],[-4,0],[-2,1],[-6,10],[-2,3],[-1,0],[-2,1],[-1,1],[-1,4],[-3,3],[-2,2],[-1,3],[-3,0],[0,2],[2,4],[1,9],[0,8],[-1,3],[0,2],[1,3],[2,5],[4,4],[3,1],[2,2],[0,3],[0,4],[1,1],[2,1],[1,1],[1,0],[2,0],[2,-2],[2,-3]],[[8975,3817],[1,0],[1,0],[0,-2],[-1,-1],[-1,-3],[-1,-1],[-2,0],[-9,0],[-2,-1],[-2,-2],[-2,-3],[2,-2],[1,0],[2,0],[1,2],[1,2],[2,0],[9,-2],[6,-4],[0,-1],[-4,-4],[-4,0],[-18,4],[-5,3],[-1,5],[1,1],[7,4],[3,3],[2,2],[1,3],[3,6],[1,2],[2,0],[5,0],[2,-1],[1,-2],[1,-1],[1,-1],[0,-1],[-2,0],[-2,-1],[-3,-2],[1,-2],[2,0]],[[8950,3840],[1,0],[3,0],[7,-7],[1,-5],[-1,-7],[-3,-3],[-11,-9],[-1,0],[-4,4],[-4,6],[-2,4],[-1,8],[3,6],[3,4],[4,1],[-2,-6],[0,-3],[1,-1],[2,1],[2,6],[1,3],[1,-2]],[[8967,3898],[1,-1],[1,3],[1,0],[1,0],[1,0],[-1,-4],[-1,-5],[-1,-3],[5,1],[3,0],[-1,-3],[1,0],[-1,-2],[-1,-1],[-2,-3],[2,0],[-1,-6],[-2,-1],[-2,0],[-4,-2],[-2,1],[-2,4],[-2,3],[-2,2],[-1,1],[0,2],[0,2],[0,1],[-1,1],[-2,-2],[-1,-3],[-2,-2],[-2,2],[2,5],[9,14],[2,1],[4,-2],[1,-3]],[[8946,3915],[-2,-6],[6,0],[2,-6],[0,-2],[-1,-3],[-1,-3],[-3,-3],[-2,-2],[-3,-1],[-4,3],[-3,5],[-4,3],[-1,-1],[-2,-4],[-1,0],[0,4],[-1,1],[0,1],[1,3],[6,10],[3,4],[4,1],[5,0],[1,-1],[0,-3]],[[8713,4367],[2,-5],[-1,-1],[-2,-4],[-1,-1],[-6,0],[-1,-1],[-5,-4],[-2,-1],[-1,0],[-4,2],[2,3],[1,1],[-2,1],[-2,0],[-1,1],[-2,3],[1,4],[1,1],[1,1],[1,1],[2,-1],[1,-2],[2,-1],[-1,2],[0,2],[-1,2],[-1,1],[3,13],[1,3],[2,2],[2,1],[3,-1],[1,-2],[0,-1],[-1,-2],[0,-3],[3,-6],[-6,-1],[3,-4],[8,-3]],[[8651,4496],[8,-4],[2,-2],[1,-3],[2,-2],[4,-4],[-2,-1],[-3,1],[-3,2],[-5,5],[-10,5],[-6,0],[-5,-4],[-9,-9],[4,-1],[9,7],[3,2],[7,-1],[2,-1],[6,-6],[3,0],[-2,-5],[-7,-6],[-3,-4],[6,0],[1,0],[2,3],[3,2],[3,4],[2,2],[3,-1],[-2,-1],[-1,-1],[-1,-2],[0,-1],[2,0],[1,-2],[1,-3],[1,-2],[-2,-4],[-11,-9],[0,-2],[1,0],[1,-1],[1,-2],[1,-2],[-3,-1],[-10,0],[-6,-2],[-4,-3],[-1,-3],[-2,-1],[-14,-9],[-1,-1],[0,-3],[0,-1],[2,-1],[0,1],[25,18],[3,-1],[6,0],[6,2],[3,5],[1,4],[2,4],[3,3],[1,-3],[-1,-5],[0,-4],[-1,-1],[-2,2],[-2,-4],[5,-7],[-2,-4],[0,-2],[-3,-2],[-14,1],[6,-2],[8,-4],[2,-3],[1,-4],[-1,-4],[-2,-2],[-2,-1],[-2,0],[-6,-4],[-1,-2],[2,-2],[2,1],[5,5],[1,1],[2,0],[3,0],[2,1],[0,2],[1,2],[-1,1],[0,2],[2,4],[3,4],[2,2],[6,-1],[3,1],[0,1],[-4,0],[-5,2],[-3,4],[0,8],[3,-2],[3,-1],[6,-1],[1,0],[0,-2],[0,-1],[3,0],[1,0],[1,-2],[1,-6],[2,-2],[0,-1],[-4,-3],[-11,-2],[-3,-5],[8,-1],[3,-2],[3,-4],[-2,-4],[-2,-2],[-2,-2],[-3,0],[0,-2],[5,0],[2,0],[2,2],[1,2],[1,3],[1,2],[2,1],[1,-2],[-1,-4],[-3,-7],[-1,-4],[0,-4],[-1,-8],[-2,-4],[-7,2],[-1,-5],[2,-6],[4,-6],[4,-6],[4,-2],[12,0],[9,3],[2,-2],[-2,-3],[-1,-3],[3,0],[1,-1],[1,-1],[-4,-2],[-4,0],[-4,0],[-3,-6],[-1,-7],[-1,-4],[-2,-4],[-2,-3],[-1,-4],[0,-3],[6,7],[3,4],[0,4],[1,0],[0,7],[2,4],[4,-1],[3,-3],[7,2],[3,-2],[-2,-6],[-1,-2],[-5,-1],[-1,-1],[-5,-10],[0,-5],[2,2],[4,6],[1,3],[5,5],[13,-6],[3,-4],[1,-3],[0,-2],[-1,-3],[-2,-5],[-3,-6],[0,-2],[0,-1],[0,-2],[-1,-1],[-1,-1],[-23,-1],[-4,-2],[-3,-4],[0,-2],[1,-1],[1,0],[2,0],[-1,0],[4,2],[5,0],[17,-4],[2,1],[1,2],[2,1],[1,0],[0,-3],[-1,-1],[0,-1],[1,-1],[2,0],[1,1],[2,0],[-2,-4],[-2,-11],[-1,-4],[-2,-6],[-1,-1],[0,-1],[-5,-1],[0,-2],[6,1],[2,1],[3,3],[1,4],[2,10],[2,3],[1,1],[1,6],[2,1],[1,-1],[1,-2],[1,-1],[8,0],[2,-1],[0,-2],[0,-1],[1,-1],[0,-2],[0,-2],[0,-1],[-1,-1],[0,-6],[0,-3],[-1,-3],[-1,-1],[-7,-6],[-1,-1],[-2,0],[1,-1],[0,-1],[2,0],[-1,0],[2,0],[4,0],[3,1],[1,2],[6,10],[2,2],[5,3],[3,1],[1,-4],[-1,-2],[-2,-3],[-2,-2],[-2,-1],[1,-4],[-1,-2],[-1,-3],[-1,-3],[1,0],[4,3],[1,1],[0,1],[1,1],[2,0],[1,-1],[1,-1],[1,-3],[-1,-3],[-1,-1],[-2,-2],[-4,-9],[-2,-4],[-2,-1],[-7,-2],[-2,-2],[-2,-4],[-2,-2],[-9,-1],[-7,-3],[-3,-1],[-3,0],[-6,4],[-16,2],[-1,0],[-1,-1],[-1,-2],[0,-2],[-1,-2],[-3,-3],[-1,-1],[0,-4],[2,1],[6,4],[1,4],[1,1],[6,0],[0,-1],[-2,-7],[1,-8],[3,-7],[2,0],[0,4],[-1,4],[-1,5],[1,4],[2,1],[2,0],[3,2],[3,-4],[2,-2],[2,-1],[3,1],[7,4],[5,1],[9,-1],[1,0],[1,1],[0,2],[1,2],[1,0],[4,3],[1,0],[2,-2],[2,1],[1,2],[1,2],[2,5],[3,0],[8,-2],[0,-1],[0,-3],[-1,-1],[5,1],[2,-1],[2,-3],[3,-1],[3,1],[3,0],[1,-4],[-4,-4],[-20,1],[0,-2],[10,-3],[3,-2],[-5,-6],[-2,-4],[-2,-5],[2,2],[5,6],[3,2],[2,0],[2,0],[1,-1],[0,-4],[-1,-1],[-2,-1],[-4,-1],[0,-2],[2,0],[3,0],[2,-2],[1,-3],[-3,-2],[-5,-8],[-3,-1],[-8,-4],[2,-1],[2,-1],[2,1],[5,1],[4,0],[2,1],[1,4],[2,-1],[-3,-8],[0,-3],[3,-1],[2,2],[1,3],[1,4],[0,3],[3,5],[4,1],[5,-1],[3,-4],[-5,-1],[-2,-1],[-1,-3],[2,0],[13,-9],[1,-2],[1,-4],[-1,-3],[0,-2],[0,-3],[0,-1],[0,-1],[0,-1],[-13,-8],[-2,-2],[1,0],[2,-2],[9,0],[2,1],[3,2],[3,0],[2,-3],[1,-4],[0,-4],[-1,-3],[-2,-3],[-2,-1],[-2,0],[-1,-2],[-1,-2],[0,-2],[0,-2],[-1,-2],[-16,-9],[-6,-7],[-3,-2],[-1,-1],[0,-3],[-1,0],[0,-1],[-12,1],[-3,1],[-3,-3],[-4,0],[-5,2],[-4,4],[0,-5],[-3,-3],[-3,-2],[-3,-1],[15,3],[2,0],[2,-1],[2,-2],[0,-3],[-1,-2],[-4,-7],[-3,-5],[-1,-2],[-2,-2],[-6,-3],[-5,-6],[-5,-9],[-2,-3],[3,1],[3,1],[6,8],[8,5],[18,25],[0,-1],[1,1],[12,2],[2,0],[2,-2],[3,-3],[-2,-3],[-3,-7],[-3,-5],[-1,-3],[0,-4],[1,4],[1,0],[7,15],[2,2],[2,0],[2,-2],[0,-1],[0,-1],[1,-1],[0,-1],[1,0],[3,2],[1,3],[2,3],[2,2],[3,0],[7,2],[1,-1],[1,-1],[1,0],[1,4],[1,1],[1,1],[0,1],[2,0],[-1,-4],[0,-4],[1,-2],[1,2],[1,1],[0,4],[1,1],[1,1],[2,0],[1,0],[2,-1],[9,4],[5,-1],[2,-6],[-5,-5],[0,-3],[1,0],[-2,-1],[-2,-2],[-2,-4],[3,-7],[0,-4],[-1,-2],[1,-2],[0,-4],[0,-4],[0,-3],[-1,-1],[-9,-4],[0,-1],[0,-1],[-1,-1],[-1,-1],[-14,0],[-2,-2],[-2,-6],[-1,-3],[-1,-3],[1,-2],[1,0],[1,3],[1,3],[1,2],[1,1],[2,3],[2,1],[2,-1],[12,0],[2,2],[1,2],[2,0],[4,-3],[2,-1],[1,-2],[1,-2],[0,-3],[-4,-3],[-1,-1],[0,-3],[-1,-2],[-2,-2],[-1,-1],[-11,-1],[-7,2],[-30,-6],[2,-1],[4,-1],[2,-1],[-3,-6],[-18,-12],[-1,-1],[1,-1],[5,1],[2,1],[-2,-6],[-2,-3],[-2,-2],[-8,-2],[-3,-1],[-3,-3],[-3,-3],[2,0],[21,4],[2,1],[2,2],[0,2],[0,4],[0,2],[1,3],[3,3],[3,3],[6,2],[1,0],[1,1],[0,2],[0,1],[0,2],[2,2],[2,1],[1,1],[4,1],[4,2],[2,0],[0,-1],[-2,-2],[1,-2],[3,0],[5,2],[1,0],[1,-1],[1,-1],[1,0],[0,1],[0,1],[1,1],[0,1],[1,1],[1,0],[3,-3],[2,-1],[2,-1],[2,0],[2,3],[2,4],[2,3],[2,-1],[0,-3],[0,-4],[1,-1],[2,4],[1,2],[0,2],[0,1],[2,2],[4,2],[1,0],[1,-4],[-1,-4],[-7,-10],[0,-1],[-1,-3],[1,-1],[1,-2],[1,-1],[-1,-2],[0,-2],[2,0],[1,3],[0,3],[0,2],[1,2],[2,2],[2,1],[6,0],[2,0],[1,-1],[0,-3],[-3,-1],[-4,-1],[5,-4],[-1,-3],[0,-2],[2,0],[1,3],[3,2],[2,-2],[1,-3],[1,-5],[-2,-1],[-5,-5],[-2,-2],[-4,0],[-7,3],[-3,0],[-3,-1],[-14,-14],[-2,-3],[4,1],[5,6],[4,1],[8,2],[2,0],[4,-3],[2,0],[-1,-4],[0,-4],[1,-4],[-1,-1],[0,-2],[0,-4],[1,-3],[2,0],[1,3],[1,3],[1,13],[1,2],[3,6],[1,-4],[1,-5],[-1,-5],[-2,-3],[3,-1],[2,3],[3,8],[1,-1],[3,-2],[1,-1],[1,-2],[0,-2],[1,-2],[1,-1],[1,6],[0,2],[5,7],[2,2],[2,-3],[2,-3],[1,-1],[1,-1],[1,-1],[0,-2],[1,-6],[0,-1],[0,-2],[-1,-2],[-1,-4],[-5,-5],[-1,-4],[0,-1],[1,-1],[0,-1],[1,-1],[0,-5],[4,-2],[6,-1],[3,-1],[5,-11],[1,-6],[-2,-4],[0,-1],[7,-4],[1,0],[1,-3],[0,-3],[0,-3],[0,-3],[-1,-1],[-1,1],[-2,-1],[-6,-3],[-7,-2],[-2,-3],[-1,0],[-1,-1],[0,-2],[0,-3],[0,-2],[-2,-3],[-3,-2],[-5,0],[-9,3],[-3,-2],[-2,-3],[-5,-14],[-2,-2],[-3,1],[-5,5],[-11,6],[-3,1],[1,-4],[2,-3],[5,-5],[7,-3],[1,-2],[2,-3],[1,0],[7,0],[1,1],[3,3],[2,1],[6,0],[1,1],[0,1],[2,2],[2,1],[3,-2],[12,-1],[15,-10],[9,-9],[2,-3],[-3,-2],[-11,2],[0,-2],[4,-2],[2,-2],[-1,-3],[0,-3],[1,-2],[0,-4],[-1,-4],[-1,-1],[-5,-3],[-1,0],[-9,0],[0,-2],[1,-2],[1,-1],[1,0],[3,0],[1,-1],[0,-2],[0,-1],[6,-1],[3,-2],[1,-4],[1,-5],[2,0],[4,4],[3,2],[1,-1],[0,-4],[-1,-5],[15,19],[4,-1],[-2,-2],[-5,-9],[-2,-2],[0,-1],[-1,-2],[1,-1],[1,-1],[18,1],[6,-1],[6,-3],[4,-5],[2,-4],[1,-4],[0,-2],[-2,-3],[0,-3],[0,-2],[2,-4],[1,-2],[-1,-1],[1,0],[-1,-1],[-1,-1],[0,-2],[1,-2],[2,-1],[-1,-2],[0,-2],[0,-1],[1,-1],[0,-2],[-2,1],[-2,5],[-2,2],[-2,-2],[-2,0],[-4,0],[-2,0],[-2,-1],[-2,-1],[-2,-2],[-4,-10],[0,-2],[-2,-1],[-3,-2],[-2,-2],[-1,-3],[-1,-3],[0,-3],[1,-4],[-1,-4],[-2,-4],[-2,0],[-3,2],[-2,-2],[-3,-3],[-6,-2],[-5,2],[-5,3],[-1,-3],[0,-4],[1,-4],[1,-3],[-1,-3],[0,-3],[0,-3],[1,-2],[2,5],[1,4],[2,3],[3,1],[8,-1],[3,-2],[3,-4],[-2,-4],[-2,-2],[-3,0],[-6,2],[-1,-1],[3,-6],[2,1],[3,-2],[3,-3],[6,-3],[1,-3],[0,-3],[-3,-2],[-2,0],[-4,3],[-15,1],[-3,1],[-3,3],[3,4],[-1,2],[-2,1],[-2,-1],[-1,-3],[-4,0],[-6,2],[-10,7],[-12,3],[-3,1],[-10,8],[-15,5],[-2,3],[2,3],[20,5],[4,2],[1,0],[4,-3],[20,2],[-5,4],[-28,1],[-2,0],[-3,-2],[-2,-1],[-2,1],[-6,4],[-2,0],[-2,0],[-2,-1],[-2,-2],[-1,-3],[-1,-3],[-1,-3],[-2,-1],[2,-1],[18,-11],[6,-7],[2,-1],[6,-1],[2,-2],[2,-2],[10,-4],[7,-7],[11,-4],[5,0],[3,0],[2,-2],[-3,-2],[-34,2],[1,-1],[-1,0],[25,-7],[2,-1],[10,0],[5,1],[2,0],[8,-6],[2,-1],[1,1],[1,1],[1,1],[2,1],[1,-1],[2,-3],[0,-2],[2,-1],[2,-3],[1,-1],[0,-4],[-2,-3],[-5,-1],[-5,-4],[-25,6],[-7,0],[-5,-5],[0,-1],[2,0],[5,1],[4,0],[2,0],[1,-1],[1,-2],[0,-2],[0,-1],[-1,0],[-6,-2],[-10,1],[-5,-1],[2,-3],[3,-2],[2,-1],[12,-2],[2,1],[1,4],[2,-1],[2,-4],[1,-2],[7,-1],[4,-1],[2,-3],[-3,-1],[-3,0],[2,-3],[6,-4],[0,-1],[0,-1],[1,-2],[1,-1],[1,0],[2,1],[2,1],[0,-2],[-3,-1],[0,-1],[2,-2],[6,1],[3,0],[2,-2],[2,-3],[1,-3],[-1,-2],[-2,0],[-16,7],[-3,0],[-3,-2],[2,-2],[16,-5],[0,-1],[-10,-4],[-5,0],[-4,4],[4,-1],[3,1],[1,3],[-2,1],[-6,2],[-9,0],[-4,-3],[-3,0],[-2,1],[-2,3],[-2,2],[-2,0],[-4,-3],[1,-3],[2,-3],[5,-4],[-1,-2],[1,-2],[8,-5],[2,0],[4,4],[2,-1],[1,-1],[2,-1],[4,0],[4,2],[-1,-4],[0,-1],[6,0],[1,0],[2,3],[2,0],[9,0],[1,0],[0,-1],[1,-1],[0,-1],[3,-2],[1,0],[12,3],[4,2],[3,1],[4,-1],[0,-2],[-1,-2],[1,-2],[1,-2],[0,-3],[-2,-1],[-2,0],[-3,-1],[-1,-3],[0,-4],[-1,-2],[1,0],[2,0],[1,1],[1,1],[1,1],[1,1],[1,-1],[0,-1],[0,-2],[-1,-1],[0,-2],[1,-2],[-1,-2],[-9,-5],[-2,-2],[-1,-4],[4,-2],[2,0],[7,4],[2,0],[4,0],[3,-2],[3,-3],[-5,0],[-3,0],[-2,-2],[1,-1],[2,0],[3,0],[0,-2],[-14,2],[-8,-1],[-4,-4],[3,0],[3,-1],[1,0],[6,-3],[1,0],[0,-2],[0,-2],[-1,0],[-1,0],[-1,0],[-1,1],[-1,1],[-1,2],[-1,1],[-1,1],[-1,0],[-1,0],[0,-3],[2,-2],[3,-4],[2,-1],[7,2],[9,0],[2,-1],[2,-1],[3,-3],[3,6],[4,0],[4,-5],[4,-6],[-1,-2],[0,-2],[0,-2],[0,-3],[-3,2],[-4,0],[-4,0],[-2,-2],[0,-3],[3,-2],[5,-3],[2,-2],[4,0],[14,5],[1,0],[2,-2],[2,0],[13,-1],[4,-2],[2,-6],[-1,-6],[-2,-5],[-7,-5],[-6,-1],[-2,-1],[-7,1],[-2,-1],[2,-1],[0,-1],[1,-1],[0,-2],[1,0],[2,2],[1,0],[5,1],[10,-2],[3,1],[3,2],[3,6],[3,6],[2,7],[1,1],[3,-1],[2,-1],[-1,-3],[-2,-6],[1,-2],[1,-5],[0,-5],[-1,-4],[1,-1],[1,-1],[1,0],[-1,-2],[-2,-2],[-1,-2],[-2,0],[-1,-2],[-1,-3],[1,-3],[2,-1],[0,-1],[0,-4],[-2,-5],[-3,-4],[-1,-2],[1,-4],[5,12],[2,1],[2,2],[1,5],[2,10],[3,9],[1,3],[1,2],[0,2],[0,3],[1,5],[1,4],[2,2],[2,-4],[1,1],[2,-2],[1,-2],[2,-2],[0,1],[2,2],[1,1],[1,-1],[1,-2],[1,-2],[1,1],[2,2],[1,0],[0,-6],[-6,-9],[2,-1],[-1,-2],[0,-1],[-1,-1],[2,-2],[-1,-2],[-1,0],[-4,-1],[-4,-2],[-2,-1],[-2,-2],[-2,-4],[-2,-6],[0,-6],[3,5],[3,4],[5,1],[4,-5],[0,-7],[-2,-7],[-5,-13],[3,2],[3,5],[6,11],[1,2],[2,0],[1,-2],[-2,-7],[0,-2],[1,-1],[2,-1],[0,-3],[0,-1],[0,-1],[-3,2],[-1,0],[-1,-3],[-2,-4],[-2,-3],[-8,-7],[0,-2],[1,-1],[1,-1],[-1,-2],[-1,-2],[2,-3],[-1,-4],[-4,-8],[3,1],[2,3],[3,8],[-2,3],[1,3],[5,5],[3,8],[3,3],[2,-1],[-1,0],[-2,-3],[0,-2],[-2,-5],[-2,-3],[2,0],[1,1],[1,1],[0,3],[2,-4],[1,-1],[2,0],[1,1],[0,4],[-1,6],[1,5],[3,1],[3,-2],[3,-2],[-1,-4],[-1,-1],[1,-1],[1,-2],[-1,-2],[-1,-2],[2,-1],[1,0],[0,-2],[-1,0],[0,-1],[-3,1],[-5,-1],[-10,-8],[-2,-2],[-1,-9],[0,-2],[-3,-6],[2,-1],[-1,-3],[-3,-3],[-1,1],[0,2],[-1,0],[-1,1],[-1,0],[0,-1],[-3,-5],[-1,-1],[0,-1],[-1,-4],[0,-2],[-2,-4],[-1,-2],[0,-3],[-1,-1],[-8,-7],[5,3],[9,11],[4,2],[1,-1],[1,-2],[1,-1],[1,1],[2,6],[1,2],[1,1],[4,0],[-1,-3],[-6,-8],[-4,-4],[-2,-3],[-2,-4],[-2,-9],[-2,-3],[-2,-3],[-8,-5],[-5,-7],[-2,-2],[0,-1],[1,-2],[0,-3],[-3,-7],[2,3],[6,7],[5,9],[2,3],[2,1],[5,3],[2,2],[2,9],[0,1],[3,3],[0,1],[3,0],[1,-1],[1,-1],[2,2],[9,15],[2,3],[3,1],[2,2],[0,-1],[1,-1],[1,0],[1,1],[1,3],[0,1],[2,1],[1,3],[1,1],[2,-1],[-4,-10],[1,-3],[-1,-3],[-2,-6],[2,2],[5,8],[5,3],[4,6],[3,1],[-1,-4],[-1,-2],[-2,-1],[-1,-3],[-1,-3],[-1,-6],[-1,-3],[-9,-15],[-1,-6],[-1,0],[-3,1],[-4,-4],[-3,-8],[-2,-7],[1,1],[10,14],[11,11],[1,2],[1,3],[1,5],[4,11],[1,1],[1,0],[1,1],[-1,2],[3,2],[7,-2],[3,2],[-1,2],[0,2],[1,2],[2,0],[1,1],[1,2],[0,2],[1,2],[2,3],[3,3],[2,2],[3,0],[0,-4],[2,-4],[0,-2],[0,-1],[0,-1],[0,-1],[1,0],[2,4],[1,-5],[1,-5],[-2,1],[-1,-3],[-2,-5],[2,-1],[1,-1],[0,-1],[0,-2],[-1,-2],[-1,0],[-1,2],[-2,-1],[0,-1],[-1,-2],[-2,-2],[-2,-2],[-3,-1],[-3,-2],[-2,-1],[-5,2],[-3,0],[-1,-1],[0,-3],[0,-1],[1,-6],[1,1],[2,4],[2,2],[1,0],[1,-1],[2,0],[2,2],[0,-2],[0,-3],[0,-2],[7,-2],[2,1],[2,6],[1,1],[3,1],[6,-1],[1,0],[1,-2],[1,-1],[1,0],[2,1],[2,2],[5,6],[3,2],[0,1],[0,4],[1,4],[1,0],[2,-2],[1,-3],[0,-4],[0,-3],[-1,-3],[1,-3],[3,2],[3,1],[1,-3],[-2,-5],[-4,-5],[-1,-2],[-2,-6],[-1,-2],[-3,-2],[0,1],[-1,0],[-1,-1],[-7,-12],[-1,-1],[-1,0],[-1,0],[-1,1],[0,1],[-2,-4],[-5,-3],[-1,-2],[2,-3],[-2,-4],[-3,-4],[-5,-3],[-3,-4],[-2,-4],[-1,-4],[0,-3],[1,0],[1,2],[2,-1],[0,-2],[-1,-3],[-1,-2],[-1,-3],[-1,-5],[-10,-8],[3,-3],[2,1],[3,2],[3,3],[3,3],[3,2],[0,1],[0,3],[-1,2],[0,10],[1,5],[3,3],[2,4],[1,1],[5,1],[1,1],[1,2],[6,6],[20,27],[3,1],[1,-2],[0,-7],[2,3],[4,14],[6,11],[0,1],[0,4],[2,3],[0,2],[1,2],[0,1],[1,2],[2,2],[3,1],[1,0],[1,-3],[-1,-1],[0,-2],[0,-3],[0,-3],[1,2],[3,4],[1,4],[2,7],[1,-2],[2,-4],[1,-5],[-2,-10],[-5,-5],[-5,-4],[-4,-6],[0,-2],[-1,-4],[-1,-2],[-1,-1],[-3,-2],[-8,-10],[-1,-4],[2,-2],[1,2],[3,5],[2,2],[2,0],[3,1],[2,2],[2,4],[1,1],[0,5],[1,3],[5,4],[2,-5],[1,-1],[1,1],[0,5],[0,2],[1,3],[1,2],[2,-1],[2,-2],[-1,-3],[0,-3],[0,-3],[3,4],[1,3],[0,5],[1,5],[1,3],[3,0],[2,-2],[0,-2],[0,-2],[0,-2],[1,-2],[1,0],[2,4],[1,1],[1,-2],[0,-5],[-2,-8],[-6,-14],[-2,-3],[0,-2],[1,-1],[1,1],[1,3],[1,1],[1,1],[1,0],[1,-1],[0,-1],[1,-3],[0,-1],[1,0],[1,-1],[1,-2],[4,-4],[2,-3],[-2,-2],[-4,-2],[-2,-2],[-1,-3],[-5,-4],[0,-2],[5,0],[1,0],[3,2],[0,1],[0,2],[0,1],[0,1],[4,0],[2,-2],[0,-4],[-2,-1],[0,-3],[0,-2],[-2,-1],[0,-3],[1,-2],[1,0],[8,-1],[1,1],[1,1],[1,1],[1,1],[3,-3],[0,-1],[1,-2],[0,-3],[0,-3],[1,-2],[1,1],[1,9],[2,2],[2,-2],[0,-3],[2,-3],[3,-2],[6,-1],[10,0],[-3,-3],[-2,-2],[-1,-4],[2,1],[3,2],[2,1],[7,-2],[0,-4],[1,-2],[1,-2],[3,0],[1,1],[0,3],[-1,4],[0,3],[1,2],[21,0],[1,0],[1,-1],[0,-2],[-2,-2],[2,-3],[1,-3],[2,-2],[2,-1],[13,1],[2,-1],[0,-4],[2,-1],[1,2],[2,2],[2,2],[9,0],[2,-1],[7,-11],[0,-1],[0,-1],[1,-1],[1,0],[1,1],[1,1],[1,0],[1,1],[1,-1],[1,-2],[-1,-4],[2,-4],[3,-2],[2,-1],[6,1],[3,0],[1,1],[0,1],[1,1],[1,1],[0,1],[8,0],[4,-3],[1,-5],[1,0],[1,-1],[2,-1],[0,-2],[-1,-3],[2,-1],[5,-1],[1,-1],[0,-3],[-1,-1],[-1,-1],[-1,0],[-1,0],[-1,-1],[-3,-1],[-2,0],[0,-1],[3,-1],[3,-2],[1,-3],[-2,-6],[-2,-3],[-4,-1],[-25,1],[-5,-4],[2,-2],[23,1],[4,0],[4,-2],[-2,-3],[-2,1],[-1,2],[-3,0],[-1,1],[-4,-3],[-1,-1],[2,-3],[-1,-3],[-5,-3],[-1,-3],[-6,-4],[-2,-1],[-3,0],[-9,2],[-3,1],[-1,0],[-1,0],[-2,-2],[-2,-1],[-1,0],[-1,1],[-1,2],[-1,1],[-2,-1],[-1,-1],[-32,-7],[0,-3],[-6,-5],[-3,-3],[5,0],[9,5],[5,0],[-2,-3],[-5,-3],[-2,-3],[-2,2],[-2,-1],[-4,-2],[1,-2],[1,-4],[0,-1],[-1,-1],[-1,-1],[-9,0],[-16,-5],[-11,-6],[-13,-12],[-5,-3],[-17,0],[-14,-6],[-15,-3],[-3,-2],[-3,-3],[-6,-1],[-3,-4],[-15,-4],[-3,0],[-10,3],[-5,-2],[-3,-1],[0,-2],[-1,0],[1,-1],[6,-3],[1,0],[0,-1],[0,-4],[1,-1],[8,4],[16,0],[20,9],[18,5],[16,10],[6,2],[2,0],[6,0],[2,1],[3,5],[9,6],[4,5],[1,-1],[1,0],[1,1],[2,-1],[2,1],[2,1],[2,2],[3,5],[2,2],[3,-2],[-2,-2],[-2,-3],[-2,-3],[-3,-8],[-1,-2],[-5,-4],[-2,-2],[-2,-3],[-1,-4],[-2,-4],[-4,-7],[-3,-3],[-3,-1],[-6,-1],[-17,4],[-21,-5],[0,-2],[14,-1],[-5,-9],[-1,-2],[-3,-2],[-1,-1],[-3,-2],[-8,0],[-1,-1],[-2,0],[-10,-4],[-6,-4],[-3,-5],[-5,-2],[-2,-1],[-7,-9],[-2,0],[-4,-4],[-6,-1],[-12,0],[-18,6],[-6,-1],[-2,-1],[-1,-2],[-1,-2],[2,-1],[4,0],[2,-1],[1,-4],[-2,-1],[-9,0],[-11,-3],[-5,-4],[-2,-1],[-3,1],[-5,4],[0,-1],[0,-1],[1,1],[2,-5],[3,-5],[4,-4],[3,-1],[0,-2],[-6,1],[-3,-3],[-1,-5],[0,-6],[-1,-4],[0,-4],[2,-5],[1,-4],[2,-4],[-1,-3],[-3,-1],[-1,-1],[-2,-3],[-1,-1],[0,1],[-1,1],[0,1],[0,3],[0,1],[-1,-1],[0,1],[-3,2],[-5,2],[-5,6],[-8,6],[-2,4],[-1,4],[-2,3],[-6,2],[-7,7],[-21,8],[-6,4],[-7,4],[-2,2],[-3,10],[-1,3],[-3,-1],[2,-4],[0,-4],[-1,-5],[-1,-4],[-5,-5],[-1,-3],[2,-3],[2,-1],[7,4],[4,0],[9,-5],[10,-1],[22,-14],[3,-5],[1,-1],[3,-1],[2,-2],[10,-11],[2,-2],[8,-3],[9,-9],[4,1],[1,0],[1,-2],[1,-5],[0,-5],[1,-4],[-3,2],[-2,2],[-3,1],[-3,0],[-8,-5],[-4,-5],[-6,-6],[-2,-1],[-2,0],[-2,-2],[-2,-3],[-1,-5],[-2,3],[-1,1],[0,1],[-1,-1],[-1,-2],[0,-2],[2,-2],[2,-1],[13,2],[7,2],[2,-1],[0,-3],[-3,-4],[-7,-6],[-9,-4],[-8,-1],[0,-2],[14,1],[6,2],[6,4],[2,2],[1,0],[1,0],[1,-2],[2,-4],[0,-1],[2,0],[1,2],[1,2],[0,3],[-5,3],[-2,2],[2,2],[1,1],[3,0],[5,3],[1,0],[2,2],[5,8],[2,3],[4,5],[0,1],[1,1],[2,5],[1,2],[1,-1],[1,-2],[1,-1],[0,3],[0,2],[-1,2],[0,2],[1,1],[2,0],[3,-2],[2,-1],[3,2],[0,1],[-3,2],[-5,1],[-3,3],[-1,5],[1,1],[3,1],[12,-4],[16,2],[6,3],[3,1],[2,2],[2,3],[5,7],[7,7],[1,2],[0,3],[1,1],[3,0],[2,1],[0,3],[0,2],[1,2],[2,0],[3,-5],[2,3],[1,1],[2,2],[1,1],[1,0],[1,-1],[1,-2],[1,-1],[1,-1],[3,0],[1,1],[1,1],[2,1],[3,0],[1,0],[1,2],[0,5],[0,2],[2,-1],[2,-1],[1,0],[0,3],[1,2],[0,2],[0,1],[1,0],[-2,1],[-3,1],[1,3],[-1,3],[-1,2],[-1,2],[2,4],[5,4],[3,2],[1,4],[10,9],[0,1],[1,2],[1,3],[1,3],[0,3],[2,1],[10,2],[11,4],[14,10],[23,9],[2,2],[2,2],[6,-1],[2,0],[7,7],[4,1],[35,-7],[2,1],[0,3],[-3,3],[-3,1],[-3,1],[-8,-1],[-3,1],[-7,6],[-13,3],[-3,-1],[-6,-2],[-2,-2],[-2,-3],[-1,-2],[-1,-1],[-1,-1],[-2,0],[-1,0],[-1,1],[-1,2],[-1,2],[-6,4],[-2,3],[1,5],[2,5],[2,4],[2,2],[3,2],[8,-2],[2,2],[1,5],[4,1],[10,-2],[1,-1],[3,2],[3,1],[4,0],[2,-2],[3,-3],[3,-2],[4,-1],[3,2],[2,-3],[3,0],[7,3],[-1,-2],[1,-3],[3,0],[5,1],[11,-3],[-1,-1],[-1,-4],[3,1],[3,2],[2,2],[2,-3],[7,6],[1,-2],[4,-3],[4,-6],[0,-2],[-1,-3],[2,0],[3,1],[0,-1],[-1,-6],[2,-10],[6,-16],[3,-6],[2,-2],[2,-2],[5,0],[3,-2],[2,-4],[-2,0],[-3,1],[-2,0],[-1,-2],[-1,-4],[0,-4],[1,-4],[2,-2],[1,-1],[5,-6],[2,-1],[2,0],[2,-1],[2,-3],[-4,0],[-1,1],[-2,1],[0,-1],[0,-1],[1,-2],[2,-2],[0,-3],[-5,-2],[0,-3],[1,-4],[2,-2],[0,-2],[-3,0],[-6,-5],[-6,-2],[-9,-9],[-14,-6],[-6,-5],[-4,-9],[6,3],[6,5],[0,-1],[0,-2],[0,-2],[-1,-1],[4,0],[2,1],[1,1],[1,0],[5,-4],[1,-1],[1,0],[0,-3],[-3,-7],[3,0],[2,-3],[0,-3],[-1,-6],[-2,-5],[-4,-6],[-2,-4],[2,1],[2,2],[1,0],[2,0],[3,-2],[1,1],[0,1],[-1,6],[3,5],[12,12],[1,1],[2,1],[0,3],[-1,2],[-3,-2],[4,5],[0,1],[0,6],[0,2],[1,1],[3,1],[3,2],[1,3],[-1,4],[4,5],[3,5],[3,5],[4,2],[3,-1],[4,-3],[3,1],[3,2],[2,0],[2,-2],[1,2],[1,1],[0,-1],[1,-1],[2,0],[1,1],[0,2],[-1,2],[4,1],[8,-8],[6,-1],[0,2],[-1,1],[-2,2],[-1,1],[0,2],[2,0],[0,4],[-1,1],[1,0],[2,-3],[2,1],[2,2],[2,2],[2,0],[2,1],[3,2],[5,-5],[1,-4],[-1,-3],[3,-3],[-2,-1],[-8,0],[-5,-3],[-4,-3],[-2,-3],[0,-1],[-1,-1],[-14,2],[2,-2],[9,-4],[4,-4],[18,-7],[4,1],[3,2],[3,2],[4,-2],[3,-3],[3,-3],[-3,-1],[-5,1],[-2,-1],[2,-2],[9,-1],[3,0],[5,3],[2,0],[1,-1],[1,-1],[1,-1],[2,0],[7,0],[2,-1],[7,-8],[0,-1],[-1,-1],[-1,-2],[0,-1],[0,-1],[1,0],[-1,-3],[-1,-1],[-1,-1],[-2,1],[0,1],[1,3],[-1,-1],[-2,1],[-1,0],[-1,1],[1,0],[2,2],[-4,2],[-14,4],[-5,0],[-3,-2],[3,-4],[21,-14],[3,-4],[-1,-7],[2,-1],[6,-4],[0,-1],[-4,-3],[-3,-3],[-1,-3],[3,-1],[2,2],[2,3],[3,3],[3,0],[2,-1],[5,-6],[0,-1],[-1,-2],[-2,-2],[5,0],[1,-1],[2,-1],[1,-2],[0,-1],[-10,-3],[-2,-2],[3,-1],[6,0],[2,-1],[1,-5],[-2,-3],[-2,-2],[-2,-3],[6,-3],[1,0],[2,2],[0,3],[1,1],[2,1],[1,-2],[-1,-3],[-3,-6],[-2,-3],[-1,0],[-2,2],[-2,0],[1,-2],[1,-3],[2,-2],[1,-1],[4,-1],[0,-1],[-1,-1],[3,-5],[-7,-10],[-2,-2],[-3,0],[-7,5],[-4,0],[0,-2],[3,0],[2,-2],[2,-3],[1,-3],[-1,-6],[-2,-4],[-3,-2],[-3,-1],[-3,1],[-5,3],[-3,1],[-12,-2],[0,-2],[2,1],[7,-2],[-1,-3],[-2,-3],[-1,-2],[-2,0],[1,-4],[2,-1],[1,2],[4,5],[1,2],[13,-3],[4,-2],[2,-6],[-1,0],[0,-1],[-1,0],[-1,-1],[3,-2],[3,-2],[3,-3],[-1,-6],[3,-5],[1,-3],[1,-6],[-1,-2],[-3,-1],[-4,1],[-1,-1],[-1,-2],[-1,-1],[-1,0],[-6,0],[-2,-1],[-2,-1],[-1,-3],[9,0],[-3,-3],[-3,-1],[-3,0],[-3,1],[-4,4],[-8,4],[-3,1],[0,-4],[3,-4],[3,-3],[6,-2],[2,-3],[2,-5],[-1,0],[0,-1],[0,-2],[3,0],[1,-2],[2,-3],[1,-3],[2,-2],[2,1],[5,1],[10,0],[2,-1],[3,-6],[2,-1],[-1,-3],[-1,-1],[-2,0],[-1,-1],[1,-1],[1,-1],[1,0],[-1,-2],[-3,-3],[-1,-2],[-2,4],[-3,1],[-5,-1],[-1,0],[-1,1],[-1,1],[-2,4],[-1,1],[-1,-1],[-1,1],[-10,4],[-6,1],[-3,-4],[8,0],[4,-1],[3,-3],[2,-3],[0,-1],[-6,-4],[-1,-2],[-1,-2],[4,-1],[2,-1],[1,-3],[-2,-2],[-2,1],[-5,4],[-2,0],[-4,0],[-11,-7],[-3,1],[-3,4],[-4,2],[-7,3],[-7,-1],[-2,2],[-1,2],[-2,1],[-3,-1],[2,-1],[3,-4],[3,-1],[8,0],[3,-2],[5,-4],[2,-2],[12,0],[1,0],[1,3],[3,0],[5,-2],[2,1],[2,0],[0,-2],[1,-1],[1,-2],[1,-2],[1,0],[10,-2],[10,3],[-1,-1],[-1,-5],[8,1],[4,-2],[-2,-10],[3,-1],[6,1],[3,-2],[0,-3],[-2,-6],[2,-3],[0,-3],[-1,-4],[-2,-1],[-1,0],[-2,2],[-2,2],[-1,1],[-2,2],[-7,-1],[1,-2],[3,-4],[1,-2],[-1,-3],[-1,0],[-7,3],[-8,5],[-7,2],[-22,15],[-3,1],[1,-4],[3,-3],[5,-4],[10,-2],[4,-4],[2,-1],[1,-1],[2,-4],[1,-1],[13,-4],[0,-2],[-2,-1],[-2,-2],[0,-2],[2,1],[1,1],[2,0],[1,-2],[-1,0],[0,-1],[3,0],[1,0],[1,-1],[0,-2],[1,-1],[1,1],[3,2],[2,1],[0,-1],[-1,-3],[-1,-2],[-2,-1],[-1,0],[-2,-1],[-2,-2],[-1,-4],[3,1],[5,3],[3,0],[2,-1],[8,-8],[-3,-1],[-2,0],[-5,1],[-3,-1],[-1,0],[-1,1],[-1,2],[-1,-1],[-1,-2],[-1,-2],[3,0],[4,-4],[3,-1],[-1,-6],[-1,-3],[1,-4],[2,-5],[-3,-5],[-4,0],[-3,1],[-3,-3],[1,-2],[3,-4],[-2,-5],[-3,-3],[-6,-4],[-3,0],[-5,3],[-4,1],[2,-4],[5,-6],[1,-3],[-1,-5],[-1,-3],[-2,-3],[-10,-6],[-2,0],[-1,-1],[-4,-6],[-4,-2],[-1,0],[-2,-4],[-3,-3],[-1,-2],[-1,-3],[-3,-4],[-2,-2],[-3,-1],[-9,0],[-1,-1],[-1,-4],[-2,-3],[-3,-3],[-1,-1],[-3,3],[-2,-1],[-1,-4],[0,-3],[-2,-2],[-11,-11],[-3,-2],[-7,0],[-2,-1],[-1,-2],[0,-7],[-1,-5],[-1,-2],[-3,-1],[-2,-2],[-1,-1],[-1,-2],[0,-1],[-1,-3],[-2,-2],[-1,-1],[1,-3],[-3,-6],[-4,-3],[-4,0],[-3,6],[-4,-7],[2,-3],[0,-3],[-1,-3],[-2,-2],[-2,-1],[-5,1],[-2,-2],[-1,2],[-2,0],[-1,-1],[-2,-1]],[[8662,4512],[0,-1],[-1,-1],[-2,-2],[-1,-1],[0,-1],[0,-1],[-1,0],[0,-2],[4,-5],[1,-3],[1,-4],[-5,4],[-5,4]],[[8937,1157],[3,-2],[0,-2],[-2,-2],[-4,-6],[-15,-14],[-6,-2],[-1,-1],[-2,2],[-1,2],[-4,5],[2,-7],[0,-2],[-2,-1],[-2,1],[-4,4],[-2,0],[0,-2],[1,-2],[0,-5],[2,-3],[-4,-1],[-2,0],[-2,1],[4,-7],[0,-3],[-3,-1],[-9,3],[-3,-1],[6,-4],[3,-3],[0,-5],[-2,-2],[-3,0],[-6,0],[-4,4],[-3,1],[-1,-2],[5,-3],[3,-4],[2,-4],[-3,1],[-5,5],[-3,1],[0,-2],[1,-1],[1,-1],[1,0],[0,-1],[-1,0],[-1,0],[-2,-2],[2,-1],[5,-1],[1,-1],[0,-3],[-1,-3],[0,-3],[0,-1],[6,-1],[2,-1],[-1,3],[-1,2],[-3,2],[3,1],[3,-5],[1,-6],[0,-7],[-2,-4],[-1,-1],[-1,0],[-2,0],[-2,0],[-3,-4],[-2,-1],[0,-1],[2,-1],[5,-3],[1,0],[3,2],[2,0],[-3,-9],[-7,-4],[-16,-1],[-3,1],[-2,0],[-1,-1],[-1,-3],[0,-1],[-1,0],[-2,3],[-6,2],[-2,2],[-6,6],[2,1],[1,1],[0,1],[-2,1],[-2,1],[-1,2],[-2,2],[0,3],[1,0],[4,0],[-2,4],[-10,-2],[-4,1],[2,1],[3,1],[1,0],[3,4],[4,2],[3,7],[3,2],[-2,4],[-2,1],[-2,-3],[-2,-3],[-2,-1],[-1,4],[1,5],[3,2],[-1,3],[-2,1],[-3,3],[-1,-2],[0,-1],[0,-1],[1,0],[0,-1],[-2,-1],[-3,2],[-4,2],[-6,4],[-1,1],[1,2],[2,3],[0,1],[2,1],[1,1],[0,2],[0,1],[0,1],[2,0],[0,1],[0,1],[0,2],[1,1],[7,5],[2,3],[-2,0],[-4,-3],[-4,-1],[-2,-5],[-2,-1],[-2,-1],[-2,-7],[-2,-2],[-1,1],[-2,4],[-4,4],[3,-9],[0,-2],[-3,-1],[-1,-1],[-2,-3],[-1,-1],[-4,0],[0,-2],[6,-1],[3,1],[2,5],[3,-2],[0,-2],[0,-4],[-4,-3],[-3,-1],[-3,-5],[-3,-1],[-3,1],[-6,5],[-4,1],[-3,3],[-1,0],[-1,1],[0,2],[-1,0],[-3,0],[-1,2],[-2,4],[-1,1],[-1,-1],[-3,-2],[-1,0],[-1,0],[-2,2],[-1,1],[-1,-1],[-1,-1],[-2,0],[-3,2],[-5,4],[-2,4],[-1,4],[-1,4],[-3,3],[-5,4],[-2,3],[-1,3],[1,1],[2,-2],[-3,8],[8,-4],[2,-1],[-5,7],[1,1],[1,1],[1,0],[0,1],[-3,1],[-9,0],[-3,3],[-1,-1],[-1,0],[0,-1],[-1,1],[-1,2],[-2,2],[-1,0],[-3,-3],[-3,-3],[-11,3],[-3,-1],[-2,1],[-1,1],[0,3],[0,4],[1,1],[1,-1],[1,7],[-1,9],[1,8],[4,2],[0,3],[-1,0],[2,1],[0,2],[-1,3],[-1,2],[0,5],[1,2],[0,4],[-7,-11],[-1,0],[-1,2],[-1,1],[-2,1],[-1,0],[-1,1],[-2,3],[-2,1],[-10,0],[-5,-2],[-2,0],[-2,4],[-2,10],[2,9],[3,9],[5,6],[4,6],[2,4],[1,5],[1,5],[1,5],[2,4],[8,12],[12,16],[7,11],[1,-3],[0,-2],[0,-1],[-1,-8],[0,-9],[0,-9],[2,-7],[0,-3],[-6,-11],[0,-1],[-1,-4],[0,-1],[-1,0],[-1,-1],[0,-3],[-1,1],[-1,1],[-1,0],[0,-3],[0,-2],[0,-1],[-1,-2],[2,-5],[2,-3],[2,0],[3,0],[1,2],[2,0],[2,-2],[-1,-2],[-1,-5],[3,-2],[2,-3],[2,-4],[2,-4],[2,-3],[1,-2],[0,-3],[-1,-3],[-1,-1],[-1,1],[-1,3],[0,-3],[1,-3],[1,-1],[1,0],[2,0],[0,2],[0,1],[1,2],[0,2],[1,0],[1,-1],[1,-2],[0,-1],[0,-2],[-1,-1],[0,-1],[0,-1],[0,-1],[1,0],[0,-2],[-1,-1],[-1,-1],[-3,-12],[-2,-2],[-3,-1],[-2,-2],[1,-6],[1,4],[2,1],[5,2],[2,2],[1,1],[1,0],[1,-2],[0,-2],[0,-3],[-1,-1],[-1,-1],[0,-1],[1,-2],[0,-2],[1,-1],[10,-3],[2,0],[2,3],[-2,5],[-2,4],[-1,4],[4,2],[0,2],[-2,1],[-1,0],[2,7],[4,2],[4,0],[12,-6],[1,-2],[1,0],[0,-1],[1,0],[1,0],[1,-1],[-1,-1],[0,-1],[0,-1],[0,-2],[0,-2],[0,-1],[2,0],[3,-6],[1,0],[1,4],[1,3],[-1,2],[-3,0],[3,1],[4,0],[5,-1],[3,-2],[1,-2],[1,-3],[0,-3],[1,-3],[3,-3],[2,-3],[0,-1],[1,2],[1,-5],[0,-1],[2,1],[1,1],[0,1],[0,1],[1,1],[0,1],[1,0],[13,-3],[15,1],[11,3],[1,-1],[1,-3],[1,-1],[2,4],[-1,1],[6,3],[20,-4],[0,1],[-10,3],[-3,2],[3,2],[7,1],[3,2],[3,-2],[2,1],[3,1],[18,0],[11,-1],[9,3],[9,0],[11,-5]]],"transform":{"scale":[0.008839778118922786,0.004145158209420939],"translate":[-141.0055486371089,41.669085592000116]}};
  Datamap.prototype.cheTopo = '__CHE__';
  Datamap.prototype.chlTopo = '__CHL__';
  Datamap.prototype.chnTopo = '__CHN__';
  Datamap.prototype.civTopo = '__CIV__';
  Datamap.prototype.clpTopo = '__CLP__';
  Datamap.prototype.cmrTopo = '__CMR__';
  Datamap.prototype.codTopo = '__COD__';
  Datamap.prototype.cogTopo = '__COG__';
  Datamap.prototype.cokTopo = '__COK__';
  Datamap.prototype.colTopo = '__COL__';
  Datamap.prototype.comTopo = '__COM__';
  Datamap.prototype.cpvTopo = '__CPV__';
  Datamap.prototype.criTopo = '__CRI__';
  Datamap.prototype.csiTopo = '__CSI__';
  Datamap.prototype.cubTopo = '__CUB__';
  Datamap.prototype.cuwTopo = '__CUW__';
  Datamap.prototype.cymTopo = '__CYM__';
  Datamap.prototype.cynTopo = '__CYN__';
  Datamap.prototype.cypTopo = '__CYP__';
  Datamap.prototype.czeTopo = '__CZE__';
  Datamap.prototype.deuTopo = '__DEU__';
  Datamap.prototype.djiTopo = '__DJI__';
  Datamap.prototype.dmaTopo = '__DMA__';
  Datamap.prototype.dnkTopo = '__DNK__';
  Datamap.prototype.domTopo = '__DOM__';
  Datamap.prototype.dzaTopo = '__DZA__';
  Datamap.prototype.ecuTopo = '__ECU__';
  Datamap.prototype.egyTopo = '__EGY__';
  Datamap.prototype.eriTopo = '__ERI__';
  Datamap.prototype.esbTopo = '__ESB__';
  Datamap.prototype.espTopo = '__ESP__';
  Datamap.prototype.estTopo = '__EST__';
  Datamap.prototype.ethTopo = '__ETH__';
  Datamap.prototype.finTopo = '__FIN__';
  Datamap.prototype.fjiTopo = '__FJI__';
  Datamap.prototype.flkTopo = '__FLK__';
  Datamap.prototype.fraTopo = '__FRA__';
  Datamap.prototype.froTopo = '__FRO__';
  Datamap.prototype.fsmTopo = '__FSM__';
  Datamap.prototype.gabTopo = '__GAB__';
  Datamap.prototype.psxTopo = '__PSX__';
  Datamap.prototype.gbrTopo = '__GBR__';
  Datamap.prototype.geoTopo = '__GEO__';
  Datamap.prototype.ggyTopo = '__GGY__';
  Datamap.prototype.ghaTopo = '__GHA__';
  Datamap.prototype.gibTopo = '__GIB__';
  Datamap.prototype.ginTopo = '__GIN__';
  Datamap.prototype.gmbTopo = '__GMB__';
  Datamap.prototype.gnbTopo = '__GNB__';
  Datamap.prototype.gnqTopo = '__GNQ__';
  Datamap.prototype.grcTopo = '__GRC__';
  Datamap.prototype.grdTopo = '__GRD__';
  Datamap.prototype.grlTopo = '__GRL__';
  Datamap.prototype.gtmTopo = '__GTM__';
  Datamap.prototype.gumTopo = '__GUM__';
  Datamap.prototype.guyTopo = '__GUY__';
  Datamap.prototype.hkgTopo = '__HKG__';
  Datamap.prototype.hmdTopo = '__HMD__';
  Datamap.prototype.hndTopo = '__HND__';
  Datamap.prototype.hrvTopo = '__HRV__';
  Datamap.prototype.htiTopo = '__HTI__';
  Datamap.prototype.hunTopo = '__HUN__';
  Datamap.prototype.idnTopo = '__IDN__';
  Datamap.prototype.imnTopo = '__IMN__';
  Datamap.prototype.indTopo = '__IND__';
  Datamap.prototype.ioaTopo = '__IOA__';
  Datamap.prototype.iotTopo = '__IOT__';
  Datamap.prototype.irlTopo = '__IRL__';
  Datamap.prototype.irnTopo = '__IRN__';
  Datamap.prototype.irqTopo = '__IRQ__';
  Datamap.prototype.islTopo = '__ISL__';
  Datamap.prototype.isrTopo = '__ISR__';
  Datamap.prototype.itaTopo = '__ITA__';
  Datamap.prototype.jamTopo = '__JAM__';
  Datamap.prototype.jeyTopo = '__JEY__';
  Datamap.prototype.jorTopo = '__JOR__';
  Datamap.prototype.jpnTopo = '__JPN__';
  Datamap.prototype.kabTopo = '__KAB__';
  Datamap.prototype.kasTopo = '__KAS__';
  Datamap.prototype.kazTopo = '__KAZ__';
  Datamap.prototype.kenTopo = '__KEN__';
  Datamap.prototype.kgzTopo = '__KGZ__';
  Datamap.prototype.khmTopo = '__KHM__';
  Datamap.prototype.kirTopo = '__KIR__';
  Datamap.prototype.knaTopo = '__KNA__';
  Datamap.prototype.korTopo = '__KOR__';
  Datamap.prototype.kosTopo = '__KOS__';
  Datamap.prototype.kwtTopo = '__KWT__';
  Datamap.prototype.laoTopo = '__LAO__';
  Datamap.prototype.lbnTopo = '__LBN__';
  Datamap.prototype.lbrTopo = '__LBR__';
  Datamap.prototype.lbyTopo = '__LBY__';
  Datamap.prototype.lcaTopo = '__LCA__';
  Datamap.prototype.lieTopo = '__LIE__';
  Datamap.prototype.lkaTopo = '__LKA__';
  Datamap.prototype.lsoTopo = '__LSO__';
  Datamap.prototype.ltuTopo = '__LTU__';
  Datamap.prototype.luxTopo = '__LUX__';
  Datamap.prototype.lvaTopo = '__LVA__';
  Datamap.prototype.macTopo = '__MAC__';
  Datamap.prototype.mafTopo = '__MAF__';
  Datamap.prototype.marTopo = '__MAR__';
  Datamap.prototype.mcoTopo = '__MCO__';
  Datamap.prototype.mdaTopo = '__MDA__';
  Datamap.prototype.mdgTopo = '__MDG__';
  Datamap.prototype.mdvTopo = '__MDV__';
  Datamap.prototype.mexTopo = '__MEX__';
  Datamap.prototype.mhlTopo = '__MHL__';
  Datamap.prototype.mkdTopo = '__MKD__';
  Datamap.prototype.mliTopo = '__MLI__';
  Datamap.prototype.mltTopo = '__MLT__';
  Datamap.prototype.mmrTopo = '__MMR__';
  Datamap.prototype.mneTopo = '__MNE__';
  Datamap.prototype.mngTopo = '__MNG__';
  Datamap.prototype.mnpTopo = '__MNP__';
  Datamap.prototype.mozTopo = '__MOZ__';
  Datamap.prototype.mrtTopo = '__MRT__';
  Datamap.prototype.msrTopo = '__MSR__';
  Datamap.prototype.musTopo = '__MUS__';
  Datamap.prototype.mwiTopo = '__MWI__';
  Datamap.prototype.mysTopo = '__MYS__';
  Datamap.prototype.namTopo = '__NAM__';
  Datamap.prototype.nclTopo = '__NCL__';
  Datamap.prototype.nerTopo = '__NER__';
  Datamap.prototype.nfkTopo = '__NFK__';
  Datamap.prototype.ngaTopo = '__NGA__';
  Datamap.prototype.nicTopo = '__NIC__';
  Datamap.prototype.niuTopo = '__NIU__';
  Datamap.prototype.nldTopo = '__NLD__';
  Datamap.prototype.nplTopo = '__NPL__';
  Datamap.prototype.nruTopo = '__NRU__';
  Datamap.prototype.nulTopo = '__NUL__';
  Datamap.prototype.nzlTopo = '__NZL__';
  Datamap.prototype.omnTopo = '__OMN__';
  Datamap.prototype.pakTopo = '__PAK__';
  Datamap.prototype.panTopo = '__PAN__';
  Datamap.prototype.pcnTopo = '__PCN__';
  Datamap.prototype.perTopo = '__PER__';
  Datamap.prototype.pgaTopo = '__PGA__';
  Datamap.prototype.phlTopo = '__PHL__';
  Datamap.prototype.plwTopo = '__PLW__';
  Datamap.prototype.pngTopo = '__PNG__';
  Datamap.prototype.polTopo = '__POL__';
  Datamap.prototype.priTopo = '__PRI__';
  Datamap.prototype.prkTopo = '__PRK__';
  Datamap.prototype.prtTopo = '__PRT__';
  Datamap.prototype.pryTopo = '__PRY__';
  Datamap.prototype.pyfTopo = '__PYF__';
  Datamap.prototype.qatTopo = '__QAT__';
  Datamap.prototype.rouTopo = '__ROU__';
  Datamap.prototype.rusTopo = '__RUS__';
  Datamap.prototype.rwaTopo = '__RWA__';
  Datamap.prototype.sahTopo = '__SAH__';
  Datamap.prototype.sauTopo = '__SAU__';
  Datamap.prototype.scrTopo = '__SCR__';
  Datamap.prototype.sdnTopo = '__SDN__';
  Datamap.prototype.sdsTopo = '__SDS__';
  Datamap.prototype.senTopo = '__SEN__';
  Datamap.prototype.serTopo = '__SER__';
  Datamap.prototype.sgpTopo = '__SGP__';
  Datamap.prototype.sgsTopo = '__SGS__';
  Datamap.prototype.shnTopo = '__SHN__';
  Datamap.prototype.slbTopo = '__SLB__';
  Datamap.prototype.sleTopo = '__SLE__';
  Datamap.prototype.slvTopo = '__SLV__';
  Datamap.prototype.smrTopo = '__SMR__';
  Datamap.prototype.solTopo = '__SOL__';
  Datamap.prototype.somTopo = '__SOM__';
  Datamap.prototype.spmTopo = '__SPM__';
  Datamap.prototype.srbTopo = '__SRB__';
  Datamap.prototype.stpTopo = '__STP__';
  Datamap.prototype.surTopo = '__SUR__';
  Datamap.prototype.svkTopo = '__SVK__';
  Datamap.prototype.svnTopo = '__SVN__';
  Datamap.prototype.sweTopo = '__SWE__';
  Datamap.prototype.swzTopo = '__SWZ__';
  Datamap.prototype.sxmTopo = '__SXM__';
  Datamap.prototype.sycTopo = '__SYC__';
  Datamap.prototype.syrTopo = '__SYR__';
  Datamap.prototype.tcaTopo = '__TCA__';
  Datamap.prototype.tcdTopo = '__TCD__';
  Datamap.prototype.tgoTopo = '__TGO__';
  Datamap.prototype.thaTopo = '__THA__';
  Datamap.prototype.tjkTopo = '__TJK__';
  Datamap.prototype.tkmTopo = '__TKM__';
  Datamap.prototype.tlsTopo = '__TLS__';
  Datamap.prototype.tonTopo = '__TON__';
  Datamap.prototype.ttoTopo = '__TTO__';
  Datamap.prototype.tunTopo = '__TUN__';
  Datamap.prototype.turTopo = '__TUR__';
  Datamap.prototype.tuvTopo = '__TUV__';
  Datamap.prototype.twnTopo = '__TWN__';
  Datamap.prototype.tzaTopo = '__TZA__';
  Datamap.prototype.ugaTopo = '__UGA__';
  Datamap.prototype.ukrTopo = '__UKR__';
  Datamap.prototype.umiTopo = '__UMI__';
  Datamap.prototype.uryTopo = '__URY__';
  Datamap.prototype.usaTopo = '__USA__';
  Datamap.prototype.usgTopo = '__USG__';
  Datamap.prototype.uzbTopo = '__UZB__';
  Datamap.prototype.vatTopo = '__VAT__';
  Datamap.prototype.vctTopo = '__VCT__';
  Datamap.prototype.venTopo = '__VEN__';
  Datamap.prototype.vgbTopo = '__VGB__';
  Datamap.prototype.virTopo = '__VIR__';
  Datamap.prototype.vnmTopo = '__VNM__';
  Datamap.prototype.vutTopo = '__VUT__';
  Datamap.prototype.wlfTopo = '__WLF__';
  Datamap.prototype.wsbTopo = '__WSB__';
  Datamap.prototype.wsmTopo = '__WSM__';
  Datamap.prototype.yemTopo = '__YEM__';
  Datamap.prototype.zafTopo = '__ZAF__';
  Datamap.prototype.zmbTopo = '__ZMB__';
  Datamap.prototype.zweTopo = '__ZWE__';

  /**************************************
                Utilities
  ***************************************/

  //convert lat/lng coords to X / Y coords
  Datamap.prototype.latLngToXY = function(lat, lng) {
     return this.projection([lng, lat]);
  };

  //add <g> layer to root SVG
  Datamap.prototype.addLayer = function( className, id, first ) {
    var layer;
    if ( first ) {
      layer = this.svg.insert('g', ':first-child')
    }
    else {
      layer = this.svg.append('g')
    }
    return layer.attr('id', id || '')
      .attr('class', className || '');
  };

  Datamap.prototype.updateChoropleth = function(data) {
    var svg = this.svg;
    for ( var subunit in data ) {
      if ( data.hasOwnProperty(subunit) ) {
        var color;
        var subunitData = data[subunit]
        if ( ! subunit ) {
          continue;
        }
        else if ( typeof subunitData === "string" ) {
          color = subunitData;
        }
        else if ( typeof subunitData.color === "string" ) {
          color = subunitData.color;
        }
        else {
          color = this.options.fills[ subunitData.fillKey ];
        }
        //if it's an object, overriding the previous data
        if ( subunitData === Object(subunitData) ) {
          this.options.data[subunit] = defaults(subunitData, this.options.data[subunit] || {});
          var geo = this.svg.select('.' + subunit).attr('data-info', JSON.stringify(this.options.data[subunit]));
        }
        svg
          .selectAll('.' + subunit)
          .transition()
            .style('fill', color);
      }
    }
  };

  Datamap.prototype.updatePopup = function (element, d, options) {
    var self = this;
    element.on('mousemove', null);
    element.on('mousemove', function() {
      var position = d3.mouse(self.options.element);
      d3.select(self.svg[0][0].parentNode).select('.datamaps-hoverover')
        .style('top', ( (position[1] + 30)) + "px")
        .html(function() {
          var data = JSON.parse(element.attr('data-info'));
          try {
            return options.popupTemplate(d, data);
          } catch (e) {
            return "";
          }
        })
        .style('left', ( position[0]) + "px");
    });

    d3.select(self.svg[0][0].parentNode).select('.datamaps-hoverover').style('display', 'block');
  };

  Datamap.prototype.addPlugin = function( name, pluginFn ) {
    var self = this;
    if ( typeof Datamap.prototype[name] === "undefined" ) {
      Datamap.prototype[name] = function(data, options, callback, createNewLayer) {
        var layer;
        if ( typeof createNewLayer === "undefined" ) {
          createNewLayer = false;
        }

        if ( typeof options === 'function' ) {
          callback = options;
          options = undefined;
        }

        options = defaults(options || {}, self.options[name + 'Config']);

        //add a single layer, reuse the old layer
        if ( !createNewLayer && this.options[name + 'Layer'] ) {
          layer = this.options[name + 'Layer'];
          options = options || this.options[name + 'Options'];
        }
        else {
          layer = this.addLayer(name);
          this.options[name + 'Layer'] = layer;
          this.options[name + 'Options'] = options;
        }
        pluginFn.apply(this, [layer, data, options]);
        if ( callback ) {
          callback(layer);
        }
      };
    }
  };

  // expose library
  if (typeof exports === 'object') {
    d3 = require('d3');
    topojson = require('topojson');
    module.exports = Datamap;
  }
  else if ( typeof define === "function" && define.amd ) {
    define( "datamaps", ["require", "d3", "topojson"], function(require) {
      d3 = require('d3');
      topojson = require('topojson');

      return Datamap;
    });
  }
  else {
    window.Datamap = window.Datamaps = Datamap;
  }

  if ( window.jQuery ) {
    window.jQuery.fn.datamaps = function(options, callback) {
      options = options || {};
      options.element = this[0];
      var datamap = new Datamap(options);
      if ( typeof callback === "function" ) {
        callback(datamap, options);
      }
      return this;
    };
  }
})();
