(function() {
  var svg;

  //save off default references
  var d3 = window.d3, topojson = window.topojson;

  var defaultOptions = {
    scope: 'world',
    responsive: false,
    aspectRatio: 0.5625,
    setProjection: setProjection,
    projection: 'equirectangular',
    dataType: 'json',
    data: {},
    done: function() {},
    fills: {
      defaultFill: '#ABDDA4'
    },
    filters: {},
    geographyConfig: {
        dataUrl: null,
        hideAntarctica: true,
        hideHawaiiAndAlaska : false,
        borderWidth: 1,
        borderColor: '#FDFDFD',
        popupTemplate: function(geography, data) {
          return '<div class="hoverinfo"><strong>' + geography.properties.name + '</strong></div>';
        },
        popupOnHover: true,
        highlightOnHover: true,
        highlightFillColor: '#FC8D59',
        highlightBorderColor: 'rgba(250, 15, 160, 0.2)',
        highlightBorderWidth: 2
    },
    projectionConfig: {
      rotation: [97, 0]
    },
    bubblesConfig: {
        borderWidth: 2,
        borderColor: '#FFFFFF',
        popupOnHover: true,
        radius: null,
        popupTemplate: function(geography, data) {
          return '<div class="hoverinfo"><strong>' + data.name + '</strong></div>';
        },
        fillOpacity: 0.75,
        animate: true,
        highlightOnHover: true,
        highlightFillColor: '#FC8D59',
        highlightBorderColor: 'rgba(250, 15, 160, 0.2)',
        highlightBorderWidth: 2,
        highlightFillOpacity: 0.85,
        exitDelay: 100,
        key: JSON.stringify
    },
    arcConfig: {
      strokeColor: '#DD1C77',
      strokeWidth: 1,
      arcSharpness: 1,
      animationSpeed: 600
    }
  };

  /*
    Getter for value. If not declared on datumValue, look up the chain into optionsValue
  */
  function val( datumValue, optionsValue, context ) {
    if ( typeof context === 'undefined' ) {
      context = optionsValue;
      optionsValues = undefined;
    }
    var value = typeof datumValue !== 'undefined' ? datumValue : optionsValue;

    if (typeof value === 'undefined') {
      return  null;
    }

    if ( typeof value === 'function' ) {
      var fnContext = [context];
      if ( context.geography ) {
        fnContext = [context.geography, context.data];
      }
      return value.apply(null, fnContext);
    }
    else {
      return value;
    }
  }

  function addContainer( element, height, width ) {
    this.svg = d3.select( element ).append('svg')
      .attr('width', width || element.offsetWidth)
      .attr('data-width', width || element.offsetWidth)
      .attr('class', 'datamap')
      .attr('height', height || element.offsetHeight)
      .style('overflow', 'hidden'); // IE10+ doesn't respect height/width when map is zoomed in

    if (this.options.responsive) {
      d3.select(this.options.element).style({'position': 'relative', 'padding-bottom': (this.options.aspectRatio*100) + '%'});
      d3.select(this.options.element).select('svg').style({'position': 'absolute', 'width': '100%', 'height': '100%'});
      d3.select(this.options.element).select('svg').select('g').selectAll('path').style('vector-effect', 'non-scaling-stroke');

    }

    return this.svg;
  }

  // setProjection takes the svg element and options
  function setProjection( element, options ) {
    var width = options.width || element.offsetWidth;
    var height = options.height || element.offsetHeight;
    var projection, path;
    var svg = this.svg;

    if ( options && typeof options.scope === 'undefined') {
      options.scope = 'world';
    }

    if ( options.scope === 'usa' ) {
      projection = d3.geo.albersUsa()
        .scale(width)
        .translate([width / 2, height / 2]);
    }
    else if ( options.scope === 'world' ) {
      projection = d3.geo[options.projection]()
        .scale((width + 1) / 2 / Math.PI)
        .translate([width / 2, height / (options.projection === "mercator" ? 1.45 : 1.8)]);
    }

    if ( options.projection === 'orthographic' ) {

      svg.append("defs").append("path")
        .datum({type: "Sphere"})
        .attr("id", "sphere")
        .attr("d", path);

      svg.append("use")
          .attr("class", "stroke")
          .attr("xlink:href", "#sphere");

      svg.append("use")
          .attr("class", "fill")
          .attr("xlink:href", "#sphere");
      projection.scale(250).clipAngle(90).rotate(options.projectionConfig.rotation)
    }

    path = d3.geo.path()
      .projection( projection );

    return {path: path, projection: projection};
  }

  function addStyleBlock() {
    if ( d3.select('.datamaps-style-block').empty() ) {
      d3.select('head').append('style').attr('class', 'datamaps-style-block')
      .html('.datamap path.datamaps-graticule { fill: none; stroke: #777; stroke-width: 0.5px; stroke-opacity: .5; pointer-events: none; } .datamap .labels {pointer-events: none;} .datamap path {stroke: #FFFFFF; stroke-width: 1px;} .datamaps-legend dt, .datamaps-legend dd { float: left; margin: 0 3px 0 0;} .datamaps-legend dd {width: 20px; margin-right: 6px; border-radius: 3px;} .datamaps-legend {padding-bottom: 20px; z-index: 1001; position: absolute; left: 4px; font-size: 12px; font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;} .datamaps-hoverover {display: none; font-family: "Helvetica Neue", Helvetica, Arial, sans-serif; } .hoverinfo {padding: 4px; border-radius: 1px; background-color: #FFF; box-shadow: 1px 1px 5px #CCC; font-size: 12px; border: 1px solid #CCC; } .hoverinfo hr {border:1px dotted #CCC; }');
    }
  }

  function drawSubunits( data ) {
    var fillData = this.options.fills,
        colorCodeData = this.options.data || {},
        geoConfig = this.options.geographyConfig;


    var subunits = this.svg.select('g.datamaps-subunits');
    if ( subunits.empty() ) {
      subunits = this.addLayer('datamaps-subunits', null, true);
    }

    var geoData = topojson.feature( data, data.objects[ this.options.scope ] ).features;
    if ( geoConfig.hideAntarctica ) {
      geoData = geoData.filter(function(feature) {
        return feature.id !== "ATA";
      });
    }

    if ( geoConfig.hideHawaiiAndAlaska ) {
      geoData = geoData.filter(function(feature) {
        return feature.id !== "HI" && feature.id !== 'AK';
      });
    }

    var geo = subunits.selectAll('path.datamaps-subunit').data( geoData );

    geo.enter()
      .append('path')
      .attr('d', this.path)
      .attr('class', function(d) {
        return 'datamaps-subunit ' + d.id;
      })
      .attr('data-info', function(d) {
        return JSON.stringify( colorCodeData[d.id]);
      })
      .style('fill', function(d) {
        //if fillKey - use that
        //otherwise check 'fill'
        //otherwise check 'defaultFill'
        var fillColor;

        var datum = colorCodeData[d.id];
        if ( datum && datum.fillKey ) {
          fillColor = fillData[ val(datum.fillKey, {data: colorCodeData[d.id], geography: d}) ];
        }

        if ( typeof fillColor === 'undefined' ) {
          fillColor = val(datum && datum.fillColor, fillData.defaultFill, {data: colorCodeData[d.id], geography: d});
        }

        return fillColor;
      })
      .style('stroke-width', geoConfig.borderWidth)
      .style('stroke', geoConfig.borderColor);
  }

  function handleGeographyConfig () {
    var hoverover;
    var svg = this.svg;
    var self = this;
    var options = this.options.geographyConfig;

    if ( options.highlightOnHover || options.popupOnHover ) {
      svg.selectAll('.datamaps-subunit')
        .on('mouseover', function(d) {
          var $this = d3.select(this);
          var datum = self.options.data[d.id] || {};
          if ( options.highlightOnHover ) {
            var previousAttributes = {
              'fill':  $this.style('fill'),
              'stroke': $this.style('stroke'),
              'stroke-width': $this.style('stroke-width'),
              'fill-opacity': $this.style('fill-opacity')
            };

            $this
              .style('fill', val(datum.highlightFillColor, options.highlightFillColor, datum))
              .style('stroke', val(datum.highlightBorderColor, options.highlightBorderColor, datum))
              .style('stroke-width', val(datum.highlightBorderWidth, options.highlightBorderWidth, datum))
              .style('fill-opacity', val(datum.highlightFillOpacity, options.highlightFillOpacity, datum))
              .attr('data-previousAttributes', JSON.stringify(previousAttributes));

            //as per discussion on https://github.com/markmarkoh/datamaps/issues/19
            if ( ! /((MSIE)|(Trident))/.test(navigator.userAgent) ) {
             moveToFront.call(this);
            }
          }

          if ( options.popupOnHover ) {
            self.updatePopup($this, d, options, svg);
          }
        })
        .on('mouseout', function() {
          var $this = d3.select(this);

          if (options.highlightOnHover) {
            //reapply previous attributes
            var previousAttributes = JSON.parse( $this.attr('data-previousAttributes') );
            for ( var attr in previousAttributes ) {
              $this.style(attr, previousAttributes[attr]);
            }
          }
          $this.on('mousemove', null);
          d3.selectAll('.datamaps-hoverover').style('display', 'none');
        });
    }

    function moveToFront() {
      this.parentNode.appendChild(this);
    }
  }

  //plugin to add a simple map legend
  function addLegend(layer, data, options) {
    data = data || {};
    if ( !this.options.fills ) {
      return;
    }

    var html = '<dl>';
    var label = '';
    if ( data.legendTitle ) {
      html = '<h2>' + data.legendTitle + '</h2>' + html;
    }
    for ( var fillKey in this.options.fills ) {

      if ( fillKey === 'defaultFill') {
        if (! data.defaultFillName ) {
          continue;
        }
        label = data.defaultFillName;
      } else {
        if (data.labels && data.labels[fillKey]) {
          label = data.labels[fillKey];
        } else {
          label= fillKey + ': ';
        }
      }
      html += '<dt>' + label + '</dt>';
      html += '<dd style="background-color:' +  this.options.fills[fillKey] + '">&nbsp;</dd>';
    }
    html += '</dl>';

    var hoverover = d3.select( this.options.element ).append('div')
      .attr('class', 'datamaps-legend')
      .html(html);
  }

    function addGraticule ( layer, options ) {
      var graticule = d3.geo.graticule();
      this.svg.insert("path", '.datamaps-subunits')
        .datum(graticule)
        .attr("class", "datamaps-graticule")
        .attr("d", this.path);
  }

  function handleArcs (layer, data, options) {
    var self = this,
        svg = this.svg;

    if ( !data || (data && !data.slice) ) {
      throw "Datamaps Error - arcs must be an array";
    }

    // For some reason arc options were put in an `options` object instead of the parent arc
    // I don't like this, so to match bubbles and other plugins I'm moving it
    // This is to keep backwards compatability
    for ( var i = 0; i < data.length; i++ ) {
      data[i] = defaults(data[i], data[i].options);
      delete data[i].options;
    }

    if ( typeof options === "undefined" ) {
      options = defaultOptions.arcConfig;
    }

    var arcs = layer.selectAll('path.datamaps-arc').data( data, JSON.stringify );

    var path = d3.geo.path()
        .projection(self.projection);

    arcs
      .enter()
        .append('svg:path')
        .attr('class', 'datamaps-arc')
        .style('stroke-linecap', 'round')
        .style('stroke', function(datum) {
          return val(datum.strokeColor, options.strokeColor, datum);
        })
        .style('fill', 'none')
        .style('stroke-width', function(datum) {
            return val(datum.strokeWidth, options.strokeWidth, datum);
        })
        .attr('d', function(datum) {
            var originXY = self.latLngToXY(val(datum.origin.latitude, datum), val(datum.origin.longitude, datum))
            var destXY = self.latLngToXY(val(datum.destination.latitude, datum), val(datum.destination.longitude, datum));
            var midXY = [ (originXY[0] + destXY[0]) / 2, (originXY[1] + destXY[1]) / 2];
            if (options.greatArc) {
                  // TODO: Move this to inside `if` clause when setting attr `d`
              var greatArc = d3.geo.greatArc()
                  .source(function(d) { return [val(d.origin.longitude, d), val(d.origin.latitude, d)]; })
                  .target(function(d) { return [val(d.destination.longitude, d), val(d.destination.latitude, d)]; });

              return path(greatArc(datum))
            }
            var sharpness = val(datum.arcSharpness, options.arcSharpness, datum);
            return "M" + originXY[0] + ',' + originXY[1] + "S" + (midXY[0] + (50 * sharpness)) + "," + (midXY[1] - (75 * sharpness)) + "," + destXY[0] + "," + destXY[1];
        })
        .transition()
          .delay(100)
          .style('fill', function(datum) {
            /*
              Thank you Jake Archibald, this is awesome.
              Source: http://jakearchibald.com/2013/animated-line-drawing-svg/
            */
            var length = this.getTotalLength();
            this.style.transition = this.style.WebkitTransition = 'none';
            this.style.strokeDasharray = length + ' ' + length;
            this.style.strokeDashoffset = length;
            this.getBoundingClientRect();
            this.style.transition = this.style.WebkitTransition = 'stroke-dashoffset ' + val(datum.animationSpeed, options.animationSpeed, datum) + 'ms ease-out';
            this.style.strokeDashoffset = '0';
            return 'none';
          })

    arcs.exit()
      .transition()
      .style('opacity', 0)
      .remove();
  }

  function handleLabels ( layer, options ) {
    var self = this;
    options = options || {};
    var labelStartCoodinates = this.projection([-67.707617, 42.722131]);
    this.svg.selectAll(".datamaps-subunit")
      .attr("data-foo", function(d) {
        var center = self.path.centroid(d);
        var xOffset = 7.5, yOffset = 5;

        if ( ["FL", "KY", "MI"].indexOf(d.id) > -1 ) xOffset = -2.5;
        if ( d.id === "NY" ) xOffset = -1;
        if ( d.id === "MI" ) yOffset = 18;
        if ( d.id === "LA" ) xOffset = 13;

        var x,y;

        x = center[0] - xOffset;
        y = center[1] + yOffset;

        var smallStateIndex = ["VT", "NH", "MA", "RI", "CT", "NJ", "DE", "MD", "DC"].indexOf(d.id);
        if ( smallStateIndex > -1) {
          var yStart = labelStartCoodinates[1];
          x = labelStartCoodinates[0];
          y = yStart + (smallStateIndex * (2+ (options.fontSize || 12)));
          layer.append("line")
            .attr("x1", x - 3)
            .attr("y1", y - 5)
            .attr("x2", center[0])
            .attr("y2", center[1])
            .style("stroke", options.labelColor || "#000")
            .style("stroke-width", options.lineWidth || 1)
        }

        layer.append("text")
          .attr("x", x)
          .attr("y", y)
          .style("font-size", (options.fontSize || 10) + 'px')
          .style("font-family", options.fontFamily || "Verdana")
          .style("fill", options.labelColor || "#000")
          .text( d.id );
        return "bar";
      });
  }


  function handleBubbles (layer, data, options ) {
    var self = this,
        fillData = this.options.fills,
        filterData = this.options.filters,
        svg = this.svg;

    if ( !data || (data && !data.slice) ) {
      throw "Datamaps Error - bubbles must be an array";
    }

    var bubbles = layer.selectAll('circle.datamaps-bubble').data( data, options.key );

    bubbles
      .enter()
        .append('svg:circle')
        .attr('class', 'datamaps-bubble')
        .attr('cx', function ( datum ) {
          var latLng;
          if ( datumHasCoords(datum) ) {
            latLng = self.latLngToXY(datum.latitude, datum.longitude);
          }
          else if ( datum.centered ) {
            latLng = self.path.centroid(svg.select('path.' + datum.centered).data()[0]);
          }
          if ( latLng ) return latLng[0];
        })
        .attr('cy', function ( datum ) {
          var latLng;
          if ( datumHasCoords(datum) ) {
            latLng = self.latLngToXY(datum.latitude, datum.longitude);
          }
          else if ( datum.centered ) {
            latLng = self.path.centroid(svg.select('path.' + datum.centered).data()[0]);
          }
          if ( latLng ) return latLng[1];
        })
        .attr('r', function(datum) {
          // if animation enabled start with radius 0, otherwise use full size.
          return options.animate ? 0 : val(datum.radius, options.radius, datum);
        })
        .attr('data-info', function(d) {
          return JSON.stringify(d);
        })
        .attr('filter', function (datum) {
          var filterKey = filterData[ val(datum.filterKey, options.filterKey, datum) ];

          if (filterKey) {
            return filterKey;
          }
        })
        .style('stroke', function ( datum ) {
          return val(datum.borderColor, options.borderColor, datum);
        })
        .style('stroke-width', function ( datum ) {
          return val(datum.borderWidth, options.borderWidth, datum);
        })
        .style('fill-opacity', function ( datum ) {
          return val(datum.fillOpacity, options.fillOpacity, datum);
        })
        .style('fill', function ( datum ) {
          var fillColor = fillData[ val(datum.fillKey, options.fillKey, datum) ];
          return fillColor || fillData.defaultFill;
        })
        .on('mouseover', function ( datum ) {
          var $this = d3.select(this);

          if (options.highlightOnHover) {
            //save all previous attributes for mouseout
            var previousAttributes = {
              'fill':  $this.style('fill'),
              'stroke': $this.style('stroke'),
              'stroke-width': $this.style('stroke-width'),
              'fill-opacity': $this.style('fill-opacity')
            };

            $this
              .style('fill', val(datum.highlightFillColor, options.highlightFillColor, datum))
              .style('stroke', val(datum.highlightBorderColor, options.highlightBorderColor, datum))
              .style('stroke-width', val(datum.highlightBorderWidth, options.highlightBorderWidth, datum))
              .style('fill-opacity', val(datum.highlightFillOpacity, options.highlightFillOpacity, datum))
              .attr('data-previousAttributes', JSON.stringify(previousAttributes));
          }

          if (options.popupOnHover) {
            self.updatePopup($this, datum, options, svg);
          }
        })
        .on('mouseout', function ( datum ) {
          var $this = d3.select(this);

          if (options.highlightOnHover) {
            //reapply previous attributes
            var previousAttributes = JSON.parse( $this.attr('data-previousAttributes') );
            for ( var attr in previousAttributes ) {
              $this.style(attr, previousAttributes[attr]);
            }
          }

          d3.selectAll('.datamaps-hoverover').style('display', 'none');
        })

    bubbles.transition()
      .duration(400)
      .attr('r', function ( datum ) {
        return val(datum.radius, options.radius, datum);
      });

    bubbles.exit()
      .transition()
        .delay(options.exitDelay)
        .attr("r", 0)
        .remove();

    function datumHasCoords (datum) {
      return typeof datum !== 'undefined' && typeof datum.latitude !== 'undefined' && typeof datum.longitude !== 'undefined';
    }
  }

  //stolen from underscore.js
  function defaults(obj) {
    Array.prototype.slice.call(arguments, 1).forEach(function(source) {
      if (source) {
        for (var prop in source) {
          if (obj[prop] == null) obj[prop] = source[prop];
        }
      }
    });
    return obj;
  }
  /**************************************
             Public Functions
  ***************************************/

  function Datamap( options ) {

    if ( typeof d3 === 'undefined' || typeof topojson === 'undefined' ) {
      throw new Error('Include d3.js (v3.0.3 or greater) and topojson on this page before creating a new map');
   }
    //set options for global use
    this.options = defaults(options, defaultOptions);
    this.options.geographyConfig = defaults(options.geographyConfig, defaultOptions.geographyConfig);
    this.options.projectionConfig = defaults(options.projectionConfig, defaultOptions.projectionConfig);
    this.options.bubblesConfig = defaults(options.bubblesConfig, defaultOptions.bubblesConfig);
    this.options.arcConfig = defaults(options.arcConfig, defaultOptions.arcConfig);

    //add the SVG container
    if ( d3.select( this.options.element ).select('svg').length > 0 ) {
      addContainer.call(this, this.options.element, this.options.height, this.options.width );
    }

    /* Add core plugins to this instance */
    this.addPlugin('bubbles', handleBubbles);
    this.addPlugin('legend', addLegend);
    this.addPlugin('arc', handleArcs);
    this.addPlugin('labels', handleLabels);
    this.addPlugin('graticule', addGraticule);

    //append style block with basic hoverover styles
    if ( ! this.options.disableDefaultStyles ) {
      addStyleBlock();
    }

    return this.draw();
  }

  // resize map
  Datamap.prototype.resize = function () {

    var self = this;
    var options = self.options;

    if (options.responsive) {
      var newsize = options.element.clientWidth,
          oldsize = d3.select( options.element).select('svg').attr('data-width');

      d3.select(options.element).select('svg').selectAll('g').attr('transform', 'scale(' + (newsize / oldsize) + ')');
    }
  }

  // actually draw the features(states & countries)
  Datamap.prototype.draw = function() {
    //save off in a closure
    var self = this;
    var options = self.options;

    //set projections and paths based on scope
    var pathAndProjection = options.setProjection.apply(self, [options.element, options] );

    this.path = pathAndProjection.path;
    this.projection = pathAndProjection.projection;

    //if custom URL for topojson data, retrieve it and render
    if ( options.geographyConfig.dataUrl ) {
      d3.json( options.geographyConfig.dataUrl, function(error, results) {
        if ( error ) throw new Error(error);
        self.customTopo = results;
        draw( results );
      });
    }
    else {
      draw( this[options.scope + 'Topo'] || options.geographyConfig.dataJson);
    }

    return this;

      function draw (data) {
        // if fetching remote data, draw the map first then call `updateChoropleth`
        if ( self.options.dataUrl ) {
          //allow for csv or json data types
          d3[self.options.dataType](self.options.dataUrl, function(data) {
            //in the case of csv, transform data to object
            if ( self.options.dataType === 'csv' && (data && data.slice) ) {
              var tmpData = {};
              for(var i = 0; i < data.length; i++) {
                tmpData[data[i].id] = data[i];
              }
              data = tmpData;
            }
            Datamaps.prototype.updateChoropleth.call(self, data);
          });
        }
        drawSubunits.call(self, data);
        handleGeographyConfig.call(self);

        if ( self.options.geographyConfig.popupOnHover || self.options.bubblesConfig.popupOnHover) {
          hoverover = d3.select( self.options.element ).append('div')
            .attr('class', 'datamaps-hoverover')
            .style('z-index', 10001)
            .style('position', 'absolute');
        }

        //fire off finished callback
        self.options.done(self);
      }
  };
  /**************************************
                TopoJSON
  ***************************************/
  Datamap.prototype.worldTopo = '__WORLD__';
  Datamap.prototype.abwTopo = '__ABW__';
  Datamap.prototype.afgTopo = '__AFG__';
  Datamap.prototype.agoTopo = '__AGO__';
  Datamap.prototype.aiaTopo = '__AIA__';
  Datamap.prototype.albTopo = '__ALB__';
  Datamap.prototype.aldTopo = '__ALD__';
  Datamap.prototype.andTopo = '__AND__';
  Datamap.prototype.areTopo = '__ARE__';
  Datamap.prototype.argTopo = '__ARG__';
  Datamap.prototype.armTopo = '__ARM__';
  Datamap.prototype.asmTopo = '__ASM__';
  Datamap.prototype.ataTopo = '__ATA__';
  Datamap.prototype.atcTopo = '__ATC__';
  Datamap.prototype.atfTopo = '__ATF__';
  Datamap.prototype.atgTopo = '__ATG__';
  Datamap.prototype.ausTopo = '__AUS__';
  Datamap.prototype.autTopo = '__AUT__';
  Datamap.prototype.azeTopo = '__AZE__';
  Datamap.prototype.bdiTopo = '__BDI__';
  Datamap.prototype.belTopo = '__BEL__';
  Datamap.prototype.benTopo = '__BEN__';
  Datamap.prototype.bfaTopo = '__BFA__';
  Datamap.prototype.bgdTopo = '__BGD__';
  Datamap.prototype.bgrTopo = '__BGR__';
  Datamap.prototype.bhrTopo = '__BHR__';
  Datamap.prototype.bhsTopo = '__BHS__';
  Datamap.prototype.bihTopo = '__BIH__';
  Datamap.prototype.bjnTopo = '__BJN__';
  Datamap.prototype.blmTopo = '__BLM__';
  Datamap.prototype.blrTopo = '__BLR__';
  Datamap.prototype.blzTopo = '__BLZ__';
  Datamap.prototype.bmuTopo = '__BMU__';
  Datamap.prototype.bolTopo = '__BOL__';
  Datamap.prototype.braTopo = '__BRA__';
  Datamap.prototype.brbTopo = '__BRB__';
  Datamap.prototype.brnTopo = '__BRN__';
  Datamap.prototype.btnTopo = '__BTN__';
  Datamap.prototype.norTopo = '__NOR__';
  Datamap.prototype.bwaTopo = '__BWA__';
  Datamap.prototype.cafTopo = '__CAF__';
  Datamap.prototype.canTopo = '__CAN__';
  Datamap.prototype.cheTopo = '__CHE__';
  Datamap.prototype.chlTopo = '__CHL__';
  Datamap.prototype.chnTopo = {"type":"Topology","objects":{"chn":{"type":"GeometryCollection","geometries":[{"type":"Polygon","properties":{"name":"Gansu"},"id":"CN.GS","arcs":[[0,1,2,3,4,5,6]]},{"type":"Polygon","properties":{"name":"Qinghai"},"id":"CN.QH","arcs":[[7,8,9,-4]]},{"type":"MultiPolygon","properties":{"name":"Guangxi"},"id":"CN.GX","arcs":[[[10]],[[11]],[[12]],[[13,14,15,16,17]]]},{"type":"Polygon","properties":{"name":"Guizhou"},"id":"CN.GZ","arcs":[[18,-17,19,20,21]]},{"type":"Polygon","properties":{"name":"Chongqing"},"id":"CN.CQ","arcs":[[22,23,-22,24,25]]},{"type":"Polygon","properties":{"name":"Beijing"},"id":"CN.BJ","arcs":[[26,27,28,29]]},{"type":"MultiPolygon","properties":{"name":"Fujian"},"id":"CN.FJ","arcs":[[[30]],[[31]],[[32]],[[33]],[[34]],[[35]],[[36]],[[37]],[[38]],[[39]],[[40]],[[41]],[[42]],[[43]],[[44]],[[45]],[[46]],[[47]],[[48,49,50,51]]]},{"type":"Polygon","properties":{"name":"Anhui"},"id":"CN.AH","arcs":[[52,53,54,55,56,57]]},{"type":"MultiPolygon","properties":{"name":"Guangdong"},"id":"CN.GD","arcs":[[[58]],[[59]],[[60]],[[61]],[[62]],[[63]],[[64]],[[65]],[[66]],[[67]],[[68]],[[69]],[[70]],[[71]],[[72]],[[73]],[[74]],[[75]],[[76]],[[77]],[[78]],[[79]],[[80]],[[81,-50,82,-14,83]]]},{"type":"Polygon","properties":{"name":"Xizang"},"id":"CN.XZ","arcs":[[-9,84,85,86,87]]},{"type":"Polygon","properties":{"name":"Xinjiang"},"id":"CN.XJ","arcs":[[-5,-10,-88,88]]},{"type":"Polygon","properties":{"name":"Hainan"},"id":"CN.HA","arcs":[[89]]},{"type":"Polygon","properties":{"name":"Ningxia"},"id":"CN.NX","arcs":[[90,-1,91]]},{"type":"Polygon","properties":{"name":"Shaanxi"},"id":"CN.SA","arcs":[[92,93,94,-26,95,-2,-91,96]]},{"type":"Polygon","properties":{"name":"Shanxi"},"id":"CN.SX","arcs":[[97,-93,98,99]]},{"type":"Polygon","properties":{"name":"Hubei"},"id":"CN.HU","arcs":[[-55,100,101,-23,-95,102]]},{"type":"Polygon","properties":{"name":"Hunan"},"id":"CN.HN","arcs":[[103,-84,-18,-19,-24,-102]]},{"type":"Polygon","properties":{"name":"Sichuan"},"id":"CN.SC","arcs":[[-96,-25,-21,104,-85,-8,-3]]},{"type":"Polygon","properties":{"name":"Yunnan"},"id":"CN.YN","arcs":[[-20,-16,105,-86,-105]]},{"type":"MultiPolygon","properties":{"name":"Hebei"},"id":"CN.HB","arcs":[[[106]],[[-28,107]],[[108,109,110,-30,111,112,113,114,-100,115]]]},{"type":"Polygon","properties":{"name":"Henan"},"id":"CN.HE","arcs":[[116,-56,-103,-94,-98,-115]]},{"type":"MultiPolygon","properties":{"name":"Liaoning"},"id":"CN.LN","arcs":[[[117]],[[118]],[[119]],[[120]],[[121]],[[122]],[[123]],[[124]],[[125]],[[126]],[[127]],[[128,-109,129,130]]]},{"type":"MultiPolygon","properties":{"name":"Shandong"},"id":"CN.SD","arcs":[[[131]],[[132]],[[133]],[[134]],[[135]],[[136]],[[137,-57,-117,-114,138]],[[139]],[[140]]]},{"type":"MultiPolygon","properties":{"name":"Tianjin"},"id":"CN.TJ","arcs":[[[141]],[[142]],[[143]],[[144,-112,-29,-108,-27,-111]]]},{"type":"Polygon","properties":{"name":"Jiangxi"},"id":"CN.JX","arcs":[[145,-51,-82,-104,-101,-54]]},{"type":"MultiPolygon","properties":{"name":"Jiangsu"},"id":"CN.JS","arcs":[[[146]],[[147]],[[148]],[[149,150,-58,-138,151]]]},{"type":"MultiPolygon","properties":{"name":"Shanghai"},"id":"CN.SH","arcs":[[[152]],[[153]],[[154,155,-150]],[[156]]]},{"type":"MultiPolygon","properties":{"name":"Zhejiang"},"id":"CN.ZJ","arcs":[[[157]],[[158]],[[159]],[[160]],[[161]],[[162]],[[163]],[[164]],[[165]],[[166]],[[167]],[[168]],[[169]],[[170]],[[171]],[[172]],[[173]],[[174]],[[175]],[[176]],[[177]],[[178]],[[179]],[[180]],[[181]],[[182]],[[183]],[[184]],[[185]],[[186]],[[187]],[[188]],[[189]],[[190]],[[191]],[[192]],[[193]],[[-156,194,-52,-146,-53,-151]]]},{"type":"Polygon","properties":{"name":"Jilin"},"id":"CN.JL","arcs":[[195,-131,196,197]]},{"type":"Polygon","properties":{"name":"Inner Mongol"},"id":"CN.NM","arcs":[[198,-197,-130,-116,-99,-97,-92,-7,199]]},{"type":"Polygon","properties":{"name":"Heilongjiang"},"id":"CN.HL","arcs":[[-198,-199,200]]},{"type":"MultiPolygon","properties":{"name":"Paracel Islands"},"id":"CN","arcs":[[[201]],[[202]],[[203]],[[204]],[[205]],[[206]],[[207]],[[208]]]}]}},"arcs":[[[5027,5721],[9,0],[9,11],[2,1],[2,-1],[7,-4],[26,-4],[4,-1],[1,-2],[-3,-2],[-3,-2],[-1,-3],[1,-3],[4,-3],[1,-1],[1,-2],[0,-3],[0,-1],[-1,-1],[-2,-2],[-2,-2],[-1,0],[-1,-1],[-1,-3],[0,-1],[-1,-1],[-1,-1],[-1,0],[-2,0],[-1,0],[-1,-1],[0,-1],[-1,-2],[-1,-1],[0,-1],[0,-2],[3,-4],[2,-1],[1,-2],[5,-2],[1,0],[2,0],[1,0],[1,1],[1,0],[1,2],[4,5],[1,1],[1,1],[3,-1],[3,-1],[9,-7],[0,-1],[1,-1],[0,-2],[1,-4],[0,-2],[1,-1],[2,-3],[2,-5],[1,-5],[1,-3],[1,-3],[0,-1],[1,-1],[7,-9],[4,-4],[4,-2],[12,-4],[10,-8],[5,-6],[1,-3],[3,-10],[16,-27],[3,-7],[0,-4],[-14,-11],[-1,-1],[-1,-2],[-1,-2],[0,-4],[0,-3],[8,-38],[7,-15],[4,-15],[2,-3],[0,-2],[3,-3],[5,-6],[2,-5],[2,-6],[1,-4],[3,-19],[1,-4],[0,-3],[-1,-5],[0,-2],[0,-1],[-2,-2],[0,-3],[0,-4],[1,-8],[0,-4],[0,-3],[-1,-2],[-1,-1],[-1,-2],[-11,-8],[-1,-1],[-1,-1],[-2,-5],[0,-2],[-2,-8],[0,-5],[0,-7],[0,-4],[1,-3],[0,-1],[5,-12],[2,-5],[0,-1],[1,-1],[1,-1],[1,-1],[1,-2],[2,-7],[0,-2],[1,-2],[0,-1],[1,-2],[1,-1],[1,-3],[0,-1],[1,-1],[1,0],[1,0],[1,0],[1,-1],[2,-1],[15,-13],[3,0],[4,-1],[2,2],[4,2],[4,2],[7,0],[3,-2],[1,-3],[-2,-2],[-2,-1],[-2,-1],[-1,-1],[-1,-2],[1,-3],[1,-4],[1,-4],[4,-7],[10,-16],[3,-6],[4,-7],[2,-2],[3,0],[2,2],[3,4],[2,2],[3,2],[2,0],[21,-10],[2,-2],[1,-2],[-1,-1],[-2,-1],[-25,-4],[-1,-1],[-2,-1],[-1,-3],[1,-4],[1,-2],[2,-1],[2,0],[2,1],[8,5],[3,1],[3,0],[5,-2],[10,1],[11,-4],[3,-1],[2,-3],[4,-8],[3,-6],[3,-4],[2,-3],[3,-7],[2,-4],[4,-5],[4,0],[3,1],[11,14],[2,3],[1,2],[1,3],[2,8],[1,8],[0,8],[1,8],[0,1],[0,2],[-1,1],[0,1],[-1,2],[-1,0],[-2,1],[-4,1],[-1,1],[-1,1],[-2,5],[0,3],[-1,1],[0,2],[1,1],[0,1],[2,0],[11,0],[1,0],[1,0],[0,2],[0,1],[0,1],[-1,1],[0,1],[-5,3],[-1,2],[-2,4],[-1,2],[-1,11],[-1,4],[-1,3],[0,2],[0,1],[0,2],[1,1],[0,1],[1,1],[1,1],[2,1],[2,1],[8,1],[6,0],[6,-1],[2,-2],[1,0],[4,-2],[7,-1],[2,-1],[4,-6],[2,-2],[3,1],[1,1],[1,2],[0,2],[1,2],[2,2],[6,4],[4,1],[2,0],[1,1],[1,1],[1,3],[1,3],[5,3],[1,2],[0,2],[0,1],[-1,1],[0,1],[-4,5],[0,2],[-1,1],[0,1],[0,3],[-1,1],[0,2],[-1,1],[-3,4],[0,1],[0,1],[0,1],[1,0],[0,1],[1,1],[1,1],[2,1],[6,1],[2,1],[2,2],[0,2],[0,1],[-1,1],[-2,2],[-1,2],[0,1],[0,1],[1,1],[2,2],[1,2],[1,2],[-1,1],[-1,2],[0,2],[0,3],[0,3],[2,8],[0,4],[0,2],[-1,2],[-2,6],[-2,2],[0,1],[-2,2],[0,1],[-2,5],[-1,2],[-1,1],[-4,4],[-1,1],[-2,6],[-1,1],[0,1],[-1,1],[-1,0],[-6,-2],[-2,-1],[-1,1],[-2,0],[-8,5],[-4,3],[-8,9],[-2,2],[-1,0],[-1,0],[-1,0],[-3,-1],[-4,-3],[-1,0],[-1,-1],[-2,0],[-2,1],[-2,1],[-1,1],[-1,1],[-1,2],[-2,5],[0,3],[0,2],[1,1],[0,1],[3,2],[0,1],[1,1],[0,1],[0,1],[0,2],[0,2],[-1,19],[-1,3],[0,2],[-1,1],[-1,2],[-2,2],[-2,2],[-2,2],[-1,1],[-1,3],[0,1],[1,1],[1,1],[4,2],[3,3],[1,1],[0,1],[1,5],[-1,11],[-1,12],[0,5],[1,6],[0,1],[1,2],[0,1],[1,1],[7,5],[0,1],[1,0],[2,0],[1,0],[3,-2],[1,0],[1,1],[2,1],[1,2],[0,1],[5,15],[0,3],[1,2],[-1,1],[0,2],[0,1],[-6,15],[0,1],[-1,1],[-4,4],[-1,2],[0,1],[0,2],[0,1],[1,1],[3,2],[0,1],[1,2],[1,1],[3,10],[1,2],[0,2],[0,2],[0,1],[0,1],[-2,4],[0,1],[-1,2],[0,4],[1,3],[2,7],[0,2],[0,2],[0,11],[1,2],[0,2],[1,2],[0,1],[1,2],[1,0],[1,1],[4,1],[9,0],[4,-2],[1,-1],[2,-2],[9,-12],[7,-6],[5,-3],[2,0],[9,0],[3,0],[3,0],[5,3],[1,0],[5,1],[6,0],[10,-2],[9,-5],[2,-2],[5,-6]],[[5506,5633],[-1,-6],[0,-5],[1,-1],[0,-1],[1,-3],[0,-4],[0,-8],[0,-4],[-2,-8],[1,-1],[2,-1],[24,-6],[7,1],[2,0],[1,0],[2,-1],[2,-3],[1,-2],[-1,-1],[-1,-2],[0,-1],[1,-1],[3,-2],[11,-4],[8,-1],[5,1],[2,-1],[1,-1],[0,-2],[2,-1],[2,-3],[3,-1],[13,-5],[1,0],[7,-4],[1,-2],[2,-2],[0,-1],[4,-10],[1,-3],[2,-3],[2,-1],[1,-1],[5,-1],[4,-1],[1,-1],[1,-1],[0,-2],[-1,-4],[0,-1],[0,-2],[0,-2],[1,-2],[2,-2],[1,-1],[1,-1],[5,-1],[2,0],[7,-4],[2,0],[1,-1],[1,0],[1,1],[1,1],[2,4],[1,1],[1,1],[1,1],[1,-1],[1,-1],[1,-2],[1,-2],[2,-2],[1,0],[1,-1],[6,-1],[6,-3],[2,-1],[2,-1],[1,-2],[2,-1],[1,-3],[0,-1],[2,-5],[0,-1],[1,-2],[1,-2],[3,-3],[1,-1],[2,-1],[2,-1],[3,0],[2,0],[4,0],[2,0],[2,0],[1,-1],[2,-1],[1,-2],[1,-1],[2,0],[5,-2],[1,-1],[1,-1],[1,-1],[0,-1],[1,-1],[3,-5],[5,-4],[2,-3],[2,-2],[0,-2],[0,-1],[0,-2],[0,-2],[-2,-5],[-1,-3],[-1,-2],[-3,-4],[-2,-4],[0,-1],[-1,-2],[0,-2],[0,-1],[0,-2],[1,-2],[1,-3],[3,-4],[1,-4],[2,-3],[0,-1],[0,-3],[1,-6],[0,-3],[0,-10],[-1,-6],[-1,-5],[-2,-11],[0,-1],[-1,-2],[-1,-2],[-1,-2],[-2,-1],[-3,-1],[-2,-1],[-2,-1],[-1,-1],[-5,-9],[0,-1],[-2,-1],[-3,-2],[-3,-3],[-2,-1],[-1,-2],[0,-1],[-1,-1],[0,-1],[0,-1],[0,-2],[1,-2],[3,-8],[1,-5],[0,-3],[0,-2],[0,-4],[-2,-19],[0,-2],[0,-2],[1,-3],[0,-3],[5,-14],[9,-17],[1,-19],[1,-14],[1,-8],[-1,-2],[0,-2],[-3,-3],[-6,-12],[-5,-8],[-1,-2],[-2,-1],[-1,-1],[-2,-1],[-7,-1],[-15,3],[-5,-1],[-2,0],[-6,-3],[-1,0],[-2,0],[-2,2],[-3,4],[-2,1],[-2,1],[-2,0],[-6,-1],[-3,-1],[-1,-1],[-3,-1],[-6,-5],[-4,-3],[-5,-1],[-1,-1],[-1,0],[-1,-2],[-1,-1],[-3,-2],[-1,0],[-1,0],[0,1],[-1,1],[-1,1],[0,1],[0,1],[-1,4],[-1,1],[-3,2],[-10,2],[-12,9],[-7,3],[-1,-1],[-1,-2],[0,-1],[0,-1],[0,-1],[1,-3],[0,-2],[0,-2],[0,-3],[-1,-1],[0,-1],[-1,-1],[-1,-1],[-1,-1],[-6,-2],[-1,0],[0,-2],[0,-1],[2,-4],[3,-8],[9,-26],[6,-8],[5,-6],[1,-2],[1,-2],[1,-1],[0,-1],[0,-1],[0,-3],[-2,-3],[-6,-2],[-7,-2],[-3,-2],[-1,0],[-3,-1],[-14,2],[-9,2],[-2,-1],[-1,-1],[-1,-1],[-3,-4],[-1,-2],[-1,-1],[-1,0],[-2,0],[-3,0],[-8,4],[-4,0],[-3,0],[-1,-1],[-2,-1],[-2,0],[-1,0],[-3,1],[-1,1],[-2,0],[-2,0],[-1,-1],[-1,-1],[-2,-5],[-1,-1],[-1,-1],[-3,-1],[-1,0],[-1,-1],[-1,-2],[-1,-1],[-1,1],[-2,1],[-1,1],[-2,4],[-1,3],[-1,1],[-2,3],[-1,2],[-7,6],[-2,2],[-4,9],[-2,4],[-1,2],[-2,2],[-2,1],[-12,3],[-1,0],[-1,1],[-1,1],[-1,3],[-2,2],[-1,1],[-2,0],[-15,-1],[-5,2],[-1,0],[-24,-5],[-7,3],[-1,-1],[-2,-1],[-2,-3],[-1,-2],[-2,-3],[-7,-5],[0,-3],[0,-1],[1,-4],[1,-2],[0,-1],[0,-2],[-1,-7],[0,-2],[0,-2],[0,-2],[1,-1],[8,-17],[0,-2],[0,-2],[0,-3],[-1,-4],[-1,-3],[-1,-1],[-1,-1],[-1,-2],[-1,-2],[-1,-3],[-3,-9],[0,-2],[-1,-1],[-1,-1],[-2,-2],[-6,-4],[-1,-1],[-1,-2],[-2,-3],[-2,-7],[-2,-4],[0,-1],[-1,-1],[-1,-2],[-3,-3],[-2,-1],[-1,-2],[-1,-2],[-1,-2],[0,-1],[0,-3],[1,-4],[4,-7],[2,-2],[1,-1],[13,0],[5,1],[1,0],[2,-1],[2,-2],[2,-3],[2,-3],[3,-1],[3,-1],[4,-2],[3,-4],[1,-5],[1,-5],[3,-4],[3,-3],[2,-3],[0,-1],[0,-3],[3,-20],[-1,-3],[0,-3],[-1,-3],[-2,-1],[-2,0],[-5,2],[-4,1],[-5,3],[-3,1],[-3,0],[-2,-1],[-4,-3],[3,-2],[3,-1],[2,-1],[1,-2],[6,-22],[0,-3],[0,-3],[-1,-3],[-2,-1],[-1,0],[-4,1],[-2,-1],[-2,-4],[-1,-10],[-2,-7],[-7,-19],[-4,-8],[0,-1],[-1,-1],[0,-2],[0,-4],[0,-3],[2,-5],[2,-1],[2,-1],[1,0],[1,-1],[1,0],[1,-2],[1,-2],[0,-3],[-1,-2],[-3,-2],[-1,-1],[-1,-2],[0,-3],[1,-5],[1,-5],[0,-4],[0,-4],[0,-3],[1,-3],[2,-4],[1,-1],[3,-2],[2,-3],[3,-4],[1,-2],[1,-3],[1,-2],[0,-3],[1,-4],[-1,-3],[-2,-9],[-8,-5],[-4,1],[-3,4],[-1,2],[-1,5],[-1,3],[-1,2],[-2,1],[-2,1],[-2,1],[-2,-1],[-6,-2],[-2,-1],[-2,1],[-2,1],[-2,0],[-3,-2],[-5,-4],[-12,-7],[-3,0],[-2,1],[-4,2],[-1,2],[-1,1],[-1,4],[-1,2],[-1,2],[-2,0],[-11,0],[-1,-1],[-1,0],[-1,-3],[0,-2],[-1,-4],[-1,-2],[-2,-3],[-2,-2],[-4,-2],[-4,-2],[-5,-5],[-1,-1],[-2,-1],[-2,-1],[-2,-2],[-3,-17],[-2,-4],[-2,-3],[-3,-1],[-5,0],[-1,-1],[-1,-1],[-1,-2],[-1,-4],[0,-2],[0,-4],[1,-3],[0,-1],[0,-1],[1,-2],[0,-3],[2,-2],[1,-3],[16,-14],[3,0],[4,0],[3,-1],[2,-1],[1,-1],[0,-2],[1,-1],[-1,-2],[-3,-8],[0,-3],[-1,-3],[0,-3],[2,-24],[0,-7],[0,-2],[-1,-2],[-1,-4],[0,-2],[-1,-2],[-1,-2],[-1,-2],[-1,-1],[-2,-1],[-2,1],[-4,0],[-4,-2],[-3,-3],[-5,-8],[-3,-2],[-6,-4],[-5,-2],[-4,0],[-23,6]],[[5214,4533],[-1,3],[-2,0],[-3,0],[-3,-1],[-2,-1],[-3,-3],[-2,-3],[-3,-4],[-1,-4],[0,-3],[0,-3],[1,-4],[8,-13],[1,-2],[0,-1],[0,-4],[-2,-2],[-1,-2],[-5,-2],[-3,-3],[-4,-4],[-6,-8],[-3,-4],[-4,-3],[-1,0],[-3,1],[-4,4],[-1,1],[-1,0],[-2,0],[-2,-2],[-3,-7],[-3,-6],[-2,-2],[-3,-1],[-2,0],[-2,1],[-2,2],[-1,2],[-1,4],[-1,1],[-2,1],[-2,0],[-16,-6],[-3,-1],[-3,0],[-2,2],[-8,6],[-2,1],[-2,0],[-9,0],[-9,1],[-10,3],[-4,1],[-5,3],[-5,4],[-12,13],[-9,14],[-2,2],[-2,3],[-10,2],[-5,3],[-2,2],[-2,3],[-1,2],[-1,3],[0,4],[2,11],[3,4],[2,4],[3,1],[7,0],[1,2],[1,2],[3,8],[1,5],[0,3],[0,3],[-1,3],[-1,2],[-12,20],[-2,5],[-4,9],[-1,5],[-1,7],[0,9],[0,3],[1,2],[1,1],[1,1],[1,1],[2,1],[2,0],[6,0],[1,0],[2,2],[0,2],[-1,1],[-1,2],[-1,2],[-2,1],[-6,2],[-2,1],[-2,2],[-2,3],[-4,8],[-1,2],[-4,3],[-9,18],[-1,5],[0,3],[0,3],[1,3],[0,5],[0,7],[-1,5],[-2,4],[-2,4],[-2,4],[-2,2],[-4,2],[-29,4],[-18,-3],[-6,0],[-4,1],[-8,4],[-2,1],[-2,3],[-2,1],[-2,1],[-4,0],[-11,-3],[-4,1],[-1,2],[0,3],[0,4],[4,11],[1,3],[1,4],[-1,2],[-2,0],[-4,-1],[-13,-11],[-4,-2],[-4,-1],[-6,-1],[-9,1],[-4,1],[-7,5],[-4,4],[-2,2],[-1,2],[0,3],[0,2],[1,7],[0,4],[0,3],[-1,2],[-1,2],[-5,6],[-1,2],[0,4],[0,4],[0,5],[0,5],[1,5],[0,9],[1,9],[-1,6],[-1,7],[-1,4],[-1,3],[-2,2],[-13,6],[-3,2],[-2,2],[-1,2],[-5,10],[-3,6],[-2,3],[-2,3],[-3,1],[-4,0],[-5,-4],[-7,-2],[-2,0],[-3,-1],[-3,-1],[-2,-3],[-5,-8],[-2,-4],[-8,-6],[-10,-7],[-2,-1],[-1,-2],[-1,-3],[1,-3],[2,-2],[4,-1],[2,-1],[1,-1],[1,-2],[0,-2],[-1,-2],[-3,-1],[-23,-3],[-3,-1],[-3,-3],[-3,-5],[-3,-11],[-2,-3],[-3,-3],[-2,-1],[-2,0],[-1,1],[-1,0],[-1,-1],[-1,1],[0,2],[-1,1],[-1,0],[-2,-1],[-1,0],[-1,0],[-2,0],[-1,0],[-2,-2],[-1,0],[-8,-4],[-4,-4],[-1,-1],[4,-6],[10,-21],[1,-3],[-1,-2],[-1,-1],[-1,-2],[0,-3],[2,-1],[4,-1],[3,-2],[2,-4],[2,-4],[-2,-2],[-1,-1],[-1,-2],[-2,-2],[3,-1],[2,1],[2,2],[1,1],[1,-3],[0,-2],[-1,-2],[-1,-3],[-1,-6],[0,-6],[1,-2],[2,-4],[0,-1],[2,-5],[8,-4],[2,-4],[1,0],[10,-10],[-1,-2],[-3,2],[-1,-1],[0,-3],[0,-3],[1,-4],[1,-3],[3,-6],[-1,0],[-1,1],[-1,0],[-2,1],[1,-3],[0,-1],[-3,-2],[-9,-11],[-2,0],[-2,-1],[-1,0],[-1,-1],[-7,-2],[-3,-2],[-1,-2],[-4,-9],[-1,-2],[-2,-2],[-2,-1],[-9,-3],[-2,-1],[-2,-2],[-2,-2],[-1,-3],[-2,-10],[-3,-6],[-2,-2],[-2,0],[-7,2],[-4,-1],[-2,-1],[-1,0],[-3,-3],[-11,-15],[-6,-7],[-2,0],[0,3],[0,3],[1,3],[0,4],[-1,4],[-1,5],[-2,3],[-1,2],[-1,3],[0,4],[2,5],[1,4],[2,3],[2,2],[1,1],[1,2],[2,4],[3,12],[1,3],[1,3],[0,3],[1,4],[0,3],[-1,3],[-1,3],[0,2],[-4,6],[-1,3],[1,2],[0,3],[0,2],[-1,6],[-5,7],[-1,3],[-2,0],[-3,4],[-1,1],[-1,0],[-2,0],[0,-1],[-1,-2],[1,-9],[-2,-8]],[[4606,4697],[-4,0],[-1,0],[-2,-1],[-1,-2],[-1,-2],[-1,-1],[-2,-1],[-11,1],[-3,0],[-1,1],[-3,18],[-3,14],[-2,5],[-3,7],[-4,5],[-1,2],[-1,1],[-3,-1],[-1,-1],[-4,-3],[-1,-1],[-1,-1],[-1,0],[-4,-7],[-2,-1],[-1,-1],[-2,-1],[-2,-1],[-4,0],[-5,0],[-5,2],[-8,6],[-2,1],[-2,1],[-2,-2],[-4,-4],[-2,0],[-1,3],[-1,7],[1,11],[1,6],[-1,6],[-1,7],[-3,9],[-1,3],[-3,3],[-2,0],[-4,0],[-2,1],[-3,3],[-2,0],[-2,1],[-1,0],[-1,0],[-6,7],[-4,6],[-4,9],[-5,17],[-7,16],[-10,18],[-1,5],[0,4],[0,4],[3,15],[1,4],[1,5],[3,4],[7,8],[4,7],[6,5],[4,-2],[12,-11],[4,-2],[4,-2],[12,-1],[3,-1],[3,-2],[3,-2],[2,0],[2,0],[1,0],[2,0],[2,-1],[2,-3],[1,-1],[2,-1],[2,-1],[1,-2],[1,-2],[2,0],[5,-2],[10,-4],[6,-1],[2,-2],[1,-1],[1,-1],[2,0],[3,1],[2,-1],[7,-2],[6,-5],[1,-3],[1,-2],[2,-1],[1,-3],[-1,-3],[2,-2],[6,-4],[4,-2],[1,-1],[0,-1],[1,-1],[2,1],[1,-5],[4,-4],[3,-2],[1,2],[3,3],[3,4],[2,2],[2,4],[1,3],[1,2],[2,2],[2,-1],[8,-3],[3,0],[2,2],[0,2],[1,3],[1,3],[1,1],[3,0],[2,1],[1,2],[1,3],[-1,3],[0,2],[1,3],[4,1],[2,2],[2,2],[0,5],[2,3],[3,1],[6,0],[2,1],[1,2],[2,8],[1,3],[1,2],[3,1],[2,2],[0,1],[0,2],[0,2],[0,3],[-1,3],[-5,10],[-3,8],[0,3],[-1,2],[0,3],[0,1],[-1,1],[-7,6],[-5,4],[-2,0],[-1,0],[-7,-1],[-1,1],[-2,0],[-2,1],[-2,2],[-2,2],[-2,3],[-1,3],[-1,3],[-1,2],[0,3],[0,8],[-1,3],[-1,3],[-2,1],[-2,-1],[-1,-1],[-5,-7],[-2,-1],[-2,-1],[-2,0],[-2,0],[-2,2],[-2,2],[-1,2],[-2,3],[-1,3],[0,3],[-1,3],[1,3],[2,3],[1,1],[5,2],[2,2],[2,2],[3,5],[1,1],[4,-4],[3,0],[2,2],[1,2],[1,3],[0,3],[0,3],[0,3],[0,2],[-1,4],[0,3],[1,4],[2,5],[9,10],[2,2],[1,3],[0,2],[2,1],[4,0],[3,1],[4,2],[2,2],[3,3],[4,8],[9,8],[3,2],[5,1],[4,2],[1,4],[3,9],[4,8],[3,5],[6,5],[1,4],[-1,4],[-2,4],[-4,6],[-2,3],[0,3],[-1,3],[-2,1],[-2,0],[-2,0],[-3,0],[-1,3],[1,4],[4,7],[2,5],[0,4],[-1,7],[-1,4],[2,3],[2,2],[4,1],[8,0],[4,0],[3,1],[4,7],[5,11],[1,2],[3,4],[2,1],[2,2],[4,0],[3,0],[11,-5],[7,-1],[3,3],[3,11],[0,6],[0,8],[-2,11],[-3,13],[-1,6],[-1,6],[3,19],[0,3],[1,0],[2,0],[2,-1],[1,-1],[3,0],[1,2],[5,6],[8,0],[3,-1],[3,-3],[4,1],[12,-4],[1,1],[0,2],[1,3],[-1,5],[-2,12],[0,3],[1,3],[1,3],[3,6],[0,2],[0,1],[0,2],[0,1],[-2,4],[-2,2],[-3,3],[-3,3],[-1,2],[0,2],[0,2],[0,3],[1,2],[13,30],[2,2],[2,2],[-4,5],[-1,1],[-2,2],[-1,2],[-1,1],[-1,1],[-1,1],[-4,1],[-1,0],[-1,1],[-2,2],[-5,8],[-6,3],[-1,0],[-3,0],[-2,0],[0,1],[-1,3],[-1,4],[-1,10],[-2,7],[-3,21],[-6,9],[-2,7],[-2,8],[-2,6],[-2,3],[-3,3],[-6,3],[-3,4],[-2,4],[-1,4],[0,3],[0,4],[2,2],[2,2],[3,1],[3,1],[2,1],[2,2],[0,4],[0,5],[0,3],[-1,2],[-2,3],[-5,6],[-6,4],[-1,2],[-5,11],[-1,2],[-1,0],[-1,1],[-10,7],[-2,2],[-1,1],[0,2],[3,12],[3,14],[0,9],[4,2],[3,2],[2,4],[1,4],[0,4],[-1,4],[-5,9],[-10,11],[-3,6],[-2,4],[-4,2],[-4,2],[-5,2],[-17,21],[-5,5],[-4,2],[-4,0],[-2,0],[-2,0],[-14,12],[-2,3],[-2,4],[0,4],[1,1],[0,1],[4,8],[1,4],[-1,5],[-1,4],[-10,9],[-2,14],[-2,14],[-1,4],[-1,1],[-2,-1],[-1,-1],[-13,-23],[-4,-5],[-1,-2],[-2,-4],[-1,-16],[-1,-5],[-3,-1],[-3,1],[-3,3],[-3,4],[-4,10],[-2,3],[-8,3],[-58,50],[-39,43],[-9,6],[-11,4],[-48,28],[-6,6],[-3,21],[-2,6],[-2,6],[-4,5],[-4,3],[-2,1],[-2,1],[-7,0],[-5,0],[-8,4],[-5,4],[-3,4],[-3,5],[-2,4],[-3,3],[-3,1],[-1,2],[-3,8],[-3,5],[-10,11],[-2,2],[-3,0],[-1,-2],[-1,-2],[-1,-7],[-1,-9],[0,-6],[0,-1],[3,-9],[2,-6],[1,-2],[1,-1],[3,-3],[1,-1],[0,-1],[1,-1],[-1,-5],[0,-1],[1,-1],[0,-1],[1,-6],[1,-3],[0,-2],[0,-2],[0,-1],[-1,-1],[-1,-1],[-1,0],[-1,1],[-5,10],[-6,8],[-13,9],[-14,6],[-13,4],[-6,4],[-39,39],[-5,7],[-1,4],[-1,3],[-1,3],[0,2],[-1,3],[-1,1],[-9,6],[-2,4],[-12,18],[-42,48],[-2,4],[0,3],[0,2],[2,4],[0,3],[-1,2],[-2,3],[-3,2],[-4,1],[-8,-1],[-3,0],[-2,2],[-5,10],[-2,2],[-2,2],[-3,1],[-6,1],[-3,2],[-3,2],[-5,2],[-6,1],[-1,-1],[-1,-1],[-1,-5],[-1,-4],[-1,-3],[-2,-2],[-1,-2],[-7,-9],[-4,-3],[-4,-2],[-5,-1],[-3,1],[-2,2],[-3,0],[-10,0],[-3,0],[-3,2],[-2,2],[-3,7],[-2,3],[-3,2],[-2,1],[-3,1],[-2,0],[-2,-2],[-2,-2],[-2,-7],[-1,-3],[-1,-6],[-2,-5],[-11,-15],[-15,-16],[-6,-3],[-8,9],[-52,38],[-3,6],[-1,1],[-49,37],[-2,1],[-37,14],[-9,0],[-9,-2],[-2,-1],[-1,-1],[-1,-1],[-1,-1],[0,-2],[-1,-5],[-1,-3],[-1,-2],[0,-1],[-2,-2],[0,-1],[-1,-2],[1,-2],[3,-17],[1,-7],[0,-3],[0,-1],[-1,-15],[0,-5],[0,-3],[-1,-3],[-1,-6],[0,-3],[0,-4],[1,-2],[0,-3],[-1,-10],[1,-4],[1,-6],[0,-3],[-1,-3],[-1,-2],[0,-3],[0,-3],[0,-4],[0,-1],[0,-1],[-1,-1],[-3,-2],[-1,-2],[0,-2],[0,-4],[1,-4],[3,-6],[0,-2],[1,-3],[0,-2],[0,-2],[-2,-48],[-1,-5],[-1,-1],[0,-1],[-1,-1],[-2,-1],[-3,0],[-11,1],[-15,6],[-5,3],[-2,2],[-2,2],[-3,3],[-18,7],[-6,4],[-3,3],[-1,3],[-1,2],[0,2],[0,1],[2,8],[0,2],[-1,2],[-1,1],[-10,4],[-7,5],[-13,12],[-3,1],[-3,1],[-8,-1],[-4,1],[-4,2],[-7,6],[-2,1],[-3,0],[-2,0],[-2,-1],[-2,1],[-3,2],[-1,3],[-2,3],[-1,8],[-1,3],[-1,1],[-2,2],[-19,8],[-1,2],[-4,6],[-11,13],[-3,2],[-2,1],[-4,0],[-2,0],[-3,3],[-3,3],[-5,9],[-9,11],[-1,2],[-1,3],[-1,3],[-1,2],[-2,1],[-11,4],[-3,2],[-3,3],[-17,23],[-6,11],[-1,1],[-1,1],[-2,-1],[-7,-4],[-3,-1],[-3,0],[-13,1],[-3,-1],[-10,-4],[-2,0],[-6,1],[-4,2],[-1,1],[-4,3],[-2,4],[-2,2],[-1,0],[-2,1],[-10,2],[-20,8],[-4,3],[-3,4],[-3,4],[-1,2],[-2,1],[-3,2],[-12,1],[-2,1],[-4,2],[-6,2],[-50,0],[-4,1],[-6,0],[-9,-4],[-4,0],[-9,-1],[-2,0],[-10,-5],[-1,0],[-4,0],[-4,2],[-1,1],[-4,0],[-4,0],[-4,1],[-5,2],[-1,0],[-2,0],[-13,-7],[-3,-1],[-14,1],[-3,-1],[-3,-2],[-4,-3],[-8,-8],[-2,-2],[-2,-1],[-21,-8],[-3,0],[-10,0],[-9,3],[-2,0],[-2,-1],[-7,-4],[-2,-2],[-1,-2],[-3,-3],[-2,-2],[-4,0],[-11,2]],[[3160,6185],[0,3],[1,39],[2,21],[0,10],[0,3],[-1,3],[-28,95],[-1,8],[-1,6],[1,4],[0,3],[1,4],[22,46],[1,4],[1,6],[1,81],[2,13],[8,4],[8,1],[41,-6],[5,2],[13,11],[17,8],[5,3],[7,8],[15,25],[8,16],[7,21],[41,74],[93,127],[78,54],[71,18],[46,-7],[5,1],[6,2],[4,4],[14,17],[14,20],[1,2],[0,3],[0,4],[0,4],[-3,10],[-1,5],[0,10],[4,95],[2,11],[1,5],[2,4],[3,3],[45,36]],[[3721,7129],[2,0],[16,3],[58,7],[59,7]],[[3856,7146],[107,-309],[-27,-31],[-4,-4],[-2,-8],[2,-11],[8,-17],[21,-35],[19,-26],[6,-3],[5,-5],[30,-36],[4,-3],[11,-6],[2,-3],[3,-4],[0,-4],[0,-6],[-15,-83],[0,-3],[2,-1],[44,2],[10,2],[4,2],[4,2],[4,3],[20,27],[6,11],[2,3],[1,1],[54,29],[56,2],[7,2],[20,13],[17,3],[40,-4],[3,-1],[3,-3],[2,-3],[2,-3],[17,-33],[1,-3],[1,-3],[2,-9],[0,-10],[0,-6],[0,-5],[-2,-5],[-8,-16],[-21,-31],[-1,-3],[-3,-8],[-5,-23],[-2,-5],[-2,-5],[-3,-4],[-2,-4],[-45,-36],[-15,-18],[-12,-20],[-4,-8],[-1,-4],[42,-1],[12,-4],[13,-15],[14,-10],[19,-10],[24,-9],[5,-4],[6,-6],[3,-4],[2,-4],[1,-2],[1,-3],[0,-4],[1,-11],[1,-2],[1,-2],[3,-3],[3,-2],[16,-5],[3,-3],[2,-4],[1,-3],[0,-3],[0,-3],[1,-2],[2,-3],[3,-1],[3,-1],[40,-1],[3,-1],[3,-2],[1,-4],[2,-10],[1,-9],[0,-4],[-1,-17],[0,-8],[1,-8],[1,-4],[2,-5],[1,-4],[2,-4],[9,-11],[17,-16],[1,-1],[3,-1],[2,-1],[3,0],[1,1],[2,0],[1,1],[4,5],[2,1],[3,0],[3,-1],[2,-2],[1,-2],[0,-1],[-1,-3],[-1,-4],[0,-4],[0,-4],[0,-3],[1,-2],[2,-3],[1,-2],[3,-9],[3,-5],[2,-2],[8,-5],[1,-2],[0,-3],[-1,-2],[-2,-3],[-1,-2],[0,-3],[2,-3],[2,-1],[2,0],[7,0],[4,-1],[27,-16],[3,0],[2,1],[2,1],[2,1],[3,0],[17,-3],[12,0],[3,3],[6,12],[6,8],[12,14],[4,8],[2,5],[-1,4],[-7,8],[-5,9],[-15,20],[-6,8],[0,2],[0,1],[0,1],[1,1],[2,1],[2,2],[26,7],[62,31],[3,1],[4,0],[3,-1],[3,-1],[12,-10],[4,-3],[4,-1],[18,-5],[21,-8],[22,-3],[4,1],[2,2],[11,15],[5,5],[6,5],[16,10],[8,8],[29,17],[47,18],[45,4],[2,-1],[2,-1],[2,-1],[1,-3],[-1,-3],[-2,-12],[-1,-4],[1,-3],[0,-3],[2,-4],[24,-50],[2,-4],[1,-4],[0,-3],[0,-3],[-1,-4],[-1,-4],[-7,-20],[-2,-4],[-1,-1],[-1,-1],[-7,-4],[-4,-3],[-3,-4],[-3,-6],[-10,-30],[-5,-10],[-6,-11],[-7,-10],[-13,-14],[-53,-41],[-4,-4],[-1,-2],[0,-2],[1,-3],[4,-8],[2,-4],[1,-5],[7,-44],[0,-4],[-1,-2],[-18,-6],[-3,-2],[-1,-2],[-1,-6],[0,-7],[1,-14],[5,-37],[1,-3],[2,-2],[35,-15],[4,-3],[31,-39],[40,-44],[5,-4],[4,-1],[2,-1],[8,-9],[4,-2],[2,-1],[2,0],[4,3],[2,1],[2,0],[5,-2]],[[4606,4697],[-2,-15],[-3,-6],[-6,-4],[-4,-4],[-2,-4],[-1,-6],[-2,-8],[0,-5],[1,-3],[2,-3],[6,-4],[1,-3],[1,-4],[0,-4],[-1,-3],[-4,-7],[-8,-23],[-3,-3],[-3,0],[-3,3],[-5,10],[-6,8],[-7,8],[-3,1],[-3,1],[-3,1],[-3,-2],[-1,-1],[0,-3],[-1,-4],[-1,-4],[-2,-2],[-2,2],[-2,3],[-2,1],[-3,1],[-4,-3],[-8,-4],[-5,-2],[-5,-5],[-3,-3],[-1,-3],[0,-4],[-5,-18],[-1,-6],[0,-5],[0,-8],[0,-5],[1,-4],[1,-4],[1,-2],[1,-1],[9,-6],[2,-3],[1,-2],[1,-4],[1,-4],[0,-6],[0,-3],[-3,-13],[-5,-13],[-3,-5],[-2,-4],[-2,-1],[-3,-1],[-1,0],[-1,0],[-3,2],[-2,1],[-1,1],[-2,0],[-2,-2],[-7,-7],[-2,-2],[-2,-1],[-5,-1],[-2,0],[-2,-1],[-1,-1],[-2,0],[-2,1],[-1,2],[-1,1],[-2,2],[-1,2],[-4,1],[-2,1],[-14,2],[-8,4],[-2,0],[-2,-1],[0,-2],[0,-1],[1,-5],[0,-3],[-1,-3],[-2,-3],[-2,-3],[-1,-3],[0,-4],[0,-6],[-1,-4],[0,-1],[0,-3],[-2,-7],[-3,-13],[-1,-2],[-2,-1],[-7,-6],[-2,-4],[-7,8],[-3,6],[-1,2],[1,3],[0,3],[1,2],[1,1],[2,3],[7,7],[1,1],[1,1],[1,2],[1,4],[0,1],[-1,2],[-2,3],[-1,3],[-1,3],[-1,1],[-2,2],[-1,2],[-1,4],[0,3],[0,3],[-2,4],[-5,6],[-1,3],[-2,3],[-7,8],[-1,0],[-3,-3],[-2,-2],[-2,0],[-6,0],[-5,-1],[-3,-1],[-1,-1],[-2,-3],[-1,-4],[-1,-11],[0,-4],[-1,-2],[-3,-1],[-4,1],[-7,3],[-2,3],[-2,3],[1,2],[1,3],[2,6],[1,3],[2,7],[0,4],[0,4],[-2,13],[0,2],[-2,3],[-8,10],[-6,13],[-3,2],[-3,1],[-6,-2],[-2,0],[-1,0],[-1,1],[-2,3],[-3,4],[-2,4],[-2,5],[-3,6],[-2,1],[-1,-1],[0,-1],[-1,-3],[0,-8],[-1,-3],[-1,-3],[-2,-3],[-2,-2],[-4,-1],[-2,-2],[-2,-2],[-1,-2],[0,-4],[1,-7],[0,-3],[-1,-3],[-5,-24],[-2,-3],[-1,-1],[-2,0],[-7,7],[-3,2],[-7,3],[-2,2],[-7,7],[-3,2],[-2,2],[-14,2],[-5,2],[-2,2],[-3,5],[-2,0],[-4,0],[-3,1],[-3,1],[-2,3],[-2,3],[-3,12],[-2,6],[-2,6],[-4,5],[-16,14],[-2,2],[-7,2],[-23,14],[-5,5],[-4,4],[-2,6],[-4,12],[-1,3],[-4,7],[-1,3],[-1,3],[1,13],[0,2],[-2,5],[-1,7],[-1,8],[0,4],[0,3],[-3,8],[-1,2],[-1,1],[-3,3],[-1,2],[-1,3],[-3,12],[-1,3],[-2,2],[-2,1],[-1,2],[-1,2],[0,3],[1,2],[1,2],[1,1],[1,1],[0,2],[0,2],[-1,3],[-11,18],[-20,25],[-2,3],[-3,1],[-3,2],[-2,1],[-2,0],[-10,-4],[-4,-1],[-2,0],[-2,1],[-1,2],[0,3],[0,2],[0,3],[-1,3],[-2,5],[-4,4],[-2,2],[-29,5],[-2,0],[-3,-1],[-2,-1],[-3,-3],[-4,-6],[-12,-10],[-9,-4],[-10,-2],[-2,0],[-1,2],[0,3],[0,3],[0,12],[-1,3],[-1,4],[-2,4],[-3,5],[-1,3],[-6,0],[-2,-1],[-1,-1],[-1,-3],[0,-4],[0,-4],[-1,-4],[-3,-3],[-26,-13],[-7,-1],[-4,-2],[-1,-2],[0,-3],[-1,-3],[-1,-3],[1,-4],[2,-9],[2,-5],[0,-6],[0,-16],[-1,-8],[0,-4],[0,-3],[0,-2],[4,-6],[1,-2],[2,0],[4,-1],[2,-1],[2,-3],[1,-4],[1,-13],[1,-3],[1,-2],[1,-2],[2,-3],[3,-2],[3,-1],[3,-1],[6,1],[1,-1],[1,-2],[2,-1],[1,-2],[2,-2],[9,-3],[2,-1],[1,-1],[1,-1],[0,-2],[-1,-2],[-2,-3],[-3,-4],[-3,-2],[-2,-1],[-4,0],[-2,-1],[-3,-3],[-1,-4],[-3,-9],[-2,-4],[-13,-19],[0,-1],[-2,-4],[-1,-5],[0,-6],[0,-3],[0,-4],[3,-9],[0,-3],[1,-4],[0,-5],[0,-3],[-1,-2],[-1,-2],[-2,-3],[-2,-2],[-9,-4],[-2,-2],[-2,-3],[-2,-4],[-2,-7],[0,-1],[0,-1],[1,-5],[1,-5],[1,-21],[2,-5],[5,-10],[0,-2],[0,-2],[1,-2],[2,-1],[1,-1],[3,-4],[15,-13],[1,-1],[2,-1],[3,0],[1,0],[1,-2],[2,-6],[0,-1],[7,-7],[9,-5]],[[3942,4436],[-1,-1],[-1,-8],[-2,-2],[-3,-1],[-1,-2],[-1,-2],[-1,-4],[-1,-3],[-1,-2],[-2,-1],[-2,0],[-1,2],[-4,6],[-2,2],[-2,1],[-10,0],[-17,5],[-4,0],[-2,-1],[-1,-3],[0,-2],[0,-4],[-2,-4],[-2,-4],[-1,-3],[0,-3],[2,-2],[2,-2],[3,0],[3,-2],[1,-2],[0,-3],[0,-5],[-3,-10],[0,-2],[2,-5],[0,-2],[-2,-4],[-3,-2],[-4,1],[-3,1],[-4,0],[-2,-4],[-2,-5],[-2,-8],[-1,-10],[0,-4],[0,-4],[1,-2],[1,-1],[1,0],[2,-1],[0,-1],[-1,-4],[-3,-6],[-2,-3],[-3,-1],[-4,1],[-2,0],[-1,-2],[-1,-6],[-2,-4],[-3,-2],[-3,-1],[-7,1],[-10,3],[-3,-1],[-3,-1],[-8,-6],[-3,-1],[-2,0],[-3,1],[-2,0],[-3,-1],[-8,-5],[-3,0],[-6,1],[-2,-2],[-1,-2],[0,-2],[2,-3],[7,-8],[0,-4],[-1,-4],[-3,-7],[-2,-4],[0,-4],[0,-3],[1,-4],[4,-7],[1,-4],[1,-5],[0,-6],[-2,-6],[-3,-3],[-2,-1],[-2,1],[-5,3],[-7,8],[-3,1],[-2,0],[-5,2],[-7,1],[-13,4],[-3,2],[-5,4],[-3,4],[-5,7],[-9,20],[-4,4],[-4,0],[-9,-4],[-5,-3],[-2,-3],[0,-1],[-1,-3],[-1,-3],[-4,-6],[-1,-6],[1,-5],[3,-3],[6,-6],[2,-3],[2,-4],[1,-4],[0,-6],[0,-4],[-1,-5],[-1,-3],[-1,-4],[0,-4],[0,-2],[1,-2],[2,-2],[1,-1],[0,-3],[-1,-2],[-3,-5],[-3,-2],[-3,1],[-1,3],[-2,3],[0,4],[-1,3],[-3,6],[-1,3],[-1,4],[0,5],[-1,5],[-1,4],[-2,3],[-3,3],[-3,1],[-2,0],[-3,0],[-13,-2],[-8,0],[-9,2],[-2,-1],[-2,-1],[-4,-3],[-2,0],[-2,1],[-1,2],[-2,5],[-2,3],[-3,2],[-13,4],[-4,1],[-4,-1],[-6,-1],[-5,-3],[-3,-1],[-2,0],[-2,1],[-5,8],[-6,6],[-2,3],[-1,4],[-1,4],[-1,13],[-1,4],[-4,13],[-1,4],[1,3],[1,3],[2,2],[0,1],[6,2],[2,1],[0,1],[0,2],[-1,2],[-1,1],[-1,2],[0,2],[1,3],[2,3],[0,3],[0,3],[-1,2],[-1,3],[-1,2],[0,7],[-2,3],[-2,2],[-8,3],[-2,1],[-2,2],[-2,3],[-2,6],[-1,3],[-1,1],[-2,1],[-23,1],[-6,2],[-3,2],[-10,12],[-5,3],[-2,2],[-1,1],[0,2],[-1,2],[0,4],[-1,18],[-1,2],[0,3],[-2,2],[-1,2],[-10,10],[-16,9],[-2,2],[-1,3],[-1,7],[-2,3],[-2,2],[-4,4],[-2,2],[-2,3],[-5,9],[-2,1],[-3,1],[-3,-1],[-2,-2],[-5,-6],[-3,-3],[-2,-1],[-9,-4],[-2,-2],[-2,-2],[-3,-1],[-2,1],[-3,5],[-1,1],[-2,-1],[-1,-2],[-1,-3],[0,-4],[0,-4],[-2,-4],[-3,-2],[-18,-3],[-5,-2],[-3,-4],[-1,-1],[0,-1],[-1,-2],[-2,-2],[-4,-2],[-17,1],[-8,4],[-4,3],[-2,0],[-1,-1],[-1,-1],[-1,0],[-4,-1],[-4,1],[-3,3],[-5,11],[-5,3],[-6,2],[-15,0],[-8,-2],[-6,-5],[-4,-5],[-4,-3],[-2,-1],[-3,2],[-3,4],[-1,4],[-2,4],[-2,1],[-8,-1],[-3,1],[-3,2],[-6,9],[-14,13],[-4,2],[-6,0],[-13,-1],[-2,-1],[-1,-1],[-1,-1],[-2,0],[-2,3],[-1,4],[-2,5],[0,5],[-1,5],[-2,2],[-2,0],[-9,-4],[-3,0],[-10,0],[-20,9],[-12,-6],[-15,4],[-2,0],[-4,0],[-3,1],[-7,3],[-2,0],[-2,-1],[-1,-1],[-1,-1],[-1,-1],[-2,1],[-4,3],[-2,0],[-2,0],[-1,-2],[-2,-1],[-1,-2],[-1,0],[-2,1],[-1,2],[-2,0],[-1,0],[-3,-3],[-1,-1],[-2,-1],[-1,0],[-3,0],[-1,1],[-2,1],[-1,3],[-3,14],[0,3],[1,3],[1,2],[0,2],[1,2],[-1,1],[-1,1],[-2,2],[-3,1],[-7,2],[-3,0],[-3,0],[-2,-2],[-2,-2],[-1,-2],[-2,-1],[-2,-2],[-3,-1],[-2,0],[-3,0],[-4,2],[-5,4],[-11,14],[-3,4],[-2,4],[-2,4],[-3,2],[-3,0],[-5,0],[-2,0],[-2,1],[-15,15],[-5,3],[-9,8],[-11,16],[-17,35],[-1,3],[0,2],[0,1],[1,1],[1,0],[1,0],[1,1],[0,1],[1,1],[0,1],[0,1],[0,1],[-1,1],[-2,2],[-2,1],[-1,0],[-28,0],[-2,0],[-1,-1],[-1,-1],[-1,-2],[-1,-2],[-1,-2],[0,-2],[-1,-6],[0,-2],[-1,-2],[-1,-2],[-15,-5],[-3,0],[-11,1],[-2,0],[-7,-10],[-6,-9],[-3,-3],[-3,-2],[-3,-2],[-5,0],[-5,0],[-10,3],[-4,1],[-12,9],[-19,17],[-5,8],[-2,1],[-2,1],[-6,-2],[-3,0],[-6,1],[-3,2],[-2,3],[-3,3],[-2,2],[-3,6],[-6,14],[-1,4],[-1,4],[0,6],[0,5],[1,5],[1,2],[-1,1],[0,1],[-2,3],[-3,6],[-1,5],[-2,3],[-2,2],[-7,5],[-2,1],[-4,0],[-4,1],[-6,3],[-1,1],[-2,2],[-1,1],[-1,3],[0,4],[-1,8],[-1,7],[-2,4],[-2,2],[-5,3],[-1,2],[-1,3],[1,5],[0,4],[-1,3],[-2,3],[-12,15],[-2,1],[-3,1],[-2,1],[-3,5],[-10,19],[-1,5],[-1,4],[-1,4],[-3,7],[-1,4],[0,5],[2,7],[1,3],[2,3],[2,1],[2,0],[4,0],[2,1],[2,1],[3,5],[4,5],[1,2],[1,7],[4,25],[1,3],[2,4],[1,3],[0,3],[-1,4],[-1,2],[-1,1],[-3,1],[-1,0],[0,1],[-1,2],[0,3],[-1,5],[1,17],[-1,13],[-5,29],[-1,3],[-2,2],[-1,2],[-2,2],[0,2],[-1,3],[0,2],[0,2],[1,2],[11,13],[4,7],[2,8],[-3,16],[-3,6],[-2,1],[-3,1],[-6,1],[-13,-1],[-5,0],[-5,2],[-2,1],[-1,3],[0,2],[0,4],[1,6],[1,11],[0,4],[-1,4],[-1,5],[-4,6],[-7,12],[-4,7],[-3,6],[0,4],[0,2],[1,1],[1,1],[1,3],[1,1],[1,3],[0,3],[0,4],[-1,7],[0,3],[1,3],[1,2],[2,1],[2,1],[10,4],[7,4],[18,15],[2,1],[0,2],[0,3],[-1,3],[-1,2],[-3,1],[-2,2],[-1,3],[0,3],[1,2],[3,6],[1,4],[0,4],[-1,9],[-1,4],[1,3],[2,7],[7,18],[2,5],[1,5],[1,8],[0,4],[-1,2],[-1,1],[-2,1],[-2,1],[-2,0],[-12,-2],[-6,0],[-9,2],[-6,3],[-5,4],[-3,2],[-2,2],[-2,4],[-2,6],[-3,9],[-1,6],[0,4],[0,2],[1,0],[0,1],[1,1],[3,6],[5,4],[3,3],[13,5],[14,2]],[[2628,5372],[31,-4],[4,0],[4,1],[5,3],[2,3],[2,3],[1,4],[0,4],[6,32],[1,1],[1,0],[3,-2],[4,-5],[2,-3],[2,-4],[6,-14],[1,-2],[1,-2],[2,-1],[1,-1],[21,-4],[54,-1],[3,-1],[31,-26],[4,-1],[3,0],[4,0],[3,2],[15,11],[7,3],[7,1],[3,1],[2,2],[1,3],[1,3],[0,5],[0,5],[-3,15],[-5,17],[-4,14],[-6,39],[-3,11],[-1,1],[-1,1],[-2,2],[-2,1],[-5,2],[-13,2],[-3,2],[-6,4],[-7,8],[-8,12],[-2,3],[-1,2],[0,1],[-1,11],[0,12],[0,8],[1,6],[1,2],[2,4],[9,11],[6,13],[3,3],[2,1],[9,3],[1,0],[11,-2],[3,0],[3,0],[9,4],[17,12],[5,3],[16,3],[2,1],[1,2],[2,3],[1,2],[0,2],[0,5],[0,6],[-1,3],[0,4],[-1,2],[-2,3],[-11,15],[-11,28],[-2,6],[-5,25],[-1,5],[-2,6],[-2,4],[-2,2],[-1,1],[-19,9],[-3,2],[-7,7],[-5,8],[-5,6],[-26,17],[-18,16],[-21,12],[-1,1],[-2,2],[-1,3],[-1,2],[0,2],[0,3],[0,2],[0,2],[0,1],[6,15],[1,4],[0,5],[0,8],[-1,5],[-1,3],[-40,87],[-5,14],[-2,8],[0,6],[1,4],[3,3],[2,2],[3,1],[6,1],[2,0],[9,-2],[2,0],[10,5],[3,3],[2,2],[11,8],[9,10],[2,3],[5,5],[8,4],[1,1],[0,1],[1,1],[0,2],[0,5],[1,2],[1,2],[0,1],[1,1],[1,0],[7,-1],[5,0],[26,7],[4,1],[15,-1],[17,4],[13,7],[5,1],[11,0],[8,4],[16,12],[4,2],[4,1],[23,2],[41,13],[15,9],[24,9],[21,13],[14,16],[2,1],[3,2],[5,1],[18,2],[50,14],[15,6],[3,1],[2,0]],[[5803,1389],[0,-1],[-1,0],[-1,5],[2,5],[3,2],[2,-4],[-1,-5],[-2,-2],[-2,0]],[[5652,1522],[-8,-3],[-1,1],[1,1],[2,1],[3,4],[2,1],[1,-1],[1,-2],[-1,-2]],[[5717,1584],[0,-1],[-1,0],[0,2],[0,1],[-1,1],[1,1],[1,-1],[0,-1],[0,-2]],[[6276,2371],[-1,-2],[-4,-2],[-1,0],[-1,-1],[-3,-3],[0,-1],[0,-1],[-1,-2],[1,-1],[0,-4],[1,-3],[0,-1],[0,-2],[-2,-1],[-1,-1],[0,-3],[0,-4],[1,-2],[1,-2],[1,-3],[3,-2],[0,-1],[2,-3],[1,-1],[3,-1],[1,-1],[0,-1],[0,-2],[0,-3],[-2,-9],[-1,-2],[0,-3],[1,-2],[0,-2],[3,-6],[6,-16],[1,-3],[0,-1],[0,-2],[0,-2],[-1,-2],[-1,-2],[0,-1],[-8,-10],[-2,-5],[-1,-1],[-2,0],[-2,1],[-1,0],[-1,-1],[-5,-6],[-4,-3],[-2,-2],[0,-1],[-1,-2],[0,-1],[0,-3],[0,-2],[0,-1],[1,-1],[1,-3],[0,-1],[0,-2],[1,-4],[-1,-14],[0,-2],[1,-3],[2,-10],[1,-7],[1,-4],[-1,0],[0,-1],[0,-1],[-2,-7],[-1,-2],[-2,-2],[-1,-2],[-2,-1],[-4,-3],[-1,-1],[-2,-4],[-2,-16],[-1,-2],[-1,-2],[-1,0],[-2,-1],[-1,0],[-1,1],[-2,0],[-4,3],[-2,0],[-1,0],[-5,0],[-1,-1],[-4,-6],[-1,-3],[-3,-13],[0,-2],[0,-3],[0,-1],[0,-1],[1,-2],[0,-1],[1,-1],[0,-1],[1,-2],[0,-3],[0,-3],[-1,-2],[-2,-3],[-1,-1],[-1,-1],[-8,-1],[-3,-2],[-7,-5],[-2,-2],[-1,-1],[0,-2],[-1,-1],[1,-6],[0,-1],[0,-2],[0,-3],[-1,-3],[-3,-9],[-9,-22],[-1,-3],[0,-2],[-1,-6],[-1,-6],[-1,-4],[0,-2],[1,-5],[0,-1],[-1,-2],[0,-6],[-1,-2],[0,-2],[2,-5],[0,-3],[1,-6],[0,-2],[1,-4],[0,-1],[0,-3],[0,-4],[-1,-4],[-1,-4],[0,-1],[0,-2],[0,-1],[1,-4],[0,-1],[1,-1],[1,-2],[2,-2],[1,-2],[0,-1],[1,-4],[-1,-2],[0,-2],[-1,-2],[-2,-4],[0,-3],[-1,-4],[0,-2],[-1,-1],[-1,-1],[-2,-2],[-1,-13],[0,-4],[-2,-2],[-3,-5],[-8,-17],[-1,-2],[-3,-2],[-3,-4],[-3,-5],[-2,-3],[-1,-1],[-2,-1],[-19,-2],[-2,-1],[0,-1],[0,-1],[3,-2],[0,-1],[1,-1],[0,-1],[1,-2],[-1,-3],[-3,-7],[-5,-6],[-1,-1],[-1,0],[-1,0],[-1,0],[-1,0],[-2,1],[-1,2],[-1,0],[-1,1],[-2,0],[-1,-1],[-1,0],[0,-2],[-1,-5],[-1,-2],[-1,-1],[-1,0],[-5,-1],[-3,-4],[-3,-1],[-3,1],[-1,0],[-1,-1],[-1,-1],[0,-1],[-3,-3],[-1,-1],[-1,-1],[-1,1],[-1,0],[-1,3],[-1,0],[-1,0],[-1,-1],[0,-2],[-1,-3],[0,-2],[0,-2],[0,-2],[0,-1],[0,-2],[-1,-3],[-1,-3],[-1,-4],[0,-1],[-2,-2],[-1,0],[-1,0],[-1,2],[-1,0],[-1,1],[-1,-1],[0,-1],[0,-1],[1,-2],[2,-5],[1,-2],[1,-1],[0,-2],[0,-2],[0,-1],[-2,-8],[0,-2],[0,-1],[1,-2],[0,-1],[1,-1],[2,-2],[5,-6],[2,-4],[1,-3],[1,-2],[0,-2],[0,-1],[0,-2],[-1,-1],[-2,-1],[-1,-1],[-2,0],[-1,1],[-6,3],[-1,0],[-1,-1],[-1,-1],[-1,-1],[-1,-4],[-2,-4],[-1,-3],[0,-1],[0,-8],[0,-2],[0,-1],[-1,-1],[-2,-2],[-3,-2],[-1,-1],[-2,0],[-3,3],[-1,2],[-1,0],[-1,0],[-1,1],[-1,-1],[-1,-1],[-2,-1],[-4,-6],[-1,-1],[-1,0],[-1,0],[-1,1],[-1,1],[-9,12],[0,1],[-1,1],[-1,0],[-1,-1],[-1,-2],[-1,0],[-1,-1],[-2,0],[-4,0],[-1,0],[0,-1],[0,-2],[1,-5],[0,-2],[1,-2],[1,-1],[0,-1],[0,-2],[1,-2],[0,-6],[-2,-20],[0,-8],[1,-4],[5,-7],[-1,-5],[0,-8],[0,-2],[-1,-1],[-1,0],[-1,-1],[-3,1],[-3,1],[-3,1],[-4,-2],[-5,-3],[-3,-1],[-2,0],[-2,0],[-4,3],[-6,3],[-3,0],[-3,0],[-3,-1],[-6,-5],[-1,-1],[-2,0],[-2,0],[-6,0],[-1,-1],[-2,0],[0,-1],[-1,-1],[0,-1],[-1,-3],[-1,-1],[-5,-41],[0,-1],[-1,-2],[-3,-4],[-1,-1],[-1,-1],[-2,0],[-7,-4],[-3,-1],[-1,0],[-1,0],[0,1],[-1,1],[-2,3],[0,1],[-1,0],[-1,1],[-1,0],[-1,-1],[-1,-2],[0,-18],[0,-2]],[[5908,1534],[0,-7],[2,-9],[3,-5],[0,-2],[-2,0],[0,2],[-1,0],[-3,-5],[-5,2],[-10,10],[2,2],[1,8],[2,4],[-1,2],[0,4],[0,2],[0,1],[1,1],[0,1],[0,2],[-3,-2],[-4,-4],[-2,-4],[2,-4],[-1,-1],[-1,0],[-1,1],[-1,1],[0,2],[0,2],[0,3],[0,3],[-1,1],[0,1],[2,8],[-4,-4],[-1,-1],[-1,3],[0,2],[-1,3],[-1,2],[-1,1],[0,5],[1,2],[-1,2],[1,4],[2,3],[2,2],[0,2],[-1,-1],[-2,1],[-1,1],[0,2],[-12,-16],[1,-1],[0,-1],[0,-1],[1,-1],[-1,0],[-1,0],[0,-1],[1,0],[2,1],[1,0],[0,-5],[-2,-2],[-4,0],[-3,2],[0,-2],[11,-6],[2,-2],[0,-2],[5,-10],[0,-1],[-1,-1],[-1,-1],[0,-9],[0,-2],[-1,-2],[-2,-3],[0,-2],[-2,-3],[-7,-4],[-3,-3],[-1,3],[0,2],[1,2],[0,2],[-1,-1],[-2,-3],[-1,-3],[0,-2],[-1,-2],[-2,-1],[-14,0],[-1,1],[-2,6],[-1,-5],[-3,-4],[-3,-2],[-4,2],[0,-1],[-1,-1],[-1,-2],[-1,0],[0,8],[0,7],[-2,-4],[-1,-6],[-1,-8],[-7,0],[-2,1],[-1,4],[-1,-3],[0,-2],[1,-2],[1,-2],[-3,0],[-7,6],[-3,1],[-5,2],[-1,3],[0,5],[3,3],[4,2],[4,1],[4,1],[1,4],[0,12],[-1,13],[-1,0],[0,-3],[-2,0],[-1,1],[-1,3],[-1,0],[-2,-2],[-1,-1],[-1,2],[-1,3],[-1,0],[-1,-1],[-1,-2],[0,3],[1,2],[2,4],[0,2],[-3,-2],[-6,-6],[-3,-2],[-4,0],[-2,0],[-2,0],[-3,2],[-1,2],[-2,7],[-1,4],[-1,2],[-1,0],[-2,0],[-1,0],[-2,3],[0,3],[0,1],[1,1],[1,1],[-1,7],[0,2],[1,0],[0,1],[1,1],[2,1],[5,1],[0,2],[1,1],[-1,-1],[-2,0],[-2,-1],[1,4],[-1,-1],[-1,-1],[-1,-1],[-1,-2],[-1,0],[-1,1],[0,2],[0,2],[2,7],[-1,0],[-2,-3],[0,-2],[-1,3],[0,2],[1,2],[1,2],[-4,-1],[-6,2],[-4,-1],[2,-1],[5,-3],[-1,-1],[0,-1],[0,-1],[0,-2],[3,1],[1,-5],[1,-6],[-2,-5],[-1,-1],[-1,-2],[-1,1],[0,3],[0,3],[-1,3],[-1,2],[-2,-1],[1,-2],[1,-1],[-1,-3],[0,1],[0,1],[-1,0],[-1,0],[1,-2],[0,-1],[-1,0],[-1,-1],[3,0],[0,-2],[-1,0],[0,-1],[4,-4],[3,-3],[1,-5],[-3,-6],[3,0],[2,0],[2,0],[-2,-5],[-3,0],[-3,3],[-3,2],[-2,-1],[0,-2],[-1,-1],[-1,-2],[-2,-1],[-2,0],[-2,-1],[2,-3],[-2,2],[-1,5],[0,4],[-3,1],[0,2],[4,2],[1,-1],[1,-3],[1,0],[0,11],[-2,-2],[0,-1],[-1,-2],[-1,0],[-3,4],[1,6],[1,5],[0,5],[-1,-1],[-1,1],[-2,-7],[-1,-2],[-2,-2],[-1,3],[-3,2],[-1,2],[1,0],[2,1],[2,1],[0,2],[-3,1],[-2,0],[-1,0],[1,4],[1,3],[1,3],[2,2],[0,5],[-1,0],[0,-2],[-1,-2],[-1,-1],[-1,5],[0,2],[-1,0],[0,-4],[0,-3],[-1,-3],[0,-3],[-2,0],[0,3],[-1,1],[-1,0],[2,-6],[1,-3],[-1,-2],[-5,-2],[0,4],[0,3],[2,6],[-4,-2],[-2,3],[0,5],[2,3],[3,2],[3,11],[3,3],[0,2],[-3,1],[-4,6],[0,-2],[0,-1],[-1,-1],[0,-1],[-1,1],[-1,0],[-2,1],[1,-1],[1,-1],[1,0],[-2,-1],[-2,0],[-1,2],[0,4],[1,2],[1,1],[2,2],[-1,4],[-1,0],[0,-1],[0,-2],[-1,0],[0,1],[-1,2],[0,-4],[-1,-3],[-1,-3],[-2,-2],[-1,0],[-3,1],[0,-1],[1,-1],[1,-1],[0,-1],[1,-1],[-3,-1],[-3,-3],[-1,-3],[2,-2],[-5,2],[0,6],[3,15],[-1,-1],[-1,-2],[-1,-2],[0,-2],[-1,-8],[-1,-6],[1,-4],[2,-4],[3,-3],[-1,0],[-1,0],[0,-1],[0,-1],[1,-1],[0,-1],[2,2],[1,-2],[1,-3],[0,-2],[0,-3],[-1,-2],[-1,-3],[0,-5],[-1,1],[-1,0],[-1,0],[-2,-1],[2,-1],[2,-1],[2,-2],[1,-2],[0,-1],[-2,0],[0,-1],[0,-1],[1,0],[-1,0],[-2,0],[0,-2],[1,0],[2,-1],[1,-1],[0,-2],[0,-1],[0,-2],[2,8],[1,3],[2,1],[-1,-2],[0,-1],[-1,-1],[0,-1],[-1,-2],[1,-3],[0,-2],[0,1],[1,1],[1,0],[0,1],[1,1],[0,-1],[0,-1],[0,1],[2,4],[1,0],[1,-5],[-1,-2],[-2,-2],[-1,-4],[1,2],[1,0],[-1,-4],[-2,-1],[-2,0],[-2,2],[0,-2],[1,-1],[1,0],[1,-1],[1,0],[-1,-3],[0,-1],[-1,-1],[-2,-1],[1,-2],[1,-1],[-5,-3],[-2,-1],[0,2],[0,3],[2,2],[2,-1],[0,1],[-2,4],[-2,-4],[-1,-2],[-2,-1],[2,-2],[0,-2],[-1,-1],[-1,-1],[0,-1],[0,-1],[2,1],[0,-2],[-9,-1],[-3,0],[-2,4],[1,6],[2,1],[3,-2],[-1,-3],[2,1],[1,2],[1,8],[-1,0],[-1,-1],[0,-1],[-1,1],[-1,1],[0,2],[1,1],[1,1],[1,1],[1,1],[0,2],[1,1],[2,1],[2,1],[0,2],[-8,-2],[-2,0],[2,4],[1,2],[-1,3],[0,-1],[-2,-3],[0,-1],[-1,1],[-1,-3],[-2,0],[-8,7],[0,2],[1,5],[-4,-4],[-1,-7],[0,-8],[2,-6],[-3,0],[1,-3],[0,-1],[-1,-1],[-1,-1],[-1,-2],[0,-3],[2,0],[1,0],[0,-2],[-1,-1],[-3,-4],[-1,-1],[0,-4],[-1,-1],[-1,0],[-2,-1],[-5,-7],[-3,-2],[0,3],[2,4],[4,10],[1,4],[0,2],[0,2],[-1,2],[-1,2],[-2,4],[0,1],[2,4],[-4,-3],[-1,-1],[0,1],[0,3],[-1,-1],[0,-1],[-1,0],[1,-2],[1,-2],[0,-2],[-1,-2],[0,-1],[1,0],[-5,-7],[-3,-2],[-4,3],[-1,-2],[0,-2],[0,-1],[-2,1],[-1,2],[-1,-4],[-2,0],[-2,1],[-2,0],[1,-4],[-1,-3],[-2,0],[-4,4],[-2,1],[-3,0],[-2,-2],[6,-6],[2,-1],[-3,0],[-1,-4],[0,-3],[1,-2],[3,0],[1,2],[3,1],[3,-1],[0,-2],[-6,-1],[-5,-3],[-7,-11],[-1,0],[0,5],[1,3],[3,5],[0,2],[-7,12],[-3,1],[-5,9],[-5,5],[-2,3],[-2,5],[0,3],[-2,2],[-3,3],[-6,2],[-2,0],[-3,0],[-3,-1],[-6,-7],[-3,-2],[-10,-4],[-3,0],[-7,2],[-3,-1],[-1,-1],[-1,-4],[-1,-2],[-2,0],[-1,1],[-3,2],[-2,1],[-1,9],[-2,4],[-3,2],[-3,-2],[-4,-7],[-6,-5],[-2,3],[-1,1],[-4,19],[-5,8],[-5,-1],[-5,-2],[-7,2],[-12,17],[-5,5],[-7,2],[-3,2],[-3,3],[0,6],[2,4],[2,4],[2,4],[0,6],[-2,4],[-3,2],[-3,0],[-3,-2],[-3,-2],[-3,-1],[-3,3],[-2,4],[-2,4],[-2,2],[-4,2],[-7,0],[-2,0],[-4,3],[-1,1],[-1,3],[-1,1],[-2,1],[-1,0],[-5,-9],[-3,-3],[-2,0],[-1,2],[-1,2],[0,2],[0,3],[3,12],[1,6],[0,7],[-1,1],[-1,3],[-1,1],[0,3],[1,4],[0,2],[-1,3],[-1,2],[-1,3],[-1,4],[2,4],[2,2],[0,3],[-4,19],[-1,4],[-2,3],[-7,1],[-3,2],[-3,3],[-1,4],[0,10],[0,2],[-1,6],[0,3],[0,3],[2,4],[1,2],[4,27],[1,5],[2,4],[3,0],[2,-3],[2,-5],[2,-3],[4,0],[2,3],[3,14],[1,4],[4,8],[1,5],[1,8],[1,5],[2,2],[5,5],[1,3],[-1,4],[-4,0],[-3,0],[-3,0],[0,1],[-2,6],[-5,6],[-2,4],[-1,1],[-2,1],[-1,-1],[-2,-3],[-1,-1],[-3,0],[-2,4],[-1,4],[-2,4],[-3,2],[-8,3],[-2,2],[-2,1],[-2,-2],[-1,-6],[-2,-2],[-24,-9],[-3,-1],[-3,0],[-3,1],[-4,2],[-1,3],[-1,10],[-3,9],[-5,5],[-7,3],[-9,1],[-12,-1],[-2,-1],[-2,-9],[-3,-1],[-2,-2],[-5,-1],[-2,1],[-3,1],[-1,-1],[-1,-1],[-1,-2],[-1,-2],[-1,-1],[-3,3],[-9,16],[-2,5],[-2,3],[-2,1],[-2,1],[-2,2],[-3,4],[-2,2],[-4,0],[-5,2],[-11,1],[-3,1],[-1,4],[-1,5],[0,2],[0,9],[-1,6],[-1,2]],[[5218,1955],[1,5],[0,4],[0,2],[4,7],[4,5],[6,9],[1,1],[0,1],[1,1],[0,2],[0,3],[0,4],[0,1],[0,2],[0,1],[1,1],[0,1],[1,1],[1,0],[1,-1],[3,-2],[1,-1],[1,0],[1,1],[1,0],[0,1],[1,1],[0,1],[0,2],[0,3],[-2,4],[1,1],[0,1],[3,0],[1,0],[1,0],[0,1],[1,0],[2,3],[2,4],[3,6],[2,2],[3,3],[2,3],[1,1],[1,2],[0,2],[2,3],[0,1],[1,1],[1,0],[1,0],[1,0],[1,0],[1,0],[1,-1],[5,-4],[0,-1],[7,-6],[2,-1],[2,-1],[4,1],[1,0],[7,3],[1,1],[0,1],[1,2],[1,1],[2,3],[0,1],[1,0],[1,1],[2,1],[1,0],[1,1],[0,1],[0,1],[1,1],[1,22],[0,14],[0,3],[1,3],[1,5],[1,3],[0,2],[0,1],[0,3],[-3,12],[0,2],[0,1],[0,1],[0,2],[1,1],[1,1],[6,2],[1,0],[1,1],[0,1],[0,2],[0,1],[0,2],[0,1],[-1,1],[-7,18],[-1,1],[0,1],[0,1],[1,1],[0,1],[1,4],[1,1],[-1,2],[-2,2],[-9,7],[-1,0],[-3,4],[-4,4],[-4,4],[-1,3],[0,3],[1,1],[1,3],[0,3],[-1,1],[-2,0],[-1,-1],[-1,-1],[-2,-3],[0,-1],[-1,-1],[-1,0],[-1,-1],[-8,0],[-1,-1],[-1,0],[-1,-2],[0,-1],[-1,-1],[0,-2],[0,-1],[0,-4],[0,-1],[0,-1],[-1,-1],[-1,-1],[-6,1],[-3,0],[-4,-1],[-1,-1],[-1,1],[-2,1],[-5,5],[-3,2],[-6,0],[-1,1],[-2,1],[-2,2],[-1,2],[-1,2],[0,5],[-1,2],[-1,5],[-1,1],[-1,1],[0,1],[-1,0],[-1,0],[-2,0],[-2,-1],[-4,-2],[-1,0],[-2,0],[-1,-1],[-1,-1],[-1,-1],[0,-5],[-2,-8],[-1,-1],[0,-2],[0,-1],[-1,-1],[-3,-2],[-5,-1],[-1,0],[-1,1],[-2,2],[-1,3],[-1,1],[-2,1],[-3,2],[-4,5],[-5,6],[-2,2],[-1,1],[-1,0],[-1,0],[-1,-1],[0,-1],[0,-2],[2,-4],[0,-1],[0,-2],[0,-1],[-1,-1],[-1,-1],[-2,0],[-3,3],[-10,10],[-2,2],[-1,4],[0,1],[-1,3],[0,1],[0,3],[1,3],[1,3],[0,1],[0,2],[-1,4],[0,6],[0,4],[0,12],[0,1],[0,2],[-1,1],[0,1],[-5,5],[-2,3],[0,1],[-1,2],[0,6],[0,2],[-1,1],[0,1],[-1,1],[-1,2],[-1,2],[-3,1],[-3,0],[-7,-1],[-5,-1],[-2,-1],[-1,-1],[-1,0],[-2,0],[-1,1],[-3,1],[-1,1],[-2,1],[-4,4],[-2,1],[-2,1],[-14,2],[-3,2],[-1,0],[-2,-1],[-3,-2],[-2,-1],[-1,-1],[-1,-3],[0,-3],[0,-3],[0,-2],[1,-1],[0,-3],[2,-2],[2,-3],[1,-1],[0,-2],[0,-1],[0,-2],[0,-1],[0,-1],[-1,-2],[-1,-1],[-1,-1],[-1,0],[-1,0],[-1,0],[-2,1],[-1,1],[-1,1],[-11,16],[-5,11],[-1,8],[-10,28],[-3,15],[0,4],[0,4],[0,2],[1,3],[1,4],[3,5],[4,6]],[[5057,2372],[3,1],[4,-4],[8,-12],[2,-2],[2,-1],[3,-1],[2,-1],[2,-1],[3,-6],[2,-2],[3,4],[3,5],[3,4],[3,1],[4,1],[2,2],[2,3],[4,13],[2,2],[7,2],[7,3],[4,3],[1,0],[1,1],[1,1],[1,3],[1,7],[2,4],[1,7],[0,2],[0,2],[3,4],[6,11],[1,2],[2,1],[3,2],[2,1],[2,1],[2,6],[2,2],[3,-1],[4,-4],[3,-5],[3,-9],[4,-1],[4,1],[3,1],[4,3],[4,1],[2,-2],[3,-2],[3,-2],[2,-3],[2,-15],[4,-8],[4,-5],[6,-2],[6,1],[2,-1],[3,-2],[4,-5],[3,0],[4,2],[8,-10],[6,-3],[4,-7],[3,-2],[3,1],[5,5],[3,1],[4,0],[3,-1],[3,-3],[11,-18],[3,-2],[3,3],[4,10],[4,6],[10,12],[5,6],[3,13],[0,12],[-3,12],[-4,10],[2,1],[2,0],[1,-1],[2,0],[1,1],[1,2],[1,2],[2,1],[13,0],[10,6],[2,2],[12,6],[8,9],[2,2],[12,7],[5,6],[4,7],[4,5],[6,2],[13,0],[17,5],[1,2],[1,3],[1,6],[0,2],[3,2],[3,0],[3,-1],[1,-4],[1,1],[1,0],[1,1],[1,1],[0,1],[1,1],[1,7],[0,1],[1,2],[0,3],[1,10],[0,2],[0,1],[-1,1],[0,1],[-2,3],[-3,7],[-1,3],[0,4],[0,2],[0,2],[1,1],[1,1],[1,1],[1,1],[1,1],[0,1],[1,5],[1,2],[1,2],[1,2],[1,0],[2,2],[2,1],[2,2],[1,1],[2,0],[6,-1],[6,1],[5,2],[2,3],[3,3],[1,1],[2,-1],[4,-4],[6,-11],[4,-7],[0,-1],[0,-1],[-1,-3],[-1,-2],[0,-2],[-1,-1],[0,-2],[1,-1],[0,-2],[1,-1],[1,-1],[1,0],[5,-2],[2,-1],[1,-1],[2,-1],[0,-1],[1,-1],[0,-1],[1,-2],[0,-3],[0,-5],[0,-3],[1,-3],[1,-3],[1,-1],[0,-1],[1,-1],[2,-2],[4,-1],[1,-1],[1,-1],[0,-2],[0,-1],[-1,-1],[-1,-3],[0,-1],[-1,-1],[0,-2],[0,-1],[0,-2],[1,-1],[1,0],[1,-1],[1,0],[2,-1],[1,0],[1,1],[3,2],[1,2],[3,5],[0,1],[2,1],[2,1],[1,1],[3,0],[1,1],[0,1],[1,4],[0,2],[1,1],[1,1],[1,2],[1,0],[1,0],[0,-1],[0,-2],[0,-1],[-1,-5],[0,-1],[0,-2],[0,-1],[1,-1],[0,-2],[1,-1],[1,0],[3,-3],[0,-2],[0,-1],[-1,-2],[0,-2],[1,-1],[1,-1],[1,0],[1,1],[1,1],[1,1],[0,1],[1,0],[1,1],[2,1],[1,1],[1,-1],[1,-1],[1,-3],[1,-2],[0,-2],[0,-2],[1,-7],[1,-1],[0,-1],[2,-1],[3,-1],[9,0],[2,1],[1,0],[1,2],[7,9],[1,2],[3,2],[2,1],[15,0],[2,1],[3,2],[2,2],[2,2],[1,2],[1,2],[0,2],[1,3],[0,19],[0,9],[0,1],[0,2],[1,1],[0,1],[1,1],[4,3],[1,1],[3,4],[1,2],[3,9],[1,2],[1,0],[1,1],[4,3],[3,2],[0,1],[1,1],[0,1],[1,1],[0,3],[1,3],[3,0],[2,-3],[2,-2],[2,-1],[1,0],[3,0],[5,0],[1,-1],[2,-1],[1,-2],[2,-4],[0,-1],[9,-9],[1,-2],[2,-6],[2,-5],[2,-5],[6,-14],[4,-4],[0,8],[0,6],[0,2],[0,1],[-1,2],[-1,2],[-3,5],[0,2],[-1,2],[0,3],[1,3],[0,3],[1,2],[2,2],[2,2],[4,3],[4,2],[1,1],[0,2],[-1,2],[-2,3],[0,2],[1,3],[1,4],[2,3],[2,4],[3,3],[5,3],[4,0],[2,-1],[1,-3],[1,-3],[0,-4],[0,-3],[-1,-5],[0,-2],[0,-2],[0,-1],[1,0],[1,-1],[2,1],[10,4],[12,3],[2,0],[1,0],[6,-9],[2,-1],[2,1],[2,2],[2,2],[2,3],[0,3],[0,3],[-2,5],[-1,3],[0,4],[1,11],[1,9],[1,6],[0,2],[0,1],[-1,0],[-1,0],[-8,-4],[-5,-4],[-8,-9],[-2,-1],[-2,0],[-1,0],[-1,2],[0,2],[-1,2],[0,2],[1,2],[0,2],[0,1],[1,2],[9,14],[5,4],[5,2],[5,1],[11,5],[2,-1],[1,-2],[1,-3],[0,-5],[2,-2],[1,-2],[2,-1],[1,1],[2,2],[1,3],[0,4],[0,3],[0,2],[0,2],[2,0],[1,-2],[1,-2],[5,-11],[2,-4],[2,-2],[2,-1],[2,0],[2,1],[2,1],[1,2],[1,7],[1,4],[4,11],[2,9],[1,4],[5,7],[2,3],[1,6],[-1,3],[-2,4],[0,2],[0,3],[2,5],[10,10]],[[5861,2715],[2,2],[2,1],[1,0],[1,0],[5,-2],[2,-1],[2,0],[3,0],[2,1],[3,3],[1,0],[1,-2],[2,-2],[5,-10],[1,0],[1,-1],[1,-3],[0,-6],[0,-3],[-2,-6],[-1,-3],[0,-3],[1,-2],[1,-1],[2,-1],[6,0],[7,3],[2,2],[2,2],[1,3],[0,4],[-1,4],[-1,9],[-1,2],[0,1],[0,1],[1,2],[1,2],[2,3],[2,1],[5,2],[2,3],[2,6],[7,22],[1,5],[3,4],[5,0],[1,-1],[2,-1],[1,-1],[3,-4],[1,-1],[6,-1],[2,-1],[1,-2],[0,-3],[-1,-4],[-3,-8],[-2,-5],[0,-5],[0,-3],[1,-2],[3,0],[5,0],[9,4],[5,-1],[2,-3],[2,-3],[2,-4],[2,-2],[1,-5],[2,-2],[2,0],[6,6],[1,1],[2,5],[6,24],[1,2],[1,2],[5,6],[2,2],[3,1],[3,2],[4,-1],[3,1],[2,0],[1,2],[7,13],[1,2],[1,3],[0,1],[-1,4],[0,2],[1,2],[1,4],[1,2],[-1,2],[0,4],[0,2],[0,2],[2,2],[2,1],[10,-3],[6,-1],[2,-1],[2,-2],[0,-2],[0,-5],[0,-3],[1,-2],[1,-1],[2,0],[25,0],[3,2],[2,3],[1,6],[2,12],[1,3],[1,2],[1,2],[3,-1],[1,-2],[5,-6],[1,-1],[5,-1],[0,-1],[14,-10],[2,-1],[5,-1],[4,-2],[3,-2],[1,0],[4,0],[1,0],[1,0],[0,-1],[3,-5],[1,-1],[0,-1],[0,-1],[0,-2],[-1,-3],[-2,-4],[0,-1],[-2,-21],[-2,-18],[-1,-6],[-4,-12],[-1,-6],[0,-2],[0,-2],[0,-4],[1,-2],[0,-1],[4,-10],[3,-4],[2,-3],[2,-1],[2,-1],[1,0],[1,0],[1,1],[0,1],[0,2],[0,3],[1,3],[0,2],[1,1],[1,0],[1,0],[2,-1],[1,0],[5,0],[6,1],[6,5],[1,0],[1,0],[0,-2],[-1,-2],[0,-2],[0,-1],[4,-1],[-1,-2],[-8,-9],[-1,-2],[0,-2],[0,-4],[0,-3],[-1,-3],[-2,-5],[-2,-4],[-2,-6],[-1,-2],[-1,-1],[-1,-1],[-1,0],[-3,0],[-1,0],[-1,-2],[0,-1],[0,-8],[0,-2],[0,-2],[-1,-2],[0,-2],[-1,-2],[0,-1],[0,-2],[3,-10],[0,-2],[0,-1],[-1,-12],[0,-2],[0,-2],[0,-1],[1,-2],[-1,-2],[0,-2],[-2,-4],[-2,-5],[-1,-1],[0,-2],[0,-4],[0,-2],[-1,-2],[-4,-8],[-1,-1],[-1,0],[-1,0],[-3,-1],[-3,-1],[-2,-1],[-2,-1],[-1,-2],[-1,-2],[-1,-2],[0,-3],[0,-5],[-1,-3],[-1,-3],[-1,-1],[0,-1],[-4,-3],[0,-1],[-1,-1],[0,-3],[1,-4],[0,-1],[0,-1],[0,-1],[-2,-5],[-1,-2],[-5,-8],[-1,-1],[-2,-2],[-1,-1],[-5,-1],[-1,0],[-1,-1],[0,-1],[-1,-2],[-3,-15],[-3,-8],[0,-3],[0,-2],[0,-5],[0,-2],[0,-1],[1,-4],[5,-9],[2,-8],[1,-1],[0,-1],[2,-1],[1,-1],[2,0],[1,1],[5,6],[3,8],[3,14],[1,3],[1,1],[0,1],[3,1],[4,4],[13,18],[2,3],[2,1],[1,-1],[1,-1],[4,-6],[1,-2],[1,0],[2,1],[6,5],[1,0],[2,0],[1,0],[3,-3],[1,-1],[0,-1],[1,-1],[0,-1],[0,-2],[0,-1],[-1,-2],[-1,-5],[0,-1],[0,-2],[0,-1],[1,-1],[4,-4],[2,-2],[0,-1],[1,-1],[0,-2],[-1,-1],[0,-2],[-1,-2],[-1,-3],[-1,-2],[-1,-3],[-1,-2],[0,-1],[0,-1],[0,-1],[2,-7],[1,-3],[1,-6],[0,-2],[-1,-1],[-1,-4],[0,-1],[0,-1],[1,-1],[4,-7],[0,-1],[0,-2],[1,-1],[0,-2],[0,-1],[-3,-12],[-5,-13],[0,-2],[0,-1],[0,-2],[1,-1],[1,0],[4,-1],[2,-2],[2,-1],[2,-4],[2,-1],[1,0],[1,2],[1,1],[6,12],[1,1],[1,0],[1,0],[3,0],[1,0],[1,1],[0,1],[1,1],[0,5],[1,5],[1,1],[0,1],[1,2],[2,1],[1,2],[1,0],[21,-5],[5,0],[5,1],[6,0],[1,0],[1,1],[1,0],[1,1],[1,1],[2,0],[1,0],[6,-3],[1,-1],[1,-1],[1,-1],[0,-2],[0,-2]],[[5828,3368],[0,-4],[-1,-4],[0,-2],[0,-2],[1,-3],[2,-11],[1,-2],[0,-2],[0,-5],[-1,-4],[-1,-6],[0,-2],[0,-2],[1,-2],[1,-1],[2,-1],[7,-5],[2,-2],[1,-2],[1,-2],[1,-2],[0,-2],[0,-5],[-1,-3],[0,-3],[-4,-11],[-5,-19],[0,-3],[-1,-3],[0,-2],[1,-2],[0,-1],[1,-2],[1,0],[1,1],[1,0],[1,2],[1,1],[1,1],[1,0],[1,0],[1,-1],[1,-1],[0,-1],[0,-2],[0,-2],[-1,-2],[0,-1],[-1,-1],[-5,-6],[-1,-2],[-1,-3],[0,-1],[0,-2],[0,-1],[0,-2],[2,-2],[2,-1],[0,-1],[1,-1],[0,-1],[-1,-1],[-2,-3],[-1,-1],[0,-1],[-1,-1],[0,-3],[0,-3],[1,-3],[2,-7],[0,-4],[0,-2],[0,-3],[0,-2],[0,-2],[1,-2],[2,-3],[1,-2],[1,-3],[2,-2],[7,-4],[0,-6],[1,-4],[1,-2],[1,-1],[2,-5],[0,-3],[0,-2],[0,-4],[0,-2],[0,-7],[0,-1],[-1,-2],[0,-1],[-1,-2],[-1,-3],[-1,-2],[-17,-16],[-5,-14],[-2,-2],[0,-1],[-1,0],[-2,0],[-2,0],[-6,2],[-1,0],[-2,-2],[-2,-3],[-1,-1],[-1,-1],[-3,-1],[-2,-1],[0,-1],[-1,-2],[0,-3],[-2,-4],[-16,-29],[-2,-1],[-1,-1],[-3,-1],[-2,0],[-1,-1],[-10,-15],[-1,-5],[0,-6],[0,-2],[0,-2],[-1,-1],[-1,-1],[-2,-2],[-2,-2],[-1,-1],[-1,0],[-7,0],[-1,0],[-1,-1],[0,-1],[0,-1],[0,-2],[0,-1],[1,-2],[4,-2],[2,-3],[7,-11],[1,-1],[2,-1],[1,0],[7,4],[1,1],[1,1],[1,2],[1,4],[0,2],[1,2],[1,2],[2,2],[2,3],[3,2],[2,1],[10,7],[2,1],[1,0],[1,0],[0,-1],[0,-1],[0,-2],[0,-1],[-1,-2],[-1,-1],[0,-2],[0,-2],[2,-2],[1,-1],[1,-1],[2,0],[1,0],[1,1],[2,2],[1,1],[4,6],[2,5],[1,4],[0,1],[1,0],[1,0],[4,-5],[1,0],[1,0],[1,0],[4,3],[4,2],[2,0],[8,0],[2,-1],[2,-1],[2,-1],[3,-3],[1,-1],[0,-1],[1,-3],[1,-3],[0,-1],[2,-2],[2,-2],[3,-5],[0,-1],[1,-1],[0,-2],[0,-4],[0,-3],[0,-5],[0,-3],[0,-1],[0,-3],[-1,-3],[0,-3],[-1,-2],[-1,-1],[-2,-3],[-2,-2],[-1,-1],[0,-1],[-1,-2],[-1,-2],[0,-2],[1,-2],[1,-2],[1,0],[2,1],[1,1],[1,-1],[1,0],[1,-1],[3,-5],[0,-1],[0,-2],[-1,-3],[0,-2],[-1,-1],[-7,-5],[-11,-13],[-2,-2],[-1,-1],[-1,0],[-5,0],[-1,-1],[-3,-2],[-2,-1],[-2,0],[-1,-1],[-1,-2],[0,-2],[0,-2],[2,-3],[1,-2],[0,-1],[3,-1],[1,-1],[3,-1],[1,0],[2,-2],[2,-2],[1,-2],[5,-13],[0,-2],[0,-3],[0,-6],[-1,-3],[0,-2],[-3,-7],[-1,-3],[-4,-5],[-4,-9],[-1,-3],[-1,-2],[-4,-18],[0,-2],[0,-2],[0,-2],[1,-6],[0,-2],[1,-2],[2,-3],[1,-1],[1,-1],[11,-2],[1,-1],[1,-1],[-1,-2],[-2,-4],[0,-1],[-1,-1],[0,-2],[0,-2],[1,-2],[1,-2],[2,-3],[0,-1],[1,-2],[0,-3],[0,-3],[1,-4],[1,-3],[0,-4],[0,-5],[1,-4],[4,-12]],[[5057,2372],[0,1],[1,18],[1,5],[3,5],[5,10],[1,2],[1,1],[2,4],[5,11],[2,3],[1,1],[1,1],[3,6],[1,3],[0,2],[0,3],[0,3],[-2,6],[-1,3],[0,4],[1,3],[0,2],[0,2],[4,9],[0,3],[0,2],[0,2],[0,3],[0,3],[1,3],[0,2],[1,1],[5,0],[1,0],[1,1],[1,1],[2,2],[0,2],[0,2],[-1,1],[0,1],[-1,1],[-4,1],[-6,2],[-4,2],[-2,2],[-2,2],[-9,16],[-1,1],[-2,2],[-5,3],[-1,11],[-1,17],[-1,4],[-2,2],[-2,-1],[-4,-3],[-2,0],[-3,-1],[-5,2],[0,1],[0,2],[0,13],[-1,4],[-1,2],[-5,3],[-2,2],[-1,1],[-4,3],[-2,3],[-3,3],[-1,2],[-1,2],[0,2],[0,1],[0,4],[0,3],[0,5],[-1,10],[0,2],[0,2],[0,1],[1,0],[1,0],[1,0],[5,-4],[1,0],[1,0],[1,0],[1,1],[0,1],[1,2],[1,2],[1,2],[0,3],[4,14],[1,3],[0,5],[0,5],[-1,2],[0,2],[-1,1],[-1,1],[0,1],[-1,1],[-1,3],[0,2],[0,1],[4,3],[3,2],[1,0],[2,2],[2,2],[5,13],[1,4],[0,4],[0,3],[0,2],[2,4],[0,4],[0,2],[0,1],[-1,4],[-1,3],[0,2],[1,2],[0,1],[3,5],[1,2],[0,1],[1,9],[0,6],[1,3],[0,2],[3,5],[8,10],[4,3],[1,2],[1,1],[1,2],[1,3],[1,1],[1,1],[0,1],[1,0],[-1,1],[-1,3],[-1,2],[-1,7],[0,1],[-2,2],[0,1],[-1,1],[0,2],[-1,2],[0,5],[0,2],[-1,2],[-1,1],[-3,6],[-3,4],[-1,1],[0,2],[0,3],[0,2],[-1,1],[0,1],[-1,1],[-1,1],[-1,1],[-1,0],[-2,1],[-2,0],[-1,0],[-2,-1],[-1,0],[-1,0],[-1,1],[-2,2],[-1,3],[0,1],[0,3],[0,7],[-1,1],[0,2],[-4,11],[0,1],[-1,1],[-1,0],[-1,1],[-2,0],[-1,0],[-1,-1],[0,-1],[-1,-2],[0,-5],[0,-1],[-1,-1],[-4,-8],[-1,-1],[-1,-1],[-1,-1],[-1,-1],[-1,0],[-1,-1],[-14,-1],[-1,1],[-1,0],[-9,5],[-1,1],[-1,0],[-1,0],[-1,0],[-1,0],[-2,-2],[-1,0],[-1,-2],[-5,-9],[-1,-1],[-3,-10],[-1,-2],[0,-5],[-1,-1],[0,-1],[-2,-2],[-1,-1],[-1,0],[-4,1],[-13,0],[-1,0],[-4,2],[-4,2],[-5,4],[-7,8],[-1,1],[-1,2],[-2,7],[0,2],[-1,2],[0,2],[1,6],[0,7],[0,2],[2,10],[0,1],[0,1],[-1,0],[-3,-1],[-1,0],[-1,0],[-1,1],[0,2],[0,3],[-1,5],[-1,5],[-1,5],[0,3],[1,1],[0,1],[1,1],[0,2],[1,3],[0,1],[1,0],[2,1],[4,2],[0,1],[1,1],[-1,2],[0,3],[-1,3],[1,2],[0,1],[1,4],[0,2],[0,1],[0,2],[-1,2],[-1,1],[-2,2],[0,2],[0,3],[-1,2],[0,1],[-2,3],[-1,3],[-1,2],[-2,3],[-1,1],[-1,1],[-1,-1],[-2,0],[-1,-3],[-3,-3],[-3,-2],[-5,8],[-2,4],[0,1],[0,2],[0,2],[1,1],[1,1],[2,3],[6,6],[6,7],[17,27],[3,5],[7,11],[6,16],[1,2],[1,1],[0,1],[2,1],[3,1],[2,1],[3,0],[1,0],[1,-1],[1,0],[1,-2],[1,-1],[2,-4],[3,-3],[3,-4],[3,-2],[4,-3],[5,-4],[3,-4],[2,-3],[2,-3],[1,-1],[2,1],[3,1],[6,5],[1,1],[0,2],[1,1],[0,2],[-1,8],[1,3],[2,3],[8,9],[2,4],[1,2],[2,3],[2,2],[1,0],[1,-1],[0,-1],[1,-1],[5,-11],[6,-12],[2,-2],[3,-3],[6,-3],[1,0],[1,0],[4,1],[8,-2],[4,0],[12,4],[2,0],[1,-1],[2,-1],[1,-1],[2,0],[2,1],[3,2],[1,0],[1,1],[1,0],[1,-1],[1,0],[2,-2],[3,-3],[2,-1],[2,0],[1,0],[1,1],[6,5],[1,1],[2,2],[1,0],[10,6],[1,1],[4,6],[1,1],[4,3],[3,1],[2,1],[7,-1],[6,-4],[2,0],[2,0],[1,0],[1,1],[0,2],[0,4],[0,2],[1,1],[1,3],[1,1],[0,1],[0,2],[1,5],[-1,3],[1,2],[0,1],[3,5],[0,2],[0,1],[1,3],[0,11],[0,1],[0,1],[2,5],[1,1],[2,6],[1,6],[1,12],[1,5]],[[5181,3161],[3,0],[2,2],[5,8],[2,1],[4,1],[10,3],[4,1],[5,-1],[1,-1],[1,0],[5,-8],[1,-1],[1,-1],[4,-2],[1,-1],[1,-1],[1,-2],[2,-6],[1,-2],[0,-1],[2,-1],[1,0],[3,2],[4,0],[2,1],[2,1],[1,3],[1,1],[1,1],[1,1],[3,1],[2,0],[8,-1],[3,0],[1,1],[1,1],[1,2],[4,4],[4,2],[2,0],[1,0],[1,-1],[2,0],[11,1],[5,1],[1,1],[2,2],[2,3],[3,1],[1,1],[2,0],[4,-1],[8,-4],[5,0],[3,0],[1,1],[2,1],[7,5],[2,3],[5,4],[1,2],[1,2],[0,2],[-1,6],[-1,5],[-3,8],[0,3],[-1,3],[0,11],[-1,3],[0,2],[-2,3],[-3,4],[-9,8],[-1,1],[0,2],[-1,7],[-1,4],[1,2],[0,1],[-1,1],[-3,5],[-1,1],[-1,2],[-5,4],[-2,1],[-2,0],[-2,0],[-1,0],[-6,-4],[-3,-2],[-2,-1],[-1,-1],[-2,-1],[-1,0],[-1,0],[-6,4],[-2,1],[-2,-1],[-2,-1],[-3,0],[-2,1],[-1,2],[-2,3],[0,5],[0,4],[0,4],[1,4],[0,2],[-1,2],[-1,2],[-3,2],[-2,1],[-4,2],[-2,1],[-5,5],[-1,1],[-2,0],[-4,-1],[-3,0],[-1,1],[-2,1],[-4,2],[-1,2],[-1,2],[0,2],[1,5],[0,3],[0,19],[-1,3],[-1,2],[-3,4],[-1,2],[-1,2],[1,2],[1,2],[4,3],[1,2],[1,2],[0,3],[1,5],[1,3],[0,1],[1,2],[4,4],[2,1],[2,0],[7,-4],[2,0],[7,4],[2,0],[4,0],[2,0],[2,1],[2,4],[1,4],[1,10],[1,5],[2,3],[1,2],[2,5],[2,5],[2,1],[1,0],[16,-23],[7,-8],[4,-2],[1,-2],[3,-7],[1,-2],[2,-1],[13,-8],[1,-1],[2,-2],[1,-2],[1,-2],[0,-3],[0,-2],[0,-3],[0,-2],[0,-2],[1,-2],[1,-1],[2,0],[2,2],[6,12],[1,4],[0,4],[0,5]],[[5358,3386],[4,-2],[8,-6],[3,-2],[3,1],[0,4],[-1,9],[0,4],[1,4],[3,5],[1,3],[1,3],[0,3],[0,3],[0,3],[-1,4],[-6,15],[-2,4],[-1,5],[0,5],[1,3],[2,1],[2,-1],[1,-1],[1,-2],[1,-2],[2,-3],[7,-9],[1,-2],[1,-3],[4,-13],[4,-11],[0,-2],[0,-3],[-1,-4],[-1,-2],[-3,-10],[-1,-2],[-5,-10],[-1,-1],[0,-2],[1,-1],[1,-2],[2,1],[5,2],[4,0],[2,-1],[2,-1],[2,-3],[0,-1],[4,-4],[3,-1],[3,1],[2,3],[1,1],[0,1],[1,6],[0,10],[0,4],[1,4],[2,0],[3,-1],[2,0],[2,2],[2,3],[2,4],[1,4],[0,2],[0,3],[1,3],[-1,3],[0,4],[-2,9],[1,6],[2,6],[6,7],[4,3],[2,0],[2,-3],[1,-2],[3,-1],[2,4],[2,5],[2,10],[1,2],[2,3],[2,2],[3,0],[5,0],[5,3],[2,1],[2,-1],[5,-5],[5,-7],[4,-6],[2,-3],[3,-3],[3,-1],[3,0],[2,1],[23,18],[1,2],[1,3],[1,2],[1,3],[0,10],[1,2],[1,1],[3,5],[1,2],[0,1],[0,1],[0,2],[0,1],[-1,2],[-1,1],[-1,2],[-7,6],[-1,2],[-1,3],[0,2],[-1,6],[-1,3],[0,7],[1,6],[2,6],[1,3],[1,3],[2,5],[2,2],[1,2],[2,1],[1,1],[1,0],[1,-2],[1,-2],[3,-7],[1,-2],[2,-1],[1,-2],[4,-2],[9,-3],[4,-4],[2,-2],[2,0],[1,0],[10,2],[2,0],[2,-2],[1,-2],[1,-3],[0,-3],[-1,-14],[1,-3],[0,-3],[2,-4],[2,-3],[3,-5],[1,-2],[2,-1],[2,0],[2,1],[4,2],[2,1],[1,2],[12,10],[11,6],[18,-1],[20,4],[1,0],[0,-1],[0,-7],[1,-11],[6,-22],[1,-2],[-1,-12],[1,-2],[1,-5],[2,-3],[1,-4],[1,-6],[-1,-4],[-6,-6],[-2,-4],[0,-4],[1,-5],[1,-4],[2,-2],[3,-1],[7,-6],[13,-4],[3,-1],[1,0],[1,0],[2,2],[2,0],[1,-1],[1,-2],[1,-3],[2,-2],[1,-2],[1,-2],[0,-1],[0,-2],[-1,-24],[1,-7],[0,-12],[-1,-8],[0,-3],[1,-3],[2,-6],[1,-2],[1,-2],[2,-1],[2,-1],[2,-1],[2,0],[1,0],[1,1],[1,0],[1,1],[0,2],[0,1],[0,1],[0,2],[-2,3],[0,1],[-1,2],[1,1],[0,3],[0,2],[0,1],[0,2],[0,1],[-1,1],[-3,6],[0,2],[-1,1],[0,2],[1,1],[0,1],[1,1],[1,0],[5,3],[1,1],[2,0],[1,-1],[2,-1],[1,-1],[1,-2],[1,-2],[1,-2],[5,-4],[0,-1],[1,-2],[0,-2],[-1,-3],[-1,-4],[-3,-16],[-1,-6],[-3,-18],[0,-1],[0,-2],[0,-3],[1,-1],[1,0],[1,0],[1,1],[1,2],[1,1],[1,0],[1,1],[2,0],[2,-1],[6,-5],[1,0],[1,0],[1,0],[1,0],[2,1],[2,0],[2,0],[1,0],[1,0],[5,-4],[1,0],[2,-1],[2,1],[3,-1],[1,1],[1,0],[2,3],[1,0],[1,0],[6,-5],[1,0],[1,0],[1,0],[1,1],[0,1],[1,2],[0,1],[0,2],[0,1],[-2,7],[0,2],[0,1],[1,2],[0,1],[3,3],[1,2],[1,1],[0,3],[6,34],[1,4],[1,2],[0,1],[6,6],[3,3],[5,3]],[[5883,4223],[8,-3],[3,-2],[2,-4],[5,-8],[4,-10],[0,-8],[0,-6],[1,-6],[1,-1],[3,-1],[13,-3],[15,-6],[6,-4],[0,-2],[-1,-1],[0,-2],[1,-2],[6,-5],[3,-2],[5,-5],[3,-2],[3,-3],[3,-4],[7,-17],[0,-7],[-1,-6],[0,-5],[0,-4],[0,-1],[0,-2],[5,-8],[0,-3],[0,-2],[-1,-2],[-1,-2],[-4,-5],[-3,-4],[-1,-2],[-1,-3],[0,-3],[0,-3],[1,-6],[3,-13],[1,-2],[0,-3],[0,-2],[-1,-11],[0,-7],[0,-7],[-2,-6],[-2,-5],[-3,-6],[-2,-2],[-2,-1],[-2,-2],[-5,0],[-2,0],[-2,1],[-1,1],[-4,3],[-1,1],[-1,3],[0,3],[-1,2],[-2,2],[-1,1],[-2,1],[-2,1],[-3,-1],[-2,-1],[-20,-14],[-3,-3],[-11,-14],[-1,-3],[-1,-2],[-1,-3],[0,-3],[-1,-3],[-2,-2],[-5,-4],[-2,-2],[-2,-3],[-2,-4],[-1,-2],[-2,-1],[-3,3],[-2,0],[-2,-1],[-1,-1],[-10,-13],[-3,-3],[-3,-2],[-3,-1],[-3,-1],[-6,1],[-3,2],[-1,2],[-2,1],[-3,1],[-3,-2],[-12,-8],[-2,-1],[-2,0],[-3,1],[-2,1],[-2,2],[-1,2],[-1,2],[-2,1],[-2,1],[-5,1],[-13,1],[-2,-1],[-4,-2],[-2,-2],[-2,-2],[-2,-4],[-1,-3],[-2,-1],[-7,0],[-3,-1],[-4,-2],[-4,-3],[-2,-2],[-3,0],[-3,1],[-2,1],[-2,3],[-1,5],[-1,3],[-1,1],[-1,1],[-2,-1],[-1,-2],[-6,-13],[-1,-2],[-2,-1],[-3,-2],[-22,-3],[-2,-1],[-2,-3],[-1,-3],[-1,-2],[-3,-14],[0,-3],[1,-3],[2,-2],[2,-2],[13,-7],[4,-2],[1,-1],[2,-2],[1,-4],[1,-5],[0,-5],[-1,-3],[-2,-16],[0,-8],[0,-13],[4,-27],[0,-3],[-1,-2],[0,-2],[-2,-8],[-1,-3],[0,-3],[1,-6],[1,-6],[-1,-3],[0,-2],[-2,-2],[-1,-1],[-2,-1],[-10,-1],[-4,-1],[-2,-1],[-1,-2],[0,-3],[3,-5],[2,-5],[1,-3],[0,-6],[1,-2],[1,-2],[1,-1],[3,-2],[2,-1],[2,0],[1,0],[2,1],[1,1],[1,2],[1,3],[1,10],[1,2],[1,2],[3,4],[1,2],[3,6],[1,2],[2,2],[2,-1],[2,-2],[3,-8],[1,-5],[1,-8],[2,-11],[1,-6],[1,-2],[2,-1],[2,-1],[2,-2],[3,-2],[7,-2],[5,-4],[6,-2],[2,-1],[2,-2],[1,-3],[1,-3],[0,-4],[0,-2],[0,-2],[-4,-21],[0,-3],[1,-4],[1,-4],[4,-6],[2,-4],[1,-5],[-1,-3],[-1,-5],[0,-3],[1,-3],[3,-3],[7,-4],[1,-1],[3,-1],[1,-2],[2,-2],[5,-14],[4,-9],[21,-29]],[[5820,3526],[11,-14],[2,-5],[-1,-1],[-1,-3],[-1,-2],[-4,-20],[-1,-13],[2,-26],[1,-6],[1,-3],[1,-1],[1,-2],[1,-3],[0,-2],[-1,-2],[-1,-2],[-1,-2],[-1,-2],[1,-3],[0,-2],[0,-2],[-1,-1],[-1,-2],[-4,-2],[-1,-1],[0,-1],[-1,-2],[0,-2],[1,-1],[1,-2],[2,-1],[3,-1],[2,-1],[2,-2],[1,-2],[0,-2],[0,-1],[-3,-9],[-1,-3],[-1,-6]],[[5358,3386],[-7,11],[-6,15],[-3,8],[-2,8],[-2,16],[0,5],[1,8],[0,4],[-1,3],[-2,4],[-3,3],[-8,5],[-4,1],[-3,0],[-2,0],[-3,1],[-4,3],[-9,9],[-3,2],[-1,0],[-1,0],[-2,-1],[-17,-13],[-1,-1],[-3,0],[-3,0],[-3,2],[-4,3],[-4,6],[-4,9],[0,2],[-1,3],[0,5],[0,3],[0,4],[0,4],[0,3],[-2,9],[0,3],[1,4],[0,3],[0,2],[-1,3],[-2,3],[-2,5],[0,5],[1,18],[-1,2],[-1,1],[-3,-1],[-1,-2],[-2,-5],[-1,-1],[-1,0],[-6,4],[-3,1],[-6,0],[-8,1],[-8,6],[-6,5],[-1,1],[0,2],[-1,3],[1,12],[-1,5],[0,3],[-2,1],[-4,4],[-1,1],[-1,1],[-2,0],[-3,1],[-1,1],[-1,1],[-1,3],[0,1],[-2,4],[-1,3],[-1,11],[0,5],[1,3],[2,6],[2,3],[1,3],[4,4],[3,4],[4,9],[2,2],[2,1],[2,0],[3,-2],[2,1],[1,1],[2,5],[1,1],[2,1],[7,0],[1,0],[1,2],[0,3],[1,11],[0,3],[1,2],[1,2],[2,2],[12,9],[2,0],[2,1],[3,0],[2,1],[1,2],[1,4],[0,3],[-2,8],[1,4],[0,3],[3,8],[2,4],[1,2],[0,3],[-2,3],[-10,10],[-9,10],[-7,5],[-2,2],[-2,3],[0,3],[0,3],[0,2],[3,3],[8,4],[2,1],[2,2],[1,2],[0,2],[-1,2],[-2,3],[-1,2],[0,3],[0,2],[2,1],[2,1],[9,-2],[1,-1],[1,1],[1,1],[1,2],[0,3],[-1,5],[0,4],[11,24],[1,3],[3,1],[3,0],[3,-2],[4,-5],[3,-3],[1,-1],[12,-2],[4,-1],[20,-11],[5,-5],[2,-2],[2,-7],[2,-7],[1,-1],[1,-1],[2,-2],[1,-1],[1,-2],[1,-2],[1,-2],[2,-1],[2,3],[3,5],[2,2],[2,2],[13,0],[2,1],[2,1],[0,2],[1,2],[0,3],[1,3],[1,2],[1,1],[2,1],[2,0],[3,-2],[3,-1],[1,0],[1,1],[4,6],[1,1],[1,-1],[0,-1],[1,-3],[1,-2],[2,-2],[3,-3],[3,-3],[2,-5],[1,-7],[2,-11],[3,-8],[2,-7],[3,-2],[1,-11],[2,-5],[2,-4],[2,-2],[2,-2],[2,-1],[3,-1],[3,0],[3,1],[4,3],[3,1],[6,-1],[12,3],[3,0],[3,-2],[3,0],[3,1],[2,2],[2,3],[1,3],[1,3],[2,3],[3,2],[2,2],[2,4],[4,6],[2,5],[3,7],[2,2],[6,5],[6,24],[17,46],[2,4],[2,2],[1,0],[2,1],[10,22],[2,6],[0,6],[-2,5],[-3,9],[-1,3],[0,3],[0,4],[1,4],[1,3],[2,5],[5,8],[4,3],[5,2],[10,-1],[3,0],[2,1],[2,1],[2,2],[2,3],[3,4],[1,2],[1,1],[1,-1],[2,-1],[1,-3],[4,-12],[2,-3],[1,-3],[2,-1],[2,-1],[2,1],[2,1],[2,2],[13,16],[2,4],[1,3],[1,5],[0,2],[-2,4],[-1,1],[0,1],[0,2],[1,3],[2,5],[7,10],[1,1],[1,3],[5,30],[1,9],[-1,5],[-1,3],[0,2],[1,2],[3,3],[14,11],[5,6],[2,3],[1,3],[0,4],[0,3],[0,3],[0,3],[1,3],[2,6],[1,5],[1,5],[0,5],[2,3],[1,1],[10,2],[4,2],[0,1],[4,4],[7,9],[2,4],[2,5],[2,6],[1,6],[1,3],[1,3],[1,1],[1,1],[1,2],[2,1],[4,6],[2,3],[1,3],[0,5],[-1,8],[-1,4],[-1,2],[-4,5],[-9,11],[-22,19],[-3,3],[-1,2],[-1,2],[0,1],[1,3],[0,2],[9,17],[4,10],[2,1],[1,1],[6,0],[2,1],[1,1],[0,3],[-1,2],[-1,3],[-6,8],[-1,2],[0,3],[2,2],[2,2],[21,4]],[[5707,4345],[11,-8],[5,-4],[8,-11],[3,-2],[3,-1],[3,-1],[4,-2],[2,-1],[0,-1],[1,-2],[3,-5],[2,-3],[11,-7],[5,-4],[4,-2],[6,-3],[17,-11],[10,-9],[10,-14],[2,-4],[10,-25],[5,-7],[4,-2],[5,-2],[39,7],[3,2]],[[7152,6477],[2,-10],[1,-5],[-1,-2],[0,-1],[-1,-2],[0,-2],[-4,-4],[0,-1],[-1,-2],[-2,-6],[-1,-2],[0,-2],[-1,-1],[-1,0],[-2,-2],[-8,-1]],[[7133,6434],[-29,-8],[-9,-7],[-2,-1],[-1,0],[-26,1],[-6,-2],[-3,-1],[-2,-2],[0,-2],[-1,-3],[-2,-8],[0,-1],[0,-2],[0,-1],[1,-3],[1,-4],[3,-5],[1,-2],[0,-3],[1,-1],[1,-1],[2,0],[1,-1],[1,-1],[1,-2],[1,-2],[2,-3],[1,0],[0,-1],[3,-1],[1,-1],[1,-1],[1,-1],[1,0],[1,2],[1,1],[1,1],[0,-1],[1,-1],[0,-1],[2,-9],[0,-1],[0,-3],[-1,-3],[-1,-2],[-2,-3],[0,-1],[-4,-8],[0,-2],[-1,-1],[0,-3],[0,-1],[1,-2]],[[7074,6326],[-4,-8],[-5,-10],[-2,-2],[-1,-1],[-2,-1]],[[7060,6304],[-19,0],[-8,2],[-2,0],[-2,0],[-6,-3],[-1,-1],[-1,-2],[-1,-1],[0,-1],[-1,-3],[-1,-2],[-1,-1],[-6,-1],[-8,-6],[-2,-1],[-1,-2],[0,-1],[0,-1],[0,-2],[1,-1],[4,-4],[1,-1],[0,-1],[1,-2],[-1,-1],[0,-2],[-1,-1],[-1,-2],[-3,-2],[-4,0],[-3,0],[-9,3],[-3,2],[-11,12],[-4,8],[-2,10],[-2,1],[-1,0],[-1,1],[-4,-1],[-9,-3],[-1,-1],[-4,1],[-1,0],[-9,-2],[-1,0],[-1,0],[-8,4],[-2,1],[-1,0],[-1,-1],[-2,-1],[-3,0],[-1,-1],[-1,-2],[-1,0],[-1,-1],[-1,0],[-2,-1],[-3,-1],[-1,-1],[-4,0],[-1,0],[-1,-1],[-1,-1],[-1,-4],[-1,-1],[0,-1],[-1,0],[-1,-1],[-1,0],[-2,1],[-1,1],[-3,4],[-2,2],[-2,1],[-2,2],[0,1],[-1,2],[-1,4],[0,2],[-1,1],[-1,2],[-1,1],[-6,1],[-1,0],[-1,0],[-3,-2],[-2,0],[-1,0],[-2,1],[-11,5],[-1,0],[-1,2],[-1,1],[-1,1],[0,3],[-5,28],[0,1],[0,2],[1,2],[1,2],[1,1],[1,1],[2,0],[1,0],[3,-1],[3,0],[2,0],[6,4],[0,1],[0,2],[-2,3],[-7,5],[-1,3],[0,2],[0,2],[0,12],[0,1],[0,1],[0,1],[-1,1],[-7,5],[-4,5],[-1,2],[0,1],[0,1],[0,1],[0,1],[1,2],[3,2],[1,0],[1,0],[2,2],[1,1],[1,2],[5,11],[1,1],[1,2],[3,3],[5,3],[40,15],[4,2],[2,2],[1,3],[3,9],[2,4],[2,3],[5,5],[1,4],[0,6],[-1,5],[-1,3],[-2,2],[-1,2],[-2,1],[-1,1],[-27,51],[0,1],[-1,2],[1,1],[1,1],[2,2],[5,2],[16,9],[2,1],[1,0],[3,0],[14,-4],[1,-1],[4,1],[2,1],[1,2],[8,9],[4,2],[1,1],[1,1],[12,25],[3,3],[2,1],[1,0],[1,-1],[1,0],[1,-1],[3,-4],[1,-1],[1,0],[1,-1],[1,0],[2,1],[1,1],[1,1],[1,2],[1,1],[1,1],[1,0],[4,1],[5,0],[5,-1],[2,-1],[2,1],[1,1],[1,1],[-1,2],[-2,3],[-16,30],[-1,2],[0,3],[0,1],[2,1],[6,-6],[3,-1],[3,-1],[3,1],[1,2],[1,1],[1,5],[1,3],[3,2],[15,10],[1,1],[0,2],[0,5],[0,3],[2,2],[1,2],[2,0],[1,0],[1,0],[2,-2],[0,-2],[0,-13],[1,-5],[1,-3],[1,-1],[1,-2],[3,-4],[5,-5],[5,-8],[1,-2],[1,-1],[11,-14],[5,-7],[7,-10],[8,-8],[16,-11],[3,-1],[2,1],[2,1],[3,2],[8,1],[1,0],[4,0],[17,-8],[21,2],[9,-2],[1,-3],[-9,-3],[-1,-3],[-1,-3],[0,-2],[-2,-3],[-2,-1],[-6,1],[-5,-1],[-8,-2],[-4,-2],[-2,-2],[-8,-10],[-1,-2],[-1,-4],[0,-2],[0,-1],[1,-2],[1,-3],[2,-3],[2,-3],[2,-1],[1,-1],[0,-1],[1,-2],[-1,-3],[0,-9],[0,-2],[0,-6],[1,-3],[1,-4],[2,-4],[1,-2],[3,-4],[7,-8],[4,-3]],[[7165,2110],[2,-2],[4,3],[5,-1],[4,-3],[-2,-6],[-3,1],[-2,-3],[-1,-5],[0,-7],[-2,-2],[-3,2],[-2,-1],[-2,-4],[-1,-5],[0,-5],[1,-1],[1,-5],[-9,-3],[-9,0],[3,6],[5,7],[4,15],[-1,2],[-2,1],[-2,0],[-1,0],[-1,-1],[-1,1],[0,2],[5,8],[0,5],[1,0],[-1,5],[2,2],[3,0],[1,0],[1,-5],[3,-1]],[[7280,2318],[4,-1],[3,1],[0,-4],[1,-5],[0,-2],[-1,-3],[-2,-1],[-1,-2],[0,-4],[-2,-5],[-2,-2],[-4,0],[-4,2],[-1,2],[-2,2],[0,2],[0,2],[2,1],[2,1],[1,1],[1,4],[-4,-3],[-1,0],[-1,4],[1,3],[2,7],[0,5],[4,-3],[4,-2]],[[7312,2321],[-1,-1],[-4,3],[-1,1],[1,1],[0,1],[3,1],[2,-2],[0,-2],[0,-2]],[[7441,2449],[0,-1],[-1,1],[0,1],[-1,1],[-1,1],[0,1],[1,2],[1,2],[0,1],[-1,1],[0,1],[0,1],[0,1],[1,1],[1,1],[2,1],[2,1],[1,2],[0,-1],[0,-1],[0,-1],[-2,-1],[-1,-2],[0,-3],[1,-1],[0,-1],[0,-1],[0,-1],[-3,-5],[0,-1]],[[7506,2496],[2,-1],[2,1],[1,0],[2,0],[1,-1],[-1,-2],[-2,-1],[-1,-2],[0,-4],[-1,-2],[-2,0],[-2,2],[-1,3],[1,2],[0,2],[-2,1],[-2,0],[0,-2],[-5,2],[-2,3],[-1,4],[1,2],[1,1],[1,0],[2,3],[1,0],[0,-2],[1,0],[1,-1],[1,-2],[1,-3],[1,-2],[2,-1]],[[7470,2552],[-1,0],[-1,6],[-1,3],[0,2],[0,5],[3,15],[1,1],[0,1],[1,1],[1,2],[0,1],[2,3],[2,0],[0,-2],[0,-1],[1,-2],[0,-3],[-1,-7],[-1,-7],[-1,-6],[1,-4],[0,-4],[-2,-1],[-2,-1],[-2,-2]],[[7548,2604],[2,-4],[2,2],[2,0],[1,-2],[2,-3],[-3,-3],[2,-2],[5,-2],[3,-2],[-2,-3],[-3,-3],[-2,4],[-2,-1],[-3,-4],[-1,-4],[2,-5],[5,-5],[1,-4],[-4,1],[-4,-1],[-4,-3],[-2,-5],[0,-3],[0,-3],[0,-1],[-3,0],[-3,2],[-3,3],[-2,3],[1,5],[0,1],[2,2],[0,1],[0,1],[0,2],[0,1],[0,1],[-2,-2],[-2,0],[-1,0],[-2,2],[2,3],[1,2],[2,0],[8,-2],[3,1],[2,5],[-3,2],[-5,-2],[-2,3],[0,3],[2,9],[0,3],[-2,0],[-1,-2],[-1,-2],[-2,5],[1,5],[2,4],[3,4],[3,1],[0,1],[3,2],[3,0],[-1,-2],[0,-4],[0,-5]],[[7538,2614],[0,-3],[0,-1],[-6,-1],[-1,0],[-1,1],[0,1],[0,2],[1,1],[1,-1],[6,3],[0,-2]],[[7517,2610],[-1,-1],[-1,-1],[-1,0],[0,1],[-1,0],[-1,2],[1,0],[0,1],[0,-1],[1,-1],[1,3],[2,1],[0,1],[1,1],[0,1],[2,1],[0,-3],[-2,-3],[-1,-2]],[[7485,2696],[-2,0],[-1,2],[-5,1],[-5,3],[-2,1],[-2,1],[-3,0],[-2,1],[-1,3],[-1,4],[-1,3],[-2,3],[-2,3],[-1,10],[2,-2],[4,-9],[2,-2],[2,-2],[6,-1],[10,-6],[5,-5],[1,-5],[-2,-3]],[[7525,2724],[-3,-2],[-9,4],[-2,4],[4,7],[4,2],[2,-2],[2,-3],[2,-5],[1,-3],[-1,-2]],[[7594,2844],[1,-2],[-1,-4],[-1,-1],[-1,0],[-2,-1],[-1,0],[0,2],[0,2],[1,1],[-1,0],[0,1],[1,1],[1,0],[2,0],[1,1]],[[7606,2852],[-1,0],[-2,3],[0,1],[3,3],[3,6],[1,0],[1,-1],[0,-2],[0,-1],[0,-1],[0,-2],[-1,-1],[-1,0],[-1,-1],[-2,-4]],[[7554,2870],[-1,-3],[-1,1],[-2,0],[1,0],[-1,-1],[-1,-1],[-1,-1],[-1,0],[-1,1],[-1,2],[1,2],[1,1],[1,0],[1,0],[2,1],[2,-1],[1,-1]],[[7543,2881],[-4,-4],[-1,-2],[-3,-1],[-2,0],[-1,1],[-1,1],[-1,-2],[-1,1],[-1,2],[0,3],[0,2],[1,0],[9,0],[1,-1],[1,-1],[0,3],[1,1],[1,1],[2,0],[0,-1],[0,-2],[-1,-1]],[[7575,2889],[0,-1],[-1,-2],[0,-1],[0,-1],[1,-2],[0,-1],[-2,1],[-1,0],[0,1],[0,2],[-2,0],[-1,2],[0,1],[2,0],[1,0],[2,1],[1,0]],[[7635,2951],[-2,0],[-2,1],[1,2],[2,0],[1,0],[0,-1],[1,0],[-1,-2]],[[7644,2960],[1,-2],[0,-3],[1,-1],[1,-1],[0,-1],[-5,-3],[-4,5],[-2,2],[1,3],[0,1],[1,0],[1,2],[2,0],[1,0],[0,-1],[1,0],[1,-1]],[[7653,3024],[0,-2],[1,-6],[-3,2],[-5,14],[-5,4],[-2,-1],[-3,-1],[-2,0],[-2,2],[-1,4],[2,3],[2,2],[1,1],[-1,1],[0,1],[0,-1],[-1,-1],[-1,2],[1,1],[1,1],[0,4],[-3,-3],[-1,-1],[-1,2],[-1,1],[-2,2],[1,-2],[0,-1],[0,-1],[1,-1],[-2,-1],[-2,1],[-2,1],[-2,1],[0,-2],[1,-1],[1,-2],[1,-1],[0,-1],[-3,-4],[-1,0],[-1,4],[-3,3],[0,-2],[0,-3],[0,-2],[0,-2],[0,-2],[-3,-4],[0,-3],[0,-2],[1,-1],[2,0],[1,0],[1,2],[0,2],[0,1],[5,4],[3,2],[1,-1],[1,-4],[5,-3],[1,-3],[2,2],[3,3],[0,-2],[1,0],[0,1],[0,1],[1,2],[3,-5],[0,-4],[-3,-8],[4,2],[2,-2],[3,-4],[3,-3],[0,-1],[-2,-1],[-4,1],[-2,0],[6,-2],[0,-2],[-4,-2],[-2,2],[-1,-2],[3,-2],[0,-2],[-2,0],[0,-1],[4,0],[-3,-2],[-3,0],[-13,3],[-1,1],[-2,4],[-1,1],[-2,0],[1,-3],[1,-2],[0,-2],[1,-2],[3,-9],[-1,-2],[-3,4],[-2,-2],[-2,-4],[-2,-2],[0,-2],[4,0],[2,-5],[0,-5],[-3,-4],[-1,2],[-2,-2],[-4,0],[-7,0],[6,-5],[2,-1],[2,1],[2,1],[2,0],[1,-3],[-1,-1],[0,-2],[-1,-1],[1,-4],[-1,-1],[-1,-1],[-1,-1],[-2,0],[-1,0],[-5,3],[-4,1],[-3,-1],[-2,-2],[1,-5],[-2,-1],[-1,-1],[-1,-1],[-2,0],[-1,0],[-1,1],[-1,2],[-2,0],[-4,0],[4,-3],[-2,-3],[-3,-4],[0,-6],[2,-2],[2,-2],[3,-1],[2,-1],[1,-1],[2,-2],[2,-2],[3,0],[-1,-3],[-2,-4],[-2,-3],[1,-4],[2,-2],[2,0],[3,-1],[1,-3],[-1,-3],[-2,-1],[-2,0],[-2,-1],[-1,-4],[0,-3],[1,-4],[2,-4],[2,2],[0,-2],[-6,-5],[-1,-1],[-2,-2],[0,2],[0,2],[1,2],[-1,2],[-8,-8],[-2,-1],[-9,0],[-2,-2],[-1,-3],[-1,-4],[0,-5],[-9,-9],[-6,0],[-2,2],[0,5],[0,3],[1,0],[1,-1],[1,1],[1,2],[-1,5],[0,2],[3,3],[3,2],[7,3],[2,1],[4,5],[3,1],[-1,1],[0,3],[-1,1],[2,2],[2,7],[2,4],[0,-1],[1,0],[2,-1],[1,1],[-1,1],[0,1],[0,1],[8,15],[0,3],[-2,1],[-8,-3],[-12,2],[-3,-1],[0,-3],[2,-6],[-2,-7],[-4,-6],[-2,-6],[2,-7],[-3,-2],[-4,0],[-1,9],[0,1],[-4,6],[-1,-1],[-3,-4],[-2,-2],[0,5],[2,16],[1,2],[5,4],[-2,2],[-2,0],[-2,0],[-2,-2],[1,7],[3,3],[3,1],[2,6],[-4,-1],[-4,-2],[-2,-4],[-2,-4],[-1,-5],[0,-2],[0,-3],[1,-4],[-1,-4],[-2,-2],[-2,0],[-2,2],[0,1],[1,4],[-2,6],[-2,6],[-3,2],[-1,-1],[-1,-3],[-1,-4],[0,-2],[1,-4],[1,-2],[4,-7],[2,-1],[1,-2],[0,-3],[-1,-2],[-2,0],[-2,2],[-3,4],[-3,5],[-3,3],[-3,-1],[-1,5],[1,3],[-1,2],[-2,3],[-1,0],[-3,-1],[-1,-1],[1,-3],[-2,-3],[-2,-2],[-1,-3],[0,-3],[0,-3],[0,-3],[1,-2],[2,-1],[2,0],[2,0],[2,-2],[0,-3],[0,-4],[-1,-6],[-5,5],[-2,0],[-2,-1],[-1,-1],[1,-6],[1,-5],[7,-7],[1,-5],[1,0],[2,5],[3,3],[3,2],[3,3],[2,-2],[2,-1],[5,1],[-1,0],[2,0],[1,-1],[0,-2],[-1,-2],[2,-1],[4,-1],[2,-2],[0,-3],[0,-2],[0,-2],[-2,2],[0,1],[-2,-3],[1,-5],[2,-3],[3,-1],[0,-2],[-1,0],[0,-1],[-1,-1],[2,-1],[1,-2],[2,-4],[0,-4],[1,-2],[-1,-2],[0,-3],[-1,0],[-2,0],[-1,0],[0,-1],[0,-4],[0,-1],[-3,-1],[-2,0],[-1,6],[-3,3],[-3,3],[-3,0],[-1,2],[-5,8],[-1,2],[-1,-1],[-1,-3],[-1,-1],[-2,-2],[-1,-1],[-2,2],[-1,0],[-3,-1],[-2,-2],[-1,-3],[1,-4],[4,-1],[4,1],[1,2],[1,1],[1,-1],[2,-2],[1,-2],[0,-20],[2,-4],[3,-1],[0,-1],[0,-3],[-1,-1],[3,0],[6,1],[2,2],[0,8],[1,4],[3,-2],[1,2],[1,3],[0,3],[1,3],[1,1],[5,4],[1,-4],[2,-2],[1,-3],[-2,-4],[3,0],[4,-4],[2,-1],[2,0],[5,3],[2,1],[0,-2],[-10,-13],[-2,-1],[-3,4],[-5,0],[-4,-3],[-3,-5],[1,-1],[2,-1],[-2,-2],[-1,1],[-1,2],[-2,1],[-2,-1],[-3,-2],[-3,-1],[-4,0],[-3,-1],[-3,-2],[-4,-4],[3,-2],[3,-2],[-1,-3],[1,-2],[-3,-1],[-2,-2],[-2,-2],[-1,-4],[-1,-2],[-2,-1],[-1,-3],[1,-5],[-6,-6],[-4,-7],[-3,-9],[-4,-8],[-4,-10],[-2,-4],[-4,0],[-1,1],[-1,5],[-1,2],[-2,1],[-4,1],[-1,1],[-5,5],[-3,2],[-8,1],[-3,2],[-4,7],[-3,4],[-2,1],[-7,2],[-1,1],[-6,5],[-2,2],[-2,1],[0,-2],[4,-4],[3,-6],[4,-3],[3,2],[3,-3],[1,-3],[1,-8],[1,-4],[4,-6],[1,-3],[0,-8],[1,-1],[7,-1],[6,-4],[2,-1],[6,1],[5,2],[5,4],[8,8],[1,2],[1,2],[0,5],[0,2],[3,2],[5,0],[2,1],[9,-7],[5,-2],[4,2],[3,-5],[1,-2],[1,-3],[-1,-3],[-4,-12],[-1,-3],[-2,-3],[-3,-1],[-2,-1],[-1,-1],[-3,-13],[-3,9],[-1,2],[-2,-4],[2,-2],[0,-6],[1,-5],[3,-3],[1,-1],[0,-4],[0,-6],[0,-1],[2,0],[0,-1],[0,-2],[-4,-6],[0,-2],[0,-8],[-1,-2],[-1,-1],[0,-1],[-2,0],[0,1],[-1,2],[-5,4],[-2,0],[-1,-4],[-2,0],[-3,-1],[-1,-2],[-1,1],[-1,1],[-3,0],[1,-6],[1,-8],[1,-5],[3,3],[-1,-6],[0,-6],[2,-4],[4,-2],[1,1],[1,3],[1,1],[3,-2],[1,-1],[2,1],[1,2],[1,2],[-2,-10],[0,-3],[4,-1],[-1,-2],[-1,-2],[-1,0],[-2,0],[3,-2],[1,0],[2,0],[-2,-3],[-1,0],[-2,-1],[0,-1],[5,1],[0,-1],[0,-2],[0,-2],[2,0],[2,-1],[0,-2],[0,-3],[-2,-3],[1,-1],[1,0],[1,-1],[2,-6],[0,-5],[-1,-3],[-3,-4],[3,0],[1,-1],[2,-1],[-9,-3],[-3,0],[0,2],[1,2],[1,2],[1,1],[-2,8],[-2,4],[1,2],[1,2],[1,2],[-1,1],[-2,5],[0,1],[-1,0],[-1,-1],[-1,-1],[-1,-1],[-1,0],[0,1],[-1,1],[-1,-3],[0,-5],[0,-4],[1,-1],[3,-1],[0,-1],[-1,-5],[-1,-3],[-3,-1],[-4,0],[-3,0],[1,3],[1,4],[1,5],[0,3],[0,1],[-1,1],[-1,1],[0,2],[0,2],[2,2],[0,2],[1,1],[-1,0],[-6,0],[-1,-2],[-1,-4],[-1,-3],[-2,2],[0,3],[2,8],[0,4],[-1,1],[-12,3],[-2,2],[-1,3],[1,3],[4,11],[-2,-2],[-1,0],[0,-2],[-2,2],[1,4],[-2,3],[-3,0],[-3,-2],[2,-4],[-1,-3],[-6,-2],[-1,-3],[0,-8],[-2,-4],[-3,-8],[-2,-2],[-2,-1],[-5,-1],[-2,-2],[-2,-3],[-5,-5],[-2,-1],[-2,-1],[1,-4],[7,-12],[2,-3],[0,-4],[1,-2],[2,1],[1,3],[1,4],[1,0],[3,-11],[-3,-2],[-2,-2],[-2,-3],[2,-4],[1,-3],[1,-1],[2,3],[2,3],[1,4],[1,4],[1,5],[1,-2],[2,-1],[2,0],[0,3],[2,-1],[1,-1],[1,-2],[-2,0],[1,-2],[4,-8],[1,-1],[2,0],[2,-2],[0,-2],[-1,-4],[-3,-2],[-7,1],[-2,-2],[3,-11],[-1,-3],[-4,-1],[-3,4],[-2,1],[-2,-2],[-4,-1],[-2,-2],[-2,4],[-2,4],[-2,4],[-4,-2],[-3,-4],[1,-4],[1,-3],[1,-3],[1,-1],[1,-2],[1,-3],[2,2],[1,-2],[-1,-2],[-2,-1],[-3,0],[-2,2],[-2,0],[-1,-2],[1,-1],[0,-1],[0,-1],[1,-1],[-2,1],[-4,3],[2,5],[1,2],[-3,3],[-2,-2],[-1,-4],[0,-4],[-2,4],[0,5],[4,7],[2,4],[0,4],[-2,2],[-4,1],[-7,-2],[-3,1],[2,5],[3,3],[4,1],[0,5],[-7,1],[-11,-5],[-1,-1],[-2,0],[-4,-4],[-1,-4],[3,-2],[4,-1],[1,-2],[-1,-4],[1,-4],[1,2],[2,2],[2,1],[3,-1],[2,-3],[-1,-3],[-2,-5],[-2,-4],[0,-5],[0,-2],[-1,-1],[-1,-1],[-2,-1],[-3,1],[-3,1],[-3,-2],[-1,-1],[-2,-3],[0,-3],[2,-5],[3,-4],[3,-2],[6,-3],[-1,-2],[2,-2],[2,-1],[2,1],[3,2],[1,7],[1,4],[1,-1],[0,-3],[0,-4],[-1,-4],[-1,-2],[-1,-2],[1,-4],[1,-7],[3,1],[1,1],[-2,-3],[-2,-3],[-3,-1],[-1,3],[-1,2],[-3,1],[-6,-1],[-1,-4],[1,-8],[4,-6],[6,4],[0,-2],[1,0],[-1,-1],[0,-1],[1,0],[0,-1],[0,-1],[-4,-2],[-5,0],[-4,2],[-3,3],[-2,1],[-5,-3],[-2,-1],[-2,1],[-1,2],[-2,1],[-3,-2],[1,-4],[-3,-3],[-4,-2],[-4,-2],[-1,1],[-1,1],[-1,2],[1,4],[1,1],[3,2],[1,0],[0,4],[-2,5],[-2,3],[-2,3],[0,-9],[0,-2],[-1,-1],[-1,2],[0,3],[1,3],[-1,1],[-1,2],[-2,4],[-1,-3],[-2,-5],[-1,-2],[1,-2],[3,-2],[1,-3],[0,-4],[-3,-2],[-5,1],[-4,1],[-4,3],[7,-8],[1,-3],[0,-4],[-1,-3],[0,-3],[3,-7],[1,-2],[0,2],[1,1],[1,0],[1,0],[1,0],[4,3],[1,2],[1,2],[2,-1],[0,-2],[0,-2],[0,-3],[1,-3],[1,1],[3,1],[2,-2],[1,-1],[0,-3],[-2,-7],[-1,-1],[-2,-1],[-2,-3],[-3,-8],[-3,-3],[-1,2],[-2,3],[-2,2],[-2,-3],[-1,-4],[0,-5],[0,-5],[1,-3],[2,0],[2,-1],[-1,-6],[-2,-6],[-4,-6],[-5,-4],[-4,-2],[-1,0],[-1,-1],[-1,3],[0,2],[0,2],[1,2],[1,1],[3,1],[-1,2],[-12,11],[-2,1],[-2,0],[-5,4],[-3,1],[1,4],[0,3],[0,8],[0,3],[-2,-2],[-1,-3],[-2,-2],[3,-13],[-2,-5],[-5,-4],[-4,-2],[-5,2],[-1,-1],[-1,1],[-4,2],[-1,-3],[-1,-3],[-2,-1],[-2,1],[-1,-8],[-4,-1],[-4,4],[-2,5],[0,6],[2,1],[2,-1],[2,2],[0,3],[-2,1],[-3,1],[-2,0],[1,5],[1,4],[-1,4],[-3,5],[-1,-8],[-1,-1],[-1,0],[-1,0],[0,-6],[-1,-4],[-2,-3],[-1,-3],[-1,-4],[-1,1],[-2,3],[-7,5],[-2,-5],[2,-6],[-2,-4],[-2,-1],[-3,2],[1,5],[-3,-3],[-3,-5],[1,-3],[7,-1],[4,-2],[2,-4],[-2,-4],[-2,-2],[-1,-1],[-2,0],[-1,-1],[-1,-1],[-1,-2],[2,-2],[2,-3],[0,-2],[-4,0],[-14,9],[-7,2],[-7,-2],[3,-1],[3,-1],[4,-1],[2,-4],[-14,5],[-4,-1],[0,-2],[14,-17],[5,-3],[14,2],[5,3],[1,2],[2,-1],[1,-2],[0,-3],[-1,-6],[-1,-2],[-1,0],[-1,-2],[-1,-7],[1,0],[2,2],[2,2],[1,2],[3,0],[3,-2],[2,-5],[0,-2],[2,-2],[1,-2],[0,-5],[0,-2],[1,-3],[-3,0],[-4,-3],[-2,1],[-2,-2],[-3,-7],[-3,-3],[-1,-1],[-3,-2],[-1,-1],[-1,-2],[-1,-4],[0,-2],[-2,-1],[-1,4],[0,8],[-3,2],[-2,-1],[-2,-3],[-2,-3],[4,-2],[1,-4],[-1,-6],[-5,-11],[-1,-3],[0,-4],[0,-3],[1,-3],[0,-2],[-1,-4],[-5,2],[-4,-3],[-7,-11],[-4,-6],[-1,-3],[0,-5],[-1,-3],[-2,-3],[-2,1],[-1,3],[1,8],[9,13],[1,6],[-2,1],[-4,-3],[0,3],[2,1],[1,2],[-1,2],[0,2],[-1,5],[-1,0],[-2,-9],[-4,-1],[-5,3],[-4,5],[1,-4],[1,-3],[2,-1],[3,0],[-4,-10],[0,-5],[1,-6],[-3,-6],[-4,-6],[-3,-9],[-1,-9],[0,-8],[2,-5],[-3,-5],[-1,-3],[-3,-7],[-1,5],[0,6],[2,6],[0,9],[0,5],[-1,5],[-2,5],[-2,3],[-2,3],[-3,2],[-11,0],[-3,1],[-4,5],[-2,1],[-2,-4],[1,-5],[3,-1],[3,1],[3,0],[6,-10],[-2,-11],[-4,-12],[-5,-3],[-9,5],[-2,-1],[-1,-2],[-1,-2],[-2,-2],[-4,-2],[-2,4],[-2,3],[-3,-2],[3,-5],[-1,-4],[-3,-7],[1,-9],[-1,-2],[-1,-6],[2,-5],[-3,-6],[-3,-2],[-1,5],[0,4],[2,4],[-2,4],[-1,6],[0,-2],[-3,-6],[1,-3],[1,-3],[0,-3],[-6,-2],[-2,-2]],[[7122,2074],[0,1],[-2,6],[-2,3],[-1,1],[-1,1],[-7,5],[-3,3],[-1,2],[-2,4],[-2,4],[-1,3],[-6,35],[-2,4],[0,2],[-2,3],[0,1],[0,1],[-3,22],[-1,1],[-1,1],[0,1],[-1,3],[-1,4],[-1,11],[0,4],[0,4],[1,6],[1,2],[0,2],[3,6],[1,3],[1,2],[-1,6],[-3,8],[-8,34],[-1,5],[-1,1],[0,1],[-6,7],[-1,2],[-1,2],[-1,2],[0,2],[0,1],[0,3],[0,2],[0,1],[-1,1],[-1,2],[-10,10],[-1,2],[0,2],[0,3],[1,12],[1,6],[1,3],[1,2],[0,2],[0,2],[0,1],[0,1],[-1,1],[-1,1],[-3,0],[-14,0],[-2,-1],[-1,-1],[-6,-2],[-5,-3],[-5,-6],[-1,-1],[-1,0],[-1,0],[-1,0],[-1,0],[-1,2],[-1,1],[0,1],[-3,14],[-1,3],[-2,4],[-5,8],[-1,2],[-1,3],[-3,7],[-3,7],[0,1],[0,1],[0,4],[-1,1],[-1,1],[-2,-1],[-2,0],[-1,-1],[-2,-1],[-3,-4],[-2,-1],[-4,-1],[-1,0],[-1,0],[0,2],[-1,2],[1,3],[-1,1],[-1,1],[-9,2],[-5,0],[-1,-1],[-2,0],[-5,0],[-2,0],[-2,1],[-3,3],[-2,2],[-1,1],[-2,4],[0,1],[-1,0],[-1,1],[-1,0],[-2,0],[-2,-1],[-2,1],[-1,1],[-4,3],[-3,1],[-8,1]],[[6912,2419],[-5,9],[0,2],[-1,2],[1,0],[1,0],[2,1],[1,1],[1,4],[0,3],[0,1],[-1,3],[-1,3],[0,11],[0,2],[-2,5],[-3,20],[-1,4],[1,2],[2,3],[6,5],[1,2],[1,2],[5,10],[1,2],[2,4],[1,2],[1,2],[1,4],[1,3],[0,2],[-1,8],[1,3],[0,3],[2,12],[1,13],[1,1],[2,5],[1,4],[0,1],[3,5],[0,3],[1,2],[0,1],[-1,1],[-1,5],[-1,2],[0,2],[0,3],[0,2],[0,1],[1,2],[1,4],[2,5],[1,3],[1,1],[1,2],[1,0],[2,1],[0,1],[1,2],[2,8],[1,1],[0,1],[1,1],[1,1],[1,1],[2,0],[2,1],[1,1],[0,1],[0,1],[-1,1],[-2,3],[-1,2],[-1,3],[0,4],[0,5],[0,2],[1,2],[4,7],[3,4],[1,0],[1,2],[3,1],[7,1],[2,1],[1,0],[1,1],[4,4],[3,2],[1,2],[1,3],[5,13],[5,10],[4,19],[0,6],[0,1],[-1,1],[-1,3],[0,2],[0,3],[0,4],[0,3],[0,1],[-1,6],[-1,6],[0,3],[1,2],[7,14],[2,3],[1,1],[2,4],[4,10],[1,0],[1,1],[1,0],[1,0],[1,0],[0,-1],[1,-2],[1,-2],[1,-1],[0,-1],[1,-1],[1,-1],[1,0],[1,0],[1,1],[1,1],[2,3],[0,2],[1,2],[0,6],[3,15],[0,2],[-1,1],[-3,1],[-1,0],[-1,1],[-2,1],[-1,2],[-4,11],[-1,8],[0,6],[0,4],[0,2],[-1,1],[-1,3],[-1,2],[-1,3],[-1,2],[0,2],[0,3],[0,3],[0,2],[1,1],[1,3],[2,4],[0,4],[3,28],[0,1],[1,2],[6,10],[3,6],[1,2],[1,1],[3,3],[1,1],[4,5],[1,1],[1,0],[1,-3],[1,0],[1,0],[3,2],[4,3],[3,2],[1,1],[2,0],[8,0],[1,0],[2,1],[3,3],[2,0],[1,1],[1,0],[3,-2],[1,0],[1,0],[1,1],[1,1],[8,11],[3,2],[6,9],[5,16],[1,5],[5,5],[1,2],[2,6],[5,10],[0,1],[1,2],[-1,3],[-1,2],[-1,2],[-1,1],[0,1],[0,3],[0,2],[-1,1],[-1,0],[0,-1],[-1,-1],[-1,0],[-1,0],[0,1],[-1,1],[0,8],[0,2],[1,2],[3,7],[0,2],[0,1],[0,3],[0,2],[0,1],[-3,8],[0,3],[-1,6],[0,3],[1,7],[1,4],[10,23],[3,5],[6,13],[7,11],[2,1],[1,1],[0,3],[1,4],[-2,15],[2,2],[1,1],[1,0],[2,-1],[2,-1],[3,2],[9,10],[2,4],[1,1],[3,2],[3,1],[3,2],[1,1],[3,5],[1,1],[1,1],[1,0],[2,0],[1,-1],[1,-1],[0,-1],[1,-1],[0,-4],[1,-2],[0,-1],[1,-2],[2,-3],[1,-2],[0,-1],[1,-3],[0,-1],[1,-2],[2,-3],[1,-2],[2,-1],[6,-5],[3,0],[4,-1],[1,-1],[3,-3],[1,-1],[1,0],[1,0],[1,1],[1,1],[2,3],[1,2],[1,2],[0,2],[1,1],[-1,7],[1,1],[0,3],[1,3],[0,1],[1,3],[2,1],[0,1],[1,1],[6,1],[3,1],[1,0],[1,1],[6,4],[4,2],[1,0],[1,0],[3,0],[1,0],[4,3],[1,0],[2,0],[5,-1],[2,0],[1,1],[1,4],[1,0],[0,1],[0,1],[1,1],[2,1],[1,1],[2,4],[1,2],[1,1],[2,0],[1,0],[2,-1],[1,0],[1,0],[2,1],[19,11],[7,1],[1,0],[1,1],[0,2],[1,3],[0,6],[2,9],[0,2],[-1,2],[-4,4],[0,1],[-1,1],[0,2],[0,1],[1,3],[3,4],[6,7],[6,5]],[[7330,3311],[9,-7],[3,0],[1,3],[1,2],[1,1],[9,3],[2,0],[1,0],[0,-1],[1,-2],[1,-1],[1,0],[8,2],[2,1],[1,2],[2,1],[0,1],[3,2],[2,2],[2,-11],[2,-3],[3,-5],[2,-3],[1,-3],[0,-2],[0,-1],[0,-1],[-2,-2],[-2,-2],[-1,-1],[0,-1],[-1,-2],[0,-2],[1,-1],[1,-1],[1,-2],[1,-4],[0,-3],[0,-1],[-4,-6],[-1,-1],[-1,-1],[0,-4],[-1,-20],[1,-10],[0,-1],[0,-1],[1,-1],[4,-6],[1,-2],[4,-3],[0,-1],[1,-2],[2,-3],[0,-3],[1,-11],[1,-8],[3,-13],[1,-3],[3,-5],[1,-3],[0,-2],[0,-2],[-1,-3],[0,-2],[-1,-3],[1,-1],[0,-2],[1,-1],[0,-3],[5,-27],[0,-2],[0,-1],[-1,-1],[-1,-1],[0,-1],[-1,0],[-1,0],[-1,-1],[-1,-1],[0,-2],[0,-2],[0,-1],[2,-3],[2,-2],[2,-1],[2,-1],[2,-1],[2,1],[5,2],[1,0],[2,0],[7,-3],[1,0],[3,-2],[1,-1],[1,0],[1,1],[1,0],[0,1],[1,0],[2,-1],[1,-2],[0,-3],[1,-1],[1,-2],[4,-2],[4,-1],[5,-1],[5,0],[2,1],[1,1],[1,3],[1,2],[1,2],[1,1],[6,2],[1,1],[1,1],[1,4],[1,2],[1,1],[3,3],[1,1],[2,0],[1,0],[1,0],[0,-1],[1,-2],[1,0],[5,4],[2,1],[1,2],[1,2],[0,2],[1,6],[0,2],[3,5],[1,3],[0,1],[0,2],[0,3],[1,3],[3,6],[2,3],[2,1],[1,0],[1,0],[4,-1],[3,0],[1,-1],[2,-2],[0,-3],[0,-2],[-1,-1],[-1,-2],[0,-2],[0,-1],[0,-2],[0,-1],[0,-1],[1,-1],[1,-1],[2,-1],[2,-1],[1,-3],[0,-1],[0,-4],[0,-2],[0,-1],[1,-2],[3,-3],[0,-2],[1,-2],[0,-3],[1,-4],[0,-4],[-1,-10],[0,-2],[0,-1],[1,-3],[2,-3],[5,-5],[0,-2],[2,-2],[1,-3],[1,-1],[0,-2],[0,-1],[0,-2],[0,-1],[1,-2],[1,-1],[1,0],[5,-1],[2,1],[1,1],[0,1],[1,1],[1,0],[1,-1],[2,-1],[1,0],[2,0],[2,1],[1,1],[6,0],[1,0],[1,1],[0,1],[1,1],[1,2],[4,3],[1,1],[1,0],[1,-1],[4,-4],[1,0],[1,-1],[2,1],[2,0],[1,1],[1,1],[7,9],[2,2],[3,3],[11,3],[4,0],[2,-1],[1,-1],[2,-1],[4,-5],[1,-1],[1,-1],[4,1],[1,0],[1,0],[2,-1],[1,-2],[5,-10],[2,-5],[1,-4],[1,-8],[0,-15],[1,-1]],[[7524,4063],[-2,-22],[-7,-45],[0,-3],[1,-2],[-1,-1],[0,-3],[-1,-2],[-2,-4],[-1,-2],[-2,-5],[0,-2],[-1,-1],[-2,-1],[-3,-3],[-1,-1],[0,-1],[-1,-2],[0,-4],[0,-3],[0,-3],[-4,-10],[-2,-8],[-1,-2],[-1,-1],[-1,0],[-1,0],[-1,0],[-1,1],[-1,0],[0,1],[-1,2],[0,1],[-1,3],[0,1],[0,1],[-1,1],[-1,1],[-1,1],[-1,0],[-1,-1],[-1,-1],[-2,-1],[-1,-1],[0,-1],[-1,-1],[-2,-8],[-1,-2],[-1,-1],[-1,-1],[-5,-2],[-1,-1],[0,-1],[-1,-2],[1,-4],[-1,-2],[0,-1],[-2,-1],[-1,-1],[0,-1],[-1,-2],[0,-1],[0,-2],[1,-1],[0,-2],[1,-1],[2,-2],[2,-1],[1,0],[4,2],[1,0],[1,-1],[1,0],[1,-2],[0,-3],[0,-4],[0,-2],[1,-2],[2,-3],[1,-2],[0,-2],[0,-5],[0,-3],[1,-2],[2,-2],[3,-3],[0,-1],[1,0],[-1,-3],[-8,1],[-2,0],[-2,-1],[-2,-2],[-4,-4],[-2,-3],[-1,-2],[-3,-6],[0,-1],[-1,-1],[-2,1],[-1,1],[-1,2],[-1,0],[-1,1],[-1,-1],[-4,-1],[-1,0],[-8,2],[-10,-1],[-1,0],[-1,1],[-2,1],[-4,3],[-4,4],[-1,1],[-1,1],[-1,0],[-2,0],[-1,0],[-1,0],[-1,0],[-6,-5],[-1,-1],[-2,-2],[0,-1],[-1,-2],[1,-4],[0,-1],[0,-2],[3,-9],[2,-8],[-1,-2],[-1,-3],[-6,-8],[-1,-4],[2,0],[1,-1],[1,-1],[0,-1],[0,-1],[-1,-4],[0,-1],[-1,-2],[0,-2],[1,-1],[0,-2],[1,-4],[2,-20],[0,-1],[1,-4],[-1,-1],[0,-2],[-1,-2],[-6,-7],[0,-1],[-1,-3],[-1,-2],[0,-6],[-1,-1],[0,-2],[-2,-1],[-12,-14],[-1,-2],[-1,-1],[-1,-4],[0,-2],[0,-2],[1,-4],[0,-2],[0,-2],[0,-2],[0,-1],[-1,-2],[0,-2],[-13,-19],[-1,-1],[-5,0],[-1,-1],[-1,-1],[-7,-6],[-9,-17],[-5,-10],[-1,-1],[-7,-6],[-1,-2],[-1,-1],[-1,-5],[-1,-2],[-1,0],[-3,-1],[-1,2],[-1,2],[-1,2],[-1,1],[-1,1],[0,1],[-1,1],[-1,0],[-1,0],[-1,-1],[-2,-3],[0,-1],[1,-2],[0,-1],[0,-1],[0,-2],[0,-1],[-4,-3],[-18,-7]],[[7286,3607],[-7,6],[-2,4],[0,4],[-1,6],[0,2],[0,1],[1,1],[0,1],[0,1],[-1,3],[-1,2],[-1,1],[0,1],[-9,3],[-4,5],[-3,2],[-2,0],[-1,0],[-4,-1],[-3,-1],[-5,-3],[-2,-1],[-5,1],[-8,4],[-2,1],[-1,-1],[-6,-2],[-6,-1],[-3,0],[-3,2],[-1,1],[-3,3],[-3,7],[-1,1],[-1,1],[-2,0],[-3,0],[-1,0],[-6,-1],[-1,1],[-3,0],[-1,1],[-1,1],[-1,3],[-1,3],[-1,2],[-1,1],[-3,3],[-8,14],[-2,7],[-1,2],[-2,3],[0,2],[-1,2],[0,2],[1,12],[0,1],[-1,0],[-1,1],[-2,-1],[-4,-3],[-1,0],[-1,0],[-1,2],[0,2],[-1,1],[-2,0],[-1,0],[-1,-1],[-3,-2],[-1,-1],[-1,0],[-1,0],[-2,1],[-1,0],[-1,1],[-1,1],[-2,2],[0,1],[-1,2],[0,3],[1,1],[1,4],[0,1],[-1,2],[0,2],[-6,5],[-1,0],[-3,0],[-2,-1],[-2,-1],[-2,-2],[-1,0],[-1,-2],[-4,-9],[-2,-5],[-2,-3],[-1,-1],[0,-1],[1,-1],[1,-1],[2,-1],[0,-1],[1,-1],[2,-7],[0,-2],[0,-3],[-2,-4],[0,-3],[0,-2],[0,-1],[0,-2],[-1,-2],[-1,-1],[-9,-6],[-7,-4],[-2,-2],[-1,-1],[-1,-2],[-5,-11],[-6,-14],[-2,-2],[0,-1],[-1,0],[-2,-1],[-4,0],[-2,0],[-1,1],[-5,5],[-1,1],[-1,0],[-1,-1],[-1,-1],[0,-2],[-1,0],[0,-1],[0,-1],[-1,-1],[-1,0],[-2,0],[-1,2],[-2,5],[-1,2],[-2,4],[-5,4],[-2,2],[0,2],[-1,1],[0,1],[1,2],[1,4],[3,14],[1,3],[1,1],[1,0],[1,0],[2,-2],[1,0],[1,0],[0,1],[1,1],[1,6],[1,1],[0,1],[1,1],[1,0],[1,0],[1,0],[1,-1],[0,-1],[1,-1],[1,0],[1,1],[0,1],[1,1],[11,25],[3,15],[0,1],[0,2],[0,2],[0,2],[-1,1],[-1,2],[-13,20],[-2,3],[-7,4],[-3,1],[-3,1],[-2,0],[-4,-1],[-2,0],[-2,-2],[-6,-7],[-2,-2],[0,-1],[0,-1],[-2,-2],[-1,-1],[-1,-4],[-3,-6],[-2,-8],[-1,-4],[-3,-4],[-3,-2],[-4,-1],[-9,0],[-4,-1],[-5,-5],[-10,-13],[-5,-5],[-5,-3],[-5,1],[-10,1]],[[6954,3707],[-2,9],[-1,3],[0,2],[-1,10],[0,3],[1,3],[0,1],[0,1],[0,1],[-1,3],[-1,2],[-6,10],[0,1],[-1,2],[0,2],[0,2],[2,11],[0,3],[-1,8],[-1,5],[-1,9],[0,1],[0,1],[-2,3],[-1,17],[-1,2],[0,1],[-2,2],[-1,2],[-6,5],[-1,1],[0,1],[-2,5],[0,1],[-1,1],[0,1],[-4,3],[-4,2],[-1,1],[-1,2],[0,4],[-1,2],[0,1],[-2,3],[-1,3],[0,1],[0,2],[1,4],[1,2],[1,1],[6,5],[1,1],[1,1],[1,1],[0,2],[0,3],[0,4],[0,3],[-1,4],[-3,6],[-7,13],[-2,2],[0,1],[-1,0],[-1,1],[-3,0],[-1,1],[-1,0],[0,2],[-1,4],[0,1],[0,1],[-1,1],[-3,4],[-1,2],[-2,4],[-2,5],[0,2],[0,2],[0,3],[1,3],[0,4],[1,4],[0,1],[1,1],[1,2],[1,1],[2,1],[2,-1],[1,-1],[1,0],[2,0],[1,1],[1,1],[1,2],[0,3],[0,1],[0,2],[-1,4],[0,2],[0,3],[0,4],[0,1],[1,2],[2,2],[3,4],[3,2],[3,1],[2,1],[1,2],[5,7],[6,6],[1,2],[2,4],[0,1],[1,4],[-1,7],[-5,3],[-2,0],[-4,-1],[-1,0],[-2,-1],[-1,0],[-1,1],[-1,2],[0,3],[0,2],[-1,1],[-4,6],[-1,2],[-1,2],[0,1],[0,2],[0,1],[-1,1],[-1,1],[-1,1],[-2,0],[-1,0],[-1,0],[-3,-2],[-1,0],[-1,0],[-1,0],[-2,1],[-4,4],[-10,13],[-3,4],[-4,2],[-2,2],[-1,0],[-2,0],[-1,0],[0,-1],[-1,-1],[-1,-2],[-1,-3],[0,-1],[-1,-3],[0,-2],[-1,-1],[0,-1],[-1,-1],[-1,-1],[-1,0],[-1,1],[-1,0],[-1,2],[-2,3],[-4,10],[-1,4],[0,1],[-1,2],[-3,3],[-2,3],[-3,1],[-1,1],[0,1],[-1,2],[-1,7],[-1,1],[-1,2],[-2,3],[-1,1],[-5,2],[-1,1],[-1,4],[0,13]],[[6828,4138],[3,14],[0,5],[0,3],[0,3],[0,2],[0,2],[1,1],[3,6],[7,17],[1,1],[1,2],[0,2],[2,8],[0,2],[1,3],[2,4],[2,3],[3,7],[1,0],[1,0],[1,0],[1,-1],[0,1],[1,0],[2,3],[1,1],[13,8],[4,2],[3,0],[3,-1],[2,0],[1,1],[1,1],[1,1],[1,0],[5,-1],[7,-2],[5,0],[3,0],[2,1],[1,0],[0,1],[1,3],[0,4],[-1,17],[2,24],[1,6],[1,0],[0,1],[0,2],[0,2],[1,2],[-3,60],[0,2],[0,1],[-1,4],[-2,5],[0,3],[0,16],[-1,7],[-1,3],[0,2],[-1,1],[-2,3],[0,1],[0,2],[0,4],[2,7],[0,5],[-1,1],[-1,1],[0,2],[0,2],[1,1],[0,1],[5,7],[4,3],[-1,1],[-5,0],[-7,-8],[-6,-10],[-3,-9],[-3,5],[-3,2],[-2,-5],[-2,1],[-6,3],[-1,-7],[-6,-17],[-3,3],[-3,1],[-4,-1],[-4,-3],[-3,2],[-2,2],[-4,6],[-1,1],[-1,1],[-2,6],[0,1],[-1,2],[-1,1],[-1,4],[-1,1],[0,1],[-5,3],[-3,4],[-2,1],[-1,1],[-1,2],[-1,1],[-2,2],[-12,2],[-1,0],[0,1],[-1,1],[-3,2],[-10,2],[1,13],[0,4],[0,1],[0,2],[-1,1],[-3,6],[0,2],[-1,4],[1,15],[0,3],[1,1],[0,1],[1,1],[0,2],[-1,2],[0,3],[0,6],[0,2],[-1,2],[-3,2],[-1,1],[-2,3],[-1,1],[-2,2],[-4,1],[-4,3],[-1,0],[-4,1],[-1,0],[-1,1],[-1,1],[-2,2],[0,1],[-1,1],[-14,6],[-1,1],[-1,0],[-3,3],[-2,3],[-1,2],[0,2],[0,4],[0,1],[0,2],[1,2],[1,2],[0,1],[0,2],[0,1],[-1,7],[10,10],[4,1],[3,-2],[5,-6],[2,-1],[12,-2],[6,0],[8,7],[1,0],[9,3],[1,1],[3,2],[1,2],[0,1],[1,1],[1,7],[1,8],[0,2],[1,1],[1,2],[4,6],[1,1],[1,2],[0,2],[0,3],[0,1],[0,1],[0,1],[-1,12],[-1,3],[0,2],[-1,1],[0,1],[-2,1],[-1,1],[1,2],[1,4],[1,2],[-1,2],[0,2],[0,2],[0,2],[4,7],[0,1],[0,2],[-1,10],[0,2],[1,2],[1,1],[1,0],[1,-1],[1,-3],[1,-1],[0,-1],[1,0],[2,0],[1,1],[1,4],[3,5],[1,2],[1,1],[4,2],[5,1],[16,2],[5,3],[1,1],[1,1],[0,1],[0,1],[-2,9],[-2,8],[-2,11],[-1,5],[0,1],[-2,3],[-1,1],[0,1],[0,1],[0,2],[0,1],[1,2],[0,1],[1,1],[1,-1],[1,0],[1,-1],[0,-1],[1,0],[1,0],[1,2],[0,1],[1,18],[0,1],[-1,2],[0,1],[-1,1],[-1,0],[-1,0],[-4,0],[-1,0],[-1,1],[-1,1],[0,1],[0,1],[0,7],[0,7],[2,7],[2,7],[2,4],[2,3],[7,11],[1,2],[1,0],[1,1],[5,0],[1,0],[4,2],[1,0],[1,0],[4,-3],[7,-7],[3,-4],[0,-1],[1,-1],[2,0],[1,1],[1,1],[1,1],[2,0],[1,0],[3,-3],[3,0],[1,-1],[2,1],[2,0],[1,-1],[2,-3],[0,-2],[1,-1],[0,-4],[-1,-3],[-1,-6],[-1,-1],[0,-2],[1,-1],[0,-2],[1,-1],[9,-11],[2,-3],[1,-4],[2,-7],[1,-4],[2,-4],[7,-11],[2,-3],[1,-1],[1,0],[2,-1],[2,0],[2,0],[33,16],[6,3],[1,1],[1,1],[0,2],[1,2],[0,4],[1,5],[1,1],[1,1],[2,2],[3,1],[1,1],[1,0],[2,2],[1,1],[1,0],[4,1],[1,0],[1,1],[1,2],[2,3],[1,1],[1,1],[3,-2],[1,-1],[1,0],[1,0],[1,0],[0,2],[1,2],[0,9],[-1,5],[-5,10],[-5,18],[-3,8],[-2,4],[0,1],[0,1],[0,2],[1,2],[0,1],[0,2],[1,1],[1,1],[1,1],[1,1],[0,2],[0,1],[0,2],[0,3],[-1,6],[0,3],[0,1],[0,2],[1,6],[0,1],[0,1],[-1,1],[-1,1],[-1,0],[-12,-1],[-2,-1],[-1,0],[-4,1],[-1,0],[-3,-1],[-2,-1],[-1,0],[-1,1],[-2,1],[-1,1],[-9,14],[-8,8],[-7,6],[-3,3],[-3,5],[-5,7],[0,2],[-1,3],[1,10],[1,3],[0,1],[0,2],[0,2],[-2,9]],[[6955,4972],[6,4],[1,0],[2,0],[1,-1],[1,-1],[1,-2],[1,-1],[2,1],[4,2],[13,11],[5,6]],[[6992,4991],[4,0],[2,1],[1,0],[1,-1],[3,-4],[7,-7],[1,-2],[1,-2],[0,-1],[1,-1],[0,-2],[1,-1],[10,-10],[1,-2],[1,-2],[1,-3],[0,-1],[1,0],[2,-2],[3,-1],[14,-5],[9,-4],[1,-1],[1,-2],[2,-3],[2,-8],[0,-1],[2,-1],[2,0],[2,0],[1,0],[10,3],[2,0],[1,-1],[1,0],[1,-1],[2,-1],[2,-4],[2,-5],[1,-2],[0,-1],[-1,-2],[0,-1],[-1,-4],[-1,-6],[0,-3],[1,-1],[0,-2],[0,-1],[1,-1],[5,-5],[1,-1],[1,-2],[0,-1],[-1,-10],[0,-2],[0,-2],[1,-1],[0,-1],[1,-2],[1,-2],[17,-13],[4,-4],[3,-2],[8,-3],[3,0],[12,1],[2,0],[3,-2],[3,-4],[2,-2],[2,-4],[1,-1],[1,-1],[3,0],[2,0],[1,1],[6,4],[1,1],[1,0],[2,1],[1,0],[2,-1],[1,-1],[1,-1],[2,-3],[1,-1],[1,-4],[2,-3],[0,-2],[1,0],[1,-1],[2,0],[1,0],[1,1],[1,1],[0,1],[1,0],[0,1],[0,1],[0,1],[1,0],[2,-2],[1,-2],[0,-1],[1,-1],[0,-1],[2,-11],[1,-1],[0,-2],[1,-2],[1,-1],[0,-1],[1,-5],[0,-1],[0,-1],[1,-1],[1,-1],[1,-1],[2,0],[1,1],[2,1],[1,0],[1,-1],[0,-1],[0,-2],[0,-7],[0,-5],[0,-13],[0,-3],[0,-1],[0,-2],[-1,0],[-2,-3],[0,-1],[-1,-1],[1,-1],[1,-1],[3,0],[13,2],[1,0],[4,-2],[1,0],[2,1],[1,1],[2,0],[4,0],[3,1],[1,1],[2,2],[1,1],[15,1],[5,3],[2,0],[2,0],[2,0],[1,-1],[2,-2],[3,-5],[1,-3],[1,-3],[0,-2],[0,-2],[0,-2],[0,-2],[-1,-3],[-2,-5],[-2,-4],[-2,-2],[-1,-2],[0,-1],[-1,-2],[-2,-22],[1,-8],[0,-2],[0,-2],[0,-1],[-2,-1],[-1,0],[-1,0],[-1,2],[-2,1],[-1,0],[-2,-3],[-2,-6],[-8,-28],[-1,-13],[-1,-1],[-5,-7],[-1,-2],[-1,-1],[0,-2],[-1,-1],[0,-1],[0,-2],[1,-1],[1,-2],[4,-2],[2,-3],[1,-1],[0,-2],[0,-1],[0,-1],[0,-2],[1,-1],[1,-2],[3,-1],[11,-3],[0,1],[1,1],[1,1],[5,3],[4,1],[1,0],[0,1],[0,1],[0,1],[0,2],[1,3],[0,1],[1,0],[1,0],[2,0],[2,-2],[2,-3],[0,-2],[1,-1],[0,-2],[0,-2],[0,-2],[0,-4],[0,-7],[0,-3],[1,-7],[4,-17],[1,-3],[1,-1],[6,-7],[0,-1],[0,-2],[-7,-6],[-2,-2],[0,-1],[-1,-1],[0,-3],[0,-2],[0,-3],[1,-3],[1,-1],[0,-2],[1,-1],[0,-1],[4,-4],[1,-1],[0,-2],[1,-1],[0,-2],[0,-1],[0,-4],[0,-3],[0,-1],[0,-2],[1,-1],[0,-1],[11,-7],[0,-1],[0,-2],[0,-1],[1,-1],[2,0],[3,0],[1,0],[2,2],[2,0],[24,-3],[1,1],[2,3],[1,0],[1,1],[3,0],[13,-4],[2,0],[1,2],[3,3],[0,1],[0,1],[0,1],[-1,4],[0,1],[0,2],[0,1],[1,3],[0,1],[1,8],[0,2],[0,1],[1,0],[0,1],[1,0],[1,1],[3,1],[1,0],[1,1],[0,2],[0,1],[1,9],[0,2],[1,1],[0,1],[1,1],[0,1],[1,1],[1,1],[0,1],[1,5],[1,1],[1,1],[1,1],[2,0],[1,-1],[3,-2],[1,-1],[2,0],[7,2],[2,0],[3,0],[1,0],[1,-1],[0,-1],[1,0],[0,-1],[1,-1],[0,-2],[0,-2],[0,-4],[0,-2],[0,-1],[1,-1],[3,-1],[1,-1],[0,-1],[2,-3],[0,-1],[4,-5],[1,-1],[2,-4],[1,-1],[2,-1],[6,-1],[1,-1],[0,-2],[0,-4],[-1,-4],[1,-2],[0,-2],[1,-5],[1,-2],[1,-1],[2,-3],[1,-2],[0,-3],[0,-12],[1,-7],[1,-4],[-1,-8],[-5,-5],[-1,-2],[-1,-2],[-1,-5],[-1,-3],[0,-3],[0,-4],[0,-1],[-1,0],[0,-1],[-1,-2],[-6,-10],[-2,-2],[-2,0],[-1,5],[0,1],[0,1],[-1,1],[-2,3],[-1,1],[0,1],[-1,2],[0,1],[-2,1],[-1,1],[-1,0],[-1,0],[-2,0],[-1,0],[-1,1],[-1,0],[0,1],[-1,2],[-3,6],[0,1],[-1,1],[-1,1],[-1,0],[-2,0],[-1,0],[-1,1],[0,1],[0,1],[0,2],[0,1],[0,1],[0,2],[-1,0],[-1,0],[-7,-2],[-1,0],[-1,0],[-1,1],[-1,1],[0,1],[-1,1],[-1,1],[-1,-1],[-2,-1],[-2,0],[-1,0],[-2,3],[-1,1],[-1,0],[-1,-1],[-2,-2],[-1,-1],[-2,0],[-1,0],[-1,0],[-1,0],[-2,-2],[-2,-1],[-1,0],[-1,0],[0,1],[-1,1],[-2,0],[-3,-3],[-1,-1],[-1,0],[-1,1],[-2,1],[-1,1],[-1,0],[-1,0],[-1,0],[-1,-1],[-1,-2],[0,-2],[0,-3],[0,-2],[1,-1],[3,-3],[1,-1],[1,-1],[0,-1],[0,-2],[0,-1],[0,-4],[0,-1],[1,-2],[0,-1],[1,-1],[0,-1],[1,0],[7,-1],[1,-1],[0,-1],[1,-1],[0,-1],[1,-2],[0,-6],[-1,-7],[1,-3],[1,-5],[1,-4],[0,-2],[0,-3],[0,-1],[-1,-2],[-2,-2],[-1,-1],[0,-4],[-1,-3],[0,-1],[0,-1],[2,-1],[1,-1],[0,-2],[0,-1],[-1,-3],[-2,-4],[-7,-5],[-2,-2],[-9,-2],[-5,-1],[-1,0],[0,-1],[-1,-3],[0,-2],[0,-6],[0,-3],[0,-1],[-1,-2],[-2,-2],[-11,-9],[-3,-3],[-2,-6],[0,-3],[0,-2],[0,-2],[0,-1],[1,-2],[1,-1],[1,-2],[0,-1],[0,-1],[-1,-1],[-3,-2],[-1,-1],[0,-1],[0,-1],[-1,-2],[0,-1],[0,-1],[-2,-1],[0,-1],[0,-2],[4,-5],[1,-1],[2,-2],[2,-2],[2,-3],[2,-1],[1,0],[1,1],[1,0],[1,0],[1,-1],[1,-3],[1,-4],[1,-2],[0,-3],[-1,-11],[0,-4],[1,-2],[5,-4],[1,-1],[2,-1],[1,-1],[1,0],[1,-3],[0,-1],[1,0],[2,-1],[12,-3],[6,-4],[1,0],[0,-1],[0,-2],[0,-1],[-1,-2],[-1,-1],[-1,-1],[-1,-2],[0,-1],[0,-2],[2,-2],[0,-1],[2,-1],[4,-2],[2,0],[2,0],[1,0],[0,1],[1,1],[1,1],[1,3],[0,1],[1,1],[0,1],[1,0],[1,0],[1,0],[1,-2],[1,-2],[0,-5],[1,-2],[0,-1],[1,-1],[1,0],[1,0],[1,0],[1,2],[1,0],[1,0],[1,0],[1,0],[1,-1],[0,-1],[1,-1],[0,-2],[0,-1],[0,-7],[0,-2],[1,-1],[0,-1],[1,-4],[0,-1],[1,-3],[-1,-4],[-8,-33],[-2,-2],[-1,-3],[-1,-1],[-1,-1],[-8,-3],[-1,-1],[0,-2],[-1,-2],[0,-2],[0,-2],[0,-1],[0,-4],[3,-7],[1,-1],[1,-1],[3,-4],[0,-1],[1,-1],[1,-4],[0,-1],[1,-1],[2,0],[42,3],[3,2],[11,11],[3,0],[9,-6],[3,-1],[2,1],[2,0],[1,1],[4,-1],[1,0],[1,0],[0,1],[1,1],[1,2],[1,1],[0,1],[1,0],[1,0],[0,-2],[1,-1],[0,-2],[0,-1],[1,-2],[0,-1],[-1,-9],[0,-3],[0,-2],[1,-1],[0,-1],[1,-1],[2,-1],[4,-1],[10,-5],[2,-2],[2,-1],[6,1],[2,-2],[1,-2],[5,-2],[3,0],[6,0]],[[6022,1293],[1,-1],[2,-1],[2,-2],[1,-4],[0,-6],[-1,-4],[0,-1],[-1,0],[-2,-1],[-1,1],[0,1],[-2,9],[0,3],[-2,1],[-1,3],[1,2],[1,0],[0,1],[1,0],[1,-1]],[[6049,1355],[-3,-4],[-2,2],[-2,4],[-3,3],[5,8],[2,4],[2,1],[-1,-2],[2,-1],[2,0],[1,-1],[1,-3],[0,-3],[-1,-3],[-3,-5]],[[6013,1408],[2,-9],[1,-4],[2,-2],[1,3],[1,1],[2,1],[3,0],[-1,0],[3,1],[2,1],[1,2],[3,2],[3,0],[2,-2],[1,-5],[0,-5],[0,-4],[-3,-9],[-1,-4],[0,-3],[1,-3],[0,-2],[-2,-1],[-1,0],[-1,2],[-1,3],[0,3],[-1,2],[-2,4],[-4,3],[-2,1],[-13,0],[-5,-3],[-6,-4],[-5,-2],[-3,4],[2,1],[1,3],[-1,4],[-2,3],[3,1],[2,0],[2,0],[0,2],[-2,0],[-1,1],[0,2],[0,2],[3,-1],[3,-1],[3,1],[1,3],[-2,-1],[-2,-1],[-2,1],[-1,3],[9,0],[4,2],[3,4]],[[6028,1431],[2,0],[2,1],[4,3],[1,1],[1,0],[1,0],[3,-3],[1,0],[-3,-4],[0,-2],[-1,-3],[1,0],[4,9],[3,3],[3,-1],[0,-2],[-2,-4],[-4,-12],[-2,-5],[-4,-2],[-3,2],[-3,4],[1,4],[-2,1],[-1,1],[-1,1],[-1,3],[-1,0],[1,-4],[-3,0],[-2,2],[-1,2],[2,3],[-2,0],[-3,1],[-1,-1],[0,2],[3,1],[1,2],[2,2],[1,1],[4,0],[2,0],[0,-3],[-4,-1],[-2,0],[3,-2]],[[6267,1555],[1,-1],[1,1],[0,1],[1,0],[5,0],[2,-1],[1,-2],[0,-2],[-1,-4],[-3,-3],[-9,-3],[-2,0],[-1,0],[-2,-1],[-2,-2],[-3,-4],[-1,-1],[-6,-2],[-3,1],[1,1],[1,0],[1,1],[3,9],[0,4],[-1,6],[2,1],[6,0],[2,1],[3,3],[2,0],[0,-1],[1,-1],[1,-1]],[[6379,1550],[-2,-4],[-1,-1],[-5,0],[-2,1],[-2,1],[-1,-2],[-1,-4],[-1,-1],[-2,1],[-1,2],[0,2],[1,3],[6,6],[1,3],[-2,5],[4,1],[7,6],[4,2],[2,-4],[-1,-3],[-1,-2],[-1,-4],[2,-4],[0,-2],[-4,-2]],[[6418,1586],[0,-3],[0,-3],[-5,-3],[-2,-3],[-2,-2],[-2,-3],[0,-4],[1,-5],[1,-4],[-1,-5],[-2,-4],[0,-4],[3,-5],[-2,-3],[-1,0],[-4,1],[-1,1],[0,1],[-1,1],[0,4],[-1,0],[-1,0],[-1,1],[0,5],[5,9],[1,7],[-2,-1],[-4,-2],[-3,0],[-2,2],[1,4],[2,4],[4,4],[3,0],[2,11],[3,-2],[2,2],[2,0],[5,0],[-1,0],[3,-1]],[[6467,1611],[2,-1],[1,2],[1,0],[0,-1],[0,-2],[-2,-2],[-1,0],[-1,1],[-3,-3],[-3,2],[1,4],[2,-1],[1,2],[1,1],[1,-2]],[[6443,1608],[0,-1],[-1,1],[-1,4],[1,2],[2,1],[2,0],[0,-4],[0,-2],[-3,-1]],[[6484,1617],[-2,-2],[-1,1],[-3,3],[0,1],[-1,1],[-1,3],[2,5],[3,3],[1,1],[2,-1],[1,-3],[0,-2],[1,-1],[1,-1],[-1,-1],[0,-2],[-1,-2],[-1,-3]],[[6567,1643],[-2,-5],[-3,2],[-1,3],[-1,3],[2,0],[2,1],[2,0],[1,-4]],[[6481,1657],[0,-1],[-2,-3],[1,-1],[0,-2],[1,-1],[0,-1],[-2,-1],[-1,1],[-1,-1],[-4,-3],[-1,-2],[1,-2],[-1,-2],[-1,-2],[-1,0],[0,1],[-1,2],[2,17],[2,1],[6,0],[1,0],[1,0]],[[6653,1659],[-13,-6],[-3,1],[2,1],[9,5],[2,1],[1,1],[1,1],[0,2],[1,0],[1,0],[0,-2],[0,-1],[-1,-2],[0,-1]],[[6504,1666],[1,-1],[2,2],[0,-3],[-3,-10],[1,-1],[1,-1],[0,-1],[0,-2],[-13,1],[-2,3],[5,12],[2,1],[1,1],[2,1],[1,2],[1,1],[0,-1],[0,-1],[0,-1],[1,-1],[0,-1]],[[6528,1669],[-9,-3],[-2,1],[0,3],[0,3],[-1,1],[0,1],[1,2],[8,3],[1,0],[1,-1],[0,-2],[-1,-2],[1,-3],[3,-1],[-2,-2]],[[6500,1689],[-2,-8],[-3,-8],[-4,-6],[-4,-2],[-1,2],[-2,3],[-1,5],[1,4],[2,0],[1,1],[2,1],[1,1],[0,2],[0,2],[2,2],[1,1],[1,-4],[1,1],[2,2],[3,1]],[[6542,1763],[2,0],[2,0],[2,-2],[0,-2],[-1,-3],[-2,0],[0,-3],[-2,-1],[-1,-2],[-2,2],[0,4],[0,5],[2,2]],[[6545,1806],[-2,-2],[-4,5],[-7,11],[-6,8],[-6,6],[-2,3],[0,3],[4,2],[4,2],[3,-1],[2,-2],[1,-4],[2,-5],[5,-4],[5,-7],[1,-9],[0,-6]],[[6520,1883],[6,-12],[3,-3],[5,-2],[0,-1],[2,-8],[1,-3],[2,-3],[0,-2],[-2,-2],[-2,-1],[-4,1],[-2,3],[-1,4],[-2,5],[-1,-4],[-2,3],[-2,4],[-2,10],[-2,0],[0,-7],[-2,2],[-4,4],[-5,9],[-1,4],[1,3],[5,1],[7,-3],[2,-2]],[[6531,1881],[-2,-1],[-3,4],[-3,7],[0,8],[1,4],[3,-2],[0,-3],[2,-3],[2,-7],[0,-7]],[[7114,2035],[2,-2],[1,-3],[-1,-8],[-1,-5],[-4,1],[-4,2],[-5,-2],[-4,3],[-2,1],[-1,0],[-2,0],[-4,0],[-4,1],[0,2],[1,3],[3,3],[2,2],[1,-1],[1,-2],[1,-2],[2,0],[1,0],[1,1],[2,1],[1,4],[1,3],[2,1],[1,-1],[1,0],[3,-4],[2,2],[1,4],[2,0],[0,-4]],[[7095,2060],[0,-2],[-1,-2],[-1,-2],[0,-1],[-1,-3],[-3,-2],[-3,0],[-2,3],[-1,2],[2,1],[2,2],[2,2],[1,1],[2,1],[1,0],[2,0]],[[7084,2055],[-2,-1],[-1,1],[0,2],[0,1],[1,2],[2,1],[1,2],[0,2],[1,3],[0,1],[2,1],[1,0],[0,-1],[0,-3],[-2,-3],[0,-5],[0,-2],[-1,-1],[-2,0]],[[6592,2559],[1,0],[2,-1],[5,-5],[1,-1],[1,0],[1,1],[1,2],[1,0],[0,1],[1,-1],[1,-2],[-1,-2],[0,-6],[0,-1],[0,-3],[1,-4],[1,-5],[0,-3],[-1,-2],[-2,-5],[0,-3],[-1,-3],[0,-1],[1,-2],[1,-1],[1,0],[1,-1],[1,0],[1,1],[1,0],[3,2],[7,7],[1,1],[2,1],[2,0],[2,0],[1,-1],[2,-2],[1,0],[5,-1],[6,0],[6,-1],[1,0],[1,0],[1,1],[1,0],[3,4],[1,1],[2,1],[2,1],[2,1],[2,1],[1,1],[4,6],[2,5],[1,2],[1,1],[1,1],[1,-1],[2,0],[3,-1],[1,0],[1,0],[1,0],[1,0],[0,1],[1,0],[0,1],[2,4],[1,1],[1,1],[1,0],[2,1],[1,-1],[1,0],[1,-1],[1,-1],[0,-2],[0,-1],[1,-7],[0,-2],[1,-1],[0,-1],[1,-1],[2,-3],[3,-3],[2,-1],[9,-5],[1,-1],[1,-2],[1,-1],[1,-1],[0,-2],[0,-2],[1,-3],[0,-2],[0,-1],[-5,-19],[-1,-3],[0,-2],[1,-1],[0,-1],[1,-1],[5,-3],[1,-1],[0,-2],[0,-1],[-2,-2],[-1,0],[-1,-1],[-3,0],[-2,-1],[-1,0],[-1,-1],[-3,-5],[-2,-1],[-2,-1],[-7,-2],[-3,-2],[-1,0],[-1,-1],[-5,-7],[-7,-6],[-2,-1],[-2,-1],[-1,-1],[-2,-2],[-1,-1],[-1,0],[-1,-1],[-1,-1],[-1,-2],[-1,-4],[-1,-3],[0,-3],[0,-9],[-1,-3],[-1,-3],[-1,-3],[-2,-3],[-2,-4],[-1,-2],[-2,-9],[-1,-3],[-1,-2],[-2,-4],[-3,-4],[-2,-3],[-13,-10],[-2,-2],[-1,-1],[0,-1],[-1,-2],[0,-1],[1,-1],[1,-2],[5,-2],[5,-1],[1,0],[1,-2],[1,-1],[2,-4],[1,-4],[1,-2],[1,0],[4,-3],[6,-3],[2,-3],[1,-2],[2,-9],[1,-3],[2,-1],[1,-1],[1,-1],[2,0],[1,0],[1,1],[1,1],[1,2],[2,5],[1,2],[2,2],[2,1],[1,1],[4,0],[4,-1],[2,0],[2,0],[1,1],[7,6],[1,1],[2,1],[2,0],[0,-1],[1,-1],[1,-3],[0,-1],[1,-1],[1,-1],[3,-1],[1,0],[1,0],[1,1],[0,1],[1,1],[0,1],[1,3],[1,8],[0,3],[2,1],[2,2],[2,1],[1,0],[2,-2],[3,-1],[2,0],[1,-1],[1,-1],[1,-1],[0,-1],[1,-2],[1,-2],[1,-1],[2,-1],[1,1],[1,0],[0,1],[1,1],[1,2],[5,12],[1,3],[1,1],[1,1],[1,0],[12,5],[1,0],[1,1],[0,1],[1,1],[1,1],[1,1],[1,0],[1,0],[0,-1],[5,-4],[2,-1],[2,0],[4,0],[2,0],[1,1],[3,1],[0,1],[10,13],[2,2],[2,2],[1,0],[2,1],[1,0],[1,0],[1,0],[2,-1],[2,-3],[1,0],[1,0],[0,1],[1,3],[1,1],[7,3],[5,2],[1,-1],[1,0],[1,-1],[1,-1],[1,-3],[5,-12],[1,-1],[1,-1],[1,-1],[6,-6],[1,0],[1,-1],[0,-1],[1,-3],[1,-4],[4,-8],[2,-3],[8,-6],[2,-3],[4,-6],[1,-1],[1,-1],[1,0],[9,4],[2,0],[2,0],[1,0],[1,1],[1,1],[0,1],[-3,14],[-1,6],[1,10],[0,2],[0,1],[-1,1],[0,2],[-3,6],[-1,1],[0,1],[-1,1],[0,2],[1,1],[0,5],[1,2],[0,3],[-1,6],[0,2],[1,2],[2,5],[1,2],[2,2],[3,2],[1,1],[4,1],[4,-1],[4,-1],[1,0],[2,1],[0,1],[0,1],[0,1],[-2,4],[-1,2],[0,3]],[[7122,2074],[-3,-2],[-4,-9],[-5,-5],[-2,-1],[-1,4],[0,2],[1,4],[-2,4],[1,5],[0,4],[-2,4],[-2,0],[-2,1],[-2,1],[-1,0],[-1,-3],[-2,-1],[-4,-1],[-3,-7],[-2,-1],[-4,4],[-2,3],[4,6],[-2,1],[-2,0],[-1,-2],[0,-4],[-1,-2],[-4,-4],[-1,-3],[0,-2],[1,-2],[1,-1],[0,-4],[0,-4],[-1,-3],[-4,-6],[-1,-3],[1,-4],[3,-6],[-1,-6],[-15,-27],[-4,0],[-1,0],[-2,0],[-3,1],[-3,2],[-2,-1],[-4,-1],[-6,4],[-9,6],[-1,3],[-1,3],[-2,2],[-2,0],[-1,-1],[0,-2],[0,-9],[2,-4],[3,-5],[3,-5],[3,-2],[3,1],[5,0],[3,5],[5,0],[3,-5],[3,-1],[1,4],[3,-1],[3,-8],[0,-6],[1,-9],[2,-1],[1,1],[2,-1],[-3,-6],[-5,5],[-4,7],[-4,5],[-7,1],[2,-3],[3,-1],[2,-2],[1,-4],[-1,-2],[-3,-2],[-3,-4],[-1,-5],[-3,-8],[-1,0],[-2,2],[-2,2],[0,3],[0,3],[0,5],[-1,5],[-3,1],[-3,1],[-9,-3],[-1,-1],[0,-4],[3,-5],[2,3],[2,2],[2,1],[1,1],[5,-2],[0,-2],[0,-4],[-5,-5],[-4,-8],[-2,-10],[3,-6],[-1,-5],[1,-8],[1,-5],[-2,-3],[-4,-2],[-1,-8],[-4,-8],[-6,2],[-5,-2],[-8,-2],[-4,2],[-2,4],[-7,3],[-1,4],[-5,-2],[-4,2],[-4,1],[-2,-3],[6,-3],[3,0],[2,2],[2,-3],[-5,-7],[-4,-3],[-2,-3],[-2,-3],[-5,-8],[-4,-5],[-2,-2],[-3,-2],[-3,-2],[-2,7],[1,5],[6,7],[-6,3],[-8,7],[0,-3],[5,-7],[3,-2],[-2,-6],[-4,-3],[-1,-1],[-1,0],[-2,-2],[-3,-1],[-2,-1],[-5,-5],[-1,-3],[-3,0],[-2,0],[-4,-4],[-4,1],[-4,-4],[-2,-2],[-3,-7],[-3,0],[-2,2],[0,5],[-1,4],[0,3],[1,1],[1,-2],[0,-2],[1,-2],[1,-1],[0,3],[0,3],[-1,3],[-1,1],[-1,2],[-1,0],[-1,1],[-1,2],[-2,2],[-5,3],[-6,0],[1,5],[-3,2],[0,3],[1,3],[2,2],[2,-1],[1,0],[1,5],[1,2],[-1,1],[-6,-1],[-1,-2],[-3,-5],[-2,-3],[-3,-1],[-9,-2],[-1,-1],[0,-2],[-2,-4],[0,-1],[-4,0],[-1,3],[1,3],[1,4],[0,2],[-2,0],[-4,0],[-2,2],[-5,6],[-2,1],[-2,0],[0,-3],[-2,0],[-1,0],[-3,2],[2,-5],[2,-2],[1,-1],[2,1],[1,3],[2,0],[3,-3],[2,-5],[0,-5],[-5,-2],[0,-1],[2,-1],[5,1],[1,-1],[3,-2],[1,-1],[3,0],[1,1],[2,1],[2,2],[1,1],[1,2],[2,0],[-3,-4],[-6,-11],[-3,-4],[-2,-6],[0,-7],[5,-7],[1,6],[1,3],[3,-1],[0,-2],[-1,-4],[-3,-4],[0,-6],[2,-4],[-6,0],[-3,2],[-5,3],[-4,3],[-4,0],[-3,-3],[-5,0],[-3,1],[-2,3],[-2,4],[-2,4],[2,3],[2,1],[5,-1],[1,1],[2,2],[2,1],[0,3],[-2,4],[-4,-1],[-2,-3],[-2,-2],[-5,6],[-7,1],[-6,-1],[-3,0],[0,5],[4,4],[7,5],[3,3],[1,2],[-2,1],[0,2],[0,3],[1,6],[-2,0],[-2,-2],[-4,-6],[-2,-7],[-2,-3],[-2,-2],[-3,2],[-3,-1],[-4,-3],[0,-4],[1,-6],[-3,2],[-1,5],[-3,1],[-4,-3],[-2,0],[-5,-2],[-3,-3],[-1,-2],[0,-4],[-1,-2],[-2,-2],[-1,-2],[1,-3],[1,-2],[0,-3],[-2,0],[-2,-1],[-1,0],[-1,-2],[-1,-2],[-1,0],[-2,1],[-2,1],[0,2],[0,3],[-1,4],[-1,3],[-1,3],[-2,2],[-2,-4],[-2,1],[-2,4],[-2,1],[-3,-1],[2,-3],[2,-1],[0,-3],[-1,-2],[0,-2],[2,-2],[2,0],[1,2],[3,-2],[2,-1],[1,-4],[2,-3],[1,-2],[-2,-3],[-2,-2],[-2,-5],[-4,-11],[0,-5],[3,-5],[-4,-3],[-2,1],[-1,2],[2,3],[-1,5],[-3,3],[-5,0],[-5,-2],[-5,1],[-3,-2],[-2,5],[0,5],[0,5],[-1,6],[2,2],[1,4],[-3,5],[1,6],[1,4],[5,4],[3,2],[1,0],[1,1],[1,1],[1,3],[-1,3],[-2,1],[-1,1],[-1,0],[-1,2],[-2,0],[-1,-2],[-1,-5],[-1,-2],[-4,-2],[-2,4],[-4,1],[-2,-3],[-3,-2],[-3,-2],[-1,-3],[-4,-2],[-2,-2],[-4,-2],[-2,0],[0,-4],[1,-4],[-3,-2],[-2,0],[-1,1],[-2,2],[0,-5],[-1,-3],[-2,-4],[-1,-2],[1,-2],[5,-1],[-2,1],[4,-1],[2,1],[2,2],[2,0],[1,-3],[-2,-2],[-3,-7],[-2,-2],[-3,-1],[-3,-3],[-1,-3],[-2,-4],[2,-2],[1,-1],[1,1],[2,1],[1,2],[0,1],[1,0],[1,0],[1,-3],[1,0],[2,-1],[3,-3],[3,-3],[1,-4],[-1,-3],[-3,-2],[-3,-3],[-5,-2],[-2,-2],[-2,-4],[-3,-1],[-1,4],[-1,1],[0,1],[-1,3],[-1,1],[0,1],[1,2],[-1,3],[2,2],[0,1],[0,3],[-1,2],[0,1],[-1,2],[-2,2],[-1,3],[-1,0],[-1,0],[-1,-1],[0,3],[0,1],[-1,2],[-1,3],[-2,1],[-1,2],[-2,0],[-2,-1],[-4,-1],[-3,-2],[-2,2],[0,-2],[-2,1],[-2,-2],[1,-2],[-1,-1],[-4,1],[-1,-1],[-4,-3],[-2,-3],[-1,-1],[-1,0],[-4,0],[-3,0],[-1,2],[-2,-1],[-1,-2],[-1,-4],[-2,1],[-1,-1],[-2,-3],[-1,1],[-1,1],[-3,0],[-1,-1],[-1,-1],[1,-1],[-3,0],[-1,-1],[0,-3],[-3,0],[0,1],[-1,1],[-2,2],[-4,2],[-2,1],[-2,0],[-1,1],[-1,-2],[-1,-2],[-1,-3],[0,-3],[0,-2],[-2,-1],[-2,0],[-1,0],[-1,0],[0,-1],[0,-2],[-3,-6],[-2,5],[-1,1],[-1,5],[0,4],[2,1],[2,2],[-2,4],[-6,6],[-2,3],[-2,4],[-1,5],[-2,7],[-1,8],[-4,3],[-1,16],[-1,5],[-2,2],[-6,1],[-2,-1],[0,-1],[-1,-1],[-1,-2],[-2,0],[-3,4],[0,4],[1,6],[-1,6],[-2,4],[-2,5],[-2,9],[-1,-3],[0,-4],[0,-4],[1,-3],[-2,3],[-1,5],[1,6],[1,4],[-1,-2],[-1,-2],[-1,-2],[0,2],[3,12],[1,6],[0,6],[1,5],[2,10],[-1,0],[-2,-8],[-1,-3],[0,-8],[-2,-9],[0,-3],[-1,3],[0,4],[1,4],[1,3],[0,4],[0,4],[2,9],[-3,-7],[-2,-15],[-2,-4],[1,5],[0,6],[0,11],[-3,-6],[-1,-3],[-2,0],[-2,2],[-1,4],[0,7],[2,7],[3,6],[4,3],[9,3],[9,6],[5,2],[2,-1],[2,-2],[2,-1],[4,2],[3,-1],[5,-3],[-3,4],[-3,1],[-6,1],[-2,1],[-4,3],[-3,1],[-5,-2],[-8,-8],[-5,-2],[-1,-2],[-6,-4],[-1,-1],[-1,-3],[-2,-2],[-3,-1],[-2,0],[-8,7],[-5,2],[-3,-4],[4,-4],[5,-1],[5,-3],[3,-8],[0,-8],[-2,-10],[-1,-5],[0,-1],[0,-2],[-2,-1],[-4,2],[-3,1],[-15,-9],[-5,-1],[-15,5],[0,-2],[3,-1],[4,-1],[1,0],[3,-4],[2,-1],[3,1],[3,2],[3,0],[3,-2],[10,-17],[2,-3],[9,-8],[-5,-3],[-2,-2],[-2,-5],[-2,3],[-8,5],[3,-5],[9,-6],[2,-3],[2,-4],[6,-8],[6,-8],[2,-7],[6,-10],[-1,-14],[-5,-11],[-3,-11],[1,-11],[1,-4],[4,-3],[5,0],[-2,-5],[0,-3],[0,-4],[-1,-3],[-2,-1],[-1,-3],[0,-4],[0,-5],[1,0],[1,-3],[0,-4],[-4,-1],[-2,-4],[-2,0],[-2,-2],[1,-3],[-1,0],[-1,-3],[-4,-5],[-2,-4],[-4,1],[-5,7],[-1,6],[-2,4],[-7,12],[-1,2],[0,4],[-1,3],[-1,1],[-7,10],[-5,7],[-3,9],[-2,14],[-2,6],[-9,22],[-2,3],[-1,-18],[0,-5],[1,-3],[4,-4],[6,-10],[13,-30],[2,-6],[2,-5],[4,-7],[2,-3],[6,-14],[-7,-2],[-2,0],[-4,3],[-2,1],[-1,-1],[-1,-1],[-1,-3],[-1,-4],[-1,-2],[-1,-1],[0,-1],[-2,0],[-2,-6],[-1,-3],[2,-5],[0,-3],[-1,-3],[-1,-2],[-2,-3],[-3,1],[-3,0],[-3,-1],[-3,-5],[-3,4],[-4,7],[-1,9],[1,4],[1,5],[-1,3],[2,4],[-1,3],[-1,4],[-1,5],[0,4],[-4,-3],[-1,-16],[-4,-7],[-5,6],[-1,-5],[1,-6],[-1,-6],[0,-6],[0,-6],[0,-4],[1,-5],[-1,-4],[0,-2],[0,-2],[2,-2],[-1,-4],[-1,-1],[-3,-1],[-1,0],[-1,-1],[0,-2],[0,-2],[-1,-4],[0,-1],[-1,-3],[-3,-4],[-5,-3],[-2,-2],[-6,5],[-1,5],[-2,10],[-1,4],[-2,3],[-2,2],[-2,-1],[-1,-3],[-2,-1],[-2,1],[-7,-5],[0,-2],[-1,-7],[-1,-2],[-4,0],[-1,0],[-1,-1],[0,-3],[-1,-2],[-7,-5],[0,-7],[-2,-7],[-4,-6],[-5,-2],[-4,2],[-18,14],[-3,5],[-2,6],[0,9],[2,4],[9,7],[2,6],[-1,0],[-3,-3],[-5,-3],[-5,0],[-3,3],[0,2],[-2,4],[0,2],[0,2],[1,4],[2,13],[1,3],[-4,-3],[-2,-6],[-1,-8],[0,-7],[2,-10],[3,-8],[0,-2],[0,-7],[0,-4],[1,-1],[1,-2],[0,-3],[1,-7],[-1,-3],[-1,-4],[-1,-8],[-1,-2],[-2,-1],[-1,-3],[-5,-2],[-8,-4],[-6,-1],[-3,6],[-2,2],[-3,2],[-3,2],[1,5],[2,4],[1,6],[-1,5],[-1,4],[-2,-5],[-1,-2],[-14,-5],[-7,0],[-3,0],[-2,2],[-1,4],[1,4],[4,7],[0,6],[-1,0],[-2,-6],[-5,-7],[-3,-3],[-3,11],[-3,7],[-4,6],[-3,3],[-1,-7],[1,-8],[1,-8],[1,-6],[3,-4],[4,-4],[4,-2],[5,-1],[-2,-3],[-12,-12],[-2,0],[0,-2],[0,-2],[-1,-1],[0,-1],[-2,-3],[-2,-1],[-1,-1],[-5,1],[0,2],[1,3],[2,2],[2,-2],[1,0],[0,3],[-1,2],[-1,1],[-2,1],[-2,-2],[-1,3],[0,5],[1,5],[-2,-4],[-3,-2],[-2,0],[-1,4],[-3,-1],[-2,0],[-5,3],[-1,-1],[-3,2],[-4,3],[3,-5],[10,-10],[2,-5],[2,-8],[-1,-7],[-5,-1],[2,-1],[2,-2],[2,-2],[1,-3],[-7,0],[-3,0],[-2,2],[-4,-5],[-4,-12],[-2,-5],[-2,-2],[-2,-1],[-2,1],[-2,3],[-1,3],[0,3],[-1,3],[-2,1],[0,-3],[0,-4],[1,-3],[1,-2],[-2,-2],[-4,-2],[-3,0],[-2,4],[-1,0],[0,-3],[-2,-2],[-3,-1],[-2,0],[-1,1],[-1,3],[1,7],[-2,-1],[-2,3],[-2,11],[-2,-4],[-2,-3],[0,-2],[1,-3],[3,-4],[1,-2],[-9,2],[-4,-1],[-2,-3],[2,1],[3,0],[4,-1],[-2,-2],[-5,-4],[-1,-1],[-2,-4],[-8,-10],[-4,-3],[-3,2],[3,2],[2,3],[3,9],[-3,2],[-4,3],[-4,2],[-3,-1],[-1,3],[-2,0],[-6,-3],[0,-2],[2,0],[-1,-1],[0,-1],[1,-2],[2,2],[3,1],[2,-1],[2,-2],[-7,-3],[-3,-3],[-3,-3],[-2,2],[-2,0],[-5,0],[0,2],[6,4],[0,1],[0,1],[-1,1],[-1,-1],[-1,-2],[-2,0],[-1,0],[-2,1],[0,1],[0,2],[-1,1],[-6,5],[-1,2],[-1,-3],[-2,-2],[-4,-2],[1,-1],[1,-5],[2,-1],[-2,-6],[-1,-2],[-2,-1],[0,-2],[2,0],[2,1],[2,2],[0,3],[1,3],[3,1],[3,0],[1,-3],[-4,-1],[-3,-3],[-5,-8],[-4,2],[-5,-3],[-3,-4],[-4,-3],[-1,0],[-2,-3],[-1,0],[-3,1],[-4,1],[-4,3],[-2,0],[-4,-2],[-4,-4],[-6,-9],[-3,-4],[-2,-6],[-2,-6],[-1,-7],[-1,0],[1,14],[1,6],[2,3],[-2,0],[-1,-1],[-1,-3],[-1,-3],[0,4],[-1,3],[-1,2],[1,4],[1,1],[4,2],[4,5],[5,1],[5,3],[2,8],[-1,0],[-5,-7],[-15,-10],[-4,-9],[0,-12],[1,-3],[4,-10],[-1,-6],[-4,0],[-6,5],[0,-2],[0,-2],[1,-1],[1,-1],[-18,-2],[-3,-2],[-2,-2],[-3,-3],[-1,0],[0,4],[2,5],[0,4],[-1,3],[-1,2],[0,2],[2,2],[0,1],[-1,1],[0,1],[-1,0],[0,1],[1,0],[0,1],[1,0],[2,-1],[1,-1],[2,-1],[1,-2],[1,0],[0,2],[0,1],[1,2],[0,2],[-1,0],[-1,-2],[-1,1],[-2,0],[-2,1],[0,2],[7,5],[0,2],[-1,0],[-1,0],[-1,-1],[0,-1],[0,5],[-3,-4],[-1,-5],[-1,-1],[-2,5],[0,4],[2,14],[-1,0],[-2,-5],[-3,3],[-5,10],[2,3],[0,1],[1,1],[0,2],[-5,-4],[2,-9],[4,-10],[-1,-10],[0,1],[-2,1],[1,-4],[-2,1],[-4,4],[-3,1],[0,-2],[2,-1],[3,-3],[2,-4],[-1,-3],[0,-2],[2,-2],[2,-2],[1,-3],[1,-3],[0,-1],[1,-2],[1,-1],[0,-3],[-1,-1],[0,-2],[0,-1],[-1,-4],[0,-4],[-1,-4],[-1,-1],[-2,-1],[-5,-5],[-2,-5],[-3,-2],[-6,-3],[-1,-2],[-8,-15],[-1,-1],[-1,1],[-1,4],[0,5],[0,4],[-1,-3],[-1,-3],[-1,-7],[-1,-3],[-4,-9],[-2,1],[0,-1],[2,-3],[0,-4],[-1,-5],[0,-5],[1,-4],[3,-8],[1,-4],[-2,3],[-1,1],[-1,0],[1,-3],[0,-1],[0,-2],[-1,-2],[1,0],[0,-2],[1,1],[0,1],[2,0],[1,0],[1,0],[0,-3],[3,1],[13,0],[2,3],[3,2],[3,0],[1,-5],[1,2],[1,-2],[1,-1],[0,1],[1,2],[3,-6],[2,-9],[2,-11],[-2,-8],[-3,-2],[-2,2],[-2,3],[-1,4],[-1,4],[1,11],[-1,5],[-2,-4],[-1,-2],[-1,-1],[2,-4],[0,-3],[-2,-2],[-2,-2],[1,-2],[-1,0],[0,-2],[5,-3],[-2,-5],[-3,-6],[-1,-7],[1,0],[1,2],[1,1],[0,2],[1,0],[1,-4],[-1,-3],[0,-2],[-2,-2],[2,-1],[2,0],[2,1],[2,2],[0,-6],[1,4],[2,-1],[1,-2],[2,-2],[-1,-3],[1,-3],[1,-2],[3,-2],[3,-2],[1,-1],[2,0],[2,0],[1,0],[3,-4],[2,-7],[4,-13],[-3,-11],[-2,-6],[-3,-3],[-2,-1],[-2,-9],[-9,-16],[-1,-1],[-4,1],[-2,0],[-1,-2],[-2,-4],[-4,-3],[-4,-2],[-3,-2],[-2,3],[-1,1],[-2,1],[-2,0],[-4,0],[-2,-1],[-1,-2],[0,-2],[-1,0],[-10,-2],[-2,2],[-3,4],[-1,1],[-2,0],[-1,1],[-1,2],[0,1],[-1,2],[-1,1],[-2,1],[-2,-1],[-1,-1],[-1,-3],[-1,-1],[-1,0],[-1,1],[-1,0],[-4,-1],[-2,-2],[-1,-5],[-1,-1],[-1,-2],[-2,-1],[-1,0],[0,2],[0,3],[1,3],[0,1],[1,6],[0,6],[0,2],[-2,4],[0,3],[-5,4],[-2,2],[3,2],[-5,6],[1,1],[2,0],[1,1],[1,2],[1,3],[1,0],[2,-4],[5,-6],[2,-3],[3,-7],[1,-1],[0,5],[1,4],[3,4],[1,4],[-1,5],[-4,5],[-2,3],[-1,3],[0,-1],[-1,-2],[0,-2],[0,-2],[-3,3],[0,3],[-1,4],[-1,5],[0,-10],[0,-4],[-2,-3],[0,4],[-1,3],[-1,4],[-1,2],[-3,1],[-5,1],[-2,2],[-2,1],[0,1],[-1,2],[0,3],[1,2],[1,1],[1,0],[1,0],[0,3],[0,6],[1,4],[-1,0],[-3,-11],[-1,0],[0,10],[-3,7],[-4,4],[-5,-1],[1,4],[0,4],[0,4],[2,3],[2,0],[1,0],[1,-2],[1,0],[2,0],[1,2],[2,3],[1,-1],[1,-1],[1,-1],[0,1],[1,2],[1,1],[1,1],[0,1],[0,2],[-1,1],[-4,-3],[-3,0],[-2,0],[-1,-4],[-1,0],[-5,10],[0,5],[3,5],[-2,2],[-1,4],[0,4],[3,2],[0,2],[-4,5],[0,2],[1,0],[0,2],[-1,2],[-1,1],[-1,0],[-1,-3],[1,-13],[-2,2],[-2,5],[-2,2],[1,-2],[2,-7],[-5,8],[-4,12],[-1,12],[5,8],[-2,4],[0,6],[3,12],[1,-1],[1,0],[1,-1],[-1,6],[-1,7],[-3,6],[-2,5],[1,4],[3,8],[1,2],[3,2],[1,-2],[2,-2],[2,-1],[-1,4],[2,3],[2,3],[1,3],[-1,15],[-1,3],[0,1],[2,4],[0,5],[-2,9],[2,0],[1,-2],[1,-2],[0,-3],[2,1],[1,3],[0,2],[0,1],[2,1],[4,-1],[13,2],[0,2],[-3,6],[2,6],[4,6],[2,6],[-3,-2],[-2,3],[1,4],[3,2],[0,2],[-5,-1],[-5,-2],[-4,0],[-3,6],[-3,-3],[0,-2],[2,0],[0,-2],[-2,-2],[0,-2],[-1,-1],[-1,0],[-2,3],[0,7],[0,15],[-3,-2],[-2,1],[-1,3],[2,3],[0,2],[-5,-2]],[[6276,2371],[4,1],[1,3],[4,8],[1,2],[1,2],[4,3],[2,1],[1,0],[1,-1],[0,-1],[1,-2],[1,-1],[0,-1],[1,-1],[1,-1],[1,1],[0,2],[0,3],[1,7],[3,15],[0,2],[0,1],[0,2],[-1,1],[0,1],[-1,1],[-3,3],[-1,1],[-1,2],[0,1],[-1,1],[1,3],[0,5],[4,14],[1,4],[0,3],[0,3],[0,3],[4,19],[1,6],[0,3],[0,2],[1,1],[1,1],[1,0],[2,0],[0,-1],[1,-1],[4,-4],[1,0],[1,-1],[1,0],[11,2],[8,-5],[3,-1],[3,1],[3,1],[2,-1],[1,0],[8,-4],[4,-3],[1,-1],[2,0],[10,1],[2,-1],[3,-1],[2,-2],[2,-3],[3,-5],[1,-2],[0,-2],[1,-3],[0,-1],[0,-1],[0,-6],[0,-2],[1,-1],[0,-1],[0,-2],[2,-3],[0,-1],[10,-9],[3,-4],[1,-1],[2,-1],[4,0],[3,-1],[4,1],[1,1],[7,3],[2,1],[5,0],[3,1],[1,2],[0,1],[0,1],[-3,6],[-1,2],[-1,4],[-1,4],[0,2],[0,2],[1,4],[2,8],[0,1],[0,2],[-1,13],[0,3],[0,2],[1,2],[2,3],[3,2],[1,1],[1,1],[1,3],[1,1],[0,2],[-1,1],[-1,1],[-1,1],[-1,0],[-5,1],[-2,0],[-1,0],[-9,-3],[-1,0],[-1,0],[-1,2],[-1,1],[-2,4],[-1,2],[0,3],[-1,8],[0,2],[0,1],[0,2],[1,1],[1,1],[1,1],[1,-1],[1,0],[0,-1],[1,-2],[1,-1],[0,-1],[1,-1],[1,0],[1,1],[1,0],[2,4],[2,3],[1,1],[5,2],[3,1],[2,1],[8,8],[5,4],[9,14],[6,7],[3,2],[3,1],[8,2],[1,1],[1,0],[1,0],[1,0],[0,-2],[0,-3],[0,-2],[0,-1],[0,-2],[1,-2],[1,-2],[8,-10],[6,-4],[1,0],[0,-1],[1,-1],[0,-1],[1,-2],[0,-1],[1,0],[1,-1],[7,-2],[1,0],[2,1],[3,1],[1,1],[2,0],[1,0],[0,-1],[1,-1],[1,-4],[1,-2],[1,-1],[2,-2],[0,-2],[0,-1],[0,-1],[0,-2],[1,0],[2,1],[9,6],[7,3],[2,1],[2,0],[2,-1],[8,-5],[1,0],[2,0],[4,2],[2,1],[1,1],[2,3],[1,2],[1,2],[2,8],[1,2],[2,3],[1,1],[1,1],[6,2]],[[3942,4436],[16,-8],[4,0],[3,2],[5,-6],[20,-9],[9,-11],[2,-2],[0,-2],[2,-1],[4,0],[1,-1],[1,-1],[1,-2],[2,-1],[7,-5],[3,-2],[1,-5],[0,-11],[0,-6],[1,-6],[9,-16],[6,-16],[10,-13],[2,-5],[1,-6],[8,-10],[2,-5],[1,-6],[-1,-7],[-1,-7],[-2,-4],[-2,-3],[0,-1],[2,-10],[0,-4],[-1,-3],[3,-2],[4,-10],[5,-5],[7,-16],[1,-1],[1,-1],[0,-1],[1,-2],[1,-2],[0,-8],[1,-4],[3,-5],[41,-53],[3,-8],[1,-1],[3,-2],[-2,-4],[-7,-11],[-5,-11],[-3,-5],[-2,-2],[-1,1],[-1,3],[-4,8],[-2,2],[-1,2],[-5,6],[-3,2],[-3,-1],[-2,-2],[-2,-6],[-2,-8],[1,-18],[-2,-5],[1,-1],[5,-2],[2,-1],[6,-8],[5,-16],[1,-1],[3,-5],[3,-7],[2,-2],[3,-3],[1,-1],[1,-7],[-1,-7],[-2,-6],[-2,-4],[2,-6],[2,-3],[5,-6],[8,-14],[3,-4],[7,-5],[2,-3],[1,-5],[-1,-5],[-3,-2],[-3,-2],[-2,-3],[0,-5],[3,-11],[1,-5],[0,-3],[0,-3],[0,-21],[1,-3],[0,-1],[2,-2],[1,-2],[1,-3],[4,-83],[7,-21],[1,-3],[1,-21],[1,-8],[2,-8],[-7,-16],[-2,-9],[0,-10],[0,-1],[2,-2],[0,-1],[0,-1],[-2,-1],[0,-1],[-2,-18],[-1,-10],[1,-4],[4,-8],[2,-6],[2,-4],[0,-1],[0,-10],[3,-29],[-1,-8],[1,-5],[1,-5],[0,-11],[1,-3],[3,-8],[1,-4],[2,-7]],[[4169,3558],[-8,-2],[-2,-2],[-3,0],[-4,-1],[-1,0],[-1,-1],[-2,-2],[-1,-3],[0,-6],[1,-5],[1,-4],[2,-5],[1,-4],[1,-5],[0,-4],[0,-4],[-1,-2],[-2,-3],[-7,-6],[-1,-1],[-1,-3],[-3,-6],[-1,-7],[-1,-4],[1,-2],[1,-4],[3,-5],[2,-3],[1,-2],[-1,-3],[0,-2],[-6,-5],[-2,-2],[-4,-1],[-2,0],[-2,2],[-1,5],[-4,13],[-6,30],[-1,1],[-1,0],[-1,0],[-1,-1],[-1,-2],[-2,-1],[-2,-1],[-10,-3],[-2,-1],[-1,-2],[-1,-2],[-1,-4],[0,-3],[1,-13],[0,-2],[2,-5],[0,-1],[0,-1],[0,-3],[0,-2],[0,-2],[1,-2],[1,-2],[2,-12],[0,-3],[0,-1],[-1,-2],[-9,-8],[-2,-3],[-1,-3],[-2,-3],[0,-6],[0,-6],[1,-11],[3,-12],[4,-10],[1,-4],[0,-4],[0,-3],[0,-4],[0,-6],[-2,-7],[-3,-8],[-6,-13],[-3,-4],[-11,-14],[-2,-2],[-5,-2],[-2,1],[-4,4],[-6,10],[-4,9],[-2,3],[-2,2],[-4,3],[-3,-10],[-2,-14],[-1,-6],[-2,-6],[-3,-4],[-3,-3],[-13,-9],[-3,-4]],[[4007,3272],[-11,15],[-3,2],[-6,2],[-1,3],[1,7],[0,4],[-2,3],[-6,5],[-8,15],[-6,4],[-10,-7],[-5,2],[-4,8],[-7,28],[-5,6],[-5,-3],[-4,-4],[-4,3],[-4,8],[-3,4],[-3,-3],[-2,-9],[-2,-4],[-3,-10],[-2,-4],[0,-3],[-1,-2],[-3,-4],[-1,-2],[1,-3],[0,-2],[1,-3],[0,-10],[-1,-5],[-2,-3],[-3,0],[-3,1],[-3,-1],[-1,-6],[-1,-3],[-6,-5],[-1,-4],[-7,5],[-9,10],[-1,2],[-2,7],[-1,1],[-2,0],[-1,1],[-11,13],[-7,2],[-7,-4],[-6,-6],[-5,-1],[-5,1],[-6,5],[-2,19],[-8,14],[-11,10],[-12,19],[-14,5],[-9,-6],[-5,-14],[-1,-12],[-1,-15],[-3,-2],[-5,17],[-6,8],[7,12],[-6,13],[11,9],[16,17],[-1,13],[-3,13],[-8,15],[-2,5],[-2,12],[-1,5],[-3,4],[-3,0],[-1,2],[0,3],[1,3],[-15,11],[-2,17],[-12,-6],[-14,-13],[-3,22],[1,4],[2,3],[3,2],[17,7],[2,3],[2,8],[4,8],[0,4],[-3,4],[-2,2],[-2,-1],[-3,-4],[-6,-5],[-5,0],[-5,4],[-5,8],[-2,4],[-2,13],[-2,4],[-11,0],[-10,-1],[-9,-1],[-15,-10],[-11,8],[-3,-2],[-6,-1],[1,-18],[-5,-14],[-12,2],[-8,6],[-2,-1],[-2,0],[-1,0],[-2,-2],[0,-2],[0,-3],[0,-2],[-6,-1],[-1,-3],[2,-10],[0,-6],[-2,-1],[-6,1],[-1,-4],[-3,7],[-1,10],[-3,-2],[-2,-13],[-7,3],[-10,-1],[-5,-6],[1,-16],[-10,1],[-1,2],[0,2],[0,2],[-1,2],[-2,2],[-1,0],[-12,3],[-3,2],[-8,7],[-10,4],[-3,1],[-3,3],[2,-6],[3,-4],[1,-2],[-4,1],[-12,8],[-6,3],[-6,-1],[-3,0],[-4,0],[-2,2],[1,10],[-2,5],[-9,14],[-5,6],[-7,3],[-5,0],[-3,-4],[-8,-19],[-3,-3],[-2,-1],[-4,-1],[-9,-1],[-4,0],[-4,-3],[-1,-3],[-3,-7],[-2,-3],[-2,-1],[-6,1],[-2,-13],[6,-10],[6,-10],[-9,-12],[-7,-12],[-12,-1],[-7,-8],[-9,-4],[-8,-5],[-3,-3],[-3,-2],[-3,-3],[-2,1],[-4,0],[-3,-1],[-2,-2],[0,-2],[1,-2],[0,-2],[-1,-2],[-1,0],[-2,0],[-2,-1],[0,-3],[-2,-2],[-9,-4],[-3,-2],[-11,-14],[-3,-2],[-3,2],[-2,1],[-2,-2],[-3,-4],[-3,-2],[-3,0],[-9,2],[-17,-2],[-15,-13],[-6,-5],[-9,-11],[-7,-5],[-6,-7],[-5,-6],[-1,-9],[-2,-9],[-3,-9],[-4,-7],[-15,-15],[-4,-7],[-3,-3],[-6,-3],[-3,-3],[-2,-3],[-4,-8],[-3,-4],[-2,-2],[-3,-1],[-2,-1],[-2,1],[-1,1],[-1,2],[0,1],[-2,-1],[-10,-10],[-5,-6],[-4,-7],[-3,-9],[1,-4],[2,-1],[6,-1],[2,-3],[0,-3],[-2,-8],[0,-7],[-4,-8],[-6,-6],[-4,-3],[-4,-1],[-2,-2],[-5,-6],[-2,-3],[-8,-3],[-6,-7],[-2,0],[-1,2],[-3,7],[-1,0],[-2,-3],[-1,-6],[-2,-5],[-2,0],[-2,3],[-1,5],[-1,0],[-2,-8],[-2,-4],[-2,1],[-3,5],[-2,10],[-2,4],[-2,1],[-1,-5],[0,-2],[0,-3],[1,-2],[-1,-3],[-1,-1],[-2,0],[-14,2],[-3,-1],[-6,-4],[-10,-14],[-6,-4],[-3,-1],[-8,2],[-7,0],[-2,1],[-6,5],[-7,3],[-23,-1],[-1,18],[1,6],[1,7],[2,5],[1,5],[-3,6],[-3,3],[-3,2],[-4,2],[-6,1],[-7,4],[-6,0],[-2,1],[-2,0],[-3,2],[-12,9],[-6,7],[-3,9],[-3,-5],[-4,0],[-4,1],[-4,-1],[-3,-4],[-7,-11],[-3,-5],[-4,-6],[-7,-3],[-7,1],[-5,4],[-3,4],[-8,10],[-10,3],[-10,1],[-5,2],[-5,2],[-7,6],[-2,1],[-3,-1],[-5,-3],[-4,-1],[-4,-1],[-10,2],[-3,0],[-3,-1],[2,14],[5,8],[5,1],[3,9],[5,2],[-2,9],[-6,3],[-7,1],[-8,9],[-7,0],[-4,4],[-6,0],[-4,-11],[-6,1],[-5,10],[0,10],[-6,6],[-4,-3],[-4,-6],[-4,-1],[-4,4],[-9,3],[-13,-7],[-3,-1],[-3,-2],[-6,2],[-6,-5],[-1,1],[-2,-2],[-1,-2],[-3,-5],[-1,-2],[-6,-5],[-2,-3],[-2,-4],[-3,-8],[-6,-4],[-19,-5],[-3,-2],[-3,-2],[-8,-14],[-3,-4],[-3,-2],[-3,-3],[-2,-5],[-4,-9],[-1,-2],[-8,-21],[-5,-10],[-6,-7],[-7,-4],[-5,-6],[-2,-9],[-1,-7],[-2,-4],[-7,-17],[-4,-10],[-1,-4],[-1,1],[-1,1],[-1,0],[0,1],[-2,0],[-5,-2],[-2,-2],[-2,-4],[-1,-4],[-2,-1],[-3,-4],[1,-3],[-2,-3],[-3,-2],[0,-3],[0,-3],[1,-3],[0,-1],[-3,-4],[-1,-5],[-1,-7],[2,-10],[3,-11],[2,-6],[-4,-5],[-3,4],[-6,1],[-4,-4],[-4,5],[-2,2],[-4,7],[-2,2],[-1,1],[-3,2],[-3,6],[-2,7],[-1,7],[0,9],[0,4],[-2,5],[0,4],[1,4],[3,7],[2,9],[4,9],[8,50],[0,4],[0,5],[-2,4],[-4,6],[-1,3],[0,5],[1,8],[0,5],[-2,4],[-4,5],[-7,7],[-4,2],[-10,2],[-3,3],[-4,6],[-2,0],[-7,-7],[-4,-5],[-2,-5],[-2,-3],[-5,2],[-3,-1],[-9,-10],[-4,-3],[-29,-7],[-4,-2],[-2,-1],[-2,0],[-4,1],[-2,-1],[-2,-5],[-1,-6],[2,-7],[2,-5],[-4,2],[-7,5],[-4,2],[-2,0],[-4,-3],[-2,0],[-2,0],[-3,3],[-2,1],[-13,0],[-3,3],[-2,-1],[-2,-4],[-2,-7],[-4,-6],[-3,-5],[-5,-4],[-4,0],[-4,2],[-3,2],[-4,1],[-2,-2],[-6,-3],[-2,0],[-2,1],[0,2],[-1,2],[-1,2],[-3,1],[-3,0],[-6,-3],[-15,-6],[-3,0],[0,4],[2,5],[1,3],[-4,2],[-4,-2],[-4,-5],[-3,-3],[-10,3],[-8,-1],[-5,0],[-6,5],[-9,17],[-5,8],[-4,3],[-3,1],[-1,-1],[-2,-1],[-5,-1],[-4,1],[-3,2],[-3,4],[-2,7],[-4,5],[-12,-1],[-5,3],[-1,3],[1,8],[-1,4],[-2,3],[-3,2],[-3,2],[-3,0],[-2,-1],[-4,-5],[-2,-1],[-3,2],[-2,3],[-2,2],[-3,0],[-1,-3],[-1,-4],[0,-4],[0,-4],[-2,-4],[-1,-3],[-1,-3],[0,-4],[0,-6],[-1,-5],[-2,-4],[-4,-3],[-6,-4],[-2,-1],[-3,2],[-6,7],[-1,1],[-2,0],[-1,-1],[-1,-1],[-2,0],[-1,2],[-4,7],[-3,3],[-3,1],[-3,0],[-4,2],[-2,2],[-1,3],[-2,8],[-1,3],[-1,2],[0,2],[1,5],[1,5],[0,7],[-2,5],[-3,1],[-14,-21],[0,-3],[2,-4],[0,-2],[0,-2],[0,-4],[0,-2],[-1,-2],[1,-1],[1,-2],[2,-3],[0,-2],[-1,-2],[0,-3],[1,-2],[1,-2],[1,-1],[0,-3],[-1,-1],[-5,-8],[-2,-1],[-2,0],[0,-1],[-4,1],[-8,-1],[-3,1],[-1,7],[0,5],[0,6],[0,5],[-2,5],[-5,7],[-2,4],[-1,6],[0,3],[0,3],[-1,2],[-2,6],[-1,5],[-1,3],[-2,4],[-4,3],[-7,5],[-4,4],[-4,5],[-3,6],[-3,6],[0,12],[-1,6],[-2,-1],[0,-2],[0,-2],[-1,-3],[-1,-1],[-3,-2],[-1,-2],[-5,-9],[-2,0],[-2,3],[-3,3],[-4,1],[-4,1],[-4,0],[-3,-1],[-5,4],[-2,1],[-3,0],[-2,-2],[-4,-6],[-3,-2],[-2,0],[-2,2],[-2,3],[-2,1],[-2,0],[-4,-3],[-2,-1],[-3,0],[-3,1],[-3,2],[-6,7],[-3,-1],[-3,-3],[-4,1],[-4,3],[-1,4],[0,3],[1,8],[0,5],[-1,14],[1,3],[4,4],[1,3],[0,5],[0,2],[1,2],[3,1],[1,2],[1,2],[1,2],[2,14],[-2,3],[-8,3],[-6,6],[-3,2],[-4,-1],[-1,-2],[-1,-2],[-1,-4],[-1,-3],[-2,0],[-1,-1],[-5,-4],[-1,-1],[-2,-7],[-2,-1],[-1,-1],[-2,0],[-3,2],[-2,0],[-9,-1],[-2,1],[-2,1],[-3,3],[-4,7],[-3,2],[-7,1],[-3,2],[1,12],[-2,3],[-3,0],[-3,2],[-1,2],[-2,5],[-1,1],[-9,2],[-3,2],[-3,2],[-3,2],[-4,0],[-4,1],[-2,6],[-2,12],[-2,2],[-3,4],[-3,7],[-1,2],[-2,1],[-12,3],[-8,7],[-2,4],[-3,7],[4,23],[-1,3],[-4,2],[-4,3],[-1,8],[0,3],[-1,2],[-4,5],[-1,5],[2,4],[2,4],[1,5],[-1,3],[-2,1],[-3,-1],[-2,1],[-1,3],[-1,7],[-2,3],[-3,0],[-3,-2],[-3,-1],[-2,4],[-1,2],[-2,1],[-2,-1],[-2,-1],[-1,3],[-1,2],[-1,1],[-2,1],[-2,0],[-3,-1],[-5,-2],[-1,-1],[-2,-4],[-1,0],[-4,-2],[-7,-5],[-4,-1],[-4,1],[-4,0],[-4,-2],[-3,-5],[-2,-4],[0,-5],[-2,-4],[-2,-1],[-3,0],[-6,2],[-2,1],[-2,2],[-4,1],[-2,1],[-1,3],[-1,2],[0,6],[-2,10],[-1,4],[-3,3],[-5,2],[-1,3],[-2,7],[-2,2],[-2,1],[-2,3],[0,4],[0,3],[-1,3],[-4,6],[-1,3],[1,4],[-1,5],[-4,6],[-11,-3],[-2,6],[0,3],[1,2],[1,1],[0,2],[0,4],[-2,4],[-3,5],[-2,1],[-2,2],[-2,1],[0,1],[-1,3],[0,2],[-2,2],[-2,1],[-2,0],[-4,0],[-4,-2],[-2,0],[0,-3],[-1,-4],[-1,-3],[-2,0],[-2,2],[-2,6],[-1,3],[-10,11],[-2,1],[-3,-1],[-2,1],[-1,1],[-3,4],[-1,1],[-3,-2],[-4,-4],[-4,-2],[-2,5],[-1,3],[-2,2],[-2,1],[-2,1],[-1,3],[-3,6],[-10,11],[-2,5],[1,3],[0,3],[0,2],[-2,2],[-1,0],[-4,-2],[-1,0],[-3,3],[-13,22],[-3,2],[-8,4],[-3,2],[-3,3],[-2,4],[-2,4],[-3,2],[-2,0],[-3,0],[-3,0],[-1,2],[-2,4],[-1,2],[-4,1],[-3,0],[-3,1],[-8,6],[-3,0],[-3,-2],[-3,1],[-2,3],[-1,5],[-1,6],[0,3],[3,10],[1,5],[-2,4],[-5,2],[-3,3],[-1,4],[-1,7],[2,16],[-1,4],[-2,2],[-4,0],[-7,-1],[-2,0],[-1,-1],[-2,-1],[-1,0],[-1,2],[-2,4],[-2,1],[-3,1],[-6,-1],[-7,5],[-3,-1],[-4,-2],[-3,-1],[-4,1],[-20,13],[-3,2],[-4,-1],[-1,-2],[-1,-2],[0,-2],[-3,-2],[-3,0],[-1,-1],[-1,-3],[-2,-10],[-1,-1],[-5,1],[-3,3],[-6,8],[-3,2],[-3,-2],[-1,-3],[1,-4],[-1,-4],[-2,-4],[-1,-4],[1,-9],[0,-3],[1,-4],[0,-2],[-1,-3],[-2,-4],[0,-9],[-2,-4],[-1,-4],[-3,-2],[-2,-1],[-3,0],[-2,0],[-2,-2],[0,-2],[1,-4],[0,-1],[0,-5],[-1,-2],[-2,-2],[-3,-1],[-2,-2],[0,-8],[-1,-3],[-4,-1],[-8,4],[-3,0],[-5,-1],[-2,2],[0,1],[-2,11],[-1,4],[-4,9],[-2,9],[-2,5],[-2,4],[-1,2],[1,5],[-1,3],[-4,8],[-5,4],[-12,5],[-14,8],[-3,3],[-2,4],[-3,8],[-2,4],[-5,5],[-16,13],[-3,2],[-3,-1],[-2,-1],[-3,0],[-3,1],[-5,4],[-9,5],[-16,13],[-12,5],[-6,0],[-6,-2],[2,5],[1,4],[1,4],[-1,5],[-1,3],[-2,3],[-2,3],[0,4],[1,1],[2,0],[1,0],[1,0],[2,2],[4,10],[-1,3],[-5,3],[-5,10],[-2,2],[-2,0],[-5,-4],[-3,-1],[-5,3],[-3,6],[-3,6],[-3,5],[-12,6],[-2,2],[-1,1],[-2,2],[-4,6],[-3,7],[-2,4],[-2,2],[-12,4],[-4,0],[-8,-3],[-7,0],[-3,-1],[-6,-6],[-2,0],[-2,1],[-4,1],[-2,2],[-3,3],[-2,4],[-2,3],[0,2],[-1,1],[0,2],[-1,1],[-2,1],[-3,1],[-2,0],[-3,-1],[-2,1],[-1,4],[-1,8],[-1,3],[-2,4],[-4,3],[-4,2],[-4,3],[-3,3],[-1,5],[0,4],[0,5],[-1,6],[-2,2],[-3,1],[-3,3],[0,2],[0,3],[0,2],[0,2],[-1,2],[-2,3],[-1,2],[0,3],[1,2],[0,3],[-1,3],[-1,1],[-3,0],[-2,1],[-1,4],[-1,4],[-1,4],[-5,4],[-5,6],[-2,2],[-8,-3],[-3,-10],[-2,-11],[-2,-6],[-2,1],[-4,6],[-3,2],[-3,-8],[-2,-4],[-1,-3],[-1,0],[-2,-2],[-2,0],[-5,1],[-4,-2],[-2,0],[-5,4],[-1,1],[-1,3],[0,5],[1,10],[2,10],[1,9],[-5,6],[-6,5],[0,6],[16,18],[3,4],[1,4],[-4,7],[-6,8],[-3,0],[-2,1],[-1,1],[-1,2],[-1,5],[-2,3],[-1,2],[0,3],[1,2],[0,2],[-4,7],[0,4],[4,9],[1,5],[2,12],[1,5],[3,11],[1,5],[-2,5],[-2,3],[-7,6],[-5,5],[-16,30],[-3,8],[-3,7],[-6,5],[-1,0],[-1,0],[-1,1],[-1,4],[1,4],[1,6],[-4,24],[-1,1],[0,2],[0,3],[1,3],[1,1],[1,2],[-1,2],[-6,14],[-5,13],[1,5],[4,3],[6,2],[23,1],[7,2],[3,3],[3,3],[4,8],[3,2],[4,-2],[3,-4],[-1,-5],[-1,-4],[-1,-5],[0,-4],[3,-14],[3,-8],[14,-17],[4,-6],[5,-6],[5,-2],[5,4],[3,1],[4,2],[2,1],[2,0],[4,-2],[2,1],[2,5],[2,15],[2,6],[3,4],[5,3],[5,2],[4,1],[3,-1],[3,-2],[3,-2],[3,1],[7,11],[16,18],[7,11],[3,9],[2,9],[-1,9],[-4,6],[-4,7],[-2,8],[-1,8],[-3,8],[-10,12],[-3,7],[0,10],[1,5],[1,13],[1,5],[3,8],[2,4],[-1,10],[-5,3],[-13,1],[-6,3],[-13,9],[-5,1],[-5,-4],[-2,2],[0,6],[-1,5],[-2,3],[-3,3],[-2,1],[-6,3],[-3,2],[-9,20],[0,1],[-15,19],[-4,9],[-3,15],[2,51],[-1,17],[-3,17],[-8,32],[1,23],[13,11],[16,9],[11,16],[2,6],[0,6],[0,6],[-2,6],[-3,9]],[[875,4912],[5,2],[2,2],[5,6],[1,1],[28,15],[2,0],[4,0],[16,-2],[4,1],[3,1],[8,6],[3,1],[5,1],[5,0],[15,-3],[8,-4],[9,-7],[0,-1],[2,0],[1,-1],[3,0],[2,0],[2,1],[2,1],[2,4],[11,24],[3,7],[6,20],[4,8],[0,2],[3,6],[0,3],[1,6],[0,4],[-1,4],[-1,2],[-6,11],[-1,2],[0,3],[-1,10],[-1,3],[-3,7],[0,3],[1,4],[2,5],[3,2],[2,2],[2,1],[2,1],[8,2],[1,1],[2,1],[2,1],[1,3],[1,2],[0,5],[0,3],[-1,3],[-1,2],[-1,2],[-1,2],[-1,3],[0,3],[1,4],[3,5],[10,16],[8,10],[3,5],[2,4],[9,32],[3,8],[1,4],[1,4],[0,9],[-1,5],[0,4],[0,1],[0,2],[3,4],[6,-4],[1,-1],[7,-9],[4,-5],[16,-14],[4,-2],[5,-1],[19,-2],[2,-1],[1,-1],[7,-5],[4,-3],[3,-1],[1,0],[4,0],[6,1],[3,0],[2,0],[3,-1],[3,-2],[10,-9],[4,-2],[1,0],[2,0],[3,1],[2,0],[1,2],[0,1],[0,8],[0,2],[0,2],[0,2],[1,3],[2,3],[1,1],[2,0],[3,-1],[3,-3],[4,-5],[7,-7],[6,-3],[3,-1],[5,0],[16,5],[3,0],[6,-2],[4,-1],[4,-3],[13,-15],[3,-2],[3,-2],[4,0],[18,2],[34,12],[13,8],[2,1],[1,2],[2,2],[1,3],[1,5],[2,13],[1,2],[1,2],[1,2],[2,2],[22,15],[11,10],[1,2],[1,3],[1,2],[1,3],[0,10],[1,4],[1,2],[2,2],[4,2],[2,1],[4,5],[2,2],[3,1],[5,-1],[35,-10],[6,0],[3,2],[9,7],[3,2],[4,0],[9,-1],[5,-1],[4,-2],[3,-5],[2,-4],[1,-5],[1,-6],[-2,-22],[0,-3],[0,-2],[1,-2],[2,-3],[3,-3],[6,-3],[4,-4],[3,-4],[2,-4],[4,-2],[6,1],[9,4],[2,0],[3,0],[6,-2],[5,-4],[3,-1],[4,-1],[16,0],[8,-2],[6,-5],[4,-1],[9,-1],[41,4],[19,11],[4,1],[3,0],[2,-1],[2,-1],[10,-7],[3,-2],[4,0],[3,0],[7,3],[57,45],[3,2],[32,9],[11,9],[13,15],[37,16],[4,0],[6,-1],[2,-1],[5,-3],[1,-1],[1,0],[2,1],[6,2],[4,2],[4,2],[4,0],[6,0],[2,-1],[2,-1],[12,-8],[0,-1],[1,-1],[2,-8],[1,-3],[1,-2],[1,-1],[1,0],[2,0],[3,2],[3,3],[16,22],[2,2],[3,1],[4,1],[26,-3],[4,0],[3,2],[2,3],[1,1],[6,11],[8,9],[2,4],[4,11],[0,1],[1,10],[1,4],[1,2],[0,1],[5,4],[1,2],[2,3],[1,4],[1,4],[2,12],[0,3],[1,2],[2,3],[2,1],[2,2],[4,2],[21,7],[2,1],[1,1],[2,2],[1,3],[2,3],[2,3],[2,1],[4,0],[15,-2],[12,1],[4,1],[4,2],[12,8],[6,1],[29,2],[2,1],[3,2],[5,2],[19,2],[3,1],[25,19],[3,0],[5,-1],[9,-6],[8,-3],[2,-1],[9,0],[32,7],[14,1],[6,1],[6,4],[3,3],[4,5],[2,2],[1,1],[1,1],[1,0],[13,-1],[12,1],[21,5],[20,2],[10,-2],[2,0],[3,-2],[1,-2],[0,-1],[1,0],[0,-1],[1,-1],[1,-1],[1,-1],[31,-12],[19,2],[2,-1],[2,-1],[5,-5],[12,-16],[3,-3],[1,0],[2,-1],[7,0],[7,-2],[5,-1],[15,1],[3,-1],[75,-48]],[[875,4912],[-1,2],[-4,6],[-6,4],[-8,3],[-6,4],[-8,15],[-14,15],[-10,5],[-37,9],[-7,4],[-7,8],[-3,9],[-11,50],[-1,5],[-7,11],[-2,9],[-1,20],[-2,10],[-15,37],[-6,19],[1,21],[5,13],[3,15],[-2,10],[-9,1],[-5,-3],[-7,-11],[-5,-3],[-4,2],[-4,4],[-3,5],[-3,6],[-4,1],[-2,0],[-2,-1],[-5,-5],[-5,-3],[-5,-2],[-23,5],[-1,1],[-6,-1],[-10,-2],[-5,1],[-5,4],[-8,8],[-4,3],[-5,1],[-12,-2],[-6,0],[-5,4],[-3,2],[-3,6],[-2,3],[-3,1],[-4,0],[-6,-2],[-7,1],[-3,0],[-4,2],[-2,3],[-4,7],[-2,4],[-3,2],[-7,-1],[-1,0],[-2,0],[-3,2],[-2,4],[-4,7],[-5,5],[-12,6],[-6,4],[-3,9],[2,21],[-1,9],[-3,1],[-6,-8],[-4,-2],[-4,0],[-2,0],[-1,-1],[-2,-2],[-2,-4],[-1,-2],[-3,-1],[-3,1],[-3,0],[-6,-6],[-3,0],[-2,2],[-3,1],[-4,-1],[-8,-3],[-4,0],[-4,6],[-2,9],[-1,11],[-1,10],[-4,10],[-5,4],[-13,1],[-5,3],[-5,6],[-3,9],[-1,9],[1,4],[2,3],[8,6],[1,2],[1,2],[0,3],[-1,4],[0,3],[2,4],[3,2],[2,2],[0,4],[-2,4],[-4,7],[-1,4],[0,10],[3,21],[-1,7],[-1,4],[-17,42],[-4,6],[-9,12],[-10,9],[-13,8],[-13,4],[-10,-5],[-1,-3],[-1,-3],[-1,-2],[-10,1],[-3,6],[-2,10],[0,11],[-1,10],[-3,8],[-4,6],[-6,3],[-20,6],[-5,4],[-5,5],[-2,2],[-3,1],[-14,-3],[-6,0],[-6,-2],[-3,-1],[-2,-3],[-2,-3],[-1,-8],[-1,-2],[-2,-1],[-1,2],[-1,8],[-2,3],[-4,10],[-5,5],[-12,1],[-5,5],[-2,4],[-3,1],[-6,-3],[-10,-4],[-3,-2],[-4,2],[-2,5],[-2,5],[-3,4],[-10,7],[-5,5],[-3,5],[0,6],[3,1],[6,-1],[3,0],[3,2],[1,4],[2,5],[2,4],[2,2],[5,0],[7,2],[2,1],[2,-1],[2,-3],[1,0],[1,0],[1,1],[0,1],[2,3],[1,2],[0,1],[4,0],[1,1],[3,4],[2,3],[2,2],[3,0],[1,-2],[6,-19],[2,-1],[3,0],[13,4],[3,3],[2,4],[2,4],[3,3],[3,1],[16,7],[1,3],[0,10],[2,4],[4,3],[8,3],[-5,5],[-5,6],[-8,14],[-3,3],[-14,11],[-5,6],[-4,8],[-2,9],[1,5],[2,4],[2,3],[2,4],[1,2],[0,3],[0,2],[1,3],[3,4],[1,3],[1,2],[-1,5],[-2,3],[-8,6],[-1,1],[-1,2],[-1,4],[1,1],[1,1],[3,5],[0,2],[-1,2],[-2,1],[0,2],[0,2],[-1,7],[1,16],[-2,8],[-12,8],[-3,7],[-3,31],[0,25],[1,5],[1,4],[7,10],[2,4],[1,5],[0,5],[-1,8],[0,2],[-1,3],[-1,1],[-3,2],[-1,2],[-1,3],[-3,4],[-3,2],[-3,1],[-3,0],[-4,0],[-3,3],[-2,4],[-3,3],[-28,14],[-23,14],[-4,1],[-11,-5],[-4,0],[-3,0],[-10,5],[-3,0],[-2,-2],[-2,-4],[-1,-2],[0,-3],[0,-2],[-1,-2],[-7,-2],[-1,-2],[0,-5],[2,-4],[2,-4],[-3,-5],[-2,-1],[-16,1],[-3,2],[-4,6],[-3,2],[-6,3],[-3,2],[-3,4],[-2,5],[-1,11],[-1,3],[-6,13],[-1,4],[1,12],[0,5],[-2,3],[-5,7],[-2,4],[0,5],[1,5],[2,4],[3,3],[4,2],[7,0],[4,2],[2,3],[1,4],[0,4],[-2,5],[-2,2],[-6,7],[-9,10],[-2,3],[-1,4],[-2,9],[-1,4],[-7,9],[-4,8],[-1,8],[1,8],[5,18],[1,4],[0,6],[-2,15],[0,5],[31,5],[3,2],[1,4],[2,9],[2,6],[2,4],[3,3],[3,2],[1,3],[-4,14],[0,12],[-1,5],[-3,5],[-3,2],[-3,1],[-2,2],[-1,5],[-1,5],[1,6],[2,4],[2,4],[4,5],[2,2],[0,3],[2,11],[2,6],[3,4],[3,4],[1,6],[0,4],[1,4],[2,3],[1,3],[5,4],[10,2],[6,3],[5,4],[5,3],[6,1],[6,-2],[6,-2],[5,-1],[5,1],[5,3],[18,18],[14,19],[6,5],[6,5],[3,2],[8,11],[3,1],[4,0],[14,-4],[5,1],[-1,9],[-2,3],[-8,9],[-1,4],[1,6],[2,5],[1,5],[0,1],[1,3],[2,3],[1,1],[3,1],[4,-2],[14,-12],[6,-4],[6,-1],[4,0],[8,5],[4,1],[3,-2],[3,-2],[3,-2],[3,1],[1,2],[1,2],[1,2],[2,1],[2,1],[2,1],[35,35],[7,6],[6,-1],[5,-5],[2,-11],[1,-15],[1,-5],[2,-4],[5,-5],[2,-3],[0,-5],[-2,-3],[-2,-4],[-1,-5],[0,-5],[3,-12],[1,-4],[3,-4],[4,1],[7,4],[4,0],[3,0],[11,-4],[3,0],[4,1],[3,3],[3,4],[2,5],[2,4],[14,8],[4,1],[4,-2],[6,-4],[3,-1],[5,5],[5,8],[5,7],[5,-2],[1,-5],[3,-16],[2,-3],[3,1],[5,6],[14,12],[5,5],[3,8],[6,19],[4,8],[9,8],[1,4],[1,4],[1,27],[3,9],[4,8],[6,6],[6,5],[3,3],[1,5],[0,6],[-2,5],[0,5],[1,5],[3,3],[6,6],[2,4],[5,5],[6,3],[18,5],[4,-1],[9,-5],[5,-2],[20,-2],[10,4],[5,0],[9,-2],[9,-5],[5,-3],[5,0],[12,4],[9,-1],[3,1],[2,1],[22,14],[5,2],[6,1],[21,-4],[7,-4],[3,0],[3,1],[17,17],[2,4],[2,4],[5,10],[3,8],[4,7],[3,3],[6,5],[3,3],[2,5],[-1,4],[-3,11],[1,4],[3,4],[3,3],[6,4],[15,14],[12,3],[6,3],[2,3],[1,6],[1,3],[2,2],[3,2],[22,11],[15,12],[3,2],[9,2],[19,16],[6,4],[8,2],[3,2],[4,3],[8,8],[2,3],[4,1],[10,-4],[4,0],[4,2],[12,10],[11,5],[9,8],[8,2],[7,0],[8,1],[5,7],[2,5],[2,10],[1,4],[2,4],[3,3],[6,3],[8,2],[42,0],[8,3],[4,8],[0,5],[-2,4],[-2,3],[-1,4],[-1,5],[-1,8],[-1,4],[7,4],[1,3],[2,6],[-3,19],[-3,4],[-1,4],[-1,4],[-4,21],[0,3],[2,5],[0,4],[0,3],[-2,5],[-4,7],[-2,4],[-1,11],[-1,4],[-1,4],[0,6],[2,4],[6,11],[2,8],[4,15],[3,7],[3,2],[3,0],[6,-2],[2,0],[8,4],[3,3],[13,6],[8,1],[3,3],[1,5],[-1,5],[-4,2],[-7,1],[-12,7],[-5,5],[-3,7],[2,7],[7,6],[14,7],[13,11],[7,4],[7,0],[8,-8],[3,-1],[4,1],[3,3],[3,4],[0,5],[0,5],[-4,9],[-1,5],[0,5],[-1,5],[-1,4],[-2,4],[-6,4],[-7,2],[-2,4],[8,18],[3,8],[0,9],[-19,60],[-19,60],[-6,10],[-3,6],[-2,5],[0,6],[0,5],[-1,4],[-2,5],[-5,6],[-3,4],[-1,4],[1,10],[5,19],[0,11],[-7,39],[0,9],[1,11],[4,20],[3,6],[1,3],[-2,3],[-3,5],[0,2],[0,3],[3,5],[6,3],[11,3],[2,2],[1,3],[-1,4],[-2,2],[-27,12],[-8,7],[-3,2],[-5,1],[-29,-8],[-8,1],[-20,15],[-4,6],[3,8],[3,3],[7,5],[17,16],[6,7],[7,3],[7,1],[12,-2],[11,5],[12,-1],[5,1],[2,3],[3,9],[4,3],[3,1],[12,-1],[26,11],[8,0],[13,-5],[4,0],[20,9],[8,1],[6,2],[14,9],[65,23],[13,11],[7,4],[6,-2],[6,-1],[4,-8],[3,-11],[1,-9],[0,-6],[1,-3],[2,-2],[11,-2],[3,-2],[3,-4],[7,-4],[8,2],[31,19],[9,2],[8,-1],[8,-6],[14,-18],[7,-6],[11,3],[7,14],[5,19],[3,24],[1,10],[-1,10],[-3,5],[-16,8],[-25,4],[-6,3],[-6,11],[1,6],[2,7],[5,14],[6,26],[12,24],[4,13],[3,7],[4,25],[4,36],[5,17],[6,16],[11,20],[2,8],[2,15],[1,4],[7,16],[15,63],[24,56],[1,4],[0,5],[-1,4],[0,5],[1,5],[2,10],[0,3],[-1,5],[0,3],[0,3],[2,6],[1,3],[-1,10],[2,3],[4,3],[4,1],[4,0],[5,0],[4,-2],[8,-8],[3,-3],[16,-3],[6,-4],[28,-20],[62,-25],[16,0],[11,-2],[4,0],[10,7],[1,0],[17,2],[20,-2],[3,0],[3,0],[8,-5],[8,-1],[15,4],[8,0],[6,-4],[5,-6],[3,-8],[0,-9],[-2,-4],[-2,-5],[-1,-4],[0,-3],[4,-2],[4,1],[12,5],[12,3],[4,3],[3,8],[3,3],[3,2],[7,1],[2,2],[5,6],[11,9],[3,4],[4,8],[3,3],[3,2],[3,1],[10,-2],[21,4],[11,0],[1,3],[0,5],[2,4],[2,4],[1,4],[1,4],[1,5],[1,4],[3,3],[4,1],[8,1],[2,3],[1,4],[0,5],[2,9],[0,4],[-1,5],[-1,4],[1,5],[1,5],[-1,6],[-12,21],[-2,7],[-1,9],[2,17],[0,9],[-13,78],[1,7],[2,6],[3,7],[2,10],[4,29],[22,57],[4,9],[6,4],[37,6],[30,-3],[9,5],[6,9],[4,3],[4,2],[16,-1],[19,8],[3,3],[2,3],[3,9],[2,5],[2,3],[8,8],[12,19],[1,3],[0,3],[-1,12],[-1,3],[2,2],[3,4],[4,9],[-3,5],[-5,5],[-4,9],[-1,6],[0,5],[0,4],[2,6],[3,4],[3,3],[6,6],[3,5],[4,9],[3,3],[5,0],[9,-3],[5,0],[23,9],[8,-1],[6,-4],[20,-6],[6,-3],[6,-1],[7,1],[6,3],[1,1],[1,2],[0,1],[2,5],[1,0],[1,0],[2,0],[31,10],[16,2],[1,-3],[2,-8],[1,-6],[-1,-4],[-2,-4],[-1,-6],[1,-5],[3,-3],[3,-1],[3,-3],[0,-9],[-6,-6],[-13,-7],[-2,-4],[-2,-5],[1,-4],[2,-4],[4,-1],[2,-2],[1,-1],[1,-3],[0,-5],[1,-3],[2,-3],[4,-2],[10,-6],[8,1],[3,-1],[10,-8],[4,-5],[1,-7],[-2,-2],[-7,-6],[-1,-2],[-1,-2],[-1,-3],[-3,-2],[-5,-5],[2,-6],[5,-6],[5,-4],[2,0],[4,1],[2,0],[1,-1],[3,-3],[4,-3],[6,-4],[3,-2],[2,0],[6,0],[3,0],[11,-6],[5,-4],[8,-12],[6,-4],[3,0],[4,1],[3,0],[4,-2],[10,-13],[1,-4],[-2,-10],[0,-6],[1,-5],[1,-5],[1,-4],[3,-4],[6,-6],[4,-1],[4,0],[4,-1],[11,-13],[5,-3],[4,-1],[10,2],[4,-1],[3,-3],[18,-27],[5,-3],[15,0],[9,-2],[4,1],[4,4],[3,3],[8,7],[3,2],[2,0],[7,-4],[2,-1],[9,1],[4,1],[7,1],[4,-3],[1,-7],[0,-5],[1,-4],[2,-2],[3,-1],[2,-1],[2,-4],[1,-5],[2,-3],[3,-1],[6,-2],[2,-3],[1,-3],[2,-8],[2,-3],[5,-1],[17,1],[6,2],[2,3],[0,4],[0,3],[2,4],[2,1],[7,-2],[7,0],[2,-3],[1,-5],[-1,-6],[-1,-5],[1,-5],[2,-5],[2,-4],[6,-7],[7,-5],[8,-4],[15,-5],[3,-2],[4,-3],[1,-3],[-2,-3],[-2,-3],[2,-6],[2,-3],[5,-14],[2,-3],[5,-5],[3,-4],[0,-8],[0,-8],[1,-7],[4,-4],[2,-3],[1,-4],[-1,-5],[-2,-4],[-1,-1],[0,-2],[0,-2],[40,-78],[5,-6],[6,-2],[4,0],[4,-1],[3,-2],[4,-4],[2,-4],[1,-4],[2,-5],[1,-5],[1,-1],[2,-2],[1,-1],[0,-3],[0,-2],[-2,-4],[0,-2],[1,-5],[2,-4],[6,-7],[5,-9],[1,-10],[0,-3],[0,-7],[-2,-12],[-1,-6],[3,-3],[3,-1],[3,-4],[-1,-4],[-1,-6],[-4,-9],[-2,-5],[-2,-10],[-2,-5],[-7,-20],[-4,-7],[-2,-4],[0,-5],[8,-30],[5,-7],[3,-8],[2,-12],[-1,-12],[-2,-7],[-2,-2],[-9,-10],[-34,-52],[-3,-6],[-7,-63],[3,-4],[11,-9],[3,-6],[1,-8],[3,-7],[2,-7],[0,-9],[1,-3],[6,-3],[3,-3],[2,-4],[0,-3],[-1,-3],[0,-4],[2,-8],[5,0],[12,8],[6,1],[7,-1],[7,-1],[6,-4],[2,-3],[4,-6],[2,-2],[11,-7],[16,-4],[3,0],[3,3],[2,3],[3,2],[4,0],[2,-2],[11,-16],[2,-1],[2,0],[1,1],[2,0],[16,-4],[4,0],[6,3],[4,0],[28,-2],[9,2],[6,1],[5,0],[6,-2],[5,-3],[14,-11],[4,-1],[8,2],[3,0],[7,-3],[3,0],[8,1],[3,0],[7,-4],[3,0],[16,4],[11,1],[18,6],[8,-1],[2,0],[6,1],[2,0],[9,-6],[5,-3],[5,0],[3,1],[3,0],[3,0],[3,-1],[4,0],[11,1],[4,0],[13,-7],[4,0],[8,1],[4,0],[3,-2],[6,-5],[4,-1],[17,-1],[26,-16],[5,-4],[26,-43],[8,-11],[8,-3],[14,-2],[7,0],[8,4],[2,-1],[3,-1],[2,-2],[14,-17],[3,-5],[4,-11],[3,-4],[4,-2],[11,0],[4,-3],[21,-15],[12,-13],[2,-4],[2,-4],[2,-4],[4,-2],[30,-11],[15,-9],[7,-1],[56,8],[3,-2],[-2,-5],[-4,-7],[-2,-5],[-2,-7],[-2,-5],[-1,-3],[0,-2],[1,-14],[-1,-15],[0,-6],[1,-3],[16,-5],[8,0],[6,-2],[4,-6],[9,-23],[17,-63],[23,-59],[1,-10],[1,-12],[1,-11],[4,-10],[5,-9],[63,-74],[4,-6],[2,-5],[1,-7],[2,-33],[2,-4],[1,-1]],[[6115,1034],[1,-1],[3,-10],[-3,-2],[-1,2],[-3,0],[-3,0],[-1,-2],[-9,-13],[-4,-9],[-2,-3],[-3,-1],[-3,1],[-2,3],[-1,4],[-1,7],[2,3],[2,1],[4,0],[-3,4],[-3,0],[-4,-3],[-1,-3],[1,-3],[2,-7],[1,-2],[-2,-6],[-1,-4],[0,-6],[-1,-3],[-2,-3],[-2,-3],[-2,-1],[0,-1],[1,-3],[-2,-3],[-2,-2],[-2,-2],[-1,0],[-3,1],[-1,-1],[-1,-1],[-1,-5],[-1,-1],[-2,-2],[0,-4],[1,-5],[0,-3],[-1,-4],[-5,-2],[0,-4],[-1,-9],[-2,-9],[-2,-8],[-3,-6],[-1,-6],[-1,1],[-1,1],[-4,-3],[-6,0],[-1,1],[-1,1],[-1,2],[-2,1],[0,-2],[2,-2],[1,-3],[2,-3],[2,-1],[2,1],[1,2],[1,0],[1,-3],[1,-2],[-1,-9],[2,4],[2,13],[1,0],[-2,-13],[-1,-10],[0,-2],[-3,-4],[-3,-6],[-2,-7],[-1,-7],[0,-9],[0,-2],[0,-3],[1,-2],[-1,-2],[-2,0],[-1,0],[-1,-1],[0,-2],[-1,0],[-1,0],[-1,-1],[-1,-2],[0,-5],[0,-2],[-1,-1],[-2,-2],[-1,-2],[-1,-2],[1,-2],[1,-2],[0,-1],[1,1],[3,-1],[2,-1],[2,-2],[2,3],[0,2],[-1,7],[-1,7],[0,3],[2,2],[1,-4],[4,-22],[-3,0],[-1,0],[-2,-1],[-1,-2],[-1,0],[-3,1],[-3,-5],[-4,-14],[1,-3],[-1,-3],[-1,-3],[-2,-2],[-3,3],[-7,-3],[-2,3],[2,2],[4,1],[7,1],[0,2],[-1,2],[-1,0],[-3,-4],[-9,-3],[-3,-4],[-1,0],[-5,1],[-7,-7],[-12,-16],[-6,-6],[-2,-5],[-3,-9],[-3,-1],[-6,3],[0,-2],[2,-2],[5,-2],[1,-3],[0,-4],[-2,-9],[0,-5],[-1,0],[0,5],[-2,3],[-1,0],[-1,-4],[1,-4],[1,-3],[0,-2],[-2,-2],[-2,-1],[-5,1],[-4,2],[0,4],[1,0],[2,-1],[2,0],[2,2],[1,2],[0,3],[1,3],[-1,3],[-1,-2],[-1,-2],[0,-2],[-1,1],[-2,3],[0,-1],[0,-2],[0,-1],[-1,2],[-1,-1],[-1,-3],[-1,-1],[-1,-1],[-1,0],[-1,0],[-5,1],[-3,-1],[-3,-2],[-7,-6],[-2,1],[-1,1],[0,2],[-1,1],[-3,1],[-3,-1],[-3,-1],[-2,-1],[1,-6],[-4,-18],[1,-8],[-2,0],[-1,2],[-1,2],[-1,1],[-1,-1],[-4,-4],[0,-2],[2,0],[2,0],[3,-2],[1,0],[2,0],[1,0],[0,-2],[0,-2],[-1,-2],[-1,-3],[-1,-3],[-2,-2],[-1,0],[-2,1],[-1,3],[-1,6],[-2,-2],[-6,-2],[-1,-2],[-1,-1],[-2,-2],[0,-1],[0,-2],[1,-4],[1,-2],[-3,2],[-2,0],[-2,-1],[-1,-3],[-1,0],[0,2],[1,5],[0,2],[-1,1],[-1,1],[0,1],[0,1],[-1,0],[0,1],[0,1],[1,1],[1,1],[0,2],[1,1],[0,1],[-1,3],[-2,-4],[-2,-2],[-1,-2],[0,-5],[-3,1],[-6,-5],[-2,3],[2,2],[1,2],[2,2],[2,1],[1,2],[-1,3],[-1,2],[-1,-4],[-2,6],[-3,4],[-4,2],[-17,4],[-19,0],[-5,-1],[-3,-3],[-1,0],[-2,3],[-1,4],[-1,4],[0,3],[-2,1],[1,4],[-2,-1],[-2,1],[-1,1],[-2,1],[-15,0],[-1,0],[-2,2],[-2,2],[-1,3],[-2,2],[-2,2],[-8,4],[-2,2],[-1,3],[-6,7],[-6,4],[-4,2],[-3,1],[-7,1],[-3,1],[-1,3],[-2,4],[0,6],[1,5],[1,4],[1,5],[1,20],[-1,10],[-2,10],[-2,8],[-3,7],[-2,3],[-2,3],[0,3],[1,5],[3,10],[1,4],[0,6],[0,6],[-1,2],[-1,1],[0,2],[0,9],[-5,22],[3,2],[2,4],[2,4],[1,4],[3,-1],[0,1],[-1,2],[0,3],[-1,5],[-1,5],[-2,0],[-3,0],[-2,2],[0,2],[2,3],[1,3],[-2,7],[1,3],[2,2],[1,1],[0,1],[-1,2],[1,0],[2,0],[0,1],[0,2],[1,1],[1,1],[0,2],[-1,2],[1,2],[1,2],[0,2],[1,3],[1,1],[4,-1],[2,0],[4,3],[1,0],[2,1],[3,8],[1,1],[3,3],[2,2],[7,3],[5,10],[4,3],[2,-4],[1,0],[0,2],[-1,4],[3,6],[6,8],[2,5],[1,3],[1,1],[8,2],[2,1],[4,4],[3,1],[1,3],[1,0],[4,2],[6,8],[2,5],[0,5],[1,2],[1,-1],[4,2],[3,1],[2,-1],[1,-1],[3,-4],[1,0],[-1,4],[-3,5],[-1,3],[3,1],[0,2],[-1,0],[0,2],[2,0],[0,1],[1,2],[-1,0],[-2,1],[-1,0],[-1,1],[-2,-4],[-4,-4],[-4,-4],[-3,-1],[-2,-2],[-1,-2],[-1,-1],[-1,0],[1,5],[-1,12],[0,4],[2,2],[2,2],[1,2],[1,1],[1,3],[1,2],[1,1],[1,0],[1,0],[1,1],[2,3],[0,2],[0,3],[1,3],[1,0],[1,0],[2,1],[4,5],[1,-1],[1,-5],[1,-2],[2,0],[5,-1],[5,-5],[3,-2],[2,1],[3,2],[2,-2],[1,-4],[0,-4],[1,0],[2,2],[2,2],[3,1],[2,2],[1,-2],[2,0],[3,1],[3,0],[0,1],[-8,4],[-2,2],[-1,6],[2,1],[5,0],[0,2],[-1,3],[-1,1],[0,-2],[-1,0],[-1,1],[-1,3],[0,3],[9,9],[3,2],[2,1],[3,0],[1,-1],[1,0],[2,2],[2,2],[3,0],[3,-1],[2,1],[1,1],[1,-2],[0,-2],[-1,-3],[1,-2],[0,-2],[-1,-1],[0,-3],[1,0],[2,6],[4,-1],[7,-6],[1,4],[0,4],[1,2],[2,-1],[1,-3],[0,-4],[0,-4],[0,-4],[3,3],[2,3],[2,4],[1,5],[2,0],[3,1],[1,0],[1,-1],[0,1],[1,1],[1,-1],[0,-1],[0,-1],[1,-1],[2,-3],[2,-5],[1,-4],[-4,-1],[1,-3],[2,-2],[3,-1],[3,1],[0,2],[-3,2],[2,2],[2,2],[0,4],[6,5],[3,1],[1,0],[2,0],[2,-1],[2,-3],[2,3],[2,6],[2,2],[-2,-1],[-2,-3],[-1,-1],[-2,-1],[-6,1],[7,4],[3,3],[1,3],[2,8],[1,4],[3,-2],[6,-3],[2,-1],[2,-1],[2,-1],[2,-2],[3,0],[3,2],[3,5],[2,1],[3,-1],[2,-2],[2,-3],[1,-3],[0,-2],[-1,-3],[1,-3],[4,-5],[0,-4],[1,-7],[1,-4],[1,0],[-1,3],[0,1],[0,2],[1,0],[0,2],[-1,2],[0,5],[-1,2],[-1,0],[-2,2],[0,2],[-1,3],[-1,7],[0,1],[0,6],[0,2],[1,2],[7,-9],[1,1],[7,-5],[4,-2],[5,-1],[0,-2],[-1,0],[0,-2],[2,-1],[3,-3],[2,-1],[2,3],[1,-5],[2,-9],[1,-5],[1,-1],[1,1],[1,2],[1,2],[-1,2],[0,2],[0,2],[1,2],[-3,5],[0,5],[-1,3],[-4,2],[-1,1],[0,2],[0,3],[0,2],[-1,3],[1,3],[4,7],[1,1],[6,-1],[2,3],[3,10],[1,2],[2,-3],[4,-20],[2,-3],[5,-5],[3,-7],[3,-3],[13,-3],[3,0],[1,1],[0,1],[1,-2],[3,4],[2,-5],[7,-34],[1,-15],[3,-16],[0,-7],[-1,-7],[1,-2],[2,-1]],[[5567,5841],[0,-11],[-1,-4],[-2,-4],[-3,-3],[-3,-1],[-5,-1],[-10,1],[-3,-1],[-1,-3],[-1,-2],[-1,-4],[0,-5],[0,-3],[-2,-2],[-2,-4],[-2,-3],[-9,-8],[-4,-4],[-7,-8],[-1,-3],[1,-4],[1,-4],[0,-5],[-9,-27],[-2,-13],[-1,-6],[-1,-9],[-1,-4],[0,-1],[8,-10],[4,-16],[1,-6],[1,-4],[1,-3],[-1,-3],[-6,-20]],[[5027,5721],[11,26],[1,2],[1,1],[2,1],[5,2],[7,3],[9,0],[2,-1],[10,-6],[2,0],[23,8],[23,3],[4,1],[4,4],[2,1],[1,2],[1,2],[2,6],[2,5],[3,4],[1,2],[3,3],[27,17],[17,7],[3,1],[4,-1],[2,0],[9,-5],[7,-2],[5,0],[10,3],[7,0],[2,0],[1,1],[5,6],[16,11],[2,2],[0,2],[0,1],[0,14],[0,4],[3,17],[1,7],[0,3],[0,2],[0,1],[0,1],[-1,2],[-2,4],[-5,12],[-2,10],[-1,3],[0,3],[0,3],[0,3],[1,4],[1,3],[1,3],[1,2],[1,2],[1,0],[4,2],[2,2],[0,1],[1,1],[0,2],[2,4],[0,2],[0,3],[0,3],[0,3],[-1,1],[0,2],[0,1],[-2,4],[-1,2],[0,1],[0,1],[0,9],[0,5],[3,46],[9,44],[2,7],[9,18],[3,6],[3,5],[0,2],[2,10],[1,3],[2,4],[3,7],[2,2],[1,3],[0,3],[2,16],[1,1],[4,13],[0,1],[1,1],[2,1],[4,1],[2,0],[1,0],[3,-2],[2,0],[2,0],[2,0],[2,1],[1,1],[2,1],[2,3],[1,1],[1,2],[1,1],[0,2],[0,2],[0,1],[0,2],[-1,1],[-1,4],[-1,5],[0,1],[0,6],[0,3],[1,0],[0,2],[1,1],[0,1],[3,3],[2,1],[2,0],[4,0],[11,-3],[6,0],[7,1],[2,1],[6,4],[4,3],[7,11],[3,1],[19,1],[3,2],[1,0],[1,-1],[2,-2],[1,-2],[0,-5],[1,-2],[1,-5],[-2,-18],[1,-5],[1,-6],[2,-5],[3,-2],[1,-2],[6,-16],[-1,-1],[1,-2],[0,-1],[1,-1],[4,-4],[4,-3],[7,-6],[0,-1],[1,-2],[-1,-3],[0,-3],[-3,-10],[-5,-11],[-26,-55],[-10,-17],[0,-1],[-1,-3],[-10,-47],[-3,-6],[-9,-15],[-16,-23],[17,-13],[9,-5],[8,-2],[2,-2],[19,-14],[3,-1],[27,-9],[11,-1],[14,7],[3,0],[23,-17],[3,-3],[7,-14],[4,-7],[2,-5],[1,-5],[-1,-7],[2,-2],[2,-2],[11,0],[1,-1],[9,-7],[12,-4],[5,-2]],[[6131,6246],[1,-2],[4,-2],[2,-3],[2,2],[2,-2],[0,-4],[1,-3],[3,-3],[2,-1],[2,-1],[0,-4],[-1,-4],[-10,-29],[-4,-19],[-3,-9],[-4,-6],[-11,-7],[-5,-4],[-3,-7],[3,-10],[2,-6],[0,-3],[-2,-5],[-2,-18],[-2,-5],[-2,-3],[-1,-4],[-1,-8],[-1,-3],[-3,-3],[-1,-3],[-1,-7],[-6,-16],[0,-2],[1,-1],[4,-8],[0,-1],[0,-8],[-1,-2],[-1,-3],[-2,-4],[-1,-2],[-1,-3],[-1,-6],[-1,-3],[-3,-3],[-7,-2],[-3,-3],[-3,-10],[-1,-3],[-6,-6],[-1,0],[-1,-2],[-5,-8],[-1,-2],[-2,-1],[-3,-1],[-5,0],[-2,-2],[-2,-2],[-1,-15],[0,-1],[-2,-3],[-2,-1],[-2,-1],[-3,0],[-1,-2],[-2,-9],[2,-6],[0,-4],[-2,-10],[2,-21],[0,-1],[0,-1],[1,-1],[0,-5],[0,-3],[0,-2],[0,-2],[1,-1],[4,0],[3,-2],[2,-3],[2,-4],[10,-28],[4,-4],[9,-6],[2,-2],[2,-5],[-2,-4],[-6,-5],[4,-3],[8,-6],[3,-5],[-2,-2],[-2,-3],[-1,-1],[0,-4],[1,-2],[1,-3],[1,-8],[1,-3],[-1,-2],[-2,-1],[-1,-2],[-3,-17],[-2,-2],[-3,-6],[-4,-2],[-7,-2],[-2,-4],[-1,-6],[0,-4],[2,-3],[4,-1],[3,-2],[0,-4],[-3,-9],[0,-2],[0,-1],[0,-2],[-1,-2],[0,-1],[-2,0],[-1,-1],[-1,-1],[0,-4],[-1,-1],[-10,-19],[-6,-7],[-1,-2],[-2,-8],[-1,-4],[-2,-1],[-1,-1],[-2,-3],[-2,-3],[0,-3],[-1,-2],[-2,-2],[-5,-2],[0,-2],[1,-1],[3,-1],[-2,-3],[-4,1],[-3,2],[-2,-1],[0,-3],[1,-3],[3,-3],[1,-1],[2,-3],[-1,-8],[-3,-8],[-4,-4],[1,-4],[4,-4],[2,-3],[-1,-4],[0,-2],[1,-1],[1,-2],[-1,-3],[0,-3],[-2,-4],[-1,0],[-1,0],[-1,0],[-1,-1],[1,-2],[2,-1],[1,-1],[3,-2],[-1,0],[0,-2],[1,-2],[0,-1],[3,2],[1,-4],[-2,-6],[-1,-3],[-2,1],[-2,1],[-1,0],[-1,-3],[1,-2],[15,-23],[1,-4],[1,-14],[0,-5],[0,-6],[-2,-10],[-1,-11],[-3,-15],[0,-6],[2,-11],[0,-5],[-1,-6],[-3,-10],[-1,-5],[1,-12],[8,-43],[1,-12],[0,-6],[1,-2],[0,-4],[2,-1],[1,1],[2,-1],[3,-9],[3,-36],[6,-20],[1,-11],[-3,-11],[-5,-7],[-2,-5],[0,-3],[1,-2],[-1,-5],[-1,-6],[-1,-3],[-1,-3],[-3,-3],[-2,-2],[-2,-1],[-5,-13],[-1,-3],[-3,-2],[-1,-4],[-1,-3],[-1,-4],[-1,-2],[-7,-8],[-3,-5],[-4,-10],[-1,-13],[-2,-7],[-1,-9],[-3,-10],[-1,-11],[-2,-14],[-2,-3],[2,-9],[0,-8],[-1,-8],[-4,-7],[-1,-6],[1,-22],[2,-15],[3,-7],[4,-4],[6,-1],[1,0]],[[6005,4985],[0,-1],[1,-19],[0,-6],[-1,-3],[-1,-3],[-1,-3],[0,-1],[0,-3],[0,-2],[2,-5],[0,-2],[2,-3],[4,-6],[2,-1],[11,-5],[1,0],[0,-1],[1,-1],[1,-2],[1,-2],[0,-4],[0,-2],[0,-2],[-5,-5],[-1,-2],[-1,-1],[0,-1],[-1,-2],[0,-1],[0,-2],[0,-2],[1,-1],[0,-2],[2,-3],[3,-5],[2,-1],[8,-5],[1,-1],[1,-1],[2,-2],[1,-1],[1,-1],[1,-1],[6,-1],[1,-1],[1,-1],[0,-3],[0,-2],[0,-1],[-1,-3],[-1,-3],[-1,-1],[-1,-13],[0,-2],[0,-3],[0,-2],[0,-1],[2,0],[1,0],[0,-1],[1,-4],[4,-10],[0,-1],[0,-1],[0,-2],[-1,-3],[-6,-12],[-1,-3],[0,-1],[0,-2],[0,-1],[0,-2],[2,-2],[2,-2],[4,-2],[5,-2],[2,-1],[10,-10],[1,-1],[2,-3],[4,-9],[0,-1],[0,-5],[0,-2],[0,-1],[0,-3],[1,-3],[9,-9],[4,-5],[7,-4],[4,-2],[2,-2],[2,-4],[1,-3],[2,-11],[2,-4],[0,-3],[2,-5],[0,-1],[0,-2],[-1,-4],[-1,-4],[-1,-2],[0,-1],[-1,-2],[1,-2],[0,-2],[3,-8],[0,-2],[1,-3],[-1,-7],[-6,-21]],[[6109,4628],[-3,-2],[-2,-3],[-1,0],[-1,-2],[-1,-4],[0,-1],[-1,-1],[-1,-2],[-1,-1],[-1,-1],[-8,0],[-3,0],[-2,-1],[-1,-1],[0,-2],[-1,-3],[-1,-2],[0,-1],[-1,-1],[-1,-1],[-3,-3],[-4,-2],[-1,-1],[0,-1],[-1,-1],[0,-1],[1,-2],[0,-1],[1,-1],[0,-1],[1,-1],[0,-1],[-1,-1],[-1,0],[-3,2],[-1,1],[-1,1],[-5,6],[-3,4],[-1,0],[-1,0],[-3,-1],[-1,0],[-3,1],[-1,2],[0,1],[0,2],[-1,6],[-1,3],[0,1],[-3,8],[-1,1],[-1,1],[-2,1],[-1,0],[-2,0],[-2,-1],[-1,-1],[-1,-2],[-1,-2],[-2,-3],[-4,-7],[-2,-2],[-2,-1],[-1,0],[-32,-5],[-5,1],[-2,1],[-2,1],[0,1],[-5,6],[-2,2],[-1,1],[-2,2],[-1,0],[-3,0],[-22,1],[-7,5],[-2,1],[-5,1],[-20,0],[-5,1],[-4,2],[-4,2],[-2,1],[-1,0],[-2,-1],[-3,-5],[-1,-1],[-2,-1],[-2,0],[-1,0],[-6,3],[-1,0],[-1,0],[-1,0],[-1,-1],[-2,-1],[-8,-13],[-2,-4],[-1,-2],[0,-1],[1,-4],[6,-1],[4,-1],[4,-2],[5,-5],[1,0],[2,-1],[1,0],[5,1],[5,0],[7,-2],[8,-4],[1,-1],[0,-1],[1,-1],[3,-6],[1,-2],[0,-1],[0,-3],[-1,-4],[-2,-13],[0,-3],[-1,-1],[-2,-5],[-1,-1],[3,-4],[4,-4],[5,-2],[4,0],[5,5],[3,1],[2,-1],[4,-3],[3,-2],[8,-1],[2,-2],[6,-6],[7,-3],[2,-2],[2,-2],[1,-4],[1,-2],[0,-1],[0,-3],[4,-9],[0,-1],[2,-8],[0,-2],[0,-3],[-2,-7],[1,-4],[0,-3],[2,-1],[1,-2],[1,-3],[-1,-3],[-2,-4],[-3,0],[-3,0],[-3,2],[-2,1],[-2,0],[-2,-2],[0,-1],[0,-2],[0,-2],[0,-1],[0,-2],[0,-1],[-6,-6],[-1,-1],[-1,0],[-1,0],[-1,0],[0,1],[-5,4],[-3,2],[-12,4],[-2,0],[-1,0],[-3,-4],[-2,-1],[-1,-1],[-2,0],[-1,1],[-1,0],[-1,1],[0,1],[-3,4],[0,1],[-1,0],[-1,0],[-2,-1],[-4,-2],[-2,-1],[-1,0],[-1,1],[-1,0],[-5,3],[-1,0],[-1,0],[-1,0],[-1,-1],[-1,-1],[-1,-2],[0,-2],[-1,-5],[0,-1],[-1,-1],[-1,0],[-2,-1],[-2,-1],[-1,0],[-1,-4],[-1,-28],[-1,-4],[-1,-1],[-9,-9],[-1,-1],[-1,-1],[0,-2],[0,-2],[0,-3],[1,-5],[1,-3],[3,-5],[1,-2],[0,-2],[2,-6],[0,-2],[1,-1],[1,-1],[8,-5],[1,-1],[1,-1],[0,-1],[1,-1],[0,-3],[1,-11],[1,-6],[2,-3],[5,-8],[0,-1],[0,-2],[0,-3],[-1,-4],[-1,-2],[0,-2],[-2,-2],[0,-2],[-3,-14],[-3,-4],[0,-2],[0,-3],[1,-9],[0,-19],[-1,-2],[-1,-4],[-3,-5],[-1,-2],[0,-2],[0,-3],[0,-2],[2,-2]],[[5707,4345],[-1,3],[-4,9],[-1,2],[-3,1],[-3,2],[-12,1],[-4,-1],[-6,-3],[-3,0],[-2,1],[-2,2],[-2,1],[-3,-1],[-3,-4],[-3,-2],[-2,-1],[-5,1],[-6,-2],[-6,0],[-3,0],[-2,-2],[-3,-2],[-2,-2],[-1,-2],[-1,-2],[0,-3],[0,-6],[-1,-3],[-1,-1],[-1,0],[-1,1],[-4,2],[-9,10],[-7,5],[-2,2],[-1,2],[-6,14],[-4,6],[-1,2],[-8,6],[-2,2],[-5,8],[-4,4],[-3,3],[-3,0],[-2,0],[-2,-1],[-4,-1],[-4,0],[-13,4],[-2,1],[0,1],[0,1],[0,1],[0,1],[0,1],[1,1],[0,1],[0,1],[0,1],[-1,2],[0,1],[0,1],[-1,1],[0,2],[0,2],[-1,6],[0,1],[0,1],[1,2],[0,2],[-1,1],[-1,1],[-1,1],[-3,0],[-2,0],[-2,-1],[-2,-2],[-1,-1],[-3,-3],[-1,-2],[-6,-7],[-1,-2],[-3,-4],[-2,-1],[-3,-1],[-4,0],[-2,1],[-3,1],[-9,8],[-8,6],[-3,2],[-1,3],[-1,2],[1,3],[3,11],[0,3],[1,17],[0,3],[-2,4],[-3,6],[-2,1],[-2,1],[-5,1],[-12,-2],[-10,2],[-5,-1],[-5,1],[-10,5],[-3,1],[-2,0],[-2,-1],[-8,-11],[-6,-5],[-3,-1],[-12,-1],[-2,0],[-12,-6],[-2,-1],[-8,0],[-2,-1],[-2,-1],[-9,-9],[-4,-2],[-2,-1],[-3,0],[-4,0],[-3,1],[-3,3],[-12,11],[-6,7],[-2,5],[-1,2],[0,1],[-1,2],[0,3],[0,3],[0,3],[0,4],[1,3],[1,5],[4,9],[1,3],[1,3],[0,3],[-2,2],[-2,2],[-7,-1],[-4,-1],[-6,-3],[-3,-1],[-10,0],[-4,-2],[-3,-4],[-2,-3],[-1,-3],[-1,-3],[-1,-3],[-2,-6],[-1,-2],[-2,0],[-2,0],[-4,2],[-2,0],[-3,-1],[-3,-2],[-12,-11],[-2,-1],[-1,-1],[-3,1],[-2,1],[-2,2],[-1,4],[-1,5],[0,5],[-2,10],[-5,13],[-3,14]],[[5567,5841],[3,-3],[4,-2],[30,-9],[4,-1],[3,-3],[3,-3],[7,-13],[2,-3],[1,-5],[0,-5],[-1,-8],[0,-1],[2,-1],[4,-1],[32,2],[12,-2],[4,0],[27,9],[4,1],[12,-2],[9,0],[14,2],[3,1],[2,2],[1,3],[0,9],[0,4],[1,3],[1,4],[0,3],[-1,33],[1,5],[1,4],[4,9],[2,2],[1,2],[3,0],[2,0],[1,-2],[1,-2],[2,-6],[4,-7],[1,-2],[2,-1],[2,1],[2,2],[5,11],[2,6],[6,23],[0,4],[0,3],[-1,2],[-6,8],[-4,7],[-1,4],[-1,5],[-1,3],[1,3],[3,11],[1,5],[0,4],[1,7],[0,3],[1,2],[5,8],[2,4],[2,9],[2,4],[3,5],[9,10],[2,2],[2,10],[3,4],[4,4],[16,10],[2,3],[1,3],[-1,6],[1,4],[1,2],[1,1],[6,2],[1,2],[1,3],[2,8],[2,4],[3,4],[7,5],[4,2],[2,0],[1,-1],[1,-2],[2,0],[2,1],[2,2],[1,3],[2,9],[1,3],[2,3],[1,2],[4,4],[2,2],[1,4],[6,27],[2,2],[3,2],[12,3],[4,2],[3,3],[2,2],[0,2],[0,1],[1,2],[0,2],[1,1],[0,1],[20,22],[4,4],[8,5],[14,12],[7,5],[1,0],[1,2],[-1,3],[0,2],[-1,5],[-3,9],[-6,12],[-1,2],[0,1],[0,2],[0,1],[0,1],[1,1],[1,1],[1,1],[2,1],[1,0],[3,0],[2,-1],[5,-1],[2,-2],[1,-1],[1,-2],[4,-11],[2,-2],[1,-3],[11,-10],[1,-1],[3,0],[3,1],[1,1],[1,1],[2,2],[1,4],[1,1],[2,1],[6,1],[4,3],[2,1],[2,-1],[1,-1],[1,-1],[1,-2],[1,-1],[2,-3],[1,-2],[3,-10],[4,-8],[1,-1],[3,-3],[1,-1],[1,-1],[1,0],[2,1],[2,1],[2,2],[1,1],[0,2],[2,6],[1,3],[2,4],[2,5],[14,24],[0,1],[1,1],[0,1],[4,7],[9,11],[1,0],[1,1],[5,2],[2,1],[4,3],[1,1],[2,3],[1,2],[1,2],[1,1],[1,0],[1,0],[4,-2],[1,0],[1,0],[1,0],[1,0],[1,1],[2,2],[3,2],[1,0],[1,0],[1,0],[0,-2],[1,-2],[-2,-7],[-1,-3],[-2,-5],[-2,-4],[-3,-6],[-5,-9],[-1,-2],[0,-2],[1,-2],[1,-2],[8,-7],[1,1]],[[6558,5445],[0,-6],[-3,-12],[-1,-3],[-3,-10],[0,-2],[-1,-1],[0,-2],[0,-2],[1,-1],[1,-1],[1,-2],[1,-1],[0,-1],[-1,-2],[-1,-1],[-2,-2],[0,-1],[0,-2],[0,-2],[3,-3],[0,-1],[1,-2],[-1,-2],[-1,-3],[0,-2],[-1,-1],[0,-2],[0,-3],[1,-5],[0,-2],[0,-3],[-3,-13],[-1,-3],[-1,-3],[-3,-3],[-1,-1],[0,-1],[-1,-2],[0,-2],[1,-4],[0,-3],[0,-4],[-1,-10],[0,-4],[1,-5],[0,-1],[-6,-3],[0,-6],[0,-2],[0,-1],[1,-1],[0,-1],[0,-2],[-2,-19],[0,-2],[0,-1],[1,-2],[2,-3],[0,-2],[1,-1],[0,-1],[0,-2],[-1,-2],[-2,-2],[-8,-8],[-5,-7],[-4,-10],[0,-1],[-2,-2],[0,-1],[-2,-1],[-6,-4],[-9,-4],[-7,-3],[-3,0],[-1,0],[-1,0],[0,-1],[0,-2],[0,-1],[0,-2],[0,-1],[-1,-1],[-1,-1],[0,-1],[-1,-1],[-1,-1],[-2,0],[-3,0],[-1,0],[-1,-1],[-1,-1],[-2,-2],[-7,-4],[-2,-1],[-2,-3],[-1,-1],[-1,-2],[0,-2],[-1,-1],[-1,-1],[-2,-1],[-9,-2],[-3,0],[-3,-2],[-5,-6],[-1,-3],[-1,-3],[-1,-1],[0,-1],[-1,-1],[-1,-2],[-6,-4],[-3,-2],[-3,-1],[-12,1],[-14,9],[-1,1],[-1,2],[-2,3],[-1,2],[-1,1],[-1,1],[-1,0],[-2,0],[-3,-3],[-2,-1],[-1,-1],[-3,-2],[0,-1],[-1,-1],[0,-1],[0,-2],[-1,-1],[-1,-3],[0,-1],[-1,-1],[0,-1],[-3,-2],[-2,-2],[-4,-2],[-6,0],[-8,2],[-2,1],[-2,1],[-1,0],[-2,1],[-2,-1],[-3,-1],[-3,-2],[-3,-1],[-29,5],[-3,-2],[-2,0],[-2,0],[-6,2],[-3,-1],[0,-16],[1,-5],[0,-12],[-1,-5],[-3,-11],[0,-1],[-1,1],[-7,7],[-2,3],[-3,2],[-7,-1],[-3,2],[-3,-2],[-6,-1],[-2,-1],[-2,-3],[-2,-3],[-2,-2],[-3,-1],[-2,-2],[-4,-6],[-3,-1],[-3,0],[-3,-2],[-2,-3],[1,-4],[-2,-2],[-4,-2],[-2,-2],[-1,-3],[-2,-7],[-1,-3],[-1,-1],[-2,-1],[-1,-1],[0,-2],[-1,-4],[-1,-1],[-2,-2],[-18,-7],[-4,0],[-4,-4],[-2,0],[-2,1],[-3,4],[-2,0],[-2,0],[-2,-1],[-2,-1],[-3,-4],[-2,0],[-1,2],[-2,1],[-2,-1],[-1,-3],[0,-2],[-1,-2],[-1,0],[-2,0],[-2,2],[-7,4],[-3,-1],[-3,-10],[-2,-3],[-14,-6],[-3,-2],[-3,-4],[-2,-2],[-7,5],[-3,-1],[2,-12],[-7,-5],[-19,-1],[-4,-2],[-3,-3],[-1,-5],[-2,-1],[-9,-5],[-11,0],[-4,-2],[-2,-1],[-1,-2],[-2,0],[-2,2],[-2,4],[-2,2],[-1,1],[-4,-1],[-2,0],[-2,-1],[-1,-2],[-1,-2],[0,-1],[-2,-1],[-2,1],[-1,2],[-1,3],[-2,2],[-1,0],[-5,0]],[[6131,6246],[4,7],[5,3],[6,1],[14,0],[5,1],[4,4],[5,12],[8,9],[1,3],[-1,22],[1,2],[4,-2],[2,1],[-2,4],[0,1],[0,1],[2,1],[4,1],[30,-4],[2,-2],[1,-1],[2,-3],[1,-1],[2,-2],[3,-1],[2,0],[2,0],[1,1],[5,3],[2,2],[3,0],[2,1],[1,1],[5,8],[7,8],[1,2],[1,2],[1,5],[0,3],[1,10],[1,2],[1,3],[4,6],[1,2],[1,3],[5,15],[12,30],[2,4],[5,6],[3,3],[1,2],[8,21],[3,7],[3,8],[3,5],[1,4],[1,1],[1,1],[10,6],[8,7],[2,1],[2,0],[5,-2],[2,-1],[2,-1],[2,-2],[7,-5],[1,-1],[7,-1],[2,-1],[1,-1],[1,-1],[11,-9],[1,-2],[2,-2],[0,-1],[1,-1],[1,0],[2,-1],[1,0],[2,1],[12,6],[1,2],[4,6],[6,11],[8,16],[2,4],[2,2],[2,1],[20,13],[16,6],[2,0],[2,-1],[2,-1],[2,-1],[2,-2],[1,-2],[1,-2],[4,-12],[1,-1],[0,-1],[2,-2],[1,-1],[1,-1],[2,-1],[2,-1],[2,0],[15,2],[8,2],[3,2],[2,1],[26,22],[1,1],[2,0],[3,0],[5,-1],[3,0],[5,1],[2,1],[15,9],[4,2],[20,3],[2,1],[2,2],[1,1],[1,2],[0,2],[0,4],[0,2],[-1,1],[0,2],[-3,5],[-2,3],[-2,5],[-1,3],[0,3],[0,1],[0,2],[0,2],[1,2],[1,3],[4,8],[3,4],[12,9]],[[6629,6605],[0,-8],[-1,-4],[0,-3],[0,-3],[1,-2],[1,-2],[0,-1],[6,-8],[5,-6],[3,-6],[1,-3],[0,-3],[1,-1],[0,-16],[3,-17],[0,-5],[1,-8],[2,-3],[1,-1],[14,-2],[7,1],[3,0],[7,-3],[2,-1],[1,-3],[0,-2],[-1,-2],[-2,-2],[-7,-4],[-26,-11],[-9,-6],[-12,-5],[-7,-2],[-3,-2],[-3,-3],[-1,-7],[-1,-14],[-1,-3],[-1,-2],[-1,0],[-2,0],[-1,2],[-1,2],[-1,5],[-1,2],[-1,1],[-1,0],[-1,0],[-1,-2],[-1,-2],[-2,-6],[-1,-2],[-1,-2],[-2,-1],[-1,-2],[-1,-2],[-2,-4],[0,-2],[0,-2],[1,-1],[1,-2],[0,-1],[12,-9],[1,-2],[1,-2],[2,-6],[1,-1],[1,-2],[1,-1],[1,-1],[5,-2],[15,-3],[1,0],[1,-1],[1,-1],[2,-3],[2,-1],[2,-2],[15,-4],[2,0],[4,1],[1,0],[1,0],[1,-1],[1,-1],[1,-1],[1,-3],[1,-2],[0,-2],[-1,-12],[1,-8],[-1,-9],[-1,-5],[0,-6],[1,-2],[0,-2],[1,-1],[1,-2],[0,-2],[1,-4],[1,-2],[1,-1],[1,-1],[2,-1],[2,0],[1,-1],[1,-1],[1,-1],[3,-6],[3,-2],[3,-6],[1,-1],[0,-2],[-1,-9],[-1,-4],[-5,-13],[-3,-6],[-2,-5],[0,-3],[-1,-3],[0,-4],[0,-2],[0,-2],[-1,-1],[-1,-2],[-3,-3],[-1,-2],[-1,-2],[0,-3],[-1,-2],[1,-15],[-1,-5],[0,-2],[-1,-2],[0,-2],[-1,-2],[-1,-2],[-1,-1],[-1,-1],[-1,-1],[-2,-1],[-6,-3],[-2,-1],[-1,-2],[-5,-11],[-2,-2],[-2,-1],[-2,-1],[-2,-1],[-2,0],[-8,1],[-2,0],[-2,-1],[-3,-1],[-1,0],[-2,0],[-3,2],[-6,7],[-2,3],[-2,2],[-1,0],[-1,0],[-5,-3],[-4,-2],[-5,-4],[-2,-1],[-5,1],[-2,-1],[-2,-1],[-1,-1],[-2,-4],[-2,-5],[-1,-8],[-2,-4],[-1,-2],[-5,-4],[-1,-1],[-1,-1],[0,-2],[-1,-2],[0,-3],[-1,-2],[1,-2],[0,-4],[2,-4],[2,-5],[2,-5],[5,-8],[0,-2],[0,-2],[-4,-8],[-9,-13],[-9,-4],[-1,-3],[0,-3],[0,-4],[-1,-2],[-2,-2],[-6,-2],[-3,-3],[-2,-3],[-1,-4],[-7,-13],[-3,-8],[0,-5],[2,-14],[0,-4],[0,-3],[-1,-4],[-2,-6],[-1,-6],[0,-3],[0,-1],[1,-5],[1,-18],[1,-4],[0,-2],[1,-1],[0,-1],[2,-1],[20,-8],[1,-1],[4,-4],[1,-1],[4,-7],[1,0],[1,-1],[1,0],[5,-2],[3,-11],[2,-5],[4,-9],[2,-4],[0,-3],[4,-16],[1,-3],[1,-2],[1,-2],[4,-5],[10,-23],[2,-3],[5,-20],[12,-14],[1,-8],[-2,-18],[0,-4],[-1,-3],[-1,-2],[-1,-3],[-4,-14],[0,-1],[-5,-11],[-5,-9],[-5,-6],[-2,-3],[-3,-10],[-5,-10],[-2,-4],[-2,-13],[-1,-6],[-1,-2],[-1,-2],[-2,-4],[-7,-8],[-6,-9],[-1,-2],[-1,-2],[-1,-7],[0,-4],[0,-6],[1,-8],[2,-5],[0,-1],[1,-1],[0,-2],[0,-1],[1,-4],[0,-21],[0,-2],[-1,-4],[-1,0],[-1,-1],[-2,0],[-8,3],[-1,0],[-2,-1],[-1,-1],[0,-1],[0,-2],[0,-3],[-1,-4],[0,-2],[-2,-4],[-6,-8],[-2,-2],[-1,0],[-1,-1],[-1,0],[-5,-1],[-4,-4],[-4,-1],[-5,0],[-2,-1],[-2,0],[-1,-2],[-1,-1],[0,-2],[0,-1],[1,-1],[3,-4],[0,-1],[1,-1],[0,-2],[-1,-1],[-1,-4],[-1,-2],[0,-2],[0,-2],[1,-1],[6,-7],[2,-2],[1,-1],[0,-1],[0,-1],[1,-4],[0,-2],[1,-1],[2,-1],[1,-2],[0,-1],[1,-2],[-1,-1],[-1,-2],[0,-2],[-1,-2],[0,-4],[0,-2],[0,-2],[2,-3],[2,-2],[2,-2],[6,-2],[2,-1],[1,-2],[1,0],[4,-6],[6,-6],[2,-3],[3,-7]],[[6954,3707],[-6,0],[-5,-3],[-10,-7],[-3,-1],[-18,-4],[-5,2],[-18,19],[-2,2],[-1,7],[-2,3],[-2,2],[-13,0],[-15,-2],[-4,1],[-4,1],[-2,-6],[-4,-9],[-1,-4],[-1,-12],[-1,-2],[-1,-3],[-1,-4],[-4,-8],[-6,-9],[-2,-2],[-7,-5],[-3,-1],[-2,-1],[-1,1],[-1,0],[-2,4],[-1,1],[-2,1],[-8,3],[-15,2],[-1,-1],[1,-2],[1,-1],[1,-1],[1,-1],[1,-1],[1,-1],[1,-4],[1,-2],[0,-1],[1,-1],[1,-1],[0,-2],[-2,-6],[0,-3],[0,-4],[-1,-4],[-1,-4],[-2,1],[-1,2],[-1,1],[-1,5],[0,2],[-1,1],[-1,0],[-1,1],[-1,0],[-6,-1],[-6,2],[-3,0],[-1,-1],[-1,-1],[-1,-2],[-1,-1],[0,-2],[-1,-4],[0,-3],[-1,-1],[0,-1],[-1,0],[-2,0],[-1,1],[-2,4],[-1,1],[-1,1],[0,-1],[-1,-2],[0,-1],[0,-2],[0,-1],[0,-1],[0,-1],[0,-1],[-1,-1],[-3,-5],[-2,-3],[-1,-1],[0,-1],[-2,-10],[0,-3],[-1,-1],[0,-1],[-1,-1],[-2,-2],[-4,-3],[-3,-2],[-2,0],[-1,1],[-1,1],[0,1],[-1,3],[0,1],[-1,1],[-1,0],[-1,0],[-4,-2],[-7,-1],[-28,-16],[-1,0],[-1,0],[-1,0],[-1,1],[-1,1],[-1,1],[-1,1],[-1,0],[-1,0],[-6,-2],[-3,0],[-3,1],[-1,1],[-1,1],[-2,3],[-1,2],[-1,0],[-1,1],[-1,0],[-2,-1],[-1,-2],[-3,-5],[-1,-3],[-1,-3],[0,-3],[0,-3],[1,-9],[0,-1],[-1,-1],[-1,-1],[-2,-1],[-2,0],[-1,0],[-8,-6],[-1,0],[-1,0],[-9,-1],[-4,-1],[-11,-11],[-15,-26]],[[6588,3516],[-7,4],[-4,2],[-8,9],[-1,1],[-2,0],[-2,-1],[-1,-2],[-3,-2],[-1,-3],[-3,-6],[-1,0],[-2,2],[0,4],[-2,3],[-3,2],[-1,3],[-1,2],[0,4],[0,11],[0,4],[-1,4],[-2,3],[-1,1],[-6,2],[-1,3],[0,3],[1,8],[2,4],[2,5],[4,8],[3,6],[1,2],[6,2],[1,2],[1,2],[1,2],[2,3],[0,1],[0,3],[0,3],[-2,5],[-1,3],[0,1],[-3,3],[-2,4],[0,1],[-1,2],[0,2],[0,2],[0,6],[1,6],[0,3],[0,3],[-1,4],[-3,10],[-1,3],[0,1],[-1,0],[-1,1],[-1,0],[-1,1],[-1,0],[-7,-2],[-4,0],[-2,0],[0,1],[-1,2],[0,3],[2,15],[0,3],[8,23],[0,5],[0,1],[-12,-11],[-4,-5],[-8,-11],[-5,-4],[-38,-62],[-3,-10],[-3,-5],[-3,-3],[-2,1],[0,1],[0,1],[-4,6],[-1,0],[-1,-3],[-1,-3],[1,-4],[-1,-3],[-1,-1],[-2,0],[-1,1],[0,2],[0,9],[-1,5],[-2,3],[-4,0],[-2,-3],[-4,-5],[-3,-3],[-2,1],[-2,4],[-3,28],[1,9],[3,6],[10,9],[2,4],[2,4],[1,7],[-1,6],[-1,0],[-4,-7],[-7,-10],[-1,-4],[-1,2],[-2,6],[-1,3],[0,5],[0,3],[-5,7],[-5,-3],[-6,-7],[-1,-1],[-2,-2],[-1,-1],[-1,0],[-1,-1],[-1,-6],[-1,-2],[-1,-1],[-1,-1],[-1,-2],[-1,-1],[-2,-1],[-3,0],[-2,-1],[-1,0],[0,-2],[-1,-1],[0,-1],[0,-2],[-1,-2],[-1,-2],[-2,-2],[-1,-1],[-1,0],[-16,1],[-9,3],[-6,1],[-2,-1],[-1,-1],[-13,-18],[-3,-2],[-8,-6],[-1,-1],[-1,0],[-1,1],[0,2],[0,10],[0,2],[-1,2],[-2,2],[-2,2],[-1,1],[-1,1],[-1,2],[0,1],[-2,9],[0,1],[-1,0],[-1,-1],[-3,-5],[-1,-1],[-1,0],[-1,0],[-2,7],[-2,5],[-2,2],[-1,2],[-7,7],[-26,32],[-11,4],[-14,13],[-3,1],[-1,0],[-1,0],[-1,0],[-1,-2],[-2,-2],[-1,-1],[-2,-1],[-2,0],[-16,1],[-15,8],[-2,1],[-1,0],[-3,0],[-7,-3],[-1,-1],[-2,0],[-1,1],[-1,1],[-1,5],[-1,1],[-1,2],[-4,3],[-2,2],[0,1],[-3,3],[-11,10],[-31,7],[-18,1],[-24,8],[-5,1],[-4,0],[-1,-2],[-1,-3],[-1,-3],[0,-3],[0,-4],[0,-1],[-1,-2],[-3,-1],[-2,0],[-4,3],[-4,3],[-13,2],[-3,0],[-6,-2],[-1,-1],[-1,-1],[-2,-2],[-3,-5],[6,-4],[1,-1],[0,-3],[0,-2],[-1,-1],[-2,-2],[-1,-2],[-1,-3],[-1,-7],[1,-3],[2,-6],[2,-4],[0,-2],[2,-2],[6,-5],[2,-3],[4,-5],[0,-3],[1,-6],[-3,-6],[-1,0],[-6,-6],[-4,-3],[-2,-1],[-2,0],[-2,1],[-3,3],[-2,0],[-1,-1],[-1,-2],[-1,-3],[1,-3],[-1,-3],[-1,-2],[-4,-4],[-2,-1],[-2,-1],[-4,2],[-2,2],[-4,2],[-3,2],[-2,3],[-8,12],[-2,3],[-2,1],[-5,2],[-2,0],[-2,2],[-1,2],[-1,1],[-2,1],[-2,1],[-53,-14],[-3,-2],[-3,-4],[-7,-15],[-1,-11],[-1,-2],[-1,-1],[-2,-2],[-3,0],[-2,1],[-1,1],[-3,4],[-1,1],[-2,0],[-2,-1],[-7,-5],[-1,0],[-1,0],[-1,-1],[0,-3],[-2,-3],[-1,-1],[-1,0],[-1,0],[-2,0],[-1,0],[-1,0],[-1,0],[0,-2],[-1,-3],[0,-4],[0,-1],[-1,1],[-1,2],[-1,0],[-1,-2],[-1,-4],[0,-2],[0,-2],[-1,0],[-1,0],[-1,-1],[0,-2],[0,-3],[0,-2],[0,-2],[-1,-3],[-2,-2],[-1,0],[0,-1],[-1,-1],[0,-2],[-1,-3],[1,-2],[0,-2],[0,-3],[-1,-2],[-4,-4],[-2,-2],[-1,-3],[0,-3],[0,-3],[0,-4],[0,-1],[1,-2],[1,-2],[-1,-2],[-1,-2],[-4,-5],[-1,0],[-5,-5],[-2,-4],[-1,-2],[1,-4],[0,-3],[0,-2],[0,-5],[-2,-9],[-1,-4],[-5,-4]],[[6109,4628],[3,-13],[2,-3],[1,-1],[5,-7],[2,-1],[2,-1],[3,-1],[2,-2],[1,-1],[1,-2],[1,-4],[1,-6],[1,-5],[5,-12],[3,-5],[5,-7],[0,-1],[1,-1],[0,-3],[0,-1],[0,-2],[0,-1],[-1,-3],[0,-2],[0,-2],[0,-2],[1,-2],[1,-1],[2,-1],[2,-1],[2,-2],[6,-8],[1,-1],[1,-1],[1,-1],[2,-1],[3,-4],[3,-5],[6,-13],[2,-4],[5,-8],[6,-6],[3,-4],[1,-2],[1,-1],[1,-4],[1,-3],[1,-7],[0,-2],[1,-3],[1,-2],[1,-1],[0,-1],[2,-1],[2,1],[1,2],[2,1],[1,2],[3,2],[2,1],[1,0],[2,0],[3,-1],[8,-6],[3,-2],[3,-4],[5,-13],[0,-2],[4,-2],[26,-10],[3,-2],[4,-9],[3,2],[1,0],[1,1],[1,1],[0,1],[1,1],[1,0],[2,-1],[1,-1],[5,-7],[7,-7],[1,-1],[2,0],[4,1],[2,0],[5,-2],[1,-1],[3,-3],[5,-6],[2,-1],[1,0],[2,0],[2,1],[1,1],[1,1],[1,1],[1,0],[3,1],[11,-1],[2,2],[2,1],[1,0],[7,0],[4,1],[13,-1],[22,-6],[2,0],[2,1],[5,2],[8,5],[23,6],[1,2],[2,3],[2,1],[2,0],[2,0],[3,-2],[4,-6],[2,-3],[1,0],[0,-1],[2,0],[1,1],[2,1],[2,1],[1,1],[2,3],[2,2],[1,0],[2,0],[3,-1],[1,-1],[1,-1],[2,-2],[0,-2],[1,-3],[1,-1],[1,-1],[1,-1],[4,-1],[1,-1],[1,0],[2,-3],[0,-1],[1,-1],[0,-2],[1,-2],[0,-2],[1,0],[5,-3],[9,-4],[2,0],[15,2],[1,1],[2,1],[1,2],[1,3],[0,3],[1,1],[1,2],[1,1],[1,1],[2,0],[1,0],[1,3],[1,3],[0,1],[1,2],[2,2],[2,2],[3,2],[1,0],[3,0],[1,-1],[2,0],[1,0],[3,3],[1,-1],[1,0],[0,-1],[0,-2],[7,-11],[0,-2],[0,-3],[0,-4],[-1,-5],[-1,-2],[0,-1],[0,-1],[-1,-2],[1,-2],[1,-7],[1,-15],[0,-2],[-1,-4],[-1,-2],[0,-1],[-2,-2],[-1,-3],[0,-1],[0,-5],[3,-9],[3,-10],[0,-5],[0,-10],[0,-3],[1,-2],[7,-10],[1,-3],[1,-2],[0,-1],[2,-13],[1,-2],[1,-2],[1,0],[1,-1],[2,1],[2,1],[1,1],[2,1],[1,-1],[1,-1],[2,-2],[0,-1],[1,-2],[0,-7],[-1,-8],[0,-2],[0,-3],[1,-2],[1,0],[1,-1],[10,3],[1,1],[3,1],[4,5],[9,15],[0,1],[1,1],[1,1],[1,0],[2,-1],[8,-6],[6,-2],[1,-1],[1,-1],[1,-2],[2,-4],[1,-2],[7,-8],[3,-2],[2,-2],[1,0],[1,0],[3,1],[2,1],[7,0],[1,0],[2,1],[1,2],[1,1],[1,1],[1,0],[3,-1],[4,-2],[2,-2],[1,-1],[1,-1],[0,-1],[1,-2],[0,-1],[0,-1],[0,-2],[0,-3],[-1,-2],[-1,-3],[-2,-4],[-1,-1],[0,-1],[0,-2],[-1,-3],[0,-2],[1,-1],[1,-9],[1,-5],[3,-1],[8,1],[2,0],[2,-1],[1,-1],[1,-1],[4,-8],[3,-3],[1,0],[2,-1],[2,1],[2,0],[1,-1],[1,-1],[1,-3],[2,-2],[4,-2],[5,-1],[2,0],[0,1],[2,0],[2,1],[3,0],[20,-3],[3,1],[3,1],[2,2],[2,2],[1,2],[2,2],[1,3],[3,10],[4,8],[1,1],[1,1],[2,1],[1,0],[1,-1],[1,-1],[1,-2],[2,-6],[1,-1],[2,-3],[0,-1],[1,-1],[0,-1],[0,-2],[0,-3],[0,-1],[0,-2],[-1,-6],[-1,-2],[0,-1],[1,-2],[0,-1],[0,-1],[1,-2],[1,-2],[1,-1],[3,-3],[0,-1],[1,-1],[0,-1],[0,-1],[1,-1],[0,-2],[2,-2],[1,0],[2,-1],[8,2],[4,3],[2,1]],[[6588,3516],[-1,-20],[0,-4],[1,-5],[1,-1],[3,-1],[4,-1],[2,-1],[3,-4],[1,-1],[1,0],[1,-1],[1,-1],[1,-2],[1,-2],[1,-3],[1,-2],[1,-2],[2,-2],[10,-6],[2,-2],[1,-2],[0,-2],[1,-6],[1,-2],[-3,-20],[-1,-3],[1,-3],[0,-5],[-1,-7],[-1,-8],[-2,-4],[-7,-9],[4,-4],[3,-4],[2,-4],[1,-1],[5,-2],[4,-1],[1,-1],[0,-3],[-1,-8],[-1,-3],[1,-2],[0,-1],[1,-1],[5,-3],[1,-1],[1,-1],[0,-1],[1,-1],[1,-3],[0,-2],[0,-2],[0,-1],[-1,-2],[0,-3],[-4,-10],[-1,-3],[-1,-2],[-1,-2],[-3,-3],[-2,-2],[-1,-1],[-9,-4],[-1,-1],[-1,-1],[0,-3],[0,-4],[-1,-2],[-1,-1],[-3,-4],[-1,-1],[-3,-2],[-1,-1],[0,-2],[-1,-3],[0,-4],[0,-4],[0,-7],[-1,-3],[0,-1],[-3,-5],[-12,-13],[-4,-2],[-9,-2],[-2,0],[-2,-1],[-4,-4],[-1,0],[-1,0],[-5,-1],[-1,0],[-1,-1],[-1,-1],[-1,-3],[-2,-6],[-1,-1],[0,-5],[1,-15],[0,-5],[0,-3],[-1,-4],[-1,-2],[-3,-4],[-5,-13],[-2,-3],[-4,-9],[-4,-18],[-1,-4],[-3,-7],[-1,-2],[0,-2],[0,-4],[1,-3],[1,-2],[1,-2],[0,-2],[1,-17],[2,-11],[1,-9],[0,-3],[0,-2],[1,-2],[1,-1],[1,-2],[8,-5],[1,0],[2,0],[2,1],[3,2],[3,4],[1,1],[1,1],[4,1],[2,1],[2,-1],[1,0],[4,-4],[4,-2],[1,-3],[-1,-3],[0,-5],[0,-3],[0,-3],[0,-2],[-4,-12],[-1,-2],[-7,-10],[-2,-4],[-1,-5],[-1,-2],[0,-12],[0,-2],[1,-1],[1,-2],[1,0],[1,1],[1,1],[1,1],[1,-1],[0,-1],[1,-1],[1,-7],[1,-1],[0,-2],[2,-2],[0,-1],[2,-3],[1,-1],[1,-2],[1,-5],[1,-2],[1,-1],[1,-1],[2,-1],[0,-1],[1,-1],[0,-1],[0,-3],[-1,-3],[-2,-3],[0,-2],[0,-1],[-1,-10],[0,-1],[-1,-2],[0,-1],[-4,-4],[-1,-3],[0,-1],[0,-5],[3,-28],[2,-12],[2,-2],[1,-2],[1,-2],[2,-1],[1,-1],[7,-1],[3,-1],[4,-3],[1,-1],[1,-3],[1,-1],[1,-1],[2,-1],[2,-1],[5,0],[2,0],[1,-3],[0,-4],[-2,-10],[0,-4],[0,-3],[0,-1],[0,-2],[0,-1],[-1,-3],[0,-2],[-3,-6],[-4,-6],[-1,-2],[-1,-2],[0,-2],[0,-4],[-1,-2],[1,-3],[0,-2],[-2,-12],[-1,-4],[0,-2],[-1,-3],[-8,-15],[-2,-3],[-1,-7],[3,-2],[1,0],[1,0],[1,0],[1,1],[1,1],[2,3],[1,1],[1,1],[1,0],[2,-3],[1,-1],[1,0],[1,-1],[1,0],[1,1],[1,0],[2,1],[7,8],[3,3],[2,0],[2,1],[1,0],[1,0],[1,-1],[1,-2],[1,0],[3,-1],[1,-1],[1,-1],[0,-4],[1,-1],[0,-2],[-1,-1],[-1,-2],[-1,-2],[-4,-4],[-1,-1],[-8,-4],[-1,-1],[-1,-1],[-1,-3],[-1,-2],[-1,-1],[-2,-1],[-2,0],[-1,-1],[-1,-1],[-1,-1],[-1,-2],[-2,-5],[-1,-3],[-1,-4],[0,-10],[0,-13],[0,-2],[0,-3],[-1,-4],[-1,-3],[-4,-5],[-2,-5],[-2,-5],[-2,-10],[-6,-12],[0,-3],[0,-2],[0,-4],[1,-9],[0,-3],[1,-3],[3,-4],[4,-5],[2,-4],[1,-2],[0,-3],[1,-3],[-1,-2],[0,-2],[-3,-8],[-3,-6],[-1,-5],[-1,-12]],[[5181,3161],[-5,9],[0,3],[1,2],[1,3],[-1,2],[-4,6],[-2,6],[-1,4],[0,3],[0,1],[4,8],[2,5],[1,3],[1,5],[0,12],[0,1],[0,1],[-1,1],[-1,0],[-2,0],[-4,-2],[-1,0],[-1,0],[-2,1],[-2,2],[-1,3],[0,7],[-1,2],[-2,2],[-19,7],[-6,-1],[-6,-6],[-1,-2],[-1,-2],[-1,-7],[-1,-6],[0,-3],[1,-8],[-1,-3],[-1,-2],[-1,-2],[-4,-2],[-1,-2],[-1,-2],[-1,-2],[-2,-1],[-2,-1],[-7,-1],[-8,-4],[-2,-1],[-2,1],[-5,2],[-2,1],[-2,-1],[-8,-7],[-2,-1],[-2,-1],[-4,-1],[-3,1],[-3,2],[-7,6],[-4,1],[-2,1],[-1,1],[-18,19],[-2,3],[-2,5],[-3,8],[-1,4],[1,3],[9,3],[7,4],[1,2],[1,2],[1,3],[-1,2],[0,3],[-1,2],[-4,13],[-1,3],[0,3],[1,2],[2,3],[0,3],[-1,3],[-4,6],[-1,3],[-2,2],[-2,2],[-8,4],[-3,1],[-3,4],[-1,3],[-1,3],[0,3],[0,10],[-1,5],[1,8],[-1,5],[0,3],[0,3],[0,2],[0,1],[0,1],[-1,2],[0,3],[0,3],[0,2],[2,1],[7,3],[2,1],[3,3],[1,0],[1,-1],[2,0],[2,0],[1,1],[1,1],[2,3],[0,1],[2,1],[1,1],[3,2],[1,0],[0,2],[1,1],[0,1],[-1,1],[0,2],[-2,1],[-2,2],[-4,0],[-1,1],[-4,0],[-2,0],[0,-1],[-1,-1],[-2,-1],[-3,-1],[-3,0],[-3,2],[-3,3],[-1,3],[-1,0],[0,-2],[-1,-3],[-1,-1],[-9,0],[-6,-1],[-4,-2],[-3,-3],[-1,-1],[-2,0],[-4,5],[-1,1],[-3,-1],[-6,-4],[-4,-1],[-3,1],[-2,1],[-2,2],[-3,2],[-2,0],[-1,-1],[-2,0],[-1,2],[0,5],[0,2],[-1,3],[-2,-2],[-1,-4],[-1,-4],[-2,-5],[1,-4],[-1,-4],[-4,-10],[-2,-5],[1,-2],[4,-11],[6,-22],[2,-15],[1,-5],[-15,-18],[-6,-4],[-5,-7],[-2,0],[-2,4],[-2,4],[-3,5],[-3,2],[-2,-1],[-1,-2],[-1,-1],[-1,-2],[-2,0],[-4,-1],[-1,0],[-2,-4],[-1,-6],[-3,-6],[-13,-13],[-1,-3],[1,-5],[-1,-2],[-1,-2],[-1,-2],[0,-2],[-1,-2],[2,-7],[2,-1],[3,1],[2,-1],[2,-12],[2,-5],[4,2],[1,1],[1,1],[2,0],[1,-1],[-1,-2],[-1,-2],[-3,-2],[-3,-4],[-1,-4],[0,-18],[-1,-4],[-4,-6],[0,-3],[0,-2],[-1,-2],[-2,0],[-1,-2],[0,-3],[0,-3],[-1,-2],[-1,-1],[-5,-4],[-14,-18],[-2,-3],[0,-2],[-4,-2],[-1,-2],[0,-2],[3,-7],[-1,-5],[-2,-2],[-6,0],[-3,-2],[-2,-1],[-9,-19],[-2,-6],[-1,-10],[-2,-4],[-2,-2],[-2,-1],[-2,1],[-2,-1],[-2,0],[-4,-1],[-1,-1],[-3,-3],[-1,-1],[-4,0],[-2,1],[-2,2],[-2,6],[-2,-4],[-2,-10],[-4,-9],[-3,-11],[0,-4],[0,-5],[3,-10],[2,-23],[-1,-6],[-1,-3],[-3,-9],[-1,-5],[0,-2],[0,-1],[-1,-1],[1,-2],[0,-2],[2,-2],[1,-2],[0,-2],[-1,-9],[1,-9],[4,-9],[5,-8],[3,-8],[3,-17],[1,-3],[1,-1],[1,-3],[0,-7],[0,-3],[2,-6],[1,-5],[1,-5],[-1,-11],[1,-5],[2,-3],[2,-3],[0,-6],[-1,-2],[-2,-6],[-1,-1],[-2,0],[-2,0],[-2,-2],[0,-2],[0,-2],[1,-3],[0,-2],[0,-1],[-2,-3],[0,-1],[0,-3],[2,-5],[0,-3],[-1,-5],[-1,-4],[-3,-4],[-4,-2],[-7,2],[-2,1],[0,3],[-3,0],[-5,-10],[-4,-3],[-5,-2],[-4,-4],[-7,-15],[-2,-2],[-2,-2],[-3,0],[-2,1],[-3,5],[-6,4],[0,7],[2,8],[0,7],[-1,3],[-2,3],[-3,2],[-3,1],[-2,-1],[-9,-10],[-6,-4],[-3,-1],[-4,0],[-4,-1],[-2,-4],[-2,-5],[-2,-4],[-4,-3],[-4,-3],[-4,-1],[-3,-1],[-3,-1],[-2,-4],[-1,-5],[-2,-2],[-12,-15],[-3,-5],[-2,-4],[-1,-4],[-1,0],[-1,0],[-29,4],[-1,0],[-1,-1],[-1,-2],[-3,-8],[-1,-2],[-2,-2],[-1,-1],[-2,-1],[0,1],[-1,0],[0,1],[-1,4],[-5,3],[-1,1],[-2,3],[-1,2],[0,2],[-1,2],[0,5],[1,4],[0,2],[-1,5],[-1,1],[-1,2],[-6,5],[-8,8],[-1,2],[-2,0],[-1,0],[0,-1],[-1,-1],[-2,-3],[0,-1],[-1,-1],[-1,0],[-1,0],[-1,0],[-2,0],[-1,2],[-1,1],[-1,3],[-1,1],[0,1],[0,2],[0,3],[1,3],[0,1],[0,2],[-1,4],[0,1],[1,1],[0,1],[3,4],[1,1],[1,1],[0,1],[1,0],[2,1],[1,2],[2,2],[0,3],[0,2],[-1,1],[-2,1],[-2,3],[-1,2],[-1,3],[-2,9],[-1,3],[-2,4],[-5,7],[-3,4],[-1,1],[-2,1],[-9,3],[-4,6],[-1,2],[-1,3],[0,2],[0,2],[1,2],[1,2],[3,4],[1,1],[1,2],[2,8],[4,13],[1,3],[0,2],[1,8],[0,3],[-1,2],[0,1],[-1,2],[-2,0],[-2,0],[-2,-1],[-1,-2],[-2,-9],[-1,-2],[-2,-1],[-2,0],[-2,2],[-1,2],[-2,4],[-1,4],[-1,16],[-1,4],[-1,3],[-2,3],[-29,38],[-2,3],[-1,5],[-1,5],[1,7],[2,6],[3,10],[0,5],[0,5],[-1,2],[-1,1],[-2,0],[-8,1],[-2,0],[-1,-1],[-4,-4],[-3,-2],[-2,-1],[-2,1],[-1,3],[-1,2],[-1,4],[0,8],[0,15],[-1,8],[-2,12],[-6,16],[-3,4],[-2,2],[-2,1],[-3,3],[-2,3],[0,3],[0,1],[1,13],[0,3],[-35,83],[-3,5],[-3,1],[-2,-1],[-1,-1],[-1,-2],[-1,-3],[-2,-6],[-2,-3],[-2,-3],[-1,-1],[-1,-1],[-2,0],[-2,0],[-3,2],[-3,4],[-3,2],[-3,0],[-2,-1],[-3,-2],[-1,-2],[-10,-22],[-5,-3],[-3,6],[0,1],[0,2],[1,3],[-2,11],[1,3],[0,1],[-1,4],[-1,3],[-2,2],[-6,3],[-3,2],[-3,2],[-2,4],[-3,10],[-2,4],[-10,14],[-4,7],[-2,6],[-2,10],[-3,11],[0,2],[0,4],[1,2],[2,3],[3,1],[8,4],[3,3],[5,5],[1,3],[0,3],[-1,2],[-3,3],[0,2],[0,2],[1,5],[0,4],[-2,4],[-1,2],[-11,9],[-3,7],[-3,5],[-5,7],[-2,4],[-1,4],[-4,14],[-1,3],[-1,2],[-2,3],[-2,2],[-3,2],[-2,0],[-2,0],[-1,1],[-2,1],[-2,3],[-1,3],[0,3],[0,3],[-1,4],[0,3],[-1,2],[-1,2],[-2,3],[-3,3],[-2,1],[-3,2],[-1,1],[-2,3],[-2,4],[-1,5],[0,3],[0,12],[0,4],[-1,2],[-1,1],[-1,0],[-4,-4],[-3,-2],[-3,-1],[-3,-1],[-2,-3],[-1,-3],[-2,-9],[-1,-5],[-2,-4],[-4,-5],[-2,-3],[-1,-4],[-2,-18],[-1,-4],[-1,-2],[-1,-3],[-1,0],[-3,0],[-2,-1],[-3,-3],[-7,-4],[-2,-1],[-1,-2],[-1,-2],[-1,-2],[-1,-3],[0,-2],[1,-3],[2,-4],[1,-2],[1,-3],[-1,-3],[-1,-5],[0,-5],[0,-6],[0,-13],[2,-10],[2,-8],[1,-3],[-3,-18],[0,-15],[0,-1],[-2,3],[0,8],[-1,4],[-7,11],[-2,1],[-2,1],[-1,2],[-1,3],[-1,2],[0,1],[-3,3],[-1,1],[0,2],[0,3],[0,3],[-1,1],[-2,0],[-1,2],[-3,5],[-11,26],[0,5],[2,2],[1,5],[0,6],[0,5],[-1,2],[-1,6],[0,3],[1,4],[0,3],[-1,3],[-1,4],[-1,3],[0,3],[-3,3],[-1,3],[0,3],[-2,6],[0,3],[0,3],[1,5],[0,3],[-1,5],[-3,11],[-1,6],[1,6],[3,17],[0,11],[-2,34],[0,35]],[[5218,1955],[-1,1],[-7,7],[-2,3],[-3,10],[-2,6],[-1,2],[-2,1],[-1,0],[-1,-1],[-2,0],[-2,2],[-3,3],[-2,3],[-2,4],[-2,4],[0,5],[-2,3],[-3,0],[-2,-2],[-6,-7],[-1,-3],[-1,-5],[-1,-5],[-1,-4],[-3,-3],[-2,-1],[-1,1],[-1,1],[-1,0],[-1,0],[-1,2],[-1,-1],[-1,-1],[0,-1],[-14,-6],[-2,-2],[-5,-5],[-6,-4],[-6,-8],[-1,-2],[-3,1],[-4,3],[-2,1],[-2,-2],[0,-4],[-1,-4],[0,-4],[-4,-5],[-4,-1],[-3,-3],[5,-35],[0,-5],[-1,-3],[-8,-7],[-1,-3],[-6,-12],[-2,-3],[-3,-3],[-2,-1],[-4,-1],[-5,3],[-3,1],[-1,-1],[-2,-3],[-1,-1],[-1,1],[-1,1],[-1,1],[-1,1],[0,1],[0,1],[0,2],[-2,0],[0,-1],[-3,-5],[-11,-13],[-3,-5],[-14,-14],[-1,-1],[-3,-1],[-2,2],[-3,3],[-9,7],[-2,3],[-2,5],[0,6],[1,6],[1,5],[-4,0],[-14,-7],[-5,-1],[-2,-1],[-2,-2],[-1,-2],[-2,-6],[-2,-2],[-3,-7],[-2,-7],[-1,-16],[-1,-4],[-2,-8],[0,-4],[0,-4],[0,-4],[-1,-4],[-1,-3],[-3,-2],[-1,2],[-2,3],[-2,1],[-2,1],[-2,7],[-1,2],[-1,1],[-3,1],[-1,0],[-4,6],[-1,1],[-1,2],[-3,8],[-1,2],[-2,3],[-11,15],[-11,21],[-2,1],[-7,-6],[-2,-2],[-1,-3],[-1,-4],[-2,-4],[-3,-7],[-1,-2],[1,-12],[0,-2],[0,-2],[-6,-12],[-1,-1],[-3,0],[-2,2],[-4,7],[-6,15],[-2,4],[0,5],[0,4],[0,4],[-2,2],[-2,3],[-1,3],[-2,2],[-2,2],[-4,2],[-2,-1],[-6,-17],[-3,-12],[-2,-2],[-2,-4],[-10,-7],[-4,-6],[-2,-7],[0,-2],[2,-4],[0,-3],[0,-2],[-9,-9],[-2,-1],[-3,-1],[-2,-2],[0,-4],[0,-4],[0,-5],[-3,-2],[-3,0],[-3,2],[-6,5],[-6,3],[-3,2],[-1,3],[-6,16],[-1,5],[-1,5],[-2,3],[-3,3],[-7,2],[-3,2],[-10,12],[-3,2],[-7,2],[-3,2],[-1,1],[-2,4],[-1,1],[-2,0],[-1,-2],[-1,-2],[-1,-1],[-3,0],[-2,3],[0,5],[-2,5],[-3,4],[-5,2],[-4,-1],[-2,-5],[-2,-9],[-1,-5],[-2,-3],[-2,-1],[-2,0],[-2,0],[-1,-2],[0,-3],[1,-2],[2,-1],[1,-2],[0,-2],[1,-1],[-1,-3],[-9,-14],[-12,-18],[-1,-5],[-1,-9],[-1,-4],[-2,-5],[-3,1],[-3,2],[-3,1],[-3,-1],[-3,-2],[-2,-4],[-2,0],[-1,2],[0,3],[0,2],[-2,1],[-2,1],[-10,4],[-3,0],[-3,0],[-4,-3],[-7,0],[-3,-1],[-1,-5],[-1,-6],[-2,-3],[-4,1],[-4,6],[-5,18],[-5,6],[-1,0],[-1,0],[-5,-1],[-4,-3],[-3,-5],[-3,-4],[-1,-6],[0,-5],[0,-5],[0,-6],[-1,0],[-3,-4],[0,-2],[0,-4],[-2,-11],[-1,-2],[-2,-1],[-2,0],[-2,1],[-2,0],[-2,-1],[-2,-2],[-2,-5],[0,-5],[5,-9],[2,-5],[1,-7],[-1,-5],[-1,-4],[0,-6],[2,-6],[4,-11],[2,-5],[1,-11],[1,-3],[4,-4],[6,-2],[1,-3],[0,-2],[0,-2],[1,-2],[3,-8],[2,-6],[1,-2],[4,-6],[0,-2],[-3,-15],[-1,-5],[1,-4],[0,-2],[3,-3],[1,-2],[0,-3],[-1,-5],[1,-3],[2,-3],[4,-1],[2,-2],[0,-5],[-2,-5],[-3,-2],[-4,0],[-2,-3],[0,-4],[2,-11],[-1,-5],[-2,-5],[-1,-26],[-2,-10],[0,-5],[2,-4],[2,-3],[8,-5],[2,-2],[2,-2],[2,-3],[0,-4],[0,-2],[0,-1],[-1,-2],[-1,-1],[-1,-2],[-1,0],[-4,1],[-2,-3],[0,-3],[1,-4],[0,-2],[0,-3],[-2,-2],[-3,-1],[-3,-1],[-2,0],[-3,3],[-3,8],[-2,3],[-4,0],[-6,-4],[-2,1],[-1,2],[0,3],[1,2],[1,3],[-1,3],[-1,0],[-1,1],[-5,2],[-4,1],[-4,1],[-18,-6],[-3,-2],[-8,-8],[-4,-2],[-3,-1],[-2,2],[-3,3],[-1,4],[-2,7],[-2,3],[1,10],[1,4],[-1,1],[-1,2],[-1,2],[3,5],[2,7],[0,3],[-1,0],[0,-1],[-4,4],[-4,4],[-1,1],[-1,6],[3,21],[0,8],[-2,4],[-3,0],[0,5],[0,4],[2,4],[1,3],[-1,3],[0,4],[-1,5],[-2,5],[-2,2],[-2,3],[0,13],[-1,4],[-6,2],[-2,-1],[-2,-2],[-3,-3],[-8,-9],[-3,-1],[-16,-8],[-7,-5],[-6,-8],[-6,-15],[-9,-14],[-7,-6],[-2,-4],[-4,-3],[-9,-3],[-4,-1],[-8,2],[-3,-1],[-3,0],[-2,2],[-2,4],[-1,4],[-5,7],[-2,2],[-4,1],[-3,0],[-3,-2],[-2,-3],[-2,-5],[-2,-3],[-2,-2],[-5,-3],[-2,-2],[-6,-6],[-1,-2],[-4,3],[-6,12],[-3,3],[-2,4],[-1,5],[-1,6],[0,4],[1,4],[1,5],[1,4],[2,4],[2,1],[1,1],[2,1],[0,2],[-1,2],[-1,1],[-2,3],[-7,3],[-14,1],[-5,5],[-2,4],[-2,15],[-3,9],[0,5],[0,6],[6,19],[1,10],[-2,9],[-2,2],[-1,4],[0,3],[2,4],[-13,3],[-3,-1],[-1,-2],[0,-5],[-1,-2],[-2,0],[-1,1],[-1,2],[-2,2],[-10,7],[-2,2],[-3,-3],[-2,-4],[-2,-2],[-3,3],[-1,1],[-1,5],[-1,2],[-2,1],[-15,6],[-6,-1],[-2,0],[-9,6],[-2,-1],[-10,-6],[-5,-1],[-13,2],[-7,3],[-7,5],[-4,5],[5,4],[-2,5],[1,5],[5,11],[1,4],[2,13],[1,3],[3,5],[1,3],[0,4],[-1,1],[-1,2],[1,4],[7,5],[10,16],[1,3],[1,3],[0,3],[-2,4],[0,3],[0,5],[2,9],[0,6],[-8,29],[0,7],[1,2],[2,2],[2,1],[2,1],[1,2],[3,10],[5,7],[1,5],[1,4],[-1,12],[0,3],[3,2],[9,-7],[5,-1],[4,5],[-1,6],[-6,12],[-1,5],[0,4],[0,4],[0,4],[-2,2],[-13,0],[-3,4],[-5,10],[-3,3],[-3,-1],[-2,-4],[-1,-4],[-2,-2],[-3,-1],[-1,-2],[-4,-4],[-2,-2],[-1,1],[-1,1],[-1,1],[-1,2],[0,5],[-1,1],[-1,-1],[-1,1],[-3,0],[-4,2],[-1,0],[-6,-1],[-3,0],[-3,2],[-3,5],[-1,5],[-1,4],[-3,0],[-3,0],[-3,0],[-6,3],[-12,2],[0,9],[8,22],[-2,7],[-6,0],[-1,0],[0,3],[1,1],[2,2],[1,1],[2,5],[0,5],[0,5],[-3,10],[-2,6],[-2,3],[-3,0],[-4,-1],[-1,2],[-3,13],[0,4],[2,3],[8,7],[1,2],[1,3],[0,2],[-1,1],[-7,16],[-1,5],[0,13],[-1,4],[-3,3],[-6,2],[-5,2],[-4,0],[-1,1],[-2,2],[0,2],[0,1],[1,2],[-1,20],[0,3],[1,7],[0,2],[-1,5],[-1,2],[1,2],[2,2],[3,1],[3,2],[8,10],[15,15],[3,8],[-1,8],[-2,-1],[-6,-3],[-17,-2],[-5,-4],[-7,-1],[-5,-4],[-2,-1],[-2,-2],[-3,2],[-4,3],[-7,7],[-4,2],[-7,0],[-6,0],[-3,-2],[-5,-4],[-3,-2],[-6,0],[-12,5],[-6,0],[-13,-4],[-35,-21],[-2,-2],[-7,-12],[-3,-3],[-2,-1],[-2,0],[-4,-2],[-3,-3],[-8,-13],[-4,-2],[-3,-2],[-2,-1],[-1,-1],[-2,-5],[-1,5],[-1,3],[-1,2],[-6,6],[-6,5],[-3,5],[31,48],[2,6],[1,6],[-1,3],[-2,5],[-1,3],[1,3],[1,1],[1,2],[1,2],[-1,9],[-12,7],[-1,7],[2,3],[4,0],[2,2],[0,5],[-1,3],[-3,3],[-1,3],[0,3],[-1,3],[-1,2],[-1,2],[-2,1],[-4,0],[-13,-3],[-1,2],[0,6],[4,36],[-1,33],[0,6],[11,11],[7,8],[3,3],[4,1],[6,-1],[4,0],[3,2],[1,5],[-2,4],[-6,7],[-3,8],[0,9],[0,18],[-1,10],[1,2],[3,1],[2,3],[3,7],[4,11],[4,21],[3,7],[6,-5],[2,-4],[1,-4],[2,-2],[3,0],[4,3],[7,15],[3,2],[7,5],[1,3],[3,11],[2,6],[3,3],[3,0],[0,2],[0,4],[0,10],[0,3],[-1,2],[-1,7],[0,2],[1,2],[3,4],[2,3],[1,4],[0,13],[0,3],[2,1],[13,-7],[3,-2],[4,-7],[2,-1],[4,2],[3,3],[3,2],[1,4],[3,12],[0,3],[7,12],[0,2],[0,3],[1,3],[1,2],[1,1],[1,2],[2,9],[2,5],[2,3],[4,6],[1,2],[3,0],[2,-1],[4,-4],[5,-3],[1,-1],[2,-1],[2,2],[9,10],[1,2],[1,4],[0,3],[0,3],[-1,2],[-2,1],[-1,1],[-1,5],[-6,12],[-1,3],[-2,0],[-2,0],[-2,0],[-1,4],[0,6],[0,4],[-3,10],[-1,4],[-1,5],[1,4],[2,3],[7,2],[2,2],[1,-1],[0,-9],[1,-3],[2,0],[6,7],[1,2],[0,6],[1,1],[2,2],[0,2],[0,5],[-4,12],[-2,3],[-3,-1],[-1,2],[0,5],[1,5],[2,6],[6,11],[2,4],[1,5],[0,5],[1,35],[1,9],[3,8],[2,5],[-1,6],[-1,3],[0,2],[0,2],[0,2],[-2,9],[0,5],[3,10],[0,6],[-2,4],[-4,9],[0,5],[1,3],[2,4],[0,3],[0,2],[-3,23],[1,5],[2,5],[1,5],[-1,5],[-3,3],[-2,2],[-2,1],[-1,5],[-2,17],[0,9],[1,7],[4,18],[1,9],[-2,8],[-1,1],[-1,1],[-1,0],[-1,3],[-1,23],[1,11],[2,11],[-1,5],[-1,3],[-1,3],[-2,2],[-1,1],[-2,0],[-2,-1],[-1,-2],[-2,-2],[-4,0],[-3,3],[-3,11],[-4,3],[-4,-3],[-3,4],[-9,6],[-2,1],[-2,-3],[0,-5],[0,-9],[-1,-9],[-1,-8],[-3,-7],[-6,-2],[-6,4],[-3,8],[-3,21],[-3,9],[-5,12],[-1,3],[2,6],[-1,1],[-1,2],[-1,2],[1,5],[0,4],[-1,2],[-5,6],[-3,4],[-1,5],[3,3],[3,2],[0,4],[-2,5],[-3,4],[-5,4],[-1,3],[1,6],[1,5],[1,24],[0,5],[-1,4],[-1,1]],[[7363,6156],[-9,-19],[-13,-14],[-1,0],[-4,4],[2,2],[-4,8],[0,9],[2,10],[1,2],[6,-1],[6,-12],[3,2],[5,6],[3,4],[-2,5],[3,2],[2,-5],[0,-3]],[[7133,6434],[-4,-11],[-1,-4],[-7,-13],[-2,-5],[-1,-4],[0,-2],[0,-3],[2,-8],[1,-3],[0,-1],[2,-3],[1,-1],[1,-1],[1,0],[2,-1],[3,-1],[2,0],[1,0],[2,-1],[0,-1],[1,-1],[0,-2],[-1,-1],[-1,-1],[-1,0],[-7,-4],[-1,0],[-1,-2],[-1,-2],[0,-1],[0,-2],[1,-1],[0,-1],[1,-1],[1,-2],[0,-2],[0,-2],[0,-2],[-1,-1],[-1,-3],[0,-5],[-1,-17],[-1,-4],[-1,-2],[0,-2],[-1,-1],[-1,0],[-1,-1],[-3,0],[-1,0],[-1,0],[-14,8],[-13,4],[-5,3],[-2,2],[-1,1],[-1,0],[-2,0],[-2,0]],[[7459,6746],[-3,-11],[-3,-7],[-5,-8],[-2,-2],[-5,-3],[-4,-4],[-5,-7],[-3,-5],[-3,-3],[-3,-1],[-2,0],[-3,-1],[-3,-1],[-4,-3],[-2,-4],[0,-5],[2,-2],[2,-1],[3,-1],[3,-1],[2,-2],[1,0],[0,-1],[0,-1],[1,-1],[-1,-4],[0,-2],[-1,-1],[-2,-2],[-2,-1],[-2,0],[-2,1],[-4,2],[-2,0],[-2,-2],[-2,-9],[-2,-8],[-3,-6],[-1,-2],[-1,-1],[0,-3],[0,-1],[0,-2],[0,-1],[1,-2],[0,-1],[31,-42],[1,-1],[1,-1],[1,-1],[1,0],[1,1],[2,1],[4,1],[1,1],[1,1],[5,7],[2,1],[1,1],[1,0],[1,0],[1,-1],[1,-2],[0,-2],[0,-2],[-2,-9],[0,-3],[1,-3],[5,-12],[3,-4],[3,-2],[2,-1],[2,-1],[2,-4],[1,0],[4,1],[23,0],[10,3],[4,0],[2,-1],[1,-2],[0,-2],[0,-2],[1,-2],[3,-4],[1,-2],[0,-3],[0,-3],[-1,-5],[-1,-2],[0,-4],[0,-2],[0,-2],[0,-1],[-1,-2],[0,-3],[0,-3],[0,-3],[0,-3],[1,-1],[2,-5],[2,-7],[3,-4],[1,-2],[0,-1],[0,-2],[-1,-10],[0,-3],[0,-1],[1,-2],[1,0],[1,0],[3,1],[4,1],[2,0],[2,-1],[2,0],[2,-2],[1,-1],[0,-2],[0,-5],[0,-8],[0,-2],[0,-1],[-2,-5],[0,-2],[0,-2],[1,-2],[2,-4],[1,-2],[1,-1],[4,-3],[4,-4],[1,-2],[1,-1],[1,-1],[2,-5],[0,-1],[0,-2],[1,-2]],[[7560,6405],[-7,-3],[-3,-3],[-3,-3],[-4,0],[-6,-4],[-8,-1],[-7,-6],[-3,-1],[-5,0],[-3,-2],[-3,-2],[-1,-5],[-1,-3],[1,-5],[1,-7],[-6,0],[-5,0],[-6,-5],[-3,-5],[-4,-5],[-3,-3],[-2,-3],[-1,-1],[-2,-4],[-4,-20],[-2,-6],[-3,-6],[-2,1],[-4,-1],[-4,-3],[-2,-4],[0,-5],[4,4],[2,1],[-1,-2],[-1,-2],[0,-2],[-1,-3],[2,3],[4,8],[1,1],[2,-1],[-3,-11],[-1,-11],[2,-1],[4,-9],[2,-6],[-2,-6],[-3,-7],[-3,-5],[-5,-2],[-4,2],[-3,-3],[-9,-16],[-6,-9],[-5,-7],[-4,-5],[-4,-6],[-3,-3],[-9,-11],[-2,-2],[-3,-1],[-2,-1],[0,2],[6,6],[1,2],[1,2],[1,3],[1,2],[2,2],[-4,-1],[-7,-4],[-4,-1],[1,3],[-2,4],[0,4],[-2,-2],[-3,-8],[-4,-3],[-4,-3],[-3,-1],[-8,-1],[-6,2],[-7,2],[-4,0],[-5,-3],[-5,-1],[-6,1],[-1,-3],[2,-4],[-3,-2],[-4,-4],[-3,-1],[-10,-9],[-5,-3],[-3,-2],[-2,-3],[-3,0],[-5,2],[-5,1],[-5,2],[-5,7],[-3,9],[-3,6],[-5,10],[-7,8],[-8,4],[-3,1],[-3,-1]],[[7259,6202],[-1,6],[1,1],[-1,1],[1,16],[0,4],[0,3],[0,2],[-1,2],[-2,3],[-1,2],[-2,1],[-12,7],[-1,1],[-1,1],[-1,1],[-1,2],[0,2],[0,2],[0,2],[0,2],[3,12],[1,2],[2,7],[0,2],[0,1],[0,2],[-1,2],[-2,3],[-2,1],[-1,1],[-4,-1],[-3,-1],[-1,0],[-2,0],[-1,1],[-1,0],[-4,3],[-1,0],[-1,0],[-2,0],[-1,0],[0,-1],[-1,-1],[0,-1],[0,-2],[0,-1],[0,-2],[-1,-1],[0,-1],[-1,-1],[-1,-1],[-1,-1],[-1,1],[-1,1],[-2,3],[0,1],[-1,1],[-1,1],[-2,1],[-6,3],[-1,0],[-1,2],[0,1],[0,3],[1,2],[1,1],[0,2],[1,1],[0,2],[0,2],[-1,4],[-1,2],[0,2],[-1,1],[-5,2],[-1,1],[-1,2],[0,1],[-1,8],[-1,2],[-1,1],[-1,1],[-2,2],[-1,1],[0,5],[-1,9],[-1,7],[-2,7],[-1,3],[0,1],[0,3],[2,13],[0,2],[0,3],[-1,4],[-1,4],[0,1],[0,1],[0,1],[1,2],[1,1],[1,0],[2,0],[1,-1],[13,-9],[0,-1],[3,0],[2,0],[8,2],[2,0],[9,-2],[1,1],[1,0],[1,1],[0,1],[1,2],[0,1],[-1,1],[-1,3],[-1,2],[-1,1],[-3,2],[-1,2],[0,1],[1,2],[1,4],[0,1],[-1,2],[-2,1],[-3,1],[-6,1],[-5,2],[-2,4],[-2,7],[-3,4],[-8,8],[-17,15],[-16,8]],[[7060,6304],[-1,-8],[0,-5],[1,-16],[0,-3],[1,-3],[3,-9],[1,-3],[0,-2],[1,-2],[0,-4],[0,-3],[1,-4],[3,-3],[0,-1],[0,-3],[-1,-8],[-3,-19],[-1,-3],[0,-3],[1,-5],[1,-1],[0,-1],[1,-1],[0,-1],[1,-1],[1,-1],[1,-1],[3,-1],[2,-1],[1,-1],[1,-1],[0,-1],[1,-2],[0,-1],[0,-1],[0,-2],[0,-2],[-1,-2],[0,-1],[-1,-1],[-1,-1],[-29,-24],[-2,-3],[0,-1],[-1,-1],[-1,-3],[0,-1],[-1,-7],[0,-10],[0,-5],[1,-6],[2,-9],[2,-5],[2,-5],[2,-4],[3,-3],[3,-3],[5,-2],[4,-2],[2,-1],[2,-2],[1,-2],[2,-6],[1,-1],[0,-1],[2,-1],[1,-1],[3,-1],[2,0],[1,1],[14,4],[2,0],[1,0],[1,-1],[1,-2],[1,-1],[2,-15],[1,-3],[3,-3],[16,-2],[4,-2],[0,-1],[1,-3],[0,-1],[1,-1],[2,-2],[1,0],[0,-1],[2,-1],[2,0],[23,8],[5,3],[17,5]],[[7183,6044],[3,-2],[3,-7],[14,-37],[8,-18],[14,-22],[6,-6]],[[7231,5952],[-2,-2],[-3,-2],[-2,0],[-1,-1],[-1,-2],[-1,-3],[-1,-2],[0,-2],[-1,-4],[0,-4],[1,-1],[0,-1],[1,-2],[0,-1],[0,-2],[0,-2],[-2,-3],[-1,-2],[-1,-2],[-6,-8],[-1,-2],[-1,-2],[-1,-2],[-2,-1],[-11,-4],[-5,0],[-3,-2],[-6,-9],[-2,-4],[-1,-4],[-1,-4],[-1,-6],[-1,-2],[-1,-2],[-4,-7],[-1,-1],[0,-1],[-1,-2],[-2,-6],[-1,-3],[-2,-3],[-2,0],[-1,-1],[-3,0],[-1,0],[-2,0],[-1,1],[-7,1],[-2,0],[-12,-2],[-5,0],[-5,0],[-9,0],[-2,0],[-5,0],[-9,-2],[-4,-1],[-3,2],[-3,0],[-4,-1],[-9,1],[-6,-1],[-3,0],[-2,1],[-1,1],[-2,-1],[-2,-1],[-3,-3],[-1,-2],[-1,-2],[-1,-2],[-3,-12],[0,-1],[-1,-2],[-10,-9],[-2,-1],[0,-2],[-1,-1],[-1,-6],[-4,-9],[-3,-4],[-2,-3],[-4,-4],[0,-1],[-1,-1],[-2,-4],[-1,-2],[-4,-3],[-3,-6],[-2,-1],[-3,-3],[-1,-1],[-1,-2],[-2,-6],[-1,-1],[-1,0],[-1,1],[0,1],[-2,3],[-1,2],[-1,3],[-1,2],[-1,2],[0,7],[0,7],[-1,3],[-3,7],[-4,-5],[-1,-2],[-2,-7],[-1,-1],[-1,-1],[-2,-2],[-1,-1],[0,-1],[-1,-4],[-1,-4],[0,-1],[-6,-9],[-1,-2],[-1,-2],[0,-4],[0,-1],[1,-2],[0,-1],[1,-1],[1,-1],[1,-2],[1,0],[1,-2],[0,-2],[0,-2],[0,-2],[0,-1],[-1,0],[-4,-6],[-2,-1],[-1,-1],[-1,0],[-1,0],[-1,0],[-1,0],[-5,4],[-2,0],[-1,0],[-1,0],[-2,0],[0,-1],[-1,0],[-4,-2],[-6,0],[-1,-1],[-1,0],[-6,-6],[-3,-1],[0,-1],[-1,-1],[0,-5],[0,-8],[0,-2],[-1,-2],[-2,-4],[-1,-2],[-1,-2],[-6,-5],[-1,-1],[0,-2],[-1,-6],[-2,-6],[-1,-4],[-1,-1],[0,-1],[0,-5],[0,-3],[-2,-8],[-1,-3],[-3,-3],[-1,-2],[-6,-10],[-1,0],[0,-1],[-1,-1],[0,-2],[0,-2],[0,-4],[0,-2],[-1,-2],[-4,-8],[-8,-10],[0,-1],[-1,-1],[0,-1],[-1,-2],[0,-3],[-1,-1],[-2,-8],[-7,-5],[-13,-7],[-7,-2],[-4,-2],[-3,-8],[-3,-5],[-3,-5],[-2,-2],[-2,-2],[-5,-12],[-3,-4],[-1,-3],[0,-1],[1,-3],[-1,-2],[-2,-3],[-1,-2],[-1,-5],[0,-3],[-2,-2],[-3,-5],[-1,-3],[-1,-2],[0,-2],[1,-1],[0,-1],[1,-1],[1,-2],[0,-2],[1,-5],[5,-14],[2,-6],[1,-5],[0,-4],[0,-3],[1,-1],[1,-1],[1,0],[1,0],[2,1],[1,-1],[1,0],[1,-1],[1,-1],[0,-1],[1,-1],[0,-2],[1,-2],[2,-2],[1,-1],[1,-1],[0,-1],[1,-2],[0,-3],[0,-1],[1,-18]],[[6842,5393],[-10,-9],[-3,-4],[-2,-3],[-2,-3],[-1,-2],[-1,0],[-2,-1],[-1,0],[-2,1],[-1,0],[-1,1],[-1,1],[-1,3],[-1,2],[-2,7],[-4,7],[-1,1],[-1,2],[-1,1],[-4,2],[-2,0],[-2,0],[-4,-1],[-2,0],[-1,0],[-1,1],[-2,1],[-1,0],[-1,0],[0,-1],[-3,-3],[-4,-5],[-1,-1],[0,-2],[-2,-6],[0,-1],[-2,-4],[0,-1],[-3,-3],[-1,-2],[-2,-2],[-3,-2],[-4,0],[-3,-1],[-2,0],[0,1],[-1,1],[1,1],[0,1],[1,2],[0,2],[-2,3],[-4,5],[-5,6],[-3,2],[-2,1],[-1,-1],[-2,-2],[-2,0],[-3,0],[-9,3],[-2,0],[-1,0],[-6,-2],[-6,-3],[-2,-1],[-6,0],[-4,1],[-2,2],[-21,19],[-2,1],[-2,0],[-3,1],[-2,0],[-1,2],[-1,2],[-1,2],[-2,0],[-3,-1],[-2,0],[-7,3],[-2,1],[-1,-1],[-3,-2],[-1,-1],[-1,0],[-3,0],[-6,2],[-11,5],[-2,2],[-2,1],[-1,1],[0,1],[-1,4],[-1,3],[-1,3],[-2,3],[-2,3],[-2,2],[-2,2],[-2,0],[-2,-2],[-1,-1],[-1,-1],[-3,1],[-1,0],[-2,-1],[-2,0],[-1,0],[-1,1],[-2,3],[-1,1],[-1,0],[-2,0],[-5,-5],[-3,-1],[-3,0],[-10,4],[-1,0]],[[6629,6605],[-4,9],[-3,3],[-9,7],[-1,2],[-1,2],[0,1],[0,9],[0,1],[0,2],[-3,9],[-1,2],[0,1],[-1,1],[-1,1],[-3,2],[-1,1],[-3,3],[-2,4],[-6,13],[-1,2],[-1,1],[-8,8],[-5,6],[-1,2],[1,1],[0,1],[2,-1],[3,-1],[1,0],[1,0],[1,0],[1,1],[0,2],[0,5],[0,1],[2,6],[0,2],[1,1],[1,1],[1,0],[1,1],[1,0],[1,-1],[3,-2],[1,0],[2,0],[1,1],[1,0],[0,1],[1,1],[0,1],[1,2],[0,2],[-1,11],[0,4],[-1,2],[-1,2],[-5,0],[-2,1],[-1,2],[-2,3],[0,2],[0,2],[0,11],[0,2],[2,9],[0,2],[0,5],[0,3],[1,1],[0,2],[1,0],[3,4],[1,2],[1,1],[1,2],[3,11],[1,2],[2,3],[1,2],[2,1],[18,7],[5,3],[1,1],[2,2],[1,1],[0,1],[1,2],[2,45],[2,12],[1,4],[4,9],[3,4],[8,10],[4,3],[2,2],[4,1],[4,3],[2,3],[3,4],[1,2],[1,5],[4,17],[1,3],[7,11],[2,2],[1,1],[1,1],[8,3],[25,-1],[2,1],[2,2],[2,2],[3,5],[7,-8],[1,-2],[0,-3],[0,-3],[-1,-9],[0,-2],[0,-2],[5,-10],[1,-3],[2,-11],[1,-2],[3,-9],[0,-2],[1,-2],[-1,-2],[0,-3],[0,-3],[2,-11],[0,-3],[-1,-1],[-1,-1],[-1,-1],[-5,-2],[0,-1],[-1,-1],[-1,-1],[0,-3],[0,-2],[0,-1],[2,-8],[1,-2],[0,-2],[0,-2],[-1,-2],[0,-1],[-1,-6],[1,-14],[0,-2],[-1,-1],[0,-2],[0,-1],[0,-2],[2,-1],[22,1],[2,0],[10,1],[6,-2],[15,-6],[2,0],[2,1],[1,2],[0,3],[2,2],[2,2],[3,-1],[4,-1],[3,0],[1,2],[-1,3],[-4,10],[-1,4],[0,3],[2,3],[2,2],[31,14],[8,5],[17,15],[19,11],[8,8],[2,2],[4,2],[3,0],[4,0],[2,-2],[2,-3],[4,-9],[7,-12],[4,-10],[3,0],[2,0],[2,3],[3,5],[1,7],[2,3],[2,2],[8,0],[4,2],[12,18],[13,14],[3,2],[3,0],[2,-1],[1,-1],[2,-1],[1,-1],[0,-2],[2,-8],[0,-1],[1,-1],[1,-1],[2,0],[2,1],[6,4],[3,0],[3,-1],[1,-3],[1,-4],[2,-2],[3,-1],[4,0],[3,1],[5,3],[12,9],[3,2],[3,0],[3,2],[1,1],[1,2],[0,1],[1,1],[2,1],[5,2],[1,2],[1,1],[3,7],[0,2],[0,2],[-1,1],[-3,6],[-1,2],[-1,1],[0,2],[0,2],[1,4],[1,2],[2,4],[1,1],[1,2],[0,3],[-1,5],[-1,4],[-1,3],[0,1],[-5,10],[-2,5],[0,5],[0,2],[0,2],[1,4],[6,11],[2,4],[28,18],[8,3],[5,-1],[3,0],[15,3],[2,-1],[10,-4],[5,-2],[7,-1],[2,1],[1,0],[1,1],[1,1],[0,1],[1,2],[1,6],[1,9],[1,3],[0,1],[1,2],[2,2],[1,1],[4,3],[0,1],[1,1],[1,3],[1,3],[1,1],[1,2],[1,1],[1,-1],[1,-1],[0,-3],[1,-2],[7,-12],[1,0],[1,1],[4,3],[18,7],[8,4],[2,0],[1,0],[0,-2],[-1,-5],[0,-3],[1,-3],[1,-2],[5,-4],[2,-3],[3,-7],[11,-16],[6,-7],[5,-4],[1,-2],[1,-2],[0,-2],[0,-3],[0,-2],[-1,-3],[0,-2],[1,-3],[1,-3],[1,-4],[1,-2],[0,-3],[-1,-3],[-1,-2],[-4,-2],[-1,0],[-1,-2],[-1,-3],[1,-3],[2,-3],[3,-3],[10,-5],[2,-1],[1,-2],[0,-2],[-1,-6],[0,-4],[1,-4],[1,-3],[2,-7],[1,-2],[0,-2],[0,-2],[0,-3],[0,-2],[1,-2],[2,-1],[3,1],[3,1],[1,0],[1,1],[1,1],[1,2],[0,2],[1,6],[0,1],[2,2],[1,0],[4,0],[2,-2],[2,-3],[0,-3],[0,-3],[0,-5],[0,-3],[0,-1],[0,-1],[2,-3],[1,-2],[0,-3],[0,-3],[-3,-10],[-1,-2],[0,-1],[1,-2],[4,-4],[2,-3],[1,-4],[0,-4],[0,-5],[-3,-12],[-2,-6],[-2,-2],[-3,0],[-2,2],[-2,4],[-2,3],[-2,1],[-2,2],[-3,-1],[-2,-1],[-2,-6],[-1,-5],[0,-5],[1,-3],[3,-7],[2,-4],[2,-2],[1,-3],[2,-12],[2,-4],[2,-2],[3,0],[4,-1],[3,-1],[1,-2],[1,-2],[0,-3],[-1,-4],[-2,-11],[0,-2],[0,-1],[0,-8],[6,-3],[1,-4],[0,-4],[0,-9],[0,-5],[2,-7],[1,-4],[2,-2],[1,-1],[3,0],[2,-1],[0,6],[1,1],[1,2],[3,2],[7,2],[24,-1],[10,-3],[3,0],[3,1],[8,7],[4,2],[2,1],[1,0],[1,-1],[1,-1],[0,-2],[0,-3],[2,-3],[4,-2],[47,-7],[4,1],[6,2],[2,0],[1,0],[1,-1],[0,-1],[0,-2],[-1,-5],[0,-4]],[[6842,5393],[0,-3],[0,-1],[1,-9],[-1,-11],[-1,-3],[-1,-4],[-4,-13],[-1,-2],[-1,-2],[-2,-2],[-8,-5],[0,-1],[-1,-1],[-1,-1],[0,-1],[0,-1],[-3,-31],[0,-6],[0,-2],[0,-1],[1,-1],[0,-1],[1,0],[2,0],[2,2],[4,3],[4,5],[2,3],[2,5],[0,1],[1,3],[1,8],[1,2],[0,2],[2,1],[1,2],[3,0],[1,0],[4,-1],[3,0],[22,9],[12,6],[5,5],[3,3],[5,7],[1,2],[1,0],[1,1],[3,0],[10,-3],[1,0],[6,3],[19,18],[1,-1],[1,-5],[0,-3],[0,-3],[-2,-2],[-2,-2],[-4,-18],[-1,-2],[-7,4],[-4,1],[-2,-2],[-2,0],[-4,-3],[-9,-11],[-1,-2],[0,-4],[1,-1],[2,-1],[1,-2],[1,-3],[-2,-3],[-4,-2],[-15,-2],[-5,-5],[-8,-17],[-4,-5],[-6,-4],[-7,-3],[-6,0],[-6,0],[0,-1],[0,-6],[-2,-4],[-4,-4],[-7,-6],[-3,-3],[-1,-4],[-1,-5],[0,-6],[-1,-4],[-7,-1],[-2,-5],[0,-3],[2,-5],[0,-3],[0,-2],[-2,-2],[-2,0],[-2,-1],[-7,-10],[-2,-4],[-4,-2],[-3,0],[-7,1],[-2,-1],[-1,-1],[-1,-2],[-1,-2],[-2,-1],[-5,-1],[-4,-2],[-2,-4],[-5,-9],[0,-2],[-3,-8],[-1,-2],[-3,-5],[-7,-13],[-1,-2],[-3,-1],[-8,-5],[-2,-3],[0,-5],[1,-5],[1,-5],[-3,-10],[0,-12],[-2,-5],[0,-1],[8,-4],[3,-1],[5,1],[3,2],[2,1],[2,0],[3,0],[1,-1],[28,-17],[2,-1],[3,-5],[2,-4],[1,-1],[0,-1],[0,-2],[-1,-9],[0,-5],[3,-3],[0,2],[1,1],[1,1],[1,1],[1,1],[2,0],[2,0],[1,0],[3,0],[1,-1],[1,0],[4,-5],[1,-2],[10,-6],[2,-2],[1,-2],[0,-2],[0,-1],[0,-12],[0,-7],[1,-6],[0,-1],[1,-3],[1,-2],[6,-10],[3,-5],[3,-3],[2,-1],[1,-1],[13,-2],[5,0],[2,1],[2,1],[1,2],[1,4],[1,1],[2,-1],[7,-2],[5,-2],[4,-3],[1,0],[1,0],[6,3],[5,1],[12,1],[2,1],[1,1],[0,1],[2,3],[1,1],[0,1],[2,0],[1,-1],[4,-4],[2,-1],[2,1],[1,0],[1,1],[1,1],[0,1],[1,1],[2,-1],[1,-2],[2,-3],[1,-4],[1,-1],[1,-1],[3,-1]],[[7885,6110],[-2,-2],[-1,1],[0,2],[0,1],[0,1],[0,1],[0,1],[1,1],[1,0],[0,-3],[1,-3]],[[8035,6147],[-2,0],[-5,3],[-1,1],[1,2],[-1,2],[1,2],[1,0],[1,-2],[4,-4],[1,-2],[0,-2]],[[8107,6162],[0,-4],[-2,-2],[-2,-3],[-2,-1],[-2,2],[-1,2],[1,3],[1,2],[2,-1],[0,2],[-1,1],[-2,1],[-1,0],[0,2],[4,1],[1,1],[1,1],[1,-2],[2,-5]],[[7976,6188],[1,-1],[-2,-2],[-2,2],[-4,-2],[-3,-2],[-2,5],[1,4],[4,4],[5,4],[3,1],[-2,-3],[0,-2],[0,-2],[1,-6]],[[7995,6207],[2,-1],[2,0],[1,0],[0,-1],[0,-1],[-1,0],[-3,-1],[-1,0],[-1,0],[-1,-1],[-2,3],[0,1],[2,1],[2,0]],[[8032,6207],[0,-4],[-2,-3],[-6,-4],[0,2],[0,1],[1,2],[0,2],[-2,1],[-8,1],[0,1],[4,0],[10,2],[3,-1]],[[8023,6214],[-7,-2],[-9,1],[-9,3],[-4,7],[5,0],[2,0],[6,-3],[9,-2],[4,-2],[3,-2]],[[8090,6262],[0,-1],[-1,1],[-1,-3],[0,-2],[-2,1],[-1,1],[-2,3],[0,1],[0,1],[1,1],[1,-1],[2,-1],[3,-1]],[[7805,6272],[6,-2],[4,1],[2,-1],[3,-3],[1,-3],[-1,-3],[-4,0],[0,-2],[1,-1],[1,-1],[1,-2],[0,-2],[0,-6],[-2,-4],[-3,-1],[-9,1],[-2,1],[0,3],[3,5],[2,3],[2,2],[0,2],[-1,2],[-2,1],[-2,0],[-2,-1],[-2,-3],[-2,-6],[-3,-4],[-4,-2],[-1,1],[-1,1],[0,2],[0,4],[0,4],[1,1],[5,3],[1,0],[2,1],[1,2],[1,1],[0,2],[1,2],[1,2],[2,0]],[[8080,6279],[-4,-3],[-5,1],[-2,4],[0,7],[4,2],[5,-2],[3,-3],[-1,-6]],[[7714,6535],[-1,-1],[-2,1],[0,2],[1,2],[1,0],[1,2],[-1,1],[1,1],[1,0],[1,1],[0,1],[0,1],[2,-2],[1,-1],[0,-4],[-1,-2],[-3,0],[-1,-1],[0,-1]],[[8517,6636],[-4,-3],[-6,-10],[5,-7],[1,-4],[-2,-1],[-10,-1],[0,1],[-1,2],[-3,3],[-2,1],[-2,-2],[-1,-5],[0,-6],[-1,-3],[-2,-1],[-7,0],[-2,-1],[-2,-1],[-1,-3],[-1,-2],[0,-4],[0,-2],[-2,-1],[-3,1],[-1,-2],[-1,-3],[0,-1],[1,-3],[-1,-2],[-4,0],[-1,1],[-2,2],[-1,1],[-1,0],[-3,-1],[-3,1],[-2,1],[-2,0],[-3,-2],[-1,-3],[-1,-2],[-2,-2],[-2,-1],[-4,-3],[-6,-2],[-4,-2],[-4,-3],[-3,-3],[-4,-2],[-2,-2],[-4,-2],[-4,-2],[-2,-3],[0,-4],[2,-3],[2,-3],[1,-3],[-1,-3],[-2,-1],[-3,0],[-4,0],[-4,-1],[-4,1],[-2,0],[-1,1],[0,1],[-2,3],[-2,2],[0,-2],[-1,-3],[-6,-9],[-8,-10],[-3,-3],[-4,-1],[-3,-2],[-1,-5],[-2,-7],[-1,-4],[-4,-2],[-8,-2],[-2,-2],[-3,-4],[-7,-5],[-2,-3],[-1,-2],[0,-3],[0,-1],[-1,0],[-2,-1],[-1,0],[-23,-28],[-2,-3],[-2,-2],[-1,-4],[1,-8],[1,-6],[-1,-3],[-1,-3],[-8,-10],[0,-2],[-1,-4],[-1,-2],[-1,-1],[-1,0],[-1,0],[-1,0],[-11,-13],[-8,-5],[-2,-5],[-4,-2],[-4,5],[-4,-4],[-2,-2],[-2,-1],[-7,-3],[-2,-1],[-2,1],[-1,2],[-2,3],[-2,2],[-2,-1],[-2,-2],[-2,-1],[-3,-1],[-5,1],[-4,1],[-3,2],[-1,0],[-6,-5],[-9,3],[-8,7],[-4,6],[2,11],[-3,-2],[-1,-2],[-1,-4],[-1,-5],[-3,-14],[-1,-7],[-1,-2],[-4,-4],[-2,2],[0,1],[-1,1],[-1,2],[-1,1],[-3,1],[-3,-2],[-3,-3],[-6,-12],[-2,-2],[-3,2],[-1,3],[1,13],[0,2],[1,1],[0,2],[0,3],[-1,0],[-1,-1],[-1,-1],[1,-2],[-3,-6],[-4,-1],[-4,0],[-4,-2],[-2,2],[-3,5],[-2,2],[-1,-4],[-1,1],[-1,3],[0,2],[-1,-1],[-1,-1],[-1,-2],[1,-3],[2,-2],[2,-4],[0,-4],[-3,-2],[-4,2],[-2,0],[-1,-2],[0,-3],[1,-1],[2,0],[3,-2],[1,0],[1,0],[2,-2],[1,-2],[-1,-2],[-7,-6],[-8,-3],[-7,-1],[-4,7],[-1,-1],[-7,-3],[-3,0],[-2,0],[-2,1],[-1,0],[-1,-2],[-2,-1],[-2,0],[0,-2],[3,-3],[-1,-3],[-4,-7],[-3,2],[-12,-3],[-4,3],[-10,-12],[-1,-3],[-2,-1],[-10,-6],[-2,-3],[-1,-1],[-2,-1],[-1,1],[-2,2],[-2,1],[-1,-1],[-1,-5],[-8,-11],[-2,2],[-1,2],[-1,1],[-1,-1],[0,-1],[0,-1],[1,-2],[0,-2],[-1,-1],[-1,0],[-2,0],[-7,-8],[-4,-2],[-4,-1],[-5,0],[-3,-1],[-2,-4],[-5,-2],[-1,-1],[-2,-3],[-3,-3],[-2,-2],[-2,-1],[-1,-2],[-2,-4],[-1,-2],[-3,1],[0,-1],[0,-1],[-1,-1],[-1,-7],[-6,-9],[-14,-13],[0,-4],[-1,-3],[-1,-3],[0,-1],[5,-1],[3,-1],[2,-2],[-4,0],[-2,-1],[-9,-10],[-1,-1],[-2,0],[-1,0],[-1,-3],[2,-6],[-2,-2],[-9,1],[-1,-1],[-2,-2],[-1,-3],[-3,-1],[-2,1],[-2,2],[-2,1],[-3,-2],[2,-2],[2,-1],[1,-3],[1,-3],[-8,5],[-3,1],[-5,1],[2,-6],[7,-4],[2,-5],[0,-2],[-1,-3],[-3,-3],[-2,-1],[-2,1],[-6,6],[-4,9],[-2,4],[-5,3],[-3,-1],[-1,-2],[3,-3],[0,-2],[-4,1],[-5,1],[-2,-1],[-5,-3],[2,0],[1,-1],[0,-2],[0,-2],[-1,-3],[0,-1],[-3,-2],[-4,0],[-2,-1],[-1,-2],[1,-2],[4,-2],[6,0],[3,-1],[3,-3],[2,-5],[0,-5],[-1,-3],[-4,2],[-3,-3],[-12,1],[-5,-1],[-7,-8],[-1,0],[-1,-4],[-2,-1],[-17,2],[-6,-1],[-9,-7],[-5,2],[-4,1],[-3,-6],[2,0],[3,0],[-8,-13],[-3,-1],[-4,1],[-2,4],[-2,5],[-3,2],[2,3],[2,1],[0,2],[0,5],[-1,3],[-2,2],[-1,3],[0,5],[5,4],[2,3],[-1,4],[-10,0],[0,1],[1,4],[4,6],[2,2],[8,1],[2,0],[2,-3],[1,0],[2,0],[2,1],[4,1],[5,5],[5,0],[2,0],[1,3],[0,2],[-2,4],[0,3],[1,3],[3,4],[4,1],[3,0],[2,-1],[0,-2],[2,-4],[2,-1],[2,0],[4,0],[3,2],[10,7],[14,6],[7,5],[2,8],[-1,5],[-5,5],[-1,5],[-2,-1],[-4,2],[-2,3],[2,3],[0,1],[-1,3],[0,1],[4,1],[1,3],[-5,4],[-2,3],[4,3],[4,0],[1,-1],[0,-1],[2,0],[4,2],[6,6],[3,4],[0,2],[-1,3],[4,6],[2,3],[2,2],[2,0],[8,-2],[4,1],[8,7],[5,1],[-2,5],[-2,-1],[-1,-2],[-2,-2],[-3,0],[-6,-3],[-3,-1],[-13,4],[-7,-1],[2,-6],[-28,-3],[-3,1],[-6,6],[-5,23],[-4,9],[-3,2],[-3,1],[-7,1],[-3,-1],[-3,-4],[-2,-1],[-1,1],[-3,2],[-2,1],[-6,0],[-1,1],[-1,2],[1,2],[2,2],[1,0],[2,6],[5,6],[3,3],[3,1],[9,0],[11,3],[4,5],[-1,3],[5,2],[5,-9],[1,1],[0,1],[1,2],[0,1],[0,1],[-2,2],[0,1],[0,1],[2,2],[0,1],[0,1],[1,1],[1,1],[-1,2],[-1,2],[-2,-1],[-1,-1],[-1,0],[-1,2],[-3,7],[-4,5],[-1,1],[-1,1],[3,2],[2,3],[0,3],[-1,3],[-1,4],[0,2],[6,3],[4,10],[5,0],[8,3],[4,7],[8,6],[3,2],[5,-1],[3,-2],[3,1],[4,5],[1,2],[0,3],[1,2],[4,1],[3,4],[4,2],[1,2],[0,2],[-1,2],[-2,1],[-4,0],[-2,1],[9,5],[8,9],[0,7],[10,6],[2,8],[-2,8],[4,1],[3,2],[4,3],[5,5],[2,6],[1,3],[0,4],[0,2],[0,2],[0,5],[3,4],[9,7],[5,8],[-2,12],[7,0],[1,1],[0,2],[0,2],[0,2],[3,2],[2,1],[1,2],[0,4],[3,4],[-2,5],[-3,6],[-8,9],[-8,2],[-4,5],[-2,5],[5,8],[0,9],[-2,4],[-3,-5],[-3,-1],[-2,5],[-7,9],[-3,3],[-25,17],[-4,2],[-3,4],[-1,7],[-1,15],[-1,1],[-1,4],[0,4],[2,2],[1,1],[2,2],[3,6],[-1,1],[-7,-5],[-3,-4],[-5,-7],[0,-7],[-2,-5],[-1,-4],[-3,-6],[-8,-1],[-16,0],[-3,-1],[-1,1],[-7,5],[-3,4],[-5,1],[-5,0],[-1,2],[-1,2],[0,2],[-1,2],[-2,1],[-4,2],[-1,1],[-4,3],[-4,-3],[-4,-5],[-2,-3],[-3,0],[-2,1],[-1,2],[-1,3],[-2,-4],[-4,5],[-4,1],[-4,-2],[-3,-4],[-2,-2],[-2,-2],[-2,-2],[-1,-2],[-1,-1],[-4,-2],[0,-1],[0,-1],[0,-2],[0,-1],[0,-5],[-3,-2],[1,-4],[-3,-2],[-1,4],[-2,4],[-4,1],[-3,0],[-1,-1],[-2,-3],[-1,-3],[0,-3],[1,-3],[-2,-2],[-2,-2],[-2,-3],[1,-4],[3,-1],[3,1],[3,-1],[2,-2],[2,-3],[-4,-1],[-5,-1],[-4,-5],[-3,-1],[-4,-1],[-2,-1],[-3,0],[-3,1],[-4,-3],[-1,-4],[0,-5],[-2,-5],[-1,-5],[-2,-2],[-3,-2],[-11,-13],[-4,-4],[-1,-5],[-1,-3],[-1,-2],[-4,-2],[-2,0],[-3,-3],[-1,-4],[0,-5],[0,-2],[-2,-3],[-4,0],[-3,-3],[-1,-5],[2,-1],[3,1],[3,1],[1,-5],[-4,-3],[-5,-3],[-6,-5],[0,-5],[0,-4],[1,-5],[0,-6],[-2,-5],[-4,-4],[-3,-4],[-2,-2],[-7,0],[-6,0],[-4,-2],[-4,-1],[-3,-5],[-5,-4],[-2,-1],[-4,-2],[-5,-2],[-1,-1],[-1,0],[-4,-4],[-6,-3],[-28,-11],[-6,-5],[0,-7],[0,-4],[-6,0],[-4,-3]],[[7459,6746],[21,27],[2,3],[1,6],[1,3],[0,3],[1,7],[0,4],[-2,12],[-4,24],[-1,2],[-1,3],[-5,8],[0,1],[0,1],[-1,7],[0,3],[0,3],[0,2],[-1,4],[0,1],[0,1],[0,2],[0,1],[0,1],[0,1],[0,2],[0,1],[0,2],[0,1],[0,1],[0,1],[1,3],[0,2],[0,4],[0,2],[0,1],[3,4],[1,1],[0,2],[0,2],[-1,6],[0,11],[0,1],[0,1],[-1,2],[0,1],[0,2],[1,1],[0,2],[1,1],[2,3],[3,6],[1,3],[1,3],[0,3],[0,4],[0,4],[-1,3],[0,2],[0,1],[-1,1],[-1,1],[-1,0],[-5,3],[-1,0],[0,1],[-8,13],[-2,1],[-1,1],[0,1],[0,2],[0,1],[0,2],[1,1],[2,5],[1,1],[0,1],[2,3],[3,4],[3,3],[3,5],[1,1],[1,1],[1,0],[3,0],[1,0],[1,0],[2,1],[1,0],[1,2],[1,0],[1,1],[0,1],[1,0],[3,0],[3,2],[1,2],[3,5],[2,8],[1,2],[0,2],[2,0],[2,-2],[5,-4],[3,-5],[1,-3],[0,-2],[0,-3],[0,-1],[0,-1],[0,-2],[5,-5],[0,-1],[1,-2],[0,-1],[0,-2],[1,-5],[0,-1],[1,-1],[1,-2],[3,-1],[3,-1],[13,-1],[10,-4],[1,-1],[2,-1],[1,-3],[0,-2],[0,-2],[-2,-3],[-5,-4],[0,-1],[-1,-1],[0,-1],[1,-2],[0,-1],[1,-1],[11,-13],[1,-2],[0,-1],[0,-2],[0,-9],[0,-1],[1,-2],[0,-1],[0,-1],[1,0],[11,-7],[1,-2],[1,-1],[0,-2],[1,-1],[0,-3],[0,-4],[0,-1],[0,-2],[0,-1],[1,-2],[5,-7],[2,-3],[0,-1],[1,-1],[0,-1],[0,-5],[0,-7],[-1,-19],[0,-2],[0,-1],[0,-2],[1,-1],[0,-1],[1,-1],[1,-1],[3,0],[2,0],[3,0],[1,1],[1,0],[1,2],[0,1],[1,3],[1,8],[1,3],[0,3],[1,2],[6,8],[14,16],[5,9],[2,5],[1,2],[3,6],[2,1],[15,10],[3,3],[1,3],[2,2],[0,1],[3,11],[1,1],[0,2],[0,2],[-1,1],[0,3],[0,1],[0,2],[0,2],[0,1],[1,1],[0,1],[1,1],[1,0],[20,4],[2,1],[2,2],[8,8],[4,5],[1,1],[17,10],[12,9],[4,0],[0,-2],[3,-4],[2,-3],[1,-1],[4,-3],[3,0],[3,3],[55,63],[2,2],[3,1],[4,1],[3,2],[2,1],[2,0],[1,-1],[2,0],[2,0],[8,4],[1,0],[1,-1],[1,0],[1,-1],[1,-1],[1,-1],[0,-1],[1,-4],[3,-8],[2,-2],[2,-2],[1,0],[0,-1],[1,0],[5,1],[3,0],[5,0],[1,1],[1,1],[2,4],[2,2],[19,14],[1,1],[1,1],[1,3],[3,12],[0,3],[0,1],[1,7],[0,2],[1,3],[1,2],[3,3],[3,3],[13,7],[1,0],[1,0],[10,-7],[2,-1],[7,-1],[1,0],[1,-1],[11,-10],[2,-1],[2,-1],[2,0],[1,0],[16,5],[3,2],[2,3],[0,1],[1,1],[0,2],[0,1],[0,2],[0,1],[0,2],[-1,2],[-7,13],[-1,3],[-2,5],[0,2],[-1,2],[0,1],[0,2],[1,2],[0,1],[2,3],[1,0],[1,1],[14,-1],[8,-4],[1,-1],[7,-6],[25,-18],[3,-1],[3,-1],[18,2],[3,1],[1,1],[4,5],[1,1],[5,3],[1,1],[1,1],[9,-1],[3,0],[2,1],[2,0],[0,1],[10,9],[1,2],[4,8],[3,21],[2,3],[11,11],[4,3],[3,2],[7,-1],[2,0],[42,18],[1,2],[1,1],[0,7],[1,5],[2,4],[0,2],[2,4],[1,1],[0,1],[0,1],[0,1],[-1,1],[0,1],[0,2],[-1,1],[1,1],[1,1],[0,2],[1,1],[0,2],[0,1],[0,1],[0,2],[0,2],[0,1],[1,3],[1,2],[0,1],[0,1],[0,1],[3,7],[0,1],[0,1],[0,2],[0,1],[-1,1],[1,1],[-1,2]],[[8188,7296],[-1,3],[0,1],[2,11],[1,2],[0,1],[3,4],[4,10],[5,-2],[3,-1],[1,-2],[8,-10],[12,-12],[13,-16],[2,-2],[1,0],[3,0],[6,0],[1,0],[2,-1],[1,0],[2,-2],[1,-1],[4,-6],[2,-2],[2,-1],[1,0],[3,1],[2,0],[1,0],[1,-1],[3,-3],[5,-2],[1,-1],[1,-2],[1,-2],[1,-3],[0,-4],[0,-2],[0,-3],[0,-2],[0,-2],[2,-2],[1,-1],[14,-10],[3,-3],[0,-2],[1,-2],[-1,-3],[-1,-2],[-1,-2],[-5,-8],[0,-1],[0,-2],[0,-3],[1,-4],[0,-2],[1,-2],[2,-1],[6,-4],[1,-1],[-1,-1],[-2,-3],[-1,-1],[-3,-2],[-1,-1],[-1,-1],[-1,-3],[0,-1],[1,-4],[6,-1],[3,0],[1,0],[1,-1],[3,-5],[2,-2],[2,2],[4,4],[18,22],[6,10],[1,1],[0,1],[0,1],[1,3],[0,3],[1,3],[1,3],[1,2],[11,13],[3,2],[3,0],[12,1],[0,-2],[2,-2],[0,-2],[0,-2],[-1,-9],[-2,-10],[0,-4],[1,-7],[0,-4],[-2,-19],[0,-2],[0,-7],[1,-5],[0,-2],[1,-2],[0,-1],[1,-1],[1,-1],[1,0],[2,-1],[6,0],[2,-1],[1,-2],[1,-3],[1,-3],[0,-4],[1,-2],[1,-2],[0,-2],[0,-2],[-2,-7],[0,-1],[-1,-1],[-1,-2],[-1,-2],[0,-1],[-1,-3],[1,-1],[1,-1],[1,0],[1,0],[3,1],[1,0],[1,0],[1,0],[1,-1],[1,-2],[3,-7],[0,-2],[2,-1],[6,-5],[1,-2],[0,-1],[0,-2],[0,-1],[0,-2],[0,-2],[0,-3],[1,-12],[14,-18],[4,-5],[0,-2],[0,-2],[0,-1],[-1,-2],[0,-1],[-1,-1],[-3,-2],[-1,-1],[0,-2],[0,-3],[0,-1],[1,-2],[2,-1],[1,-1],[4,-2],[8,-2],[1,-1],[1,-1],[0,-1],[1,-3],[0,-2],[0,-2],[-1,-7],[0,-1],[0,-2],[0,-1],[1,-1],[1,-2],[1,-1],[1,-3],[0,-1],[1,-7],[1,-1],[1,-1],[1,0],[4,1],[0,1],[2,1],[1,1],[3,-1],[11,-4],[2,-1],[1,-1],[0,-1],[0,-1],[0,-2],[-2,-5],[-2,-4],[0,-1],[-1,-1],[-2,-2],[-1,-2],[-2,-6],[0,-1],[-1,-1],[-1,-1],[-1,-1],[-3,-3],[-1,-1],[0,-1],[-1,-2],[0,-1],[0,-3],[-1,-1],[0,-1],[0,-1],[-1,-2],[-2,-1],[-1,-1],[-7,-5],[-1,-1],[0,-2],[2,-5],[0,-1],[0,-2],[0,-2],[-3,-15],[0,-5],[0,-2],[1,-3],[6,-15],[0,-3],[0,-2],[-1,-6],[0,-1],[-2,-6],[0,-1],[0,-1],[0,-1],[6,-3],[5,0],[2,0],[7,4],[1,0],[1,0],[1,0],[0,-1],[0,-1],[1,-4],[0,-11],[1,-8],[1,-3],[16,-44],[1,-1],[4,-6],[6,-11],[1,-2],[1,-3],[0,-2],[0,-2],[0,-3],[1,-3],[1,-1],[3,-2],[3,-5],[1,-1],[2,0],[2,-1],[2,-1],[1,-1],[1,-1],[0,-2],[0,-3],[0,-2],[0,-2],[-1,-3],[0,-2],[0,-2],[1,-1],[3,-6],[0,-1],[0,-2],[-6,-5],[-1,-1],[0,-2],[0,-4],[0,-3],[0,-2],[-2,-6],[-2,-8],[0,-2],[-1,-1],[0,-1],[-2,-2],[-1,-1],[-3,-3],[-1,-1],[-1,-1],[-1,-2],[-1,-3],[-2,-2],[-2,-3],[-1,-1],[-1,-2],[-1,0],[-2,-4],[1,-2],[1,-1],[2,0],[10,0],[2,-1],[2,-1],[1,-2],[1,-3],[1,-3],[0,-1]],[[7612,5283],[-2,0],[-1,3],[1,3],[1,3],[1,2],[1,-1],[0,-2],[1,-4],[0,-1],[0,-2],[-2,-1]],[[7942,5750],[1,0],[0,-1],[2,-2],[-5,0],[-1,0],[-1,1],[1,1],[1,1],[1,1],[0,-1],[1,0]],[[7708,5855],[0,-2],[-2,0],[-1,2],[-2,1],[-1,2],[1,8],[1,0],[3,-2],[1,-5],[0,-4]],[[7686,5866],[-1,0],[-1,2],[-1,4],[0,3],[2,0],[2,-2],[0,-4],[-1,-3]],[[7703,5872],[-1,-2],[-2,1],[-1,1],[-1,1],[-1,2],[0,1],[0,1],[2,0],[3,-2],[1,-3]],[[7711,5922],[-2,0],[-1,0],[-2,1],[-1,3],[1,3],[3,-4],[2,-1],[0,-2]],[[7467,5106],[0,4],[-1,1],[-1,1],[-1,2],[-2,1],[-1,1],[-1,0],[-2,0],[-6,-1],[-3,-1],[-12,-12],[-4,-3],[-20,1],[-9,-2],[-3,-1],[-2,-1],[-1,-1],[0,-1],[-1,-1],[0,-1],[0,-2],[0,-8],[-2,-13],[0,-2],[-1,-2],[-6,-12],[-4,-11],[0,-1],[0,-2],[0,-5],[0,-1],[0,-2],[-2,-6],[0,-2],[0,-1],[0,-1],[0,-1],[-1,0],[-2,-5],[-3,-6],[-2,-3],[-2,-2],[-1,-1],[-2,0],[-2,-1],[-1,1],[-2,0],[-9,6],[-3,0],[-3,0],[-5,-1],[-9,-4],[-1,-1],[-1,-1],[0,-1],[-1,-2],[0,-1],[-1,-1],[0,-2],[1,-3],[0,-1],[-1,-2],[0,-1],[-3,-5],[0,-2],[-1,-3],[-1,-3],[0,-20],[-1,-6],[-1,-3],[-3,-7],[-5,-5],[-3,-2],[-5,1],[-10,-2],[-8,-4],[-4,0],[-2,0],[0,2],[0,1],[1,6],[0,1],[0,2],[0,1],[-1,1],[0,1],[-3,3],[-2,5],[0,3],[0,1],[1,2],[2,3],[0,4],[0,2],[0,1],[-1,1],[-1,1],[-4,1],[-1,1],[-1,1],[-1,2],[-1,1],[0,2],[0,10],[-1,3],[-1,2],[-1,1],[-1,1],[-2,1],[-9,1],[-9,5],[-2,1],[-2,0],[-1,0],[-4,-2],[-6,-2],[-7,0],[-3,-3],[-1,-3],[1,-8],[0,-2],[-2,-15],[-1,-3],[0,-1],[0,-1],[-1,-1],[-1,0],[-7,3],[-4,0],[-3,0],[-2,-1],[-1,-1],[-11,-15],[-1,-1],[-1,-1],[-2,0],[-3,0],[-12,3],[-5,5],[-11,19],[-5,6],[-1,0],[-1,0],[-1,-1],[-6,-4],[-1,0],[-1,-1],[-3,-4],[-2,-4],[-1,-1],[0,-2],[0,-1],[0,-12],[0,-2],[-1,-1],[-1,-2],[-1,-1],[-1,-1],[-1,-1],[-1,0],[-1,1],[-2,1],[-3,3],[-2,3],[-1,4],[0,3],[-1,8],[0,2],[0,1],[0,1],[1,1],[-1,2],[0,2],[-13,38],[-3,6],[-4,7],[-3,3],[-1,1],[-2,4],[-1,5],[-1,8],[0,3],[0,1],[-2,3],[-1,2],[-2,2],[-18,21],[-2,1],[-1,-1],[0,-1],[-1,-2],[-1,-1],[-1,-1],[-3,-1],[-8,-2],[-13,3],[-2,0],[-10,-5],[-4,-1],[-5,-1],[-2,0],[-1,0],[-5,-4],[-2,-4],[-6,-13],[-1,-20],[-1,-6],[-1,-1],[-2,-3],[-1,-2],[0,-1],[0,-1],[0,-3],[1,-3],[0,-4],[-1,-4],[-1,-3]],[[7231,5952],[10,-7],[7,-3],[3,-6],[3,-5],[2,1],[3,0],[5,-4],[2,-2],[1,-4],[-1,-1],[2,-3],[3,-1],[2,1],[2,1],[2,1],[4,-2],[12,0],[15,-4],[19,-5],[4,-1],[4,-1],[3,-2],[3,-2],[4,-4],[1,-1],[1,0],[1,1],[1,1],[2,1],[2,2],[2,6],[3,4],[6,5],[18,3],[7,-1],[3,0],[3,0],[9,-8],[9,-6],[-4,-11],[2,-2],[3,-4],[2,-10],[4,-9],[9,-4],[7,-9],[4,-16],[6,-8],[9,-5],[6,-9],[2,-14],[0,-12],[-7,1],[-4,7],[-12,9],[-8,1],[-5,-5],[-2,-4],[0,-4],[-2,-10],[-3,-5],[-3,-6],[-1,-5],[0,-5],[-2,-11],[-2,-41],[1,-11],[1,-10],[2,-4],[3,-2],[4,-1],[3,-3],[2,-5],[2,-3],[10,-7],[5,-6],[23,-11],[25,-5],[5,4],[9,1],[2,0],[4,1],[16,-1],[9,1],[9,2],[1,5],[2,3],[14,14],[3,4],[-1,10],[-4,9],[-3,6],[2,5],[4,2],[8,4],[9,8],[2,2],[2,0],[8,5],[21,19],[18,22],[4,10],[-3,8],[-2,2],[-2,0],[-5,0],[-2,1],[-2,1],[-1,1],[0,1],[15,0],[9,2],[3,3],[1,0],[3,2],[6,7],[3,2],[15,4],[2,1],[4,4],[7,2],[6,3],[5,5],[6,4],[8,-1],[6,-3],[2,-1],[2,1],[3,2],[2,1],[3,-1],[5,-2],[2,-3],[-2,-5],[2,-4],[3,-3],[3,-3],[3,-1],[2,-1],[1,-2],[0,-3],[1,-2],[2,-2],[2,0],[2,1],[2,0],[7,-1],[4,-2],[2,-3],[-1,-4],[-2,-3],[0,-3],[3,-4],[-2,-3],[0,-4],[1,-4],[2,-4],[3,-1],[3,-1],[19,2],[6,3],[5,5],[-2,5],[-1,2],[3,-1],[3,-3],[3,-3],[3,-4],[-5,0],[-3,-3],[0,-5],[3,-4],[4,-4],[4,-4],[3,-6],[1,-6],[1,-2],[2,-1],[5,-1],[3,-1],[2,-2],[3,-5],[3,-1],[10,3],[4,1],[-3,4],[-4,2],[-7,-3],[1,3],[1,2],[2,2],[1,0],[3,1],[1,-1],[1,-1],[2,-1],[11,-3],[34,1],[3,-1],[0,-2],[-1,-1],[-1,-2],[0,-2],[1,-2],[1,-1],[5,1],[2,2],[-3,10],[0,2],[2,2],[4,8],[1,3],[3,2],[3,0],[2,2],[-1,5],[1,0],[2,-2],[2,-1],[2,0],[2,3],[3,-3],[2,-5],[0,-5],[-3,-3],[-1,-5],[2,-7],[4,-6],[3,-4],[3,4],[1,4],[3,2],[3,1],[3,-2],[3,-8],[3,-1],[26,0],[1,0],[0,1],[1,1],[1,0],[1,-1],[0,-2],[1,-1],[13,-2],[4,0],[1,3],[2,1],[9,-1],[2,0],[2,-1],[0,-6],[-2,-4],[-4,3],[-6,-4],[-5,-4],[-1,-1],[-2,-1],[4,-9],[0,-5],[-6,7],[-1,0],[0,-2],[2,-10],[1,-2],[4,-6],[0,-2],[1,-4],[1,-3],[1,-2],[3,-1],[0,-2],[-7,-1],[-2,-1],[-2,-3],[1,-2],[2,-2],[-1,-3],[-5,3],[-5,-1],[-5,-4],[-2,-8],[-1,0],[0,10],[0,1],[-2,1],[-1,-2],[0,-3],[-1,-5],[-1,-4],[1,-3],[0,-1],[1,-1],[1,-1],[1,-3],[-2,-3],[-6,-5],[-1,-4],[1,-4],[3,-1],[2,2],[0,5],[4,0],[8,-2],[4,2],[-2,-22],[1,-5],[-1,-3],[-1,-3],[-2,-2],[-1,2],[0,3],[1,3],[1,2],[-2,1],[-1,-2],[-3,-1],[-3,-2],[-3,0],[-1,0],[-2,1],[-1,0],[-1,0],[-1,-2],[0,-2],[1,-2],[1,-1],[1,0],[1,-2],[1,-3],[1,-2],[-1,-2],[-2,-1],[-3,-1],[-1,0],[-1,1],[0,1],[-1,0],[-1,0],[0,-1],[0,-2],[-1,-2],[-2,-3],[-1,-1],[-6,-2],[-19,6],[-3,2],[0,4],[0,5],[2,3],[12,0],[-2,2],[-7,4],[-6,6],[-2,1],[2,11],[1,5],[-3,2],[-2,-2],[-1,-6],[1,-6],[-1,-4],[-3,0],[-3,2],[-2,-1],[-1,-5],[-2,-4],[-2,-2],[-3,-2],[-2,-1],[0,5],[0,7],[0,6],[2,2],[1,1],[-1,1],[-1,1],[-2,1],[-4,0],[-7,1],[-6,1],[-2,-2],[2,-5],[4,-8],[-6,-5],[-21,-7],[-1,-2],[-4,-3],[0,-2],[-1,-2],[-1,-3],[-1,-2],[-1,-1],[-3,-1],[-7,-4],[-3,0],[0,1],[4,3],[2,1],[0,2],[-5,-1],[-3,-2],[-2,-5],[-1,-7],[2,-7],[0,-4],[-1,-1],[-2,0],[-2,2],[-1,2],[0,3],[-2,-1],[-2,0],[-2,1],[-1,-1],[-2,-2],[-2,-1],[-4,2],[-1,4],[1,3],[3,2],[3,0],[1,1],[1,1],[2,3],[1,2],[4,1],[2,2],[0,3],[-3,1],[-4,-2],[-3,-1],[-1,-2],[-2,-2],[-1,-2],[-2,-1],[-2,0],[0,2],[-1,2],[-1,1],[-4,0],[-1,-3],[1,-4],[1,-2],[1,-1],[-1,-4],[-2,-5],[-2,-1],[-2,0],[-3,0],[-4,-2],[-2,-4],[1,-4],[3,-1],[-2,-1],[-3,0],[-6,1],[-3,0],[-26,-11],[-4,-5],[-7,-9],[-1,-2],[-3,0],[-3,5],[-2,-1],[0,-2],[0,-5],[0,-2],[-1,-2],[-2,0],[-1,1],[-2,1],[-8,0],[-3,2],[1,4],[0,2],[-2,1],[-3,0],[-2,1],[-4,5],[-3,1],[-4,-2],[-9,-7],[-3,0],[-1,-2],[0,-2],[1,-1],[1,0],[-2,-6],[0,-5],[1,-4],[5,0],[2,3],[3,8],[2,4],[4,2],[3,-2],[5,-10],[1,-1],[2,0],[1,-1],[0,-4],[1,-1],[2,0],[2,2],[1,2],[1,0],[-1,-5],[-1,-3],[-2,-2],[-1,-4],[1,-2],[2,-8],[-3,0],[-2,0],[-3,-2],[-1,-3],[1,-3],[0,-2],[-1,-2],[-1,-4],[-1,-2],[-2,-3],[-1,0],[-3,2],[-1,10],[-3,2],[1,6],[-4,2],[-5,0],[-4,-1],[-1,-1],[-1,-3],[0,-1],[-1,-1],[-2,-1],[-1,-1],[-5,-11],[1,-4],[3,-6],[2,-5],[-2,-2],[-2,1],[-4,2],[-2,1],[-2,-2],[-1,-2],[1,-3],[2,-2],[-1,-4],[1,-3],[2,-3],[1,-4],[2,-17],[1,-3],[1,-5],[-1,-3],[-11,-7],[-1,0],[0,1],[-1,0],[-4,-3],[-4,1],[-3,0],[-1,-4],[-1,-2],[-4,-1],[-6,0],[-2,-1],[-5,-4],[-2,-2],[-8,-3],[-3,-2],[-2,2],[-3,2],[-2,1],[-3,0],[0,2],[3,3],[0,3],[0,3],[2,4],[2,5],[4,14],[-2,6],[-1,3],[-3,1],[-2,10],[-1,2],[-3,-2],[-2,-8],[0,-8],[4,-4],[-2,-4],[-4,2],[-3,2],[-2,0],[-1,1],[0,1],[-1,0],[0,6],[-5,0],[-7,-3],[-5,0],[0,2],[-1,2],[-2,1],[-1,-1],[0,-2],[-1,-2],[1,-1],[2,-1],[1,-3],[-1,-4],[-2,-7],[-1,-5],[1,-5],[2,-4],[2,-2],[3,-1],[3,-2],[3,-4],[0,-4],[0,-10],[1,-4],[2,-2],[2,0],[5,4],[0,-7],[2,0],[5,5],[1,1],[0,3],[1,1],[2,-2],[1,-3],[0,-2],[0,-1],[-1,-2],[-2,-3],[-7,-3],[-3,-3],[-6,-10],[-2,-3],[-1,4],[2,4],[1,3],[0,3],[-3,-2],[-1,-1],[-3,-5],[-1,-2],[-1,-1],[-4,0],[-2,0],[-7,-11],[-2,-5],[-4,-22],[-2,-8],[-6,1],[-3,6],[-2,2],[-2,-2],[-3,-3],[-1,-2],[5,-7],[1,-1],[-1,-2],[-4,-5],[-1,-2],[0,-3],[0,-2],[1,-2],[0,-3],[0,-3],[-2,-1],[-1,0],[-2,-2],[-3,-1],[-2,2],[-2,5],[1,6],[-5,-2],[-6,-15],[-5,-4],[1,6],[0,2],[-1,2],[-1,1],[-3,0],[-2,-1],[0,-2],[-1,-2],[-2,-2],[-3,-1],[-2,0],[-1,-1],[-1,-3],[-2,-7],[-4,-16],[-1,-5],[-1,-2],[-4,-7],[-1,-2],[0,-7],[-1,-3],[-1,-2],[-5,-8],[-1,-3],[-1,-2],[-6,-3],[-6,1],[-1,-1],[-1,-2],[0,-3],[0,-6],[-1,-2],[-2,-5],[-3,-12],[-3,-24],[-2,-5],[-3,-2],[-4,-1],[-5,-3]],[[7716,5956],[-1,0],[0,2],[0,1],[1,2],[2,2],[3,0],[0,-3],[-1,-1],[-1,0],[-1,0],[-1,-1],[-1,-2]],[[7735,5982],[0,-1],[-1,1],[0,-1],[-1,1],[0,2],[1,0],[1,-2]],[[7218,6126],[3,-1],[1,3],[5,-2],[-1,-4],[-3,-3],[-5,1],[-7,4],[-3,3],[3,9],[2,-4],[7,-2],[-2,-4]],[[7238,6130],[0,-4],[-8,2],[-6,3],[-6,3],[-4,1],[-2,3],[1,0],[4,-1],[9,-2],[2,-2],[10,-3]],[[7229,6140],[-1,-2],[-3,1],[-2,3],[-2,11],[-1,4],[4,1],[2,-6],[3,-12]],[[7259,6202],[-6,0],[-7,-1],[-7,-3],[-12,-14],[-9,-9],[-4,-2],[-3,1],[0,-5],[2,-1],[1,-3],[3,-3],[1,-6],[1,-5],[1,-7],[1,-5],[-6,1],[-2,0],[-4,3],[0,-2],[0,-1],[1,-1],[-21,-44],[-1,-5],[0,-5],[0,-16],[-3,-6],[0,-5],[0,-4],[-1,-4],[-1,-6]],[[7286,3607],[1,-2],[0,-5],[0,-2],[0,-2],[0,-1],[0,-2],[-1,-4],[-2,-3],[-4,-6],[-1,-2],[-1,-1],[-2,-1],[-7,-1],[-3,-3],[0,-9],[0,-7],[-1,-2],[-8,-17],[0,-1],[0,-2],[1,-2],[1,-3],[4,-6],[1,-2],[2,-2],[2,-7],[10,-7],[12,-14],[4,-3],[1,-1],[0,-2],[-1,-3],[0,-1],[1,-1],[4,-1],[0,-1],[2,-1],[1,-2],[2,-5],[1,-3],[0,-2],[0,-3],[1,-5],[1,-2],[1,-1],[2,-2],[4,-2],[4,-5],[4,-19],[2,-4],[1,-1],[1,-1],[0,-1],[1,-1],[0,-2],[0,-2],[0,-5],[-2,-10],[0,-3],[0,-2],[2,-11],[1,-2],[0,-1],[4,-6],[0,-1],[0,-2],[-1,-2],[0,-1],[-2,-1],[0,-1],[0,-1],[1,-2],[2,-2],[1,-1],[0,-1],[1,-24],[1,-7],[0,-6],[0,-1],[-1,-2],[-4,-8]],[[7687,4302],[0,-1],[-4,0],[-4,0],[-2,3],[0,2],[0,2],[2,1],[3,0],[5,-4],[0,-3]],[[7568,4314],[0,-2],[-2,0],[-5,7],[-2,2],[-5,2],[-2,1],[-1,0],[-1,1],[-1,2],[1,1],[0,1],[0,1],[0,6],[1,4],[2,3],[2,3],[-5,2],[-7,5],[-5,6],[0,6],[6,5],[5,0],[5,-5],[3,-6],[7,-25],[1,-8],[3,-12]],[[7501,5021],[-1,-1],[-3,0],[-2,4],[-2,1],[0,1],[1,1],[1,-1],[2,-1],[1,-1],[3,0],[1,-1],[-1,-2]],[[7803,4158],[-2,0],[-3,-1],[-3,-1],[-5,1],[-2,-1],[-1,-1],[-1,0],[-3,-3],[-1,-1],[-3,-1],[-1,-1],[-1,-2],[-1,-2],[-4,-11],[-1,-3],[-1,-4],[1,-9],[0,-3],[0,-1],[0,-2],[-1,-3],[0,-2],[-1,-1],[0,-3],[0,-1],[-1,-1],[-1,0],[-1,0],[-1,1],[-1,0],[-1,0],[-1,0],[-4,-4],[-1,-1],[0,-2],[0,-1],[2,-19],[0,-2],[0,-2],[0,-1],[-1,-1],[-2,-2],[-3,-1],[-12,-2],[-12,1],[-2,0],[0,-1],[-1,0],[-1,-2],[0,-1],[1,-5],[4,-7],[1,-5],[2,-13]],[[7733,4031],[-3,-4],[-1,0],[-1,-1],[-1,1],[-5,1],[-1,0],[-9,-1],[-1,-1],[-5,-3],[-2,-1],[-1,0],[-5,0],[-1,0],[-1,-1],[0,-1],[-1,-3],[1,-1],[1,-1],[0,-1],[1,-1],[0,-1],[1,-1],[1,-6],[0,-6],[0,-1],[0,-1],[-2,-1],[-9,-5],[-1,-1],[-2,0],[-3,1],[-2,-1],[-2,-3],[-7,-12],[-7,-10],[-3,11],[0,1],[-5,22],[0,2],[-1,1],[-1,0],[-2,0],[-1,-2],[-1,-1],[-1,0],[-2,-1],[-1,0],[-1,1],[-1,2],[0,2],[-1,2],[0,5],[0,1],[-1,1],[-1,0],[-8,-3],[-15,-2],[-14,6],[-12,13],[-5,8],[-10,12],[-3,5],[-1,3],[0,4],[0,7],[-1,3],[-4,4],[-2,0],[-2,1],[-5,0],[-3,-1],[-3,-1],[-2,-1],[-2,0],[-2,2],[-1,1],[-1,2],[-2,0],[-2,0],[-3,-2],[-4,-1],[-4,0],[-2,-1],[-1,-1],[-4,-7]],[[7467,5106],[-5,-2],[-4,0],[-1,-1],[-1,-1],[-1,-3],[-1,-6],[-1,-2],[-1,-5],[0,-7],[1,-7],[1,-4],[-2,-20],[0,-12],[2,-7],[-2,-5],[1,-1],[-1,-3],[0,-4],[0,-4],[2,-2],[1,2],[2,8],[1,3],[2,0],[8,-2],[4,0],[2,2],[2,2],[1,3],[2,-1],[3,-4],[2,-2],[2,-1],[5,0],[2,-1],[4,-4],[1,-4],[0,-5],[-3,-3],[13,-18],[9,-9],[2,-4],[1,-2],[6,-3],[0,-1],[0,-1],[0,-1],[1,-1],[1,-1],[5,1],[3,-1],[4,-2],[4,-2],[7,-9],[5,-3],[6,-1],[7,0],[5,-2],[7,-5],[7,-5],[9,-12],[7,-4],[19,-10],[3,-3],[1,-5],[1,-7],[1,-6],[4,-10],[2,-8],[1,-6],[1,-4],[0,-7],[4,-9],[1,-4],[1,-2],[1,-2],[1,-3],[2,-17],[4,-14],[1,-6],[0,-2],[-3,-2],[1,-3],[1,-2],[2,-2],[2,-2],[2,-6],[1,-6],[0,-4],[2,-3],[3,-3],[3,-3],[1,-4],[0,-11],[1,-10],[-2,1],[-1,0],[-1,-1],[-1,-2],[7,-10],[2,-5],[4,-12],[2,-4],[9,-14],[1,-3],[3,-11],[0,-4],[1,-20],[1,-3],[2,-4],[3,-4],[2,-2],[2,-1],[3,-1],[2,-3],[-1,-3],[0,-1],[-1,-1],[-1,0],[0,-2],[0,-1],[2,-3],[0,-1],[0,-4],[-1,-1],[-1,0],[9,-10],[3,-8],[-2,-7],[3,-1],[1,-4],[0,-10],[1,-6],[2,-1],[2,1],[3,1],[4,-2],[1,-3],[-2,-10],[1,-4],[3,-31],[-1,-9],[-4,-10],[-4,-5],[1,-9],[0,-8],[-2,2],[-1,2],[0,2],[1,1],[-1,2],[-2,-4],[0,-3],[5,-7],[7,-3],[1,-8],[6,-6],[5,-3],[7,-7],[4,-3],[4,-4],[3,-2],[2,-4],[0,-6],[1,0],[2,3],[6,-3],[3,-3],[3,-2],[4,-1],[9,-7],[14,-9],[5,-8],[2,-8],[3,-22],[0,-4],[0,-1],[0,-1],[-3,-1],[-4,-1],[-1,0],[-1,-2],[1,-1],[5,-16],[4,-1],[1,-3],[0,-5],[2,-5],[1,-1],[1,1],[1,1],[1,1],[1,-1],[1,-2],[1,-1],[2,1],[5,-1],[4,-1],[7,-4],[10,-7],[4,-5],[6,-5],[2,-2],[6,-6],[5,-6],[4,-5],[1,-7],[3,-6],[5,-11],[0,-10],[4,-10],[3,-9],[2,-7],[-3,-5],[-7,-4],[-9,0],[-11,5],[-7,3],[-5,0],[-3,3],[-8,4],[-5,4],[-4,3],[-4,1],[-2,1],[-1,3],[-1,2],[-1,1],[-3,1],[-10,8],[-16,12],[-5,2],[-10,-7],[-7,-8],[-11,-10],[-5,1],[-4,3],[-4,2],[-5,0],[-5,-2],[-4,3],[-2,5],[0,7],[0,1],[1,3],[-1,2],[-2,1],[-1,1],[-2,5],[-1,2],[-9,17],[-6,8],[-11,4],[-18,11],[-11,0],[-7,1],[-6,0],[-10,-8],[-11,-7],[-10,-14],[-5,-6],[-7,-3],[-8,0],[-7,1],[-7,4],[-6,4],[-5,5],[-5,7],[-12,24],[-1,4],[-1,4],[1,5],[1,10],[1,6],[-4,-5],[-2,4],[-2,14],[-2,4],[-4,5],[-3,3],[-4,2],[-2,0],[-4,2],[-3,0],[-8,-2],[-3,2],[-2,5],[-3,3],[-5,-1],[6,-5],[2,-2],[3,-6],[1,-3],[-1,-2],[-1,-2],[-3,-11],[-1,-4],[-2,-3],[-3,-1],[-2,-2],[3,-2],[3,1],[3,3],[2,4],[3,4],[1,-1],[2,-3],[3,-3],[4,-1],[4,0],[2,-2],[-4,-10],[0,-4],[1,-5],[2,-3],[12,-8],[2,-2],[6,-9],[5,-7],[4,-7],[3,-6],[4,-5],[6,-2],[15,-8],[7,-2],[9,1],[18,13],[13,3],[2,2],[4,2],[-1,5],[1,5],[2,1],[8,-2],[14,-7],[11,2],[3,0],[10,-5],[5,-17],[4,-11],[3,-12],[-3,-5],[-9,4],[-2,3],[-2,4],[-2,3],[1,-8],[2,-5],[3,-3],[13,-6],[11,-4],[8,-2],[6,-1],[5,-3],[9,-5],[12,-10],[11,-15],[6,-7],[12,-26],[3,-3]],[[7887,4106],[-3,-1],[-2,3],[-1,5],[-3,12],[1,2],[6,0],[2,0],[3,-2],[1,-3],[1,-5],[-1,-5],[-2,-4],[-2,-2]],[[7875,4114],[-2,-1],[-2,2],[-6,8],[-19,18],[-3,4],[1,1],[2,-1],[3,0],[12,-6],[5,-5],[4,-3],[2,-4],[2,-4],[2,-5],[-1,-4]],[[7803,4158],[4,-4],[9,-6],[5,-7],[16,-16],[14,-8],[6,-6],[9,-9],[9,-17],[16,-31],[12,-31],[4,-18],[-2,-10],[-9,-5],[-16,-1],[-12,-2],[-17,-2],[-15,-6],[-8,-5],[-6,-3],[-7,-5],[-10,-14],[-3,-3],[-9,-3]],[[7793,3946],[-2,9],[-1,5],[-3,5],[-1,1],[-1,2],[-3,2],[-1,1],[-1,0],[-1,1],[-1,0],[-2,-1],[-2,0],[-1,0],[-2,2],[-1,1],[-1,1],[-1,2],[-3,7],[-2,2],[-1,1],[-1,-1],[-1,0],[-1,-1],[-1,-1],[0,-1],[-1,-3],[-1,-1],[0,-1],[-1,-1],[-1,0],[-1,0],[-6,4],[-2,1],[0,1],[0,2],[0,1],[2,4],[0,1],[0,2],[0,1],[-2,4],[-1,1],[0,2],[1,1],[0,1],[1,3],[1,1],[0,1],[0,2],[-2,8],[0,1],[0,2],[0,3],[0,2],[0,2],[-1,2],[-1,0],[-3,1],[-2,0],[-5,0]],[[7897,4149],[-10,-5],[-6,2],[-5,5],[-9,5],[-3,3],[-4,3],[-12,4],[-7,2],[-5,6],[-4,4],[-5,3],[-13,9],[-5,6],[-5,2],[-5,3],[-2,5],[-3,8],[-10,9],[-3,5],[-3,5],[-1,2],[0,4],[3,2],[4,6],[7,3],[9,-3],[7,-3],[4,-1],[6,-5],[8,-6],[5,-6],[5,-5],[6,-4],[6,-6],[3,-4],[3,-3],[3,-3],[10,-6],[20,-6],[12,-8],[10,-10],[1,-10],[-1,-7],[-11,-5]],[[7765,3091],[0,-2],[-2,0],[-3,1],[-1,0],[0,-2],[1,-2],[-1,-1],[-2,2],[-1,3],[-1,3],[1,2],[4,2],[1,-2],[2,-2],[2,-2]],[[7778,3193],[-1,-1],[-1,1],[0,-1],[0,-4],[0,-2],[-2,0],[-2,2],[-2,0],[-1,-1],[-2,-1],[-3,0],[-1,3],[3,4],[3,1],[3,-1],[1,1],[0,1],[0,1],[1,1],[1,0],[2,1],[2,0],[0,-2],[0,-2],[-1,-1]],[[7757,3198],[1,0],[1,1],[2,0],[-1,-1],[-1,-2],[0,-1],[0,-1],[-1,-1],[-1,1],[0,2],[-1,0],[-1,0],[-2,-2],[-1,-1],[-1,-2],[-1,1],[0,1],[0,2],[1,1],[0,2],[0,1],[2,0],[0,1],[1,1],[1,0],[1,-2],[1,-1]],[[7766,3230],[3,-2],[2,-3],[-1,-2],[-2,-2],[-3,-1],[-2,0],[-2,2],[-2,1],[-3,-2],[-1,1],[-1,1],[1,3],[3,3],[1,1],[1,-1],[1,0],[5,1]],[[7794,3261],[-1,-4],[-2,0],[-1,0],[-2,0],[-1,2],[-1,-2],[0,-1],[0,-1],[1,-2],[-1,-2],[-2,1],[-1,-2],[-2,-1],[-3,-3],[-3,0],[-3,-1],[0,5],[1,3],[0,2],[-2,1],[-1,3],[-1,3],[0,2],[0,4],[-2,2],[1,1],[2,1],[5,2],[3,5],[3,5],[1,3],[0,3],[3,2],[5,-1],[2,-5],[0,-4],[-1,-3],[-2,-8],[2,-5],[3,-5]],[[7849,3368],[-1,-1],[-1,0],[-1,1],[0,1],[-1,1],[1,1],[-1,2],[1,3],[1,2],[1,0],[1,-1],[0,-3],[1,-1],[0,-2],[-1,-1],[0,-1],[0,-1]],[[7889,3532],[0,-3],[-1,-2],[-1,-2],[-2,-1],[0,1],[-2,2],[-2,2],[-2,1],[-2,0],[-1,0],[0,2],[0,2],[0,3],[1,1],[1,0],[8,-2],[2,-2],[1,-1],[0,-1]],[[7904,3526],[-3,-11],[-2,0],[-2,3],[-4,0],[2,6],[-2,4],[-3,4],[0,6],[2,3],[3,2],[4,0],[3,-2],[1,-2],[1,-2],[0,-7],[0,-3],[0,-1]],[[7954,3682],[2,0],[0,-2],[0,-1],[-4,3],[-2,-1],[-5,5],[0,1],[0,1],[0,1],[2,0],[1,-1],[1,-2],[3,-3],[2,-1]],[[7957,3697],[1,0],[0,1],[1,0],[1,-1],[0,-1],[0,-1],[1,0],[0,-1],[0,-1],[-2,0],[-1,0],[-1,-1],[-2,2],[-2,0],[-2,2],[-2,3],[-2,3],[0,1],[-1,1],[2,1],[1,1],[0,1],[1,0],[2,-2],[0,-2],[-1,-2],[1,-1],[0,-1],[5,-2]],[[7935,3693],[5,-10],[2,-6],[-4,-5],[1,5],[-1,0],[-2,-1],[0,3],[-2,-2],[-1,-1],[-1,0],[0,1],[-1,3],[0,2],[-1,2],[-1,1],[-1,2],[0,2],[-2,2],[-3,-1],[-3,-1],[0,-1],[-1,0],[-1,0],[-1,1],[0,1],[0,2],[0,3],[1,2],[1,2],[1,3],[2,3],[3,1],[2,1],[2,1],[1,-1],[1,-2],[0,-2],[0,-4],[0,-2],[1,-1],[2,-3],[1,0]],[[7912,3704],[-2,-2],[-3,1],[-1,1],[0,1],[0,4],[1,3],[7,5],[1,0],[1,-2],[0,-3],[-4,-8]],[[7963,3706],[-1,-2],[-3,1],[-3,3],[-1,2],[-3,2],[-2,5],[-1,1],[0,2],[3,3],[4,0],[2,-1],[4,-4],[0,-1],[-1,-2],[1,-6],[1,-3]],[[7961,3732],[2,0],[3,1],[1,-1],[0,-1],[-1,-1],[0,-1],[0,-1],[-1,-1],[-4,-2],[-2,1],[-1,1],[0,2],[0,2],[1,1],[1,1],[1,-1]],[[7910,3739],[-3,-2],[-5,1],[-2,5],[2,4],[1,3],[2,1],[1,0],[0,-2],[1,0],[2,-4],[1,-3],[0,-3]],[[7968,3750],[2,-1],[5,1],[2,1],[1,0],[0,-2],[0,-2],[0,-2],[-1,-1],[-1,-1],[2,-3],[0,-3],[-2,-4],[-1,-2],[-1,-2],[0,-1],[0,-3],[2,-2],[0,-1],[0,-3],[-2,1],[-1,1],[-2,3],[-1,3],[0,2],[1,2],[1,2],[-1,3],[-1,2],[-1,2],[-3,2],[-2,2],[0,3],[0,3],[0,1],[1,1],[1,1],[1,1],[1,-4]],[[7920,3753],[0,-2],[-3,-5],[-1,2],[-1,3],[1,2],[1,0],[0,1],[1,0],[2,-1]],[[7941,3755],[-1,-2],[-5,3],[-2,1],[2,1],[3,3],[2,-3],[1,-3]],[[7977,3771],[0,-2],[0,-1],[0,-1],[2,-2],[-1,-1],[-1,0],[-1,0],[-1,-2],[0,-3],[0,-2],[-1,0],[-2,1],[0,2],[1,1],[0,1],[-1,2],[1,4],[0,2],[1,0],[3,2],[0,-1]],[[7893,3759],[-3,-2],[-2,1],[-2,1],[-2,1],[-1,3],[0,1],[0,2],[1,1],[0,1],[0,2],[0,4],[0,1],[1,3],[1,7],[1,1],[2,0],[2,-1],[1,-1],[0,-4],[2,-2],[1,0],[1,-3],[1,-5],[-2,-6],[-2,-5]],[[7901,3783],[-1,-1],[-1,1],[0,3],[-1,0],[-1,1],[-1,1],[0,1],[1,2],[3,3],[2,0],[1,-1],[1,-1],[0,-1],[-1,-1],[0,-3],[-2,-4]],[[8038,3797],[-2,0],[-1,4],[2,0],[2,-3],[-1,-1]],[[7940,3790],[1,-1],[2,1],[2,0],[4,-2],[1,-2],[1,-1],[1,-2],[1,-2],[2,0],[4,0],[1,-1],[0,-4],[1,-2],[3,-2],[1,-2],[0,-4],[-1,-1],[-2,-1],[-1,-1],[0,-3],[-1,-3],[0,-6],[-1,-5],[-2,1],[-6,8],[-2,3],[-8,4],[-6,1],[-8,4],[-3,1],[-2,0],[-3,-3],[-2,-1],[-1,-1],[-1,0],[-1,0],[-1,0],[0,1],[0,2],[0,1],[0,3],[0,2],[-3,1],[-3,5],[0,1],[0,2],[0,2],[0,1],[1,2],[0,1],[0,5],[-2,6],[-1,3],[2,2],[18,-1],[3,-1],[2,-2],[3,-3],[1,-1],[4,0],[1,-1],[1,-4]],[[7942,3814],[0,-1],[2,1],[1,0],[0,-1],[-1,-1],[0,-2],[-1,0],[-1,-1],[-1,0],[0,-1],[0,-2],[-3,-5],[-1,1],[-2,3],[0,1],[0,7],[1,1],[3,3],[1,0],[0,-1],[1,-1],[1,0],[0,-1]],[[7920,3816],[0,-2],[0,-1],[0,-1],[-3,-3],[-2,-2],[0,1],[0,1],[1,0],[0,1],[-2,2],[-1,0],[0,1],[3,3],[1,-1],[0,1],[1,1],[1,0],[1,-1]],[[8024,3816],[-1,-5],[0,3],[-1,1],[-1,2],[0,2],[2,-2],[1,-1]],[[8017,3815],[0,-2],[-2,3],[0,2],[0,1],[1,1],[1,-1],[0,-4]],[[7977,3836],[1,-2],[0,-3],[0,-3],[-2,-1],[-2,3],[0,3],[-1,0],[-1,0],[-2,-4],[-2,-2],[-3,0],[-2,0],[-2,0],[-2,-1],[-2,0],[0,1],[1,2],[3,1],[1,2],[1,2],[1,1],[1,1],[2,0],[7,1],[3,-1]],[[7908,3852],[-1,-1],[0,-1],[0,-1],[-1,-1],[0,-3],[0,-1],[-1,-1],[-1,-1],[0,-1],[-1,4],[1,3],[1,1],[0,2],[1,1],[1,0],[0,2],[1,1],[1,-2],[-1,-1]],[[7946,3829],[-2,-2],[-2,1],[-2,1],[-6,2],[-4,2],[-7,7],[2,3],[1,1],[1,2],[1,0],[3,2],[4,0],[1,1],[2,2],[5,2],[3,2],[2,-4],[-1,-1],[0,-2],[1,-1],[2,-1],[-5,-6],[0,-1],[3,-2],[1,0],[-2,-2],[0,-2],[0,-2],[-1,-2]],[[7959,3888],[1,0],[2,0],[4,-3],[2,0],[4,5],[-1,-4],[1,-2],[2,0],[2,0],[2,-1],[1,-3],[1,-7],[-3,1],[-6,4],[-3,1],[-7,-4],[-3,0],[-2,4],[1,0],[1,1],[-1,1],[-1,2],[0,2],[0,3],[0,2],[1,1],[1,0],[1,-3]],[[7955,3900],[-1,-1],[-2,1],[1,3],[1,1],[1,-2],[0,-1],[0,-1]],[[7923,3921],[2,0],[1,0],[-1,-1],[-1,-1],[0,-1],[-1,-2],[-1,3],[-1,0],[-1,0],[-1,1],[3,0],[1,1]],[[8005,3940],[0,-1],[1,1],[0,-1],[-1,-1],[-1,0],[0,-1],[-1,0],[1,-1],[0,-1],[0,-1],[-1,2],[-1,0],[-1,0],[-1,1],[1,1],[-1,1],[1,1],[0,1],[0,1],[1,1],[0,1],[0,-1],[1,-1],[1,-1],[1,-1]],[[8041,3955],[-1,-1],[0,1],[-1,-2],[0,-1],[0,-2],[0,-1],[0,-1],[-1,0],[0,1],[-1,0],[-1,2],[-1,-1],[-1,1],[0,2],[1,0],[0,1],[0,1],[1,0],[1,0],[1,2],[1,0],[1,0],[0,-1],[1,-1]],[[7988,3953],[2,-1],[2,1],[1,1],[1,0],[3,1],[1,-1],[-1,-2],[1,-2],[-1,-1],[-5,0],[-1,0],[-2,0],[-2,-1],[-1,-1],[-2,1],[-2,0],[-1,2],[1,2],[0,4],[-1,3],[0,1],[2,-1],[3,-2],[2,-4]],[[8022,3991],[1,-2],[1,0],[1,0],[0,-1],[-1,-2],[-1,0],[-1,1],[-2,1],[0,1],[1,0],[0,2],[1,0]],[[7793,3946],[-2,0],[-7,-4],[-3,-3],[-4,-5],[-3,-2],[-2,-3],[-1,-2],[-1,-3],[-1,-2],[-1,-2],[-4,-1],[-9,-1],[-4,-4],[-4,-3],[-3,-4],[-5,-10],[-2,-9],[-1,-5],[1,-3],[1,-3],[0,-2],[-2,-4],[-1,0],[-3,-1],[-1,0],[1,-4],[1,-1],[1,-1],[0,-2],[-3,0],[-2,-1],[-6,-10],[-3,-7],[-3,-2],[-4,-1],[-3,3],[-2,6],[-2,8],[-3,5],[-5,5],[-5,2],[-8,1],[-22,-1],[-8,-4],[-3,-5],[-1,-4],[-2,-9],[-1,-8],[-2,-4],[-2,-2],[-4,2],[-4,4],[-5,2],[-5,0],[-3,-2],[-7,-15],[-8,-9],[6,0],[3,5],[5,5],[3,9],[4,2],[4,0],[3,-4],[2,-4],[4,-2],[2,0],[3,2],[2,4],[1,4],[3,7],[5,17],[4,1],[8,1],[16,-1],[5,-4],[1,-6],[4,-17],[2,-5],[5,-2],[1,-3],[-7,-9],[-4,-9],[-4,-5],[-2,-3],[1,-3],[3,-6],[5,-4],[3,-1],[4,-1],[5,2],[1,0],[1,-1],[2,-3],[2,-1],[-2,6],[-3,2],[-8,-1],[-4,1],[-4,3],[-3,5],[3,4],[6,11],[8,11],[10,-3],[3,-8],[4,-3],[6,-1],[5,5],[5,1],[1,1],[6,4],[19,19],[3,5],[3,7],[6,3],[11,3],[10,-3],[10,-4],[9,-8],[13,-19],[7,-16],[6,-13],[6,-14],[3,-6],[2,-5],[4,-4],[4,-5],[4,-1],[-1,-5],[1,0],[3,3],[3,0],[5,-2],[3,-3],[6,-3],[3,-3],[5,-2],[2,-1],[4,-8],[2,-1],[2,1],[3,0],[12,0],[2,0],[3,3],[3,2],[4,-1],[1,-5],[-3,-4],[-8,-5],[-6,-4],[-8,-3],[-2,-4],[-5,-5],[-2,-5],[-4,-5],[-3,2],[-2,-2],[-1,-4],[0,-8],[-5,-5],[-9,-10],[-10,-16],[-2,-5],[-3,-2],[-3,-2],[-3,0],[-2,-1],[-2,0],[-3,-2],[-2,0],[-3,2],[-8,-11],[-1,0],[0,4],[-1,5],[-1,1],[-2,0],[-2,-1],[-2,-2],[-1,-2],[-1,-1],[-3,-1],[0,-1],[1,-1],[1,-4],[0,-3],[-2,-2],[-2,-3],[2,-8],[0,-6],[4,1],[1,8],[1,6],[4,3],[-1,-2],[-1,-6],[4,-1],[-1,-3],[-2,-5],[-3,-1],[0,-1],[0,-1],[0,-2],[1,-3],[2,1],[2,1],[2,2],[2,3],[1,2],[1,4],[1,3],[1,3],[2,3],[2,1],[3,2],[7,3],[5,4],[0,2],[1,1],[3,3],[2,1],[2,0],[0,-1],[0,-1],[0,-2],[1,-1],[1,0],[2,1],[3,2],[0,-2],[-2,-6],[1,-4],[1,-2],[2,-2],[2,2],[3,3],[2,7],[1,3],[3,-1],[0,4],[-2,1],[-1,1],[-2,2],[-3,-2],[-2,0],[-3,0],[-1,1],[-3,-1],[1,5],[4,3],[4,7],[3,1],[3,3],[3,0],[3,3],[2,-1],[3,1],[2,3],[2,-1],[3,-6],[2,-2],[2,-1],[0,-2],[0,-7],[-1,-2],[-2,-1],[-1,-1],[1,-4],[-5,-2],[-1,-1],[0,-3],[1,-1],[3,-2],[1,-2],[-1,-2],[1,-2],[4,-8],[0,-3],[-1,-2],[-1,-2],[0,-4],[-1,3],[-2,2],[-1,-1],[-1,-2],[1,-2],[-2,-3],[-1,-2],[1,-2],[0,-5],[-1,-4],[2,-3],[1,1],[0,-3],[0,-2],[-1,-1],[-3,0],[1,-1],[0,-2],[-2,-2],[1,-4],[2,-4],[3,-1],[2,0],[2,4],[2,1],[0,-1],[2,-5],[1,-1],[-4,-1],[-2,-1],[-1,-3],[1,-6],[-2,2],[-1,-4],[0,-1],[1,-2],[-4,0],[-4,-3],[-9,-7],[-4,1],[-5,1],[1,9],[0,4],[1,7],[0,6],[1,7],[-1,4],[2,1],[1,1],[2,1],[2,0],[-2,4],[-2,0],[-2,-3],[-1,-1],[-1,3],[0,3],[1,9],[-2,0],[-1,-1],[-2,-1],[-1,-2],[2,-6],[1,-11],[-1,-11],[-2,-7],[-2,-7],[-2,-1],[-2,-4],[-3,-2],[-6,0],[-3,2],[-1,5],[-1,4],[1,14],[0,3],[2,5],[-3,2],[-2,-1],[1,-5],[-1,-5],[1,-14],[0,-6],[-3,4],[-1,6],[-3,17],[-1,2],[-1,2],[-2,-1],[0,-2],[0,-3],[2,-2],[-2,-1],[-2,0],[-4,1],[0,-2],[1,-3],[-1,-5],[-2,-5],[-2,-3],[0,-2],[1,-3],[-1,-3],[-2,-1],[-1,3],[-1,3],[-1,-2],[-2,-4],[-1,-4],[-4,3],[-2,0],[-2,-3],[6,-3],[0,-1],[-1,0],[0,-1],[3,0],[0,-2],[-2,-1],[-2,-1],[-2,-2],[-3,-3],[16,-1],[5,3],[15,0],[1,-3],[0,-4],[-1,-4],[-1,-4],[-2,-2],[-3,1],[-2,2],[-2,1],[-4,-1],[-1,-3],[1,-8],[1,0],[1,0],[1,2],[-2,3],[3,3],[10,-3],[4,3],[6,-6],[1,-3],[-2,-1],[-2,-2],[-1,-4],[1,-4],[0,-1],[1,0],[2,0],[1,-1],[0,-1],[-1,-1],[1,-1],[1,-1],[3,0],[1,-1],[-3,-3],[-2,-2],[0,-4],[-10,5],[-3,3],[-3,5],[-1,-7],[-2,-2],[-3,2],[-1,4],[-1,1],[-4,0],[-3,-2],[-1,-5],[2,2],[1,-3],[0,-2],[1,-1],[2,0],[1,-3],[3,-1],[2,-1],[3,-2],[1,1],[2,1],[2,-1],[1,-3],[1,0],[1,-1],[-1,-1],[-1,-3],[0,-1],[1,-4],[2,1],[1,0],[2,-2],[1,-1],[1,-2],[-2,-4],[-2,-1],[-1,-1],[0,-2],[4,0],[0,-2],[-2,0],[-1,-1],[-1,-2],[-1,-2],[-1,-1],[-1,0],[0,-1],[-1,-2],[1,-2],[0,-2],[0,-1],[-2,-3],[-17,-6],[-25,3],[-3,1],[-2,2],[-2,3],[-2,2],[0,1],[-1,1],[-2,1],[-1,1],[-4,0],[-1,1],[-1,2],[-1,6],[-1,1],[-1,3],[-1,0],[-1,0],[-2,1],[-3,2],[-2,2],[-1,1],[0,2],[0,1],[-2,1],[0,-2],[3,-7],[2,-1],[3,-1],[2,-1],[2,-4],[2,-4],[1,-3],[1,-2],[7,-3],[3,-5],[0,-5],[-2,-3],[-3,-1],[-2,-3],[11,4],[6,1],[3,-3],[5,4],[7,-2],[6,-7],[3,-2],[3,-9],[0,-6],[2,-8],[2,-5],[3,-7],[2,-5],[-5,-1],[-3,0],[-2,0],[-2,-3],[2,0],[4,1],[6,-1],[4,-4],[-1,-3],[-3,-3],[0,-3],[2,-3],[2,-3],[1,-6],[3,-4],[-1,-5],[1,-2],[0,-2],[1,1],[2,-1],[1,-2],[-2,-5],[-4,-9],[-2,-4],[-4,-4],[-1,1],[1,11],[-3,1],[-4,4],[-3,2],[-4,1],[-4,0],[-3,-3],[-1,-1],[-1,-1],[-1,-2],[0,-2],[-1,0],[-2,2],[-1,0],[-2,-2],[-1,-4],[1,-2],[2,0],[1,-1],[0,-4],[-2,-3],[-1,0],[-2,-1],[-2,-1],[-2,-3],[0,-3],[-1,-3],[0,-4],[2,1],[1,1],[0,-3],[0,-3],[-1,-3],[-1,-2],[-2,-2],[-1,0],[-1,0],[-1,0],[-1,1],[-3,6],[-2,1],[-1,1],[-2,7],[-3,5],[-5,3],[3,9],[-2,9],[-5,7],[1,3],[1,2],[2,-1],[3,1],[2,1],[1,3],[-6,-2],[-2,1],[-5,5],[-3,1],[-1,-2],[1,-5],[-4,-4],[-2,-10],[-7,-3],[-2,-1],[-1,6],[-2,0],[-4,-1],[-1,-1],[2,-5],[3,-2],[4,0],[7,1],[1,-4],[-1,-8],[-1,-5],[-1,-5],[-2,-4],[-2,-5],[-6,-5],[-3,-5],[-4,-6],[-1,-5],[0,-9],[-1,-7],[-2,-5],[-3,-5],[-7,-2],[-4,3],[-3,2],[-4,-1],[-6,-2],[-3,1],[-3,3],[-3,3],[-3,3],[-2,3],[-4,1],[-8,1],[-6,-1],[-3,0],[-2,4],[-1,10],[-1,6],[-3,-1],[2,-3],[1,-3],[0,-12],[0,-3],[2,-2],[2,0],[9,0],[2,-1],[6,-3],[5,-7],[2,-2],[14,-5],[2,-2],[1,-4],[1,-4],[0,-4],[1,-6],[-2,-6],[-5,-3],[-3,-3],[-2,-8],[-4,-9],[-5,-10],[-3,-5],[-3,-1],[-2,4],[-6,7],[-3,3],[0,-5],[2,-3],[4,-6],[0,-2],[1,-5],[-1,-2],[-1,-3],[1,-2],[1,0],[-2,-9],[-5,-4],[-3,-4],[-2,-4],[-1,-2],[1,-6],[2,-5],[0,-4],[3,-2],[3,-1],[1,0],[3,-2],[1,-4],[-1,-4],[-2,-4],[-2,-1],[-1,-1],[-2,-5],[0,-2],[1,-2],[1,-2],[-1,-5],[-1,-1],[-2,0],[-10,6],[-3,0],[0,-4],[-1,-6],[-2,-4],[-3,-1],[0,-2],[2,0],[3,-1],[2,-2],[0,-1],[0,-2],[-1,-2],[0,-4],[0,-2],[-1,-1],[-1,-1],[-1,-1],[-1,-1],[0,-2],[1,-2],[0,-4],[-1,-3],[-1,-1],[-1,-2],[2,-4],[-3,-1],[-3,-3],[-2,-1],[-1,4],[-1,6],[-1,1],[-2,-1],[-3,0],[0,-3],[0,-1]],[[9424,7327],[1,-2],[2,-3],[1,-3],[0,-12],[0,-4],[-2,-4],[-2,-2],[-1,-3],[-1,-6],[0,-10],[-1,-5],[-2,-4],[-5,-5],[-2,-4],[-2,-4],[0,-5],[1,-4],[2,-3],[0,-4],[-1,-5],[-2,-4],[-6,-5],[-1,-2],[0,-1],[-1,-2],[0,-3],[-1,-2],[-3,-1],[-2,-1],[-1,-3],[0,-3],[-1,-7],[2,-8],[3,-7],[0,-7],[-5,-4],[-5,1],[-3,-1],[-2,-2],[-1,-4],[-1,-4],[-1,-2],[-22,0],[-7,4],[-3,1],[-4,-1],[-2,-3],[-2,-4],[-2,-4],[-1,1],[-13,0],[-10,-3],[-16,-9],[-14,-12],[-4,-11],[4,-5],[5,0],[6,1],[5,-1],[4,-3],[4,-5],[3,-6],[2,-6],[0,-8],[-2,-5],[-6,-6],[-1,5],[-1,11],[-1,3],[-2,3],[-3,0],[-3,-3],[-2,-4],[3,-4],[-1,-4],[-2,-2],[0,1],[-4,3],[0,2],[0,6],[0,2],[-1,1],[-2,0],[-1,0],[-2,1],[-11,11],[-1,5],[-9,11],[-3,9],[-1,10],[3,22],[-3,10],[-7,1],[-9,0],[-7,4],[2,2],[2,4],[1,4],[1,3],[-2,3],[-5,-1],[-7,-4],[-7,-1],[-2,1],[-2,1],[-2,2],[-5,8],[-1,1],[-2,0],[-2,0],[-3,-2],[-2,-1],[1,-2],[1,-2],[1,-1],[2,-1],[-3,-1],[-5,-1],[-1,-1],[-1,-3],[0,-6],[-1,-3],[-1,-3],[-6,-19],[-1,-8],[-1,-3],[-2,-7],[-2,-5],[-1,-3],[0,-4],[0,-5],[1,-2],[2,0],[2,-3],[0,-2],[-1,-2],[-1,-2],[0,-4],[-4,-1],[-1,-3],[0,-4],[3,-1],[0,-1],[-4,-6],[-1,-9],[0,-10],[0,-10],[-5,-9],[-5,-1],[-6,2],[-7,1],[1,-4],[0,-3],[-2,-3],[-4,-2],[0,-2],[0,-2],[-1,-3],[-1,-2],[-2,0],[-1,1],[-1,3],[-1,2],[-5,3],[-1,2],[-8,8],[-2,0],[-2,-2],[-2,-1],[-1,-1],[-3,2],[-1,3],[-2,1],[-2,-1],[0,-2],[1,-2],[1,-3],[-2,-1],[-2,3],[-2,-1],[-1,-4],[-1,-5],[-1,0],[-9,-3],[-1,-1],[-1,-2],[0,-2],[0,-2],[1,-1],[1,-1],[0,-1],[-1,-4],[-2,-2],[-5,0],[0,-2],[2,0],[1,0],[1,-2],[2,-1],[-2,-2],[-1,-2],[1,-2],[-1,-2],[-1,-1],[-2,-1],[-2,-2],[0,-3],[1,-1],[4,-1],[0,-2],[0,-2],[-1,-1],[-1,-2],[-4,-4],[-4,-5],[-5,-6],[-2,-2],[-3,0],[-4,3],[0,-5],[-2,-3],[-4,-6],[-3,-2],[-4,0],[-4,-1],[-1,-4],[-2,-11],[-6,-2],[-4,1],[-10,4],[-16,1],[-2,-5],[-4,-1],[-13,1],[-1,-1],[-2,-2],[-2,-4],[-2,-1],[-2,2],[-1,0],[-7,-3],[-2,0],[-6,4],[-7,-1],[-4,4],[-4,1],[-16,2],[-27,-9],[-7,0],[-3,-2],[-3,-3],[0,-4],[3,-23],[2,-4],[1,-4],[6,-2],[1,-5],[8,-18],[1,-5],[1,-6],[4,-5],[7,-6],[4,-3],[3,-4],[2,-6],[0,-4],[3,-6],[-2,-10],[-17,-37],[-6,-8],[-7,1],[2,4],[-2,1],[-3,-3],[-4,0],[-2,3],[0,6],[-3,-2],[-1,-1],[-2,1],[0,3],[0,5],[0,4],[-2,3],[-1,-2],[-2,-2],[-1,-1],[-5,5],[-7,-1],[-3,-2],[-2,-1],[-1,-5],[-3,-1],[-3,0],[-3,3],[-4,-2],[-16,1],[-4,2],[-1,0],[-2,-4],[-1,-1],[-1,1],[-6,5],[-6,2],[-2,3],[-1,6],[-3,-2],[-2,1],[-3,2],[-3,1],[-3,-2],[-2,-1],[-3,-2],[-3,1],[-1,0],[-1,2],[-1,1],[-2,1],[-1,-1],[-3,-1],[-1,0],[-7,0],[-3,2],[-3,1],[1,1],[2,1],[1,2],[0,2],[-2,1],[-1,0],[-1,-1],[-1,0],[-3,0],[-3,4],[-3,1],[-1,-1],[-2,-5],[-1,-2],[-1,1],[-1,1],[-1,1],[-1,1],[-2,-2],[-2,0],[-1,2],[-1,1],[-1,1],[-1,0],[2,5],[3,3],[3,3],[4,1],[-2,3],[-9,4],[-3,1],[1,2],[1,1],[-5,3],[-3,2],[-1,2],[-1,4],[0,3],[2,1],[3,0],[-1,3],[-4,7],[-1,3],[-1,0],[-6,6],[-3,3],[-3,0],[-2,0],[-1,2],[0,6],[-2,1],[-3,-1],[-4,-3],[-2,-3],[-2,-7],[-9,-4],[-2,-7],[-1,-2],[-3,2],[-5,6],[-2,3],[-3,1],[-2,0],[-3,-2],[1,-3],[2,-2],[3,-2],[2,-2],[-3,-2],[-5,-6],[-2,-1],[-5,0],[-3,0],[-3,-7],[-5,-7],[0,-3],[1,-4],[1,-3],[-1,-4],[-3,-7],[-2,-12],[-6,-11],[-1,-4],[1,-6],[2,-9],[0,-6],[-5,-2],[0,2],[1,0],[0,1],[0,1],[1,1],[-5,1],[-5,-5],[-8,-12],[-2,-3],[-4,-11],[-1,-4],[-5,-5],[-1,-2],[-4,-12],[-2,-3],[-2,-3],[-19,-13],[-4,-4],[3,-5],[-2,-4],[-3,-3],[-1,-3],[0,-3],[-2,-1],[-1,-1],[-2,-1],[-1,-2],[1,-3],[-1,-3],[-4,-6],[-2,-3],[-3,0],[-3,-1],[-2,-4],[2,0],[3,0],[1,-1],[-2,-3],[-2,-1],[-5,-1],[-2,-3],[-2,-1],[-2,2],[-2,3],[-2,1],[-3,-1],[-2,-1],[-6,-6],[-2,0],[-7,4],[-1,0],[-2,0],[-1,-1],[-1,-2],[-1,-1],[-1,-1],[-5,-1],[0,-1]],[[8188,7296],[-11,2],[-3,1],[-3,4],[-1,2],[-2,2],[-1,1],[-2,1],[-6,0],[-2,1],[-1,0],[0,1],[-4,5],[-3,2],[-2,2],[-2,0],[-5,1],[-2,1],[-2,1],[-1,1],[-2,2],[0,2],[-1,2],[-1,1],[0,3],[0,1],[0,2],[0,3],[1,2],[1,2],[0,1],[1,1],[6,6],[3,1],[0,1],[1,2],[-1,3],[0,2],[0,5],[5,-5],[7,-2],[1,1],[3,5],[1,0],[0,3],[1,3],[0,5],[0,5],[0,3],[-1,2],[0,1],[-4,7],[-3,6],[-1,4],[-9,34],[-3,8],[-9,19],[-1,2],[-3,11],[0,2],[0,2],[1,2],[0,1],[1,1],[3,3],[1,3],[1,1],[0,2],[0,3],[0,2],[0,5],[-1,2],[0,2],[-1,2],[-1,1],[-1,1],[-3,2],[-7,3],[-2,2],[-1,1],[-1,1],[-1,1],[0,1],[-1,2],[-4,14],[-3,7],[-2,4],[0,2],[-1,7],[0,1],[-1,1],[0,1],[-1,1],[-7,7],[-1,1],[0,1],[0,1],[0,1],[0,3],[0,6],[1,6],[0,3],[0,2],[0,3],[0,1],[0,1],[-1,1],[-1,4],[-1,1],[-1,2],[-1,0],[-2,0],[-6,-5],[-3,-3],[-3,-5],[-9,-10],[-22,-13],[-11,-12],[-4,-3],[-2,-1],[-13,-3],[-2,-2],[-2,-2],[-1,-3],[-1,-1],[-1,-1],[-35,-19],[-9,-2],[-4,0],[-3,2],[-4,7],[4,32],[0,9],[0,8],[0,2],[-1,2],[-3,4],[-3,4],[-24,38],[-3,7],[-4,13],[-1,3],[0,3],[0,4],[1,10],[0,2],[-1,4],[-3,14],[0,3],[0,1],[2,6],[0,2],[0,1],[0,4],[0,6],[-3,26],[-2,16],[0,2],[1,4],[0,1],[1,3],[1,1],[0,1],[1,1],[1,1],[3,2],[11,4],[4,3],[8,6],[1,2],[3,4],[2,4],[0,2],[0,2],[1,4],[0,3],[-1,2],[-1,2],[-2,4],[-1,3],[1,7],[-1,2],[0,2],[-3,5],[-4,9],[-5,9],[-1,2],[-1,1],[-2,2],[-13,4],[-1,1],[-1,1],[-1,1],[0,3],[-1,1],[0,1],[-1,0],[0,1],[-2,1],[-1,2],[-1,2],[-1,3],[-1,3],[0,2],[0,2],[1,1],[0,2],[2,5],[0,1],[1,2],[0,3],[0,2],[-1,2],[-3,4],[-2,5],[-1,2],[-1,1],[-1,0],[-1,1],[-2,1],[-2,1],[-3,0],[-3,0],[-13,-6],[-1,-1],[-3,0],[-3,1],[-2,1],[-10,9],[-1,1],[-2,3],[0,1],[0,2],[0,1],[1,1],[9,7],[2,1],[0,1],[3,4],[9,20],[2,6],[1,2],[0,3],[0,4],[0,3],[-1,1],[0,2],[-2,2],[-1,2],[-1,1],[-1,3],[-1,4],[0,2],[0,1],[1,1],[1,0],[2,0],[12,-1],[3,0],[2,-1],[6,-4],[3,-1],[7,-2],[2,-1],[2,-2],[1,-2],[1,-1],[1,-1],[1,-2],[1,-3],[0,-1],[1,-1],[14,-15],[8,-4],[2,-2],[0,-1],[1,-2],[2,-3],[1,-2],[1,-1],[2,0],[1,0],[1,1],[1,0],[4,5],[12,17],[2,4],[1,6],[1,2],[1,1],[2,0],[1,-1],[1,0],[1,-1],[3,-4],[1,-2],[2,-5],[1,-2],[3,-11],[2,-7],[0,-1],[1,-2],[1,-1],[10,-9],[6,-7],[6,-6],[7,-4],[1,0],[1,0],[1,1],[2,2],[5,8],[2,3],[1,3],[0,2],[1,1],[0,4],[-1,9],[0,3],[0,4],[0,2],[1,1],[2,4],[1,3],[1,2],[0,1],[-1,9],[-1,7],[2,31],[0,1],[1,1],[1,-1],[1,0],[41,4],[1,1],[2,0],[1,2],[1,2],[1,3],[0,3],[1,1],[3,8],[1,3],[1,2],[1,2],[2,3]],[[8101,8055],[1,2],[2,2],[2,3],[2,2],[2,1],[2,1],[1,1],[2,0],[2,0],[11,-6],[5,-2],[66,6],[3,1],[14,7],[2,0],[1,-2],[1,-1],[1,-2],[2,-1],[2,1],[1,3],[0,1],[1,1],[1,1],[1,0],[2,0],[0,-1],[0,-1],[0,-2],[1,-3],[-1,-3],[0,-4],[2,-4],[-2,-4],[0,-5],[1,-4],[2,-4],[1,0],[1,1],[0,-1],[0,-3],[0,-2],[1,-2],[1,-3],[0,-3],[0,-1],[0,-3],[0,-5],[2,-6],[2,-2],[2,-1],[0,-2],[0,-3],[0,-2],[-2,-1],[-2,1],[-2,0],[-1,-3],[2,-3],[-1,-1],[-2,-2],[-1,-1],[-1,-4],[0,-3],[0,-4],[1,-2],[3,-4],[8,-7],[2,-5],[0,-3],[-3,-5],[0,-2],[1,-3],[1,-1],[1,-2],[-1,-4],[-2,-1],[-2,-1],[-2,-1],[-1,-4],[2,-1],[6,-4],[2,-1],[0,-2],[-1,-1],[-2,-1],[6,-3],[2,-2],[2,-4],[1,-2],[0,-4],[1,-1],[3,0],[2,-1],[1,-1],[-3,-4],[-1,-4],[1,-3],[5,0],[4,2],[2,3],[2,-1],[0,-6],[1,-3],[5,-3],[0,-2],[0,-2],[1,-4],[1,-3],[2,-1],[7,1],[4,-2],[1,-4],[0,-1],[-1,-3],[-1,-2],[0,-1],[1,-1],[0,-1],[1,-3],[0,-2],[1,-2],[2,-2],[3,-2],[3,1],[3,2],[4,0],[3,-1],[4,-9],[4,-2],[4,-1],[2,1],[2,1],[1,3],[0,2],[0,3],[1,2],[4,-3],[3,0],[7,1],[8,0],[2,1],[4,4],[2,0],[1,-2],[1,-3],[1,0],[1,0],[5,-1],[4,-1],[2,0],[2,2],[1,1],[1,3],[-1,1],[0,1],[-1,2],[1,2],[0,1],[1,1],[1,1],[0,5],[1,2],[1,2],[2,0],[2,-2],[1,-2],[0,-3],[3,-3],[4,-1],[8,2],[2,-2],[1,-1],[-1,-2],[0,-3],[1,-5],[0,-1],[2,-4],[3,-2],[1,-2],[0,-3],[0,-4],[0,-1],[1,1],[2,1],[22,5],[2,0],[3,-1],[2,-1],[0,-1],[1,0],[2,0],[1,1],[0,2],[1,1],[2,0],[6,-4],[3,-1],[4,3],[2,4],[2,10],[1,4],[3,2],[4,-1],[4,-2],[4,-1],[3,2],[10,7],[3,1],[6,-1],[2,1],[3,1],[2,0],[0,-2],[0,-3],[1,-5],[0,-1],[0,-5],[0,-1],[1,-4],[0,-2],[0,-3],[0,-6],[0,-3],[-1,-2],[0,-1],[0,-2],[0,-1],[0,-2],[3,-5],[1,-3],[1,-2],[2,-2],[7,-8],[1,-1],[2,-3],[0,-1],[5,-5],[11,-8],[4,-1],[6,-1],[30,-15],[2,2],[1,1],[2,0],[1,0],[4,0],[3,0],[2,1],[2,3],[3,1],[1,1],[1,3],[1,0],[1,0],[1,-1],[2,0],[1,0],[2,1],[2,2],[6,6],[2,1],[2,1],[2,1],[4,0],[1,1],[4,1],[13,3],[2,0],[9,-4],[1,-1],[1,-1],[5,-6],[1,-1],[1,-1],[5,-2],[7,-3],[2,-1],[2,-2],[2,-4],[1,-1],[1,0],[6,0],[1,0],[1,0],[1,1],[2,0],[3,0],[3,-1],[5,-2],[2,0],[1,-1],[1,-1],[0,-1],[1,-3],[2,-11],[2,-5],[7,-12],[2,-5],[3,-6],[1,-2],[1,-7],[0,-1],[0,-2],[-1,-5],[-1,-2],[-7,-3],[-1,-2],[-2,-2],[-1,-1],[-1,-3],[-1,-4],[-1,-3],[0,-3],[0,-2],[2,-5],[1,-4],[5,-10],[1,-1],[0,-6],[1,-6],[-1,-11],[0,-2],[0,-5],[1,-4],[1,-1],[2,-4],[5,1],[2,3],[1,3],[2,4],[2,4],[1,2],[2,1],[1,1],[3,0],[2,0],[1,0],[2,-2],[1,-1],[0,-1],[2,-5],[1,-1],[1,-1],[1,-1],[2,0],[2,0],[1,1],[1,1],[1,1],[4,3],[6,3],[3,1],[2,0],[5,-3],[6,-6],[1,-1],[1,0],[6,-2],[2,-1],[7,-5],[1,-1],[1,-1],[0,-1],[1,-2],[0,-1],[1,-2],[1,-2],[1,-1],[0,-1],[-1,0],[-1,-1],[-4,0],[-1,0],[-1,0],[-2,-2],[-1,-1],[-1,-1],[0,-1],[0,-1],[0,-2],[0,-1],[1,-4],[0,-1],[0,-2],[0,-1],[0,-2],[-1,-3],[-2,-7],[-1,-2],[0,-1],[1,-2],[1,-2],[4,-6],[1,-1],[3,-8],[10,-15],[0,-1],[0,-2],[0,-3],[-1,-5],[0,-2],[0,-1],[0,-1],[0,-1],[0,-1],[8,-9],[3,-3],[2,-1],[2,0],[2,-1],[0,-1],[1,-1],[0,-3],[0,-3],[4,-13],[6,-4],[11,-4],[26,11],[3,2],[2,2],[2,2],[0,2],[1,1],[0,2],[0,3],[0,19],[-1,6],[0,2],[0,2],[1,4],[1,4],[-1,4],[-1,5],[-1,4],[0,5],[2,2],[1,-1],[4,-4],[2,-1],[3,-1],[3,1],[3,3],[2,4],[2,9],[2,5],[1,3],[2,2],[9,7],[9,6],[7,3],[1,-1],[4,-5],[4,-8],[1,-1],[1,-5],[2,-7],[2,-15],[0,-7],[-3,-26],[0,-4],[1,-8],[0,-2],[1,-3],[1,-2],[1,-3],[2,-2],[7,-8],[1,-2],[1,-3],[1,-3],[1,-6],[1,-4],[1,-4],[8,-14],[5,-13],[2,-3],[7,-6],[1,-2],[2,-3],[2,-6],[1,-2],[4,-7],[1,-2],[0,-1],[1,-5],[1,-6],[-1,-8],[1,-1],[0,-2],[1,-2],[2,-3],[6,-8],[1,-3],[1,-4],[3,-10],[1,-3],[2,-3],[1,-2],[0,-1],[1,-1],[2,-1],[1,0],[7,1],[1,0],[2,-1],[3,-3],[1,-1],[1,0],[2,0],[2,0],[24,14],[8,3],[2,1],[1,1],[1,1],[0,1],[0,1],[0,1],[0,2],[-2,3],[-1,4],[0,3],[0,1],[0,2],[0,2],[0,1],[0,2],[0,2],[0,2],[0,3],[0,3],[0,2],[0,1],[0,2],[1,1],[1,3],[1,2],[2,2],[6,4],[2,0],[9,1],[2,1],[9,8],[10,5],[9,3],[35,1],[4,1],[1,1],[0,1],[1,1],[1,1],[0,2],[0,3],[1,3],[0,6],[0,4],[1,1],[0,1],[1,1],[3,2],[6,6],[3,3],[5,2],[2,-1],[3,-1],[9,-8],[1,-3],[1,-3],[0,-3],[1,-16],[0,-3],[1,-3],[0,-1],[1,-1],[0,-1],[2,-1],[2,-1],[4,-1],[2,0],[2,0],[1,1],[1,2],[5,9],[3,5],[5,5],[2,2],[7,2],[2,1],[1,1],[0,1],[1,1],[1,2],[5,18],[1,2],[1,2],[1,0],[2,-2],[0,-1],[0,-1],[1,-4],[0,-1],[-1,-3],[-2,-14],[0,-3],[0,-3],[1,-2],[1,-3],[1,-3],[2,-5],[0,-3],[0,-3],[1,-2],[0,-2],[0,-1],[0,-6],[0,-2],[1,-2],[0,-2],[1,-2],[1,-3],[4,-5],[0,-1],[1,-1],[0,-1],[0,-1],[0,-2],[-3,-7],[0,-2],[0,-2],[0,-1],[0,-2],[0,-1],[3,-1],[28,-11],[5,-3],[1,-1],[2,-4],[7,-7],[4,-3],[6,-4],[2,-2],[7,-7],[2,-2],[2,-5],[1,-2],[0,-4],[2,-3],[0,-2],[1,-1],[1,0],[1,-1],[2,0],[4,2],[2,1],[2,1],[12,15],[2,0],[2,0],[4,-2],[10,-15],[2,-2],[2,-1],[1,0],[3,1],[10,7],[4,1],[3,1]],[[7815,9932],[3,-3],[2,-3],[2,-1],[9,-6],[11,-4],[5,-3],[3,-3],[2,-2],[1,-2],[1,-1],[0,-2],[3,-12],[1,-5],[1,-2],[2,-1],[8,-4],[2,-1],[1,-1],[1,-3],[0,-1],[1,-4],[0,-2],[2,-4],[2,-3],[0,-2],[0,-1],[1,-1],[-1,-2],[0,-1],[-1,-4],[-1,-1],[-1,-1],[-2,-2],[-3,-4],[-2,-2],[-1,-2],[-3,-5],[-2,-5],[-1,-2],[-31,-35],[-54,-48],[-1,-1],[0,-1],[0,-1],[0,-2],[2,-1],[3,-2],[21,-8],[12,-8],[3,-4],[1,-1],[7,-8],[4,-3],[15,-8],[18,-6],[2,-2],[2,-2],[2,-2],[1,-1],[1,-2],[5,-12],[5,-7],[2,-2],[1,-1],[2,-1],[7,-2],[6,-1],[9,2],[5,2],[3,3],[2,1],[2,4],[9,15],[6,14],[1,2],[3,11],[3,5],[0,1],[1,1],[1,0],[2,0],[1,0],[22,-9],[1,-1],[1,-2],[1,-2],[1,-5],[1,-3],[2,-2],[2,-4],[8,-7],[1,-2],[4,-7],[1,-4],[0,-1],[2,-2],[1,-2],[0,-1],[2,-1],[9,-4],[27,-4],[1,-1],[2,-1],[3,-3],[1,-2],[0,-2],[0,-1],[0,-2],[0,-1],[-1,-2],[0,-1],[-1,-3],[-2,-2],[0,-1],[-2,-1],[-2,-2],[-5,-2],[-5,-2],[-2,0],[-1,-1],[-1,-1],[-1,-2],[0,-1],[0,-2],[0,-2],[0,-3],[19,-66],[2,-5],[0,-3],[0,-2],[0,-1],[-1,-2],[0,-1],[-1,-3],[-5,-6],[-1,-2],[-1,-2],[0,-1],[0,-2],[0,-1],[0,-2],[0,-3],[1,-2],[20,-36],[0,-1],[1,-1],[0,-2],[0,-1],[-1,-3],[0,-4],[0,-1],[0,-2],[1,-3],[2,-4],[5,-9],[11,-17],[4,-2],[6,-4],[2,-2],[6,-7],[4,-4],[2,-1],[2,-1],[3,1],[2,0],[2,2],[1,0],[1,0],[2,0],[1,-1],[1,-1],[2,-4],[3,-2],[2,-2],[2,-1],[2,-2],[2,0],[16,0],[11,1],[2,0],[2,0],[4,-1],[3,-2],[0,-1],[2,-2],[5,-8],[2,-2],[1,-1],[1,-1],[4,-1],[4,-1],[7,1],[16,5],[2,1],[1,1],[1,1],[1,1],[0,1],[0,2],[1,1],[0,5],[1,3],[1,3],[1,2],[1,1],[1,1],[1,1],[2,1],[3,0],[23,-3],[26,7],[2,0],[2,-1],[7,-4],[5,-4],[4,-3],[4,-1],[6,0],[4,1],[4,1],[5,3],[0,1],[1,1],[1,1],[0,2],[0,3],[1,3],[1,2],[0,1],[1,1],[1,1],[3,0],[3,1],[27,-7],[16,8],[9,0],[1,0],[1,1],[1,2],[2,5],[0,2],[1,1],[2,2],[1,1],[9,5],[1,1],[1,1],[0,1],[0,2],[1,3],[0,1],[0,2],[1,1],[0,2],[1,1],[1,0],[1,1],[1,0],[7,2],[2,1],[2,2],[1,2],[1,1],[0,2],[1,1],[0,2],[0,8],[0,2],[0,1],[1,1],[0,2],[1,1],[1,1],[1,0],[1,1],[1,1],[1,0],[3,0],[34,-9],[9,-5],[16,-15],[5,-6],[4,-7],[1,-3],[1,-3],[1,-6],[1,-2],[1,-2],[1,-2],[11,-12],[3,-4],[3,-7],[1,-2],[1,-5],[1,-2],[3,-4],[12,-10],[1,-1],[3,1],[4,-2],[2,-5],[0,-6],[3,-5],[3,-2],[4,-1],[2,-2],[0,-5],[5,1],[2,-1],[2,-1],[0,-1],[0,-4],[1,-4],[6,-5],[2,-4],[0,-2],[1,-3],[0,-5],[1,-2],[1,-1],[0,-2],[0,-5],[-2,-5],[-2,-4],[-3,-2],[1,-2],[0,-1],[-3,-2],[-1,-2],[0,-2],[-1,-3],[-1,-1],[-3,4],[-2,2],[-1,-1],[0,-3],[-1,-2],[-1,-2],[-1,-2],[-1,2],[-2,0],[-1,-1],[-2,-1],[1,-1],[1,-1],[2,-3],[-3,-2],[-8,-9],[-2,-3],[-2,-4],[-9,0],[-2,-4],[1,-3],[6,-8],[2,-4],[-1,-2],[-3,-4],[1,-2],[2,-8],[1,-10],[0,-10],[-3,-7],[-6,-5],[-2,-1],[-3,0],[-2,-1],[-1,-2],[-2,-5],[-1,-1],[-1,0],[-2,-1],[-2,-2],[-1,-1],[-1,-3],[-2,-1],[-8,-2],[-1,-1],[-1,-2],[0,-6],[0,-3],[-1,0],[-4,3],[-2,-3],[-1,-14],[-1,-4],[-2,-3],[-1,-2],[-2,0],[-2,-2],[-2,-2],[0,-2],[-1,-3],[-1,-2],[-1,-1],[-1,-2],[1,-2],[1,-2],[1,-2],[-3,-2],[1,-3],[-1,-3],[-3,-2],[-6,-1],[-6,-7],[2,-5],[-2,-4],[-5,-1],[-2,3],[-2,-2],[-3,-3],[-1,-3],[-1,-3],[1,0],[2,-6],[0,-1],[2,-1],[4,0],[2,-1],[-3,-3],[-3,0],[-7,1],[-1,-1],[2,-2],[2,-2],[2,-2],[1,0],[1,-1],[1,-3],[-1,-2],[-2,-2],[-1,-1],[-2,0],[-2,-1],[-2,-6],[-2,-2],[-4,-1],[-2,-1],[0,-3],[2,-2],[3,-2],[1,-2],[0,-5],[2,-3],[2,-3],[-4,-3],[2,-5],[-3,-2],[-4,-1],[-2,-4],[0,-2],[2,-2],[2,-1],[2,-1],[1,-3],[0,-3],[-1,-5],[0,-23],[0,-2],[-2,-2],[-1,-2],[-2,-1],[-1,1],[-3,4],[-2,1],[-4,-2],[1,-6],[3,-4],[2,0],[2,-1],[4,-4],[1,-2],[0,-4],[1,-3],[2,-1],[1,-1],[1,-1],[1,-4],[-1,1],[-2,-3],[-1,-3],[2,-2],[0,-3],[6,-17],[0,-3],[-3,-7],[-1,-3],[0,-3],[2,-6],[0,-3],[-1,-6],[-3,-5],[-1,-4],[2,-3],[-1,-3],[-1,-1],[-1,0],[0,-5],[0,-1],[0,-1],[1,-1],[0,-1],[0,-2],[-1,-2],[-1,-2],[-2,-2],[-1,-2],[-2,-1],[-1,-1],[-2,-6],[-1,-2],[-3,-3],[-4,-1],[-8,0],[-2,1],[-2,3],[-1,4],[-13,0],[-8,3],[-5,0],[-2,-1],[-1,-2],[-1,-2],[0,-3],[-1,-3],[-1,-2],[-2,-1],[0,-2],[0,-9],[-1,-6],[-1,-4],[-2,-4],[-2,-4],[-15,-44],[-4,-4],[0,-3],[-2,-7],[-4,-9],[-2,-5],[0,-4],[-1,-3],[0,-4],[0,-9],[-1,-4],[-1,-3],[-3,-6],[-8,-12],[-1,-4],[0,-3],[3,-7],[-3,-5],[1,-4],[1,-5],[-1,-5],[-1,-4],[-1,-12],[-1,-5],[5,-4],[2,-10],[0,-21],[-2,-8],[-1,-1],[-3,-3],[0,-2],[0,-1],[0,-1],[0,-1],[0,-8],[-1,-3],[-1,-2],[-3,-1],[-2,-2],[-1,4],[0,3],[-1,1],[0,2],[-1,2],[-1,1],[-5,4],[-1,1],[-1,1],[0,1],[0,2],[0,1],[0,5],[0,1],[0,2],[-1,1],[-1,1],[-1,1],[0,1],[0,1],[0,2],[0,1],[-1,1],[0,1],[-2,2],[0,2],[-1,2],[-1,1],[0,1],[-2,2],[-1,2],[-1,1],[-2,4],[-2,5],[-1,3],[0,3],[0,1],[1,4],[0,9],[0,2],[-2,3],[0,1],[0,2],[0,1],[0,3],[0,2],[0,1],[0,2],[-1,1],[-3,6],[-5,5],[-1,0],[-2,-1],[-25,-22],[-58,-68],[-24,-36],[-5,-5],[-10,-7],[-23,-12],[-8,-5],[-2,-1],[-1,-2],[-1,-2],[-1,-2],[-4,-14],[-3,-10],[-3,-4],[-18,-15],[-48,-30],[-13,-12],[-25,-18],[-2,-2],[-2,-3],[-5,-15],[-3,-12],[-1,-2],[-12,-14],[-2,-4],[8,-13],[6,-8],[2,-4],[6,-18],[1,-2],[5,-8],[9,-9],[3,-3],[19,-9],[2,-2],[2,-1],[2,-2],[32,-51],[5,-6],[8,-6],[8,-5],[5,-3],[5,-1],[1,1],[2,1],[6,9],[2,2],[2,2],[8,1],[1,1],[2,2],[4,11],[2,4],[2,2],[2,0],[8,-2],[3,-2],[1,-2],[0,-4],[-1,-6],[1,-2],[2,-2],[2,-1],[2,-1],[3,0],[3,0],[1,2],[1,3],[0,4],[2,4],[1,2],[1,1],[2,0],[1,-2],[3,-7],[0,-2],[-1,0],[-1,-2],[-1,-2],[0,-2],[0,-1],[5,0],[0,-1],[-1,-3],[-2,-5],[-12,-22],[-22,-24],[-1,-1],[-3,-1],[-1,-1],[-13,0],[-12,1],[-20,8],[-5,1],[-2,-1],[-3,-1],[-2,-2],[-1,0],[0,-1],[-1,-1],[0,-2],[0,-1],[-1,-2],[0,-1],[0,-3],[1,-13],[0,-5],[-1,-7],[-3,-9],[0,-2],[0,-3],[1,-4],[7,-16],[21,-33]],[[3856,7146],[54,-8],[54,-9],[66,-10],[66,-11],[67,-10],[66,-11],[24,9],[57,21],[8,0],[65,-13],[66,-13],[65,-13],[32,4],[18,-2],[19,-6],[16,-14],[24,-48],[15,-19],[10,-6],[56,-16],[57,-16],[56,-16],[5,-3],[1,0],[47,-31],[48,-31],[5,-1],[64,16],[64,15],[-1,-54],[1,-3],[16,-3],[48,0],[3,-2],[4,-4],[6,-9],[7,2],[30,39],[3,2],[9,1],[9,6],[3,1],[2,1],[7,7],[12,8],[64,40],[74,39],[73,39],[70,13],[70,14],[70,13],[21,4],[9,-1],[14,-4],[35,0],[43,-9],[5,0],[6,2],[16,11],[5,0],[43,-6],[4,0],[8,4],[25,2],[1,1],[1,1],[1,2],[1,0],[1,0],[1,1],[24,21],[40,21],[13,3],[9,-1],[2,1],[3,1],[1,2],[2,2],[2,3],[6,4],[43,22],[3,5],[4,14],[4,4],[15,15],[7,9],[5,11],[4,14],[3,5],[19,24],[22,37],[7,7],[20,14],[39,19],[11,10],[4,2],[18,2],[5,3],[5,5],[25,37],[3,2],[12,-1],[8,4],[3,1],[4,3],[2,7],[0,9],[-1,8],[-2,9],[-15,31],[-4,7],[-5,6],[-19,14],[-5,7],[-15,24],[-4,8],[-3,10],[0,10],[-2,5],[-3,4],[-10,8],[-2,5],[0,11],[2,7],[2,9],[4,7],[4,5],[2,3],[4,11],[4,6],[1,4],[1,4],[0,5],[-1,9],[1,4],[1,7],[31,72],[3,6],[5,5],[9,8],[3,2],[7,2],[4,2],[2,2],[1,2],[2,2],[2,1],[2,0],[4,0],[16,-5],[45,1],[5,-2],[5,-3],[13,-16],[7,-13],[3,-5],[5,-4],[22,-12],[37,-11],[7,-1],[14,-7],[60,-6],[3,-1],[5,-4],[2,-2],[12,-1],[5,2],[6,5],[10,13],[17,14],[8,11],[4,1],[10,1],[7,3],[7,3],[5,6],[5,8],[3,7],[2,3],[2,2],[8,6],[34,37],[9,15],[5,12],[1,6],[1,12],[1,2],[2,2],[22,10],[5,1],[6,-1],[27,-12],[6,-1],[31,5],[34,0],[22,5],[27,8],[5,3],[4,4],[12,16],[36,34],[3,5],[1,0],[9,-1],[2,0],[3,1],[8,5],[4,3],[10,16],[4,9],[-2,7],[-3,6],[-3,8],[1,7],[24,52],[27,32],[3,4],[1,5],[3,8],[3,2],[3,3],[7,3],[10,1],[2,1],[11,10],[3,5],[2,1],[2,0],[21,-6],[56,-4],[6,2],[3,6],[0,18],[2,7],[6,10],[1,3],[0,4],[-3,9],[13,3],[3,-1],[2,-1],[1,0],[8,7],[2,0],[2,-3],[4,-11],[3,-4],[8,-5],[4,-1],[16,2],[3,2],[3,4],[4,10],[3,4],[4,3],[4,2],[8,2],[5,1],[13,11],[3,1],[9,1],[2,1],[6,6],[5,4],[5,2],[6,0],[5,-1],[6,-2],[10,-6],[7,-1],[13,0],[2,1],[5,3],[2,1],[6,-2],[6,-4],[6,-2],[6,4],[6,13],[4,5],[5,2],[3,-2],[1,-4],[1,-4],[2,-4],[3,-1],[2,2],[4,5],[3,1],[3,-4],[5,-11],[4,-4],[4,-2],[42,-13],[7,1],[15,3],[8,1],[12,-3],[4,0],[4,-1],[5,-5],[4,-1],[3,1],[3,2],[4,3],[2,3],[3,5],[2,3],[2,1],[3,-2],[3,0],[3,2],[3,4],[3,5],[2,2],[1,2],[0,2],[-1,3],[-2,4],[-1,3],[1,4],[2,9],[0,5],[0,2],[0,2],[0,1],[-1,1],[0,1],[-1,1],[-1,1],[0,1],[0,3],[0,2],[1,2],[1,3],[0,2],[-2,1],[-3,0],[-2,0],[-1,2],[-1,2],[0,2],[-1,3],[-3,3],[-5,6],[-2,4],[-1,1],[0,2],[-1,3],[0,16],[-1,5],[-4,9],[-5,8],[-11,15],[-7,6],[-10,3],[-2,3],[-2,5],[-2,5],[-2,3],[-26,26],[-6,2],[1,3],[4,7],[1,2],[-2,4],[-3,2],[-6,2],[-15,7],[-4,3],[-2,3],[-1,4],[-2,19],[-1,5],[-3,4],[-3,3],[-15,6],[-33,16],[-9,10],[-18,32],[-10,14],[-11,6],[-32,2],[-4,1],[-8,6],[-4,1],[-10,0],[-9,-2],[-11,-5],[-4,-1],[-26,2],[-8,-3],[-4,-4],[-35,-51],[-9,-8],[-6,-7],[-6,-12],[-6,-9],[-7,1],[-41,41],[-3,3],[-13,5],[-19,9],[-5,1],[-12,-1],[-41,-10],[-11,0],[-31,7],[-10,-3],[-16,-12],[-18,-25],[-10,-7],[-10,6],[-41,48],[-4,6],[-2,6],[-3,16],[-5,34],[0,2],[4,5],[38,20],[5,5],[1,8],[-3,53],[0,5],[0,5],[2,5],[37,63],[3,8],[0,5],[-2,9],[1,5],[34,82],[34,82],[3,7],[34,81],[6,-6],[55,-34],[32,-13],[36,-2],[47,-27],[2,-1],[1,1],[3,1],[1,0],[2,-1],[1,-1],[2,0],[1,0],[6,1],[3,9],[2,1],[4,4],[14,8],[6,5],[3,2],[3,4],[3,4],[10,5],[3,2],[3,4],[3,4],[1,5],[2,0],[10,10],[4,2],[4,1],[3,2],[2,7],[2,2],[10,2],[3,2],[1,2],[1,5],[1,2],[1,2],[4,5],[5,4],[3,3],[4,2],[8,7],[4,2],[7,-4],[5,1],[8,4],[16,3],[2,1],[2,1],[1,1],[2,0],[2,-1],[3,-1],[4,-1],[17,2],[16,6],[3,2],[2,2],[5,6],[8,9],[2,3],[3,4],[1,4],[1,5],[1,11],[0,3],[-1,2],[-2,3],[0,1],[0,16],[1,4],[2,2],[2,1],[1,2],[-1,2],[-1,6],[-1,1],[-3,1],[-3,1],[-10,-3],[-1,1],[-2,2],[-2,1],[-1,0],[-4,-1],[-3,1],[1,3],[-1,2],[-5,0],[-1,2],[0,3],[2,3],[1,1],[2,1],[5,0],[2,0],[1,3],[5,6],[2,8],[1,1],[0,2],[0,1],[-1,0],[-1,1],[0,2],[0,1],[1,2],[1,1],[4,16],[-1,2],[2,3],[1,1],[2,1],[2,1],[2,2],[1,3],[1,2],[2,0],[2,2],[1,6],[2,1],[2,1],[2,1],[4,2],[3,3],[6,11],[1,3],[2,11],[0,6],[-3,6],[3,3],[2,5],[0,5],[-2,5],[9,11],[4,6],[2,2],[12,10],[5,6],[0,7],[3,0],[2,2],[1,3],[1,5],[0,3],[1,4],[1,4],[-1,4],[1,0],[1,1],[2,3],[-4,7],[0,4],[2,1],[5,-1],[2,0],[0,2],[-1,3],[0,3],[1,6],[1,2],[1,1],[4,1],[1,2],[4,7],[2,2],[3,2],[0,5],[-2,5],[-1,4],[8,2],[2,1],[0,2],[1,2],[1,3],[2,5],[0,3],[-1,3],[1,6],[2,4],[8,10],[2,3],[0,7],[1,5],[1,5],[3,8],[3,4],[1,1],[2,1],[5,-1],[2,1],[3,4],[22,25],[9,6],[6,7],[3,2],[4,1],[2,1],[1,2],[1,3],[1,2],[2,2],[1,0],[2,0],[2,0],[3,2],[1,2],[1,2],[2,2],[2,0],[7,0],[4,1],[2,4],[4,9],[4,5],[0,3],[1,3],[-1,3],[-1,3],[-1,3],[0,4],[2,3],[2,1],[2,1],[2,2],[1,3],[3,5],[2,3],[-1,2],[-1,2],[0,2],[1,2],[1,3],[0,1],[-1,0],[-1,2],[0,2],[-3,4],[0,2],[0,3],[1,4],[0,3],[-3,5],[-14,11],[-3,7],[0,3],[-1,1],[0,2],[0,2],[1,1],[1,2],[2,1],[1,3],[0,4],[4,7],[1,4],[0,8],[0,1],[2,4],[0,2],[0,3],[1,3],[3,6],[-3,2],[-9,5],[-7,2],[-12,7],[-7,4],[-5,5],[-2,1],[-3,-1],[-7,-4],[-4,-1],[-17,-2],[-3,-1],[-7,-4],[-4,-2],[-10,0],[-6,2],[-4,3],[-2,3],[-1,3],[-1,4],[1,5],[0,3],[3,6],[0,3],[-2,5],[1,3],[-2,7],[5,6],[15,7],[3,2],[7,7],[6,2],[2,2],[2,2],[1,2],[1,3],[2,2],[4,2],[1,2],[1,2],[1,2],[1,3],[1,1],[5,5],[10,14],[12,12],[15,11],[6,7],[10,16],[6,6],[12,8],[1,2],[-1,5],[3,3],[5,1],[30,3],[16,-4],[8,1],[8,2],[8,4],[4,5],[2,1],[3,0],[8,-2],[1,0]],[[7815,9932],[8,1],[8,3],[22,14],[40,12],[22,0],[10,0],[4,2],[6,6],[4,2],[9,2],[11,5],[4,1],[3,-2],[4,-3],[3,-5],[4,-2],[4,0],[7,3],[64,3],[23,9],[25,1],[1,1],[3,4],[4,2],[5,6],[3,2],[4,0],[5,-1],[5,-1],[5,-4],[2,-1],[6,-1],[2,-1],[7,-4],[1,-1],[1,3],[0,4],[2,5],[3,0],[3,-2],[2,-4],[-1,-1],[-1,-1],[-2,-3],[3,-2],[3,3],[5,9],[4,1],[4,-3],[6,-8],[4,-3],[5,-1],[20,-3],[4,-1],[15,-9],[3,-4],[4,-7],[1,-2],[2,-2],[4,-1],[2,-2],[5,-7],[4,-2],[3,1],[3,3],[4,3],[4,1],[4,-1],[3,-1],[4,-3],[3,-4],[2,-6],[3,-5],[0,-1],[3,-9],[3,-3],[4,-4],[4,-2],[7,-2],[8,-4],[4,-1],[19,0],[2,0],[4,-3],[2,-1],[1,-3],[0,-2],[0,-2],[0,-2],[4,-3],[4,-1],[9,2],[3,-2],[5,-9],[3,-1],[5,2],[-1,5],[-2,5],[0,5],[3,3],[3,3],[17,4],[9,0],[8,-1],[50,-30],[4,-1],[3,-2],[4,-8],[4,-2],[10,4],[6,1],[3,-4],[2,-6],[4,-5],[9,-7],[-1,-7],[-1,-2],[-1,-1],[-4,-1],[-1,-1],[-3,-4],[-3,-5],[0,-4],[4,-2],[4,1],[7,4],[16,3],[4,-2],[1,-3],[-1,-2],[0,-2],[0,-3],[0,-2],[1,-2],[18,-22],[2,0],[3,4],[2,1],[8,0],[3,-2],[3,-6],[-6,-2],[-3,-2],[-2,-3],[4,-8],[1,-4],[-2,-2],[-4,-1],[-5,-1],[-3,-3],[-1,-6],[0,-6],[3,-6],[3,-5],[3,0],[1,0],[2,1],[2,1],[0,3],[1,2],[3,0],[14,-12],[7,-4],[-1,-6],[-1,-6],[0,-6],[2,0],[5,0],[5,-1],[2,-3],[1,-4],[3,-5],[5,-7],[1,-3],[-1,-6],[-4,-11],[4,-3],[13,-4],[3,-3],[-8,-3],[-13,-10],[0,-4],[1,-3],[2,-2],[3,0],[17,-7],[7,-2],[6,-2],[4,-6],[-1,-4],[-5,-14],[-1,-3],[0,-2],[-4,-1],[-4,-2],[-3,-3],[-1,-6],[1,-4],[33,-64],[2,-2],[9,-2],[0,-2],[2,-2],[0,-2],[0,-1],[-2,-2],[-1,-1],[1,-6],[1,-4],[0,-4],[-3,-5],[-5,-7],[1,-3],[4,-4],[12,-4],[5,-3],[4,-7],[-5,-9],[-1,-4],[-1,-6],[2,-5],[4,-2],[8,0],[3,-1],[1,-2],[2,-2],[0,-4],[-2,-4],[-3,-2],[-3,0],[-3,-1],[-3,-3],[-2,-4],[-1,-4],[0,-5],[3,-7],[3,-3],[4,0],[4,5],[0,3],[-1,2],[0,1],[0,2],[1,2],[1,2],[1,1],[2,1],[3,1],[3,0],[3,-1],[1,-4],[0,-3],[-2,-4],[-3,-6],[-5,-7],[-1,-3],[-1,-4],[0,-4],[0,-7],[1,-5],[2,-10],[0,-6],[1,-6],[3,-4],[5,-5],[9,-15],[2,-1],[11,-10],[5,-7],[13,-31],[3,-3],[4,-3],[3,-4],[2,-5],[0,-2],[-1,-2],[0,-2],[-1,-1],[0,-3],[0,-1],[0,-2],[1,-2],[1,-2],[5,-8],[3,-2],[2,-6],[1,-7],[0,-7],[-1,-2],[-4,-5],[-2,-3],[-2,-7],[-1,-3],[-2,-3],[2,-4],[8,-7],[1,-3],[0,-5],[0,-3],[-1,-3],[-1,-2],[-1,-3],[-1,-3],[0,-4],[3,-7],[9,-6],[28,-15],[2,-3],[0,-6],[-1,-5],[-1,-5],[-1,-5],[-11,-18],[-3,-10],[1,-13],[5,-9],[2,-5],[-1,-4],[-1,-4],[-2,-6],[-1,-7],[1,-5],[5,-9],[17,-6],[6,-7],[1,-6],[0,-5],[0,-5],[2,-5],[4,-3],[7,-4],[4,-3],[3,-4],[3,-4],[4,-2],[9,-1],[2,0],[2,1],[1,3],[1,1],[2,1],[2,0],[4,-2],[3,-3],[6,-7],[3,-3],[4,-2],[17,0],[5,0],[3,3],[4,2],[8,-4],[4,1],[2,2],[2,4],[1,1],[2,1],[27,3],[13,-4],[1,1],[5,8],[1,1],[2,-2],[8,-5],[4,-2],[5,-1],[1,0],[1,-2],[0,-3],[0,-3],[0,-3],[-2,-1],[-2,-1],[-1,-1],[-3,-5],[0,-5],[1,-3],[5,-2],[3,1],[7,3],[5,0],[3,-2],[8,-6],[8,-2],[3,-3],[6,-14],[2,-4],[4,-3],[4,-1],[4,2],[5,7],[4,2],[4,0],[5,-1],[4,-3],[7,-8],[4,-1],[4,3],[1,5],[0,11],[1,4],[4,1],[9,-1],[4,-1],[4,-3],[2,-4],[2,-3],[0,-4],[0,-13],[2,-5],[2,-3],[3,-3],[8,-1],[8,1],[7,-1],[4,-10],[0,-9],[1,-2],[2,-3],[3,-2],[8,-1],[3,-2],[1,-3],[-1,-12],[3,-1],[4,-2],[6,-4],[-2,-6],[1,-5],[2,-4],[8,-3],[3,-3],[7,-8],[27,-24],[8,-5],[7,2],[7,4],[8,3],[9,0],[4,-2],[2,-3],[1,-4],[5,-2],[4,-1],[4,1],[10,5],[4,0],[4,-1],[0,-4],[-1,-4],[-7,-12],[-15,-42],[-1,-5],[2,-5],[4,-3],[4,-1],[3,-2],[2,-4],[0,-3],[-1,-5],[0,-3],[1,-3],[1,-5],[0,-2],[2,-1],[13,6],[5,-1],[2,-3],[0,-5],[-2,-5],[-1,-5],[0,-5],[4,-7],[1,-5],[1,-2],[2,-2],[2,-2],[2,-1],[1,-1],[1,-4],[0,-7],[-1,-3],[-8,-10],[0,-2],[-1,-3],[-2,-9],[-1,-2],[-3,-1],[-4,-4],[-5,-8],[-2,-5],[1,-3],[1,-5],[0,-6],[1,-3],[29,-33],[8,-11],[6,-17],[0,-8],[-1,-10],[1,-9],[4,-4],[3,-1],[6,-5],[4,-1],[6,0],[3,0],[4,1],[7,4],[7,4],[34,8],[3,0],[6,-2],[5,-3],[5,-5],[7,-9],[1,-1],[1,-1],[2,0],[1,0],[2,1],[1,0],[9,8],[3,0],[13,-4],[3,0],[6,1],[3,0],[2,-1],[5,-3],[1,0],[2,0],[1,0],[2,1],[4,4],[1,1],[10,2],[10,-1],[5,-2],[7,1],[5,1],[2,1],[8,7],[2,1],[11,2],[6,0],[5,-2],[9,-5],[4,-1],[8,0],[1,1],[2,2],[2,1],[3,5],[1,2],[0,2],[0,2],[-1,5],[0,2],[0,1],[1,2],[4,7],[8,11],[2,3],[1,4],[0,2],[-1,2],[-1,1],[0,2],[0,2],[0,2],[1,2],[1,1],[2,2],[3,1],[3,0],[2,-2],[4,-4],[2,-1],[3,0],[3,1],[2,1],[2,3],[3,6],[2,3],[13,9],[0,2],[13,6],[2,2],[1,1],[0,1],[1,1],[1,2],[1,4],[3,2],[2,2],[16,5],[6,0],[6,-2],[2,-1],[3,-3],[2,-1],[3,0],[9,2],[3,0],[3,-1],[6,-3],[2,-1],[10,2],[3,1],[3,3],[2,6],[1,3],[1,3],[1,2],[2,2],[3,1],[5,2],[3,2],[2,3],[4,8],[2,3],[3,2],[9,0],[3,1],[6,4],[3,1],[2,0],[9,0],[5,1],[3,2],[7,8],[2,2],[3,1],[10,2],[14,7],[27,4],[3,1],[3,-3],[8,-6],[3,-2],[3,0],[3,1],[3,2],[2,2],[5,-11],[7,-8],[8,-5],[6,-1],[3,0],[-2,-3],[-3,-8],[-1,-23],[-3,-7],[-2,-1],[-10,-11],[-4,-6],[-1,-8],[1,-6],[6,-11],[-2,-5],[2,-4],[4,-3],[3,-1],[2,-2],[2,-6],[1,-7],[0,-4],[4,-8],[12,-14],[-1,-6],[1,-2],[-8,-13],[-2,-1],[-3,-2],[-1,-5],[0,-6],[-1,-6],[-2,-3],[-7,-7],[-2,-3],[-2,-2],[-7,-13],[0,-2],[-4,-2],[-5,-7],[-3,-2],[-2,-1],[-6,1],[-3,2],[-3,0],[-2,-2],[-1,0],[-4,0],[-4,-1],[-3,-2],[-3,-3],[-3,-3],[-3,-8],[-2,-3],[-7,-5],[-3,-2],[-2,-5],[0,-6],[-2,-4],[-1,-4],[1,-5],[1,-2],[3,-5],[1,0],[1,-2],[0,-3],[-1,-5],[1,-5],[3,-5],[2,-5],[-2,-6],[-3,-3],[-8,-1],[-4,-2],[-2,-4],[-4,-15],[-1,-2],[-1,-1],[-1,-2],[0,-4],[0,-10],[-1,-6],[-7,-19],[-1,-4],[1,-3],[3,-5],[1,-4],[0,-5],[-3,-12],[-2,-12],[-1,-5],[-2,-2],[-1,-3],[-4,-4],[-7,-5],[-3,-5],[-1,-3],[0,-6],[-1,-3],[-7,-13],[0,-2],[0,-4],[1,-3],[1,-1],[2,0],[1,-1],[3,-3],[4,-4],[2,-5],[-1,-5],[-8,-5],[-3,-4],[2,-5],[2,2],[2,0],[2,-1],[1,-3],[-4,-3],[0,-3],[2,-3],[0,-5],[-2,-2],[-4,-2],[-2,-2],[1,-3],[-1,-2],[-3,-5],[-1,-1],[-2,1],[-1,4],[-1,1],[-2,-1],[-1,-2],[-1,-3],[-1,-2],[-2,-3],[-4,-5],[-3,-2],[-4,0],[-1,-2],[1,-7],[4,-10],[0,-4],[-4,-11],[-6,-23],[-2,-2],[-3,3],[-2,0],[-1,-2],[-1,-3],[0,-4],[0,-1],[-3,-3],[0,-1],[0,-1],[0,-1],[-1,-1],[-2,1],[-1,1],[0,1],[-1,1],[-4,0],[-3,-3],[-2,-4],[0,-7],[-1,-1],[-1,-1],[-2,-2],[0,-4],[1,-2],[4,-3],[1,-2],[-4,-3],[-3,-10],[-4,-5],[1,-3],[4,-2],[2,-2],[1,-3],[-2,-3],[-4,-4],[1,-1],[1,-3],[1,-1],[-3,-1],[-2,-1],[-3,-1],[-2,-4],[2,-6],[-2,-1],[-6,-1],[-2,-1],[-6,-5],[-5,-2],[-4,-3],[-5,-2],[-7,-3],[-2,-2],[-1,-3],[-7,-15],[-1,-4],[-1,-5],[0,-4],[-4,-26],[1,-6],[-2,-6],[2,-7],[2,-9],[1,-7],[0,-4],[-1,-2],[-1,-3],[-3,-4],[-4,-4],[-4,-3],[-4,-6],[-5,-2],[-4,-6],[-2,-1],[-6,1],[-13,8],[-71,24],[-71,23],[-3,1],[-3,3],[-3,3],[-2,3],[-2,5],[-2,5],[-1,4],[-3,3],[-6,-1],[-5,-8],[-10,-23],[-2,-2],[-8,0],[-3,-1],[-3,-2],[-2,-3],[-2,-6],[0,-4],[3,-6],[0,-6],[-2,-3],[-23,-25],[-1,-1],[-2,-2],[-1,-4],[-2,-2],[-2,-1],[-11,2],[-4,0],[-2,-1],[-3,-2],[-6,-7],[-3,-2],[-11,-5],[-3,0],[-3,1],[-2,2],[-3,2],[-2,-1],[-7,-2],[-1,-1],[-1,-2],[0,-4],[0,-2],[-3,-4],[-2,-1],[-3,-1],[-3,0],[-7,-3],[0,-4],[7,-14],[6,-4],[2,-3],[7,-17],[16,-83],[17,-83],[-1,-3],[-2,-1],[-2,-3],[-3,-3],[-1,-3],[0,-4],[-1,-4],[0,-4],[2,-9],[0,-5],[-4,-13],[-3,-16],[-1,-5],[1,-14],[0,-7],[0,-2],[0,-2],[2,-5],[0,-3],[0,-2],[-1,-5],[0,-1],[0,-3],[2,-5],[0,-2],[-1,-3],[-4,-7],[0,-4],[1,-3],[14,-14]],[[6147,1],[-2,-1],[0,1],[0,2],[1,1],[1,0],[0,-1],[0,-1],[0,-1]],[[6238,71],[-2,0],[0,1],[2,0],[0,-1]],[[6235,181],[0,-1],[-1,1],[1,0]],[[6213,199],[-1,-1],[-1,1],[0,1],[1,1],[1,-1],[0,-1]],[[6225,213],[-1,-1],[0,1],[1,0]],[[6397,234],[-1,0],[-1,2],[1,0],[1,-2]],[[6332,279],[-1,-1],[-1,1],[0,1],[0,1],[1,0],[0,-1],[1,-1]],[[6332,305],[0,-1],[-1,0],[0,1],[0,1],[1,0],[0,-1]]],"transform":{"scale":[0.006117644072407241,0.0037797847568756915],"translate":[73.60225630700012,15.775376695000048]}};
  Datamap.prototype.civTopo = '__CIV__';
  Datamap.prototype.clpTopo = '__CLP__';
  Datamap.prototype.cmrTopo = '__CMR__';
  Datamap.prototype.codTopo = '__COD__';
  Datamap.prototype.cogTopo = '__COG__';
  Datamap.prototype.cokTopo = '__COK__';
  Datamap.prototype.colTopo = '__COL__';
  Datamap.prototype.comTopo = '__COM__';
  Datamap.prototype.cpvTopo = '__CPV__';
  Datamap.prototype.criTopo = '__CRI__';
  Datamap.prototype.csiTopo = '__CSI__';
  Datamap.prototype.cubTopo = '__CUB__';
  Datamap.prototype.cuwTopo = '__CUW__';
  Datamap.prototype.cymTopo = '__CYM__';
  Datamap.prototype.cynTopo = '__CYN__';
  Datamap.prototype.cypTopo = '__CYP__';
  Datamap.prototype.czeTopo = '__CZE__';
  Datamap.prototype.deuTopo = '__DEU__';
  Datamap.prototype.djiTopo = '__DJI__';
  Datamap.prototype.dmaTopo = '__DMA__';
  Datamap.prototype.dnkTopo = '__DNK__';
  Datamap.prototype.domTopo = '__DOM__';
  Datamap.prototype.dzaTopo = '__DZA__';
  Datamap.prototype.ecuTopo = '__ECU__';
  Datamap.prototype.egyTopo = '__EGY__';
  Datamap.prototype.eriTopo = '__ERI__';
  Datamap.prototype.esbTopo = '__ESB__';
  Datamap.prototype.espTopo = '__ESP__';
  Datamap.prototype.estTopo = '__EST__';
  Datamap.prototype.ethTopo = '__ETH__';
  Datamap.prototype.finTopo = '__FIN__';
  Datamap.prototype.fjiTopo = '__FJI__';
  Datamap.prototype.flkTopo = '__FLK__';
  Datamap.prototype.fraTopo = '__FRA__';
  Datamap.prototype.froTopo = '__FRO__';
  Datamap.prototype.fsmTopo = '__FSM__';
  Datamap.prototype.gabTopo = '__GAB__';
  Datamap.prototype.psxTopo = '__PSX__';
  Datamap.prototype.gbrTopo = '__GBR__';
  Datamap.prototype.geoTopo = '__GEO__';
  Datamap.prototype.ggyTopo = '__GGY__';
  Datamap.prototype.ghaTopo = '__GHA__';
  Datamap.prototype.gibTopo = '__GIB__';
  Datamap.prototype.ginTopo = '__GIN__';
  Datamap.prototype.gmbTopo = '__GMB__';
  Datamap.prototype.gnbTopo = '__GNB__';
  Datamap.prototype.gnqTopo = '__GNQ__';
  Datamap.prototype.grcTopo = '__GRC__';
  Datamap.prototype.grdTopo = '__GRD__';
  Datamap.prototype.grlTopo = '__GRL__';
  Datamap.prototype.gtmTopo = '__GTM__';
  Datamap.prototype.gumTopo = '__GUM__';
  Datamap.prototype.guyTopo = '__GUY__';
  Datamap.prototype.hkgTopo = '__HKG__';
  Datamap.prototype.hmdTopo = '__HMD__';
  Datamap.prototype.hndTopo = '__HND__';
  Datamap.prototype.hrvTopo = '__HRV__';
  Datamap.prototype.htiTopo = '__HTI__';
  Datamap.prototype.hunTopo = '__HUN__';
  Datamap.prototype.idnTopo = '__IDN__';
  Datamap.prototype.imnTopo = '__IMN__';
  Datamap.prototype.indTopo = '__IND__';
  Datamap.prototype.ioaTopo = '__IOA__';
  Datamap.prototype.iotTopo = '__IOT__';
  Datamap.prototype.irlTopo = '__IRL__';
  Datamap.prototype.irnTopo = '__IRN__';
  Datamap.prototype.irqTopo = '__IRQ__';
  Datamap.prototype.islTopo = '__ISL__';
  Datamap.prototype.isrTopo = '__ISR__';
  Datamap.prototype.itaTopo = '__ITA__';
  Datamap.prototype.jamTopo = '__JAM__';
  Datamap.prototype.jeyTopo = '__JEY__';
  Datamap.prototype.jorTopo = '__JOR__';
  Datamap.prototype.jpnTopo = '__JPN__';
  Datamap.prototype.kabTopo = '__KAB__';
  Datamap.prototype.kasTopo = '__KAS__';
  Datamap.prototype.kazTopo = '__KAZ__';
  Datamap.prototype.kenTopo = '__KEN__';
  Datamap.prototype.kgzTopo = '__KGZ__';
  Datamap.prototype.khmTopo = '__KHM__';
  Datamap.prototype.kirTopo = '__KIR__';
  Datamap.prototype.knaTopo = '__KNA__';
  Datamap.prototype.korTopo = '__KOR__';
  Datamap.prototype.kosTopo = '__KOS__';
  Datamap.prototype.kwtTopo = '__KWT__';
  Datamap.prototype.laoTopo = '__LAO__';
  Datamap.prototype.lbnTopo = '__LBN__';
  Datamap.prototype.lbrTopo = '__LBR__';
  Datamap.prototype.lbyTopo = '__LBY__';
  Datamap.prototype.lcaTopo = '__LCA__';
  Datamap.prototype.lieTopo = '__LIE__';
  Datamap.prototype.lkaTopo = '__LKA__';
  Datamap.prototype.lsoTopo = '__LSO__';
  Datamap.prototype.ltuTopo = '__LTU__';
  Datamap.prototype.luxTopo = '__LUX__';
  Datamap.prototype.lvaTopo = '__LVA__';
  Datamap.prototype.macTopo = '__MAC__';
  Datamap.prototype.mafTopo = '__MAF__';
  Datamap.prototype.marTopo = '__MAR__';
  Datamap.prototype.mcoTopo = '__MCO__';
  Datamap.prototype.mdaTopo = '__MDA__';
  Datamap.prototype.mdgTopo = '__MDG__';
  Datamap.prototype.mdvTopo = '__MDV__';
  Datamap.prototype.mexTopo = '__MEX__';
  Datamap.prototype.mhlTopo = '__MHL__';
  Datamap.prototype.mkdTopo = '__MKD__';
  Datamap.prototype.mliTopo = '__MLI__';
  Datamap.prototype.mltTopo = '__MLT__';
  Datamap.prototype.mmrTopo = '__MMR__';
  Datamap.prototype.mneTopo = '__MNE__';
  Datamap.prototype.mngTopo = '__MNG__';
  Datamap.prototype.mnpTopo = '__MNP__';
  Datamap.prototype.mozTopo = '__MOZ__';
  Datamap.prototype.mrtTopo = '__MRT__';
  Datamap.prototype.msrTopo = '__MSR__';
  Datamap.prototype.musTopo = '__MUS__';
  Datamap.prototype.mwiTopo = '__MWI__';
  Datamap.prototype.mysTopo = '__MYS__';
  Datamap.prototype.namTopo = '__NAM__';
  Datamap.prototype.nclTopo = '__NCL__';
  Datamap.prototype.nerTopo = '__NER__';
  Datamap.prototype.nfkTopo = '__NFK__';
  Datamap.prototype.ngaTopo = '__NGA__';
  Datamap.prototype.nicTopo = '__NIC__';
  Datamap.prototype.niuTopo = '__NIU__';
  Datamap.prototype.nldTopo = '__NLD__';
  Datamap.prototype.nplTopo = '__NPL__';
  Datamap.prototype.nruTopo = '__NRU__';
  Datamap.prototype.nulTopo = '__NUL__';
  Datamap.prototype.nzlTopo = '__NZL__';
  Datamap.prototype.omnTopo = '__OMN__';
  Datamap.prototype.pakTopo = '__PAK__';
  Datamap.prototype.panTopo = '__PAN__';
  Datamap.prototype.pcnTopo = '__PCN__';
  Datamap.prototype.perTopo = '__PER__';
  Datamap.prototype.pgaTopo = '__PGA__';
  Datamap.prototype.phlTopo = '__PHL__';
  Datamap.prototype.plwTopo = '__PLW__';
  Datamap.prototype.pngTopo = '__PNG__';
  Datamap.prototype.polTopo = '__POL__';
  Datamap.prototype.priTopo = '__PRI__';
  Datamap.prototype.prkTopo = '__PRK__';
  Datamap.prototype.prtTopo = '__PRT__';
  Datamap.prototype.pryTopo = '__PRY__';
  Datamap.prototype.pyfTopo = '__PYF__';
  Datamap.prototype.qatTopo = '__QAT__';
  Datamap.prototype.rouTopo = '__ROU__';
  Datamap.prototype.rusTopo = '__RUS__';
  Datamap.prototype.rwaTopo = '__RWA__';
  Datamap.prototype.sahTopo = '__SAH__';
  Datamap.prototype.sauTopo = '__SAU__';
  Datamap.prototype.scrTopo = '__SCR__';
  Datamap.prototype.sdnTopo = '__SDN__';
  Datamap.prototype.sdsTopo = '__SDS__';
  Datamap.prototype.senTopo = '__SEN__';
  Datamap.prototype.serTopo = '__SER__';
  Datamap.prototype.sgpTopo = '__SGP__';
  Datamap.prototype.sgsTopo = '__SGS__';
  Datamap.prototype.shnTopo = '__SHN__';
  Datamap.prototype.slbTopo = '__SLB__';
  Datamap.prototype.sleTopo = '__SLE__';
  Datamap.prototype.slvTopo = '__SLV__';
  Datamap.prototype.smrTopo = '__SMR__';
  Datamap.prototype.solTopo = '__SOL__';
  Datamap.prototype.somTopo = '__SOM__';
  Datamap.prototype.spmTopo = '__SPM__';
  Datamap.prototype.srbTopo = '__SRB__';
  Datamap.prototype.stpTopo = '__STP__';
  Datamap.prototype.surTopo = '__SUR__';
  Datamap.prototype.svkTopo = '__SVK__';
  Datamap.prototype.svnTopo = '__SVN__';
  Datamap.prototype.sweTopo = '__SWE__';
  Datamap.prototype.swzTopo = '__SWZ__';
  Datamap.prototype.sxmTopo = '__SXM__';
  Datamap.prototype.sycTopo = '__SYC__';
  Datamap.prototype.syrTopo = '__SYR__';
  Datamap.prototype.tcaTopo = '__TCA__';
  Datamap.prototype.tcdTopo = '__TCD__';
  Datamap.prototype.tgoTopo = '__TGO__';
  Datamap.prototype.thaTopo = '__THA__';
  Datamap.prototype.tjkTopo = '__TJK__';
  Datamap.prototype.tkmTopo = '__TKM__';
  Datamap.prototype.tlsTopo = '__TLS__';
  Datamap.prototype.tonTopo = '__TON__';
  Datamap.prototype.ttoTopo = '__TTO__';
  Datamap.prototype.tunTopo = '__TUN__';
  Datamap.prototype.turTopo = '__TUR__';
  Datamap.prototype.tuvTopo = '__TUV__';
  Datamap.prototype.twnTopo = '__TWN__';
  Datamap.prototype.tzaTopo = '__TZA__';
  Datamap.prototype.ugaTopo = '__UGA__';
  Datamap.prototype.ukrTopo = '__UKR__';
  Datamap.prototype.umiTopo = '__UMI__';
  Datamap.prototype.uryTopo = '__URY__';
  Datamap.prototype.usaTopo = '__USA__';
  Datamap.prototype.usgTopo = '__USG__';
  Datamap.prototype.uzbTopo = '__UZB__';
  Datamap.prototype.vatTopo = '__VAT__';
  Datamap.prototype.vctTopo = '__VCT__';
  Datamap.prototype.venTopo = '__VEN__';
  Datamap.prototype.vgbTopo = '__VGB__';
  Datamap.prototype.virTopo = '__VIR__';
  Datamap.prototype.vnmTopo = '__VNM__';
  Datamap.prototype.vutTopo = '__VUT__';
  Datamap.prototype.wlfTopo = '__WLF__';
  Datamap.prototype.wsbTopo = '__WSB__';
  Datamap.prototype.wsmTopo = '__WSM__';
  Datamap.prototype.yemTopo = '__YEM__';
  Datamap.prototype.zafTopo = '__ZAF__';
  Datamap.prototype.zmbTopo = '__ZMB__';
  Datamap.prototype.zweTopo = '__ZWE__';

  /**************************************
                Utilities
  ***************************************/

  //convert lat/lng coords to X / Y coords
  Datamap.prototype.latLngToXY = function(lat, lng) {
     return this.projection([lng, lat]);
  };

  //add <g> layer to root SVG
  Datamap.prototype.addLayer = function( className, id, first ) {
    var layer;
    if ( first ) {
      layer = this.svg.insert('g', ':first-child')
    }
    else {
      layer = this.svg.append('g')
    }
    return layer.attr('id', id || '')
      .attr('class', className || '');
  };

  Datamap.prototype.updateChoropleth = function(data) {
    var svg = this.svg;
    for ( var subunit in data ) {
      if ( data.hasOwnProperty(subunit) ) {
        var color;
        var subunitData = data[subunit]
        if ( ! subunit ) {
          continue;
        }
        else if ( typeof subunitData === "string" ) {
          color = subunitData;
        }
        else if ( typeof subunitData.color === "string" ) {
          color = subunitData.color;
        }
        else {
          color = this.options.fills[ subunitData.fillKey ];
        }
        //if it's an object, overriding the previous data
        if ( subunitData === Object(subunitData) ) {
          this.options.data[subunit] = defaults(subunitData, this.options.data[subunit] || {});
          var geo = this.svg.select('.' + subunit).attr('data-info', JSON.stringify(this.options.data[subunit]));
        }
        svg
          .selectAll('.' + subunit)
          .transition()
            .style('fill', color);
      }
    }
  };

  Datamap.prototype.updatePopup = function (element, d, options) {
    var self = this;
    element.on('mousemove', null);
    element.on('mousemove', function() {
      var position = d3.mouse(self.options.element);
      d3.select(self.svg[0][0].parentNode).select('.datamaps-hoverover')
        .style('top', ( (position[1] + 30)) + "px")
        .html(function() {
          var data = JSON.parse(element.attr('data-info'));
          try {
            return options.popupTemplate(d, data);
          } catch (e) {
            return "";
          }
        })
        .style('left', ( position[0]) + "px");
    });

    d3.select(self.svg[0][0].parentNode).select('.datamaps-hoverover').style('display', 'block');
  };

  Datamap.prototype.addPlugin = function( name, pluginFn ) {
    var self = this;
    if ( typeof Datamap.prototype[name] === "undefined" ) {
      Datamap.prototype[name] = function(data, options, callback, createNewLayer) {
        var layer;
        if ( typeof createNewLayer === "undefined" ) {
          createNewLayer = false;
        }

        if ( typeof options === 'function' ) {
          callback = options;
          options = undefined;
        }

        options = defaults(options || {}, self.options[name + 'Config']);

        //add a single layer, reuse the old layer
        if ( !createNewLayer && this.options[name + 'Layer'] ) {
          layer = this.options[name + 'Layer'];
          options = options || this.options[name + 'Options'];
        }
        else {
          layer = this.addLayer(name);
          this.options[name + 'Layer'] = layer;
          this.options[name + 'Options'] = options;
        }
        pluginFn.apply(this, [layer, data, options]);
        if ( callback ) {
          callback(layer);
        }
      };
    }
  };

  // expose library
  if (typeof exports === 'object') {
    d3 = require('d3');
    topojson = require('topojson');
    module.exports = Datamap;
  }
  else if ( typeof define === "function" && define.amd ) {
    define( "datamaps", ["require", "d3", "topojson"], function(require) {
      d3 = require('d3');
      topojson = require('topojson');

      return Datamap;
    });
  }
  else {
    window.Datamap = window.Datamaps = Datamap;
  }

  if ( window.jQuery ) {
    window.jQuery.fn.datamaps = function(options, callback) {
      options = options || {};
      options.element = this[0];
      var datamap = new Datamap(options);
      if ( typeof callback === "function" ) {
        callback(datamap, options);
      }
      return this;
    };
  }
})();
